/*
   image layout in .img file:

   ------------------------------------
   ( 1) bootargs size
   ( 1) bootcmd size
   ( x) bootargs string
   ( x) bootcmd string
   (48) address & sz on flash
   (64) image header
   ( x) u-boot
   ( x) kernel
   ( x) ramdisk  
   ------------------------------------
   */

/*
  The file structure of "misc.img"
  -------------------------------------------------------------------------------------------------
 |    1byte      |    1byte     |    mbytes     |   nbytes     |   64bytes  |                     |
  -------------------------------------------------------------------------------------------------
 |bootargs length|bootcmd length|bootargs string|bootcmd string|image header|u-boot|kernel|ramdisk|
  -------------------------------------------------------------------------------------------------
  m = bootargs length; n = bootcmd length;
  u-boot, kernel and ramdisk have their own image header.
*/
#include <stdio.h>
#include <string.h>

#define do_encrypt	data ^= 0xe3

char firmware_st[]  = "00900000";   /*uboot base*/
char firmware_len[] = "009FFFFF";   /*uboot max length*/
char kernel_st[]    = "00300000";   /*kernel base*/
char kernel_len[]   = "0087FFFF";   /*kernel end address*/
char ramdisk_st[]   = "00e00000";   /*rootfs base*/
char ramdisk_len[]  = "071FFFFF";   /*rootfs end address*/


char bootargs[]     = "root=/dev/mtdblock3 rootfstype=jffs2 console=ttyS0,115200 mtdparts=cnc-nand:896k(uboot1),128k(env),2m(logo),100m@14m(rootfs),100m@114m(update),42m@214m(user)";
char bootcmd[]      = "gpio write 8 0;nand read 0x3000000 0x300000 0x300000;bootm 0x3000000";
char *img_ptr[]     = {firmware_st, firmware_len, kernel_st, kernel_len, ramdisk_st, ramdisk_len};

int main()
{
	FILE *fp_bin;
	FILE *fp_img;
	unsigned char data, i, j, pad = 0;
	unsigned short bootargs_len, bootcmd_len;
	long count;
	char *ptr;

	if ((fp_bin = fopen("misc.temp", "rb")) == NULL)
	{
		printf("can not open file misc.temp!\n");
		return 0;
	}
	if ((fp_img = fopen("misc.img", "wb")) == NULL)
	{
		printf("can not open file misc.img!\n");
		return 0;
	}

	bootargs_len = strlen(bootargs);
	bootcmd_len = strlen(bootcmd);
	if (((bootargs_len + bootcmd_len) % 4) != 0)
	{
		pad = 4 - ((bootargs_len + bootcmd_len) % 4);
		bootcmd_len += pad;
	}

	data = bootargs_len >> 8;
	do_encrypt;
	fputc(data, fp_img);
	data = ( bootargs_len << 8 ) >> 8;
	do_encrypt;
	fputc(data, fp_img);
	data = bootcmd_len >> 8;
	do_encrypt;
	fputc(data, fp_img);
	data = ( bootcmd_len << 8 ) >> 8;
	do_encrypt;
	fputc(data, fp_img);

	ptr = bootargs;
	while (*ptr != '\0')
	{
		data = *ptr++;
		do_encrypt;
		fputc(data, fp_img);
	}

	ptr = bootcmd;
	while (*ptr != '\0')
	{
		data = *ptr++;
		do_encrypt;
		fputc(data, fp_img);
	}

	while (pad != 0)
	{
		pad--;
		data = ' ';
		do_encrypt;
		fputc(data, fp_img);
	}

	for (i = 0; i < 6; i++)
	{
		ptr = img_ptr[i];
		for (j = 0; j < 8; j++)
		{
			data = *ptr++;
			do_encrypt;
			fputc(data, fp_img);
		}
	}

	count = 0;
	while (!feof(fp_bin))
	{
		count++;
		data = fgetc(fp_bin);
		do_encrypt;
		fputc(data, fp_img);
		if ((count % 409600) == 0)
			printf(".");
	}
	printf("\n");

	fclose(fp_bin);
	fclose(fp_img);

	return 0;
}
