#!/bin/sh

dmesg -n 8

source /etc/wifi_driver.sh

#Check if wifi driver path has been overridden from uboot 
WIFI_DRIVER_PATH=`fw_printenv -q WIFI_DRIVER_PATH`
if [ -z ${WIFI_DRIVER_PATH} ]; then
	#check if path has been setup at compile time	
	WIFI_DRIVER_PATH=${WIFI_DRIVER}
	if [ -z ${WIFI_DRIVER_PATH} ]; then
		# use default path
		WIFI_DRIVER_PATH="qualcomm/ar9374/v5.0.0.28"
	fi
fi


#####################################################
# setup parameters for calling driver load script
#####################################################
 
#########################################################
# --mode=<ap,sta,p2p>
# default : auto
#########################################################
 
WIFI_MODES=`fw_printenv -q WIFI_MODES`
 
for WIFI_MODE in ${WIFI_MODES}
do
	if [ "${WIFI_MODE}" == "AP" ]; then
		mode="${mode}ap,"
   fi
   if [ "${WIFI_MODE}" == "STA" ]; then
      mode="${mode}sta,"
   fi       
   if [ "${WIFI_MODE}" == "P2P" ]; then
      mode="${mode}p2p,"
   fi
   if [ "${WIFI_MODE}" == "TCMD" ]; then
   	#TCMD is for calibration so it overrides all other modes
      mode="tcmd"
   fi
done
 
mode=`echo ${mode} | sed -e s/,*$//`
if [ "${mode}" == "" ]; then
   mode="auto"
fi
 
params="--mode ${mode}"

#########################################################
 

#########################################################
# --mac=<xx:xx:xx:xx:xx:xx>
# default : programmed in otp or eeprom
#########################################################

mac=`fw_printenv -q wifiaddr`
if [ "${mac}" != "" ]; then
	params="${params} --mac ${mac} "
fi

#########################################################


#########################################################
# --reg-domain=<US/GB...>
# default : 00
#########################################################

reg_domain=`fw_printenv -q WIFICOUNTRY`
if [ "${reg_domain}" != "" ]; then
	params="${params} --reg-domain ${reg_domain} "
fi

#########################################################

#########################################################
# WIFI_UTILS_PATH will be added to PATH
# loaded driver will have to link WIFI_UTILS_PATH to installed
# driver application path.
# eg. if loaded driver has all its apps installed under
# /usr/bin/qualcomm/ar6203/v3.2.0.7 then
# it will link WIFI_UTILS_PATH to this path
# As a result all driver specific apps will be available on PATH transparently
#########################################################

WIFI_UTILS_PATH="/usr/bin/"${WIFI_DRIVER_PATH}
ln -s ${WIFI_UTILS_PATH} /usr/bin/wifi_utils 

#########################################################
# WIFI_CONF_PATH will always point to /etc/conf/wifi
# loaded driver will have to link /etc/conf/wifi to installed
# driver apps configurations path.
# eg. if loaded driver has all its apps config files installed under
# /etc/conf/qualcomm/ar6203/v3.2.0.7 then
# it will link WIFI_CONF_PATH to this path
# Solutions will always refer to WIFI_CONF_PATH for golden copy of
# driver c apps configuration
#########################################################

WIFI_CONF_PATH="/etc/conf/"${WIFI_DRIVER_PATH}
ln -s ${WIFI_CONF_PATH} /etc/conf/wifi

##############################################################
# declare the wifi device so applications can use it if needed
##############################################################
wifi_device=`grep WIFI_DEVICE ${WIFI_CONF_PATH}/wifi_driver.info`
if [ "${wifi_device}" == "" ]; then
	echo "WIFI_DEVICE=qualcomm_ar9374_hw30" >> ${WIFI_CONF_PATH}/wifi_driver.info
fi

##############################################################

##############################################################
# Create a link to cnc_nw_plugins
############################################################## 

CNC_NW_PLUGINS_PATH="/usr/lib/cnc_nw_plugins"
WIFI_CNC_NW_PLUGINS_PATH="/usr/lib/"${WIFI_DRIVER_PATH}"/cnc_nw_plugins"
CURRENT_DIR=`pwd`

mkdir -p ${CNC_NW_PLUGINS_PATH}

cd ${WIFI_CNC_NW_PLUGINS_PATH}
for plugin in *.so
do
	ln -sf ${WIFI_CNC_NW_PLUGINS_PATH}/${plugin} ${CNC_NW_PLUGINS_PATH}/${plugin}
done
cd ${CURRENT_DIR}

##############################################################

echo "PARAMS : ${params} WIFI_UTILS_PATH : ${WIFI_UTILS_PATH}"

#############################################################
# /lib/modules/<kernel version> folder is required for rmmod
#############################################################

mkdir -p /lib/modules/`uname -r`

#############################################################


chmod +x ${WIFI_UTILS_PATH}/wifi_driver_load.sh 
chmod +x ${WIFI_UTILS_PATH}/wifi_driver_unload.sh
${WIFI_UTILS_PATH}/wifi_driver_load.sh ${params}
 

# script is now being run on background. This indicates that the 
# script has ended. Applaunch will wait for this file to exist
# before running wifi manager
touch /tmp/wifi_load_end
