#!/bin/sh

# This script should load wifi driver
# Available parameters are
# --mode AP,STA,P2P/TCMD/ART
# --mac xx:xx:xx:xx:xx:xx
# --reg-domain US/JP/...

MODULE_PATH=/lib/modules/qualcomm/ar9374/v5.0.0.108
MODULE_FIRMWARE_PATH="/lib/firmware/qualcomm/ar9374/v5.0.0.108/hw1.3 /lib/firmware/qualcomm/ar9374/v5.0.0.108/hw3.0"
USER_BIN_PATH=/usr/bin/qualcomm/ar9374/v5.0.0.108
ETC_CONF_PATH=/etc/conf/qualcomm/ar9374/v5.0.0.108
USER_LIB_PATH=/lib/qualcomm/ar9374/v5.0.0.108
JFFS2_BDATA_PATH="/mnt/jffs2/firmware/qualcomm/ar9374/v5.0.0.108"
LIB_BDATA_PATH="/lib/firmware/qualcomm/ar9374/v5.0.0.108"
BDATA_HW="hw1.3 hw3.0"
BDATA_PRE="bdata"

for BDATA_HW_DIR in ${BDATA_HW}; do
	if [ -d ${JFFS2_BDATA_PATH}/${BDATA_HW_DIR} ]; then
	cd ${JFFS2_BDATA_PATH}/${BDATA_HW_DIR}
	for BDATA in ${BDATA_PRE}*; do
		SRC=${JFFS2_BDATA_PATH}/${BDATA_HW_DIR}/${BDATA}
		DST=${LIB_BDATA_PATH}/${BDATA_HW_DIR}/${BDATA}
		if [ -f ${SRC} -a -f ${DST} ]; then
			SRC_CSUM=` md5sum ${SRC} | awk '{print $1}'`
			DST_CSUM=` md5sum ${DST} | awk '{print $1}'`
			if  [ ${SRC_CSUM} != ${DST_CSUM} ]; then
				cp ${SRC} ${DST}
				sync
				echo "Customer bdata found in ${SRC}, copy it to ${DST}"
			fi
		fi
	done
	fi
done

while [ "$#" -ne 0 ]
do
    case $1 in
        --mode )
        shift
        mode="$1"
        shift
        ;;
        --mac )
        shift
        for module_firmware_path in ${MODULE_FIRMWARE_PATH}; do
		      echo -e -n "\\x${1//:/\\x}" > ${module_firmware_path}/softmac.bin
    	  done
        shift
        ;;
        --reg-domain )
        shift
        reg_domain="$1"
        shift
        ;;
        * )
        echo "Unsupported argument : $1"
        shift
    esac
done

params="ath6kl_p2p=0x19 debug_quirks=0x8200 htc_bundle_send=1 htc_bundle_send_timer=10"

if [ ! -z ${reg_domain} ]; then

    case ${reg_domain} in
    
        ( US )
        params="${params} reg_domain=0x8348"
        ;;

        ( AU )
        params="${params} reg_domain=0x8024"
        ;;

        ( GB )
        params="${params} reg_domain=0x833A"
        ;;

        ( PE )
        params="${params} reg_domain=0x825C"
        ;;
        
        ( JP )
        params="${params} reg_domain=0x8188"
        ;;

        ( KR )
        params="${params} reg_domain=0x819A"
        ;;

        ( PA )
        params="${params} reg_domain=0x824F"
        ;;

    esac       
fi    

if [ "${mode}" == "tcmd" ]; then
    params="testmode=1"
fi

insmod $MODULE_PATH/compat.ko
insmod $MODULE_PATH/cfg80211.ko
insmod $MODULE_PATH/ath6kl_usb.ko ${params}
 
${USER_BIN_PATH}/iw dev wlan0 set power_save off
${USER_BIN_PATH}/iw dev p2p0 set power_save off
