#ifndef __CNC_H__
#define __CNC_H__

#include "cnc_common_typedef.h"
#include "cnc_filemgr.h"
#include "cnc_system.h"
#include "cnc_xml.h"

#ifdef __cplusplus
extern "C" {
#endif

#define DEFAULT_TVOUTPUT_MODE     TVOUT_MODE_1080I30
#define DEFAULT_SDTVOUTPUT_MODE     TVOUT_MODE_576I //TVOUT_MODE_PAL_N
#define DEFAULT_LANGUAGE           CNC_LANG_SPANISH
//#define MODE_1080P24_SUPPORTED

/* E2PROM USAGE SCHEDULE */
/* Attention: evey one to use e2prom must apply a unique block, and register offset in the following*/
#define E2PROM_LANUCH_MANUFACTOR_TEST_FLAG_OFFSET 0x0 //used by manufacture test software.
#define E2PROM_IP_UPGRADE_KERNEL_FLAG_OFFSET 0x1 //used by "update.sh" script in IP upgrade, this byte indicates we should upgrade kernel in uboot stage.
#define E2PROM_CHANNEL_3_4_INDICATION_OFFSET 0x3 //this byte ensure the kernel and uboot select the same RF frequency between channel 3 and channel 4.

#define E2PROM_MAC_ADDR_OFFSET    0x0114
#define E2PROM_MAC_ADDR_LENGTH    6
#define E2PROM_MAC_ADDR_END       0x0114+6

// add by wenjia.cao for ARIB OTA in Brazil
#define EEPROM_OTA_INFO_OFFSET 0x300
#define EEPROM_OTA_INFO_LENGTH 17
#define EEPROM_OTA_INFO_END     0x300+17

#define EEPROM_OTA_PARAM_OFFSET 0x200
#define EEPROM_OTA_PARAM_LENGTH  16
#define EEPROM_OTA_PARAM_END    0x200+16

#define EEPROM_BOOTLOAD_FLAG_OFFSET 0x01
#define EEPROM_BOOTLOAD_FLAG_LENGTH  1
#define EEPROM_BOOTLOAD_FLAG_END    0x01+1

#define EEPROM_BOOTLOAD_TOTAL_OFFSET 0x04
#define EEPROM_BOOTLOAD_TOTAL_LENGTH  4
#define EEPROM_BOOTLOAD_TOTAL_END    0x04+4

// For User Setting
#define E2PROM_SETTING_OFFSET     0x03E8   // (1000)d
#define E2PROM_SETTING_LENGTH     336   // sizeof(CNC_Setting_Core_t)
#define E2PROM_SETTING_END        0x03E8+336

#define CNC_AUDIO_2SFORMAT        1

//#define BT_NTFS_SUPPORTED

//#define RC_SYSTEM_CODE_SUPPORTED  //remote controller support client code?
#define RC_SYSTEM_CODE            0xff00//0x2c40
#define  PATH_CONFIG  "./config"


/**********************Global debug output control of middle level**************************/
#define CNC_GLOBAL_DBG_LEVEL 2
#if (CNC_GLOBAL_DBG_LEVEL < 0 ||CNC_GLOBAL_DBG_LEVEL > 4)
#error "CNC_GLOBAL_DBG_LEVEL should be between 0 and 4(include 0 and 4)"
#endif

/**************************Mid av debug output control*******************************/
#define CNC_AV_DBG_LEVEL 0 
#define CNC_AV_PANIC(...)
#define CNC_AV_ERR(...)
#define CNC_AV_INFO(...)
#define CNC_AV_DBG(...)

#if (CNC_AV_DBG_LEVEL > 0)
#undef CNC_AV_PANIC
#define CNC_AV_PANIC(...) CNC_UTIL_Print("[CNC_av-p] "__VA_ARGS__)
    #if (CNC_AV_DBG_LEVEL > 1)
    #undef CNC_AV_ERR
    #define CNC_AV_ERR(...) CNC_UTIL_Print("[CNC_av-e] "__VA_ARGS__)
        #if (CNC_AV_DBG_LEVEL > 2)
        #undef CNC_AV_INFO
        #define CNC_AV_INFO(...) CNC_UTIL_Print("[CNC_av-i] "__VA_ARGS__)
            #if (CNC_AV_DBG_LEVEL > 3)
            #undef CNC_AV_DBG
            #define CNC_AV_DBG(...) CNC_UTIL_Print("[CNC_av-d] "__VA_ARGS__)
            #endif
        #endif
    #endif
#endif

/**************************Mid setting debug output control*******************************/
#define CNC_SETTING_DBG_LEVEL CNC_GLOBAL_DBG_LEVEL
#define CNC_SETTING_PANIC(...)
#define CNC_SETTING_ERR(...)
#define CNC_SETTING_INFO(...)
#define CNC_SETTING_DBG(...)

#if (CNC_SETTING_DBG_LEVEL > 0)
#undef CNC_SETTING_PANIC
#define CNC_SETTING_PANIC(...) CNC_UTIL_Print("[CNC_setting-p] "__VA_ARGS__)
    #if (CNC_SETTING_DBG_LEVEL > 1)
    #undef CNC_SETTING_ERR
    #define CNC_SETTING_ERR(...) CNC_UTIL_Print("[CNC_setting-e] "__VA_ARGS__)
        #if (CNC_SETTING_DBG_LEVEL > 2)
        #undef CNC_SETTING_INFO
        #define CNC_SETTING_INFO(...) CNC_UTIL_Print("[CNC_setting-i] "__VA_ARGS__)
            #if (CNC_SETTING_DBG_LEVEL > 3)
            #undef CNC_SETTING_DBG
            #define CNC_SETTING_DBG(...) CNC_UTIL_Print("[CNC_setting-d] "__VA_ARGS__)
            #endif
        #endif
    #endif
#endif

/**************************Mid setup debug output control*******************************/
#define CNC_SETUP_DBG_LEVEL CNC_GLOBAL_DBG_LEVEL
#define CNC_SETUP_PANIC(...)
#define CNC_SETUP_ERR(...)
#define CNC_SETUP_INFO(...)
#define CNC_SETUP_DBG(...)

#if (CNC_SETUP_DBG_LEVEL > 0)
#undef CNC_SETUP_PANIC
#define CNC_SETUP_PANIC(...) CNC_UTIL_Print("[CNC_setup-p] "__VA_ARGS__)
    #if (CNC_SETUP_DBG_LEVEL > 1)
    #undef CNC_SETUP_ERR
    #define CNC_SETUP_ERR(...) CNC_UTIL_Print("[CNC_setup-e] "__VA_ARGS__)
        #if (CNC_SETUP_DBG_LEVEL > 2)
        #undef CNC_SETUP_INFO
        #define CNC_SETUP_INFO(...) CNC_UTIL_Print("[CNC_setup-i] "__VA_ARGS__)
            #if (CNC_SETUP_DBG_LEVEL > 3)
            #undef CNC_SETUP_DBG
            #define CNC_SETUP_DBG(...) CNC_UTIL_Print("[CNC_setup-d] "__VA_ARGS__)
            #endif
        #endif
    #endif
#endif

/**************************Mid standby debug output control*******************************/
#define CNC_STANDBY_DBG_LEVEL CNC_GLOBAL_DBG_LEVEL
#define CNC_STANDBY_PANIC(...)
#define CNC_STANDBY_ERR(...)
#define CNC_STANDBY_INFO(...)
#define CNC_STANDBY_DBG(...)

#if (CNC_STANDBY_DBG_LEVEL > 0)
#undef CNC_STANDBY_PANIC
#define CNC_STANDBY_PANIC(...) CNC_UTIL_Print("[CNC_standby-p] "__VA_ARGS__)
    #if (CNC_STANDBY_DBG_LEVEL > 1)
    #undef CNC_STANDBY_ERR
    #define CNC_STANDBY_ERR(...) CNC_UTIL_Print("[CNC_standby-e] "__VA_ARGS__)
        #if (CNC_STANDBY_DBG_LEVEL > 2)
        #undef CNC_STANDBY_INFO
        #define CNC_STANDBY_INFO(...) CNC_UTIL_Print("[CNC_standby-i] "__VA_ARGS__)
            #if (CNC_STANDBY_DBG_LEVEL > 3)
            #undef CNC_STANDBY_DBG
            #define CNC_STANDBY_DBG(...) CNC_UTIL_Print("[CNC_standby-d] "__VA_ARGS__)
            #endif
        #endif
    #endif
#endif

/**************************Mid usb debug output control*******************************/
#define CNC_USB_DBG_LEVEL CNC_GLOBAL_DBG_LEVEL
#define CNC_USB_PANIC(...)
#define CNC_USB_ERR(...)
#define CNC_USB_INFO(...)
#define CNC_USB_DBG(...)

#if (CNC_USB_DBG_LEVEL > 0)
#undef CNC_USB_PANIC
#define CNC_USB_PANIC(...) CNC_UTIL_Print("[CNC_usb-p] "__VA_ARGS__)
    #if (CNC_USB_DBG_LEVEL > 1)
    #undef CNC_USB_ERR
    #define CNC_USB_ERR(...) CNC_UTIL_Print("[CNC_usb-e] "__VA_ARGS__)
        #if (CNC_USB_DBG_LEVEL > 2)
        #undef CNC_USB_INFO
        #define CNC_USB_INFO(...) CNC_UTIL_Print("[CNC_usb-i] "__VA_ARGS__)
            #if (CNC_USB_DBG_LEVEL > 3)
            #undef CNC_USB_DBG
            #define CNC_USB_DBG(...) CNC_UTIL_Print("[CNC_usb-d] "__VA_ARGS__)
            #endif
        #endif
    #endif
#endif

/**************************Mid file debug output control*******************************/
#define CNC_FILE_DBG_LEVEL CNC_GLOBAL_DBG_LEVEL
#define CNC_FILE_PANIC(...)
#define CNC_FILE_ERR(...)
#define CNC_FILE_INFO(...)
#define CNC_FILE_DBG(...)

#if (CNC_FILE_DBG_LEVEL > 0)
#undef CNC_FILE_PANIC
#define CNC_FILE_PANIC(...) CNC_UTIL_Print("[CNC_file-p] "__VA_ARGS__)
    #if (CNC_FILE_DBG_LEVEL > 1)
    #undef CNC_FILE_ERR
    #define CNC_FILE_ERR(...) CNC_UTIL_Print("[CNC_file-e] "__VA_ARGS__)
        #if (CNC_FILE_DBG_LEVEL > 2)
        #undef CNC_FILE_INFO
        #define CNC_FILE_INFO(...) CNC_UTIL_Print("[CNC_file-i] "__VA_ARGS__)
            #if (CNC_FILE_DBG_LEVEL > 3)
            #undef CNC_FILE_DBG
            #define CNC_FILE_DBG(...) CNC_UTIL_Print("[CNC_file-d] "__VA_ARGS__)
            #endif
        #endif
    #endif
#endif

#ifdef __cplusplus
}
#endif

#endif
