/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Cavium, Inc.
 Cavium, Inc reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium, Inc.

 ALL RIGHTS RESERVED.

 Copyright 2012, Cavium, Inc.
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */

/**
 * @ingroup media
 * @file cnc_common_error.h
 * Some common errors used between modules in media playback.
 */

#ifndef _CNC_COMMON_ERROR_H_
#define _CNC_COMMON_ERROR_H_

#ifdef __cplusplus
extern "C" {
#endif

/* ======================== System Includes =============================== */
// NONE


/* ========================= User Includes ================================ */
// NONE


/* =========================== Macro definitions ========================== */
#define CNC_FLAG_UNSET                   (-1)

/* =========================== Type definitions =========================== */
/**
 * \enum   E_cncErrCode_t
 * \brief  This defines errors common to multiple modules in crisp extensions. This is mostly used in cncmedia.
 */
typedef enum E_cncErrCodeTag
{
    E_CNC_ERR_NONE                        = 0,         /*!< Success */
    E_CNC_ERR_INVALID_PARAM               = -1000,     /*!< Invalid parameters like NULL pointers, invalid instance reference. */
    E_CNC_ERR_INVALID_REQUEST,                         /*!< Request not supported during current state. Request not valid for given instance. */
    E_CNC_ERR_INVALID_PLAYBACK_CONTENT,                /*!< Forbidden content for playback (Ex: 403). Unsupported image size */
    E_CNC_ERR_CONTENT_NOT_SUBSCRIBED,                  /*!< Content not subscribed for playback/recording */
    E_CNC_ERR_CONTENT_NOT_AUTHORIZED,                  /*!< Content not authorized for playback/recording */
    E_CNC_ERR_UNSUPPORTED_AUDIO,                       /*!< Non supported audio codec */
    E_CNC_ERR_UNSUPPORTED_VIDEO,                       /*!< Non supported video codec */
    E_CNC_ERR_UNSUPPORTED_MEDIA,                       /*!< Non supported media format */
    E_CNC_ERR_RESOURCE_ALLOCATION,                     /*!< Resource not available for performing given request */
    E_CNC_ERR_NOT_INITIALIZED,                         /*!< Module not initialized */
    E_CNC_ERR_RE_INITIALIZE,                           /*!< Trying to initialize more than once */
    E_CNC_ERR_MEMORY_ALLOCATION,                       /*!< Not able to allocate memory */
    E_CNC_ERR_PTHREAD_OPERATION,                       /*!< Error during pthread thread creation, mutex operation, condition wait etc. */
    E_CNC_ERR_SOCKET_OPERATION,                        /*!< Deprecated. Don’t use. */
    E_CNC_ERR_TUNING_OPERATION,                        /*!< Deprecated. Don’t use. */
    E_CNC_ERR_NETWORK_DISCONNECT,                      /*!< Error occurred because of network cable removal */
    E_CNC_ERR_NETWORK_TIMEOUT,                         /*!< Unable to read data from network */
    E_CNC_ERR_SERVER_NOT_FOUND,                        /*!< Not able to connect to the server */
    E_CNC_ERR_CONTENT_NOT_FOUND,                       /*!< Content not found on server (Ex: HTTP error 404)  */
    E_CNC_ERR_CONNECTION_FAILED,                       /*!< Connection failed while connecting to server */
    E_CNC_ERR_SERVER_NOT_RESPONDING,                   /*!< Server not responding */
    E_CNC_ERR_INVALID_RESPONSE,                        /*!< Invalid response from server */
    E_CNC_ERR_NO_DATA,                                 /*!< Buffer underflow for long time (ex: 30 sec) */
    E_CNC_ERR_EOS,                                     /*!< End of Stream */
    E_CNC_ERR_SOS,                                     /*!< Start of Stream */
    E_CNC_ERR_NOT_IMPLEMENTED,                         /*!< API not implemented */
    E_CNC_ERR_UNABLE_TO_FIND_LICENSE,                  /*!< License not found */
    E_CNC_ERR_UNABLE_TO_DECRYPT_LICENSE,               /*!< Unable to decrypt license */
    E_CNC_ERR_INVALID_LICENSE,                         /*!< Invalid license */
    E_CNC_ERR_LICENSE_EXPIRED,                         /*!< License expired */
    E_CNC_ERR_LICENSE_RENEW_FAILED,                    /*!< License renew failed */
    E_CNC_ERR_STORAGE_DEVICE_ACCESS,                   /*!< Not able to access storage device */
    E_CNC_ERR_STORAGE_DEVICE_FULL,                     /*!< Storage Device got FULL, No free memory available */
    E_CNC_ERR_LOWER_LAYER,                             /*!< Lower layer error (Ex: System errors while accessing message queue, semaphores etc)*/
    E_CNC_ERR_INSUFFICIENT_MEMORY                      /*!< Deprecated. Don’t use. */
} E_cncErrCode_t;
/* ======================== Function declarations ========================= */
// NONE

#ifdef __cplusplus
}
#endif

#endif /* END: _CNC_COMMON_ERROR_H_ */
