/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Cavium, Inc.
 Cavium, Inc. reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium, Inc.

 ALL RIGHTS RESERVED.

 Copyright 2011-2012, Cavium, Inc.
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */


/**
 * @brief     Common type definitions used throughout all the CRISP libraries
 * @file       cnc_common_typedef.h
 * @ingroup core
 */

#ifndef _CNC_COMMON_TYPEDEF_H_
#define _CNC_COMMON_TYPEDEF_H_

#ifdef __cplusplus
extern "C" {
#endif

/* ============================ System Includes =========================== */
// NONE


/* ============================= User Includes ============================ */
// NONE


/* =========================== Macro definitions ========================== */
// NONE


/* =========================== Type definitions =========================== */
typedef char                          CNC_CHAR;
typedef unsigned char                 CNC_UINT8;
typedef signed char                   CNC_SINT8;
typedef unsigned char                 CNC_BYTE;
/* Will be updated to the 'signed char' once all the warnings are removed from the code. */
// typedef char                          CNC_SINT8;

typedef unsigned short                CNC_UINT16;
typedef signed short                  CNC_SINT16;
typedef unsigned short                CNC_HWORD;

typedef unsigned int                  CNC_UINT32;
typedef signed int                    CNC_SINT32;
/* Attention: Ideally 'unsigned long' should be treated as CNC_WORD */
typedef unsigned int                  CNC_WORD;

typedef signed long int               CNC_SLINT32;
typedef unsigned long int             CNC_ULINT32;

typedef long long int                 CNC_SLLINT64;
typedef unsigned long long int        CNC_ULLINT64;

typedef unsigned long                 CNC_ULONG32;
typedef signed long                   CNC_SLONG32;

typedef unsigned long long            CNC_ULONG64;
typedef unsigned long long            CNC_UINT64;
typedef signed long long              CNC_SLONG64;
typedef signed long long              CNC_SINT64;
typedef unsigned long long            CNC_DWORD;

typedef float                         CNC_SFLOAT32;
typedef double                        CNC_SFLOAT64;
typedef long double                   CNC_SFLOAT96;

/* TODO: Should remove these duplicate definitions once all the warnings/errors are removed.*/
typedef float                         CNC_FLOAT32;
typedef double                        CNC_FLOAT64;
typedef long double                   CNC_FLOAT96;

typedef enum CNC_BOOL_TAG
{
    CNC_BOOL_FALSE = 0,
    CNC_BOOL_TRUE = 1
} CNC_BOOL;

typedef struct S_cncRationalTag
{
    CNC_SINT32  s32_num;
    CNC_SINT32  s32_den;
} S_cncRational_t;

typedef void        CNC_VOID;
typedef void *      CNC_HANDLE;
#define CNC_NULL    ((void*)0)

typedef enum
{
      RESULT_OK = 0,
      RESULT_FAIL,
      RESULT_BAD_PARAMETER,             /* Bad parameter passed       */
      RESULT_NO_MEMORY,                 /* Memory allocation failed   */
      RESULT_UNKNOWN_DEVICE,            /* Unknown device name        */
      RESULT_ALREADY_INITIALIZED,       /* Device already initialized */
      RESULT_INVALID_HANDLE,            /* Handle is not valid        */
      RESULT_FEATURE_NOT_SUPPORTED,     /* Feature unavailable        */
      RESULT_TIMEOUT,                   /* Timeout occured            */
      RESULT_FAILD_GET_SEMAPHORE,
      RESULT_FAILD_LOCK_MUTEX,  /* 10 */
      RESULT_NEWFREQENCY_OK,
      RESULT_NEWFREQENCY_FAIL,
      RESULT_NEWFREQENCY_SAME,  /* 13 */
      RESULT_ABORT,
      RESULT_RESTART_REQUIRED,  /* 15 */ 

      RESULT_NUM
}Result_Type_e;

/* ===================== Global variable declarations ===================== */
// NONE


/* ======================== Function declarations ========================= */
// NONE

#ifdef __cplusplus
}
#endif

#endif /* END: _CNC_COMMON_TYPEDEF_H_ */

