#ifndef __include_mtcookie_h__
#define __include_mtcookie_h__

#include <string>
#include <strings.h>
#include <stdlib.h>
#include "mtchain.h"
#include "mturl.h"
#include "mtexpires.h"
#include "mtmutex.h"
#include "mtdebug.h"

class MtCookieChain;
class MtDomainCookie;
class MtSetCookie;
class MtCookie;

/*
 * BS:
 *----
 * Creates a cookie pointer which points to the cookie.
 * Uses the setcookie function to do this.
 *
 * Dependency:
 *------------
 * MtSetCookie
 */
class MtCookieChain: public MtChain
{
    public:
        //MtChain qcookie;
        MtSetCookie *setcookie; /*BS: Cookie pointer*/
        MtCookieChain(MtSetCookie *setcookie) :
            setcookie(setcookie)
        {
        }
        ;
        /*
         * BS: Constructor which calls the setcookie function to make the cookie
         * pointer to point to the cookie created.
         */
};

/*
 * BS:
 *----
 * Contains the name of the domain to be placed in the cookie.
 *
 *
 * Dependency:
 *------------
 * Class MtChain
 */
class MtDomainCookie: public MtChain
{
    public:
        //MtChain qdomain;
        std::string domain; /*BS: String to store the domain name*/
        MtChain qcookie; /*BS: Chain to store the cookie*/
    public:
        MtDomainCookie(std::string &domain) :
            domain(domain)
        {
        }
        ; /* BS:
         * Constructor which
         * places the domain
         * name in the string
         * domain
         */
};

/*
 * BS:
 *----
 * The response packet from the server contains information about setting a
 * cookie.
 * This class contains the different fields of a cookie. This class represents
 * the cookie.
 *
 * Dependency:
 *------------
 * MtDomainCookie, MtCookieChain
 */
class MtSetCookie
{
    public:
        MtDomainCookie *domain; /*BS: The domain */
        std::string path; /*BS: The path*/

        /* BS:
         * URL: http://www.example.net/index.html
         * Domain name: www.example.net
         * Registered domain name: example.net
         * Path: /index.html
         */

        std::string name; /*BS: The name of the cookie given by the server*/
        std::string value; /*BS: Cookie value given by the server */
        MtExpires expires; /*BS: Expiry time given by the server*/
        bool secure; /*BS: Any security or encryptions*/
        std::string version; /*BS: */

        MtCookieChain qaccess; /*BS: Authorizations for accessing the cookie*/
        MtCookieChain qpath; /*BS: Path of the created cookie */

    public:
        MtSetCookie(MtDomainCookie *domaincookie, std::string &path,
                std::string &name, std::string &value, MtExpires expires,
                bool secure, std::string &version) :
            domain(domaincookie), path(path), name(name), value(value),
                    expires(expires), secure(secure), version(version),
                    qaccess(this), qpath(this)
        {
        }
        ;
        /*
         * BS:
         * Constructor which places different fields in the cookie.
         */
};

/*
 * BS:
 *----
 * The cookie handling object.
 * This is used to handle the creation and deletion of cookies.
 *
 *
 * Dependency:
 *------------
 * MtChain
 * MtMutex
 */
class MtCookie
{
    private:
        MtMutex mutex; /* BS: Used in deleting a cookie */
        MtChain qaccess; /* BS: Authorization */
        MtChain qdomain; /* BS: Domain name*/
    public:
        static const int MaxCookie = 1024;
        MtCookie(void);
        ~MtCookie(void);
        int setCookie(MtURL &url, std::string &setcookie);
        std::string getCookie(MtURL &url);
    private:
        void update(void);
        int setCookieString(MtURL &url, std::string &setcookie);
        std::string getCookieString(MtDomainCookie *domaincookie,
                std::string *path, bool secure);
        void deleteSetCookie(MtSetCookie *setcookie);
        MtDomainCookie *domainCookie(std::string &domain);
        MtDomainCookie *subdomainCookie(std::string &domain,
                MtDomainCookie * domaincookie);
};

#endif
