#ifndef __include_mturl_h__
#define __include_mturl_h__

#include <string>

/* BS:
 * Defining a class for the URL parsing
 * Scheme refers to http
 * Host is the host address
 * Path refers to the path after the host address
 * like http://google.com/about.html
 * Query refers to the information needed to query which is followed by "?"
 */
class MtURL
{
    public:
        bool hasScheme;
        bool hasHost;
        bool hasIPv6;
        bool hasPort;
        bool hasUsername;
        bool hasPassword;
        bool hasPath;
        bool hasQuery;
        bool hasAnchor;

        std::string scheme;//scheme here refers to http
        std::string host;
        std::string port;
        std::string username;
        std::string password;
        std::string path;
        std::string query;
        std::string anchor;

        /* BS:
         * Making the parse function private so that only members can access
         */
    private:
        void parse(const char *url);

    public:
        MtURL(void)
        {
            parse((const char*) "");
        }
        ;/* BS: constructor*/
        MtURL(const char *url)
        {
            parse(url);
        }
        ;/* BS: constructor*/
        MtURL(const std::string *url)
        {
            parse((const char*) url->c_str());
        }
        ;
        ~MtURL(void);
        /* BS: destructor*/

        /* BS:
         * Function to store the parsed address into a proper string.
         */
        std::string toString(void)
        {
            std::string urlString = "";
            if (hasScheme)
            {
                urlString += scheme + ":";
            }

            /* BS:
             * Checking if there is any host
             * Possible conditions of failing
             * For Ex if user gives a wrong address without host:
             *
             * http://com?help
             *
             * In the above case there is no host
             */
            if (hasHost)
            {
                /* BS:
                 * Appending "//" along with the scheme name http:
                 */
                urlString += "//";
                if (hasUsername)
                {
                    urlString += username;
                    if (hasPassword)
                    {
                        urlString += ":" + password;
                    }
                    urlString += "@";
                }
                if (hasIPv6)
                {
                    urlString += "[";
                }
                urlString += host;
                if (hasIPv6)
                {
                    urlString += "]";
                }
                if (hasPort)
                {
                    urlString += ":" + port;
                }
            }
            if (hasPath)
            {
                urlString += path;
            }
            if (hasQuery)
            {
                urlString += "?" + query;
            }
            if (hasAnchor)
            {
                urlString += "#" + anchor;
            }
            return (urlString);
        }
        ;

        /* BS: Operator overloading for "="*/
        MtURL &operator =(const std::string &a)
        {
            parse((const char*) a.c_str());
            return *this;
        }
        ;
        MtURL &operator =(const char *a)
        {
            parse(a);
            return *this;
        }
        ;
};

#endif
