/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Cavium, Inc.
 Cavium, Inc. reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium, Inc.

 ALL RIGHTS RESERVED.

 Copyright 2011-2012, Cavium, Inc.
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */

/**
 * @file cnc_json.h
 * @ingroup core
 * The CNC_JSON library defines a generic interface for JSON parsing and processing
 */

#ifndef __CNC_SETTING_UTILS_H__
#define __CNC_JSON_H__

#include <stdarg.h>

#include "cnc_common_typedef.h"
#include "cnc_util.h"
#include "jansson.h"

#ifdef __cplusplus
extern "C" {
#endif


/**
 * @brief                       : Opens json file and gets the handle
 * @param *p_jsonFileName       : [INPUT] Name of the json file to open
 * @param **pp_jsonFileHandle   : [OUTPUT] Pointer reference to hold the file handle
 * @return                      : RESULT_OK on success, Other then RESULT_OK on Failure
 */
Result_Type_e CNC_JSON_OpenFile(const char *p_jsonFileName, json_t **pp_jsonFileHandle);

/**
 * @brief                       : Close json file.
 * @param *p_jsonFileHandle     : [INPUT] Pointer reference which hold the file handle
 * @return                      : RESULT_OK on success, Other then RESULT_OK on Failure
 */
Result_Type_e CNC_JSON_CloseFile(json_t *p_jsonFileHandle);

/**
 * @brief                    : Load the json file and dump the json file with changes
 * @param  *p_jsonFileHandle : [INPUT] Reference pointer  to hold the file handle
 * @param  *p_jsonFilePath   : [INPUT] Reference  pointer to the json file path
 * @param  *p_key            : [INPUT] Required key, whose value is going to replace
 * @param  *p_newValue       : [INPUT] value to be replace
 * @return                   : RESULT_OK on success, Other then RESULT_OK on Failure
 */
Result_Type_e CNC_JSON_UpdateFile(json_t *p_jsonFileHandle, const CNC_CHAR  *p_jsonFilePath, CNC_CHAR *p_key, CNC_CHAR *p_newValue);

/**
 * @brief           : Take the JSON buffer as input and give the value associated with string (string:value)
 * @param  *p_data  : [INPUT] JSON buffer
 * @param  *p_param : [INPUT] String to search
 * @param  *p_str   : [OUTPUT] On success update value to this pointer
 * @param  strSize  : [INPUT] Array size of p_str.
 * @return          : RESULT_OK on success, Other then RESULT_OK on Failure
 */
Result_Type_e CNC_JSON_ReadValueForString(const char *p_data, const char *p_param, char *p_str, int strSize);


#ifdef __cplusplus
}
#endif

#endif //__CNC_SETTING_UTILS_H__
