/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Cavium, Inc.
 Cavium, Inc. reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium, Inc.

 ALL RIGHTS RESERVED.

 Copyright 2011-2012, Cavium, Inc.
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */

/**
 * @file cnc_os.h
 * @ingroup core
 * The CNCOS module provides OS related functions
 */

#ifndef  __CNC_OS_H__
#define  __CNC_OS_H__

#include <wait.h>
#include <pthread.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/msg.h>
#include <sys/param.h>
#include <signal.h>
#include <sys/time.h>
#include <semaphore.h>
#include <stdarg.h>
#include <sys/time.h>
#include <memory.h>
#include <mqueue.h>
#include <time.h>
#include <sys/socket.h> /* basic socket definitions */
#ifndef _LINUX_IF_H
#include <net/if.h>
#endif
#include <netinet/in.h>
#include <net/if_arp.h>
#include <netinet/in_systm.h>
#include <netinet/ip_icmp.h>
#include <net/route.h>
#include <netinet/ip.h>
#include <linux/netlink.h>
#include <linux/rtnetlink.h>
#include <linux/sockios.h>
#include <arpa/inet.h>  /* inet(3) functions */
#include <netdb.h>
#include <linux/fb.h>
#include <dirent.h>
#include <ctype.h>
#include <sys/mount.h>
#include "csapi.h"
#include "cnc_common_typedef.h"

#ifdef __cplusplus
extern "C" {
#endif

typedef struct _cncUTC_t
{
      CNC_UINT16 Mjd;
      CNC_UINT8 hh;
      CNC_UINT8 mm;
      CNC_UINT8 ss;
} cncUTC_t;

typedef struct _cncDateTime_t
{
      CNC_UINT16 year;
      CNC_UINT8 month;
      CNC_UINT8 day;
      CNC_UINT8 weekday;
      CNC_UINT8 hh;
      CNC_UINT8 mm;
      CNC_UINT8 ss;
} cncDateTime_t;

typedef enum
{
	FILE_SEEK_SET 		= 1, 
	FILE_SEEK_CUR 		= 2,
	FILE_SEEK_END 		= 3,
	FILE_TRUNCATE 		= 4,
	FILE_FLUSH 			= 5,
	FILE_GET_POSITION 	= 6,
	FILE_GET_CREATE_TIME  = 7,
	FILE_GET_MODIFY_TIME  = 8,
	FILE_GET_LENGTH	           =9
} cncOSFileIoctl_t;

typedef enum
{
	DIR_RENAMCE_NODE 	= 1
} cncOSDirIoctl_t;

typedef struct
{
	char *oldname;
	char *newname;
} cncOSRename_t;

typedef struct
{
	CNC_UINT32 flag; 			//0 ʾǸļڵ㣬1ʾĿ¼ڵ/
	char name[256]; 		//ڵַϼ·/
	void 	*handle; 			//ײʵ߹Ϣʹ/
} cncOSDir_t;

typedef struct
{
	const char *szfilename; //ļ
	int  ilength;//ļ
} cncOSFileLength_t;

typedef enum
{
	ACTIVE_TIMER, 
	DEACTIVE_TIMER
} cncOSTimerControl_t;


typedef unsigned int     		cncOSMessageQueue_t;

typedef sem_t 				cncOSSemaphore_t;
typedef unsigned int 			cncOSPartition_t;
typedef	 pthread_mutex_t    	cncOSMutex_t;


typedef pthread_t  			cncOSTaskHandle_t;
typedef unsigned int  			cncOSTaskDesc_t;        
typedef unsigned int 			cncOSTaskFlag_t;
typedef void* 				cncOSTimerHandle;
typedef void (*cncOSTimerCallback_t)(void *context, cncOSTimerHandle timer);


#define CNC_OS_AllocateMemory(partition,size)                 malloc(size)
#define CNC_OS_DeallocateMemory(partition,ptr)                free(ptr)
#define CNC_OS_InitSemaphore(sem_p,initval)		     sem_init(sem_p,0,initval)


Result_Type_e CNC_OS_InitSystemTime(void);
Result_Type_e CNC_OS_SetSystemTime(cncUTC_t *time);
Result_Type_e CNC_OS_GetSystemTime(cncUTC_t *time);

cncOSMessageQueue_t *CNC_OS_CreateMessageQueue(const char *Qname,
	unsigned int ElementSizeInByte, unsigned int NoElements);
Result_Type_e CNC_OS_DeleteMessageQueue(cncOSMessageQueue_t * MsgQueue_p);
Result_Type_e CNC_OS_ReleaseMessagebuffer(cncOSMessageQueue_t * MsgQueue_p, void * Msg_p);
void *CNC_OS_ReceiveMessage(cncOSMessageQueue_t * MsgQueue_p);
void *CNC_OS_ReceiveMessageTimeOut(cncOSMessageQueue_t * MsgQueue_p,unsigned int TimeOutMs);
Result_Type_e CNC_OS_ReceiveMessage2(cncOSMessageQueue_t * MsgQueue_p, 
	void *receiveBuff, CNC_UINT32 buffSizeInByte);
Result_Type_e CNC_OS_ReceiveMessageTimeOut2(cncOSMessageQueue_t * MsgQueue_p,
    void *receiveBuff, CNC_UINT32 buffSizeInByte, unsigned int TimeOutMs);
Result_Type_e CNC_OS_SendMessage(cncOSMessageQueue_t * MsgQueue_p,
	 void * Msg_p, unsigned int MsgSizeInByte, int Priority);

cncOSSemaphore_t * CNC_OS_CreateSemaphoreFifo           (cncOSPartition_t * Partition_p, const int InitialValue);
Result_Type_e CNC_OS_DeleteSemaphore(cncOSPartition_t * Partition_p, cncOSSemaphore_t * Semaphore_p);
Result_Type_e CNC_OS_SignalSemaphore(cncOSSemaphore_t * Semaphore_p);
Result_Type_e CNC_OS_WaitSemaphore(cncOSSemaphore_t * Semaphore_p);  /* Nothing to do*/
Result_Type_e CNC_OS_WaitSemaphoreTimeOut(cncOSSemaphore_t * Semaphore_p, unsigned int TimeOutMs);
Result_Type_e CNC_OS_GetSemaphoreValue(cncOSSemaphore_t * Semaphore_p, int *sval);
Result_Type_e CNC_OS_EnterCriticalSection(cncOSSemaphore_t *Lock_p);
Result_Type_e CNC_OS_ExitCriticalSection(cncOSSemaphore_t *Lock_p);

/*task Management*/
Result_Type_e CNC_OS_CreateTask (void *Function,
                                       void *Param,
                                       cncOSPartition_t *StackPartition,
                                       CNC_UINT32 StackSize,
                                       void *Stack,
                                       cncOSPartition_t* TaskPartition,
                                       cncOSTaskHandle_t* Task,
                                       cncOSTaskDesc_t** Tdesc,
                                       CNC_UINT32 Priority,
                                       char* Name,
                                       cncOSTaskFlag_t Flags );
Result_Type_e CNC_OS_SetTaskPriority(cncOSTaskHandle_t Task, int Priority);
Result_Type_e CNC_OS_ResumeTask  ( cncOSTaskHandle_t  Task);
Result_Type_e CNC_OS_StartTask  ( cncOSTaskHandle_t  Task);
Result_Type_e CNC_OS_DeleteTask (cncOSTaskHandle_t Task,
                                                                        cncOSPartition_t* TaskPartition);
cncOSTaskHandle_t CNC_OS_GetCurrentTaskHandle(void);

/*  Mutex Management */
cncOSMutex_t * CNC_OS_CreateMutexFifo(void);
Result_Type_e CNC_OS_DeleteMutex(cncOSMutex_t * Mutex_p);
Result_Type_e CNC_OS_LockMutex(cncOSMutex_t * Mutex_p);
Result_Type_e CNC_OS_UnlockMutex(cncOSMutex_t * Mutex_p);

/*system*/
void CNC_OS_DelayTaskMs(int millisecon);
#define CNC_OS_Malloc malloc
#define CNC_OS_Calloc calloc
#define CNC_OS_Free   free
#define CNC_OS_Memset memset
#define CNC_OS_Memcpy memcpy
//void* CNC_OS_Malloc( U32 byBufSize );
//void CNC_OS_Free( void* pBuf );
//void CNC_OS_Memset( void* pBuf ,char value,int size);
//void CNC_OS_Memcpy( void* pDest , const void* pSrc, int size);

/*file*/
CNC_UINT32 CNC_OS_CSPFileOpen(const char *filename, const char *mode);
CNC_SINT32 CNC_OS_CSPFileRead(CNC_UINT32 fd, CNC_BYTE *buffer, CNC_SINT32 nbytes);
CNC_SINT32 CNC_OS_CSPFileWrite(CNC_UINT32 fd, const CNC_BYTE *buffer, CNC_SINT32 nbytes);
Result_Type_e CNC_OS_CSPFileClose(CNC_UINT32 fd);
Result_Type_e CNC_OS_CSPFileDelete(const char *name);
Result_Type_e CNC_OS_CSPFileCopy(char *dst, const char *src);
Result_Type_e CNC_OS_CSPFileClean(char * path);
CNC_SINT32 CNC_OS_CSPFileIoctl(CNC_UINT32 fd, cncOSFileIoctl_t op, void *arg);


/**
 * Creates a timer
 *
 * The timer is created disabled. To enable it call CNC_OS_TimerControl().
 *
 * @note The name pointer may not be valid on return from the call. If the
 *		 host requires this string then it must be duplicated.
 *
 * @param[in] name A pointer to a name for the timer
 *			  It may not be NULL and is better to be unique for debug.
 * @param[in] countMs The intial timeout value. A value of 0 will cause the
 *			  timer to fire in CS_MIN_TIMER_INTERVAL ticks.
 * @param[in] callback A callback function to invoke when the timer fires, 
 *			  it may not be NULL.
 * @param[in] callbackContext A pointer to be passed to the callback when it is
 *			  invoked.
 *
 * @return NULL if the timer can not be created
 * @return A pointer to the created timer instance
 */
cncOSTimerHandle CNC_OS_TimerCreate(const char *name,
                             CNC_UINT32 countMs,
                             cncOSTimerCallback_t callback,
                             void *callbackContext);

/**
 * Releases a timer instance
 *
 * Timers should be disabled before they are released, though it is not
 * an error if they are not.
 *
 * @param[in] thiz The timer
 */
Result_Type_e CNC_OS_TimerRelease(cncOSTimerHandle thiz);

/**
 * Sets the counter for the timer.
 *
 * The timer must be disabled to change the timeout counter.
 *
 * @param[in] thiz The timer
 * @param[in] ms The new timeout value in ms
 */
Result_Type_e CNC_OS_TimerSetCount(cncOSTimerHandle thiz, CNC_UINT32 ms);

/**
 * Enables and disables a timer
 *
 * @param[in] thiz The timer
 * @param[in] ctrl active run or pause the timer.
 */
Result_Type_e CNC_OS_TimerControl(cncOSTimerHandle thiz, cncOSTimerControl_t ctrl);

/**
 * Initialize the timer.
 */
Result_Type_e CNC_OS_TimerInit(void);

/*
 * From Modii Time Mgr implementation
 */

#define CNC_F_SIGNAL_REGISTER       0x0001
#define CNC_F_TASK_EXIT             0x0002
#define CNC_F_CHILD_EXIT_DONE       0x0004

#define CNC_TIMER_CYCLE_USEC        10000L
#define CNC_TIMER_CYCLE_SEC         0L

#define CNC_TF_BUSY                 0x0001
#define CNC_TF_CYCLIC               0x0002
#define CNC_TF_CALLBACK             0x0004

#if CNC_TIMER_CYCLE_USEC <= 1000L
#define CNC_TM_1MSEC                1000/CNC_TIMER_CYCLE_USEC
#else
#define CNC_TM_1MSEC                1
#endif

#define CNC_TM_10MSEC               (CNC_TM_1MSEC*10)
#define CNC_TM_100MSEC              (CNC_TM_1MSEC*100)
#define CNC_TM_1SEC                 (CNC_TM_1MSEC*1000)
#define CNC_TM_1MIN                 (CNC_TM_1SEC*60)

/* =========================== Type definitions =========================== */
typedef struct S_osalTMmgrDataTag
{
    pthread_mutex_t     s_mutex;
    pthread_t           s_childThreadId;
    CNC_UINT16         u16_flags;
    CNC_SINT32         s32_myTaskId;
    CNC_CHAR           *p_ch_myname;
    void                *pv_arg;
#ifdef CNC_ENABLE_HIGH_RESOLUTION_TIMER
    timer_t             s_highResolutionTimerId;
#endif
} S_osalTMmgrData_t;


typedef struct S_osalTimerEntryTag
{
    CNC_SINT32         s32_timerId;
    CNC_UINT32         u32_flags;
    CNC_UINT32         u32_ctc;
    CNC_UINT32         u32_tmoIms;
    pthread_mutex_t     s_mutex;
    void                (*p_callBack)(int, void *);
    void                *pv_arg;
} S_osalTimerEntry_t;

typedef enum E_osalTimerTypeTag
{
   CNC_TIMER_TYPE_ONE_SHOT = 0,
   CNC_TIMER_TYPE_CYCLIC
} E_osalTimerType_t;

typedef enum E_osalTimerErrCodeTag
{
    CNC_TIMER_SUCCESS = 0,         /* SUCCESS or Timer ID */
    CNC_ERR_TIMER_FULL = -1,
    CNC_ERR_TIMER_NOID = -103,
    CNC_ERR_TIMER_INVALID,
    CNC_ERR_TIMER_NOTREADY,
    CNC_ERR_TIMERTHREAD_CREATION
} E_osalTimerErrCode_t;


/* ===================== Global variable declarations ===================== */
// NONE


/* ======================== Function declarations ========================= */
/* When using Osal Timers, make sure to call the osal_timerInit from a thread
 * instead of directly called from main() thread. sleep() and usleep() will not work
 * in main thead after osal_timerInit is called. But, sleep and usleep will
 * continue to work properly on all other threads. Root cause: osal_timerInit
 * modifies the SIGALRM which affects the functioning of sleep and usleep */
extern E_osalTimerErrCode_t osal_timerInit(CNC_SINT32 maxTimerCount);
extern CNC_SINT32 osal_timerStart(CNC_UINT32 u32_timerDurationInMsec, E_osalTimerType_t e_timerType,
                                   void *pv_arg, void (*p_callBack)(CNC_SINT32, void *));
extern E_osalTimerErrCode_t osal_timerCancel(CNC_SINT32 s32_timerId);
extern E_osalTimerErrCode_t osal_timerDeInit(void);

#ifdef __cplusplus
} 
#endif 

#endif
