/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Cavium, Inc.
 Cavium, Inc reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium, Inc.

 ALL RIGHTS RESERVED.

 Copyright 2012, Cavium, Inc.
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */

/**
 * Defines the APIs for Soft Demuxer based on FFMPEG
 * @file cnc_softdemux.h
 * @ingroup core
 */

#ifndef _CNC_SOFT_DEMUX_H_
#define _CNC_SOFT_DEMUX_H_

/* =============================== System Includes ================================= */
#include <stdio.h>
#include <sys/stat.h>
/* =============================== Includes ================================= */
#include "cnc_common_typedef.h"
#include "cnc_mediaformat.h"
#include "cnc_mediasource.h"
#include "cnc_common_typedef.h"

#include "libavformat/avformat.h"
#include "libavutil/mathematics.h"
#include "libavutil/avstring.h"
#include "libavutil/avutil.h"
#include "libavfilter/avfilter.h"
#include "libavfilter/avfiltergraph.h"
#include "libavfilter/buffersink.h"
#include "libavfilter/vsrc_buffer.h"
#include "libavfilter/avcodec.h"
#include "libswscale/swscale.h"
#include "libavutil/pixdesc.h"
#include "libavutil/mem.h"
#include "libavutil/opt.h"

#include "cnc_http.h"
#include "cnc_trace.h"

#ifdef __cplusplus
extern "C" {
#endif

/* =============================== Macros ================================= */
#define VORBIS_HEADER_SIZE                 1024*20
#define CSM_MIN_PTS                       (2ll)
#define DEFAULT_VIDEO_TRACK                0
#define DEFAULT_AUDIO_TRACK                0
#define DEFAULT_SUB_TRACK                  0
#define DEFAULT_DATA_TRACK                 0
#define MAXLEN                             60  /* TODO: it should be MAX Media Format Enum Type - Please Correct it */
#define MEDIA_FORMAT_CTRL_LOC             "/cavium/configs/"
#define MEDIA_FORMAT_CTRL_FILE            "cnc_media_format_control.json"
#define tolerability                       0.2
#define FFMPEG_MAX_HEAD_DATA_SIZE          200//!< the size limit of header data
#define H264_NALU_LENTH                    4  //!< the NALU size in byte of h.264 stream


#ifndef PUT_BYTE
#define PUT_BYTE(_p, _b) \
	*_p++ = (unsigned char)_b; 
#endif

#ifndef PUT_BUFFER
#define PUT_BUFFER(_p, _buf, _len) \
	memcpy(_p, _buf, _len); \
_p += _len;
#endif

#ifndef PUT_LE64
#define PUT_LE64(_p, _var) \
	*_p++ = (unsigned char)((_var)>>0);  \
	*_p++ = (unsigned char)((_var)>>8);  \
	*_p++ = (unsigned char)((_var)>>16); \
	*_p++ = (unsigned char)((_var)>>24); \
	*_p++ = (unsigned char)((_var)>>32); \
	*_p++ = (unsigned char)((_var)>>40); \
	*_p++ = (unsigned char)((_var)>>48); \
	*_p++ = (unsigned char)((_var)>>56); 
#endif

#ifndef PUT_LE32
#define PUT_LE32(_p, _var) \
	*_p++ = (unsigned char)((_var)>>0);  \
	*_p++ = (unsigned char)((_var)>>8);  \
	*_p++ = (unsigned char)((_var)>>16); \
	*_p++ = (unsigned char)((_var)>>24); 
#endif

#ifndef PUT_BE32
#define PUT_BE32(_p, _var) \
	*_p++ = (unsigned char)((_var)>>24); \
	*_p++ = (unsigned char)((_var)>>16); \
	*_p++ = (unsigned char)((_var)>>8);  \
	*_p++ = (unsigned char)((_var)>>0); 
#endif

#ifndef PUT_LE16
#define PUT_LE16(_p, _var) \
	*_p++ = (unsigned char)((_var)>>0);  \
	*_p++ = (unsigned char)((_var)>>8);  
#endif

#ifndef PUT_BE16
#define PUT_BE16(_p, _var) \
	*_p++ = (unsigned char)((_var)>>8);  \
	*_p++ = (unsigned char)((_var)>>0);  
#endif

#ifndef AV_RB16
#   define AV_RB16(x)                           \
    ((((const uint8_t*)(x))[0] << 8) |          \
      ((const uint8_t*)(x))[1])
#endif

/* ===================== Typedef definitions ====================== */
typedef enum _cncSoftDemuxerEventType_t
{
    CNC_SOFTDEMUXER_EVENT_NONE   = 0,
    CNC_SOFTDEMUXER_EVENT_INTERRUPT,
    CNC_SOFTDEMUXER_EVENT_SIDX_PARSED,
    CNC_SOFTDEMUXER_EVENT_GET_PTS,
} cncSoftDemuxerEventType_t;

typedef struct _cncSoftDemuxerEventInfo_t
{
    cncSoftDemuxerEventType_t  eventType;
    CNC_UINT64                 pts;
    CNC_VOID                  *p_v_streamHandle;  // Need to remove this
    CNC_VOID                  *p_v_softDemuxerHandle; 
} cncSoftDemuxerEventInfo_t;

typedef E_cncErrCode_t (*F_cncSoftDemuxerEventCbf_t)(CNC_VOID *p_v_handle, cncSoftDemuxerEventInfo_t *p_eventInfo);

typedef struct _cncSoftDemuxerEventCallback_t
{
    F_cncSoftDemuxerEventCbf_t  p_f_callback;
    CNC_VOID                   *p_v_handle;
} cncSoftDemuxerEventCallback_t;

// This is the enumeration of play mode.
typedef enum {
    FFMPEG_FAST_NORMAL = 0,
    FFMPEG_FAST_FORWARD,
    FFMPEG_FAST_BACKWARD
} CNC_Media_FastPlayMode_e;

// Enumerate the data packet type here.
typedef enum {
    PACKET_TYPE_VID = 0,
    PACKET_TYPE_AUD,
    PACKET_TYPE_SUB,
    PACKET_TYPE_SCRIPT,
    PACKET_TYPE_DATA,
    PACKET_TYPE_UNKNOWN
} CNC_Media_PacketType_e;

#ifdef AVSYNC_DBG
typedef struct tagRecord
{
    CNC_UINT32                             audio_offset;
    CNC_UINT32                             audio_frame_size;
    CNC_UINT32                             audio_DTS;
    CNC_UINT32                             audio_PTS;
} Record_t;
static FILE                                *hsyncVid = NULL;
static FILE                                *hsyncAud = NULL;
static Record_t                            vRecord = {0, 0, 0, 0};
static Record_t                            aRecord = {0, 0, 0, 0};
#endif

typedef struct _ADTSContext
{
    CNC_SINT32                             write_adts;
    CNC_SINT32                             objecttype;
    CNC_SINT32                             sample_rate_index;
    CNC_SINT32                             channel_conf;
}ADTSContext;

typedef struct _ffMpeg_Stream_Info_Tag
{
    CNC_SINT64                             first_pts;
    CNC_SINT64                             offset_pts;
    CNC_SFLOAT64                           time_scale;
    CNC_SFLOAT64                           fps;
    CNC_SINT32                             stream_index;
    CNC_SINT32                             profile;
    cncMediaFormat_t                       format;
    CNC_SINT32                             width;  // width and height just for video,bit rate for audio
    CNC_SINT32                             height; //channel num for audio
    CNC_SINT32                             s32_audSampleRate; /* To Populate Audio Sample Rate */
    CNC_SINT32                             s32_audChannels;   /* To Populate Audio Channels */
    CNC_UINT16                             pid;
    cncAudioSampleFormat_t                 e_sampleFormat;    /* To Populate Audio Sample Format s16, s32 - TODO: Convert this into Enum */
    cncMediaMetaData_t                     ffmpeg_meta_data;
    CNC_UINT8                              u8_bigEndian;
    CNC_SINT32                             bit_rate;
    CNC_SINT32                             es_bit_rate;
    S_cncRational_t                        s_sampleAspectRatio;
    S_cncRational_t                        s_displayAspectRatio;
    CNC_BOOL                               b_isPtsOverFlow;
    CNC_SINT64                             prevMaxPts;
    CNC_SINT64                             prev_pts;
    struct timeval                         ptsOverflowTime; //Used after pts overflow
} ffMpeg_Stream_Info_t;

typedef struct _ffMpeg_Demuxer_Info_Tag
{
    CNC_SINT32                             status;   //0==idle;1==run;
    cncMediaFormat_t                       e_mediaFormat;
    cncMediaClass_t                        media_type;
    CNC_SINT32                             vid_cur_track;
    CNC_SINT32                             aud_cur_track;
    CNC_SINT32                             sub_cur_track;
    CNC_SINT32                             data_cur_track;
    CNC_SINT32                             vid_track_num;
    CNC_SINT32                             aud_track_num;
    CNC_SINT32                             sub_track_num;
    CNC_SINT32                             data_track_num;
    ffMpeg_Stream_Info_t                   vid_stream[CNC_MEDIA_VID_MAX_STREAM_NUM];
    ffMpeg_Stream_Info_t                   aud_stream[CNC_MEDIA_AUD_MAX_STREAM_NUM];
    ffMpeg_Stream_Info_t                   sub_stream[CNC_MEDIA_SUB_MAX_STREAM_NUM];
    ffMpeg_Stream_Info_t                   data_stream[CNC_MEDIA_DATA_MAX_STREAM_NUM];
    CNC_BOOL                               is_H264_in_MP4;
    CNC_BOOL                               is_AVS_in_MP4;
    CNC_BOOL                               is_VC1_in_WMV;
    CNC_BOOL                               is_AAC_in_Aud;
    CNC_BOOL                               is_WMA_in_Aud;
    CNC_BOOL                               is_PCM_in_Aud;
    CNC_BOOL                               is_COOK_in_Aud;
    CNC_BOOL                               is_VC1_in_MKV;
//  CNC_BOOL                               is_RV_in_Rmvb;
    CNC_SINT32                             ext_codec;
    AVPacket                               pkt;
    CNC_SINT32                             nal_length;
    CNC_SINT32                             has_nalu_header;
#ifndef FAST_FRAMES_SEND
    CNC_SINT32                             bRemnants;
    CNC_SINT32                             ReadOff;
#endif
    CNC_SINT32                             switch_aud;
    CNC_UINT32                             lasttime; //ms
    ADTSContext                            aac_config;
    CNC_SINT32                             bvidhead;
    CNC_SINT32                             baudhead;
    CNC_UINT8                             *pFrame;
    CNC_BOOL                               b_isWMAStreamSeekNotSupported;
} ffMpeg_Demxuer_Info_t;

typedef struct _ffMpeg_FastPlay_Support_Tag
{
    CNC_SINT64                             pts[2];
    CNC_SINT64                             system_time[2];
    CNC_SINT64                             delta_pts;
    CNC_SINT64                             delta_sys_time;
    CNC_SINT64                             maxstamp;
    CNC_SINT64                             seekstamp;
    CNC_SINT64                             last_I_offset;//nearest I key frame offset in TS&M2TS file
    CNC_SINT64                             cur_ptr;
    CNC_SINT32                             show_pb_frame_enable; /*pb means P frame & B frame*/
    CNC_SINT32                             show_pb_frame_index;
    CNC_SINT32                             show_pb_frame_max_num;
    CNC_Media_FastPlayMode_e               mode;
    CNC_Media_FastPlayMode_e               lastmode;
    CNC_SINT32                             step;          //time in Sec
    CNC_SINT32                             speed;
    CNC_SINT32                             vid_iframe_count;
    CNC_SINT32                             sleep_time; //time*100ms = step*1000/speed. meida player get frame every 100ms.
    CNC_SINT32                             pts_valid_flag;
    CNC_SINT32                             key_valid_flag;
    CNC_SINT32                             find_2nd_field;
    CNC_SINT32                             finding_2nd_field;
    CNC_SINT32                             jumping;
} ffmpeg_FastPlay_Support_t;

typedef struct S_mediaStoreTag
{
    CNC_SINT32                             a_s32_unSupportedFormat[MAXLEN];
    CNC_SINT32                             s32_noOfUnsupportedContainerFormatCnt;
    CNC_SINT32                             s32_noOfUnsupportedVideoFormatCnt;
    CNC_SINT32                             s32_noOfUnsupportedAudioFormatCnt;
    CNC_SINT32                             s32_noOfUnsupportedImageFormatCnt;
    CNC_SINT32                             s32_noOfUnsupportedFormatCnt;
    CNC_SINT32                             s32_isJsonFileExists;
    CNC_SINT8                              s8_isDx50NotSupported;
    CNC_SINT8                              s8_isDx60NotSupported;
    CNC_SINT8                              s8_isDivXNotSupported;
    CNC_SINT8                              s8_isXvidNotSupported;
} S_mediaStore_t;

#if 0
/*
 * Fixed or Variable Frame Rate Specifier
 */
typedef enum E_videoFrameTypeTag {
    VIDEO_FRAMES_FIXED = 0,
    VIDEO_FRAMES_VARIABLE
} E_videoFrameType_t;
#endif
/**
*This is the main structure of frame.
*/
typedef struct CNC_Media_Frame_t_ {
    CNC_Media_PacketType_e                packtype;    //!< the packet type, see CNC_Media_PacketType_e
    CNC_UINT8                            *framedata;   //!< the address  of data
    CNC_UINT32                            framesize; //!<the frame size, without header data
    CNC_UINT8                            *headerdata;  //!< the address of header data
    CNC_UINT32                            headersize;    //!< size of header data
    CNC_SINT64                            pts;    //pts of this frame
    CNC_BOOL                              isItKeyFrame;	/* Indicates whether the current frame is keyframe or not. This is valid only for video frames */
} CNC_Media_Frame_t;

typedef struct FFmpegAVFormatMappingTag
{
    cncMediaFormat_t                      format;
    CNC_CHAR                             *p_ch_fmtName;
} FFmpegAVFormatMapping_t;

typedef struct trackMappingTableInfoTag
{
    CNC_SINT8                             trackIndex;
    CNC_SINT8                             trackNum;
    cncMediaDataType_t                    trackType;
}trackMappingTableInfo_t;

/**
 * Core Object of cncSoftDemuxer
 */
typedef struct CNC_SoftDemuxerTag
{
    AVFormatContext                      *pFormatCtx;

    CNC_VOID                             *p_httpClientHandle;
    cncMediaFormatInfo_t                 *p_mediaFormatInfo;
    cncMediaSourceInfo_t                 *p_mediaSourceInfo;
    ffmpeg_FastPlay_Support_t             fastPlaySupportMode;
    ffMpeg_Demxuer_Info_t                 ffMpegDemuxerInfo;

    CNC_UINT8                             first_I_flag;
    CNC_UINT8                             predata[48];
    CNC_UINT8                             nalu[H264_NALU_LENTH];
    CNC_UINT8                             vidheaderdata[FFMPEG_MAX_HEAD_DATA_SIZE];
    CNC_UINT8                             audheaderdata[FFMPEG_MAX_HEAD_DATA_SIZE];
    CNC_SINT32                            key_frame_idx;
    CNC_SINT32                            vidheadercount;
    CNC_SINT32                            audheadercount;
    CNC_BOOL                              bVid_index_set;
    CNC_BOOL                              bAud_index_set;
    CNC_SINT32                            pcm_bit_reverse_flag;
    CNC_UINT8                             vorbis_header[VORBIS_HEADER_SIZE];
    CNC_SINT32                            s32_isItVariableFrame;
    CNC_SINT32                            rm_cnt;
    CNC_BOOL                              isCustomIo;
    CNC_BOOL                              isVideoStream;
    CNC_BOOL                              isAudioStream;
    cncSoftDemuxerEventCallback_t         eventCallback;
    CNC_VOID                             *p_v_streamHandle;         // TODO: Need to remove this 
    CNC_BOOL                              b_mediaSegmentChanged;
    //cncSegmentIndexTableList_t          masterSegmentIndexList;   // Internal to softdemuxer
    cncSegmentIndexTableList_t            segmentIndexList;
    trackMappingTableInfo_t               trackMappingTable[CNC_MEDIA_VID_MAX_STREAM_NUM + CNC_MEDIA_AUD_MAX_STREAM_NUM + CNC_MEDIA_SUB_MAX_STREAM_NUM + CNC_MEDIA_DATA_MAX_STREAM_NUM];
    CNC_SINT8                             numOfTracks;
}cncSoftDeMuxerObj_t;

/******************************************************************************
 * Function Prototypes
 ******************************************************************************/
CNC_VOID CNC_SoftDemuxer_Init();

CNC_VOID CNC_SoftDemuxer_DeInit();

E_cncErrCode_t CNC_SoftDemuxer_Open(CNC_VOID **pp_v_handle);

CNC_SINT32 CNC_SoftDemuxer_Close(CNC_VOID *p_v_handle);

CNC_SINT32 CNC_SoftDemuxer_GetHeaderData(CNC_VOID *p_v_handle, CNC_Media_PacketType_e type, CNC_UINT8 **pHeadData);

CNC_SINT32 CNC_SoftDemuxer_GetMediaInfo(CNC_VOID *p_v_handle,  cncMediaTrackList_t *p_trackList, cncSubtitleInfoAndData_t *p_subtitleInfo);

CNC_SINT32 CNC_SoftDemuxer_GetMediaFrame(CNC_VOID *p_v_handle, CNC_Media_Frame_t *pAVFrame);

cncMediaFormat_t CNC_SoftDemuxer_GetMediaType(CNC_VOID *p_v_handle, CNC_CHAR *filename);

CNC_VOID CNC_SoftDemuxer_FreeMediaFrame(CNC_VOID *p_v_handle, CNC_Media_Frame_t *pAVFrame);

CNC_SINT32 CNC_SoftDemuxer_SwitchAudio(CNC_VOID *p_v_handle, CNC_SINT32 aud_track_index);

CNC_SINT32 CNC_SoftDemuxer_Flush(CNC_VOID *p_v_handle);

CNC_SINT32 CNC_SoftDemuxer_SetSeekable(CNC_VOID *p_v_handle, CNC_SINT32 s32_isSeekable);

CNC_SINT32 CNC_SoftDemuxer_GetSeqHdr(CNC_VOID *p_v_handle, CSVID_SequenceHeader *ghdr);

CNC_SINT32 CNC_SoftDemuxer_Seek_Audio(CNC_VOID *p_v_handle, CNC_SINT64 s64_absSeekOffset, CNC_SINT32 s32_isByteSeekFlag);

CNC_SINT32 CNC_SoftDemuxer_SwitchVideo(CNC_VOID *p_v_handle, CNC_SINT32 vid_track_index);

CNC_SINT32 CNC_SoftDemuxer_SetFastPlay(CNC_VOID *p_v_handle, CNC_Media_FastPlayMode_e mode, CNC_SINT32 speed __attribute__((unused)), CNC_SINT64 startpts, CNC_UINT8 smooth);

CSVID_FRAME_RATE CNC_SoftDemuxer_GetFrameRateFromFps(CNC_VOID *p_v_handle, CNC_SFLOAT64 fps);

CNC_SINT32 CNC_SoftDemuxer_SetPTS(CNC_VOID *p_v_dstHandle, CNC_VOID *p_v_srcHandle, CNC_SINT64 offsetPts);

CNC_SINT32 CNC_SoftDemuxer_StreamOpen(CNC_VOID                         *p_v_handle, 
                                       CNC_CHAR                        *p_filename,
                                       CNC_VOID                        *p_v_deviceHandle,
                                       CNC_SINT32                     (*p_f_deviceRead) (CNC_VOID * opaque, CNC_UINT8 * buf, CNC_SINT32 buf_size),
                                       CNC_SINT64                     (*p_f_deviceByteSeek) (CNC_VOID * opaque, CNC_SINT64 offset, CNC_SINT32 whence),
                                       CNC_SINT64                     (*p_f_deviceTimeSeek) (CNC_VOID * opaque, CNC_SINT32 streamIndex, CNC_SINT64 timestamp, CNC_SINT32 flags),
                                       cncMediaFormatInfo_t            *p_mediaFormatInfo,
                                       cncMediaSourceInfo_t            *p_mediaSourceInfo,
                                       cncSoftDemuxerEventCallback_t   *p_eventCbf);

CNC_SINT32 CNC_SoftDemuxer_GetAlbumArt(CNC_VOID *p_v_handle, cncMediaFormatInfo_t *p_mediaFormatInfo, CNC_CHAR *p_ch_picType);

CNC_SINT32 CNC_SoftDemuxer_FreeAlbumArt(CNC_VOID *p_v_handle);

CNC_SINT32 CNC_SoftDemuxer_Seek_By_Stream(CNC_VOID *p_v_handle, CNC_SINT64 s64_absSeekOffset, CNC_SINT32 s32_isByteSeekFlag,CNC_BOOL stream_type);

CNC_SINT32 CNC_SoftDemuxer_TrickPlay(CNC_VOID *p_v_handle, cncTrickInfo_t *trickInfo);

CNC_SINT32 CNC_SoftDemuxer_SeekSupportStatus(CNC_VOID *p_v_handle, CNC_BOOL *p_isSeekSupported);

/**
 * @brief         Probes the Media format of file
 * @param [IN]    p_fileName        : Content which format should be required.
 * @param [OUT]   p_mediaFormatInfo :will be updated by p_fileName format details.
 * @see           cncMediaFormatInfo_t
 * @return
 */
CNC_SINT32 CNC_SoftDemuxer_ProbeMediaFormatDetails(CNC_CHAR *p_fileName, cncMediaFormatInfo_t   *p_mediaFormatInfo);

#ifdef __cplusplus
}
#endif

#endif // _CNC_SOFT_DEMUX_H_
