/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Cavium, Inc.
 Cavium, Inc reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium, Inc.

 ALL RIGHTS RESERVED.

 Copyright 2012, Cavium, Inc.
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */

/**
 * @ingroup media
 * @file cnc_subtitle.h
 * Data types for managing media subtitle data
 */

#ifndef _CNC_COMMON_SUBTITLE_H_
#define _CNC_COMMON_SUBTITLE_H_

#ifdef __cplusplus
extern "C" {
#endif

#include "cnc_common_typedef.h"

#define CNC_SUBTITLE_MAX_URL_LENGTH           (1024)

/* ----------------------- Supported Subtitle TYPE ----------------------------- */
typedef enum _cncSubtitleType_t
{
    CNC_SUBTITLE_TYPE_NONE        = 0,
    CNC_SUBTITLE_TYPE_SRT,
    CNC_SUBTITLE_TYPE_TTML,
    CNC_SUBTITLE_TYPE_DVB,
    CNC_SUBTITLE_TYPE_SMPTE_TT,
    CNC_SUBTITLE_TYPE_WEB_VTT
} cncSubtitleType_t;

typedef enum _cncSubtitleInputMethodType_t
{
    CNC_SUBTITLE_INPUT_METHOD_NONE =0,
    CNC_SUBTITLE_INPUT_METHOD_FILE,                         /* Input is complete single file */
    CNC_SUBTITLE_INPUT_METHOD_FILE_CHUNK,                   /* Input is received as chunk of separate well formed files */
    CNC_SUBTITLE_INPUT_METHOD_HTTP,                         /* Input is complete single http url eg. srt, ttml */
    CNC_SUBTITLE_INPUT_METHOD_HTTP_CHUNK,                   /* Input is received as chunk of separate well formed http urls. eg. hls+webvtt */
    CNC_SUBTITLE_INPUT_METHOD_CHUNK                         /* Tuner reads and gives chunk of well formed input data. eg. SS tuner + ttml */
} cncSubtitleInputMethodType_t;

typedef struct _cncSubtitleFileInput_t
{
    CNC_CHAR    filePath[CNC_SUBTITLE_MAX_URL_LENGTH];
} cncSubtitleFileInput_t;

typedef struct _cncSubtitleHTTPInput_t
{
    CNC_CHAR    url[CNC_SUBTITLE_MAX_URL_LENGTH];
} cncSubtitleHTTPInput_t;

typedef union _cncSubtitleInput_t
{
    cncSubtitleFileInput_t      fileParameters;
    cncSubtitleHTTPInput_t      httpParameters;
} cncSubtitleInput_t;

typedef struct _cncDirectFBInterface_t
{
    CNC_VOID            *dfb;
    CNC_VOID            *p_primaryDFBsurface;
    CNC_SINT32           width;
    CNC_SINT32           height;
} cncDirectFBInterface_t;

typedef enum _cncSubtitlePresentationStyle_t
{
    CNC_SUBTITLE_PRESENTATION_STYLE_NONE = 0,
    CNC_SUBTITLE_PRESENTATION_STYLE_POP_ON,
    CNC_SUBTITLE_PRESENTATION_STYLE_PAINT_ON,
    CNC_SUBTITLE_PRESENTATION_STYLE_ROLL_UP
} cncSubtitlePresentationStyle_t;

typedef enum _cncSubtitleColor_t
{
    CNC_SUBTITLE_COLOR_NONE = 0,
    CNC_SUBTITLE_COLOR_WHITE,
    CNC_SUBTITLE_COLOR_BLACK,
    CNC_SUBTITLE_COLOR_RED,
    CNC_SUBTITLE_COLOR_GREEN,
    CNC_SUBTITLE_COLOR_BLUE,
    CNC_SUBTITLE_COLOR_YELLOW,
    CNC_SUBTITLE_COLOR_MEGENTA,
    CNC_SUBTITLE_COLOR_CYAN,
} cncSubtitleColor_t;

typedef enum _cncSubtitleCharacterOpacity_t
{
    CNC_SUBTITLE_CHARACTER_OPACITY_NONE = 0,
    CNC_SUBTITLE_CHARACTER_OPACITY_OPAQUE,
    CNC_SUBTITLE_CHARACTER_OPACITY_SEMI_TRANSPERENT
} cncSubtitleCharacterOpacity_t;

typedef enum _cncSubtitleCaptionOpacity_t
{
    CNC_SUBTITLE_CAPTION_OPACITY_NONE = 0,
    CNC_SUBTITLE_CAPTION_OPACITY_OPAQUE,
    CNC_SUBTITLE_CAPTION_OPACITY_SEMI_TRANSPERENT,
    CNC_SUBTITLE_CAPTION_OPACITY_TRANSPERENT
} cncSubtitleCaptionOpacity_t;

typedef enum _cncSubtitleCharacterSize_t
{
    CNC_SUBTITLE_CHARACTER_SIZE_NONE = 0,
    CNC_SUBTITLE_CHARACTER_SIZE_50P,
    CNC_SUBTITLE_CHARACTER_SIZE_100P,
    CNC_SUBTITLE_CHARACTER_SIZE_150P,
    CNC_SUBTITLE_CHARACTER_SIZE_200P,
} cncSubtitleCharacterSize_t;

typedef enum _cncSubtitleFontFamily_t
{
    CNC_SUBTITLE_FONT_FAMILY_NONE = 0,
    CNC_SUBTITLE_FONT_FAMILY_DEFAULT,
    CNC_SUBTITLE_FONT_FAMILY_MONOSPACE_SERIF,
    CNC_SUBTITLE_FONT_FAMILY_PROPORTIONAL_SERIF,
    CNC_SUBTITLE_FONT_FAMILY_MONOSPACE_SANS_SERIF,
    CNC_SUBTITLE_FONT_FAMILY_PROPORTIONAL_SANS_SERIF,
    CNC_SUBTITLE_FONT_FAMILY_CASUAL,
    CNC_SUBTITLE_FONT_FAMILY_CURSIVE,
    CNC_SUBTITLE_FONT_FAMILY_SMALL_CAPS
} cncSubtitleFontFamily_t;

typedef enum _cncSubtitleCharacterEdge_t
{
    CNC_SUBTITLE_CHARACTER_EDGE_NONE = 0,
    CNC_SUBTITLE_CHARACTER_NONE_EDGE,
    CNC_SUBTITLE_CHARACTER_RAISED_EDGE,
    CNC_SUBTITLE_CHARACTER_DEPRESSED_EDGE,
    CNC_SUBTITLE_CHARACTER_UNIFORM_EDGE,
    CNC_SUBTITLE_CHARACTER_DROP_SHADOWED_EDGE
} cncSubtitleCharacterEdge_t;

typedef enum _cncSubtitleCaptionWindowOpacity_t
{
    CNC_SUBTITLE_CAPTION_WINDOW_OPACITY_NONE = 0,
    CNC_SUBTITLE_CAPTION_WINDOW_OPACITY_OPAQUE,
    CNC_SUBTITLE_CAPTION_WINDOW_OPACITY_SEMI_TRANSPERENT,
    CNC_SUBTITLE_CAPTION_WINDOW_OPACITY_TRANSPERENT
} cncSubtitleCaptionWindowOpacity_t;

typedef struct _cncSubtitleFccAttributes_t
{
    cncSubtitlePresentationStyle_t      presentationStyle;
    cncSubtitleColor_t                  characterColor;
    cncSubtitleCharacterOpacity_t       characterOpacity;
    cncSubtitleColor_t                  captionBackgroundColor;
    cncSubtitleCaptionOpacity_t         captionBackgroundOpacity;
    cncSubtitleCharacterSize_t          characterSize;
    cncSubtitleFontFamily_t             fontFamily;
    cncSubtitleCharacterEdge_t          characterEdge;
    cncSubtitleColor_t                  windowBackgroundColor;
    cncSubtitleCaptionWindowOpacity_t   windowBackgroundOpacity;
} cncSubtitleFccAttributes_t;

typedef enum _cncSubtitleStatus_t
{
    CNC_SUBTITLE_STATUS_NONE,
    CNC_SUBTITLE_STATUS_ENABLED,
    CNC_SUBTITLE_STATUS_DISABLED
} cncSubtitleStatus_t;

typedef struct _cncSubtitleScreenDimension_t
{
    CNC_SINT32  xPos;
    CNC_SINT32  yPos;
    CNC_SINT32  width;
    CNC_SINT32  height;
} cncSubtitleScreenDimension_t;

typedef enum _cncSubtitleScreenMode_t
{
    CNC_SUBTITLE_SCREEN_MODE_NONE,
    CNC_SUBTITLE_SCREEN_MODE_FULL_SCREEN,
    CNC_SUBTITLE_SCREEN_MODE_SUB_WINDOW
} cncSubtitleScreenMode_t;

typedef struct _cncSubtitleInfoAndData_t
{
    cncSubtitleInputMethodType_t    subtitleInputMethodType;
    cncSubtitleInput_t              subtitleInput;
    cncSubtitleType_t               subtitleType;
    cncDirectFBInterface_t          DirectFBItf;
    cncSubtitleFccAttributes_t      fccAttributes;
    cncSubtitleStatus_t             subtitleStatus;
    cncSubtitleScreenDimension_t    subtitleScreenDimension;
    cncSubtitleScreenMode_t         subtitleScreenMode;
} cncSubtitleInfoAndData_t;

/* =========================== Function Prototypes ======================== */
// NONE

#ifdef __cplusplus
}
#endif

#endif /* END:  _CNC_COMMON_SUBTITLE_H_ */
