/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Cavium, Inc.
 Cavium, Inc. reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium, Inc.

 ALL RIGHTS RESERVED.

 Copyright 2011-2012, Cavium, Inc.
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */

/**
 * @file csblit.h
 * @ingroup csapi
 * Low Level Blitting API
 */

#ifndef _CSBLIT_H_
#define _CSBLIT_H_

#ifdef __cplusplus
extern "C" {
#endif

#include "global.h"

typedef void *CSBLIT_HANDLE;

typedef struct BLIT_INITPARAMS_s
{
	int  reserved;
}BLIT_INITPARAMS_t;

typedef enum  CSBLIT_PRIMARY_LAYER_e
{
    CSBLIT_PRIMARY_LAYER_0 = 0,
    CSBLIT_PRIMARY_LAYER_1 = 1,
    CSBLIT_PRIMARY_LAYER_MAX
} CSBLIT_PRIMARY_LAYER_t;

typedef enum
{
	BLIT_NO_ERROR = 0,
	BLIT_ERROR_OPEN_FAILED,		/* open filed                 */
	BLIT_ERROR_IOCTL_FAILED,		/* ioctl filed                */
	BLIT_ERROR_INVALID_PARAMETERS,	/* Bad parameter passed       */
	BLIT_ERROR_UNKNOWN_DEVICE,	/* Unknown device name        */
	BLIT_ERROR_DEVICE_BUSY,		/* Device is currently busy   */
	BLIT_ERROR_INVALID_HANDLE,	/* Handle is not valid        */
	BLIT_ERROR_ALREADY_INITIALIZED,	/* Device already initialized */
	BLIT_ERROR_NOT_INITIALIZED,	/* Device not initialized     */
	BLIT_ERROR_NO_MEMORY,	/* Device not enough memory     */
	BLIT_ERROR_MEMORY_MAP    	/* Device memory map error     */
} CSBLIT_ErrCode_t;


typedef enum  CSBLIT_AluMode_e
{
    CSBLIT_ALU_CLEAR            = 0,      //0      0000
    CSBLIT_ALU_NOR              = 1,      //~(D|P) 0001
    CSBLIT_ALU_AND_INVERT       = 2,      //D&~P   0010
    CSBLIT_ALU_COPY_INVERT      = 3,      //~P     0011
    CSBLIT_ALU_AND_REV          = 4,      //P&~D   0100
    CSBLIT_ALU_INVERT           = 5,      //~D     0101
    CSBLIT_ALU_XOR              = 6,      //D^P    0110
    CSBLIT_ALU_NAND             = 7,      //~(D&P) 0111
    CSBLIT_ALU_AND              = 8,      //D&P    1000
    CSBLIT_ALU_EQUIV            = 9,      //~(P^D) 1001
    CSBLIT_ALU_NOOP             = 10,     //D      1010
    CSBLIT_ALU_OR_INVERT        = 11,     //D|~P   1011
    CSBLIT_ALU_COPY             = 12,     //P      1100
    CSBLIT_ALU_OR_REVERSE       = 13,     //P|~D   1101
    CSBLIT_ALU_OR               = 14,     //D|P    1110
    CSBLIT_ALU_SET              = 15      //1      1111
    //CSBLIT_ALU_ALPHA_BLEND      = 16
} CSBLIT_AluMode_t;

typedef enum BLIT_SourceType_e
{
    CSBLIT_SOURCE_TYPE_COLOR,
    CSBLIT_SOURCE_TYPE_BITMAP
} CSBLIT_SourceType_t;

typedef enum CSBLIT_ColorType_e
{
  CSBLIT_COLOR_TYPE_A0 = 2,       /*one single color, can't be set to output colortype*/
  CSBLIT_COLOR_TYPE_RGB565 = 0,
  CSBLIT_COLOR_TYPE_ARGB8888 =6,
} CSBLIT_ColorType_t;

typedef struct CSBLIT_Bitmap_s
{
  CSBLIT_ColorType_t			ColorType;
  unsigned int                                   Width;
  unsigned int                                   Height;
  unsigned int                                   Pitch;
  unsigned int                                   Offset;
  void*                                 Data_p;
  unsigned int                                   Size;
} CSBLIT_Bitmap_t;

typedef struct CSBLIT_ColorARGB_s
{
  unsigned char Alpha;
  unsigned char R;
  unsigned char G;
  unsigned char B;
} CSBLIT_ColorARGB_t;

typedef struct CSBLIT_ColorRGB_s
{
  unsigned char R;
  unsigned char G;
  unsigned char B;
} CSBLIT_ColorRGB_t;

typedef struct CSBLIT_ColorKeyRGB_s
{
  unsigned char      RMin;
  unsigned char      RMax;
  unsigned char    ROut;

  unsigned char      GMin;
  unsigned char      GMax;
  unsigned char    GOut;

  unsigned char      BMin;
  unsigned char      BMax;
  unsigned char    BOut;

  unsigned char    RGBEnable;
} CSBLIT_ColorKeyRGB_t;

typedef union CSBLIT_ColorValue_u
{
  CSBLIT_ColorARGB_t           ARGB8888;
  CSBLIT_ColorRGB_t            RGB565;
} CSBLIT_ColorValue_t;

typedef struct CSBLIT_Color_s
{
  CSBLIT_ColorType_t            Type;
  CSBLIT_ColorValue_t           Value;
} CSBLIT_Color_t;

typedef struct CSBLIT_Rectangle_s
{
  int PositionX;
  int PositionY;
  unsigned int Width;
  unsigned int Height;
} CSBLIT_Rectangle_t;

typedef struct CSBLIT_Source_s
{
    CSBLIT_SourceType_t     	Type;
    union {
	    CSBLIT_Color_t   	Color;
            CSBLIT_Bitmap_t  	Bitmap;
    } Data;
    CSBLIT_Rectangle_t    	Rectangle;
    CSBLIT_ColorKeyRGB_t  	ColorKeyRGB;	/* ColorKey default is not used */
    CSBLIT_Color_t	    	DefaultColor; 	/* not valid if source type is CSBLIT_SOURCE_TYPE_COLOR */
    unsigned char		    	VReverse;
} CSBLIT_Source_t;

typedef struct CSBLIT_Destination_s
{
    CSBLIT_Bitmap_t	      	Bitmap;
    CSBLIT_Rectangle_t	Rectangle;
    unsigned char		    	VReverse;
} CSBLIT_Destination_t;

typedef struct CSBLIT_FillRectParams_s
{
    CSBLIT_Bitmap_t        	Bitmap;
    CSBLIT_Rectangle_t     	Rectangle;
    CSBLIT_Color_t         	Color;
    CSBLIT_AluMode_t 		AluMode;
} CSBLIT_FillRectParams_t;

typedef struct CSBLIT_CopyRectParams_s
{
    CSBLIT_Bitmap_t        	SrcBitmap;
    CSBLIT_Rectangle_t     	SrcRectangle;
    CSBLIT_Bitmap_t        	DstBitmap;
    int                   	DstPositionX;
    int                   	DstPositionY;
    CSBLIT_AluMode_t 		AluMode;
} CSBLIT_CopyRectParams_t;

typedef struct CSBLIT_ScalorParams_s
{
    CSBLIT_Source_t		Src;
    CSBLIT_Destination_t   	Dst;
    unsigned int				VInitialPhase;	/*default is 4*/
    unsigned int				HInitialPhase;	/*default is 0*/
    CSBLIT_AluMode_t 		AluMode;
} CSBLIT_ScalorParams_t;

typedef struct CSBLIT_CompParams_s
{
    unsigned char			BlendEnable;
    unsigned char			IsS0OnTopS1;
    unsigned int				ROPAlphaCtrl;
    CSBLIT_Source_t		Src0;
    CSBLIT_Source_t		Src1;
    CSBLIT_Destination_t   	Dst;
    CSBLIT_AluMode_t 		AluMode;
} CSBLIT_CompParams_t;

typedef struct CSBLIT_Antiflicker_copy_s
{
    unsigned char			BlendEnable;
    unsigned char			IsS0OnTopS1;
    unsigned int			ROPAlphaCtrl;
    CSBLIT_Source_t		Src0;
    CSBLIT_Source_t		Src1;
    CSBLIT_Destination_t   	Dst;
    CSBLIT_AluMode_t 		AluMode;
} CSBLIT_Antiflicker_copy_t;

CSBLIT_HANDLE CSBLIT_Open(BLIT_INITPARAMS_t init_param);
CSAPI_RESULT CSBLIT_Close(CSBLIT_HANDLE handle);
void *CSBLIT_Malloc_DMA_Buffer(CSBLIT_HANDLE handle,char **buffer, unsigned int size, unsigned int *platform_stuff);
CSAPI_RESULT CSBLIT_Free_DMA_Buffer(CSBLIT_HANDLE handle,void *platform_stuff);
void *CSBLIT_Get_Primary_Layer(CSBLIT_HANDLE handle,char **buffer, CSBLIT_PRIMARY_LAYER_t primary_layer, unsigned int *size);
CSAPI_RESULT CSBLIT_Fill(CSBLIT_HANDLE handle, CSBLIT_FillRectParams_t params, int isupdate);
CSAPI_RESULT CSBLIT_Copy(CSBLIT_HANDLE handle, CSBLIT_CopyRectParams_t params, int isupdate);
CSAPI_RESULT CSBLIT_Scalor(CSBLIT_HANDLE handle,CSBLIT_ScalorParams_t params, int isupdate);
CSAPI_RESULT CSBLIT_Composite(CSBLIT_HANDLE handle,CSBLIT_CompParams_t params, int isupdate);
CSAPI_RESULT CSBLIT_WaitForSync(CSBLIT_HANDLE handle);
CSAPI_RESULT CSBLIT_UpdatePrimaryLayer(CSBLIT_HANDLE handle, CSBLIT_PRIMARY_LAYER_t primary_layer, int isdatasync);
CSAPI_RESULT CSBLIT_UpdatePrimaryLayerRect(CSBLIT_HANDLE handle, CSBLIT_PRIMARY_LAYER_t primary_layer, int isdatasync, CSBLIT_Rectangle_t rect);

#ifdef __cplusplus
}
#endif

#endif //_CSBLIT_H_
