/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Cavium, Inc.
 Cavium, Inc. reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium, Inc.

 ALL RIGHTS RESERVED.

 Copyright 2011-2012, Cavium, Inc.
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */

/**
 * @file csevt.h
 * @ingroup csapi
 * Low Level Event API
 */
#ifndef __CSEVT_H__
#define __CSEVT_H__

#ifdef __cplusplus
extern "C" {
#endif

#include "global.h"

typedef void *CSEVT_HANDLE;


/*!
 *@brief enum for event type.
 *     @see
*/
typedef enum
{
   EVT_READ    = 0x01, ///< Set when a read will not block
   EVT_WRITE   = 0x02, ///<  Set when a write will not block
   EVT_EXCEPT  = 0x04, ///<  Set when out-of-band data is available
   EVT_USER    = 0x08, ///< Set when user want to manual control
   EVT_INVALID = 0x10  ///< Set when the file descriptor is invalid
} CSEVT_TYPE;

/*!
 *@brief define the prototype for event callback function.
 *     @see
*/
typedef void (* event_proc)(void *puser, int fd, int events);


/*!
 *@brief Apply for an event monitor request.
 *
 *     @return	CSEVT_HANDLE  Evnet handle.
 *     @return	NULL	Failed, need more memory space.
 *		@note
 *     @see
 */
CSEVT_HANDLE CSEVT_Init(void);


/*!
 *@brief Cancel an event monitor request.

 *     @param	handle Evnet handle. In/Out : In
 *     @return	CSAPI_SUCCEED CSAPI_SUCCEED
 *						CSAPI_FAILED CSAPI_FAILED
 *		@note
 *     @see
 */
CSAPI_RESULT CSEVT_Term(CSEVT_HANDLE handle);


/*!
 *@brief Register an event to this module.
 *
 *     @param	handle Evnet handle. In/Out : In
 *     @param	fd The handle of the module or file that need to be monitored. In/Out : In
 *     @param	func When the event happened, this function would be invoked. If events is EVT_USER,func would be the trigger function.void (* func)(void *puser, int fd, int events);In/Out : In
 *     @param	puser Parameter to the func. In/Out : In
 *     @param	events Event types. In/Out : In
 *     @return	CSAPI_SUCCEED	Success\n
 *						CSAPI_FAILED Failure
 *		@note
 *     @see
 */
CSAPI_RESULT CSEVT_Register(CSEVT_HANDLE handle, int fd, event_proc func, void *puser, int events);

/*!
 *@brief Unregister an event to this module.
 *
 *     @param	handle Evnet handle. In/Out : In
 *     @param	fd The handle of the module or file that need to be monitored. In/Out : In
 *     @return	CSAPI_SUCCEED	Success\n
 *						CSAPI_FAILED Failure
 *		@note
 *     @see
 */
CSAPI_RESULT CSEVT_UnRegister(CSEVT_HANDLE handle, int fd);

/*!
 *@brief When the event is EVT_USER, this will start the monitor task with self-defined trigger function.
 *
 *     @param	handle Evnet handle. In/Out : In
 *     @param	fd The handle of the module or file that being monitored.In/Out : In
 *     @param	events Event types. In/Out : In
 *     @param	context Parameter to the monitor function. In/Out : In
 *     @return	CSAPI_SUCCEED	Success\n
 *						CSAPI_FAILED Failure
 *		@note
 *     @see
 */
CSAPI_RESULT CSEVT_ReportEvent(CSEVT_HANDLE handle, int fd, int events, void *context);

#ifdef __cplusplus
}
#endif

#endif
