/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Cavium, Inc.
 Cavium, Inc. reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium, Inc.

 ALL RIGHTS RESERVED.

 Copyright 2011-2012, Cavium, Inc.
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */

/**
 * @file csfrontpanel.h
 * Low Level Front Panel API
 * @ingroup csapi
 */
#ifndef __CSAPI_FRONTPANEL_H__
#define __CSAPI_FRONTPANEL_H__

#include "global.h"

#ifdef __cplusplus
extern "C" {
#endif


/******************************************************************************************************/
/**
*return Err
*/
typedef enum
{
        CSFP_NO_ERROR = 0,
        CSFP_ERROR_OPEN_FPC_FAILED,              ///< open fpc device filed                     
        CSFP_ERROR_OPEN_INPUT_FAILED,            ///< open fpc device filed                     
        CSFP_ERROR_OPEN_SERIAL_FAILED,           ///<open fpc device filed                     
        CSFP_ERROR_IOCTL_SETSYSTEMCODE_FAILED,   ///<ioctl set systemcode failed               
        CSFP_ERROR_IOCTL_SETKEYSCAN_FAILED,      ///< ioctl enable/disable keyscan failed       
        CSFP_ERROR_IOCTL_SETRC_FAILED,           ///<ioctl enable/disable rc failed            
        CSFP_ERROR_IOCTL_ENABLESERIALIN_FAILED,  ///< ioctl enable serial port  failed          
        CSFP_ERROR_IOCTL_DISABLESERIALIN_FAILED, ///< ioctl disable serial port failed          
        CSFP_ERROR_IOCTL_RESETSERIALIN_FAILED,   ///< ioctl reset serial port failed            
        CSFP_ERROR_IOCTL_SETSERIALIN_FAILED,     ///< ioctl set attribute of serial port failed 
        CSFP_ERROR_IOCTL_SETRCPROTOCAL_FAILED,   ///< ioctl set protocal type of rc failed      
        CSFP_ERROR_IOCTL_SETBitTimeCnt_FAILED,   ///< ioctl set bit time count failed           
        CSFP_ERROR_IOCTL_GETBitTimeCnt_FAILED,   ///< ioctl get bit time count failed           
        CSFP_ERROR_NOTOPEN_SERIALIN,             ///< serial port is not opened                 
        CSFP_ERROR_INVALID_PARAMETERS,           ///< Bad parameter passed                      
        CSFP_ERROR_UNKNOWN_DEVICE,               ///< Unknown device name                       
        CSFP_ERROR_DEVICE_BUSY,                  ///< Device is currently busy                  
        CSFP_ERROR_INVALID_HANDLE,               ///< Handle is not valid                       
        CSFP_ERROR_ALREADY_INITIALIZED,          ///< Device already initialized                
        CSFP_ERROR_CLOSE_FPC,                    ///< close front panel device is failed        
        CSFP_ERROR_CLOSE_SERIAL,                 ///<close serial device failed                
        CSFP_ERROR_CLOSE_INPUT,                  ///<close input device failed                 
        CSFP_ERROR_REGISTER_CALLBACK             ///< register call back failed                 
} CSFP_ErrCode;

typedef void* CSFP_HANDLE;

/**
*Modes of 7-segment display.
* @see CSFP_SetLEDDisplayMode()
*/
typedef enum
{
	CSFP_LEDDISP_OFF = 0,   ///<This mode will switch off all the 7-segments LED display
	CSFP_LEDDISP_BLINK = 1, ///<This mode will cause blink effect of characters over the LED display
	CSFP_LEDDISP_ON = 2     ///<This mode will switch on all/particular segment of display
}CSFP_LEDDISPMODE;

/**
*Attributes of display.
* @see CSFP_SetLEDDisplayAttr()
*/
typedef enum
{
	CSFP_LEDDISP_ON_BRIGHT = 0,  ///<This control value will set the hardware to display characters bright. This is the default value of this enum
	CSFP_LEDDISP_ON_DIM = 1,     ///<This control value will set the hardware to display dim
	CSFP_LEDDISP_FAST_BRIGHT = 2,///<This control value will set the hardware to display characters bright with fast blink rate
	CSFP_LEDDISP_FAST_DIM = 3,   ///<This control value will set the hardware to display characters dim with fast blink rate
	CSFP_LEDDISP_SLOW_BRIGHT = 4,///<This control value will set the hardware to display characters bright with slow blink rate
	CSFP_LEDDISP_SLOW_DIM = 5    ///<This control value will set the hardware to display characters dim with slow blink rate
}CSFP_LEDDISPATTR ;

typedef void* CSFP_LEDDISP_HANDLE;

/**
*Events Control Values
* @see  CSFP_SetNotify()
*/

typedef enum
{
	CSFP_KEYSCAN_IN = 1,  ///<This control value will select the key scan driver in callback function
	CSFP_RCBYTE_IN = 2    ///<This control value will select the remote controller driver in callback function
}CSFP_NOTIFYEVENT;

/**
*Protocal of remote controller.
*/
typedef enum
{
	CSFP_RC_NEC = 0,      ///<Set protocal as RC5
	CSFP_RC_RC5 = 1,       ///<Set protocal as NEC
	CSFP_RC_SONY,
	CSFP_RC_MAX ///<RC type max number
}CSFP_RC_TYPE;


/**
*Open a connection to the front panel.

*@return Open handle to frontpanel connection. This handle holds descriptor for opened frontpanel LED device and remote controller device. It holds initialized frontpanel control structure, and returns the handle of type CSFP_HANDLE.
*@note Open interface will also register an event for frontpanel in CS-Event module. 
*/
CSFP_HANDLE  CSFP_Open(void);

/**
*Close a connection to the front panel device
*@param handle  Valid handle to the front panel connection.  In/Out: In
*@return CSAPI_SUCCEED Success
*        CSAPI_FAILED  Failure
*@note Close interface will unregister the event from CS-Event Module. This also resets the control parameters to dummy value. Close interface will also close the opened serial device.
*/
CSAPI_RESULT CSFP_Close(CSFP_HANDLE handle);

/**
* EnableKeyScan Enable the key scan process
*@param handle Handle  to the front panel device. In/Out: In
*@return CSAPI_SUCCEED Success
*        CSAPI_FAILED  Failure
*@note This interface helps application to independently enable key scan process, once after the front panel device is opened successfully
*@see CSFP_DisableKeyScan
*/
CSAPI_RESULT CSFP_EnableKeyScan (CSFP_HANDLE handle);

/**
* DisableKeyScan to disable the key scan process
*@param  handle Handle  to the front panel device. In/Out: In
*@return CSAPI_SUCCEED Success
*        CSAPI_FAILED  Failure
*@note This interface will allow application to disable the keyscan process
*/
CSAPI_RESULT CSFP_DisableKeyScan(CSFP_HANDLE handle);

/**
*SetKeyScanMap to update the global keyscan map table with user requested value.
*@param handle Handle to the front panel device. In/Out: In
*@param raw_input raw_input to Key map table index. In/Out: In
*@param sys_code sys_code to The system code which user specified. In/Out: In
*@return CSAPI_SUCCEED Success
*        CSAPI_FAILED  Failure
*@note This interface allows application to change system code of key scan map table.
*/
CSAPI_RESULT CSFP_SetKeyScanMap(CSFP_HANDLE handle, unsigned char raw_input, unsigned char sys_code);

/**
*EnableSystemCode to Enable the filter of key from remote controller.
*@param handle Handle to the front panel device.  In/Out: In
*@return CSAPI_SUCCEED Success
*        CSAPI_FAILED  Failure
*@note By CSFP_SetSystemCode, the front panel only receive the data from the dedicated front panel.
*/
CSAPI_RESULT CSFP_EnableSystemCode(CSFP_HANDLE handle);

/**
*DisableSystemCode to Disable the filter of key from remote controller.
*@param handle Handle to the front panel device. In/Out: In
*@return CSAPI_SUCCEED Success
*        CSAPI_FAILED  Failure
*@note The front panel can receive data from all kind of remote controller after this function is called.
*/
CSAPI_RESULT CSFP_DisableSystemCode(CSFP_HANDLE handle);

/**
*SetSystemCode to Set a system code to front panel controller.
*@param handle Handle to the front panel device. In/Out: In
*@param system_code system_code to A system code. In/Out: In
*@return CSAPI_SUCCEED Success
*        CSAPI_FAILED  Failure
*@note Normally, the front panel can receive data from all kind of remote controller (RC5 and NEC6); 
  this function allows that the front panel just receives data from the dedicated remote controller.
*/
CSAPI_RESULT CSFP_SetSystemCode(CSFP_HANDLE handle, int system_code);

/**
*EnableRemoteController to Enable the remote control filter process.
*@param handle Handle to the front panel device. In/Out: In
*@return CSAPI_SUCCEED Success
*        CSAPI_FAILED  Failure
*@note This interface helps application to independently enable remote control filter process, once after the front panel device is opened successfully.
*@see CSFP_DisableRemoteController
*/
CSAPI_RESULT CSFP_EnableRemoteController (CSFP_HANDLE handle);

/**
*DisableRemoteController  to Disable the Remote control filter process.
*@param handle Handle to the front panel device. In/Out: In
*@return CSAPI_SUCCEED Success
*        CSAPI_FAILED  Failure
*@note This interface helps application to disables remote control filter process, with valid front panel handle.
*/
CSAPI_RESULT CSFP_DisableRemoteController (CSFP_HANDLE handle);

/**
*SetRemoteControlleType to configurate Remote control protocal.
*@param handle Handle to the front panel device. In/Out: In
*@param rctype rctype to specify protocal. In/Out: In
*@return CSAPI_SUCCEED Success
*        CSAPI_FAILED  Failure
*@see CSFP_RC_TYPE
*/
CSAPI_RESULT CSFP_SetRemoteControllerType(CSFP_HANDLE handle, CSFP_RC_TYPE rctype);

/**
*SetBitTimeCnt to specify how many cycles the remote controller sample the input signal one time.
*@param handle Handle to the front panel device. In/Out: In
*@param bittime bittime to specify how many cycles the remote controller sample the input signal one time. In/Out: In
*@return CSAPI_SUCCEED Success
*        CSAPI_FAILED  Failure
*/
CSAPI_RESULT CSFP_SetBitTimeCnt(CSFP_HANDLE handle, unsigned short bittime);

/**
*GetBitTimeCnt to get current how many cycles the remote controller sample the input signal one time.
*@param handle Handle to the front panel device. In/Out: In
*@param bittime bittime to return current time of the remote controller sample the input signal one time. In/Out: Out
*@return CSAPI_SUCCEED Success
*        CSAPI_FAILED  Failure
*/
CSAPI_RESULT CSFP_GetBitTimeCnt(CSFP_HANDLE handle, unsigned short *bittime);

/**
*SetLEDDisplay to Display the characters on the 7-segment display with preseted values of modes and attributes.
*@param handle Handle to the front panel device. In/Out: In
*@param num_str to Pointer to string to display on LED display. In/Out: In
*@return CSAPI_SUCCEED Success
*        CSAPI_FAILED  Failure
*@note This interface will allow application to display some strings on LED display. Supported string types are numeric (0 to 9) and limited alphabets (readable on 7-segment display).
Before calling this interface, CSFP_SetLEDDisplayMode needs to be called.
*@see CSFP_GetLEDDisplay
*/
CSAPI_RESULT CSFP_SetLEDDisplay(CSFP_HANDLE handle, const char * const num_str);

/**
*Display single character on LED display.
*@param handle Handle to the front panel device. In/Out: In
*@param str_display Pointer to the character to display. In/Out: In
*@return CSAPI_SUCCEED Success
*        CSAPI_FAILED  Failure
*@note This interface allows the application to display single character on the LED display. Allowed characters are numeric (0 to 9) and limited alphabets (readable on 7-segment display).
Before calling this interface, application needs to call CSFP_SetLEDDisplayMode and CSFP_SetLEDDisplayPos.
*@see CSFP_GetLEDDisplayChar
*/
CSAPI_RESULT CSFP_GetLEDDisplay(CSFP_HANDLE handle, unsigned char *str_display );

/**
*SetLEDDisplayChar to Display single character on LED display.
*@param handle Handle to the front panel device. In/Out: In
*@param char_value Char_value to Pointer to the character to display. In/Out: In
*@return CSAPI_SUCCEED Success
*        CSAPI_FAILED  Failure
*@note This interface will allow the application to display single character on the LED display. Allowed characters are numeric (0 to 9) and limited alphabets (readable on 7-segment display).
Before calling this interface, application needs to call CSFP_SetLEDDisplayMode and CSFP_SetLEDDisplayPos.
*@see CSFP_GetLEDDisplayChar
*/
CSAPI_RESULT CSFP_SetLEDDisplayChar(CSFP_HANDLE handle, unsigned char *char_value);

/**
*SetLEDDisplayMode  to Set the different mode on which character will display on the LED display.
*@param handle Handle to the front panel device. In/Out: In
*@param mode mode to Mode of type CSFP_LEDDISPMODE which supports ON/OFF/BLINK. In/Out: In
*@return CSAPI_SUCCEED Success
*        CSAPI_FAILED  Failure
*@note This interface will allow application to set different modes like ON/OFF/BLINK to display characters on LED display.
*@see CSFP_GetLEDDisplayMode
*/
CSAPI_RESULT CSFP_SetLEDDisplayMode (CSFP_HANDLE handle, CSFP_LEDDISPMODE mode );

/**
*SetLEDDisplayAttr  to Set the different attributes on which character will display on the LED display.
*@param handle Handle to the front panel device. In/Out: In
*@param attr attr to Attributes of type CSFP_LEDDISPATTR which supports six different control values refer 1.2.2 Enums. In/Out: In
*@return CSAPI_SUCCEED Success
*        CSAPI_FAILED  Failure
*@note This interface will allow application to set different attributes like BRIGHT/DIM/FAST-BRIGHT/FAST-DIM/SLOW-BRIGHT/SLOW-DIM to display characters on LED display.
*@see CSFP_GetLEDDisplayAttr
*/
CSAPI_RESULT CSFP_SetLEDDisplayAttr (CSFP_HANDLE handle, CSFP_LEDDISPATTR attr );

/**
*SetLEDDisplayRaw  to Display the raw characters on the 7-segment display.
*@param handle Handle to the front panel device. In/Out: In
*@param raw_value raw_value to Pointer to string to display on LED display. In/Out: In
*@param raw_len raw_len to the len about string to display on LED display. In/Out: In
*@return CSAPI_SUCCEED Success
*        CSAPI_FAILED  Failure
*@note This interface will allow the application to display single "raw character" on the LED display.
		Before calling this interface, application needs to call CSFP_SetLEDDisplayMode and CSFP_SetLEDDisplayPos
*@see CSFP_GetLEDDisplayRaw
*/
CSAPI_RESULT CSFP_SetLEDDisplayRaw(CSFP_HANDLE handle, unsigned char *raw_value, int raw_len);

/**
*Display the raw characters on the 7-segment display.
*@param handle Handle to the front panel device. In/Out: In
*@param char_value ointer to the character to display. In/Out: In
*@return CSAPI_SUCCEED Success
*        CSAPI_FAILED  Failure
*@note This interface allows the application to display single character on the LED display. Allowed characters are numeric (0 to 9) and limited alphabets (readable on 7-segment display).
	Before calling this interface, application needs to call CSFP_SetLEDDisplayMode and CSFP_SetLEDDisplayPos.
*@see CSFP_GetLEDDisplayChar
*/
CSAPI_RESULT CSFP_GetLEDDisplayChar(CSFP_HANDLE handle, unsigned char *char_value);

/**
*GetLEDDisplayMode  to Get the mode of the currently displaying characters from 7-Segment LED display.
*@param handle Handle to the front panel device. In/Out: In
*@param mode mode to Pointer to receive mode of currently displaying character from LED display.In/Out: In
*@return CSAPI_SUCCEED Success
*        CSAPI_FAILED  Failure
*@note This interface will allow application to get the mode of currently displaying character from 7-segment display.
*/
CSAPI_RESULT CSFP_GetLEDDisplayMode(CSFP_HANDLE handle, CSFP_LEDDISPMODE *mode );

/**
*GetLEDDisplayAttr  to Get the attributes of the currently displaying characters from 7-Segment LED display.
*@param handle Handle to the front panel device. In/Out: In
*@param attr attr to Pointer to receive attributes of currently displaying character from LED display. In/Out: In
*@return CSAPI_SUCCEED Success
*        CSAPI_FAILED  Failure
*@note This interface will allow application to get the attributes of currently displaying character from 7-segment display.
*/
CSAPI_RESULT CSFP_GetLEDDisplayAttr(CSFP_HANDLE handle, CSFP_LEDDISPATTR *attr);

/**
*GetLEDDisplayRaw  to Get the raw value of currently displaying single characters from 7-Segment LED display.
*@param handle Handle to the front panel device. In/Out: In
*@param raw_value raw_value to Pointer to receive raw value of single character from LED display. In/Out: In
*@return CSAPI_SUCCEED Success
*        CSAPI_FAILED  Failure
*@note This interface will allow application to get the raw value of currently displaying single character from 7-segment display.
*/
CSAPI_RESULT CSFP_GetLEDDisplayRaw(CSFP_HANDLE handle, unsigned char *raw_value);

/**
*SetLEDDisplayPos to  Set the position for display characters on LED display.
*@param handle Handle to the front panel device  In
*@param row Row to Row on which character need to display. Presently it is fixed to one. In/Out: In
*@param column column to Column on which characters need to display. Presently four columns are supported. In/Out: In
*@return CSAPI_SUCCEED Success
*        CSAPI_FAILED  Failure
*@note This interface allows application to fix on what position user characters will display on 7-segment display. 
    Presently number of rows is fixed to one. And column will vary from zero to three.
    Row is not used now, column can be set 0-3 to display, other cloumn value will display the character on all LED
*/
CSAPI_RESULT CSFP_SetLEDDisplayPos(CSFP_HANDLE handle, unsigned char row, unsigned char column);

/**
*SetNotify to  Notify the application about occurrences of events.
*@param handle Handle to the front panel device. In/Out: In
*@param call_back_function call_back_function to Pointer to callback function. In/Out: In
*@param fp_event fp_event to User request event, can use CSFP_KEYSCAN_IN | CSFP_RCBYTE_IN for all input event. In/Out: In
*@return CSAPI_SUCCEED Success
*        CSAPI_FAILED  Failure
*@note Application can use this interface to request for notification for particular event. Once the event has occurred, 
   This interface will notify the application through the callback function.
   Prototype of the callback function is: call_back_function (CSFP_HANDLE, CSFP_NOTIFYEVENT *, unsigned char *);
*/
CSAPI_RESULT CSFP_SetNotify(CSFP_HANDLE handle, void (*call_back_function)(CSFP_HANDLE, CSFP_NOTIFYEVENT *, unsigned char *),
		CSFP_NOTIFYEVENT fp_event);

/**
*GetErrCode  to Get error code of frontpanel APIs.
*@param handle Handle to the front panel device. In/Out: In
*@return CSFP_ErrCode  Error value
*@note Application can get the error value through the return value of this API.
*/
CSFP_ErrCode CSFP_GetErrCode(CSFP_HANDLE handle);

/**
*GetErrString  to Get the error string of frontpanel APIs.
*@param handle Handle to the front panel device. In/Out: In
*@return Char*  Pointer to string of error type.
*@note Application can get the error string through the return value of this API.
*/
char *CSFP_GetErrString(CSFP_HANDLE handle);

#ifdef __cplusplus
}
#endif

#endif /*  __CSAPI_FRONTPANEL_H__ */
