/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Cavium, Inc.
 Cavium, Inc. reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium, Inc.

 ALL RIGHTS RESERVED.

 Copyright 2011-2012, Cavium, Inc.
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */

/**
 * @file cshdmirx.h
 * @ingroup csapi
 * Low Level HDMI API
 */
#ifndef __CSAPI_HDMIRX_H__
#define __CSAPI_HDMIRX_H__

#ifdef __cplusplus
extern "C" {
#endif

#include "global.h"

typedef enum
{
    HDMI_PORT_A      = 0,                /* The value of the port is also    */
    HDMI_PORT_B      = 1,                /* the port index (0-3)             */
    HDMI_PORT_NONE   = 0xFE,
    HDMI_PORT_ALL    = 0xFF
}HDMI_PORT;
typedef void *CSHDMIRX_HANDLE;

typedef struct {
    int speed;    ///< I2C_SPEED_100K 1; I2C_SPEED_400K 2
    unsigned char loop;    ///< I2C_ADDR_NOLOOP 0, I2C_ADDR_LOOP 1
    unsigned int subaddr_num;///<The number of sub address.Default is 1.
    unsigned int write_delayus;///<The time delayed when writing.
} CSHDMIRX_Attr;


#define CEC_MSG_LEN   17
#define MAX_INFOFRAME_DATA_LEN   28
#define HDMI_RX_INT_PIN  13

//CEC Opcode
//One Touch Play Feature 0x44
#define CEC_OPCODE_ACTIVE_SOURCE  0x82
#define CEC_OPCODE_IMAGE_VIEW_ON  0x04
#define CEC_OPCODE_TEXT_VIEW_ON  0x0D
//Routing Control Feature
#define CEC_OPCODE_ACTIVE_SOURCE  0x82
#define CEC_OPCODE_INACTIVE_SOURCE  0x8D
#define CEC_OPCODE_REQUEST_ACTIVE_SOURCE  0x85
#define CEC_OPCODE_ROUTING_CHANGE  0x80
#define CEC_OPCODE_ROUTING_INFORMATION  0x81
#define CEC_OPCODE_SET_STREAM_PATH  0x86
//Standby Feature
#define CEC_OPCODE_STANDBY  0x36
//Ont Touch Record Feature
#define CEC_OPCODE_RECORD_OFF  0x0B
#define CEC_OPCODE_RECORD_ON  0x09
#define CEC_OPCODE_RECORD_STATUS  0x0A
#define CEC_OPCODE_RECORD_TV_SCREEN  0x0F
//Timer Programming Feature
#define CEC_OPCODE_CLEAR_ANALOGUE_TIMER  0x33
#define CEC_OPCODE_CLEAR_DIGITAL_TIMER  0xA1
#define CEC_OPCODE_SET_ANALOGUE_TIMER  0x34
#define CEC_OPCODE_SET_DIGITAL_TIMER  0x97
#define CEC_OPCODE_EXTERNAL_TIMER  0xA2
#define CEC_OPCODE_SET_TIMER_PROGRAM_TITLE  0x67
#define CEC_OPCODE_TIMER_CLEARED_STATUS  0x43
#define CEC_OPCODE_TIMER_STATUS  0x35
//System Information Feature
#define CEC_OPCODE_CEC_VERSION  0x9E
#define CEC_OPCODE_GET_CEC_VERSION  0x9F
#define CEC_OPCODE_GIVE_PHYSICAL_ADDRESS  0x83
#define CEC_OPCODE_GET_MENU_LANGUAGE  0x91
#define CEC_OPCODE_POLLING_MESSAGE  // none
#define CEC_OPCODE_REPORT_PHYSICAL_ADDRESS  0x84
#define CEC_OPCODE_SET_MENU_LANGUAGE  0x32
//Deck Control Feature
#define CEC_OPCODE_DECK_CONTROL  0x42
#define CEC_OPCODE_DECK_STATUS  0x1B
#define CEC_OPCODE_GIVE_DECK_STATUS  0x1A
#define CEC_OPCODE_PLAY  0x41
//Tuner Control Feature
#define CEC_OPCODE_GIVE_TUNER_DEVICE_STATUS  0x08
#define CEC_OPCODE_SELECT_ANALOGUE_SERVICE   0x92
#define CEC_OPCODE_SELECT_DIGITAL_SERVICE     0x93
#define CEC_OPCODE_TUNER_DEVICE_STATUS    0x07
#define CEC_OPCODE_TUNER_STEP_DECREMENT  0x06
#define CEC_OPCODE_TUNER_STEP_INCREMENT 0X05
//Vendor Specific Commands Features
#define CEC_OPCODE_CEC_VERSION  0x9E
#define CEC_OPCODE_DEVICE_VENDOR 0x87
#define CEC_OPCODE_GET_CEC_VERSION 0x9F
#define CEC_OPCODE_GIVE_DEVICE_VENSOR_ID 0x8C
#define CEC_OPCODE_VENDOR_COMMAND 0x89
#define CEC_OPCODE_VENDOR_COMMAND_WITH_ID 0xA0
#define CEC_OPCODE_VENDOR_REMOTE_BUTTON_DOWN 0x8A
#define CEC_OPCODE_VENDOR_REMOTE_BUTTON_UP  0x8B
//OSD Display Feature
#define CEC_OPCODE_SET_OSD_STRING   0x64
//OSD Transfer Feature
#define CEC_OPCODE_GIVE_OSD_NAME   0x46
#define CEC_OPCODE_SET_OSD_NAME 0x47
//Menu Control Feature
#define CEC_OPCODE_MENU_REQUEST 0x8D
#define CEC_OPCODE_MENU_STATUS  0x8E
#define CEC_OPCODE_USER_CONTROL_PRESSED 0x44
#define CEC_OPCODE_USER_CONTROL_RELEASED 0x45
//Remote Control Passthough Feature
#define CEC_OPCODE_USER_CONTROL_PRESSED 0x44
#define CEC_OPCODE_USER_CONTROL_RELEASED 0x45
//Power Status Feature
#define CEC_OPCODE_GIVE_DEVICE_POWER_STATUS 0x8F
#define CEC_OPCODE_REPORT_POWER_STATUS 0x90
//Genneral Protocol Messages
#define CEC_OPCODE_FEATURE_ABORT 0x00
#define CEC_OPCODE_ABORT              0xFF
//System Audio Control Feature
#define CEC_OPCODE_GIVE_AUDIO_STATUS 0x71
#define CEC_OPCODE_GIVE_SYSTEM_AUDIO_MODE_STAUTS 0x7D
#define CEC_OPCODE_REPORT_AUDIO_STATUS  0x7A
#define CEC_OPCODE_REPORT_AUDIO_DESCRIPTOR 0xA3
#define CEC_OPCODE_REPORT_SHORT_AUDIO_DESCRIPTOR 0XA4
#define CEC_OPCODE_SET_SYSTEM_AUDIO_MODE 0x72
#define CEC_OPCODE_SYSTEM_AUDIO_MODE_REQUEST 0x70
#define CEC_OPCODE_SYSTEM_AUDIO_MODE_STATUS 0x7E
#define CEC_OPCODE_USER_CONTROL_PRESSED 0x44
#define CEC_OPCODE_USER_CONTROL_RELEASED 0x45
//Audio Rate Control Features
#define CEC_OPCODE_SET_AUDIO_RATE 0x9A
//Audio Return Channel Control Feature
#define CEC_OPCODE_INITIATE_ARC 0xC0
#define CEC_OPCODE_REPORT_ARC_INITIATED 0xC1
#define CEC_OPCODE_REPORT_ARC_TERMINATED 0xC2
#define CEC_OPCODE_REQUEST_ARC_INITIATION 0xC3
#define CEC_OPCODE_REQUEST_ARC_TERMINATION 0xC4
#define CEC_OPCODE_TERMINATE_ARC  0xC5
//Capability Discovery and Contol Feature
#define CEC_OPCODE_CDC_MESSAGE 0xF8



//Operand Dsecriptions
//Power Status
#define POWER_ON  0x00
#define POWER_STANDBY  0x01
#define POWER_STANDBY_TO_ON  0x02
#define POWER_ON_TO_STANDBY  0x03

//Interrupt
typedef enum
{
    INT_CP_LOCK=0x02,
    INT_CP_UNLOCK,
    INT_STDI_DATA,
    INT_MPU_STIM_INTRQ=0x17,
    INT_DVALID_CH1=0x22,
    INT_CP_UNLOCK_CH1,
    INT_CP_LOCK_CH1,
    INT_AVI_INFO=0x30,
    INT_AUDIO_INFO,
    INT_SPD_INFO,
    INT_MS_INFO,
    INT_VS_INFO,
    INT_ACP_PCKT,
    INT_ISR1_PCKT,
    INT_ISR2_PCKT,
    INT_GAMUT_MDATA=0x40,
    INT_AUDIO_C_PCKT,
    INT_GEN_CTL_PCKT,
    INT_HDMI_MODE,
    INT_AUDIO_CH_MD,
    INT_AV_MUTE,
    INT_INTERNAL_MUTE,
    INT_CS_DATA_VALID,
    INT_DE_REGEN_LCK=0x50,
    INT_V_LOCKED,
    INT_VIDEO_3D,
    INT_TMDS_CLK_B,
    INT_TMDS_CLK_A,
    INT_TMDSPLL_LCK_B,
    INT_TMDSPLL_LCK_A,
    INT_CABLE_DET_B,
    INT_CABLE_DET_A=0x60,
    INT_HDMI_ENCRPT_B,
    INT_HDMI_ENCRPT_A,
    INT_NEW_AVI_INFO=0x70,
    INT_NEW_AUDIO_INFO,
    INT_NEW_SPD_INFO,
    INT_NEW_MS_INFO,
    INT_NEW_VS_INFO,
    INT_NEW_ACP_PCKT,
    INT_NEW_ISRC1_PCKT,
    INT_NEW_ISRC2_PCKT,
    INT_NEW_GAMUT_MDATA=0x80,
    INT_AUDIO_PCKT_ERR,
    INT_PACKET_ERROR,
    INT_CHANGE_N,
    INT_CTS_PASS_THRSH,
    INT_FIFO_OVERFLO,
    INT_FIFO_UNDERFLO,
    INT_FIFO_NEAR_OVFL,
    INT_FIFO_NEAR_UFLO=0x90,
    INT_NEW_TMDS_FRQ,
    INT_AUDIO_FLT_LINE,
    INT_NEW_SAMP_RT,
    INT_PARITY_ERROR,
    INT_AUDIO_MODE_CHNG,
    INT_VCLK_CHNG,
    INT_DEEP_COLOR_CHNG,
    INT_AKSV_UPDATE_A=0xA0,
    INT_AKSV_UPDATE_B,
    INT_RI_EXPIRED_A,
    INT_RI_EXPIRED_B,
    INT_AVI_INF_CKS_ERR,
    INT_AUD_INF_CKS_ERR,
    INT_SPD_INF_CKS_ERR,
    INT_MS_INF_CKS_ERR,
    INT_VS_INF_CKS_ERR=0xB0,
    INT_BG_MEAS_DONE,
    INT_CEC_TX_READY=0xC0,
    INT_CEC_TX_ARBITRATION,
    INT_CEC_TX_RETRY_TIMEOUT,
    INT_CEC_RX_RDY0,
    INT_CEC_RX_RDY1,
    INT_CEC_RX_RDY2,
    INT_CEC_INTERRUPT_BYTE0=0xD0,
    INT_CEC_INTERRUPT_BYTE1,
    INT_CEC_INTERRUPT_BYTE2,
    INT_CEC_INTERRUPT_BYTE3,
    INT_CEC_INTERRUPT_BYTE4,
    INT_CEC_INTERRUPT_BYTE5,
    INT_CEC_INTERRUPT_BYTE6,
    INT_CEC_INTERRUPT_BYTE7,
    INT_NUM
}IntNumT;



typedef enum
{
    NOTIFY_CABLE=0,
    NOTIFY_HDCP,
    NOTIFY_AVI_INFOFRAME,
    NOTIFY_AUD_INFOFRAME,
    NOTIFY_ACP_INFOFRAME,
    NOTIFY_SPD_INFOFRAME,
    NOTIFY_MS_INFOFRAME,
    NOTIFY_VS_INFOFRAME,
    NOTIFY_ISRC1_INFOFRAME,
    NOTIFY_ISRC2_INFOFRAME,
    NOTIFY_GAMUT_MDATA_INFOFRAME,
    NOTIFY_AUDIO_C_INFOFRAME,
    NOTIFY_GEN_CTL_INFOFRAME,
    NOTIFY_CEC,
    NOTIFY_AVMUTE,
    NOTIFY_TMDS,
    NOTIFY_STDI,
    NOTIFY_AUDIO,
    NOTIFY_NUM
}NOTIFY_TYPE;
typedef enum
{
    HDCP_AKSV_UPDATE=0,
    HDCP_RI_EXPIRED,
    HDCP_HDMI_ENCRPT,
    HDCP_HDMI_NO_ENCRPT,
    HDCP_MSG_NUM
}HdcpMsgTypeT;
typedef enum
{
    TMDS_CLK_NO_ACTIVITY=0,
    TMDS_CLK_ACTIVITY,
    TMDS_PLL_NO_LOCK,
    TMDS_PLL_LOCK,
    TMDS_NEW_FREQ,
    TMDS_V_LOCKD,
    TMDS_V_NO_LOCKD,
    TMDS_MSG_NUM
}TmdsMsgTypeT;
typedef enum
{
    STDI_NO_VALID=0,
    STDI_VALID,
    STDI_MSG_NUM
}StdiMsgTypeT;
typedef enum
{
    AUDIO_FS_CHANGE=0,
    AUDIO_CS_DATA_VALID,
    AUDIO_CS_DATA_INVALID,
    AUDIO_MSG_NUM
}AudioMsgTypeT;
typedef enum
{
    AV_UNMUTE=0,
    AV_MUTE,
    AV_MUTE_NUM
}AvMuteTypeT;
typedef enum
{
    SELCET_INT2=0,
    SELCET_INT1
}SelectIntT;
#define INT_BASE_RAW_ADDRESS 0x0
#define INT_BASE_ST_ADDRESS 0x1
#define INT_BASE_CLR_ADDRESS 0x2
#define INT_BASE_MB2_ADDRESS 0x3
#define INT_BASE_MB1_ADDRESS 0x4


#define INT_ADDRESS_OFFSET 0x05

typedef enum
{
    DUR_4_XTAL_PERIOD=0,
    DUR_16_XTAL_PERIOD,
    DUR_64_XTAL_PERIOD,
    DUR_ACT_UNTIL_CLEAR
}DUR_SEL;
typedef enum
{
    IRQ1_OP_OPEN_DRAIN=0,
    IRQ1_OP_DRIVER_LOW,
    IRQ1_OP_DRIVER_HIGH,
    IRQ1_OP_DISABLE
}IRQ1_OP_SEL;
typedef enum
{
    IRQ2_POL_DRIVER_LOW=0,
    IRQ2_POL_DRIVER_HIGH,
}IRQ2_POL_SEL;
typedef enum
{
    IRQ2_DISABLE=0,
    IRQ2_SEL_SCLK,
    IRQ2_SEL_MCLK,
    IRQ2_SEL_HPA_A
}IRQ2_MUX_SEL;
typedef struct
{
    DUR_SEL Intrq1_dur_sel;
    IRQ1_OP_SEL Intrq1_op_sel;
    DUR_SEL Intrq2_dur_sel;
    IRQ2_POL_SEL Intrq2_pol;
    IRQ2_MUX_SEL Intrq2_mux_sel;
}IntConfigT;

#define INT_MODULE_NUM ((INT_NUM>>4)+1)

typedef enum
{
    PACKET_AVI=0,
    PACKET_AUD,
    PACKET_SPD,
    PACKET_MS,
    PACKET_VS,
    PACKET_ACP,
    PACKET_ISRC1,
    PACKET_ISRC2,
    PACKET_GAMUT,
    PACKET_ACR,
    PACKET_GEN_CTL,
    PACKET_NUM
} CSHDMIRX_InfoFrameType;

#define PACKET_HEADER_LEN  3

enum
{
    PACKET_AVI_ID_ADDR=0xE0,
    PACKET_AVI_VERSION_ADDR,
    PACKET_AVI_LEN_ADDR,
    PACKET_AUD_ID_ADDR,
    PACKET_AUD_VERSION_ADDR,
    PACKET_AUD_LEN_ADDR,
    PACKET_SPD_ID_ADDR,
    PACKET_SPD_VERSION_ADDR,
    PACKET_SPD_LEN_ADDR,
    PACKET_MS_ID_ADDR,
    PACKET_MS_VERSION_ADDR,
    PACKET_MS_LEN_ADDR,
    PACKET_VS_ID_ADDR,
    PACKET_VS_VERSION_ADDR,
    PACKET_VS_LEN_ADDR,
    PACKET_ACP_ID_ADDR,
    PACKET_ACP_VERSION_ADDR,
    PACKET_ACP_LEN_ADDR,
    PACKET_ISRC1_ID_ADDR,
    PACKET_ISRC1_VERSION_ADDR,
    PACKET_ISRC1_LEN_ADDR,
    PACKET_ISRC2_ID_ADDR,
    PACKET_ISRC2_VERSION_ADDR,
    PACKET_ISRC2_LEN_ADDR,
    PACKET_GAMUT_ID_ADDR,
    PACKET_GAMUT_VERSION_ADDR,
    PACKET_GAMUT_LEN_ADDR
};

enum
{
    PACKET_AVI_DATA_ADDR=0x00,
    PACKET_AVI_DATA_LEN=0x1C,
    PACKET_AUD_DATA_ADDR=(PACKET_AVI_DATA_ADDR+PACKET_AVI_DATA_LEN),
    PACKET_AUD_DATA_LEN=0x0E,
    PACKET_SPD_DATA_ADDR=(PACKET_AUD_DATA_ADDR+PACKET_AUD_DATA_LEN),
    PACKET_SPD_DATA_LEN=0x1C,
    PACKET_MS_DATA_ADDR=(PACKET_SPD_DATA_ADDR+PACKET_SPD_DATA_LEN),
    PACKET_MS_DATA_LEN=0x0E,
    PACKET_VS_DATA_ADDR=(PACKET_MS_DATA_ADDR+PACKET_MS_DATA_LEN),
    PACKET_VS_DATA_LEN=0x1C,
    PACKET_ACP_DATA_ADDR=(PACKET_VS_DATA_ADDR+PACKET_VS_DATA_LEN),
    PACKET_ACP_DATA_LEN=0x1C,
    PACKET_ISRC1_DATA_ADDR=(PACKET_ACP_DATA_ADDR+PACKET_ACP_DATA_LEN),
    PACKET_ISRC1_DATA_LEN=0x1C,
    PACKET_ISRC2_DATA_ADDR=(PACKET_ISRC1_DATA_ADDR+PACKET_ISRC1_DATA_LEN),
    PACKET_ISRC2_DATA_LEN=0x1C,
    PACKET_GAMUT_DATA_ADDR=(PACKET_ISRC2_DATA_ADDR+PACKET_ISRC2_DATA_LEN),
    PACKET_GAMUT_DATA_LEN=0x1C,
};

typedef enum _DF_VIDEO_FORMAT_
{
    NOT_STANDARD_VIDEO_MODE = -1,
    DISP_YUV_720P_60FPS        =0,
    DISP_YUV_1080I_30FPS       =1,
    DISP_YUV_480P              =2,
    DISP_YUV_NTSC              =3,
    DISP_YUV_720P_50FPS        =4,
    DISP_YUV_1080I_25FPS       =5,
    DISP_YUV_576P              =6,
    DISP_YUV_PAL               =7,
    DISP_YUV_1080P_60FPS       =8,
    DISP_YUV_1080P_50FPS       =9,
    DISP_YUV_1080P_24FPS       =10,
    DISP_YUV_1080P_25FPS       =11,
    DISP_YUV_1080P_30FPS       =12,
    DISP_YUV_1080I_25FPS_AS    =13,
    DISP_RGB_VGA_60FPS         =14,
    DISP_RGB_SVGA_60FPS        =15,
    DISP_RGB_XGA_60FPS         =16,
    DISP_RGB_SXGA_60FPS        =17,
    DISP_RGB_UXGA_60FPS        =18,
    DISP_RGB_720P_60FPS        =19,
    DISP_1440x288P_50_M1       =20,
    DISP_1440x288P_50_M2       =21,
    DISP_1440x288P_50_M3       =22,
    DISP_2880x576I_50          =23,
    DISP_2880x288P_50_M1       =24,
    DISP_2880x288P_50_M2       =25,
    DISP_2880x288P_50_M3       =26,
    DISP_1440x576P_50          =27,
    DISP_1440x240P_60_M1       =28,
    DISP_1440x240P_60_M2       =29,
    DISP_2880x480I_60          =30,
    DISP_2880x240P_60_M1       =31,
    DISP_2880x240P_60_M2       =32,
    DISP_1440x480P_60          =33,
    DISP_RGB_DMT_640x480_60    =34,
    DISP_RGB_DMT_640x480_75    =35,
    DISP_RGB_DMT_800x600_60    =36,
    DISP_RGB_DMT_800x600_75    =37,
    DISP_RGB_DMT_848x480_60    =38,
    DISP_RGB_DMT_1152x864_75   =39,
    DISP_RGB_DMT_1280x768_60   =40,
    DISP_RGB_DMT_1280x960_60   =41,
    DISP_RGB_DMT_1280x1024_60  =42,
    DISP_RGB_DMT_1360x768_60   =43,
    DISP_RGB_DMT_1440x900_60   =44,
    DISP_RGB_CVT_640x480_75    =45,
    DISP_RGB_CVT_800x600_60    =46,
    DISP_RGB_CVT_800x600_75    =47,
    DISP_RGB_CVT_1024x768_60   =48,
    DISP_RGB_CVT_1280x1024_60  =49,
    DISP_RGB_CVT_848x480_60    =50,
    DISP_RGB_CVT_848x480_75    =51,
    DISP_RGB_CVT_1024x576_60   =52,
    DISP_RGB_CVT_1024x576_75   =53,
    DISP_RGB_CVT_1280x720_60   =54,
    DISP_RGB_CVT_1280x720_75   =55,
    DISP_RGB_CVT_640x400_75    =56,
    DISP_RGB_CVT_768x480_60    =57,
    DISP_RGB_CVT_768x480_75    =58,
    DISP_RGB_CVT_1024x640_60   =59,
    DISP_RGB_CVT_1024x640_75   =60,
    DISP_RGB_CVT_1280x800_75   =61,
    DISP_RGB_CVT_800x600_RED   =62,
    DISP_RGB_CVT_1024x768_RED  =63,
    DISP_RGB_CVT_1280x960_RED  =64,
    DISP_RGB_CVT_848x480_RED   =65,
    DISP_RGB_CVT_1024x576_RED  =66,
    DISP_RGB_CVT_1280x720_RED  =67,
    DISP_RGB_CVT_1600x900_RED  =68,
    DISP_RGB_CVT_768x480_RED   =69,
    DISP_RGB_CVT_1024x640_RED  =70,
    DISP_RGB_CVT_1280x800_RED  =71,
    DISP_RGB_CVT_1600x1000_RED =72,
       DISP_64x64                 =73,
       DISP_VGA_1366x768           =74,
    DISP_YUV_1080P_60FPS_1,
       DISP_YUV_480P_3D,
    DISP_YUV_NTSC_3D,
    DISP_YUV_576P_3D,
    DISP_YUV_PAL_3D,
    DISP_YUV_720P_60FPS_3D,
    DISP_YUV_1080P_30FPS_3D,
    DISP_YUV_1080I_30FPS_3D,
    DISP_YUV_720P_50FPS_3D,
    DISP_YUV_1080P_24FPS_3D,
    DISP_YUV_1080P_25FPS_3D,
    DISP_YUV_720P_30FPS_3D,
    DISP_YUV_1080P_30FPS_3D_TEMP,
    DISP_YUV_1080I_30FPS_3D_TEMP,
    DISP_YUV_720P_59P94,
    DISP_YUV_1080I_29P97,
    DISP_YUV_480P_59P94,
    DISP_YUV_NTSC_29P97, 
    DISP_YUV_1080P_59P94,
    DISP_YUV_1080P_23P98,
    DISP_YUV_720P_59P94_3D,
    DISP_YUV_1080P_23P98_3D,
    DISP_RGB_VGA_59P94FPS,
    DISP_FMT_MAX,
}DF_VIDEO_FORMAT;

#include "cstvout.h"
/*!
*@brief Translate input formats to output formats 
*/
static inline CSTVOUT_MODE CSHDMIRX_FORMAT_to_CSTVOUT_MODE(DF_VIDEO_FORMAT df_video_format, CSTVOUT_HDMI_ASPECT_RATIO aspect_ratio, CSTVOUT_HDMI_3D_FORMAT threeD)
{
    CSTVOUT_MODE tvout_mode = TVOUT_MODE_1080P60; /* default to 1080p60 */
    
    switch(df_video_format) {
        case DISP_YUV_720P_60FPS:
            if(threeD == HDMI_3D_TOPBOTTOM) {
                tvout_mode = TVOUT_MODE_720P60_3DTB;
            } else {
                tvout_mode = TVOUT_MODE_720P60;
            }
        break;
        case DISP_YUV_1080I_30FPS:
            if(threeD == HDMI_3D_SIDEBYSIDEHALF) {
                tvout_mode = TVOUT_MODE_1080I60_3DSS;
            } else {
                tvout_mode = TVOUT_MODE_1080I30;
            }
        break;
        case DISP_YUV_480P:
            tvout_mode = TVOUT_MODE_480P;
            if (aspect_ratio == CSTVOUT_HDMI_ASPECT_16x9) {
                tvout_mode = TVOUT_MODE_480P60_16TO9;
            }
        break;
        case DISP_YUV_NTSC:
            tvout_mode = TVOUT_MODE_480I;
            if (aspect_ratio == CSTVOUT_HDMI_ASPECT_16x9) {
                tvout_mode = TVOUT_MODE_480I60_16TO9;
            }
        break;
        case DISP_YUV_NTSC_29P97:        
            tvout_mode = TVOUT_MODE_480I5994;
            if (aspect_ratio == CSTVOUT_HDMI_ASPECT_16x9) {
                tvout_mode = TVOUT_MODE_480I5994_16TO9;
            }
        break;
        case DISP_YUV_720P_50FPS:
            if(threeD == HDMI_3D_TOPBOTTOM) {
                tvout_mode = TVOUT_MODE_720P50_3DTB;
            } else {
                tvout_mode = TVOUT_MODE_720P50;
            }
        break;
        case DISP_YUV_1080I_25FPS_AS: /* ??? */
        case DISP_YUV_1080I_25FPS:
            if(threeD == HDMI_3D_SIDEBYSIDEHALF) {
                tvout_mode = TVOUT_MODE_1080I50_3DSS;
            } else {
                tvout_mode = TVOUT_MODE_1080I25;
            }
        break;
        case DISP_YUV_576P:
            tvout_mode = TVOUT_MODE_576P;
            if (aspect_ratio == CSTVOUT_HDMI_ASPECT_16x9) {
                tvout_mode = TVOUT_MODE_576P50_16TO9;
            }
        break;
        case DISP_YUV_PAL:
        case DISP_2880x576I_50: /* !!! not supported !!! */
            tvout_mode = TVOUT_MODE_576I;
            if (aspect_ratio == CSTVOUT_HDMI_ASPECT_16x9) {
                tvout_mode = TVOUT_MODE_576I_16TO9;
            }
        break;
        case DISP_YUV_1080P_60FPS:
        case DISP_YUV_1080P_60FPS_1: /* ??? */
            tvout_mode = TVOUT_MODE_1080P60;
        break;
        case DISP_YUV_1080P_50FPS:
            tvout_mode = TVOUT_MODE_1080P50;
        break;
        case DISP_YUV_1080P_24FPS:
            if(threeD == HDMI_3D_TOPBOTTOM) {
                tvout_mode = TVOUT_MODE_1080P24_3DTB;
            } else {
                tvout_mode = TVOUT_MODE_1080P24;
            }
        break;
        case DISP_YUV_1080P_25FPS:
            tvout_mode = TVOUT_MODE_1080P25;
        break;
        case DISP_YUV_1080P_30FPS:
            tvout_mode = TVOUT_MODE_1080P30;
        break;
        case DISP_RGB_VGA_60FPS:
            tvout_mode = TVOUT_RGB_640X480_60FPS;
        break;
        case DISP_RGB_VGA_59P94FPS: 
            tvout_mode = TVOUT_RGB_640X480_5994FPS;
        break;
        case DISP_RGB_SVGA_60FPS:
            tvout_mode = TVOUT_RGB_800X600_60FPS;
        break;
        case DISP_RGB_XGA_60FPS:
            tvout_mode = TVOUT_RGB_1024X768_60FPS;
        break;
        case DISP_RGB_SXGA_60FPS:
            tvout_mode = TVOUT_RGB_1280X1024_60FPS;
        break;
        case DISP_RGB_UXGA_60FPS: /* !!! RGB 1600x1200p60 not supported !!! */
            tvout_mode = TVOUT_MODE_1600X1200_60FPS;
        break;
        case DISP_RGB_720P_60FPS:
            tvout_mode = TVOUT_RGB_1280X720_60FPS;
        break;
        case DISP_1440x576P_50: /* !!! not supported !!! */
        case DISP_1440x288P_50_M1: /* !!! not supported !!! */
        case DISP_1440x288P_50_M2: /* !!! not supported !!! */
        case DISP_1440x288P_50_M3: /* !!! not supported !!! */
        case DISP_2880x288P_50_M1: /* !!! not supported !!! */
        case DISP_2880x288P_50_M2: /* !!! not supported !!! */
        case DISP_2880x288P_50_M3: /* !!! not supported !!! */        
            tvout_mode = TVOUT_MODE_1080P50;
        break;
        case DISP_1440x240P_60_M1: /* !!! not supported !!! */
        case DISP_1440x240P_60_M2: /* !!! not supported !!! */
        case DISP_2880x480I_60: /* !!! not supported !!! */
        case DISP_2880x240P_60_M1: /* !!! not supported !!! */
        case DISP_2880x240P_60_M2: /* !!! not supported !!! */
        case DISP_1440x480P_60: /* !!! not supported !!! */
        case DISP_RGB_DMT_640x480_60: /* ??? */
        case DISP_RGB_DMT_640x480_75: /* ??? */
        case DISP_RGB_DMT_800x600_75: /* ??? */
        case DISP_RGB_DMT_848x480_60: /* ??? */
        case DISP_RGB_DMT_1152x864_75: /* ??? */
        case DISP_RGB_DMT_1280x768_60: /* ??? */
        case DISP_RGB_DMT_1280x960_60: /* ??? */
        case DISP_RGB_DMT_1280x1024_60: /* ??? */
        case DISP_RGB_CVT_640x480_75: /* ??? */
        case DISP_RGB_CVT_800x600_75: /* ??? */
        case DISP_RGB_CVT_1024x768_60: /* ??? */
        case DISP_RGB_CVT_1280x1024_60: /* ??? */
        case DISP_RGB_CVT_848x480_60: /* ??? */
        case DISP_RGB_CVT_848x480_75: /* ??? */
        case DISP_RGB_CVT_1024x576_60: /* ??? */
        case DISP_RGB_CVT_1024x576_75: /* ??? */
        case DISP_RGB_CVT_1280x720_60: /* ??? */
        case DISP_RGB_CVT_1280x720_75: /* ??? */
        case DISP_RGB_CVT_640x400_75: /* ??? */
        case DISP_RGB_CVT_768x480_60: /* ??? */
        case DISP_RGB_CVT_768x480_75: /* ??? */
        case DISP_RGB_CVT_1024x640_60: /* ??? */
        case DISP_RGB_CVT_1024x640_75: /* ??? */
        case DISP_RGB_CVT_1280x800_75: /* ??? */
        case DISP_RGB_CVT_1024x768_RED: /* ??? */
        case DISP_RGB_CVT_1280x960_RED: /* ??? */
        case DISP_RGB_CVT_848x480_RED: /* ??? */
        case DISP_RGB_CVT_1024x576_RED: /* ??? */
        case DISP_RGB_CVT_1280x720_RED: /* ??? */
        case DISP_RGB_CVT_1600x900_RED: /* ??? */
        case DISP_RGB_CVT_768x480_RED: /* ??? */
        case DISP_RGB_CVT_1024x640_RED: /* ??? */
        case DISP_RGB_CVT_1280x800_RED: /* ??? */
        case DISP_RGB_CVT_1600x1000_RED: /* ??? */    
        case DISP_64x64: /* ??? */
        case DISP_YUV_480P_3D: /* !!! not supported !!! */
        case DISP_YUV_NTSC_3D: /* !!! not supported !!! */
        case DISP_YUV_576P_3D: /* !!! not supported !!! */
        case DISP_YUV_PAL_3D: /* !!! not supported !!! */     
        case DISP_YUV_1080P_30FPS_3D: /* !!! not supported !!! */   
        case DISP_YUV_1080I_30FPS_3D: /* !!! not supported !!! */
        case DISP_YUV_1080P_25FPS_3D: /* !!! not supported !!! */
        case DISP_YUV_720P_30FPS_3D: /* !!! not supported !!! */
        case DISP_YUV_1080P_30FPS_3D_TEMP: /* !!! not supported !!! */
        case DISP_YUV_1080I_30FPS_3D_TEMP: /* !!! not supported !!! */
            tvout_mode = TVOUT_MODE_1080P60;
        break;    
        case DISP_RGB_CVT_800x600_60: /* ??? */
        case DISP_RGB_CVT_800x600_RED: /* ??? */
        case DISP_RGB_DMT_800x600_60: /* ??? */
            tvout_mode = TVOUT_RGB_800X600_60FPS;
        break; 
        case DISP_RGB_DMT_1360x768_60: /* ??? */
            tvout_mode = TVOUT_RGB_1360X768_60FPS;
        break; 
        case DISP_RGB_DMT_1440x900_60: /* ??? */
            tvout_mode = TVOUT_RGB_1440X900_60FPS;
        break;      
        case DISP_VGA_1366x768:
            tvout_mode = TVOUT_RGB_1366X768_60FPS;
        break;        
        case DISP_YUV_720P_50FPS_3D:
            tvout_mode = TVOUT_MODE_720P50_3DFP;
        break;
        case DISP_YUV_720P_60FPS_3D:
            tvout_mode = TVOUT_MODE_720P60_3DFP;
        break;
        case DISP_YUV_1080P_24FPS_3D:
            tvout_mode = TVOUT_MODE_1080P24_3DFP;
        break;
        case DISP_YUV_720P_59P94:
            if(threeD == HDMI_3D_TOPBOTTOM) {
                tvout_mode = TVOUT_MODE_720P5994_3DTB;
            } else {
                tvout_mode = TVOUT_MODE_720P5994;
            }
        break;
        case DISP_YUV_1080I_29P97:
            if(threeD == HDMI_3D_SIDEBYSIDEHALF) {
                tvout_mode = TVOUT_MODE_1080I5994_3DSS;
            } else {
                tvout_mode = TVOUT_MODE_1080I5994; // I am guessing the TVOUT MODE has a typo and 59.94 are fields per second, hence it is the same as 29.97 frames per second
            }
        break;
        case DISP_YUV_480P_59P94:
            tvout_mode = TVOUT_MODE_480P5994;
            if (aspect_ratio == CSTVOUT_HDMI_ASPECT_16x9) {
                tvout_mode = TVOUT_MODE_480P5994_16TO9;
            }
        break;
        case DISP_YUV_1080P_59P94:
            tvout_mode = TVOUT_MODE_1080P5994;
        break;
        case DISP_YUV_1080P_23P98:
            if(threeD == HDMI_3D_TOPBOTTOM) {
                tvout_mode = TVOUT_MODE_1080P2397_3DTB;
            } else {
                tvout_mode = TVOUT_MODE_1080P2397;
            }
        break;
        case DISP_YUV_720P_59P94_3D:
            tvout_mode = TVOUT_MODE_720P5994_3DFP;
        break;
        case DISP_YUV_1080P_23P98_3D:
            tvout_mode = TVOUT_MODE_1080P2397_3DFP;
        break;
        default:
        break;
    }
    
    return tvout_mode;
}
               
typedef enum
{
    CSHDMIRX_AUD_IF_I2S,
    CSHDMIRX_AUD_IF_SPDIF,
    CSHDMIRX_AUD_IF_DSD,
    CSHDMIRX_AUD_IF_DST,
    CSHDMIRX_AUD_IF_AUTO
} CSHDMIRX_AUD_INTERFACE;
typedef  enum
{
    FS_44_1_KHZ=0,
    FS_48_KHZ=2,
    FS_32_KHZ=3,
    FS_88_2_KHZ=8,
    FS_768_KHZ=9,
    FS_96_KHZ=10,
    FS_176_KHZ=12,
    FS_192_KHZ=14
}AUDIO_FS;
typedef struct CecMsg
{
    unsigned char Header;
    unsigned char Data[CEC_MSG_LEN-2];
    unsigned char Len;
}CecMsgT;
typedef struct _CSHDMIRX_InfoFrame_t
{
    unsigned char HB[3];
    unsigned char Data[MAX_INFOFRAME_DATA_LEN];
}CSHDMIRX_InfoFrame_t;


typedef struct
{
    unsigned char ApplicationType;
    unsigned char AudioDataType;
    unsigned char Copyright;
    unsigned char Emphasis;
    unsigned char ChannelStatusMode;
    unsigned char CategoryCode;
    unsigned char SourceNumber;
    unsigned char ChannelNumber;
    unsigned char AudioSampleRate;
    unsigned char ClockAccuracy;
    unsigned char BitWidth;
}AudioDescriptionT;

typedef void (*IntCallback)(void);
CSAPI_RESULT CSHDMIRX_Open(void);
CSAPI_RESULT CSHDMIRX_Close(void);
CSAPI_RESULT CSHDMIRX_SelectPort(HDMI_PORT Port);
CSAPI_RESULT CSHDMIRX_SetHpd(unsigned char Level);
CSAPI_RESULT CSHDMIRX_SetHpdAll(unsigned char Level);
CSAPI_RESULT CSHDMIRX_GetHpdStatus(unsigned char *hpd_status);
CSAPI_RESULT CSHDMIRX_GetPlugStatus(unsigned char *plug_status);
CSAPI_RESULT CSHDMIRX_DetectFormat(DF_VIDEO_FORMAT *VIDEO_FORMAT);
CSAPI_RESULT CSHDMIRX_GetRxTvMode(DF_VIDEO_FORMAT *videoFormat,unsigned int *srcWidth, unsigned int *srcHeight, unsigned int *srcFPS, unsigned int *is_interlace, int *srcAspectRatio, unsigned int *threeD);
CSAPI_RESULT CSHDMIRX_GetFs(unsigned char *pFs);
CSAPI_RESULT CSHDMIRX_GetAudioDataType(unsigned char *pAudioDataType);
CSAPI_RESULT CSHDMIRX_GetAudioDescription(AudioDescriptionT *pAudioDescription);
CSAPI_RESULT CSHDMIRX_GetInfoFrame(CSHDMIRX_InfoFrameType type, CSHDMIRX_InfoFrame_t *info);
CSAPI_RESULT CSHDMIRX_SetIntpyeI2S(unsigned char I2SBitWidth);
CSAPI_RESULT CSHDMIRX_SetIntpyeSpdif(unsigned char I2SBitWidth);
CSAPI_RESULT CSHDMIRX_SetAudioMute(unsigned char Enable);
CSAPI_RESULT CSHDMIRX_SetManSigPhase(int phase);
CSAPI_RESULT CSHDMIRX_RegisterNotify(NOTIFY_TYPE NotifyType,IntCallback cablecallback);
CSAPI_RESULT CSHDMIRX_CecSentMsg(CecMsgT *pCecMsg);
CSAPI_RESULT CSHDMIRX_ForceRiMismatch(void);
CSAPI_RESULT CSHDMIRX_SetKsv(unsigned char *BksvList,unsigned char BksvCount);
CSAPI_RESULT CSHDMIRX_SetKsvReady(unsigned char Ready);
CSAPI_RESULT CSHDMIRX_SetBstatus(unsigned short Bstatus);
CSAPI_RESULT CSHDMIRX_GetBstatus(unsigned short *Bstatus);
CSAPI_RESULT CSHDMIRX_SetRepeaterMode(unsigned char Enable);
CSAPI_RESULT CSHDMIRX_SetEdid(unsigned char *pEdidBuf,unsigned short Length);
CSAPI_RESULT CSHDMIRX_GetSelectedPort(HDMI_PORT *pSelectPort);
CSAPI_RESULT CSHDMIRX_ChipPowerDown(void);
CSAPI_RESULT CSHDMIRX_ChipPowerUp(void);
CSAPI_RESULT CSHDMIRX_SetCecPhysicalAddress(unsigned short CecPhysicalAddress);
CSAPI_RESULT CSHDMIRX_GetAudioDataType(unsigned char *pAudioDataType);
CSAPI_RESULT CSHDMIRX_SetAudioDataType(unsigned char AudioDataType);
int          CSHDMIRX_StdiIsInvalid(void);
CSAPI_RESULT CSHDMIRX_SPDIF_Supported(unsigned int *supported);

#ifdef __cplusplus
}
#endif

#endif                /* __CSAPI_HDMIRX_H */
