/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Cavium, Inc.
 Cavium, Inc. reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium, Inc.

 ALL RIGHTS RESERVED.

 Copyright 2011-2012, Cavium, Inc.
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */

/**
 * @file csosd.h
 * @ingroup csapi
 * Low Level OSD API
 */

#ifndef __CSAPI_OSD_H__
#define __CSAPI_OSD_H__

#include "global.h"

#ifdef __cplusplus
extern "C" {
#endif

typedef void *CSOSD_HANDLE;


/*!
*@brief Define the resolution of OSD layer.

* @see

*/
typedef enum
{
	OSD_MODE_576I = 0,
	OSD_MODE_480I,  ///< To indicate the resolution of OSD layer is 720x576.
	OSD_MODE_576P,  ///< To indicate the resolution of OSD layer is 720x480.
	OSD_MODE_480P,   ///< To indicate the resolution of OSD layer is 720x480.
	OSD_MODE_720P50, ///< To indicate the resolution of OSD layer is 1280x720.
	OSD_MODE_720P60, ///< To indicate the resolution of OSD layer is 1280x720.
	OSD_MODE_1080I25, ///< To indicate the resolution of OSD layer is 1920x1080.
	OSD_MODE_1080I30, ///< To indicate the resolution of OSD layer is 1920x1080.

	OSD_MODE_SECAM,  ///< To indicate the resolution of OSD layer is 720x576.
	OSD_MODE_PAL_M,   ///< To indicate the resolution of OSD layer is 720x576.
	OSD_MODE_PAL_N,   ///< To indicate the resolution of OSD layer is 720x480.
	OSD_MODE_PAL_CN,  ///< To indicate the resolution of OSD layer is 720x576.
	OSD_MODE_1080P24, ///< To indicate the resolution of OSD layer is 1920x1080.
	OSD_MODE_1080P25, ///< To indicate the resolution of OSD layer is 1920x1080.
	OSD_MODE_1080P30, ///< To indicate the resolution of OSD layer is 1920x1080.
	OSD_MODE_1080P50, ///< To indicate the resolution of OSD layer is 1920x1080.
	OSD_MODE_1080P60, ///< To indicate the resolution of OSD layer is 1920x1080.

	OSD_MODE_640X480_60FPS, ///< To indicate the resolution of OSD layer is 640x480.
	OSD_MODE_800X600_60FPS, ///< To indicate the resolution of OSD layer is 600x800.
	OSD_MODE_1024X768_60FPS, ///< To indicate the resolution of OSD layer is 1024x768.
	OSD_MDOE_1280X720_60FPS, ///< To indicate the resolution of OSD layer is 1280x720.
       OSD_MODE_800X480_60FPS, ///< To indicate the resolution of OSD layer is 800x480.
       OSD_MDOE_1440X900_60FPS, ///< To indicate the resolution of OSD layer is 1440x900.
       OSD_MODE_1280X1024_60FPS, ///< To indicate the resolution of OSD layer is 1280x1024.

	OSD_MODE_UNUSED,
} CSOSD_MODE;

/*!
*@brief Define the OSD layer supported by the OSD driver.
  @note Currently don't support OSD_LAYER_1.
* @see

*/
typedef enum
{
	OSD_LAYER_0 = 0,///<Hardware osd layer 0.
	OSD_LAYER_1///<Hardware osd layer 1.
} CSOSD_LAYER;

/*!
*@brief Define the bits per pixel of the OSD buffer.

* @see

*/
typedef enum
{
	OSD_COLOR_DEPTH_16 = 16,///<Color format is 16bits per pixel.
	OSD_COLOR_DEPTH_32 = 32,///<Color format is 32bits per pixel.
} CSOSD_COLOR_DEPTH;

/*!
*@brief Define the resolution of OSD layer.

* @see

*/
typedef enum
{
	OSD_COLOR_FORMAT_RGB565    = 2,///<OSD color format is RGB565.
	OSD_COLOR_FORMAT_ARGB4444  = 3,///<OSD color format is ARGB4444.
	OSD_COLOR_FORMAT_ARGB1555  = 5,///<OSD color format is ARGB1555.
	OSD_COLOR_FORMAT_ARGB8888  = 6,///<OSD color format is ARGB8888.

} CSOSD_COLOR_FORMAT;

/*!
*@brief Define configuration of OSD layer
* @see

*/
typedef struct tagOSD_Config {
	CSOSD_MODE mode;///<The resolution of OSD layer.
	CSOSD_COLOR_DEPTH color_depth;///<The bits per pixe.
	CSOSD_COLOR_FORMAT color_format;///<The color format of OSD buffer.
} CSOSD_Config;

/*!
*@brief Define a color key.

* @see

*/
typedef struct tagOSD_KeyColor {
	char r_min;///<The minimum of Red color
	char r_max;///<The maximum of Red color

	char g_min;///<The minimum of Green color
	char g_max;///<The maximum of Green color

	char b_min;///<The minimum of Blue color
	char b_max;///<The maximum of Blue color
} CSOSD_KeyColor;

/*!
*@brief Define a rectangle of the osd layer.

* @see

*/
typedef struct tagOSD_Rect {
	unsigned int left;///<The left of osd layer rectangle
	unsigned int top;///<The top of osd layer rectangle
	unsigned int right;///<The right of osd layer rectangle
	unsigned int bottom;///<The bottom of osd layer rectangle
} CSOSD_Rect;

/*!
*@brief Define all the error type.

* @see

*/
typedef enum
{
	OSD_NO_ERROR = 0, ///< No error
	OSD_ERROR_OPEN_FAILED,///< open filed
	OSD_ERROR_IOCTL_FAILED,	///<  ioctl filed
	OSD_ERROR_INVALID_PARAMETERS,	///< Bad parameter passed
	OSD_ERROR_UNKNOWN_DEVICE,	///< Unknown device name
	OSD_ERROR_DEVICE_BUSY,		///<  Device is currently busy
	OSD_ERROR_INVALID_HANDLE,///< Handle is not valid
	OSD_ERROR_ALREADY_INITIALIZED,///<  Device already initialized
	OSD_ERROR_NOT_INITIALIZED,	///< Device not initialized
	OSD_ERROR_OPERATION_ERROR	///< Function invoke error
} CSOSD_ErrCode;


/*!
 *@brief Open a connection to a graphics layer.
 *
 *     @param	Layer  The graphics layer ID. There are two graphics layer supported. In/Out: In
 *  OSD_LAYER_0: graphics layer 0
 *  OSD_LAYER_1: graphics layer 1
 *  In/Out : In
 *      @param  The status of the video buffer. In/Out : In
 *     @return	A pointer to CSOSD instance. Success
 *     @return  NULL Failed to open OSD layer.
 *		@note This function should be the first call for your application before you call other OSD functions.
 *     @see
 */
CSOSD_HANDLE CSOSD_Open(CSOSD_LAYER layer);

/*!
 *@brief Close a connection to the graphics layer.
 *
 *     @param	handle Handle to the graphics layer. In/Out : In
 *     @return	CSAPI_SUCCEED	Success\n
 *						CSAPI_FAILED Failure
 *		@note
 *     @see
 */
CSAPI_RESULT CSOSD_Close(CSOSD_HANDLE handle);

/*!
 *@brief Enable the graphics layer which is associated with handler.
 *
 *     @param	hendle Handle to the graphics layer. In/Out : In
 *     @return	CSAPI_SUCCEED	Success\n
 *						CSAPI_FAILED Failure
 *		@note This function allows the display hardware to output the contents of graphics layer.
 *     @see
 */
CSAPI_RESULT CSOSD_Enable(CSOSD_HANDLE handle);

/*!
 *@brief Disable the graphics layer which is associated with handler.
 *
 *     @param	hendle Handle to the graphics layer. In/Out : In
 *     @return	CSAPI_SUCCEED	Success\n
 *						CSAPI_FAILED Failure
 *		@note This function does not permit the display hardware to output the contents of graphics layer.
 *     @see
 */
CSAPI_RESULT CSOSD_Disable(CSOSD_HANDLE handle);

/*!
 *@brief Configure the parameters to the graphics layer.
 *
 *     @param	hendle Handle to the graphics layer. In/Out : In
 *     @param config  A pointer to the parameters which will be placed in the graphics layer. In/Out : In
 *     @return	CSAPI_SUCCEED	Success\n
 *						CSAPI_FAILED Failure
 *		@note The graphics layer should be configured before write any data in to display hardware.
 *     @see
 */
CSAPI_RESULT CSOSD_SetConfig(CSOSD_HANDLE handle, CSOSD_Config * config);

/*!
 *@brief Obtain the configuration of the graphics layer from display hardware.
 *
 *     @param	handle  Handle to the graphics layer. In/Out : In
 *     @param config  A pointer to the parameters which will be placed in the graphics layer. In/Out : In
 *     @return	CSAPI_SUCCEED	Success\n
 *						CSAPI_FAILED Failure
 *		@note
 *     @see
 */
CSAPI_RESULT CSOSD_GetConfig(CSOSD_HANDLE handle, CSOSD_Config * config);
/*!
 *@brief Get the base address of frame buffer.
 *
 *     @param	handle Handle to the graphics layer. In/Out : In
 *     @param addr   A pointer to frame buffer which is returned from OSD driver. In/Out : out
 *     @return	CSAPI_SUCCEED	Success\n
 *						CSAPI_FAILED Failure
 *		@note
 *     @see
 */
CSAPI_RESULT CSOSD_GetBaseAddr(CSOSD_HANDLE handle, unsigned char ** addr);
/*!
 *@brief Set an alpha value to the display hardware.
 *
 *     @param	handle  Handle to the graphics layer. In/Out : In
 *     @param alpha   A pointer to frame buffer which is returned from OSD driver. In/Out : In
 *     @return	CSAPI_SUCCEED	Success\n
 *						CSAPI_FAILED Failure
 *		@note
 *     @see
 */
CSAPI_RESULT CSOSD_SetAlpha(CSOSD_HANDLE handle, int alpha);

/*!
 *@brief Get the alpha value from the display hardware.
 *
 *     @param	handle   Handle to the graphics layer. In/Out : In
 *     @param alpha   A pointer to frame buffer which is returned from OSD driver. In/Out : In
 *     @return	CSAPI_SUCCEED	Success\n
 *						CSAPI_FAILED Failure
 *		@note
 *     @see
 */
CSAPI_RESULT CSOSD_GetAlpha(CSOSD_HANDLE handle, int * alpha);
/*!
 *@brief Enable to set key color.
 *
 *     @param	handle   Handle to the graphics layer. In/Out : In
 *     @return	CSAPI_SUCCEED	Success\n
 *						CSAPI_FAILED Failure
 *		@note
 *     @see
 */
CSAPI_RESULT CSOSD_EnableKeyColor(CSOSD_HANDLE handle);

/*!
 *@brief Disable to set key color.
 *
 *     @param	handle  Handle to the graphics layer. In/Out : In
 *     @return	CSAPI_SUCCEED	Success\n
 *						CSAPI_FAILED Failure
 *		@note
 *     @see
 */
CSAPI_RESULT CSOSD_DisableKeyColor(CSOSD_HANDLE handle);

/*!
 *@brief Set a value of color key into the display hardware.
 *
 *     @param	handle  Handle to the graphics layer. In/Out : In
 *     @param key_color    A pointer to frame buffer which is returned from OSD driver. In/Out : In
 *     @return	CSAPI_SUCCEED	Success\n
 *						CSAPI_FAILED Failure
 *		@note
 *     @see
 */
CSAPI_RESULT CSOSD_SetKeyColor(CSOSD_HANDLE handle, CSOSD_KeyColor * key_color);

/*!
 *@brief Get a value of color key from the display hardware.
 *
  *     @param	handle   Handle to the graphics layer. In/Out : In
 *     @param key_color   A pointer to a structure returned from the display hardware. In/Out : Out
 *     @return	CSAPI_SUCCEED	Success\n
 *						CSAPI_FAILED Failure
 *		@note
 *     @see
 */
CSAPI_RESULT CSOSD_GetKeyColor(CSOSD_HANDLE handle, CSOSD_KeyColor * key_color);

/*!
 *@brief Copy the OSD data to the display hardware.
 *
 *     @param	handle   Handle to the graphics layer. In/Out : In
 *     @return	CSAPI_SUCCEED	Success\n
 *						CSAPI_FAILED Failure
 *		@note There are two frame buffers, one is source, and another is destination. The data will be filled into the source buffer while write any data through the base address. This function copies the source data to destination buffer.
 *     @see
 */


CSAPI_RESULT CSOSD_Opaque(CSOSD_HANDLE handle,int onoff);
/*!
 *@brief control the opaque value to void colorkey or not.
 *
 *     @param	handle   Handle to the graphics layer. In/Out : In
 *     @return	CSAPI_SUCCEED	Success\n
 *						CSAPI_FAILED Failure
 *		@note
 *     @see
 */


CSAPI_RESULT CSOSD_Flip(CSOSD_HANDLE handle);

/*!
 *@brief Copy the OSD data to the display hardware.
 *
 *     @param	handle   Handle to the graphics layer. In/Out : In
 *     @param src_rect   Source rect position
 *     @param	dst_rect Destination rect position
 *     @return	CSAPI_SUCCEED	Success\n
 *						CSAPI_FAILED Failure
 *		@note There are two frame buffers, one is source, the other is destination. The data will be filled into the source buffer while write any data through the base address. This function copies the source data to destination buffer.
 *     @see
 */
CSAPI_RESULT CSOSD_FlipRect(CSOSD_HANDLE handle,
			CSOSD_Rect *src_rect, CSOSD_Rect *dst_rect);
/*!
 *@brief Wait for a vertical synchronization.
 *
 *     @param	handle   Handle to the graphics layer. In/Out : In
 *     @return	CSAPI_SUCCEED	Success\n
 *						CSAPI_FAILED Failure
 *		@note
 *     @see
 */
CSAPI_RESULT CSOSD_WaitVSync(CSOSD_HANDLE handle);

/*!
 *@brief Get current error code of CSOSD module.
 *
 *     @param	handle   Handle to the graphics layer. In/Out : In
 *     @return	CSOSD_ErrCode
 *		@note
 *     @see
 */
CSOSD_ErrCode CSOSD_GetErrCode(CSOSD_HANDLE handle);

/*!
 *@brief Get current error code character string of CSOSD module.
 *
 *     @param	handle   Handle to the graphics layer. In/Out : In
 *     @return	point of string.
 *		@note
 *     @see
 */
char *CSOSD_GetErrString(CSOSD_HANDLE handle);

CSAPI_RESULT CSOSD_ConfigDistRect(CSOSD_HANDLE handle,CSOSD_Rect *dst_rect);

#ifdef __cplusplus
}
#endif

#endif

