/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Cavium, Inc.
 Cavium, Inc. reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium, Inc.

 ALL RIGHTS RESERVED.

 Copyright 2011-2012, Cavium, Inc.
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */

/**
 * @file cspwrctl.h
 * Low Level Power Control API
 * @ingroup csapi
 */
#ifndef __CSAPI_PWRCTL_H__
#define __CSAPI_PWRCTL_H__

#include "global.h"

#ifdef __cplusplus
extern "C" {
#endif


typedef void* CSPWR_HANDLE;

typedef enum
{
        CSPWR_NO_ERROR = 0,
        CSPWR_ERROR_OPEN_PWR_FAILED,
        CSPWR_ERROR_IOCTL_SETUSBWPOWER_FAILED,     /* ioctl enable/disable USB entering into low power failed       */
        CSPWR_ERROR_IOCTL_SETXPORTCLK_FAILED,   
        CSPWR_ERROR_IOCTL_SETVIBCLK_FAILED,          
        CSPWR_ERROR_IOCTL_SETAUDCLK_FAILED,  
        CSPWR_ERROR_IOCTL_SETCRYPTOCLK_FAILED, 
        CSPWR_ERROR_IOCTL_SETGRAPHICSCLK_FAILED,   
        CSPWR_ERROR_IOCTL_SETDFCLK_FAILED,    
        CSPWR_ERROR_IOCTL_SETVIDECODCLK_FAILED,  
        CSPWR_ERROR_IOCTL_SETAUDDACIFCLK_FAILED,
        CSPWR_ERROR_IOCTL_SETAUDPLLCLK_FAILED,
        CSPWR_ERROR_IOCTL_SETSDCLK_FAILED,  
        CSPWR_ERROR_IOCTL_SETTVE1CLK_FAILED, 
        CSPWR_ERROR_IOCTL_SETTVE0CLK_FAILED,  
        CSPWR_ERROR_IOCTL_SETAUDDECCLK_FAILED,
        CSPWR_ERROR_IOCTL_SETVIDDECCLK_FAILED,
        CSPWR_ERROR_IOCTL_SETVDAC0CLK_FAILED,
        CSPWR_ERROR_IOCTL_SETVDAC1CLK_FAILED,
		CSPWR_ERROR_IOCTL_SETHDMICLK_FAILED,
        CSPWR_ERROR_IOCTL_SETVIBINCLK_FAILED,
        CSPWR_ERROR_IOCTL_SETJPEGCLK_FAILED,
        CSPWR_ERROR_IOCTL_SETAHB2CLK_FAILED,
        CSPWR_ERROR_IOCTL_SETCECCLK_FAILED,
        CSPWR_ERROR_IOCTL_SETIRCLK_FAILED,
        CSPWR_ERROR_CLOSE_PWRC,                    /* close cs_pwr device is failed        */
} CSPWR_ErrCode;

typedef enum _POWER_MODE_
{
	_clock_sleep =0,
	_clock_wake  =1,
}POWER_MODE;


CSPWR_HANDLE CSPWR_Open(void);
CSAPI_RESULT CSPWR_Close(CSPWR_HANDLE handle);
CSAPI_RESULT CSPWR_SetUSB_LowPower(CSPWR_HANDLE handle, POWER_MODE mode);
CSAPI_RESULT CSPWR_SetXPORT_LowPower(CSPWR_HANDLE handle, POWER_MODE mode);
CSAPI_RESULT CSPWR_SetVIB_LowPower(CSPWR_HANDLE handle, POWER_MODE mode);
CSAPI_RESULT CSPWR_SetAUDCLK_LowPower(CSPWR_HANDLE handle, POWER_MODE mode);
CSAPI_RESULT CSPWR_SetCRYPTO_LowPower(CSPWR_HANDLE handle, POWER_MODE mode);
CSAPI_RESULT CSPWR_SetGRAPHICS_LowPower(CSPWR_HANDLE handle, POWER_MODE mode);
CSAPI_RESULT CSPWR_SetDF_LowPower(CSPWR_HANDLE handle, POWER_MODE mode);
CSAPI_RESULT CSPWR_SetVIDECOD_LowPower(CSPWR_HANDLE handle, POWER_MODE mode);
CSAPI_RESULT CSPWR_SetAUDDACIF_LowPower(CSPWR_HANDLE handle, POWER_MODE mode);
CSAPI_RESULT CSPWR_SetAUDPLL_LowPower(CSPWR_HANDLE handle, POWER_MODE mode);
CSAPI_RESULT CSPWR_SetSD_LowPower(CSPWR_HANDLE handle, POWER_MODE mode);
CSAPI_RESULT CSPWR_SetTVE1_LowPower(CSPWR_HANDLE handle, POWER_MODE mode);
CSAPI_RESULT CSPWR_SetTVE0_LowPower(CSPWR_HANDLE handle, POWER_MODE mode);
CSAPI_RESULT CSPWR_SetAUDDEC_LowPower(CSPWR_HANDLE handle, POWER_MODE mode);
CSAPI_RESULT CSPWR_SetVIDDEC_LowPower(CSPWR_HANDLE handle, POWER_MODE mode);
CSAPI_RESULT CSPWR_SetVDAC0_LowPower(CSPWR_HANDLE handle, POWER_MODE mode);
CSAPI_RESULT CSPWR_SetVDAC1_LowPower(CSPWR_HANDLE handle, POWER_MODE mode);
CSAPI_RESULT CSPWR_SetHDMI_LowPower(CSPWR_HANDLE handle, POWER_MODE mode);
CSAPI_RESULT CSPWR_SetJPEG_LowPower(CSPWR_HANDLE handle, POWER_MODE mode);
CSAPI_RESULT CSPWR_SetAHB2_LowPower(CSPWR_HANDLE handle, POWER_MODE mode);
CSAPI_RESULT CSPWR_SetCEC_LowPower(CSPWR_HANDLE handle, POWER_MODE mode);
CSAPI_RESULT CSPWR_SetIR_LowPower(CSPWR_HANDLE handle, POWER_MODE mode);
CSAPI_RESULT CSPWR_SetVIBIN_LowPower(CSPWR_HANDLE handle, POWER_MODE mode);
CSAPI_RESULT CSPWR_SetHDMI_TMDS_LowPower(POWER_MODE mode);

#ifdef __cplusplus
}
#endif

#endif

