/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Cavium, Inc.
 Cavium, Inc. reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium, Inc.

 ALL RIGHTS RESERVED.

 Copyright 2011-2012, Cavium, Inc.
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */

/**
 * @file cssec.h
 * @ingroup csapi
 * Low Level Crypto API
 */

/**
 * @file   cssec_1800l.h
 * @author Fan Xiaodong <xiaodong.fan@caviumnetworks.com>
 * @date   Nov 2011
 *
 * @brief  Header file for Crypto API
 *
 *
 */


#ifndef __CSAPI_CSSEC_H__
#define __CSAPI_CSSEC_H__

#include "global.h"

#ifdef __cplusplus
extern "C" {
#endif

typedef void * CSSEC_CHL_HANDLE;

//typedef char BOOL;

typedef struct tagCSSEC_Init_Params{
}CSSEC_Init_Params;

typedef struct tagCSSEC_Term_Params{
}CSSEC_Term_Params;

typedef void (*CSSEC_NotifyCallback)(CSSEC_CHL_HANDLE handle, void *pContent);

typedef enum _CSSEC_ERROR_CODE
{
    SEC_CORE_SUCCESS                       = 0,
    SEC_CORE_ERROR                         = -1,
    SEC_CORE_UNSUPPORTED_OPERATION         = -2,
	SEC_CORE_INVALID_PARAM                 = -3,
	SEC_CORE_BUFFER_OVERFLOW		       = -4,
	SEC_CORE_BUFFER_UNDERFLOW		       = -5,
	SEC_CORE_CLOSE                         = -6,
    SEC_ERROR_IOCTL_FAILED                 = -7

}CSSEC_ERROR_CODE;

typedef enum _CSSEC_CHL_INDEX{
	CRYPTO_CHL0 = 0,
	CRYPTO_CHL1,
	CRYPTO_CHL2,
    CRYPTO_CHL_MAX,
} CSSEC_CHL_INDEX;

typedef enum _CSSEC_ALGORITHM_IDX{
	TDES = 0,
    AES128 = 1,
	SMS4   = 2,
    AES192 = 9,
    AES256 = 13,
}CSSEC_ALGORITHM_IDX;

typedef enum _CSSEC_CIPHER_MODE_IDX{
	ECB	= 0,
	CBC,
	CFB128,
	OFB128,
	CTS,
	CTR,
	//RESERVED
}CSSEC_CIPHER_MODE_IDX;

typedef enum _CSSEC_SHORT_BLOCK_IDX{
	CLEAR 	= 0,
	DVS042,
	//RESERVED
}CSSEC_SHORT_BLOCK_IDX;

typedef enum _CSSEC_MODE_IDX{
	DECRYPT_AES128_IDSA	    = ((AES128 << 16) & 0xFF0000) | ((CBC << 8) & 0xFF00) | ((DVS042 << 0) & 0xFF),
	DECRYPT_AES128_ECB	    = ((AES128 << 16) & 0xFF0000) | ((ECB << 8) & 0xFF00) | ((CLEAR << 0)  & 0xFF),
	DECRYPT_AES128_CIPlus	= ((AES128 << 16) & 0xFF0000) | ((CBC << 8) & 0xFF00) | ((CLEAR << 0)  & 0xFF),
	DECRYPT_TDES_CBC_DVS042 = ((TDES << 16) & 0xFF0000)  | ((CBC << 8)  & 0xFF00) | ((DVS042 << 0) & 0xFF),
    DECRYPT_DES_ECB_CLEAR	= ((TDES << 16) & 0xFF0000)  | ((ECB << 8)  & 0xFF00) | ((CLEAR << 0)  & 0xFF),
	DECRYPT_AES128_CTR_DVS042 = ((AES128 << 16) & 0xFF0000)  | ((CTR << 8)  & 0xFF00) | ((DVS042 << 0) & 0xFF),
    DECRYPT_AES128_CTR_CLEAR	= ((AES128 << 16) & 0xFF0000)  | ((CTR << 8)  & 0xFF00) | ((CLEAR << 0)  & 0xFF),

	ENCRYPT_AES128_IDSA	    = ((1<< 31) & 0xFF000000)| ((AES128 << 16)& 0xFF0000) | ((CBC << 8)& 0xFF00) | ((DVS042 << 0)& 0xFF),
    ENCRYPT_AES128_ECB	    = ((1<< 31) & 0xFF000000)| ((AES128 << 16)& 0xFF0000) | ((ECB << 8)& 0xFF00) | ((CLEAR << 0) & 0xFF),
    ENCRYPT_AES128_CIPlus	= ((1<< 31) & 0xFF000000)| ((AES128 << 16)& 0xFF0000) | ((CBC << 8)& 0xFF00) | ((CLEAR << 0) & 0xFF),
	ENCRYPT_TDES_CBC_DVS042 = ((1<< 31) & 0xFF000000)| ((TDES << 16)  & 0xFF0000) | ((CBC << 8)& 0xFF00) | ((DVS042 << 0)& 0xFF),
    ENCRYPT_DES_ECB_CLEAR	= ((1<< 31) & 0xFF000000)| ((TDES << 16)  & 0xFF0000) | ((ECB << 8)& 0xFF00) | ((CLEAR << 0) & 0xFF),
	ENCRYPT_AES128_CTR_DVS042 = ((1<< 31) & 0xFF000000)| ((AES128 << 16)  & 0xFF0000) | ((CTR << 8)& 0xFF00) | ((DVS042 << 0)& 0xFF),
    ENCRYPT_AES128_CTR_CLEAR  = ((1<< 31) & 0xFF000000)| ((AES128 << 16)  & 0xFF0000) | ((CTR << 8)& 0xFF00) | ((CLEAR << 0) & 0xFF),
} CSSEC_MODE_IDX;

typedef enum _CSSEC_STATUS
{
    SEC_IDLE                = 0,
	SEC_BUSY                = 1,
	SEC_DONE	            = 2
}CSSEC_STATUS;

typedef  unsigned char CSSEC_ProtectedKey[2][16] ;

CSAPI_RESULT CSSEC_Init(CSSEC_Init_Params *pxInitParameters);
CSAPI_RESULT CSSEC_Term(CSSEC_Term_Params *pxTermParameters);
CSAPI_RESULT CSSEC_PV_Read(unsigned int Cmd, unsigned int *pValue);
CSAPI_RESULT CSSEC_PV_Write(unsigned int Cmd, unsigned int Value);

CSAPI_RESULT CSSEC_PV_Generic_Read(	unsigned int Cmd,
                                    unsigned char *pValue,
                                    unsigned int Size);

CSAPI_RESULT CSSEC_PV_Generic_Write(unsigned int Cmd,
                                    unsigned char *pValue,
                                    unsigned char *pMask,
                                    unsigned int  Size);

CSAPI_RESULT CSSEC_GetSoftwareVersion(unsigned char * pSoftwareVersion, int Size);

CSAPI_RESULT CSSEC_GetChipRevision(unsigned char * pChipVersion, int Size);

CSSEC_ERROR_CODE CSSEC_Error(CSSEC_CHL_HANDLE Handle);

CSSEC_CHL_HANDLE CSSEC_CHL_Open(CSSEC_CHL_INDEX ChannelID);

CSAPI_RESULT CSSEC_CHL_Close(CSSEC_CHL_HANDLE Handle);


CSAPI_RESULT CSSEC_CHL_WriteData(CSSEC_CHL_HANDLE Handle,
                                 unsigned char *pSrc,
                                 unsigned int *pSize);

CSAPI_RESULT CSSEC_CHL_ReadData(CSSEC_CHL_HANDLE Handle,
                                unsigned char *pDst,
                                unsigned int *pSize);

CSAPI_RESULT CSSEC_CHL_CheckReadDataSize(CSSEC_CHL_HANDLE Handle, unsigned int *pSize);

CSAPI_RESULT CSSEC_CHL_CheckWriteDataSize(CSSEC_CHL_HANDLE Handle, unsigned int *pSize);

CSAPI_RESULT CSSEC_CHL_FlushData(CSSEC_CHL_HANDLE Handle);

CSAPI_RESULT CSSEC_CHL_Start(CSSEC_CHL_HANDLE Handle);

CSAPI_RESULT CSSEC_CHL_Stop(CSSEC_CHL_HANDLE	Handle);

CSAPI_RESULT CSSEC_CHL_SetHostKey(CSSEC_CHL_HANDLE	Handle,
                                  CSSEC_MODE_IDX Mode,
                                  const unsigned char *pHostKey,
                                  int	KeySize,
                                  int KeyPath);


CSAPI_RESULT CSSEC_CHL_SetContentKey(CSSEC_CHL_HANDLE	Handle,
                                     CSSEC_MODE_IDX Mode,
                                     CSSEC_ProtectedKey *pProtectingKey,
                                     const unsigned char *pContentKey,
                                     int KeySize,
                                     int KeyPath);

CSAPI_RESULT CSSEC_CHL_SetCryptoMode(CSSEC_CHL_HANDLE	Handle,
                                     CSSEC_MODE_IDX Mode,
                                     const unsigned char *pInitVector,
                                     int VectorSize);


CSAPI_RESULT CSSEC_CHL_GetCryptoIVector(CSSEC_CHL_HANDLE Handle,
                                        unsigned char *pInitVector,
                                        int VectorSize);

CSAPI_RESULT CSSEC_CHL_CheckCryptoDone(CSSEC_CHL_HANDLE Handle, CSSEC_STATUS *pStatus);

CSAPI_RESULT CSSEC_CHL_InputDataUnderflow_Notify(CSSEC_CHL_HANDLE Handle,
                                                 unsigned int UnderflowThreshold,
                                                 CSSEC_NotifyCallback CallbackFunc,
                                                 void *pContent,
                                                 int NotifyEnable);

CSAPI_RESULT CSSEC_CHL_OutputDataOverflow_Notify(CSSEC_CHL_HANDLE Handle,
                                                 unsigned int OverflowThreshold,
                                                 CSSEC_NotifyCallback CallbackFunc,
                                                 void *pContent,
                                                 int  NotifyEnable);

CSAPI_RESULT CSSEC_CHL_CryptoFinish_Notify(CSSEC_CHL_HANDLE Handle,
                                           CSSEC_NotifyCallback CallbackFunc,
                                           void *pContent,
                                           int NotifyEnable);

CSAPI_RESULT CSSEC_CHL_GetInputWritePointer(CSSEC_CHL_HANDLE Handle,
                                            unsigned char **ppInputWriterPointer,
                                            int *pMaxAllowedSize);

CSAPI_RESULT CSSEC_CHL_UpdateInputWritePointer(CSSEC_CHL_HANDLE Handle, int SizeInByte);

CSAPI_RESULT CSSEC_CHL_GetOutputReadPointer(CSSEC_CHL_HANDLE Handle,
                                            unsigned char **ppOutputReadPointer,
                                            int *pMaxAllowedSize);

CSAPI_RESULT CSSEC_CHL_UpdateOutputReadPointer(CSSEC_CHL_HANDLE Handle, int SizeInByte);

CSAPI_RESULT CSSEC_OTP_Read(unsigned int Cmd, unsigned int *pValue);



#ifdef __cplusplus
}
#endif

#endif  // __CSAPI_CSSEC_H__


