/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Cavium, Inc.
 Cavium, Inc. reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium, Inc.

 ALL RIGHTS RESERVED.

 Copyright 2011-2012, Cavium, Inc.
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */

/**
 * @file cstuner.h
 * @ingroup csapi
 * Low Level Tuner API
 */

#ifndef	_CSTUNER_H_
#define	_CSTUNER_H_

#ifdef __cplusplus
extern "C" {
#endif


//DVBT_P10046
//DVBC_P10023
//DVBT_P10048
//DMBT_LGS8913
//DVBT_LG

#define  DVBC_P10023

#define	TUNER_FD					0x0

typedef enum
{
	TUNER_ID_0 = 0,
	TUNER_ANALOG_PALDK =1,
	TUNER_ANALOG_NTSC   = 4,
	TUNER_DVBC_6M = 26,
	TUNER_DVBC_8M = 28
}TUNER_ID_E;



typedef enum TUNER_BANDWIDTH {
	BANDWIDTH_8_MHZ,
	BANDWIDTH_7_MHZ,
	BANDWIDTH_6_MHZ,
	BANDWIDTH_AUTO
} TUNER_BANDWIDTH_E;

typedef enum{
	TUNER_STATUS_PLL_LOCKED = 0,
	TUNER_STATUS_FE_LOCKED,
	TUNER_STATUS_UNLOCKED,
	TUNER_STATUS_SCANNING,
	TUNER_STATUS_NOT_FOUND
}TUNER_STATUS_E;

typedef enum{
	INVERSION_NORMAL = 0,
	INVERSION_INVERTED,
	INVERSION_AUTO
}TUNER_SPECTRAL_INVERSION_E;

typedef enum{
	QAM_4 = 0,
	QAM_16 = 1,
	QAM_32 = 2,
	QAM_64 = 3,
	QAM_128 = 4,
	QAM_256 = 5,
	QAM_AUTO = 6
}TUNER_QAM_MODULATION_E;

//struct define
typedef struct{
	unsigned int	symbol_rate;
	TUNER_QAM_MODULATION_E modulation;
}TUNER_QAM_PARAMS_S;

typedef struct{
	unsigned int	frequency;
	TUNER_QAM_PARAMS_S	qam_params;
	TUNER_SPECTRAL_INVERSION_E	inversion;
	unsigned int	bw;
}TUNER_PARAMS_S;

typedef struct{
	unsigned char	name[128];
	unsigned int	frequency_min;
	unsigned int	frequency_max;
	unsigned int	frequency_stepsize;
	unsigned int	frequency_tolerance;
	unsigned int	symbol_rate_max;
	unsigned int	symbol_rate_min;
	unsigned int	symbol_rate_tolerance;
	unsigned int	notifier_delay;
}TUNER_INFO_S;

typedef struct{
      unsigned int nfft;  /* can be AUTO */
      unsigned int guard; /* can be AUTO */
      unsigned int constellation;
      unsigned int code_rate;
}TUNER_STATUS_BA;


int	cs_tuner_init(void);
int	cs_tuner_open(TUNER_ID_E tuner_id);
int	cs_tuner_close(int fd);
int	cs_tuner_set_params(int fd, TUNER_PARAMS_S *params);
int	cs_tuner_get_params(int fd, TUNER_PARAMS_S *params);
int	cs_tuner_get_info(int fd, TUNER_INFO_S *info);
int	cs_tuner_read_ber(int fd, unsigned int *ber);
int	cs_tuner_read_snr(int fd, unsigned int qam,unsigned int *snr);
int	cs_tuner_read_status(int fd, TUNER_STATUS_E *status);
#ifdef TUNER_TYPE_ISDB
int   cs_tuner_read_signal_strength(int fd, unsigned int *signal_strength,TUNER_STATUS_BA *par_status);
#else
int   cs_tuner_read_signal_strength(int fd, unsigned int *signal_strength);
#endif
int 	cs_tuner_manage(void);

#ifdef __cplusplus
}
#endif

#endif	/*	_CSTUNER_LG_H_	*/
