/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Cavium, Inc.
 Cavium, Inc. reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium, Inc.

 ALL RIGHTS RESERVED.

 Copyright 2011-2012, Cavium, Inc.
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */

/**
 * @file cstvout.h
 * @ingroup csapi
 * Low Level Display Output API (including HDMI and CEC)
 */

#ifndef __CSAPI_TVOUT_H__
#define __CSAPI_TVOUT_H__

#include "global.h"
#include "si_cpCEC_enums.h"

#ifdef __cplusplus
extern "C" {
#endif

//#define DEBUG_PRINTF(fmt,...)   do{ printf(fmt,##__VA_ARGS__);} while(0)
#define DEBUG_PRINTF(fmt,...)   do{ ;} while(0)

typedef void *CSTVOUT_HANDLE;

/*!
*@brief Define the output channel.
*
*@see CSTVOUT_Open( )
*/
typedef enum
{
	TVOUT_CHANNEL0 = 0,///<Output channel 0
	TVOUT_CHANNEL1 = 1///<Output channel 1
}CSTVOUT_CHANNEL_ID;

/*!
*@brief Disables CEC support when ORed with channel ID passed to CSTVOUT_Open()
*
*@see CSTVOUT_Open( )
*/
#define CSTVOUT_TVOUTOPEN_NO_CEC_SUPPORT    (0x80)

/*!
*@brief Define output mode
*
*@see
*/
typedef enum
{
        OUTPUT_MODE_CVBS_SVIDEO = 0,
        OUTPUT_MODE_YPBPR,/* TVE1 not support */
        OUTPUT_MODE_RGB
} CSTVOUT_OUTPUT_MODE;

/*!
*@brief Define the mode of output format timings.
*
*@see CSTVOUT_SetMode( )
*/
typedef enum
{
	TVOUT_MODE_576I = 0,///<Define the timing mode is 576i@50Hz AR 4:3. /* id = 0 */
	TVOUT_MODE_480I,///<Define the timing mode is 480i@60Hz AR 4:3.  /* id = 1 */
	TVOUT_MODE_576P,///<Define the timing mode is 576p@50Hz AR 4:3     /* id = 2 */
	TVOUT_MODE_480P,///<Define the timing mode is 480p@60Hz Ar 4:3     /* id = 3 */
	TVOUT_MODE_720P50,///<Define the timing mode is 720p@50hz.   /* id = 4 */
	TVOUT_MODE_720P60,///<Define the timing mode is 720p@60hz.   /* id = 5 */
	TVOUT_MODE_1080I25,///<Define the timing mode is 1080i@25hz.  /* id = 6 */
	TVOUT_MODE_1080I30,///<Define the timing mode is 1080i@30hz.  /* id = 7 */

	TVOUT_MODE_SECAM,///<Define the timing mode is secam    /* id = 8 */
	TVOUT_MODE_PAL_M,///<Define the timing mode is pal_m    /* id = 9 */
	TVOUT_MODE_PAL_N,///<Define the timing mode is pal_n    /* id = 10 */
	TVOUT_MODE_PAL_CN,///<Define the timing mode is pal_cn   /* id = 11 */

	TVOUT_MODE_1080P24,///<Define the timing mode is 1080P@24Hz( only for HDMI interface)  /* id = 12 */
	TVOUT_MODE_1080P25,///<Define the timing mode is 1080P@25Hz( only for HDMI interface)  /* id = 13 */
	TVOUT_MODE_1080P30,///<Define the timing mode is 1080P@30Hz( only for HDMI interface)  /* id = 14 */
	TVOUT_MODE_1080P50,///<Define the timing mode is 1080P@50Hz( only for HDMI interface)  /* id = 15 */
	TVOUT_MODE_1080P60,///<Define the timing mode is 1080P@60Hz( only for HDMI interface)  /* id = 16 */

	TVOUT_MODE_576I_16TO9,///<Define the timing mode is 576i@50Hz AR 16:9. /* id = 17 */
	TVOUT_MODE_480I5994,///<Define the timing mode is 480i@59.94Hz AR 4:3.  /* id = 18 */
	TVOUT_MODE_480I5994_16TO9,///<Define the timing mode is 480i@59.94Hz AR 16:9.  /* id = 19 */
	TVOUT_MODE_480I60_16TO9,///<Define the timing mode is 480i@60Hz AR 16:9.  /* id = 20 */
	TVOUT_MODE_576P50_16TO9,///<Define the timing mode is 576p@50Hz AR 16:9     /* id = 21 */
	TVOUT_MODE_480P5994,///<Define the timing mode is 480P@59.94Hz AR 4:3( only for HDMI interface)  /* id = 22 */
	TVOUT_MODE_480P5994_16TO9,///<Define the timing mode is 480P@59.94Hz, AR 16:9( only for HDMI interface)  /* id = 23 */
	TVOUT_MODE_480P60_16TO9,///<Define the timing mode is 480P@60Hz AR 16:9( only for HDMI interface)  /* id = 24 */
	TVOUT_MODE_720P5994,///<Define the timing mode is 720P@59.94Hz( only for HDMI interface)  /* id = 25 */
	TVOUT_MODE_1080I5994,///<Define the timing mode is 1080I@59.94Hz( only for HDMI interface)  /* id = 26 */
	TVOUT_MODE_1080P2397,///<Define the timing mode is 1080P@23.97Hz( only for HDMI interface)  /* id = 27 */
	TVOUT_MODE_1080P5994,///<Define the timing mode is 1080P@59.94Hz( only for HDMI interface)  /* id = 28 */

	TVOUT_MODE_720P5994_3DFP,///<Define the timing mode is 720P@59.94Hz 3D FP( only for HDMI interface)  /* id = 29 */
	TVOUT_MODE_720P60_3DFP,///<Define the timing mode is 720P@60Hz 3D FP( only for HDMI interface)  /* id = 30 */
	TVOUT_MODE_720P50_3DFP,///<Define the timing mode is 720P@60Hz 3D FP( only for HDMI interface)  /* id = 31 */
	TVOUT_MODE_1080P2397_3DFP,///<Define the timing mode is 1080P@23.97Hz 3D FP( only for HDMI interface)  /* id = 32 */
	TVOUT_MODE_1080P24_3DFP,///<Define the timing mode is 1080P@24Hz 3D FP( only for HDMI interface)  /* id = 33 */

	TVOUT_MODE_1080I50_3DSS,///<Define the timing mode is 1080I@50Hz 3D SS( only for HDMI interface)  /* id = 34 */
	TVOUT_MODE_1080I5994_3DSS,///<Define the timing mode is 1080I@59.94Hz 3D SS( only for HDMI interface)  /* id = 35 */
	TVOUT_MODE_1080I60_3DSS,///<Define the timing mode is 1080I@60Hz 3D SS( only for HDMI interface)  /* id = 36 */

	TVOUT_MODE_720P50_3DTB,///<Define the timing mode is 720P@50Hz 3D TB( only for HDMI interface)  /* id = 37 */
	TVOUT_MODE_720P5994_3DTB,///<Define the timing mode is 720P@59.94Hz 3D TB( only for HDMI interface)  /* id = 38 */
	TVOUT_MODE_720P60_3DTB,///<Define the timing mode is 720P@60Hz 3D TB( only for HDMI interface)  /* id = 39 */
	TVOUT_MODE_1080P2397_3DTB,///<Define the timing mode is 1080P@23.97Hz 3D TB( only for HDMI interface)  /* id = 40 */
	TVOUT_MODE_1080P24_3DTB,///<Define the timing mode is 1080P@24Hz 3D TB( only for HDMI interface)  /* id = 41 */

	TVOUT_RGB_640X480_60FPS,///<Define the timing mode is 640*480@60fps(only for VGA interface) /* id = 42 */
	TVOUT_RGB_800X600_60FPS,///<Define the timing mode is 800*600@60fps(only for VGA interface) /* id = 43 */
	TVOUT_RGB_1024X768_60FPS,///<Define the timing mode is 1024*768@60fps(only for VGA interface) /* id = 44 */
	TVOUT_RGB_1280X720_60FPS,///<Define the timing mode is 1280*720@60fps(only for VGA interface) /* id = 45 */
	TVOUT_RGB_800X480_60FPS,///<Define the timing mode is 800*480@60fps(only for VGA interface) /* id = 46 */
	TVOUT_RGB_1440X900_60FPS,///<Define the timing mode is 1440*900@60fps(only for VGA interface) /* id = 47 */
	TVOUT_RGB_1280X1024_60FPS,///<Define the timing mode is 1280*1024@60fps(only for VGA interface) /* id = 48 */
	TVOUT_RGB_1360X768_60FPS,///<Define the timing mode is 1366*768@60fps(only for VGA interface) /* id = 49 */
	TVOUT_RGB_1920X1080P30,///<Define the timing mode is 1920*1080P30(only for VGA interface) /* id = 50 */
	TVOUT_RGB_1920X1080P60,///<Define the timing mode is 1920*1080P60(only for VGA interface) /* id = 51 */
	TVOUT_RGB_1920X1080I30,///<Define the timing mode is 1920*1080I30(only for VGA interface) /* id = 52 */
	TVOUT_RGB_1366X768_60FPS,///<Define the timing mode is 1366*768@60fps(only for VGA interface) /* id = 53 */
	TVOUT_RGB_640X480_5994FPS,///<Define the timing mode is 1366*768@60fps(only for VGA interface) /* id = 54 */
	TVOUT_MODE_1600X1200_60FPS,///<Define the timing mode is 1600*1200P@60Hz/* id = 55 */

	TVOUT_MODE_MAX,
} CSTVOUT_MODE;

/*!
*@brief Define the WSS format of the tvout layer.
*/
typedef enum CSTVOUT_WSSSTD_ {
	VBI_WSS_PAL = 0,///<WSS format is PAL
	VBI_WSS_NTSC,///<WSS format is NTSC
	VBI_WSS_END///<WSS format is end
} CSTVOUT_WSSSTD;

/*!
*@brief Define the WSS format
*/
typedef enum CSTVOUT_WSSTYPE_ {
	TVE_VBI_WSS = 0,///<WSS
	TVE_VBI_CGMS,///<CGMS
	TVE_VBI_END///<type format is end
} CSTVOUT_WSSTYPE;

/*!
*@brief Define enum of WSS aspect ratio.
*/
typedef enum CSTVOUT_WSSARATIO_ {
	WSS_AR_4TO3_FULL = 0,///<WSS_AR_4TO3_FULL
	WSS_AR_14TO9_BOX_CENTER,///<WSS_AR_14TO9_BOX_CENTER
	WSS_AR_14TO9_BOX_TOP,///<WSS_AR_14TO9_BOX_TOP
	WSS_AR_16TO9_BOX_CENTER,///<WSS_AR_16TO9_BOX_CENTER
	WSS_AR_16TO9_BOX_TOP,///<WSS_AR_16TO9_BOX_TOP
	WSS_AR_16TO9P_BOX_CENTER,///<WSS_AR_16TO9P_BOX_CENTER
	WSS_AR_14TO9_FULL,///<WSS_AR_14TO9_FULL
	WSS_AR_16TO9_FULL,///<WSS_AR_16TO9_FULL
	WSS_AR_END///<MAX number
} CSTVOUT_WSSARATIO;

/*!
*@brief Define WSS information
*/
typedef struct CSTVOUT_WSSINFO_ {
	CSTVOUT_WSSTYPE WssType;///<wss type
	CSTVOUT_WSSARATIO	ARatio;///<wss aspect ratio
} CSTVOUT_WSSINFO;

/*!
*@brief Define the txt standard.
*/
typedef enum  {
	VBI_TXT_PALB = 0,///<Txt mode is PAL-B
	VBI_TXT_NTSCB,///<Txt mode is PAL-B
	VBI_TXT_END///<end
}CSVOUT_TxtStandard_t;


#define TTX_LINESIZE          45
#define TTX_MAXLINES          32

/*!
*@brief Define the TXT page information.
*/
typedef struct
{
	unsigned int ValidLines;///<Valid lines of TXT page                      /* bit-field lines  0..31 */
	unsigned char  Lines[TTX_MAXLINES][TTX_LINESIZE];///<Lines arry
} CSVOUT_TxtPage_t;

/*!
*@brief Define compont output type
*/
typedef enum
{
	CSVOUT_COMP_YUV = 0,///< Output YUV data format
	CSVOUT_COMP_RGB///< Output RGB data format
}CSVOUT_CompChannType_t;

/* from background to foreground */

/*!
*@brief Define the order of gfx, video0 and video1 layer.
*
*The gfx layer includes gfx0 and gfx1 and from back to front is gfx0, gfx1.and one video layer.
*@see CSTVOUT_SetZorder
*/
//typedef enum
//{
//	DF_ZORDER_V_G0_G1 = 0x2100,///<Back fo front: video,gfx0,gfx1
//	DF_ZORDER_V_G1_G0 = 0x1200,///<Back fo front: video,gfx1,gfx0
//	DF_ZORDER_G0_V_G1 = 0x2001,///<Back fo front: gfx0,video,gfx1
//	DF_ZORDER_G0_G1_V = 0x1002,///<Back fo front: gfx0,gfx1,video
//	DF_ZORDER_G1_V_G0 = 0x0201,///<Back fo front: gfx1,video,gfx0
//	DF_ZORDER_G1_G0_V = 0x0102///<Back fo front: gfx1,gfx0,video
//} CSTVOUT_DF_ZORDER;
//
typedef enum
{
    DF_ZORDER_G1_G2_V1_V2 = 0X2301, //<front to Back: gfx0, gfx1, video0, video1
    DF_ZORDER_V1_V2_G1_G2 = 0X123,  //<front to Back: video0, video1, gfx0, gfx1
    DF_ZORDER_G2_G1_V2_V1 = 0X3210, //<front to Back: gfx1, gfx0, video1, video0
    DF_ZORDER_V2_V1_G2_G1 = 0X1032, //<front to Back: video1, video0, gfx1, gfx0
    DF_ZORDER_G1_V1_G2_V2 = 0X1302, //<front to Back: gfx0, video0, gfx1, video1
    DF_ZORDER_V2_G2_V1_G1 = 0X2031  //<front to Back: video1, gfx1, video0, gfx0
} CSTVOUT_DF_ZORDER;

/*!
*@brief Define the error code for the CSTVOUT API.
*
*
*/
typedef enum
{
    TVOUT_NO_ERROR = 0,///<No error.
    TVOUT_ERROR_OPEN_FAILED,///<Open filed.          /* open filed                 */
    TVOUT_ERROR_IOCTL_FAILED,///<Ioctl filed.         /* ioctl filed                */
    TVOUT_ERROR_INVALID_PARAMETERS,///<Bad parameter passed.   /* Bad parameter passed       */
    TVOUT_ERROR_UNKNOWN_DEVICE,///<Unknown device name.       /* Unknown device name        */
    TVOUT_ERROR_DEVICE_BUSY,///<Device is currently busy.          /* Device is currently busy   */
    TVOUT_ERROR_INVALID_HANDLE,///<Handle is not valid.       /* Handle is not valid        */
    TVOUT_ERROR_ALREADY_INITIALIZED,///<Device is already initialized.  /* Device already initialized */
    TVOUT_ERROR_NOT_INITIALIZED,///<Device is not initialized.   /* bDevice not initialized     */
    TVOUT_ERROR_REGISTER_CALLBACK,///< TVOUT_ERROR_REGISTER_CALLBACK
    TVOUT_ERROR_OPEN_INPUT_FAILED,///< TVOUT_ERROR_OPEN_INPUT_FAILED
    TVOUT_ERROR_FB_ACCESS///< TVOUT_ERROR_FB_ACCESS
} CSTVOUT_ErrCode;

/*!
*@brief Define the opened envents, if set CSTVOUT_HDMI_PLUG_IN notify function will notify when the HDMI cable plugs in, while CSTVOUT_HDMI_PLUG_OUT is set notify function will notify when the HDMI cable plugs out, but both be set the notify function will notify when the HDMI cable plugs in and out.
*
*
*/
typedef enum
{
	CSTVOUT_HDMI_PLUG_IN = 1,///<Test HDMI cable plugs in
	CSTVOUT_HDMI_PLUG_OUT = 2///<Test HDMI cable plugs out
}CSTVOUT_HDMI_HPD_NOTIFYEVENT;

/*!
*@brief Define open or close HPD event notify function.
*
*/
typedef enum
{
	CSTVOUT_HDMI_HPD_ON = 0,///<Open HPD event notify function
	CSTVOUT_HDMI_HPD_OFF = 1///<Close HPD event notify function
}CSTVOUT_HDMIPLUG_REG;

typedef  void (*hpd_callback_function)( CSTVOUT_HDMI_HPD_NOTIFYEVENT *);

/*!
*@brief Open a connection to a TVOUT channel.
*
*@param None
*@return CSTVOUT_HANDLE	A valid handle on open success.
*@note None
*@see CSTVOUT_Close
*
*/
CSTVOUT_HANDLE CSTVOUT_Open(CSTVOUT_CHANNEL_ID id);

/*!
*@brief Close the connection to a TVOUT channel.
*
*@param handle	Handle of TVOUT channel.	In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@note None
*@see None
*
*/
CSAPI_RESULT   CSTVOUT_Close(CSTVOUT_HANDLE handle);

/*!
*@brief Enable the TVOUT device.
*
*@param handle	Handle of TVOUT driver.	In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@note None
*@see None
*
*/
CSAPI_RESULT   CSTVOUT_Enable(CSTVOUT_HANDLE handle);

/*!
*@brief Get Output mode
*
*@param handle	Handle of TVOUT driver.	In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@note None
*@see None
*
*/

CSAPI_RESULT   CSTVOUT_GetOutputMode(CSTVOUT_HANDLE handle,unsigned int * OutputMode);
/*!
*@brief Enable HPD.
*
*@param handle	Handle of TVOUT driver.	In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@note None
*@see None
*
*/
CSAPI_RESULT   CSTVOUT_EnableCheckHpd(CSTVOUT_HANDLE handle);

/*!
*@brief Disable the TVOUT device.
*
*@param handle	Handle of TVOUT driver.	In/Out: In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@note None
*@see None
*
*/
CSAPI_RESULT   CSTVOUT_Disable(CSTVOUT_HANDLE handle);

/*!
*@brief Set the mode of video output format.
*
*@param handle	Handle of TVOUT driver.	In/Out: In
*@param vid_mod	Define the new mode of video output format.	In/Out: In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@note None
*@see CSTVOUT_MODE
*
*/
CSAPI_RESULT   CSTVOUT_SetMode(CSTVOUT_HANDLE handle, CSTVOUT_MODE vid_mod);

/*!
*@brief Get the TV output mode.
*
*@param handle	Handle of TVOUT driver.	In/Out: In
*@param vid_mod	Point to the mode of video output format.	In/Out: Out
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@note None
*@see None
*
*/
CSAPI_RESULT   CSTVOUT_GetMode(CSTVOUT_HANDLE handle, CSTVOUT_MODE * vid_mod);

/*!
*@brief Get the error string pointer of the current operation result.
*
*@param handle	An opened TVOUT handle.	In/Out: In
*@param zorder	the order of gfx and video layer.	In/Out: In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@note None
*@see CSTVOUT_DF_ZORDER
*
*/
CSAPI_RESULT   CSTVOUT_SetZorder(CSTVOUT_HANDLE handle, CSTVOUT_DF_ZORDER zorder);

/*!
*@brief Enable the HD2SD function.
*
*HD2SD function means capture the HD output (HDMI or YpbPr) to display in SD device(CVBS). This function must be call before CSVID_SetMode().
*@param handle	An opened TVOUT handle.  In/Out: In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@note
*@see CSTVOUT_DisableHD2SD.
*
*/
CSAPI_RESULT CSTVOUT_EnableHD2SD(CSTVOUT_HANDLE handle);

/*!
*@brief Disable the HD2SD function.
*
*This function must be call before CSVID_SetMode().
*@param handle	An opened TVOUT handle.	In/Out: In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@note
*@see CSTVOUT_EnableHD2SD.
*
*/
CSAPI_RESULT CSTVOUT_DisableHD2SD(CSTVOUT_HANDLE handle);

/*!
*@brief Set the background color.
*
*The background is in behind of video and gfx layer, so if you want to see the effect after you set the color, you should disable video and gfx layer. Guide to disable video and gfx layer is in CSVID and CSOSD module.
*@param handle	An opened TVOUT handle.	In/Out: In
*@param r	Red value.	In/Out: In
*@param g	Greed value.	In/Out: In
*@param b	Blue value.	In/Out: In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@note None
*@see None
*
*/
CSAPI_RESULT   CSTVOUT_SetBkColor(CSTVOUT_HANDLE handle, unsigned char r, unsigned char g, unsigned char b);

/*!
*@brief Set the brightness levels of the TV output.
*
*The function is used to adjust the luminance intensity of the TV output image.\n
*This value is unsigned and scaled from 0 to 254. Default value is 128.
*@param handle	Handle of TVOUT driver.	In/Out: In
*@param brightness	The level of the TV output brightness.	In/Out: In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@note None
*@see None
*
*/
CSAPI_RESULT   CSTVOUT_SetBrightness(CSTVOUT_HANDLE handle, unsigned int brightness);

/*!
*@brief Get the brightness levels of the TV output.
*
*@param handle	Handle of TVOUT driver.	                    In/Out: In
*@param brightness	Stores the level of video brightness.	In/Out: Out
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@note None
*@see None
*
*/
CSAPI_RESULT   CSTVOUT_GetBrightness(CSTVOUT_HANDLE handle, unsigned int * brightness);

/*!
*@brief Set the relative difference between higher and lower intensity luminance values of the TV output image.
*
*This value is signed and scaled from 0 to 63. Default value is 32.
*@param handle	Handle of TVOUT driver.	In/Out: In
*@param contrast	Set the new levels of contrast.	In/Out: In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@note None
*@see None
*
*/
CSAPI_RESULT   CSTVOUT_SetContrast(CSTVOUT_HANDLE handle, unsigned int constrast);

/*!
*@brief Get the relative difference between higher and lower intensity luminance values of the TV output image.
*
*@param handle	Handle of TVOUT driver.	In/Out: In
*@param contrast	Set the new levels of contrast.	In/Out: Out
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@note None
*@see None
*
*/
CSAPI_RESULT   CSTVOUT_GetContrast(CSTVOUT_HANDLE handle, unsigned int * constrast);

/*!
*@brief Set the color intensity of the video image which is TV output.
*
*This value is unsigned and scaled from 0 to 254. Default value is 64.
*@param handle	Handle of TVOUT driver.  In/Out: In
*@param saturation	Set the new levels of saturation.	In/Out: In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@note None
*@see None
*
*/
CSAPI_RESULT   CSTVOUT_SetSaturation(CSTVOUT_HANDLE handle, unsigned int saturation);

/*!
*@brief Get the color intensity of the video image which is TV output.
*
*@param handle	Handle of TVOUT driver.	In/Out: In
*@param saturation	Set the new levels of saturation.	In/Out: Out
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@note None
*@see None
*
*/
CSAPI_RESULT   CSTVOUT_GetSaturation(CSTVOUT_HANDLE handle, unsigned int * saturation);

/*!
*@brief Set tvout hue function.
*
*The HueLevel is from 0 to 100%, and CSTVOUT_SetHue is get tvout hue level
*@param handle	An opened TVOUT handle.	In/Out: In
*@param HueLevel	The level of the hue. 	In/Out: In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@note None
*@see None
*
*/
CSAPI_RESULT CSTVOUT_SetHue(CSTVOUT_HANDLE handle, unsigned int HueLevel);

/*!
*@brief Get the hue of the video image which is TV output.
*@param handle	An opened TVOUT handle.	In/Out: In
*@param HueLevel	The level of the hue.	In/Out: Out
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@note None
*@see None
*
*/
CSAPI_RESULT CSTVOUT_GetHue(CSTVOUT_HANDLE handle, unsigned int *HueLevel);
/*!
*@brief Set tvout sharpness function.
*
*@param handle	An opened TVOUT handle.	In/Out: In
*@param enable	0 disable, 1 enable. 	In/Out: In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@note None
*@see None
*
*/
CSAPI_RESULT   CSTVOUT_SetSharpness(CSTVOUT_HANDLE handle, unsigned int enable);
/*!
*@brief Get the sharpness of the video image which is TV output.
*@param handle	An opened TVOUT handle.	In/Out: In
*@param enable	0 disable, 1 enable.	In/Out: Out
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@note None
*@see None
*
*/
CSAPI_RESULT   CSTVOUT_GetSharpness(CSTVOUT_HANDLE handle, unsigned int *enable);
/*!
*@brief Get the error code of the current operation result.
*
*@param handle	An opened TVOUT handle.	In/Out: In
*@return CSTVOUT_ErrCode	The error code of the current operation.
*@note None
*@see None
*
*/
CSTVOUT_ErrCode CSTVOUT_GetErrCode(CSTVOUT_HANDLE handle);

/*!
*@brief Get the error string pointer of the current operation result.
*
*@param handle	An opened TVOUT handle.	In/Out: In
*@return char*	The error string pointer of the current operation result.
*@note None
*@see None
*
*/
char *CSTVOUT_GetErrString(CSTVOUT_HANDLE handle);

/*!
*@brief Set Comp Channel.
*
*@param handle	Handle of TVOUT driver.	In/Out: In
*@param CompChan	Set comp channel.	In/Out: In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@note None
*@see CSTVOUT_CompChannType_t
*
*/
CSAPI_RESULT CSTVOUT_SetCompChannel(CSTVOUT_HANDLE handle, CSVOUT_CompChannType_t CompChan);

/*!
*@brief WSS control function.
*
*@param handle	An opened TVOUT handle.	In/Out: In
*@param Enable	Enable or disable WSS.	In/Out: In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@note None
*@see None
*
*/
CSAPI_RESULT CSTVOUT_WSS_Ctrl(CSTVOUT_HANDLE handle, char Enable);
/*!
*@brief Setting WSS's format function.
*
*@param handle	An opened TVOUT handle.	In/Out: In
*@param WssStd	See CSTVOUT_WSSSTD struct.	In/Out: In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@note None
*@see None
*
*/
CSAPI_RESULT CSTVOUT_WSS_SetFormat(CSTVOUT_HANDLE handle, CSTVOUT_WSSSTD WssStd);

/*!
*@brief Setting WSS's information function.
*
*@param handle	An opened TVOUT handle.	In/Out: In
*@param WssInfo	See CSTVOUT_WSSINFO struct.	In/Out: Out
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@note None
*@see None
*
*/
CSAPI_RESULT CSTVOUT_WSS_SetInfo(CSTVOUT_HANDLE handle, CSTVOUT_WSSINFO *WssInfo);

/*!
*@brief Setting TXT contorl function.
*
*@param handle	An opened TVOUT handle.	In/Out: In
*@param Enable	Enable or disable TXT.	In/Out: Out
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@note None
*@see None
*
*/
CSAPI_RESULT CSTVOUT_TXT_Ctrl(CSTVOUT_HANDLE handle, char Enable);

/*!
*@brief Setting TXT's format function.
*
*@param handle	An opened TVOUT handle.	In/Out: In
*@param TxtStd	See CSVOUT_TxtStandard_t struct.	In/Out: In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@note None
*@see None
*
*/
CSAPI_RESULT CSTVOUT_TXT_SetFormat(CSTVOUT_HANDLE handle, CSVOUT_TxtStandard_t TxtStd);

/*!
*@brief Setting TXT's information function.
*
*@param handle	An opened TVOUT handle.	In/Out: In
*@param TxtPage	See CSVOUT_TxtPage_t struct.	In/Out: Out
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@note None
*@see None
*
*/
CSAPI_RESULT CSTVOUT_TXT_SetInfo(CSTVOUT_HANDLE handle, CSVOUT_TxtPage_t *TxtPage);

/*!
*@brief Enable the HD2SD function.
*
*HD2SD function means capture the HD output (HDMI or YpbPr) to display in SD device(CVBS). This function must be call before CSVID_SetMode().
*@param handle	An opened TVOUT handle.  In/Out: In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@note
*@see CSTVOUT_DisableHD2SD.
*
*/
CSAPI_RESULT CSTVOUT_EnableHD2SD(CSTVOUT_HANDLE handle);

/*!
*@brief Disable the HD2SD function.
*
*This function must be call before CSVID_SetMode().
*@param handle	An opened TVOUT handle.	In/Out: In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@note
*@see CSTVOUT_EnableHD2SD.
*
*/
CSAPI_RESULT CSTVOUT_DisableHD2SD(CSTVOUT_HANDLE handle);

/*!
*@brief Config CVBS overscan.
*
*@param handle		An opened TVOUT handle.	In/Out: In
*@param x_start			In/Out: In
*@param x_end		An opened TVOUT handle.	In/Out: In
*@param y_start		An opened TVOUT handle.	In/Out: In
*@param y_end		An opened TVOUT handle.	In/Out: In
*@param enable_flag	Indicate enable or disable overscan function.	In/Out: In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@note: 1.This handle must be TVOUT_CHANNEL1 not TVOUT_CHANNEL0.\n
*2. if enable_flag is 1, all coordinate parameters will be used. if them not invalid(such as it beyond current CVBS TV mode.), CVBS overscan will be disabled.
*3. if enable_flag is 0, it just disable CVBS overscan function, all coordinate parameters will not be used(means these parameters will be ignored. Even you set random value is ok.).\n
*@see CSTVOUT_EnableHD2SD.
*
*/
CSAPI_RESULT CSTVOUT_SetOverScan(CSTVOUT_HANDLE handle, unsigned int x_start, unsigned int x_end, unsigned int y_start, unsigned int y_end, int enable_flag);

/*
Below define are for NTSC
*/
#define CGMS_4_3_NORMAL 0x0
#define CGMS_16_9_NORMAL 0x1
#define CGMS_4_3_LETTERBOX 0x2
#define CGMS_COPY_FREE 0x00
#define CGMS_COPY_NOMORE 0x20
#define CGMS_COPY_ONCE 0x40
#define CGMS_COPY_NONE 0xC0
#define CGMS_APS_OFF 0x00
#define CGMS_APS_SPLIT_OFF 0x200
#define CGMS_APS_SPLIT_2LINE 0x100
#define CGMS_APS_SPLIT_4LINE 0x300
#define CGMS_ANALOG_SOURCE 0x400

/*
Below define are for PAL
*/
#define WSS_4_3_NORMAL 0x08
#define WSS_14_9_LETTERBOX_CENTRE 0x01
#define WSS_14_9_LETTERBOX_TOP 0x02
#define WSS_16_9_LETTERBOX_CENTRE 0x0b
#define WSS_16_9_LETTERBOX_TOP 0x4
#define WSS_G16_9_LETTERBOX_CENTRE 0x0d
#define WSS_14_9_NORMAL 0x0e
#define WSS_16_9_NORMAL 0x07
#define WSS_COPYRIGHT_FREE 0x00
#define WSS_COPYRIGHT_ASSERTED 0x1000
#define WSS_COPY_YES 0x00
#define WSS_COPY_NO 0x2000

/*!
*@brief Enable CGMS function, And set the correct configuration.
*
*@param handle		An opened TVOUT handle.	In/Out: In
*@param data_pal		CGMS options for PAL.		In/Out: In
*@param data_ntsc	CGMS options for NTSC.	In/Out: In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@note If you want to enable CGMS, you need to call this after CSTVOUT_SetMode. Please note, it only effect CVBS.
*@see None
*
*/
CSAPI_RESULT CSTVOUT_EnableCGMS(CSTVOUT_HANDLE handle, unsigned int data_pal, unsigned int data_ntsc);

/*!
*@brief Disable CGMS function.
*
*@param handle	An opened TVOUT handle.	In/Out: In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@note None
*@see None
*
*/
CSAPI_RESULT CSTVOUT_DisableCGMS(CSTVOUT_HANDLE handle);

/*CSTVOUT_GetHDMIEdidRawData get hdmi edid raw data*/
/*edid_data_len is returned edid raw data length;CSTVOUT_GetHDMIEdidRawData return edid raw data pointer*/

/*!
*@brief Return pointer of the edid raw data starting address,and the length od the edid raw data.
*
*@param *edid_data_len	Return edid raw data length	Out
*@return const unsigned char *	Pointer points to the start address of the raw data array.
*@note None
*@see None
*
*/
const unsigned char * CSTVOUT_GetHDMIEdidRawData(unsigned int *edid_data_len);

/* CSTVOUT_HDMIPLUG_SetNotify when hdmi cable plug in or plug out call_back_function will be called */
/*plug_event = CSTVOUT_HDMI_PLUG_IN check plug in event;plug_event = CSTVOUT_HDMI_PLUG_OUT check plug out event;plug_event = CSTVOUT_HDMI_PLUG_IN|CSTVOUT_HDMI_PLUG_OUT check plug in and out*/
/* Returns CSAPI_SUCCEEDED or CSAPI_FAILED */

/*!
*@brief Close the connection to a TVOUT channel.
*
*@param handle	Handle of TVOUT channel.	In
*@param callback	The function be called when the event is satisfied.	In
*@param plug_event	Open or close this notify function.	In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@note None
*@see CSTVOUT_HDMIPLUG_REG
*
*/
CSAPI_RESULT CSTVOUT_HDMIPLUG_SetNotify(CSTVOUT_HANDLE handle, hpd_callback_function callback,
		CSTVOUT_HDMIPLUG_REG plug_event);


/*!
*@brief Define HDMI Aspect Ratios
*
*/
typedef enum
{
	CSTVOUT_HDMI_ASPECT_UNSPECIFIED = 0,///< unspecified, use default
	CSTVOUT_HDMI_ASPECT_4x3 = 1,///< 4x3 aspect ratio
	CSTVOUT_HDMI_ASPECT_16x9 = 2///< 16x9 aspect ratio
}CSTVOUT_HDMI_ASPECT_RATIO;

// HDMI CEC
#define CSTVOUT_HDMI_CEC_MAX_ARGS   16
/*!
*@brief Structure of HDMI CEC send message.
*
*
*/
typedef struct {
    CEC_LOG_ADDR_t dst_la;///<Structure of HDMI CEC send message.
    CEC_OPCODE_t opcode;///<message number defined by HDMI specification
    unsigned char arg_count;///<number of arguments
    unsigned char args[CSTVOUT_HDMI_CEC_MAX_ARGS];///<Arguments of sending message
}CSTVOUT_HDMI_CEC_SEND_MESSAGE;

/*!
*@brief Structure of HDMI CEC receive message.
*
*
*/
typedef struct {
    CEC_LOG_ADDR_t src_la;///<Logical address of the source
    CEC_LOG_ADDR_t dst_la;
    unsigned char is_broadcast;///<is the message a broadcast message or a direct addressed message
    CEC_OPCODE_t opcode;///<is the message a broadcast message or a direct addressed message
    unsigned char arg_count;///<number of arguments
    unsigned char args[CSTVOUT_HDMI_CEC_MAX_ARGS];///<Arguments of sending message
}CSTVOUT_HDMI_CEC_RECEIVE_MESSAGE;

/*!
*@brief ACK status of sent HDMI CEC message.
*
*
*/
typedef enum {
    HDMI_CEC_TX_ACKED,///<Message was ACKed.
    HDMI_CEC_TX_FAILED///<Failed to send message.
}CSTVOUT_HDMI_CEC_TX_ACK;

typedef void (*cec_callback_func)(CSTVOUT_HDMI_CEC_RECEIVE_MESSAGE *);
typedef void (*cec_ack_callback)(CSTVOUT_HDMI_CEC_SEND_MESSAGE *msg, CSTVOUT_HDMI_CEC_TX_ACK ack);

CSAPI_RESULT CSTVOUT_HDMI_CEC_SendMessage(CSTVOUT_HANDLE handle, CSTVOUT_HDMI_CEC_SEND_MESSAGE *msg,
                                          int wait_ack, unsigned int timeout_ms, cec_ack_callback cb);
/*!
*@brief Feature of HDMI CEC <One Touch Play>, will send message <Image View On> and <Active Source>.
*       This will power up the TV and set the active souce to the HDMI port which sending these messages.
*
*@param handle	An opened TVOUT handle.	In/Out: In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@note <One Touch Play> is a default feature on CNC1900, if CEC was enabled,
	   <Image View On> and <Active Source> will be sent automatically when HDMI
	   cable plugged in. Keep this function for compatibility.
*@see None
*
*/
CSAPI_RESULT CSTVOUT_HDMI_CEC_OneTouchPlay(CSTVOUT_HANDLE handle);

/*!
*@brief Register the notification of receive a HDMI CEC message.
*
*@param handle	An opened TVOUT handle.	In/Out: In
*@param callback	Registed callback function, when receive a HDMI CEC message, this function will be called, and the received message will be the parameter of the callback function.	In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@note None
*@see None
*
*/
CSAPI_RESULT CSTVOUT_HDMI_CEC_SetNotify(CSTVOUT_HANDLE handle, cec_callback_func callback);

typedef enum {
    HDMI_AUDIO_DISABLE = 0,
    HDMI_AUDIO_ENABLE,
}CSTVOUT_HDMI_AUDIO_CTRL;

typedef enum {
    HDMI_HDCP_DISABLE = 0,
    HDMI_HDCP_ENABLE,
}CSTVOUT_HDMI_HDCP_CTRL;

typedef enum
{
	HDMI_AUDIO_CLK_DISP0 =0,
	HDMI_AUDIO_CLK_VPB   =1,
}CSTVOUT_HDMI_IN_BY;

typedef enum
{
	HDMI_AUDIO_IN_GPIO =0,
	HDMI_AUDIO_IN_VPB  =1,
}CSTVOUT_HDMI_IN_ENA;


/*
*@brief enable HDMI in function.
*
*@param handle	An opened TVOUT handle.          In/Out: In
*@param en HDMI_AUDIO_CLK_DISP0 to enable HDMI in function ,HDMI_AUDIO_CLK_VPB to disable.
*@return CSAPI_SUCCEED	Success\n
		CSAPI_FAILED	Failure
*/

CSAPI_RESULT CSTVOUT_HDMI_IN_BYPASS(CSTVOUT_HANDLE handle, CSTVOUT_HDMI_IN_BY en);
/*
*@brief enable HDMI in function.
*
*@param handle	An opened TVOUT handle.          In/Out: In
*@param en HDMI_AUDIO_OUT_TTL to enable TTL mode ,HDMI_AUDIO_OUT_VIB to enbale vib mode.
*@return CSAPI_SUCCEED	Success\n
				CSAPI_FAILED	Failure
*/
CSAPI_RESULT CSTVOUT_HDMI_IN_ENABLE(CSTVOUT_HANDLE handle, CSTVOUT_HDMI_IN_ENA en);


/*!
*@brief enable or disable HDMI audio input stream
*
*@param handle	An opened TVOUT handle.	In/Out: In
*@param en HDMI_AUDIO_DISABLE to disable or HDMI_AUDIO_ENABLE to enable
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@note None
*@see None
*
*/
CSAPI_RESULT CSTVOUT_HDMI_Audio_Ena(CSTVOUT_HANDLE handle, CSTVOUT_HDMI_AUDIO_CTRL en);

/*!
*@brief enable or disable HDMI HDCP authentication.
*
*@param handle	An opened TVOUT handle.	In/Out: In
*@param en HDMI_HDCP_DISABLE to disable or HDMI_HDCP_ENABLE to enable
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@note On CNC1900, we can disable HDCP even on chip with AKSV
*@see None
*
*/
CSAPI_RESULT CSTVOUT_HDMI_HdcpEna(CSTVOUT_HANDLE handle, CSTVOUT_HDMI_HDCP_CTRL en);

/*!
*@brief Set the HDMI logical address of board.
*
*@param  handle	An opened TVOUT handle.	In
*@param  src_la	source logical address.	In
*@return CSAPI_FAILED set source logical address failed.
*        CSAPI_SUCCEED set source logical address succeed.
*@note   This function was added to set HDMI Tx Logical address for CEC on CNC1800L,
*        but manually set logical address may cause conflict.
*        for CNC1900, HDMI driver will automatically get a proper logical address for PLAYBACK device.
*        Now this function will do nothing, keep it for compatibility
*@see    None
*
*/
CSAPI_RESULT CSTVOUT_HDMI_CEC_SetSrcLA(CSTVOUT_HANDLE handle, CEC_LOG_ADDR_t src_la);

/*!
*@brief Set Capture ID of HDMI CEC message.
*
*@param handle	   An opened TVOUT handle. In/Out: In
*@param capture_id Capture id of HDMI CEC message. It selects the received commands
*                  that will be captured in the receive FIFO and acknowledged.
*                  If the command destination matches one of the bits set in this
*                  parameter or is a Broadcast cycle; it will be captured. In/Out: In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@note   This function was added to make HDMI Tx didn't acknowledge CEC message on CNC1800L,
*        but it will still send acknowledge for broadcast message even all capture ID set to 0.
*        for CNC1900, to avoid HDMI Tx acknowledge any CEC message, just need to set
*        CSTVOUT_TVOUTOPEN_NO_CEC_SUPPORT flag when invoke CSTVOUT_Open, it will close CEC clock.
*        Now this function will do nothing, keep it for compatibility
*@see None
*
*/
CSAPI_RESULT CSTVOUT_HDMI_CEC_SetCaptureID(CSTVOUT_HANDLE handle, unsigned char capture_id[2]);

/*!
*@brief To send a HDMI CEC message.
*
*@param handle	An opened TVOUT handle. In/Out: In
*@param msg	Message to send.	In
*@param wait_ack	Wait the sent message be ACKed or not, if non-zero, function won't return until message be ACKed or time out.	In
*@param timeout_ms	Time out value in millisecond, only valid when wait_ack is non- zero.	In
*@param cb	callback function for ACK, will be called when the message was ACKed or time out. Only valid when wait_ack is non-zero.	In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@note None
*@see None
*
*/

/*!
*@brief Set the IT Content bit in AVI InfoFrame
*
*@param handle	   An opened TVOUT handle. In/Out: In
*@param set        1 for set the IT Content bit, 0 for clear the IT Content bit.
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@note None
*@see None
*
*/
CSAPI_RESULT CSTVOUT_HDMI_SetITC(CSTVOUT_HANDLE handle, int set);

/*!
*@brief 3D format set in vendor specific info frame
*
*
*/
typedef enum {
    HDMI_3D_FRAMEPACKING = 0x00,
    HDMI_3D_FIELDALTERNATIVE,
    HDMI_3D_LINEALTERNATIVE,
    HDMI_3D_SIDEBYSIDEFULL,
    HDMI_3D_LDEPTH,
    HDMI_3D_LDEPTHGRAPHICS,
    HDMI_3D_TOPBOTTOM,
    HDMI_3D_SIDEBYSIDEHALF = 0x08,
    HDMI_3D_NONE = 0xFF
}CSTVOUT_HDMI_3D_FORMAT;

/*
Suspend the HDMI driver thread.
HDMI don't read EDID/CEC and don't detect the HPD. That means all the HDMI callback function can not be called in this process.
*/

/*!
*@brief Suspend the HDMI driver thread.
*
*@param handle	An opened TVOUT handle.	In/Out: In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@note HDMI don't read EDID/CEC and don't detect the HPD. That means all the HDMI callback function can not be called in this process.
*@see CSTVOUT_HDMI_Suspend
*
*/
CSAPI_RESULT CSTVOUT_HDMI_Suspend(CSTVOUT_HANDLE handle);

/*
Resume the HDMI driver thread.
*/

/*!
*@brief Resume the HDMI driver thread
*
*@param handle	An opened TVOUT handle.	In/Out: In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@note None
*@see CSTVOUT_HDMI_Suspend
*
*/
CSAPI_RESULT CSTVOUT_HDMI_Resume(CSTVOUT_HANDLE handle);

typedef enum {
    HDMI_COLOR_RANGE_FULL = 0,
    HDMI_COLOR_RANGE_LIMITE = 1,
}CSTVOUT_HDMI_COLOR_RANGE;

struct HDMI_Audio_Param
{
      unsigned int AudioFsId;
      unsigned int AudioTypeId;
      unsigned int AudioLenId;
      unsigned int AudioChannelId;
      unsigned int AudioSourceFormat;
};

/*!
*@brief to set Audio
*
*@param en pointer to HDMI_Audio_Param
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@note None
*@see None
*
*/
CSAPI_RESULT  HDMISetAudio(int AudioFsId, int AudioTypeId, int AudioLenId, int num_channels, int source_format);

typedef enum
{
	SAMPLE_PCM = 0,///<SPDIF will transfer PCM data which our decoded.
	SAMPLE_COMPRESS = 1,///<SPDIF will transfer source data which bypass by our decoder.
}SAMPLE_TYPE;

typedef enum
{
	AUDIO_I2S   = 0,
	AUDIO_SPDIF = 1,
	AUDIO_DSD   = 2,
	AUDIO_HBR   = 3,
}DATA_FORMAT;

typedef enum
{
	//these values are according to TPI register Descriptions
	SAMPLE_RATE_32K     = 0x03, //32k
	SAMPLE_RATE_44K     = 0x00, //44.1k
	SAMPLE_RATE_48K     = 0x02, //48k
	SAMPLE_RATE_88K     = 0x08, //88.2k
	SAMPLE_RATE_96K     = 0x0A, //96k
	SAMPLE_RATE_176K    = 0x0C, //176.4k
	SAMPLE_RATE_192K    = 0x0E,  //192k
}SAMPLE_RATE;


typedef enum
{
	TWO_CHANNELS    = 0x01,
	FOUR_CHANNELS   = 0x03,
	SIX_CHANNELS    = 0x05,
	EIGHT_CHANNELS  = 0x07,
}CHANNELS;

typedef enum
{
	BIT_WIDTH_24    = 0x0B,
	BIT_WIDTH_23    = 0x09,
	BIT_WIDTH_22    = 0x05,
	BIT_WIDTH_21    = 0x0D,
	BIT_WIDTH_20    = 0x0A,
	BIT_WIDTH_19    = 0x08,
	BIT_WIDTH_18    = 0x04,
	BIT_WIDTH_17    = 0x0C,
	BIT_WIDTH_16    = 0x02,
}BIT_WIDTH;

typedef struct{
    DATA_FORMAT   data_format;
    SAMPLE_RATE   sample_rate;
    CHANNELS      channel_number;
    BIT_WIDTH     bit_width;
    unsigned int  audio_changed;
    SAMPLE_TYPE    sample_type;
    unsigned char ApplicationType;
    unsigned char Copyright;
    unsigned char Emphasis;
    unsigned char ChannelStatusMode;
    unsigned char CategoryCode;
    unsigned char SourceNumber;
    unsigned char CsChannelNumber;
    unsigned char ClockAccuracy;
}AudioDescription;


/*!
*@brief setting audio paramters for HDMI;
*
*@param handle	An opened TVOUT handle.	In/Out: In
*@param opt	    AudioDescription struct point.	In/Out: In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@see None
*@Notes: current default;
*   data_format    = AUDIO_I2S; 
*	sample_rate    = SAMPLE_RATE_44K/SAMPLE_RATE_48K;
*	channel_number = TWO_CHANNELS;
*	bit_width      = BIT_WIDTH_24;
*	audio_changed  = 0;
*	sample_type     = SAMPLE_PCM;
*/

CSAPI_RESULT CSTVOUT_HDMI_SetAudioDescription(CSTVOUT_HANDLE handle, AudioDescription *opt);
/*!
*@brief get audio paramters for HDMI;
*
*@param handle	An opened TVOUT handle.	In/Out: In
*@param opt	    AudioDescription struct point.	In/Out: In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@see None
*/
CSAPI_RESULT CSTVOUT_HDMI_GetAudioDescription(CSTVOUT_HANDLE handle, AudioDescription *opt);

#define HDCP1X_KSV_SIZE 5
#define MAX_HDCP1X_KSV 31

typedef struct {
    unsigned char bksv[HDCP1X_KSV_SIZE];
    unsigned char rptr_dev_cnt;
    unsigned char rptr_dev_exd;
    unsigned char rptr_casc_exd;
    unsigned char rptr_depth;
    unsigned char ksv_list[HDCP1X_KSV_SIZE * (MAX_HDCP1X_KSV+1)];
} downstream_info_ksv_list_t;

/*!
*@brief to get down stream Info on the HDMI port
*
*@param en pointer to downStreamInfo
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@note None
*@see None
*
*/
CSAPI_RESULT CSTVOUT_getHDCP1xDownstreamInfo(void *downStreamInfo);
/*!
*@brief to trigger HDCP 1.x authentication
*
*@param state  For compatibility, keep the interface same, but same state can't work on
*              CNC1900, so for CNC1900, this function just for triggering HDCP 1.x,
*              no matter what parameter passed.
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@note None
*@see None
*
*/
CSAPI_RESULT CSTVOUT_setHDCP1xAuthState(char state);

typedef enum
{
	CSTVOUT_HDCP_AUTH_CHANGE = 0,   ///<HDCP Authorization state change
	CSTVOUT_HDCP_SUPPORTED = 1,     ///<HDCP Supported indication
	CSTVOUT_HDCP_NOT_SUPPORTED = 2, ///<HDCP NOT Supported indication
}CSTVOUT_HDCPSTATE_EVENT;
typedef void (*hdcpstate_callback_function) (int );
CSAPI_RESULT CSTVOUT_HDCPSTATE_SetNotify(CSTVOUT_HANDLE handle,hdcpstate_callback_function callback);

CSAPI_RESULT CSTVOUT_CECLogicalAddressGet(void *cecLogical);
CSAPI_RESULT CSTVOUT_CECPhysicalAddressGet(void *cecPhysical);

/*
 * This function is should only be used for controlling AVMute when using CSTVOUT_HDMI_IN_BYPASS
 * since AVMute is not automatically passed from the source. Otherwise, leave setting
 * AVMute to the hdmi kernel driver.
 * 
 * @param handle CSTVOUT handle
 * @param state 1: enable AVMute, 0: disable AVMute
 * @return CSAPI_SUCCEED Success, CSAPI_FAILED   Failure
 */
CSAPI_RESULT CSTVOUT_HDMI_SetAVMuteEnable(CSTVOUT_HANDLE handle, char state);

/**
 * To set RealTime Kthread priority for HDMI driver
 * @return CSAPI_SUCCEED Success, CSAPI_FAILED   Failure
 */

CSAPI_RESULT CSTVOUT_HDMI_SetRTKthreadPrio(void);

/**
 * To set Normal Kthread priority for HDMI driver
 * @return CSAPI_SUCCEED Success, CSAPI_FAILED   Failure
 */

CSAPI_RESULT CSTVOUT_HDMI_SetNormalKthreadPrio(void);

typedef enum
{
    TVOUT_MHL_REV_11 = 0x11,
    TVOUT_MHL_REV_20 = 0x20
}CSTVOUT_MHL_SINK_VERSION;

CSAPI_RESULT CSTVOUT_MHL_GetSinkVersion(CSTVOUT_HANDLE handle, CSTVOUT_MHL_SINK_VERSION *version);


CSAPI_RESULT CSTVOUT_HDMI_Check_HdcpStatus(CSTVOUT_HANDLE handle,unsigned int* hdcp_status);

CSAPI_RESULT CSTVOUT_MHL_SEND_RCP_KEY(CSTVOUT_HANDLE handle,unsigned int key_value);

CSAPI_RESULT CSTVOUT_MHL_Check_SinkPPModeSupport(CSTVOUT_HANDLE handle,unsigned int* pp_mode_available);

CSAPI_RESULT CSTVOUT_HDMI_Get_MajorDeviceID(CSTVOUT_HANDLE handle,unsigned int* major_id);

#ifdef __cplusplus
}
#endif

#endif

