/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Cavium, Inc.
 Cavium, Inc. reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium, Inc.

 ALL RIGHTS RESERVED.

 Copyright 2011-2012, Cavium, Inc.
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */

/**
 * @file csvid.h
 * @ingroup csapi
 * Low Level Video API
 */
#ifndef __CSAPI_VID_H__
#define __CSAPI_VID_H__

#include <sys/select.h>


#ifdef __cplusplus
extern "C" {
#endif

/* do not modify these two macros*/
#define USER_DATA_BLOCK_SIZE 0x8000

typedef void *CSVID_HANDLE;

/*!
*@brief Define the video decoder channel ID.
*
*@see CSVID_SetStreamType().
*/

#define CSVID_WIN_MIN_WIDTH     (1)
#define CSVID_WIN_MIN_HEIGHT    (1)

#define FrameRate(a, b)               \
    if( a >= VID_FRAME_RATE_7500)     \
        b = VID_FRAME_RATE_7500;      \
    else if(a >= VID_FRAME_RATE_7200) \
        b = VID_FRAME_RATE_7200;      \
    else if(a >= VID_FRAME_RATE_7000) \
        b = VID_FRAME_RATE_7000;      \
    else if(a >= VID_FRAME_RATE_6000) \
        b = VID_FRAME_RATE_6000;      \
    else if(a >= VID_FRAME_RATE_5994) \
        b = VID_FRAME_RATE_5994;      \
    else if(a >= VID_FRAME_RATE_5000) \
        b = VID_FRAME_RATE_5000;      \
    else if(a >= VID_FRAME_RATE_4800) \
        b = VID_FRAME_RATE_4800;      \
    else if(a >= VID_FRAME_RATE_3000) \
        b = VID_FRAME_RATE_3000;      \
    else if(a >= VID_FRAME_RATE_2997) \
        b = VID_FRAME_RATE_2997;      \
    else if(a >= VID_FRAME_RATE_2500) \
        b = VID_FRAME_RATE_2500;      \
    else if(a >= VID_FRAME_RATE_2400) \
        b = VID_FRAME_RATE_2400;      \
    else if(a >= VID_FRAME_RATE_2397) \
        b = VID_FRAME_RATE_2397;      \
    else if(a >= VID_FRAME_RATE_1800) \
        b = VID_FRAME_RATE_1800;      \
    else if(a >= VID_FRAME_RATE_1500) \
        b = VID_FRAME_RATE_1500;      \
    else if(a >= VID_FRAME_RATE_1200) \
        b = VID_FRAME_RATE_1200;      \
    else if(a >= VID_FRAME_RATE_1000) \
        b = VID_FRAME_RATE_1000;      \
    else if(a >= VID_FRAME_RATE_0800) \
        b = VID_FRAME_RATE_0800;      \
    else if(a >= VID_FRAME_RATE_0400) \
        b = VID_FRAME_RATE_0400;      \
    else                              \
        b = VID_FRAME_RATE_UNKNOWN;


typedef enum
{
	VID_DEV_DEC0 = 0,
	VID_DEV_DEC1,
	VID_DEV_DEC2,
	VID_DEV_DEC3,
	VID_DEV_VPB,
    VID_DEV_CAP,
	VID_DEV_MAX
} CSVID_DEV;

typedef enum
{
    VID_DEV_ENC0 = 0,
    VID_DEV_ENC1,
    VID_DEV_ENC2,
    VID_DEV_ENC3,
	VID_ENC_DEV_MAX
} CSVID_ENC_DEV;

typedef enum CSVID_DEV_TYPE_e_
{
	VID_DEV_TYPE_DECODER = 0,
	VID_DEV_TYPE_ENCODER,
    VID_DEV_TYPE_CAPTURE,
    VID_DEV_TYPE_TRANSCODING,
    VID_DEV_TYPE_MULTI_CHANNEL,
	VID_DEV_TYPE_MAX,
} CSVID_DEV_TYPE;

typedef enum{
	VID_DEV_PLAY_MODE_STOP = 0,
	VID_DEV_PLAY_MODE_RUN,
	VID_DEV_PLAY_MODE_PAUSE,
	VID_DEV_PLAY_MODE_FREEZE,
	VID_DEV_PLAY_MODE_STEP
}CSVID_DEV_STATUS;

typedef enum {
    VID_CODA_RESET_SAFETY,//It waits until AXI bus completes working and starts reset process.(recommended mode)
    VID_CODA_RESET_FORCE, //It does not matter whether AXI bus is working or not and forces to reset.
                          //It might affect what other blocks do with bus, so we do not recommend this mode.
    VID_CODA_RESET_ON_BOOT//This is the default reset mode that is executed since system booting.
                             //This mode is actually executed in VPU_Init(), so does not have to be used independently.
}CSVID_CODA_RESET_MODE;

/*!
*@brief Define the stream type.
*
*@see CSVID_SetStreamType().
*/
typedef enum {
    VID_STREAM_TYPE_UNKNOWN = -1,
	VID_STREAM_TYPE_H264 = 0,///<Stream type is H264.
	VID_STREAM_TYPE_VC1,///<Stream type is VC1.
	VID_STREAM_TYPE_MPEG2,///<<Stream type is MPEG2.
	VID_STREAM_TYPE_MPEG4,///<Stream type is MPEG4.
	VID_STREAM_TYPE_H263,///<Stream type is H263
	VID_STREAM_TYPE_DIVX,///<Stream type is DIV3.
	VID_STREAM_TYPE_RV,///<Stream type is RV.
	VID_STREAM_TYPE_AVS,///<Stream type is AVS
	VID_STREAM_TYPE_MJPG,///<Stream type is MJPG
	VID_STREAM_TYPE_THO,///<Stream type is THO
	VID_STREAM_TYPE_VP3,///<Stream type is VP3
	VID_STREAM_TYPE_VP8,///<Stream type is VP8
	VID_STREAM_TYPE_MVC,///<Stream type is MVC
	VID_STREAM_TYPE_YUV,///<Stream type is YUV
#ifdef SUPPORT_SOFT_DECODE
    VID_STREAM_TYPE_VP9,///<Stream type is VP9
#endif
	VID_STREAM_TYPE_MAX_NUM,///<MAX number
	VID_STREAM_TYPE_ENCODE = 255,
} CSVID_STREAM_TYPE;

/* Below two define are used to compatible with old csvid header file. */
#define VID_STREAM_TYPE_H264_TS VID_STREAM_TYPE_H264
#define VID_STREAM_TYPE_MPEG2_TS VID_STREAM_TYPE_MPEG2
/*!
*@brief Define hardeware video layer ID.
*
*@see
*/
typedef enum
{
	VIDEO_LAYER_0 = 0,///<Video layer 0.
	VIDEO_LAYER_1,///<Video layer 1.
}VIDEO_LAYER_NUMBER;

/*!
*@brief Define the input mode.
*
*@see CSVID_SetInputMode().
*/
typedef enum
{
	VID_INPUT_NOBLOCK = 0,///<The input mode is DEMUX0.(TS mode)
	VID_INPUT_BLOCK///<The input mode is STILLPIC.(File mode)
} CSVID_INPUT;

/*!
*@brief The output position of video.
*
*@see CSVID_SetOutputPostion().
*/
typedef struct _CSVID_RECT_t_
{
	short left;///<The left of view port
	short right;///<The right of view port
	short top;///<The top of view port
	short bottom;///<The bottom of view port
}CSVID_Rect;

/*!
*@brief The crop size of the origin video.
*
*@see CSVID_SetOutputPostion().
*/
typedef struct _CSVID_CROP_t_
{
    short CropXOff;///<Video source X offset which user want to crop.
    short CropXWidth;///<Video source width which user want to crop.
    short CropYOff;///<Video source Y offset which user want to crop.
    short CropYHeight;///<Video source Height which user want to crop.
}CSVID_Crop;

/*!
*@brief Frame info of YUV.
*
*@see CSVID_WriteYuvFrameWithPTS().
*/
typedef struct _CSVID_YUV_FRAME_t_
{
    unsigned int width;
    unsigned int height;
    unsigned char *p_y_data;
    unsigned char *p_cb_data;
    unsigned char *p_cr_data;
    unsigned int cbcr_interleave;
    unsigned int y_stride;
    unsigned int cbcr_stride;
}CSVID_Yuv_Frame;

/*!
*@brief Define the error code for the CSVID API.
*
*@see CSVID_GetErrCode()
*/
typedef enum
{
	VID_NO_ERROR = 0,///<No error.
	VID_ERROR_OPEN_FAILED,///<Open filed.		/* open filed                 */
	VID_ERROR_IOCTL_FAILED,///<loctl filed.		/* ioctl filed                */
	VID_ERROR_INVALID_PARAMETERS,///<Bad parameter passed.	/* Bad parameter passed       */
	VID_ERROR_UNKNOWN_DEVICE,///<Unknown device name.	/* Unknown device name        */
	VID_ERROR_DEVICE_BUSY,///<Device is currently busy		/* Device is currently busy   */
	VID_ERROR_INVALID_HANDLE,///<Handle is not valid.	/* Handle is not valid        */
	VID_ERROR_ALREADY_INITIALIZED,///<Device is already initialized.	/* Device already initialized */
	VID_ERROR_NOT_INITIALIZED,///<Device is not initialized.	/* Device not initialized     */
	VID_ERROR_INVALID_STATUS,///<Status of device is invalid.	/* invalid status	      */
	VID_ERROR_WRITE_DATA,///<Write data to demux CPB buffer error.	/* write data to demux CPB buffer error	 */
	VID_ERROR_PLAY_STILLPICTURE,///<Play still picture error.	/* play still picture error	*/
	VID_ERROR_MEMORY_MAP,///<Mempry map failed.    /*mempry map failed*/
	VID_ERROR_NO_CPB_MEMORY,///<cpb buffer not enough.    /*cpb buffer not enough*/
	VID_ERROR_NO_CPB_DIR_MEMORY,///<cpb dir buffer not enough.    /*cpb dir buffer not enough*/
	VID_ERROR_NO_YUV_MEMORY,///<YUV buffer not enough    /*YUV buffer not enough*/
	VID_ERROR_SET_YUV_STREAM,///<YUV stream type has already been set    /*YUV stream type has already been set.*/
} CSVID_ErrCode_e;

/*!
*@brief Define frame rate of stream.
*
*@see
*/
typedef enum tagVID_FrameRate
{
	VID_FRAME_RATE_UNKNOWN = 0,///<Unknown frame rate	// - unknown
	VID_FRAME_RATE_RESERVED_1,///<Reserved1	// - reserved
	VID_FRAME_RATE_0400 = 400,///<Frame rate is 4.00 fps	// - 4.00 fps
	VID_FRAME_RATE_0600 = 600,///<Frame rate is 6.00 fps	// - 6.00 fps
	VID_FRAME_RATE_0800 = 800,///<Frame rate is 8.00 fps	// - 8.00 fps
	VID_FRAME_RATE_1000 = 1000,///<Frame rate is 10.00 fps	// - 10.00 fps
	VID_FRAME_RATE_1200 = 1200,///<Frame rate is 12.00 fps	// - 12.00 fps
	VID_FRAME_RATE_1500 = 1500,///<Frame rate is 15.00 fps	// - 15.00 fps
	VID_FRAME_RATE_1800 = 1800,///<Frame rate is 18.00 fps	// - 18.00 fps
	VID_FRAME_RATE_2397 = 2397,///<Frame rate is 23.97 fps	// - 23.97 fps
	VID_FRAME_RATE_2400 = 2400,///<Frame rate is 24.00 fps	// - 24.00 fps
	VID_FRAME_RATE_2500 = 2500,///<Frame rate is 25.00 fps	// - 25.00 fps
	VID_FRAME_RATE_2997 = 2997,///<Frame rate is 29.97 fps	// - 29.97 fps
	VID_FRAME_RATE_3000 = 3000,///<Frame rate is 30.00 fps	// - 30.00 fps
	VID_FRAME_RATE_4800 = 4800,///<Frame rate is 48.00 fps	// - 48.00 fps
	VID_FRAME_RATE_5000 = 5000,///<Frame rate is 50.00 fps	// - 50.00 fps
	VID_FRAME_RATE_5994 = 5994,///<Frame rate is 59.94 fps	// - 59.94 fps
	VID_FRAME_RATE_6000 = 6000,///<Frame rate is 60.00 fps	// - 60.00 fps
	VID_FRAME_RATE_7000 = 7000,///<Frame rate is 70.00 fps	// - 70.00 fps
	VID_FRAME_RATE_7200 = 7200,///<Frame rate is 72.00 fps	// - 72.00 fps
	VID_FRAME_RATE_7500 = 7500,///<Frame rate is 75.00 fps	// - 75.00 fps
}CSVID_FRAME_RATE;

//!!!Note: please must be to update kernel you will can properly to use video (decoder & encoder).
/*Video interrupt type*/
typedef enum _VID_INT_TYPE{
    VID_INI_UNKNOWN        =  -1    ,
    VID_INT_FW_ERROR       =  (1<<0),
    VID_INT_CMD_ACK        =  (1<<1),
    VID_INT_LOG_MESSAGE    =  (1<<2),
    VID_INT_ENC_FRAME_DONE =  (1<<3),//encoder frame interrupt flags
    VID_INT_MEM_ALLOC      =  (1<<4),
    VID_INT_MEM_FREE       =  (1<<5),
    VID_INT_FRAME_OUTPUT   =  (1<<6),
    VID_INT_FRAME_RELEASE  =  (1<<7),
    VID_INT_DEC_PIC_READY  =  (1<<8),//decoder frame interrupt flags
    VID_INT_AV_SYNC_OK     =  (1<<9),//av sync interrupt flags
    VID_INT_DISPLAY_VSYNC  =  (1<<10),//vsync from vpp
    VID_INT_YUV_BUF_INFO   =  (1<<11),//YUV buffer info for playback
    VID_INT_DEC_SEQ_CHANGE =  (1<<12),//decoder SEQ change
    
    VID_INT_ENABLE         =  (1<<31),//interrupt enable bit
}CSVID_INT;

/*!
*@brief Define frame rate of stream.
*
*@see.
*/
#define VID_FRAME_RATE(r) ((unsigned int)((r) * 100))

/*!
*@brief The parameters for SequenceHeader.
*
*@see CSVID_GetSequenceHeader().
*/
typedef struct tagVID_SequenceHeader
{
	unsigned int w;///<Width of video pictures 		/* width */
	unsigned int h;///<High of video pictures 		/* height */
	unsigned int frame_rate;///<frame_rate*100. 	/* in frames per 1sec */
} CSVID_SequenceHeader;

/*!
*@brief Define aspect ratio data structure.
*
*@see
*/
typedef struct tagVID_SequenceHeader_v2
{
	unsigned int w;///<Width of video pictures 		/* width */
	unsigned int h;///<High of video pictures 		/* height */
	CSVID_FRAME_RATE frame_rate;///<in frames per 1sec 	/* in frames per 1sec */
} CSVID_SequenceHeader_v2;

/*!
*@brief Define timecode data structure.
*
*@see
*/
typedef union
{
	struct {
		unsigned int pictures: 6;///<Range: 0~59
		unsigned int seconds: 6;///<Range: 0~59
		unsigned int marker_bit: 1;///<1
		unsigned int minutes: 6;///<Range: 0~59
		unsigned int hours: 5;///<Range: 0~23
		unsigned int drop_frame_flag: 1;///<Range: 0~23
		unsigned int reserved: 7;///<
    	}bits;

	unsigned int val;///<
}CSVID_TimeCode ;

/*!
*@brief Define video decode error level.
*
*@see CSVID_ErrNotify()
*/
typedef enum{
    VID_LEVEL0 = 0,///<Decode error level 0.
    VID_LEVEL1,///<Decode error level 1.
    VID_LEVEL2,///<Decode error level 2.
    VID_LEVEL3,///<Decode error level 3.
    VID_LEVEL4,///<Decode error level 4.
    VID_LEVEL5,///<Decode error level 5.
    VID_LEVEL6,///<Decode error level 6.
    VID_LEVEL7,///<Decode error level 7.
}CSVID_ERROR_THRESHOLD;

/*!
*@brief Define aspect ratio data structure.
*
*If the aspectratio is 4:3, then the ratio_w is 4 and ratio_h is 3.
*@see
*/
typedef struct tagVID_ASPECTRATIO
{
	int ratio_w;
	int ratio_h;
}CSVID_ASPECTRATIO;

/*!
*@brief Define the decoding type.
*
*@see CSVID_SetDecoderMode().
*/
typedef enum
{
	VID_FRAME_ANY = 0,///<Video decoder decodes all pictures. 	/* all frames */
	VID_FRAME_I,///<Video decoder decodes only pictures with I coding type.		/* I-frames only*/
	VID_FRAME_IP,///<Video decoder decodes only pictures with I or P coding type. 		/* I-frames and P-frames only */
	VID_FRAME_NONE,///<
} CSVID_DECODING_MOD;

/*!
*@brief The flag of AV sync.
*
*@see
*/
typedef enum
{
	VID_SYNC_PICTURE_READY = 2,///<Now can display picture. 	/* can display picture */
	VID_SYNC_AV_SYNC = 1,///<AV sync. 			/* AV sync*/
	VID_SYNC_TIMEOUT = -1,///<AV SYNC timeout. 		/* timeout */
} CSVID_SYNC;

/*!
*@brief Define vidoe decoder init parameters.
*
*@see
*/
typedef struct tagVID_Init_Params
{
	CSVID_FRAME_RATE local_framerate;///<Specify local video frame rate. If video steam has no frame rate, then decoder will use this value.
}CSVID_Init_Params;

/*!
*@brief The status of video buffer.
*
*@see
*/
typedef enum
{
	VID_BUFFER_EMPTY = 0,///<The buffer is empty.
	VID_BUFFER_FULL = 1,///<The buffer is full.
	VID_BUFFER_NORMAL = 2,///<The buffer is normal.
} CSVID_BUFFER_STATUS;

/*!
*@brief Define speed of player.
*
*@see
*/
typedef enum
{
	VID_PLAYSPEED_1_32X = 0,///<1/32x
	VID_PLAYSPEED_1_16X,///<1/16x
	VID_PLAYSPEED_1_8X,///<1/8x
	VID_PLAYSPEED_1_4X,///<1/4x
	VID_PLAYSPEED_1_2X,///<1/2x
	VID_PLAYSPEED_1X,///<1x
	VID_PLAYSPEED_2X,///<2x
	VID_PLAYSPEED_4X,///<4x
	VID_PLAYSPEED_8X,///<8x
	VID_PLAYSPEED_16X,///<16x
	VID_PLAYSPEED_32X,///<32x
}CSVID_PLAYSPEED;

typedef enum
{
    /* mode0: Load stream in chunk of 512 byte. */
    /* Generate display interrupt when the whole frame is decoded. */
    VID_LLD_DISABLE = 0,
    /* mode1: Load stream in chunk of 512 byte. */
    /* Generate display interrupt when x% percent of the frame is decoded. */
    /* Only support H.264 */
    /* NOTE: Reorder will be disabled automatically. */
    VID_LLD_PARTIAL_FRAME_MODE = (1<<0),
    /* mode2: Load stream without chunk unit. Start decoding earlier. */
    /* Generate display interrupt when the whole frame is decoded. */
    /* When one frame data is incomplete the decoder will go back and start from the frame header again. */
    /* This may lead to performance degradation. Only suitable in low-bitrate scenario*/
    VID_LLD_STREAM_LOW_DELAY_MODE = (1<<1),
    /* mode3: Low latency av sync */
    VID_LLD_AVSYNC_LOW_DELAY_MODE = (1<<2),
    /* Combine all modes */
    VID_LLD_ALL = (VID_LLD_PARTIAL_FRAME_MODE) | (VID_LLD_STREAM_LOW_DELAY_MODE) | (VID_LLD_AVSYNC_LOW_DELAY_MODE)
}CSVID_LOW_LATENCY_MODE;

/**
*Define decoding info
*/
typedef struct tagVID_DecodingInfo
{
    unsigned int skipped_frames;    // Number of skipped frames (not displayed)
    unsigned int repeated_frames;   // Number of repeated frames diplayed 
    unsigned int error_frames;      // Number of decoding errors
    unsigned int decoded_I_frames;  // Number of I-frames decoded
    unsigned int decoded_IDR_frames;// Number of IDR-frames decoded
    unsigned int valid_frames;      // Number Non repeated frames displayed 
}CSVID_DecodingInfo;

/****************************************************************************
*                    Encoder attributes define Start                        *
*****************************************************************************/
typedef enum
{
    /* Start encoding when full frame is captured */
    VID_LLE_DISABLE = 0,
    /* Start when x% percent of the frame is captured */
    /* and sync to input pictures line-to-line */
    VID_LLE_PARTIAL_FRAME_MODE = 1
}CSVID_ENC_LOW_LATENCY_MODE;

typedef struct
{
    unsigned int   Format3D:4; 		//2D, SbS, TaB, FP, PVFP-2D-i, PVFP-3D
    unsigned int   IsInterlace:1;
    unsigned int   HorSample:1;
    unsigned int   VerSample:1;
    unsigned int   TopFieldFirst:1;
    unsigned int   VactSpace1:8;		//for interlace or progressive frame packing, see HDMI Spec
    unsigned int   VactSpace2:8;		//for interlace frame packing, see HDMI Spec
    unsigned int   PicWidth:16;
    unsigned int   PicHeight:16;
    unsigned int   FrameRate;
}PicFormat;

typedef struct{
    unsigned int sliceMode:1;    //0: one slice per picture; 1: multiple slices per picture.
    unsigned int sliceSizeMode:1;//0 means the sliceSize is defined by the amount of bits, and 1 means the sliceSize is defined by the number of MBs.
    unsigned int sliceSize:30;   //size of a slice in bits or in MB numbers included in a slice
} EncSliceMode;

typedef struct
{
    unsigned char Mp4_DataPartitionEn:1;
    unsigned char Mp4_RvlcEn:1;
}EncMp4Param;

typedef struct
{
    unsigned char H263_AnnexIEn:1;
    unsigned char H263_AnnexJEn:1;
    unsigned char H263_AnnexKEn:1;
    unsigned char H263_AnnexTEn:1;
}EncH263Param;

typedef struct
{
    unsigned int Avc_DisableDeblocking:1;
}EncAvcParam;

typedef struct
{
    unsigned char Mjpg_SrcFormat; //0-4:2:0, 1-4:2:2 horizontal, 2-4:2:2 vertical, 3-4:4:4, 4-4:0:0
    unsigned char *HuffVal;
	unsigned char *HuffBits;
	unsigned char *QMatTab;
	unsigned char *CInfoTab;
}EncMjpgParam;

typedef struct
{
    unsigned int FrameRateInfo:16;  //use VID_FRAME_RATE(). actual frame rate = FrameRateInfo / 100
    unsigned int RcIntervalMode:2; // RC interval mode: 0 = NORMAL, 1 = FRAME, 2 = SLICE, 3 = MB.
    unsigned int MbInterval:14;    // For Mode 3, number of MBs
    unsigned int BitRate:16; // Maximum bitrate value is 65535 kbps. 0 to disable rc.
    unsigned int IntraRefresh:16;
    unsigned int RateCtrlMode:2; // rate control enable (0:constant QP, 1:CBR, 2:ABR)
    unsigned int StrictCbr:1; //use strict CBR for RC. May have stuffing bits in the stream.
    unsigned int QpY:6;      // 1~31 for MPEG4; 0~51 for H.264
    unsigned int EnableAutoSkip:1;
    unsigned int GopSize:15; //Maximum value will be 32767.
    unsigned int MeSearchRangeX:2; // 0:(-64~63) 1:(-48~47) 2:(-32~31) 3:(-16~15)
    unsigned int MeSearchRangeY:2; // 0:(-48~47) 1:(-32~31) 2:(-16~15)
    unsigned int Profile:2; //H.264: 0:BP 1:MP 2:HP  MPEG4:ignored
    EncSliceMode EncSliceMode;
    union {
		EncMp4Param Mp4Param;
		EncH263Param H263Param;
		EncAvcParam AvcParam;
		EncMjpgParam MjpgParam;
	} EncStdParam;
}EncOpenParam;

typedef struct
{
    unsigned int SrcBufNo:8;//DpbBufNo:8;
    unsigned int CodecStd:4;
    unsigned int SubFrameSyncEn:1;
    unsigned int EncOneJpeg:1;
    unsigned int ReportPerField:1;
    unsigned int JpegFrameAddr;
    unsigned int PtsFifoStartAddr;
    unsigned int PtsFifoEndAddr;
    unsigned int EsFifoStartAddr;
    unsigned int EsFifoEndAddr;
    unsigned int MjpgEsFifoStartAddr;
    unsigned int MjpgEsFifoEndAddr;
    PicFormat PicFormat;
    EncOpenParam EncOpenParam;
}EncCpbAttr;

/*********************************************************************************
*                            Encoder attributes define End                       *
**********************************************************************************/
typedef struct{
    unsigned int  BufNo;
    unsigned char *Yaddr;
    unsigned char *Caddr;
    unsigned int  YuvFormat;
    unsigned int  LinePitch;
    unsigned int  PicWidth;
    unsigned int  PicHeight;
}VideoYUVInfoT;

/*!
*@brief Initialize the video decoder.
*
*@param params	Some parameters which video decoder initializs needed.	In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@see CSVID_Term.
*
*/
CSAPI_RESULT CSVID_Init(CSVID_Init_Params params);
/*!
*@brief stop and reset video decoder
*
*@see CSVID_Init.
*
*/
void CSVID_Term(void);
/*
*@brief monitor video device status
*
*@param1  A valid handle on success.	           In/Out: In
*@param2  interrupt value at return CSAPI_SUCCEED  In/Out: Out
*@param3  monitoring video device timeout time     In/Out: In
*
*/
CSAPI_RESULT CSVID_Event(CSVID_HANDLE handle, unsigned int *val, struct timeval time_out);

/*!
*@brief Open a connection to the video channel.
*
*@param None
*@return CSVID_HANDLE	A valid handle on open success.
*@see None
*
*/
CSVID_HANDLE CSVID_Open(CSVID_DEV dev);

/*!
*@brief Close the connection to the video channel.
*
*@param handle	A valid handle on success.	In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@see None
*
*/
CSAPI_RESULT CSVID_Close(CSVID_HANDLE handle);

/*!
*@brief Open a connection to the video encoder channel.
*
*@param1  An encoder device                     In/Out: In  //recommend used VID_DEV_ENC0
*@param2  encoder attributes                    In/Out: In
*@return  CSVID_HANDLE	A valid handle on open success.
*@see None
*
*Note1: The encoder only support HDMI RX input now
*Note2: Must be calling in the following sequence before call the CSVID_OpenEncoder()
*Note3: Then through the CSHDMIRX_DetectFormat() API to get input source resolution, please reference encoder case of csvid_test.c
*Note4: you can calling both the CSVID_SetSrcPosition() or CSVID_SetDstPosition() APIs to set different position.
*
*   CSHDMIRX_Open();
*   CSTVOUT_HDMI_IN_ENABLE(tvout_handle, 1); //The tvout_handle form CSTVOUT_Open();
*/
CSVID_HANDLE CSVID_OpenEncoder(CSVID_ENC_DEV dev, EncCpbAttr* EncAttr);
//CSVID_HANDLE CSVID_EncOpen(CSVID_ENC_DEV dev, EncCpbAttr* EncAttr, CSVID_Rect *SrcRect, CSVID_Rect *DstRect);
/*!
*@brief Close the connection to the video encoder channel.
*
*@param handle	A valid handle on success.	In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@see None
*
*/
//CSAPI_RESULT CSVID_EncClose(CSVID_HANDLE handle);
CSAPI_RESULT CSVID_CloseEncoder(CSVID_HANDLE handle);
/*!
*@brief Open a connection to the video capture channel for HDMI RX.
*
*@param1  An capture device                     In/Out: In  //recommend used VID_DEV_CAP
*@param2  HDMI rx input width                   In/Out: In
*@param3  HDMI rx input height                  In/Out: In
*@param4  HDMI rx input interlace               In/Out: In
*@return CSVID_HANDLE	A valid handle on open success.
*@see None
*
*Note1: The capture only support HDMI RX input now
*Note2: Must be calling in the following sequence before call the CSVID_OpenCapture()
*Note3: Then through the CSHDMIRX_DetectFormat() API to get input source resolution, please reference capture case of csvid_test.c
*Note4: you can calling both the CSVID_SetSrcPosition() or CSVID_SetDstPosition() APIs to set different position.
*
*   CSHDMIRX_Open();
*   CSTVOUT_HDMI_IN_ENABLE(tvout_handle, 1); //The tvout_handle form CSTVOUT_Open();
*
*/
CSVID_HANDLE CSVID_OpenCapture(CSVID_DEV dev, unsigned int input_width, unsigned int input_height, unsigned int input_interlace);
/*!
*@brief Close the connection to the video capture channel.
*
*@param handle	A valid handle on success.	In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@see None
*
*/
CSAPI_RESULT CSVID_CloseCapture(CSVID_HANDLE handle);
/*!
*@brief Open a connection to the video transcoding channel.
*
*@param1  An transcoding device                     In/Out: In  //recommend used VID_DEV_ENC0
*@param2  encoder attributes                    In/Out: In
*@param3 decoder handle	A valid handle from CSVID_OpenTranscoding() on success.	In
*@param4 encoder handle	A valid handle from CSVID_OpenTranscoding() on success.	In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@see None
*/

CSAPI_RESULT CSVID_OpenTranscoding(CSVID_DEV_TYPE dev, EncCpbAttr* EncAttr, CSVID_HANDLE *dec_handle, CSVID_HANDLE *enc_handle);
/*!
*@brief Close the connection to the video transcoding channel.
*
*@param1 decoder handle	A valid handle on success.	In
*@param2 encoder handle	A valid handle on success.	In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@see None
*/
CSAPI_RESULT CSVID_CloseTranscoding(CSVID_HANDLE dec_handle, CSVID_HANDLE enc_handle);
/*!
*@brief Start the transcoding of the specified data.
*
*@param1 decoder handle	A valid handle on success.	In
*@param2 encoder handle	A valid handle on success.	In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@see None
*/
CSAPI_RESULT CSVID_PlayTranscoding(CSVID_HANDLE dec_handle, CSVID_HANDLE enc_handle);
/*!
*@brief Stop the transcoding of the specified data.
*
*@param1 decoder handle	A valid handle on success.	In
*@param2 encoder handle	A valid handle on success.	In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@see None
*/
CSAPI_RESULT CSVID_StopTranscoding(CSVID_HANDLE dec_handle, CSVID_HANDLE enc_handle);
/*!
*@brief Start the decoding and display of the specified data.
*
*The function is not blocking, and returns immediately. After calling the function, data should be injected. The user must set corresponding stream type (by calling CSVID_SetStreamType()) according to the input stream, and then video will accept/decode the specified data. Otherwise, it will return fail.
*@param handle	A valid handle on success.	In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@see None
*
*/
CSAPI_RESULT CSVID_Play(CSVID_HANDLE handle);

/*!
*@brief Stop the decoding and display of the specified data.
*
*It will stop the decoding and playing of the specified data, and then the display will remain the previous picture. All the display configurations (i.e. position, states, freeze\A1\AD ) are unchanged after calling CSVID_Stop().
*@param handle	A valid handle on success.	In/Out: In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@see None
*
*/
CSAPI_RESULT CSVID_Stop(CSVID_HANDLE handle);

/*!
*@brief Start the encoding and save es stream data .
*
*The function is not blocking, and returns immediately. After calling the function, data should be save.
*@param handle	A valid handle on success.	In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@see None
*
*/

CSAPI_RESULT CSVID_PlayEncoder(CSVID_HANDLE handle);
/*!
*@brief Stop the encoding and will be stop recv es stream.
*
*It will stop the encoding and will be stop recv es stream.
*@param handle	A valid handle on success.	In/Out: In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@see None
*
*/
CSAPI_RESULT CSVID_StopEncoder(CSVID_HANDLE handle);
/*!
*@brief Start the capture data to the TV or monitor
*
*The function is not blocking, and returns immediately. After calling the function,
*@param handle	A valid handle on success.	In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@see None
*
*/

CSAPI_RESULT CSVID_PlayCapture(CSVID_HANDLE handle);
/*!
*@brief Stop the capture data to the TV or monitor.
*
*It will stop the capture data to the TV or monitor.
*@param handle	A valid handle on success.	In/Out: In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@see None
*
*/
CSAPI_RESULT CSVID_StopCapture(CSVID_HANDLE handle);

/*!
*@brief Pause the decoding and display of the specified data.
*
*The pause function can only be used in non real-time data input mode, where data input is completely controlled by application.\n
*The user may call CSVID_Resume() to resume the display, exactly where it was paused.
*@param handle	A valid handle on success.	In/Out: In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@see None
*
*/
CSAPI_RESULT CSVID_Pause(CSVID_HANDLE handle);

/*!
*@brief Freeze the display, while decoding goes on.
*
*The freeze function will always stop the display, while the decoding is going on. It has the same behavior as CSVID_Pause() concerning the display, but leaves the decode running.\n
*The user may call CSVID_Resume() to resume the display, with the last decoded picture.
*@param handle	A valid handle on success.	In/Out: In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@see None
*
*/
CSAPI_RESULT CSVID_Freeze(CSVID_HANDLE handle);

/*!
*@brief Resume a previously paused or freezed decoding.
*
*If the display was paused, it resumes exactly where it was paused.\n
*If the display was freezed, the display resumes with the last decoded picture.
*@param handle	A valid handle on success.	In/Out: In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@see None
*
*/
CSAPI_RESULT CSVID_Resume(CSVID_HANDLE handle);

/*!
*@brief Display next field when paused or freezed.
*
*If the video is paused, the next field is temporally the field following the current field.\n
*If the video is freezed (display freezed but decode running), the next field is the next available field from the running decode, i.e. the first field of last decoded picture. Generally, it is temporally not the field following the current field displayed.
*@param handle  A valid handle on success.  In/Out: In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@see None
*
*/
CSAPI_RESULT CSVID_Step(CSVID_HANDLE handle, unsigned char StepFrameNum);

/*!
*@brief Set video layer which video channel bind to.
*
*@param handle	An opened video handle.	In/Out: In
*@param layer	Video layer ID.
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@see None
*
*/
CSAPI_RESULT CSVID_Set_Output_Layer(CSVID_HANDLE handle, VIDEO_LAYER_NUMBER layer);

/*!
*@brief Get the video layer which video channel bind to.
*
*@param handle	An opened video handle.	In/Out: In
*@param layer	Video layer ID.	In/Out: Out
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@see None
*
*/
CSAPI_RESULT CSVID_Get_Output_Layer(CSVID_HANDLE handle, VIDEO_LAYER_NUMBER *layer);

/*!
*@brief Set the video channel output order in video layer which the channel bind to.
*
*@param handle	An opened video handle.	In/Out: In
*@param Order	From 0 to 7 map back to front.	In/Out: In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@see None
*
*/
CSAPI_RESULT CSVID_SetLayerOrder(CSVID_HANDLE handle, unsigned int order);

/*!
*@brief Set the video display layer position.
*
*@param handle	An opened video handle.	In/Out: In
*@param src	Soruce stream format.	In
*@param dst	Destination display format.	In/Out: In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@see None
*
*/
CSAPI_RESULT CSVID_SetLayerPostion(CSVID_HANDLE handle, CSVID_Rect dst);

/*!
*@brief Enable the video layer which video channel bind to.
*
*@param handle	An opened video handle.	In/Out: In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@see None
*
*/
CSAPI_RESULT CSVID_EnableOutput(CSVID_HANDLE handle);

/*!
*@brief Disable the video layer which video channel bind to.
*
*@param handle	An opened video handle.	In/Out: In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@see None
*
*/
CSAPI_RESULT CSVID_DisableOutput(CSVID_HANDLE handle);

/*!
*@brief Set multi video window support or not.
*       Supporting multi-win may increase video latency.
*       In multi-win app call this before opening devices.
*
*@param
*@param
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@note In multi-win app *MUST* call this before opening devices.
*@see None
*
*/
CSAPI_RESULT  CSVID_EnableMultiWin(void);

/*!
*@brief Set multi video window support or not.
*       Supporting multi-win may increase video latency.
*       In multi-win app call this before opening devices.
*
*@param
*@param
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@note In multi-win app *MUST* call this before opening devices.
*@see None
*
*/
CSAPI_RESULT  CSVID_DisableMultiWin(void);

/*!
*@brief Set the type of the input stream.
*
*@param handle	A valid handle on success.	In/Out: In
*@param stream_type	The new stream type which the user want to set.	In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@note The user must set the stream type before video start. The type of the input stream is set according to the fields of the parameters structure. The video driver will discard all data not be selected, and only accept/decode the specified data.
*@see None
*
*/
CSAPI_RESULT CSVID_SetStreamType(CSVID_HANDLE handle, CSVID_STREAM_TYPE stream_type);

/*!
*@brief Set the position of view port.
*
*@param handle	A valid handle on success.	In/Out: In
*@param src	The position of the input window.	In/Out: In
*@param dst	The position of the output window.	In/Out: In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@note Setting the input and the output window defines what should be displayed from the decoded stream, where and with which scaling factor. This is done by calling this function.
*@see None
*
*/
CSAPI_RESULT CSVID_SetOutputPostion(CSVID_HANDLE handle, CSVID_Crop src, CSVID_Rect dst);
/*!
*@brief Set the position for input source.
*
*@param1 handle	        A valid handle on success.	In/Out: In
*@param2 srcWidth	    The source input width.	    In/Out: In
*@param3 srcHeight	    The source input height.	In/Out: In
*@param4 pRect          to set position for input source  In/Out: In
*@param5 is_interlace   set inpunt source interlace or progressive
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@see None
*
*/
CSAPI_RESULT CSVID_SetSrcPosition(CSVID_HANDLE handle, unsigned int srcWidth, unsigned int srcHeight, CSVID_Rect *pRect, unsigned int is_interlace);
/*!
*@brief Set the position for output destination.
*
*@param1 handle	        A valid handle on success.	In/Out: In
*@param2 dstWidth	    The destination output width.	    In/Out: In
*@param3 dstHeight	    The destination output height.	In/Out: In
*@param4 pRect          to set position for output destination  In/Out: In
*@param5 is_interlace   set inpunt destination interlace or progressive
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@see None
*
*/
CSAPI_RESULT CSVID_SetDstPosition(CSVID_HANDLE handle, unsigned int dstWidth, unsigned int dstHeight, CSVID_Rect *pRect, unsigned int is_interlace);
/*!
*@brief Set the global alpha of a view port.
*
*@param handle	A valid handle on success.	In/Out: In
*@param alpha	video layer 0 alpha range 0 ~ 0x3ff.	In/Out: In
*               0      disable video layer 0 output
*               0x3ff  enable  video layer 0 output
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@note If the user call this function and set the alpha to 0, it will be blank on viewport.
*@see None
*
*/
CSAPI_RESULT CSVID_SetOutputAlpha(CSVID_HANDLE handle, unsigned int alpha);

/*!
*@brief Set the type of input mode.
*
*The user must set the input mode before video decoding. The video driver needs a correct input mode to inject data.
*@param handle	A valid handle on success.	In/Out: In
*@param input_mod	The mode of input, there are three input-channels to choice: VID_INPUT_NOBLOCK, and VID_INPUT_BLOCK.	In/Out: In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@see None
*
*/
CSAPI_RESULT CSVID_SetInputMode(CSVID_HANDLE handle, CSVID_INPUT input_mod);

/*!
*@brief Enable automatic video synchronization with local system clock.
*
*Calling CSVID_DisablePTSSync and CSVID_EnablePTSSync can not nested.
*@param handle	A valid handle on success.	In/Out: In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@see None
*
*/
CSAPI_RESULT CSVID_EnableTrickMode(CSVID_HANDLE handle);

/*!
*@brief Disable automatic video synchronization with local system clock.
*
*Calling CSVID_DisablePTSSync and CSVID_EnablePTSSync can not nested.
*@param handle	A valid handle on success.	In/Out: In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@see None
*
*/
CSAPI_RESULT CSVID_DisableTrickMode(CSVID_HANDLE handle);

/*!
*@brief Set the decoder mode of video driver.
*
*To set a default type (CSVID_FRAME_ANY) of video stream decoded at initialization of the video decoder. So the user needs to set the type first in the device init for the different data. Setting the required decoded mode may not be possible depending on the memory profile of the decoder, so user can't call this function when playing. If the driver be set the wrong decoding mode of the data, it will decode nothing.
*@param handle	A valid handle on success.	In/Out: In
*@param decoding_mod	The new decode mode which the user want to set.	In/Out: In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@see None
*
*/
CSAPI_RESULT CSVID_SetDecoderMode(CSVID_HANDLE handle, CSVID_DECODING_MOD decoding_mod);

/*!
*@brief Set player speed.
*
*Before set fast forward level you must first enable trick mode and set decoder mode, if not this founction will not work.
*@param handle	A valid handle on success.	In/Out: In
*@param speed	Fast forward level.	In/Out: In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@see CSVID_EnableTrickMode
*@see CSVID_SetDecoderMode
*
*/
CSAPI_RESULT CSVID_SetPlaySpeed(CSVID_HANDLE handle, CSVID_PLAYSPEED speed);
CSAPI_RESULT CSVID_GetPlaySpeed(CSVID_HANDLE handle, CSVID_PLAYSPEED *speed);
/*!
*@brief Send a default sequence header to video decoder.
*
*@param handle	A valid handle on success.	In/Out: In
*@param hdr  Specify a sequence header.
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@see
*
*/
CSAPI_RESULT CSVID_SetSequenceHeader(CSVID_HANDLE handle, CSVID_SequenceHeader hdr);

/*!
*@brief Enable automatic video synchronization with local system clock.
*
*Calling CSVID_DisablePTSSync and CSVID_EnablePTSSync can not nested.
*@param handle	A valid handle on success.	In/Out: In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@see None
*
*/
CSAPI_RESULT CSVID_EnablePTSSync(CSVID_HANDLE handle);

/*!
*@brief Disable automatic video synchronization with local system clock.
*
*Calling CSVID_DisablePTSSync and CSVID_EnablePTSSync can not nested.
*@param handle	A valid handle on success.	In/Out: In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@see None
*
*/
CSAPI_RESULT CSVID_DisablePTSSync(CSVID_HANDLE handle);

/*!
*@brief Get cunrrent PTS from video decoder.
*
*Calling CSVID_DisablePTSSync and CSVID_EnablePTSSync can not nested.
*@param handle	A valid handle on success.	In/Out: In
*@param vid_pts	Pointer to the pts.	Get
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@see None
*
*/
CSAPI_RESULT CSVID_GetSequenceHeader(CSVID_HANDLE handle, CSVID_SequenceHeader *hdr);

/*!
*@brief Get the current parameters of sequence header from the source stream.
*
*The function is not blocking, and it will return the current parameters of sequence header from the source stream. If there is no sequence header in the stream or there is no sequence header can be got normally, it will return fail.
*@param Handle	A valid handle on success.	In/Out: In
*@param Hdr	The source sequence header of the stream.	In/Out: Out
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@see None
*
*/
CSAPI_RESULT CSVID_GetSRCSequenceHeader(CSVID_HANDLE handle, CSVID_SequenceHeader * hdr);

/*!
*@brief Get the current parameters of sequence header from the source stream.
*
*The function is not blocking, and it will return the current parameters of sequence header from the source stream. If there is no sequence header in the stream or there is no sequence header can be got normally, it will return fail.
*@param Handle	A valid handle on success.	In/Out: In
*@param Hdr	The source sequence header of the stream.	In/Out: Out
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@see None
*
*/
CSAPI_RESULT CSVID_GetSequenceHeader_v2(CSVID_HANDLE handle, CSVID_SequenceHeader_v2 * hdr);

/*!
*@brief Get the current parameters of sequence header from the source stream.
*
*The function is not blocking, and it will return the current parameters of sequence header from the source stream. If there is no sequence header in the stream or there is no sequence header can be got normally, it will return fail.
*@param Handle	A valid handle on success.	In/Out: In
*@param Hdr	The source sequence header of the stream.	In/Out: Out
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@see None
*
*/
CSAPI_RESULT CSVID_GetSRCSequenceHeader_v2(CSVID_HANDLE handle, CSVID_SequenceHeader_v2 * hdr);

/*!
*@brief Set 3to2 poll down function.
*
*When source frame rate = 23.97 or 24, and the source pictures coded in frame mode, and there is no 3:2 pull down flag, firmware will force pull down the source to 30fps frame rate when display.
*@param Handle	An opened video handle.	In/Out: In
*@param enable_flag	1: enable 3to2 poll down;\n
*                   2: disable 3to2 poll down.	In/Out: In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@see None
*
*/
CSAPI_RESULT CSVID_Force3to2PollDown(CSVID_HANDLE handle, unsigned int enable_flag);

/*
Interface for setting AV sync mode
handle: video decoder handle. get it from CSVID_Open().
enable_flag:
0 - PICTURE_READY---------(we display picture and can see the sync process(picture will shake) if we open the video alpha.)--------------SYNC_OK
1 - PICTURE_READY---------(we don't display the frames, still black screen)---------SYNC_OK
2 - PICTURE_READY---------(fast sync mode. only first frame will show if you open video alpha)-----------SYNC_OK
3 - PICTURE_READY---------(fast sync mode. we don't diplay the first frame. black screen)---------------SYNC_OK
Note: There are two stage in AV sync. One is PICTURE_READY(this is the first stage. The first frame is ready to display). Another is SYNC_OK(this is the second stage. means A/V sync is done).
*/

/*!
*@brief Set AV sync notify mode.
*
*@param Handle	An opened video handle.	In/Out: In
*@param enable_flag	See note. In/Out: In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@note enable_flag:\n
*0 - PICTURE_READY---------(we display picture and can see the sync process(picture will shake) if we open the video alpha.)--------------SYNC_OK\n
*1 - PICTURE_READY---------(we don't display the frames, still black screen)---------SYNC_OK\n
*2 - PICTURE_READY---------(fast sync mode. only first frame will show if you open video alpha)-----------SYNC_OK\n
*3 - PICTURE_READY---------(fast sync mode. we don't diplay the first frame. black screen)---------------SYNC_OK\n
*There are two stage in AV sync. One is PICTURE_READY(this is the first stage. The first frame is ready to display). Another is SYNC_OK(this is the second stage. means A/V sync is done).
*@see None
*
*/
CSAPI_RESULT CSVID_WaitSync(CSVID_HANDLE handle, unsigned int enable_flag);
/*!
*@brief Define video decoder error frame process mode
*@note
*Value						Error I/P frame													Error B frame\n
*VID_ERROR_SKIP_MODE_0 : 	Display															Display\n
*VID_ERROR_SKIP_MODE_1 : 	If has error, skip this frame and search next sequence header.	                     Only skip this B frame.\n
*VID_ERROR_SKIP_MODE_2 : 	If has error, skip this frame and search next sequence header.	                     Display\n
*VID_ERROR_SKIP_MODE_3 : 	If has error, skip this frame and search next sequence header.	                     Skip this B frame and search next sequence header.\n
*@see
*/
typedef enum
{
	VID_ERROR_SKIP_MODE_0 = 0,
	VID_ERROR_SKIP_MODE_1 = 1,
	VID_ERROR_SKIP_MODE_2 = 2,
	VID_ERROR_SKIP_MODE_3 = 3,
	VID_ERROR_SKIP_MODE_INVALID = 4,
}CSVID_ERROR_SKIP_MODE_e;

/*!
*@brief Set error skip mode with video decoder.
*
*@param Handle	An opened video handle.	In/Out: In
*@param errorskipmode	1: Skip the error pictures In/Out: output
*                       0: Output all pictures include the pictures which have error.	In/Out: In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@see None
*
*/
CSAPI_RESULT CSVID_SetErrorSkipMode(CSVID_HANDLE handle, CSVID_ERROR_SKIP_MODE_e errorskipmode);

/*
function : CSVID_StartDelay
startdelay_ms : unit is ms
If the delay time is positive, video will slow than the system clock; if the delay time is negative, video will fast than the system clock;
*/

/*!
*@brief Set delay time which decoder delay start decoding.
*
*@param handle	An opened video handle.	In/Out: In
*@param startdelay_ms	Delay time.	In/Out: In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@see None
*
*/
CSAPI_RESULT CSVID_StartDelay(CSVID_HANDLE handle, int startdelay_ms);

/*!
*@brief Get the error code of video driver.
*
*@param Handle	Handle of video driver.	In/Out: In
*@return CSVID_ErrCode	The error code of video driver
*@see None
*
*/
CSVID_ErrCode_e CSVID_GetErrCode(void);

/*!
*@brief Get timecode of the current stream.
*
*H.264 don't support now.
*@param handle	A valid handle on success.	In/Out: In
*@param timecode	Pointer to data structure CSVID_TimeCode.	In/Out: Out
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@see CSVID_TimeCode
*
*/
CSAPI_RESULT CSVID_GetTimeCode(CSVID_HANDLE handle, CSVID_TimeCode *timecode);

/*!
*@brief Get aspect ratio of the current stream.
*
*@param handle	A valid handle on success.	In/Out: In
*@param aspect_ratio	Pointer to data structure CSVID_ASPECTRATIO.	In/Out: Out
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@see CSVID_ASPECTRATIO
*
*/
CSAPI_RESULT CSVID_GetAspectRatio(CSVID_HANDLE handle, CSVID_ASPECTRATIO * aspect_ratio);

/*!
*@brief Notify timecode reaching and callback function will be invoked.
*
*H.264 don't support now.
*@param handle	An opened video handle.	In/Out: In
*@param call_back_function	Pointer to the callback function.	In/Out: In
*@param timecode	The callback function is invoked while video source timecode reaching and passing this value.	In
*@param event_enable	It controls event on/off.\n
*                          0: disable
*                         1: enable	In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@see CSVID_TimeCode
*
*/
CSAPI_RESULT CSVID_TimeCodeReportNotify(CSVID_HANDLE handle,void (*call_back_function)(CSVID_HANDLE),CSVID_TimeCode timecode,int event_enable);

/*!
*@brief Notify error during video playing through callback function.
*
*The error threshold for callback is communicated through an error threshold value of 0 through 7.
*0: no error.(normally, don't use this value.)
*1: least error to viewer.
*\A1\AD\A1\AD
*7:most error to viewer including fatal non-recoverable stop.
*h.264 don't support now.
*@param handle	An opened video handle. In/Out: In
*@param call_back_function	Pointer to the callback function.	In/Out: In
*@param error_threshold	Specifies event notification requirement in terms of an error metrics.	In/Out: In
*@param event_enable	It controls event on/off.\n
*                       0: disable
*                       1: enable	In
*
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@see CSVID_ERROR_THRESHOLD
*
*/
CSAPI_RESULT CSVID_ErrNotify(CSVID_HANDLE handle,void (*call_back_function)(CSVID_HANDLE,CSVID_ERROR_THRESHOLD),CSVID_ERROR_THRESHOLD error_threshold,int event_enable ,unsigned int timeout_value);

/*!
*@brief Notify new aspect ratio coming during video playing through callback function.
*
*H.264 don't support now.
*@param handle	An opened video handle.	In/Out: In
*@param call_back_function	Pointer to the callback function. In/Out: In
*@param event_enable	It controls event on/off.\n
*                        0: disable
*                        1: enable	In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@see CSVID_ASPECTRATIO
*
*/
CSAPI_RESULT CSVID_AspectRatioChangeNotify ( CSVID_HANDLE handle,void (* call_back_function)(CSVID_HANDLE, CSVID_ASPECTRATIO *),int event_enable);

/*!
*@brief Notify upper layer on A/V sync.
*
*@param handle	An opened video handle	In/Out: In
*@param call_back_function	Pointer to the callback function. Callback function is invoked if event_enable equal to 1 and A/V sync established or timeout reached.	In/Out: In
*@param timeout_value	Value is in microsecond.	In/Out: In
*@param event_enable	It controls event on/off.\n
*                        0: disable
*                        1: enable	In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@see None
*
*/
CSAPI_RESULT CSVID_SyncNotify ( CSVID_HANDLE handle,void (* call_back_function)(CSVID_HANDLE, CSVID_SYNC *),unsigned int timeout_value, int event_enable );

/*!
*@brief Notify upper layer on video data under flow.
*
*@param handle	A valid handle on success.	In/Out: In
*@param call_back_function	Pointer to the callback function.	In/Out: In
*@param event_enable	This value is 0 or !0. In/Out: In
*@param timeout_value	Time out value.	In/Out: In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@see None
*
*/
CSAPI_RESULT CSVID_DataUnderflowNotify(CSVID_HANDLE handle, void (*call_back_function) (CSVID_HANDLE),int event_enable,unsigned int timeout_value);

/*!
*@brief Notify upper layer on video frame sequence header change.
*
*@param handle	A valid handle on success.	In/Out: In
*@param call_back_function	Pointer to the callback function.	In/Out: In
*@param event_enable	This value is 0 or !0. In/Out: In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@see None
*
*/
CSAPI_RESULT CSVID_FormatChangeNotify(CSVID_HANDLE handle, void (*call_back_function) (CSVID_HANDLE, CSVID_SequenceHeader *),int event_enable);

/*!
*@brief Notify upper layer on video frame sequence header change.
*
*@param handle	A valid handle on success	In
*@param call_back_function	Pointer to the callback function.  In/Out: In
*@param event_enable	This value is 0 or !0.  In/Out: In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@see None
*
*/
CSAPI_RESULT CSVID_SRCFormatChangeNotify(CSVID_HANDLE handle, void (*call_back_function) (CSVID_HANDLE,CSVID_SequenceHeader *),int event_enable);

/*!
*@brief Notify upper layer on all the video data be decoded completly.
*
*@param handle	A valid handle on success. In/Out: In
*@param call_back_function	Pointer to the callback function. In/Out: In
*@param event_enable	This value is 0 or !0.  In/Out: In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@see None
*
*/
CSAPI_RESULT CSVID_DecodeFininshNotify( CSVID_HANDLE handle,void (* call_back_function)(CSVID_HANDLE), int event_enable );

/*!
*@brief Notify upper layer when parse user data.
*
*@param handle	A valid handle on success. In/Out: In
*@param call_back_function	Pointer to the callback function. In/Out: In
*@param event_enable	This value is 0 or !0.  In/Out: In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@see None
*
*/
CSAPI_RESULT CSVID_UserDataNotify ( CSVID_HANDLE handle,void (* call_back_function)(CSVID_HANDLE *, unsigned char *, int), int event_enable );

/*!
*@brief Get cunrrent PTS from video decoder.
*
*Calling CSVID_DisablePTSSync and CSVID_EnablePTSSync can not nested.
*@param handle	A valid handle on success. In/Out: In
*@param vid_pts	Pointer to the pts.	In/Out: Out
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@see
*
*/
CSAPI_RESULT CSVID_GetPTS(CSVID_HANDLE handle, unsigned int *vid_pts);

/*!
*@brief Prepare PFM function.
*
*Set driver to access decoder input memory buffer directly for write rather than TS demux. Initialize buffer management variables.
*@param handle	An opened video handle.	In/Out: In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@see None
*
*/
CSAPI_RESULT CSVID_PFMOpen(CSVID_HANDLE handle);

/*!
*@brief Close for PFM function.
*
*Return decoder buffer write access to TS demux and clears PFM state.
*@param handle	An opened video handle.	In/Out: In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@see None
*
*/
CSAPI_RESULT CSVID_PFMClose(CSVID_HANDLE handle);

/*!
*@brief Reset PFM function.
*
*Reset all variables related to PFM memory management.
*@param handle	An opened video handle. In/Out: In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@see None
*
*/
CSAPI_RESULT CSVID_PFMReset(CSVID_HANDLE handle);

/*!
*@brief Get current decoder buffer size available.
*
*@param handle	An opened video handle. In/Out: In
*@param bufsize	Pointer to the returned value.  In/Out: 	Out
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@see None
*
*/
CSAPI_RESULT CSVID_GetPFMBufferSize(CSVID_HANDLE handle, unsigned int *bufsize);

/*!
*@brief Write video ES data and current PTS to video decoder buffer.
*
*The input data must be 32 bytes align.
*@param handle  An opened video handle. In/Out: In
*@param src Video ES data buffer start address. In/Out: In
*@param size    Length of input data.   In/Out: In
*@param pts Current pts base on 90KHz clock.    In/Out: In
*@return CSAPI_SUCCEED  Success\n
*        CSAPI_FAILED   Failure
*@see None
*
*/
CSAPI_RESULT CSVID_WritePFMData(CSVID_HANDLE handle, unsigned char *src, int size, unsigned long long pts);

/*!
*@brief Write video ES data and current PTS to video decoder buffer.
*
*The input data must be 32 bytes align.
*@param handle  An opened video handle. In/Out: In
*@param src Video ES data buffer start address. In/Out: In
*@param size    Length of input data.   In/Out: In
*@param pts Current pts base on 90KHz clock.    In/Out: In
*@return CSAPI_SUCCEED  Success\n
*        CSAPI_FAILED   Failure
*@see None
*
*/
CSAPI_RESULT CSVID_WritePFMDataWithPTS(CSVID_HANDLE handle, unsigned char *src, int size, unsigned long long pts);

/*!
*@brief Write video ES data and current PTS to video decoder buffer. Do alignment to 512 bytes at the end of each frame.
*
*The input data must be 32 bytes align.
*@param handle  An opened video handle. In/Out: In
*@param src Video ES data buffer start address. In/Out: In
*@param size    Length of input data.   In/Out: In
*@param pts Current pts base on 90KHz clock.    In/Out: In
*@param end_of_frame End of frame.    In/Out: In
*@return CSAPI_SUCCEED  Success\n
*        CSAPI_FAILED   Failure
*@see None
*
*/
CSAPI_RESULT CSVID_WritePFMAlignedDataWithPTS(CSVID_HANDLE handle, unsigned char *src, int size, unsigned long long pts, int end_of_frame);

/*!
*@brief Write Yuv frame and PTS to video buffer. 
*
*@param handle  An opened video handle. In/Out: In
*@param yuv_frame YUV frame info. In/Out: In
*@param pts Current pts base on 90KHz clock.    In/Out: In
*@return CSAPI_SUCCEED  Success\n
*        CSAPI_FAILED   Failure
*@see None
*
*/
CSAPI_RESULT CSVID_WriteYuvFrameWithPTS(CSVID_HANDLE handle, CSVID_Yuv_Frame *yuv_frame, unsigned long long pts);

/*!
*@brief Read video ES data from video encoder buffer.
*
*@param handle      An opened video handle. In/Out: In
*@param recv_buf    receiving Video ES data buffer start address. In/Out: In/Out
*@param buf_size    Length of receiving buffer.   In/Out: In
*@param recv_size   received real data size .    In/Out: Out
*@return CSAPI_SUCCEED  Success\n
*        CSAPI_FAILED   Failure
*@Note:
*    size of recv_buf recommend more then 4M otherwise will be dorpping frame
*
*/
CSAPI_RESULT CSVID_ReadEncData(CSVID_HANDLE handle, unsigned char *recv_buf, int buf_size, unsigned int *recv_size);

/*!
*@brief Read video ES stream of part frame and pts from video encoder fifo.
*
*@param handle      An opened video handle. In/Out: In
*@param recv_buf    receiving Video ES a frame buffer start address. In/Out: In/Out
*@param buf_size    Length of receiving buffer.   In/Out: In
*@param recv_size   received real data size .     In/Out: Out
*@param pts         received pts of frame base on 90KHz clock.  In/Out: Out
*@param end_of_frame    Represents if the current chunk is the end of a frame.  In/Out: Out
*@return CSAPI_SUCCEED  Success\n
*        CSAPI_FAILED   Failure
*@Note:
*    size of recv_buf recommend more then 4M otherwise will be dorpping frame
*
*/
CSAPI_RESULT CSVID_ReadEncChunkWithPts(CSVID_HANDLE handle, unsigned char *recv_buf, int buf_size, unsigned int *recv_size, unsigned long long *pts, int *end_of_frame);

/*!
*@brief Read video ES stream of one slice and pts from video encoder fifo.
*
*@param handle      An opened video handle. In/Out: In
*@param recv_buf    receiving Video ES a frame buffer start address. In/Out: In/Out
*@param buf_size    Length of receiving buffer.   In/Out: In
*@param recv_size   received real data size .     In/Out: Out
*@param pts         received pts of frame base on 90KHz clock.  In/Out: Out
*@param end_of_frame    Represents if the current slice is the end of a frame.  In/Out: Out
*@return CSAPI_SUCCEED  Success\n
*        CSAPI_FAILED   Failure
*@Note:
*    size of recv_buf recommend more then 4M otherwise will be dorpping frame
*
*/
CSAPI_RESULT CSVID_ReadEncSliceWithPts(CSVID_HANDLE handle, unsigned char *recv_buf, int buf_size, unsigned int *recv_size, unsigned long long *pts, int *end_of_frame);

/*!
*@brief Read video ES stream of one frame and pts from video encoder fifo.
*
*@param handle      An opened video handle. In/Out: In
*@param recv_buf    receiving Video ES a frame buffer start address. In/Out: In/Out
*@param buf_size    Length of receiving buffer.   In/Out: In
*@param recv_size   received real data size .     In/Out: Out
*@param pts         received pts of frame base on 90KHz clock.  In/Out: Out
*@return CSAPI_SUCCEED  Success\n
*        CSAPI_FAILED   Failure
*@Note:
*    size of recv_buf recommend more then 4M otherwise will be dorpping frame
*
*/
CSAPI_RESULT CSVID_ReadEncOneFrameWithPTS(CSVID_HANDLE handle, unsigned char *recv_buf, unsigned int buf_size, unsigned int *recv_size, unsigned long long *pts);
/*!
*@brief Notify upper layers upon empty_threshold. Callback function will be invoked.
*
*@param handle	An opened video handle. In/Out: In
*@param empty_threshold	It's used to invoke callback function event. If remaining data in decoder input buffer is less than or equal to it, callback function event will be invoked.	In/Out: In
*@param call_back_function	Pointer to the callback function.	In/Out: In
*@param Event_enable	It controls event on/off.\n
*                        0: disable
*                        1: enable	In

*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@see None
*
*/
CSAPI_RESULT CSVID_SetNotifyPFMDataEmpty(CSVID_HANDLE handle, unsigned int *empty_threshold, void (*call_back_function) (CSVID_HANDLE), unsigned char event_enable);

/*!
*@brief Notify upper layers upon full_threshold. Callback function will be invoked.
*
*@param handle An opened video handle.  In/Out: In
*@param full_threshold Used to invoke callback function event. If data in decoder input buffer is less than or equal to it, callback function event will be invoked.  In/Out: In
*@param call_back_function Pointer to the callback function. In/Out: In
*@param event_enable It controls event on/off.

*       0: disable
*       1: enable
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@see
*
*/
CSAPI_RESULT CSVID_SetNotifyPFMDataFull(CSVID_HANDLE handle, unsigned int *full_threshold, void (*call_back_function) (CSVID_HANDLE), unsigned char event_enable);

/*!
*@brief When stop player this function will be called.
*
*@param An opened video handle.  In/Out: In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*
*/
CSAPI_RESULT CSVID_PFM_Write_Data_Finish(CSVID_HANDLE handle);
/*
Interface for appointing next video frame size is change. Size means resolution/width,height.
handle: video decoder handle. get it from CSVID_Open().
After calling this functon, you must call CSVID_WritePFMData()/CSVID_WritePFMDataWithPTS() to write 2KB zero data.
*/

/*!
*@brief Interface for appointing next video frame size is change. Size means resolution/width,height.
*
*@param handle: video decoder handle. get it from CSVID_Open().
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@note After calling this functon, you must call CSVID_WritePFMData()/CSVID_WritePFMDataWithPTS() to write 2KB zero data.
*@see
*
*/
CSAPI_RESULT CSVID_PFM_SetFrameSizeChangeFlag(CSVID_HANDLE handle);

/*!
*@brief Get current video buffer status
*
*@param handle A valid handle on success. In/Out: In
*@param buffer_status  Pointer of the status of the video buffer. In/Out: In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@see
*
*/
CSAPI_RESULT CSVID_GetBufferStatus(CSVID_HANDLE handle, CSVID_BUFFER_STATUS *buffer_status);

/*!
*@brief Get current video buffer directory status
*
*@param handle A valid handle on success. In/Out: In
*@param total_dir_num pointer of total dir number
*@param free_dir_num pointer of free dir number
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@see
*
*/
CSAPI_RESULT CSVID_GetDirNumber(CSVID_HANDLE handle, unsigned int *total_dir_num, unsigned int *free_dir_num);

/*!
*@brief Write yuv data to video buffer.
*
*@param handle An opened video handle
*@param yuv_buf Pointoer of Yuv data buffer
*@param yuv_w The width of yuv data frame
*@param yuv_h The height of yuv data frame
*@param framerate The data's frame rate
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@note  The yuv data from input device such as camera.
*@see
*
*/
CSAPI_RESULT CSVID_Write_YUV_Data(CSVID_HANDLE handle, unsigned int *yuv_buf, unsigned int yuv_w, unsigned int yuv_h, CSVID_FRAME_RATE framerate);

/*!
*@brief Define struct of video firmware version
*
*@see
*
*/
typedef struct tagVID_FIRMWARE_VERSION
{
	CSVID_STREAM_TYPE type; ///<Invalid
	unsigned int year;///< year
	unsigned int month;///<month
	unsigned int day;///<day
	unsigned int time;///<Invalid
}CSVID_Firmware_Version;

/*!
*@brief Get the firmware version number.
*
*@param video_firmware_version Firmware vertion including date and type. In/Out: Out
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@see
*
*/
CSAPI_RESULT CSVID_GetFirmware_Version(CSVID_Firmware_Version *video_firmware_version);

/*!
*@brief Define struct of VIB parameters
*@see
*
*/
typedef union
{
	struct {
		unsigned int para_enable:1;///<1:para enable. 0:para disable
		unsigned int work_mode:1;	///<0:Auto detect input video source format; 1: configure video format by src_width and src_height
		unsigned int yuv_order:2;	///<0: UYVY. 1: YUYV. 2: VYUY. 3: YVYU
		unsigned int src_height:12;	///<Video Source Height in pels
		unsigned int src_width:12;	///<Video Source Width in pels
		unsigned int src_format:1;	///<0: Interlaced. 1: Progressive
		unsigned int yuv_mode:1;	///<*0: YUV_420. 1: YUV_422
		unsigned int height_change_detetion:1;///<1: disable image active height of frame or field transition for video source format change detection. 0: enable image active height of frame or field transition for video source format change detection
		unsigned int width_change_detetion:1;	///<1: disable image active width transition for video source format change detection. 0: enable image height active width transition for video source format change detection
    	}bits;

	unsigned int val;
}CSVID_VIB_PARAMS;

/*!
*@brief Confiurate VIB.
*
*@param handle  video decoder handle. get it from CSVID_Open().
*@param para VIB parameters.
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@see
*
*/
CSAPI_RESULT CSVID_VIB_Config(CSVID_HANDLE handle, CSVID_VIB_PARAMS para);

/*
Interface for configurate the threshold of starting decode.
handle: video decoder handle. get it from CSVID_Open().
size_kbytes: unit is kbyte.
This function should be called before CSVID_Play().
*/

/*!
*@brief Set decoder stream frame size.
*
*@param handle  handle An opened video handle. In/Out: In
*@param size_kby teThe size of the decoder stream frame. Unit is kbyte. In/Out: In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@note This function should be called before CSVID_Play().
*@see
*
*/
CSAPI_RESULT CSVID_SetDecodeFrameSize(CSVID_HANDLE handle, unsigned int size_kbytes);

/*
Interface for output in decoding order, only valid for AVS/VC1/H264
handle: video decoder handle. get it from CSVID_Open().
flag:
0 - enable the decoder reorder;
1 - disable the decoder reorder;
Disable the decoder reorder means that decoder output the decoded pictures without reordering.
*/

/*!
*@brief Interface for output in decoding order, only valid for AVS/VC1/H264.
*
*@param handle An opened video handle. In/Out: In
*@param flag 1 - enable the decoder reorder; 0 - disable the decoder reorder; Disable the decoder reorder means that decoder output the decoded pictures without reordering.
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@see
*
*/
CSAPI_RESULT CSVID_SetDecoderReorder(CSVID_HANDLE handle, unsigned int flag);

/*
Interface for output without I-frame ready
handle: video decoder handle. get it from CSVID_Open().
flag:
0 - dont output the pictures which:
	a) the pictures decoded before the first I picture;
	b) the pictures displayed before the first I picture;
1 - output the pictures which before the first I picture (include decoded & display);
*/

/*!
*@brief Interface for output without I-frame ready.
*
*@param handle An opened video handle.  In/Out: In
*@param flag 0 - dont output the pictures which:
*       a) the pictures decoded before the first I picture;
*       b) the pictures displayed before the first I picture;\n
*1 - output the pictures which before the first I picture (include decoded & display);  In/Out: In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@see
*
*/
CSAPI_RESULT CSVID_SetOutputWithoutIFrame(CSVID_HANDLE handle, unsigned int flag);

/*
Interface for custom the DPB size for one decode channel.
handle: the handle must be NULL.
ByteSize: value of the DPB size for one decode channel.
Note: This function must be called before CSVID_Init().
We don't suggest using this function. If you want to call this function, please contact SDK or Firmware team to get a appropriate value of ByteSize.
*/

/*!
*@brief Interface for custom the DPB size for one decode channel.
*
*@param handle An opened video handle. this handle can be NULL. In/Out: In
*@param ByteSize Value of the DPB size for one decode channel. In/Out: In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@note This function must be called before CSVID_Init(). We don't suggest using this function. If you want to call this function, please contact SDK or Firmware team to get a appropriate value of ByteSize.
*@see
*
*/
CSAPI_RESULT CSVID_SetDPBSizeForOneChannel(CSVID_HANDLE handle, unsigned int ByteSize);

/*!
*@brief Define enum of Rotate type.
*
*/
typedef enum {
	ROTATE_0 = 0,
	ROTATE_90 = 1,
	ROTATE_180 = 2,
	ROTATE_270 = 3,
	ROTATE_INVALID
} CSVID_ROTATE_TYPE;

typedef enum {
    VID_MIRROR_NONE = 0,
    VID_MIRROR_VER,
    VID_MIRROR_HOR,
    VID_MIRROR_HOR_VER
} CSVID_MIRROR_TYPE;

/*
Interface for rotation function.
handle: video decoder handle. get it from CSVID_Open().
rotate: angle of rotate.
Note: This function must be called after CSVID_Open(). Only VID_DEV_0 support this function. And make sure the DPB size is equal or larger than 0x3b10000.
*/

/*!
*@brief Interface for rotation function.
*
*@param handle An opened video handle.  In/Out: In
*@param rotate Angle of rotate.  In/Out: In
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@note This function must be called before CSVID_Open(). Only VID_DEV_0 support this function. And make sure the DPB size is equal or larger than 0x3b10000.
*@see
*
*/
CSAPI_RESULT CSVID_SetRotate(CSVID_HANDLE handle, unsigned int enable, CSVID_ROTATE_TYPE rotate, CSVID_MIRROR_TYPE mirror);

/*
Interface for xxx
handle: video decoder handle. get it from CSVID_Open().
flag:
0 - search the next I frame, do not search the next sequence when error after the picture decode finish except of picture size changed;
1 - search the next sequence when error after the picture decode finish;
*/

/*!
*@brief Error handle when picture decode finish.
*
*@param handle Video decoder handle.
*@param flag 0 - search the next I frame, do not search the next sequence when error after the picture decode finish except of picture size changed;\n
*			1 - search the next sequence when error after the picture decode finish;
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@see
*
*/
CSAPI_RESULT CSVID_FixedSequence(CSVID_HANDLE handle, unsigned int flag);

/*
Interface for appointing next video frame is error.
handle: video decoder handle. get it from CSVID_Open().
This API must be called before CSVID_WritePFMDataWithPTS(). That means the next video frame which CSVID_WritePFMDataWithPTS() will write to decoder is a error frame.
Decoder don't decode this frame and search next sequence header.
*/

/*!
*@brief Interface for appointing next video frame is error.
*
*@param handle: video decoder handle. get it from CSVID_Open().
*@return
*@note This API must be called before CSVID_WritePFMDataWithPTS(). That means the next video frame which CSVID_WritePFMDataWithPTS() will write to decoder is a error frame.
		Decoder don't decode this frame and search next sequence header.
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@see
*
*/
CSAPI_RESULT CSVID_SetFrameErrorFlag(CSVID_HANDLE handle);

/*
Reload video firmware.
handle: no useful. Can be NULL.
This function will reload video firmware. when calling this function, all the setting is gone. You should set again.
*/

/*!
*@brief Reload video firmware.
*
*@param handle: No useful. Can be NULL.
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@note This function will reload video firmware. when calling this function, all the setting is gone. You should set again.
*@see
*
*/
CSAPI_RESULT CSVID_Reload_Firmware(CSVID_HANDLE handle);

/*
Set video A/V sync mode.
version: 0-for fixed frame rate. 1-for varibale frame rate.
Note: Please call this function between CSVID_SetStreamType() and CSVID_Play(). Default value is 0.
*/

/*!
*@brief Set video A/V sync mode.
*
*@param handle: video decoder handle. get it from CSVID_Open().
*@param version: 0-for fixed frame rate. 1-for varibale frame rate.
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@note Please call this function between CSVID_SetStreamType() and CSVID_Play(). Default value is 0.
*@see
*/
CSAPI_RESULT CSVID_SetAVSyncVersion(CSVID_HANDLE handle, unsigned int version);

/*
Set Video firmware work mode.
There are two mode. One is F2D mode which support only one video channel. Another is F2F mode which can support mutil-video channel.
F2F is default setting in video firmware internally. But this mode need more BUFFER than F2D and rarely used. So Apps can call this function to force
Video firmware to use F2D mode.
Note: This function should be called After CSVID_Init().
Attention: Currently the video firmware is default using F2D mode. Not support F2F mode. Don't call this fucntion anymore. for futher use.
*/

/*!
*@brief Set Video firmware work mode.
*
*@param handle: video decoder handle. get it from CSVID_Open().
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@note There are two mode. One is F2D mode which support only one video channel. Another is F2F mode which can support mutil-video channel.
*F2F is default setting in video firmware internally. But this mode need more BUFFER than F2D and rarely used. So Apps can call this function to force
*Video firmware to use F2D mode.\n
*This function should be called After CSVID_Init().\n
*Currently the video firmware is default using F2D mode. Not support F2F mode. Don't call this fucntion anymore. for futher use.
*@see
*
*/
CSAPI_RESULT CSVID_SetVideoF2DMode(CSVID_HANDLE handle);

/*
Force video frame type.
frametype: 0-auto detect; 1-progressive; 2-interlace; 3-use coda detected vaule.
*/

/*!
*@brief Force video frame type.
*
*@param handle: video decoder handle. get it from CSVID_Open().
*@param frametype: 0-auto detect; 1-progressive; 2-interlace; 3-use coda detected vaule.
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@see
*
*/
CSAPI_RESULT CSVID_SetVideoFrameType(CSVID_HANDLE handle, unsigned int frametype);

/*
Inform video firmware the current stream has audio or not. it will helpful for A/V sync.
hasaudio: 0-Has audio; 1-Only video
*/

/*!
*@brief Inform video firmware the current stream has audio or not. it will helpful for A/V sync.
*
*@param handle: video decoder handle. get it from CSVID_Open().
*@param hasaudio: 0-Has audio; 1-Only video
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@see
*
*/
CSAPI_RESULT CSVID_SetVideoWithAudio(CSVID_HANDLE handle, unsigned int hasaudio);

/*!
*@brief Set GOP (Group of Pictures) size in the middle of encoding.
*
*@param handle: video decoder handle. get it from CSVID_OpenEncoder().
*@param gop: GOP size
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@see
*
*/
CSAPI_RESULT CSVID_SetEncGop(CSVID_HANDLE handle, unsigned int gop);

/*!
*@brief Set intra QP in the middle of encoding.
*
*@param handle: video decoder handle. get it from CSVID_OpenEncoder().
*@param intraQp: QP for I frame
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@see
*
*/
CSAPI_RESULT CSVID_SetEncIntraQp(CSVID_HANDLE handle, unsigned int intraQp);

/*!
*@brief Set bitrate in the middle of encoding.
*
*@param handle: video decoder handle. get it from CSVID_OpenEncoder().
*@param bitrate: bit rate in unit of Kbps
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@see
*
*/
CSAPI_RESULT CSVID_SetEncBitRate(CSVID_HANDLE handle, unsigned int bitrate);

/*!
*@brief Set framerate in the middle of encoding.
*
*@param handle: video decoder handle. get it from CSVID_OpenEncoder().
*@param framerate: frame rate. See VID_FRAME_RATE() definition
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@see
*
*/
CSAPI_RESULT CSVID_SetEncFrameRate(CSVID_HANDLE handle, unsigned int framerate);

/*!
*@brief Set intra mb refresh number in the middle of encoding.
*
*@param handle: video decoder handle. get it from CSVID_OpenEncoder().
*@param refreshNum: intra mb refresh number.
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@see
*
*/
CSAPI_RESULT CSVID_SetEncIntraMbRefreshNum(CSVID_HANDLE handle, unsigned int refreshNum);

/*!
*@brief Set slice mode in the middle of encoding.
*
*@param handle: video decoder handle. get it from CSVID_OpenEncoder().
*@param slice: See EncSliceMode definition for details.
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@see
*
*/
CSAPI_RESULT CSVID_SetEncSliceMode(CSVID_HANDLE handle, EncSliceMode *slice);

/*!
*@brief Force to generate an I frame in the middle of encoding.
*
*@param handle: video decoder handle. get it from CSVID_OpenEncoder().
*@param
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@see
*
*/
CSAPI_RESULT CSVID_SetEncForceIFrame(CSVID_HANDLE handle);

/*!
*@brief Force to generate video header (SPS/PPS) in the middle of encoding.
*
*@param handle: video decoder handle. get it from CSVID_OpenEncoder().
*@param
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@see
*
*/
CSAPI_RESULT CSVID_SetEncGenHeader(CSVID_HANDLE handle);

/*!
*@brief Force to a Skip frame.
*
*@param handle: video decoder handle. get it from CSVID_OpenEncoder().
*@param encodeFrame: 1 = all MBs skipped frame will be forced; 0 = a frame will not be encoded (will be dropped)
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@see
*
*/
CSAPI_RESULT CSVID_SetForceSkipFrame(CSVID_HANDLE handle, unsigned int encodeFrame);

/*!
*@brief to enable or disable YUV interupt event from video MIPS
*
*@param1 handle: video decocder handle,get it from CSVID_Open(). In/Out: In
*@param2 enable: 1 enable yuv interupt, 0 disable yuv interrupt  In/Out: Out
*@return CSAPI_SUCCEED Succeed
*        CSAPI_FAILED  Failure
*/
CSAPI_RESULT CSVID_EnableYUVEvent(CSVID_HANDLE handle, unsigned int enable);
/*
*@brief Get yuv information from video MIPS
*
*@param1 handle: Video decoder handle, get it from CSVID_Open(). In/Out: In
*@param2 pYuvInfo: The yuv information, use this info to get real    In/Out: Out
*                  yuv data from video MIPS. Please refer to
*                  DumpYUVFrame() of csvid_test.c.
*
*@return CSAPI_SUCCEED Succeed
*        CSAPI_FAILED  Failure
*
* Note1: YUV format is YUV420
* Note2: After get one yuv frame, then must to release this one frame use CSVID_ReleaseYUVInfo() API
*
*/
CSAPI_RESULT CSVID_QueryYUVInfo(CSVID_HANDLE handle, VideoYUVInfoT *pYuvInfo);
/*
*@brief Release yuv information from video MIPS
*
*param1 handle: Video decoder handle, get it from CSVID_Open(). In/Out: In
*param2 pYuvInfo: The yuv information from CSVID_QueryYUVInfo(),    In/Out: In
*                 use this yuv info to release yuv information,
*                 so let's to read next yuv frame
*
*@return CSAPI_SUCCEED Succeed
*        CSAPI_FAILED  Failure
*/
CSAPI_RESULT CSVID_ReleaseYUVInfo(CSVID_HANDLE handle, VideoYUVInfoT *pYuvInfo);

/*!
*@brief Set video firmware work on low latency mode.
*
*@param1 handle: video decoder handle. get it from CSVID_Open().
*@param2 lowlatencymode: //default VID_LLD_DISABLE
                        VID_LLD_DISABLE            //disable lowlatencymode
                        VID_LLD_PARTIAL_FRAME_MODE //start display earlier. Only support H.264. NOTE:reorder will be off
                        VID_LLD_STREAM_LOW_DELAY_MODE = (1<<1) //start decoding earlier.
*@param3 LowLatencyMbRowPercent: //0~100 to represent 0 to full frame
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@note
*@see
*/
CSAPI_RESULT CSVID_SetLowLatencyMode(CSVID_HANDLE handle, CSVID_LOW_LATENCY_MODE lowlatencymode, unsigned int LowLatencyMbRowPercent);
/*!
*@brief Set video firmware encoder work on low latency mode.
*
*@param1 handle: video encoder handle. get it from CSVID_OpenEncoder().
*@param2 lowlatencymode: //default VID_LLD_DISABLE
                        VID_LLE_DISABLE            //disable lowlatencymode
                        VID_LLE_PARTIAL_FRAME_MODE
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@note
*@see
*/
CSAPI_RESULT CSVID_SetEncLowLatencyMode(CSVID_HANDLE handle, CSVID_ENC_LOW_LATENCY_MODE lowlatencymode);

/*!
*@brief Set video latency (Buffering) when audio reference is not available.
*
*@param1 handle: video decoder handle. get it from CSVID_Open().
*@param2 lowlatencymode: Value in msecs FROM 0 TO 4095 msecs.

*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@note
*@see
*/
CSAPI_RESULT CSVID_SetVideoLatency(CSVID_HANDLE handle, unsigned int VideoLantecy);

/*!
*@brief Enable HW Deintlace module.
*
*@param handle: video decoder handle. get it from CSVID_Open().
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@note Default setting is Enabled.
*@see
*/
CSAPI_RESULT CSVID_EnableDeintlace(CSVID_HANDLE handle);

/*!
*@brief Disable HW Deintlace module.
*
*@param handle: video decoder handle. get it from CSVID_Open().
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@note This function is used for Trick mode(FF) to avoid two pictures intlace togather.
*@see
*/
CSAPI_RESULT CSVID_DisableDeintlace(CSVID_HANDLE handle);

/*!
*@brief Video firmware will output a black frame after calling CSVID_Stop().
*
*@param handle: video decoder handle. get it from CSVID_Open().
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@note
*@see
*/
CSAPI_RESULT CSVID_ShowBlackScreenWhenStop(CSVID_HANDLE handle);

/*!
*@brief Video firmware will keep the last frame after calling CSVID_Stop().
*
*@param handle: video decoder handle. get it from CSVID_Open().
*@return CSAPI_SUCCEED	Success\n
*        CSAPI_FAILED	Failure
*@note This function is defaule mode if not call CSVID_ShowBlackScreenWhenStop() .
*@see
*/
CSAPI_RESULT CSVID_KeepLastFrameWhenStop(CSVID_HANDLE handle);

/**
*Video firmware will discard the frames which pts less than mark_in_pts and 
*display from the frame with mark_in_pts.
*
*@param handle: video decoder handle. get it from CSVID_Open().
*       pts: the pts of first frame to display.
*@return CSAPI_SUCCEED	Success\n
*       CSAPI_FAILED	Failure
*@note This function should be called before CSVID_Play .
*/
CSAPI_RESULT CSVID_SetStartPTS(CSVID_HANDLE handle, unsigned long long pts);

/**
*Get frame count info like skipped/repeated/error/Iframe
*
*@param handle: video decoder handle. get it from CSVID_Open().
*       pInfo: pointer to the info
*@return CSAPI_SUCCEED	Success\n
*       CSAPI_FAILED	Failure
*/
CSAPI_RESULT CSVID_GetDecodingInfo(CSVID_HANDLE handle,CSVID_DecodingInfo *pInfo);

#ifdef __cplusplus
}
#endif

#endif

