/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Cavium, Inc.
 Cavium, Inc. reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium, Inc.

 ALL RIGHTS RESERVED.

 Copyright 2011-2012, Cavium, Inc.
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */

/**
 * @file cnc_appmgr.h
 * @ingroup appframework
 * The CNCAPPMGR library is a small, efficient application controller based on GUI state machines.
 */
#ifndef __CNC_APPMGR_H__
#define __CNC_APPMGR_H__

#include "cnc_util.h"
#include "cnc_osd.h"
#include "cnc_input.h"
#include "cnc_input_keys.h"


#ifdef __cplusplus
extern "C" {
#endif

/// Defined The Desktop Window Handle to NULL
#define DESKTOP			            NULL
/// Maximum UI Timer Index
#define MAX_UI_TIME                 20
/// UI Timer Status Descriptor
#define UI_TIME_IDLE 	            0
#define UI_TIME_START	            1
#define UI_TIME_PAUSE	            2
#define UI_TIME_INUSE	            3
/// UI Loop Time out
#define WINDOW_SLEEP                (10*1000)    //In Ms.
/// UI Base Message ID Define
#define MSG_NULL   		            (0xffff)
#define MSG_CREATE       	        (0x0080)
#define MSG_PAINT          	        (0x0081)
#define MSG_CLOSE          	        (0x0082)
#define MSG_KEYDOWN   	            (0x0083)
#define MSG_TIME                    (0x00840)
/// UI User Message ID Define
#define MSG_USER   	                (0xff00)
#define MSG_AV_PLAY		            (MSG_USER + 1)
#define MSG_RESERVED		        (MSG_USER + 2)
#define MSG_OTAUPDATE               (MSG_USER + 3)
#define MSG_VOD_DRAWPIC             (MSG_USER + 4)
#define MSG_CA_AUTHO_FAIL	        (MSG_USER + 5)
#define MSG_INSUFFICIENT_BALLANCE   (MSG_USER + 6)
#define MSG_EXTERNWORKTIME          (MSG_USER + 7)
#define MSG_EMAILBOX_FULL           (MSG_USER + 8)
#define MSG_NEW_NEWMAIL             (MSG_USER + 9)
#define MSG_NEW_MSG                 (MSG_USER + 10)
#define MSG_AGE_TOO_LITTLE          (MSG_USER + 11)

#define MSG_MEDIA_EVENT             (MSG_USER + 20)
#define MSG_OTT_PLAY_ERROR          (MSG_USER + 21)

#define MSG_REMOTEIP_PIN            (MSG_USER + 31)
#define MSG_REMOTEIP_PAIRED         (MSG_USER + 32)
#define MSG_REMOTEIP_CONNECTED      (MSG_USER + 33)
#define MSG_REMOTEIP_DISCONNECTED   (MSG_USER + 34)

#define MSG_DLNA_EVENT              (MSG_USER + 40)

#define MSG_NET_CONNECTING          (MSG_USER + 200)
#define MSG_NET_CONNECTED           (MSG_USER + 201)
#define MSG_NET_SCAN_RESULTS		(MSG_USER + 202)
#define MSG_NET_CONNECTION_FAILED   (MSG_USER + 203)
#define MSG_NET_WFD_READY           (MSG_USER + 204)
#define MSG_NET_WFD_WAIT_FOR_PBC    (MSG_USER + 205)
#define MSG_NET_WFD_DISPLAY_PIN     (MSG_USER + 206)
#define MSG_NET_WFD_CONNECTING      (MSG_USER + 207)
#define MSG_NET_WFD_CONNECTED       (MSG_USER + 208)
#define MSG_NET_WFD_DISCONNECTED    (MSG_USER + 209)
#define MSG_NET_WFD_CONNECT_FAILED  (MSG_USER + 210)
	
#define MSG_AIRPLAY_CLEAR			(MSG_USER + 300)
#define MSG_AIRPLAY_DRAW			(MSG_USER + 301)


/// UI Message Queue Handle
    typedef unsigned int cncAppMessageQueueHandle_t;

    typedef struct _cncAppTime_t
    {
        char time_flag; ///< APP Timer status: 0 = idle;1 = start;2 = pause;3 = In use
        int time_count; ///< Timer Tick Count.
        int time_startTick;///< Timer Start Tick.
    } cncAppTime_t;

    typedef struct _cncAppContext_t
    {
        int x;  ///< X position
        int y;  ///< Y position
        int w;  ///< Windows wide
        int h;   ///< Windows heigh
        int (*windows_cb)(void *mW, int message, int wparam, int lparam);   ///< Windows MSG Callback

        cncAppMessageQueueHandle_t MsgQueue_handle; ///< Windows MsgQueue Handle

        char alive_flag; ///< Windows Alive Flag:true = alive; false = need to be destroy

        cncAppTime_t ui_time[MAX_UI_TIME];///< Windows Timer
        char ui_time_index; ///< Windows Timer Sum.

        // Add by caowenjia, for backup host window
        cncOSDBitmap_t * bmp_backup_data;///< Windows Area Backup

        struct _cncAppContext_t * mW_Host;///< Host Windows
    } cncAppContext_t;

    typedef struct _cncAppMessageQueue_t
    {
        int pRead;///< Read point
        int pWrite;///< Write point
        unsigned int count;///< Msg count

        unsigned int MsgSize;///< Msg memory size
        unsigned int maxNum;///< Maximum Num of Msg, maxNum = BufSize / MsgSize

        unsigned char *pBuffer;///< Buffer
        unsigned int BufSize;///< Buffer Size

        pthread_mutex_t msgQueue_Mutex; ///< Mutex
    } cncAppMessageQueue_t;

    typedef struct _cncAppMessage_t
    {
        unsigned int message;///< Message ID
        unsigned int wparam;///< Param 1
        unsigned int lparam;///< Param 2
    } cncAppMessage_t;


    /**
     * Message Queue Create Function.
     * @param handle Message Queue Handle
     * @param MsgSize Message memory size
     * @param pBuf Buffer Point
     * @param BufSize Buffer Size
     * @return if successful returns 0, error returns: -1
     */
    int CNC_APP_MessageQueueCreate(cncAppMessageQueueHandle_t *handle, unsigned int MsgSize, unsigned char *pBuf, unsigned int BufSize);
    /**
     * Message Queue Send Function.
     * @param handle Message Queue Handle
     * @param pMsgData Message Data need to been send.
     * @return if successful returns 0, error returns: -1
     */
    int CNC_APP_MessageQueueSend(cncAppMessageQueueHandle_t handle, void *pMsgData);
    /**
     * Message Queue Receive Function.
     * @param handle Message Queue Handle
     * @param pMsgData Message Data need to been Received.
     * @return if successful returns 0, error returns: -1
     */
    int CNC_APP_MessageQueueReceive(cncAppMessageQueueHandle_t handle, void *pMsgData);
    /**
     * Message Queue Free Function.
     * @param handle Message Queue Handle
     * @return if successful returns 0, error returns: -1
     */
    int CNC_APP_MessageQueueFree(cncAppMessageQueueHandle_t handle);
    /**
     * Message Queue Clean Function.
     * @return if successful returns 0, error returns: -1
     */
    int CNC_APP_MessageQueueClean(void);
    /**
     * UI Omit Key Function
     * @param enable Enable Flag
     * @return void
     */
    void CNC_APP_KeyOmit(const unsigned char enable);
    /**
     * UI Send Key Function
     * @param key Key Value
     * @return void
     */
    void CNC_APP_KeySend(unsigned int key, cncInputKeyEvent_t event);

    /**
     * Send Message To Current Active Windows
     * @param Msg Msg ID
     * @param wparam Param1
     * @param lparam Param2
     * @return void
     */
    void CNC_APP_MessageSendToActiveWindows(unsigned int Msg, unsigned int wparam, unsigned int lparam);
    /**
     * Send Message To Appoint Windows
     * @param mW Windows Handle
     * @param Msg Msg ID
     * @param wparam Param1
     * @param lparam Param2
     * @return void
     */
    void CNC_APP_MessageSend(cncAppContext_t * mW, unsigned int Msg, unsigned int wparam, unsigned int lparam);
    /**
     * Get Message From Windows Message Queue
     * @param mW Windows Handle
     * @return if successful returns 1, error returns: 0
     */
    int CNC_APP_MessageGet(cncAppContext_t * mW);
    /**
     * Parse Message (automatic launch window callback function)
     * @param mW Windows Handle
     * @return if successful returns 1, error returns: 0
     */
    void CNC_APP_ParseMessage(cncAppContext_t * mW);

    /**
     * Create APP Timer.
     * @param mW Windows Handle
     * @return Timer ID, if error returns: 0xff
     */
    unsigned char CNC_APP_TimerCreate(cncAppContext_t * mW);
    /**
     * Parse Timer
     * @param mW Windows Handle
     * @param Time_ID Timer ID
     * @return if successful returns 1, error returns:  -1
     */
    int CNC_APP_TimerPause(cncAppContext_t * mW, unsigned char Time_ID);
    /**
     * Stop Timer
     * @param mW Windows Handle
     * @param Time_ID Timer ID
     * @return if successful returns 1, error returns:  -1
     */
    int CNC_APP_TimerStop(cncAppContext_t * mW, unsigned char Time_ID);
    /**
     * Start Timer (First call this function it will start Timer Tick , Repeat is reset Timer Tick)
     * @param mW Windows Handle
     * @param mSec Timer delay times, by millisecond
     * @param Time_ID Timer ID
     * @return if successful returns 1, error returns:  -1
     */
    int CNC_APP_TimerStart(cncAppContext_t * mW, int mSec, unsigned char Time_ID);

    /**
     * App Reset Function, should been called in the end of window callback
     * @param mW Windows Handle
     * @return if successful returns 1, error returns:  -1
     */
    int CNC_APP_Reset(cncAppContext_t * mW);
    /**
     * Get Current Active Window Handle
     * @return mW Windows Handle
     */
    cncAppContext_t *CNC_APP_GetActiveWindow(void);
    /**
     * Init Windows, will install callback, create messagequeue, reset timer...
     * @param mW Windows Handle
     * @return void
     */
    void CNC_APP_Init(cncAppContext_t * mW);
    /**
     * Destroy Windows
     * @param mW Windows Handle
     * @return void
     */
    void CNC_APP_Destroy(cncAppContext_t * mW);
    /**
     * Cleanup Windows, free all resource
     * @param mW Windows Handle
     * @param Clear_flag Clear Flag: 1=Clear Whole Area
     * @return void
     */
    void CNC_APP_Cleanup(cncAppContext_t * mW, char Clear_flag);

#ifdef HAVE_STANDBY
    /**
     * Send Standby Key to Standby Control
     * @param key Key Value
     * @param status Output Current status
     * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
     */
    Result_Type_e CNC_Standby_Key(CNC_UINT32 key, CNC_DEV_STATUS_e *status);
    /**
     * Enter Standby
     * @param params Unused Params
     * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
     */
    Result_Type_e CNC_Standby_Enter(void *params);
    /**
     * Exit Standby
     * @param params Unused Params
     * @return if successful returns RESULT_OK, error returns: RESULT_FAIL
     */
    Result_Type_e CNC_Standby_Exit(void *params);
    /**
     * Get Standby Status
     * @return Standby Status
     */
    CNC_DEV_STATUS_e CNC_Standby_Status( void );
#endif //HAVE_STANDBY

#ifdef __cplusplus
}
#endif

#endif //__CNC_APPMGR_H__