/*
# COPYRIGHT NOTICE
#
#***************************************************************************
The following software code is the property of Cavium, Inc.
Cavium, Inc. reserves all rights in this software, including but not limited
to copyright and shall be vested with all rights incidental thereto.
Rights granted to you by or under any license shall be limited to those
explicitly granted. You may not record, store, reproduce, reverse engineer,
decompile, inter-operate and/or transmit this software in any medium,
electronic or otherwise, without the express written consent of Cavium, Inc.
ALL RIGHTS RESERVED.
Copyright 2012-2013, Cavium, Inc.
#***************************************************************************
#
# END OF COPYRIGHT NOTICE
#
*/
#ifndef __CNC_BT_H_
#define __CNC_BT_H_

typedef enum {
    CNC_BT_TYPE_INPUT = 0x0001,
    CNC_BT_TYPE_OTHER = 0x0002,
    CNC_BT_TYPE_ALL = 0xFFFF
} cncBtDeviceType_t;

typedef struct {
    cncBtDeviceType_t type;
    char address[32];
    char name[64];
    int paired;
} cncBtDevice_t;

typedef void (*cncBtCallbackDevice_t)(cncBtDevice_t *);

/** 
 * Start bluetooth discovering. Blocking function. The dicovering can
 * last for several seconds. The callback function will be called for
 * every device found with type in the deviceMask
 * 
 * @param callback Callback that will be called for every device discovered,
 *                 can be called several times for the same device if new 
 *                 information is available (like the name of the device)
 * @param deviceMask Only notify of devices of this kind
 * @return 0 if successful. -1 if there were some problem
 */
int CNC_BT_StartDiscovering(cncBtCallbackDevice_t callback,
                             cncBtDeviceType_t deviceMask);

/* void CNC_BT_EndDiscovery(void); */


/** 
 * Get the information of the known devices by the bluetooth adapter
 * 
 * @param devs Array where the devices will be copied
 * @param ndev The size of devs, in number of elements
 * 
 * @return The number of devices, if the returned value is bigger than
 * ndev then only the first ndev devices are copied to devs
 */
int CNC_BT_GetDevices(cncBtDevice_t *devs, unsigned int ndev);


/** 
 * Pair a bluetooth device with the given pin. It also trust the
 * device so it connects automatically and connects it.
 * 
 * @param dev Device to be paired
 * @param pin Pin to be introduced in the device.
 * 
 * @return 0 if paired. -1 if there were some problem
 */
int CNC_BT_PairDevice(cncBtDevice_t *dev, const char *pin);

/* void CNC_BT_CancelPairDevice(cncBtDevice_t *dev); */

/** 
 * Remove a device. It will disconnect the device and forget it. A
 * removed device will not connect automatically anymore
 * 
 * @param dev Device to be removed
 * 
 * @return 0 if removed. -1 if there were some problem
 */
int CNC_BT_RemoveDevice(cncBtDevice_t *dev);

#endif
