/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Monta-Vista.
 Monta-Vista reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Monta-Vista

 ALL RIGHTS RESERVED.

 Copyright 2011-2012, Monta-Vista
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */

/**
 * Module implementing broadcast TV stacks (DVB-T, ISDB-T, DVB-C), PVR/Timeshift, Channel Management and supporting functions.
 * @file    cnc_btv.h
 * @ingroup btv
 */

#ifndef CNC_BTV_H_
#define CNC_BTV_H_

#include "cnc_common_typedef.h"
#include "cnc_system.h"
#include "cnc_btv/cnc_tuner.h"
#include "cnc_sqlite.h"

#ifdef __cplusplus
extern "C" {
#endif

/*
 * Base Broadcast Digital TV Standard
 */
typedef enum
{
    CNC_BTV_DVBT,
    CNC_BTV_DVBC,
    CNC_BTV_DVBC_GX,
    CNC_BTV_DVBS,
    CNC_BTV_DVBS2,
    CNC_BTV_ISDBT,
    CNC_BTV_Standard_MAX
} cncBTVStandardProtocol_t;
/**
 * Initialize the BTV work Standard.
 *
 * This function should be called before initialization of the BTV modul.
 * It will set the Base Broadcast Digital TV Standard, then SI , BTVSearch ... can work in the right way;
 * It only worked in the first call, do not change this in Run time.
 * If not call this function, default value is CNC_BTV_DVBT.
 * @return NULL
 */
void CNC_BTV_InitStandard(cncBTVStandardProtocol_t type);
/**
 * Get the Base Broadcast Digital TV Standard.
 *
 * This function will return the Base Broadcast Digital TV Standard currently used.
 * Can be called anytime.
 * @return CNC_BTV_Standard_e
 */
cncBTVStandardProtocol_t CNC_BTV_GetStandard(void);


/*
 * SI Module
 */
typedef CNC_SINT32  CNC_SI_Handle_t;
typedef void (*CNC_SI_DemuxCallBack)(CNC_UINT32 Channel_Handle, CNC_UINT32 Filter_Handle, void* userparam, CNC_BOOL timedout, CNC_UINT8 const* data, CNC_UINT32 size);

#ifndef GX_DB_SEARCH
#define TableUpdate
#endif

#define	CNC_DB_MAX_SERVICE_NAME_LENGTH		    20	/*The maximum length of the program name*/
#define	CNC_DB_MAX_MULTI_SERVICE_NAME_NUM	    4	       /*Maximum multi-language program name*/
#define	CNC_DB_NB_CHAR_DVB_TRIGRAM			    4        /*Three-character string length*/
#define	CNC_DB_NB_CHAR_ENTIRE_NAME			    32	/*Three-character string corresponding to the actual length of the string*/




CNC_UINT8 CNC_TV_GetAudioStreamNumber(void);
CNC_UINT16 CNC_TV_GetAudioStreamPid(int index);
CNC_UINT16 CNC_TV_GetAudioStreamESType(int index);
char * CNC_TV_GetAudioStreamLanguage(int index);
void CNC_TV_ScrambleSet(CSDEMUX_AV_SCRAMBLE_STATUS status);
CSDEMUX_AV_SCRAMBLE_STATUS CNC_TV_ScrambleGet(void);

/* Program Module
 *
 */
#define TELTEXT_USED
#define SUBTITTLE_USED

typedef enum
{
    CNC_Program_DB_HANDLE_TS = 0,
    CNC_Program_DB_HANDLE_TV,
    CNC_Program_DB_HANDLE_RADIO,
#ifdef TELTEXT_USED
    CNC_Program_DB_HANDLE_TELTEXT,
#endif
#ifdef SUBTITTLE_USED
    CNC_Program_DB_HANDLE_SUBTITLE,
#endif
    CNC_Program_DB_HANDLE_REFERENCE,
    CNC_Program_DB_HANDLE_TIMESHIFT,
    CNC_Program_DB_HANDLE_MOSAIC,
    CNC_Program_DB_HANDLE_FMRADIO,
    CNC_Program_DB_HANDLE_BROADCAST,
    CNC_Program_DB_HANDLE_MHP,
    CNC_Program_DB_HANDLE_BAT,
    CNC_Program_DB_HANDLE_GXPRO,
    CNC_Program_DB_HANDLE_DVBCTV,
    CNC_Program_DB_HANDLE_DVBCRADIO,
    CNC_Program_DB_HANDLE_DVBCTS,
#ifdef MANU_DFACTUR_TEST
      CNC_PROGRAM_DB_HANDLE_PERMANENT_TS,
      CNC_PROGRAM_DB_HANDLE_PERMANENT_TV,
      CNC_PROGRAM_DB_HANDLE_PERMANENT_DVBCTS,
      CNC_PROGRAM_DB_HANDLE_PERMANENT_DVBCTV,
#endif
    CNC_PROGRAM_DB_HANDLE_CC,
    CNC_PROGRAM_DB_HANDLE_OCS,
    CNC_PROGRAM_DB_HANDLE_HLS,
    CNC_Program_DB_HANDLE_NUM
} cncBTVProgramDBHandle_t;

typedef enum
{
    CNC_Program_LIST_TV,
    CNC_Program_LIST_RADIO,
    CNC_Program_LIST_FAV,

    CNC_Program_LIST_NUM
} cncBTVProgamListType_t;

#define DB_NAME_MAX_LENGTH  32
#define DB_MAX_AUDIO_NUM      10
#define DB_MAX_AUDIO_LANGUAGE_NUM      10
#define DB_MAX_AUDIO_LANGUAGE_LENGTH      4
#define DB_MAX_BOUQUET_LANGNUM    5
#define Pmt_Es_Max 8
#define DB_HLS_URL_LENGTH  256
#define CNC_PROGRAM_INVALID_ONID 0xFFFF
#define CNC_PROGRAM_INVALID_TSID 0xFFFF
#define CNC_PROGRAM_INVALID_SID 0xFFFF
#define CNC_PROGRAM_INVALID_PID 0xFFFF
#define CNC_PROGRAM_INVALID_GPIOFLAG 0xFF
#define CNC_PROGRAM_DEFAULT_VOLUME 16

typedef enum
{
    CNC_SERVICE_TYPE_TV = 0,
    CNC_SERVICE_TYPE_RADIO,
#ifdef TELTEXT_USED
    CNC_SERVICE_TYPE_TELETEXT,
#endif
#ifdef SUBTITTLE_USED
    CNC_SERVICE_TYPE_SUBTITLE,
#endif
    CNC_SERVICE_TYPE_REFERENCE,
    CNC_SERVICE_TYPE_TIMESHIFT,
    CNC_SERVICE_TYPE_MOSAIC,
    CNC_SERVICE_TYPE_FM_RADIO,
    CNC_SERVICE_TYPE_DATA_BROADCAST,
    CNC_SERVICE_TYPE_MHP,
    CNC_SERVICE_TYPE_GXSERVICE,//guangxi service_type

    CNC_SERVICE_TYPE_ISDB_TV,
#ifdef MANU_DFACTUR_TEST
    CNC_SERVICE_TYPE_ISDB_PERMANENT_TV,
#endif
    CNC_SERVICE_TYPE_ISDB_RADIO,

    CNC_SERVICE_TYPE_DVBC_TV,
#ifdef MANU_DFACTUR_TEST
    CNC_SERVICE_TYPE_DVBC_PERMANENT_TV,
#endif
    CNC_SERVICE_TYPE_DVBC_RADIO,

    CNC_SERVICE_TYPE_DVBT_TV,
    CNC_SERVICE_TYPE_DVBT_RADIO,

    CNC_SERVICE_TYPE_ALL,
    CNC_SERVICE_TYPE_FAV,
    CNC_SERVICE_TYPE_CC,
    CNC_SERVICE_TYPE_OCS,
    CNC_SERVICE_TYPE_HLS,
    CNC_SERVICE_TYPE_UNKNOWN
} cncBTVServiceType_t;

#ifdef MANU_DFACTUR_TEST
typedef enum
{
      Atv_Type_C=0,
      Atv_Type_T
}Atv_Type_e;
#endif
/*subtitle Text*/
#ifdef SUBTITTLE_USED
#define MAX_SUBTEXLTEXT   10

typedef  enum
{
    TTX_SUBTITLE = 0,
    DVB_SUBTITLE,
    TELETEXT
} cncBTVSubtitleType_t;

typedef struct
{
    CNC_UINT16 CompositionPageID;
    CNC_UINT16 AncillaryPageID;
} cncBTVSubtitlePage_t;

typedef struct
{
    CNC_UINT32 frequency;
    CNC_UINT16  Atv_Num;

    CNC_UINT8 lock_flag;
    CNC_UINT8 favor_flag;
#ifdef MANU_DFACTUR_TEST
	CNC_BOOL pre_saved;
#endif
} cncBTVAnalogInfo_t;

typedef struct
{
    CNC_UINT8 MagazineID;
    CNC_UINT8 PageID;
} cncBTVTeletext_t;

typedef struct
{
    cncBTVSubtitleType_t Type;
    char Language[DB_MAX_AUDIO_LANGUAGE_LENGTH];
    CNC_UINT16 Pid;
    CNC_UINT8 IsHardHearing;    // 1-is hard of hearing, 0-is not
    union
    {
        cncBTVSubtitlePage_t Subtitle;
        cncBTVTeletext_t Teletext;
    } Descriptor;
} cncBTVPesStreamInfo_t;

typedef struct
{
    CNC_UINT16   service_id;						/*program ID*/
    CNC_UINT8 Number; //<=MAX_SUBTEXLTEXT
    cncBTVPesStreamInfo_t Stream[MAX_SUBTEXLTEXT];
    CNC_UINT8 current_sub_index;
    CNC_UINT8 current_teltext_index;
} cncBTVSubtitleTextStream_t;
#endif

typedef struct
{
    CNC_UINT16   service_id;						/*program ID*/
    CNC_UINT16   pid;
    CNC_UINT16   com_id;
    CNC_UINT8    com_tag;
    CNC_UINT8    DMF;
    CNC_UINT8    Timing;
} cncBTVClosedCaptionStream_t;       //closed caption

typedef struct
{
    CNC_UINT16   service_id;						/*program ID*/
    CNC_UINT8 stream_type;
    CNC_UINT16 esnum;
    CNC_UINT16 espid[Pmt_Es_Max];
} cncBTVPMTElementaryStreamOC_t;

#ifdef MANU_DFACTUR_TEST
typedef struct
{
      CNC_UINT32 frequency;
      CNC_UINT8 type;
}Find_Condition;
#endif
typedef struct _cncBTVServiceInfo_t
{
    CNC_UINT16      channel_number;
    CNC_UINT16      service_id;						/*program ID*/
    CNC_UINT16      stream_id;	  					/*stream ID*/
    CNC_UINT16      network_id;
    CNC_UINT16      original_network_id;

    CNC_UINT16      logic_number;
    cncBTVServiceType_t service_type;
    char        prog_name[DB_NAME_MAX_LENGTH];  /* service name */
    char        prov_name[DB_NAME_MAX_LENGTH];  /* provider name */

    CNC_UINT16      pcr_pid;							  	  /*PCR PID*/
    CNC_UINT16      pmt_pid;							  	  /*PMT PID*/

    CNC_UINT16      video_pid;								  /*VideoPID*/
    CNC_UINT8        video_tag;
    CNC_UINT8        volume;
    CNC_UINT8        audio_num;
    CNC_UINT16      audio_pid[DB_MAX_AUDIO_NUM];			  /*AudioPID*/
    CNC_UINT8        audio_tag[DB_MAX_AUDIO_NUM];
    CSAUD_STREAM_TYPE audio_stream_type[DB_MAX_AUDIO_NUM];
    char      audio_language[DB_MAX_AUDIO_LANGUAGE_NUM][DB_MAX_AUDIO_LANGUAGE_LENGTH];
    CNC_UINT8        audio_language_num;
    CNC_UINT8        apid_index[DB_MAX_AUDIO_NUM];
#ifdef SUBTITTLE_USED
    cncBTVSubtitleTextStream_t Rdata;
#endif

    cncBTVClosedCaptionStream_t cc;        /*Closed Caption--stream type == 0x06*/

    cncBTVPMTElementaryStreamOC_t OcService;

    CSVID_STREAM_TYPE video_stream_type;
    CNC_UINT32     audio_volume;
    CNC_UINT32     audiochannle;
    CNC_UINT16     refserviceId;

    CNC_UINT8       scramble_flag;                                                 /*0  no scrambling, 1  scrambling*/
    CNC_UINT8       lock_flag;	                                                    /* 0 no lock, 1  locking*/
    CNC_UINT8       hide_flag;                                                         /* 0 no hide , 1 hiding */
    CNC_UINT8       favor_flag;                                                       /*0 represents a non-favorite, 1 favorite*/
    CNC_UINT8       tuner_type;                                                      /*tuner type 0-dvbc, 1-ISDB, */
    cncBTVServiceType_t       service_type_ext;                               /*Type of program expansion dvbc isdbt*/
    CNC_UINT32      atv_frequency;                                                /*save atv frequency*/
    CNC_UINT8       gpio15_input;
    CNC_UINT8       ginga_flag;                                                     /*ginga_flag: 0 1*/

    CNC_UINT8      pmt_rating;                     /*For Patagonia (parental control), from ARIB standard. add by wenjiacao*/

    CNC_UINT8      reserved2;
#ifdef MANU_DFACTUR_TEST
    CNC_BOOL     pre_saved;
#endif
    CNC_UINT8      sd_hd_flag;
    CNC_UINT16    relation_logic_num;
} cncBTVServiceInfo_t;


typedef struct _cncBTVTransportStreamInfo_t
{
    CNC_UINT16   ts_number;
    CNC_UINT16   network_id;					/*network ID*/
    CNC_UINT16   orig_network_id;
    CNC_UINT16   stream_id;	  					/*stream ID*/
    CNC_UINT16   service_num;
    CNC_UINT16   version_number;				/*version nember*/
    CNC_UINT16   load_version;					/*loader version*/
    CNC_UINT32   hardware_version;
    CNC_UINT32   software_version;
    CNC_UINT32   serial_number_start;
    CNC_UINT32   serial_number_end;
    CNC_UINT16   download_pid;
    CNC_UINT8     network_name[DB_NAME_MAX_LENGTH];			/*network name */
    cncTuner_t tune_param;
    CNC_UINT32   isforce;						/*true is force, false is normal*/
#ifdef MANU_DFACTUR_TEST
	  CNC_BOOL    pre_saved;
#endif
} cncBTVTransportStreamInfo_t;

typedef struct _cncBTVBouquetInfo_t
{
    CNC_UINT16   bouquet_id;       									/*bouquet ID*/
    CNC_UINT16   service_id;
    CNC_UINT8     BouquetName[DB_NAME_MAX_LENGTH];						/*bouquet name*/
    CNC_UINT8	    MultilingualLanguage_num;					/*Total number of multilingual language business group*/
    //U8     MultilingualLanguage[DB_MAX_BOUQUET_LANGNUM][4];						/*Multilingual business group language description */
    //U8     MultilingualBouquetName[DB_MAX_BOUQUET_LANGNUM][DB_NAME_MAX_LENGTH];		/*Multilingual business group name*/
} cncBTVBouquetInfo_t;

typedef struct _cncBTVHLSChannel_t
{
    CNC_UINT8      channel_num;
    CNC_UINT8       favor_flag;                                       //0: cancel favor 1:mark favor
    char    url[DB_HLS_URL_LENGTH];
}cncBTVHLSChannel_t;

typedef void *cncBTVServiceHandle_t;

typedef enum
{
      /* PLAYING MESSAGE */
      CNC_AVPLAY_MSG_PLAY_OK = 0,
      CNC_AVPLAY_MSG_PLAY_AVSYNC,   /* audio&video sync succeed notify , then the video will be displayed */
      CNC_AVPLAY_MSG_PLAY_FAIL,
      CNC_AVPLAY_MSG_SIGN_LOCK,
      CNC_AVPLAY_MSG_SIGN_UNLOCK,
      CNC_AVPLAY_MSG_PARENTAL_LOCK,   /* locked by parental control rating */
      CNC_AVPLAY_MSG_ADULT_LOCK,   /* locked by adult rating(-18) */
      CNC_AVPLAY_MSG_SERVICE_LOCKED,  /* locked by user control */
      CNC_AVPLAY_MSG_SERVICE_SCMBLE,  /* scramble program can't play */
      CNC_AVPLAY_MSG_PARENTAL_UNLOCK, /* unlocked by parental control rating */

      CNC_AVPLAY_MSG_NUM
}cncBTVPlayMessageStatus_t;

typedef enum
{
      CNC_TV_CALLBACK_PLAYING = 0,
      CNC_TV_CALLBACK_STOP,
      CNC_TV_CALLBACK_CHANNELSWITCH,
      CNC_TV_CALLBACK_CHANNELSWITCHED,
      CNC_TV_CALLBACK_FREQUENCYCHANGED,

      CNC_TV_CALLBACK_NUM
}cncBTVCallbackType_t;

typedef struct _cncBTVPlayMessage_t
{
      cncAVDevice_t Dev;
      cncBTVPlayMessageStatus_t MsgType;
      CNC_UINT32 wParam;
      CNC_UINT32 lParam;
}cncBTVPlayMessage_t;

typedef void (*cncBTVPlayCallback_t)(cncAVDevice_t dev, void *param);

/**
 * Initialize the BTV Modue
 * @return RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_TV_Init(void);

/**
 * De-initialize the TV Module
 * @return RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_TV_Destroy(void);

/**
 * Open the TV playback using the input CNCAV device
 * @param dev AV device
 * @return RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_TV_Open(cncAVDevice_t dev);

/**
 * Close the tv playback module
 * @param dev AV Device
 * @return RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_TV_Close(cncAVDevice_t dev);

/**
 * Start TV playback
 * @param dev AV Device
 * @param ChannelNumber Channel Number
 * @param Type Service Type
 * @return RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_TV_Start(cncAVDevice_t dev, CNC_UINT16 ChannelNumber, cncBTVServiceType_t Type);

/**
 * Stop TV playback
 * @param dev AV Device
 * @return RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_TV_Stop(cncAVDevice_t dev);

/**
 * Send a notification to the TV Player
 * @param dev AV Device
 * @param type Notification Message
 */
void CNC_TV_PlayingNotify(cncAVDevice_t dev, cncBTVPlayMessageStatus_t type);

/**
 * Get the current channel data
 * @param dev AV Device
 * @param ChannelData Channel Service Information
 * @return RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_TV_GetCurChannelData(cncAVDevice_t dev, cncBTVServiceInfo_t *ChannelData);

/**
 * Switch the TV channel being played
 * @param dev AV Device
 * @param ChannelNumber New Channel Number
 * @param Type Service Type
 * @param force Force Switch
 * @return RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_TV_ChannelSwitch(cncAVDevice_t dev, CNC_UINT16 ChannelNumber, cncBTVServiceType_t Type, CNC_BOOL force);

/**
 * Register a callback function for TV Playback Events
 * @param dev AV Device
 * @param CallbackType Callback Type
 * @param pfnCallback Callback Function
 * @return RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_TV_RegisterCallback(cncAVDevice_t dev, cncBTVCallbackType_t CallbackType, cncBTVPlayCallback_t pfnCallback);

/**
 * Set parental control on the current channel
 * @param status rate
 */
void CNC_TV_SetParental(CNC_UINT8 status, CNC_UINT8 rate);

/**
 * Set a lock (block) on the current channel
 * @param flag
 */
void CNC_TVCtl_Set_LockFlag(char flag);

/**
 * Get the lock flag for the current channel
 * @return lock flag
 */
char CNC_TVCtl_GetLockFlag(void);

/**
 * Create the parental block task
 * @return RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_TVPlay_ParentalCreateTask(void);

/**
 * Parental Access
 * @return RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_TVPlay_ParentalAccess(int accessLevel);

/**
 * Send a message to the TV player
 * @param dev AV Device
 * @param msg Message Id
 * @param lParam parameter
 * @return RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_TVPlay_SendMessage(cncAVDevice_t dev, CNC_UINT8 msg, CNC_UINT32 lParam);

/**
 * Show channel string on the active LED
 * @param dev AV type
 * @param Service Service Information
 * @return RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_TV_ShowLEDStr(cncAVDevice_t dev, cncBTVServiceInfo_t *Service);

/**
 * Play analog TV
 * @param AV_Params  analog TV parameters
 * @return
 */
Result_Type_e CNC_TV_PlayATV(CNC_BASE_AV_Params_t *AV_Params);

/**
 * @param 
 * @return
 */
Result_Type_e CNC_TV_TuneToProgramOpen(cncBTVServiceInfo_t *p_serviceInfo);

/**
 * @param 
 * @return
 */
CNC_VOID CNC_TV_TuneToProgramClose(CNC_VOID);

/**
 * @param 
 * @return
 */
CNC_SINT32 CNC_TV_ReadTsData(CNC_UINT8 *p_buf, CNC_SINT32 *p_size);

/**
 * Freeze frame on the analog TV signal
 * @param dev AV Device
 * @return RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_AV_FreezeATV(cncAVDevice_t dev);

/**
 * Get the current network id from given ISDB TS handle
 * @return If the TS info is availale, return the network i, else return 0
 */
CNC_UINT16 CNC_Program_GetCurNetworkId(void);

/**
 * Get the count of transport streams in the given ISDB network
 * @param [in] NetWorkId currently unused
 * @return The TS(transport stream) count in the network
 */
CNC_UINT16 CNC_Program_GetTsCount(CNC_UINT16 NetWorkId);
/**
 * CNC_Program_GetTsNumberByChNumber
 * @todo read it further and carefully
 */
CNC_UINT16 CNC_Program_GetTsNumberByChNumber(CNC_UINT16 ChNumber, cncBTVServiceType_t Type);
/**
 * Get the ISDB TS information by given TS index
 * @param [in] TsNumber
 * @param [out] TsInfo
 * @return if successful returns RESULT_OK, error returns: RESULT_BAD_PARAMETER or RESULT_FAIL
 */
Result_Type_e CNC_Program_GetTsByTsNumber(CNC_UINT16 TsNumber, cncBTVTransportStreamInfo_t *TsInfo);
/**
 * Get the TS information by given TS index and network type
 * @param [in] TsNumber
 * @param [out] TsInfo
 * @param [in] Type
 * @return if successful returns RESULT_OK, error returns: RESULT_BAD_PARAMETER or RESULT_FAIL
 */
Result_Type_e CNC_Program_GetTsByTsNumber_Ext(CNC_UINT16 TsNumber, cncBTVTransportStreamInfo_t *TsInfo, cncBTVServiceType_t Type);

/**
 * Get the list of transport streams
 * @param NetWorkId Network Id
 * @param TsInfo [out] transport streams
 * @param Num  [out] number of transport streams
 * @return duplicate count of transport streams
 */
CNC_UINT16 CNC_Program_GetTsList(CNC_UINT16 NetWorkId, cncBTVTransportStreamInfo_t *TsInfo, CNC_UINT16 Num);

/**
 * Convert logic channel number to physical channel number
 * @param [in] logic ISDB logical channel number
 * @param [in] Type
 * @return Physical channel number
 * @todo check the input Type, if it's not ISDB, return directly
 */
CNC_UINT16 CNC_Program_GetChannelNumberbyLogic(CNC_UINT16 logic, cncBTVServiceType_t Type);

/**
 * Get the service count by given TS id
 * @param [in] TsId
 * @return If the TS exist, return the service count, else return 0
 */
CNC_UINT16 CNC_Program_GetServiceCountByTsId(CNC_UINT16 TsId);
/**
 * Get the service information with the given channel number
 * @param [in] ChNumber
 * @param [in] Type CNC_Service_Type_e
 * @param [out] ServiceInfo
 * @return If successful returns RESULT_OK, error returns: RESULT_BAD_PARAMETER or RESULT_FAIL
 */
Result_Type_e CNC_Program_GetServiceByChNumber(CNC_UINT16 ChNumber, cncBTVServiceType_t Type, cncBTVServiceInfo_t *ServiceInfo);
/**
 * Get the service information with the given channel number
 * @param [in] ch_lcn
 * @param [in] Type CNC_Service_Type_e
 * @param [out] ServiceInfo
 * @return If successful returns RESULT_OK, error returns: RESULT_BAD_PARAMETER or RESULT_FAIL
 */
Result_Type_e CNC_Program_GetServiceByLogicNum(CNC_UINT16 Chlcn, cncBTVServiceType_t Type, cncBTVServiceInfo_t *ServiceInfo);

/**
 * Get the service information by given service id
 * @param [in] ServiceId
 * @param [out] SubtitleInfo Allocated structure to store the output subtitle info
 * @return If successful returns RESULT_OK, error returns: RESULT_BAD_PARAMETER or RESULT_FAIL
 */
Result_Type_e CNC_Program_GetSubtitleBySID(CNC_UINT16 ServiceId, cncBTVSubtitleTextStream_t *SubtitleInfo);

/**
 * Get the service information by given service id
 * @param [in] ServiceId
 * @param [out] ServiceInfo Allocated structure to store the output service info
 * @return If successful returns RESULT_OK, error returns: RESULT_BAD_PARAMETER or RESULT_FAIL
 * @note The api would go over all the service types till getting a matched service id or reach the end.
 */
Result_Type_e CNC_Program_GetServiceBySID(CNC_UINT16 ServiceId, cncBTVServiceInfo_t *ServiceInfo);
/**
 * Get requested number of services from given transport stream
 * @param [in] TsId The give TS
 * @param [out] ServiceInfo Output service informatin list
 * @param [in] Num Max output service information record
 * @return Count of service information record
 * @note The API would try to try all service types to get the service list. According to the implementation, it seems that all service handles should be available before this API is invoked.
 */
CNC_UINT16 CNC_Program_GetServiceList(CNC_UINT16 TsId, cncBTVServiceInfo_t *ServiceInfo, CNC_UINT16 Num);

/**
 * Get the count of normal services according to the given service type
 * @param [in] Type Service type
 * @return Return the actual count of services if the services were available, else return 0
 */
CNC_UINT16 CNC_Program_GetServiceCountByType(cncBTVServiceType_t Type);
/**
 * Get the services according to the given service type
 * @param [in] Type Service type
 * @param [out] ServiceInfo Output link list of the services
 * @param [in] Num Max count of the output services
 * @return Return the actual count of output services if the services were available, else return 0
 */
CNC_UINT16 CNC_Program_GetServiceByType(cncBTVServiceType_t Type, cncBTVServiceInfo_t *ServiceInfo, CNC_UINT16 Num);

/**
 * Get the favorite services' count according to the given service type
 * @param [in] type Service type
 * @return Return the actual favorite services' count if the service type was available, else return 0
 */
CNC_UINT16 CNC_Program_GetFavServiceCount(cncBTVServiceType_t type);
/**
 * Output the favorite service list according to the given service type
 * @param [out] ServiceInfo Output favorite service list
 * @param [in] type requested service type
 * @return Return the actual favorite service count if the service type is available, else return 0
 */
CNC_UINT16 CNC_Program_GetFavServiceByType(cncBTVServiceInfo_t *ServiceInfo, cncBTVServiceType_t type);

/**
 * Update the changed service information to the database
 * @param [in] ServiceInfo The changed service information needed to be update the database
 * @return Return RESULT_OK only when the update operation excuted, else return RESULT_FAIL
 * @note The input ServiceInfo should NOT have the same address as the item in the database already
 */
Result_Type_e CNC_Program_SaveOneServiceToDB(cncBTVServiceInfo_t *ServiceInfo);
/**
 * Remove the specified the service infomation from the database
 * @param [in] ServiceInfo The service information to be deleted
 * @return Return RESULT_OK if the erase action was performed, no matter it success or fail, else return RESULT_FAIL
 * @todo Return OK only when erase action sucess?
 */
Result_Type_e CNC_Program_DeleteOneServiceFromDB(cncBTVServiceInfo_t *ServiceInfo);
/**
 * Exchange the two services within the database
 * @param [in] Service1
 * @param [in] Service2
 * @return Return RESULT_OK when the operation is done or when the operation was verified to be not needed, return RESULT_FAIL or the reason of the failure in other cases
 */
Result_Type_e CNC_Program_SwapServiceInDB(cncBTVServiceInfo_t *Service1, cncBTVServiceInfo_t *Service2);
/**
 * Save the specified service information database to NVM
 * @param [in] Type Specify the service type to be saved
 * @return Return the actual action result if the service information database is available, else return RESULT_FAIL
 */
Result_Type_e CNC_Program_SaveDBToFileByType(cncBTVServiceType_t Type);
/**
 * Get the global handle according to given handle type
 * @param [in] HandleType a given handle type, could be:
 * - CNC_Program_DB_HANDLE_TS
 * - CNC_Program_DB_HANDLE_TV
 * - CNC_Program_DB_HANDLE_RADIO
 * - CNC_Program_DB_HANDLE_TELTEXT
 * - CNC_Program_DB_HANDLE_SUBTITLE
 * - CNC_Program_DB_HANDLE_GXPRO
 * - CNC_Program_DB_HANDLE_REFERENCE
 * - CNC_Program_DB_HANDLE_TIMESHIFT
 * - CNC_Program_DB_HANDLE_MOSAIC
 * - CNC_Program_DB_HANDLE_FMRADIO
 * - CNC_Program_DB_HANDLE_BROADCAST
 * - CNC_Program_DB_HANDLE_MHP
 * - CNC_Program_DB_HANDLE_BAT
 * - CNC_Program_DB_HANDLE_DVBCTV
 * - CNC_Program_DB_HANDLE_DVBCRADIO
 * - CNC_Program_DB_HANDLE_DVBCTS
 * @return return the requested global handle match to the given type, or return null if type not matched
 */
CNC_CHAR * CNC_Program_GetDBHandle(cncBTVProgramDBHandle_t HandleType);

/**
 * Restore the services & TS informations from file system.
 */
void CNC_Program_DB_Load(void);

/**
 * Get program sqlite database handle
 * @note
 * @return handle point
 */
SQLDB_HANDLE CNC_Program_Get_DBHandle(void);

/**
 * Allocate databases for all services and TS
 * @note The databases are implemented in the way of vector in C++ enviroment
 * @todo Change the magic numbers used in the API to macro definitions
 */
Result_Type_e CNC_Program_DB_Init(void);
/**
 * Clear all the services and TS database
 * @return Return RESULT_OK if all operation successed, else return RESULT_FAIL
 * @note All the services and TS database would be destroied after invoking this API
 */
Result_Type_e CNC_Program_Reset(void);
/**
 * Get the Analog TV services list
 * @param [out] AtvServiceInfo Output link list of the analog TV services
 * @param [in] Type Service type
 * @param [in] Num Max count of the output services
 * @return Return the actual count of output analog TV services if the services were available, else return 0
 * @todo This API has different parameter sequence, neet to adjust it?
 */
CNC_UINT16 CNC_Program_GetAtvServiceByType(cncBTVAnalogInfo_t *AtvServiceInfo, cncBTVServiceType_t Type, CNC_UINT16 Num);

/**
 * Get the transport stream count for DVBC
 * @param NetWorkId Network Id
 * @return TS count
 */
CNC_UINT16 CNC_Program_GetTsCount_DVBC(CNC_UINT16 NetWorkId);

/**
 * Get the current DVBC Network Id
 * @return Network Id
 */
CNC_UINT16 CNC_Program_GetCurNetworkId_DVBC(void);

/**
 * Create a PMT Filter
 * @return
 */
Result_Type_e CNC_SI_CreatPmtFilter(void);

/**
 * Allocate Demux Filter for PID and other parameters
 * @param [in] PID Packet ID to request the demux filter
 * @param [in] TableID
 * @param [in] TableIDExtension
 * @param [in] VersionNo
 * @param [in] Version_match
 * @param [in] Section_number
 * @param [in] callback Pointer to a function which would process the filtered data
 * @param [in] timeout
 * @param [in] userparam A parameter for the callback function
 * @param [in] enable Wheather enable the demux filter immediately
 *
 * @return CNC_SI_Handle_t
 * - Return -1 if fail
 * - Return a si_id if success
 *
 * @see CNC_SI_SetFilter_Ex
 */
CNC_SI_Handle_t CNC_SI_SetFilter(CNC_UINT16 PID,
                                                        CNC_UINT8 TableID,
                                                        CNC_SINT32 TableIDExtension,
                                                        CNC_SINT8 VersionNo,
                                                        CNC_BOOL Version_match,
                                                        CNC_SINT32 Section_number,
                                                        CNC_SI_DemuxCallBack callback,
                                                        CNC_UINT32 timeout,
                                                        void* userparam,
                                                        CNC_BOOL enable);


/**
 * Release Filter
 * @return
 */
Result_Type_e CNC_SI_ReleaseFilter(CNC_SI_Handle_t si_handle);

/**
 * Delete a PMT Filter
 * @return
 */
Result_Type_e CNC_SI_DeletePmtFilter(void);

/**
 * Create a PAT Filter
 * @return
 */
Result_Type_e CNC_SI_CreatPatFilter(void);

/**
 * Delete a PAT Filter
 * @return
 */
Result_Type_e CNC_SI_DeletePatFilter(void);

/**
 * Enable the PAT Filter
 * @return
 */
Result_Type_e CNC_SI_EnablePatFilter(void);

/**
 * Disable the PAT Filter
 * @return
 */
Result_Type_e CNC_SI_DisablePatFilter(void);

/**
 * Set the TDT Timer
 */
void CNC_SI_SetTDTTimer(void);

/*
 * SEARCH Module
 */

/**
 * Here define the program search method
 */
typedef enum
{
    CNC_SEARCH_ONE_FREQ,		//!< Search one frequency only
    CNC_SEARCH_NETWORK,		//!< Auto search, e.g. get NIT, then continue the search based on NIT
    CNC_SEARCH_ALL_FREQ,		//!< Blind scan
} cncBTVSearchScope_t;

/**
 * This are the seach states
 */
typedef enum
{
    CNC_SEARCH_SEARCHING,
    CNC_SEARCH_STOP
} cncBTVSearchStatus_t;


/**
 * This structure defines the params for program/service scan
 *
 */
typedef struct
{
    cncBTVSearchScope_t scanType;		//!< Scan one freq/auto scan/blind scan
    cncTunerModulationType_t tuner_type;	//!< DVBC, ISDB, DVBT, DVBS, Analog
    cncTunerInputMode_t input_mode;                //!< serial , parallel>
    union
    {

        union
        {
            struct
            {
                CNC_UINT32 frequency;
                CNC_UINT32 symbol_rate;
                cncTunerQAMMode_t modulation;
                unsigned int bandwidth;
            } one;			//!< Params for signal channel scan
            struct
            {
                CNC_UINT32 frequency;
                CNC_UINT32 symbol_rate;
                cncTunerQAMMode_t modulation;
                unsigned int bandwidth;
            } net;			//!< Params for auto scan
            struct
            {
                CNC_UINT32 *frequencies;//!< the frequencies array
                CNC_UINT32 freq_nums;//!< numbers of frequencies
                CNC_UINT32 symbol_rate;
                cncTunerQAMMode_t modulation;
                unsigned int bandwidth;
            } all;			//!< Params for frequency_list scan/blind scan without NIT
        } dvbc;	//!< These params are defined for DVB-C program/service scan

        union
        {
            struct
            {
                CNC_UINT32 frequency;
            } one;			//!< Param for single channel scan
            struct
            {
                CNC_UINT32 frequency;
            } net;			//!< Param for autoscan with NIT
            struct
            {
                CNC_UINT32 *frequencies;	//!< the frequencies array
                CNC_UINT32 freq_nums;		//!< numbers of frequencies
            } all;			//!< Params for frequency list scan/blind scan
        } isdb;	//!< These params are defined for ISDB program scan

        union
        {
            struct
            {
                CNC_UINT32 frequency;
                CNC_UINT32  bandwidth;
            } one;			//!< Params for DVB-T signal channel scan
            struct
            {
                CNC_UINT32 frequency;
                CNC_UINT32  bandwidth;
            } net;			//!< Parmas for DVB-T auto scan
            struct
            {
                CNC_UINT32 *frequencies;	//!< the frequencies array
                CNC_UINT32 freq_nums;		//!< numbers of frequencies
                CNC_UINT32  bandwidth;
            } all;			//!< Params for DVB-T frequency list scan / blind scan
        } dvbt;	//!< These params are defined for DVB-T program/service scan

        union
        {
            struct
            {
                CNC_UINT32 frequency;
            } one;			//!< Param for single analog channel scan
#ifdef MANU_DFACTUR_TEST
            struct
            {
                  CNC_UINT32 *frequencys;
                  CNC_UINT8  type;
                  CNC_UINT8  freq_nums;
            }all;
#endif
        } atv;	//!< This param is defined for analog TV scan

        union
        {
            struct
            {
                cncTunerDVBS2_t frequency;
            } one;			//!< Param for single channel searching
            struct
            {
                cncTunerDVBS2_t *frequencies;
                CNC_UINT32 freq_nums;//numbers of frequencies
            } all;			//!< Params for frequency list searching
        } dvbs2;
    } freq;
} cncBTVSearchFrequencyInfo_t;

CNC_BOOL CNC_Search_OutputTSInfo(void *freq_info, void *context);
CNC_BOOL CNC_Search_OutputServiceInfo(void *serv_info, void *context);

/**
* Send message to seach task, application may call it.
* @param [in, out] freq_info
* @note If the scanType is CNC_SEARCH_ALL_FREQ, it should allocate new memory for the field 'frequencies' because the original memory space may be released when the caller function quit
*/
void CNC_Search_SendMessageToSearch(cncBTVSearchFrequencyInfo_t *freq_info);

/**
 * Return the current task status:
 * @return CNC_Search_Status:
 * - CNC_SEARCH_SEARCHING
 * - CNC_SEARCH_STOP
 */
cncBTVSearchStatus_t CNC_Search_GetStatus(void);

/**
 * Get the Searched TS elementary stream
 * @return stream id
 */
CNC_UINT32 CNC_Search_GetSearchedTSES(void);

/**
 * Lock the search process
 */
void CNC_Search_EnterHook(void);

/**
 * Unlock the search process
 */
void CNC_Search_ExitHook(void);

/**
 * Exit the search
 */
void CNC_Search_Exit(void);

/**
 * Get the sum of video services
 * @return sum
 */
CNC_UINT32 CNC_Search_GetVideoSum(void);

/**
 * Get the sum of audio services
 * @return sum
 */
CNC_UINT32 CNC_Search_GetAudioSum(void);

/**
 * Get the sum of all services
 * @return sum
 */
CNC_UINT32 CNC_Search_GetAllSum(void);

/**
 * Get the BTV service by index
 * @param index Index
 * @param freq [out] Tuner Frequency
 * @return Service Information
 */
cncBTVServiceInfo_t *CNC_Search_GetServiceByIndex(CNC_UINT32 index, CNC_UINT32 * freq);

/**
 * Intialize the enviroment for program/service seach
 * @note This API would allocate memory(vector) for potential service need. And it would allocate timer, message queues, and task for program/service searching.
 */
CNC_BOOL CNC_Search_InitSearchTask(void);

/**
 * Get the currently outputting frequency
 * @param servinfo BTV Service Info
 * @return Output Frequency
 */
CNC_UINT32 CNC_Search_GetCurrentFrequency(cncBTVServiceInfo_t *servinfo);

/*
 * EPG Module
 */
#define MAX_EVENT_NAME_LEN               (64)
#define CNC_EPG_MAX_SERVICE_NUM     (96)
#define CNC_EPG_MAX_RESERVED_NUM   (20)
#define CNC_EPG_MAX_RATING_NUM       (5)
#define CNC_EPG_MAX_EVENT_NUM      (96)
#define CNC_EPG_MAX_EVENT_LIST_NUM  (6)

typedef enum
{
    SERVICE_RUN_UNDEFINE = 0,
    SERVICE_RUN_NOTRUN,
    SERVICE_RUN_INFEW,
    SERVICE_RUN_PAUSED,
    SERVICE_RUN_RUNNING,
    SERVICE_RUN_RESERVED
} cncBTVEPGServiceStatus_t;

typedef enum
{
    SERVICE_NO_NVOD = 0,
    SERVICE_NVOD,
    SERVICE_TYPE_SUM
} cncBTVNVODServiceType_t;

typedef struct
{
    CNC_UINT16 data_com_id;          //data_component_id (0x0008-full-seg, 0x0012-one-seg)
    CNC_UINT8  entry_com;            //entry_component
    CNC_UINT8  language_tag;         //first language == 0b 000(0x00)
    CNC_UINT8  DMF;                  //0b 0011(0x03)
    CNC_UINT8  iso_language_code[3]; //language code of this caption
    CNC_UINT8  text[18];             //corresponding ES description (Maxumum of 16 bytes)
} cncBTVCloseCaptionDataContent_t;

typedef struct _cncBTVEPGRating_t
{
    CNC_UINT8 country_code[4];
    CNC_UINT8 Rating;
} cncBTVEPGRating_t;

typedef struct _cncBTVEPGEventInfo_t
{
    CNC_UINT16 EventId;
    cncUTC_t StartTime;
    struct
    {
        CNC_UINT8 hh;
        CNC_UINT8 mm;
        CNC_UINT8 ss;
    } Duration;
    CNC_UINT8 RunStatus;
    CNC_UINT8 fFreeMode;
    CNC_UINT8 ContentNibble;
    CNC_UINT8 RatingCount;
    cncBTVEPGRating_t RatingInfo[CNC_EPG_MAX_RATING_NUM];
    char EventName[MAX_EVENT_NAME_LEN];
    CNC_UINT16 TextLen;
    char *Text;
    char Language[4];
    cncBTVCloseCaptionDataContent_t cc;
} cncBTVEPGEventInfo_t;

/**
 * This is the structure for EPG event booking
 */
typedef struct _cncBTVEPGReservedEventInfo_t
{
//    U16     OrigNetId;
    CNC_UINT16     TsId;               //!< see path TS, service, and Event
    CNC_UINT16     ServiceId;          //!< see path TS, service, and Event
    CNC_UINT16     ServiceType;        //!< Type of the service
    cncBTVEPGEventInfo_t ReservedEvent;  //!< see path TS, service, and Event
} cncBTVReservedEventInfo_t;

typedef void (*cncBTVReservedCallback_t)(cncBTVReservedEventInfo_t *Event);


/**
 * Initialize the enviroment for EPG
 * @note To allocate the memory, OS resources (including semaphore, message queue and task), and Demux filter resource  for EPG
 * @return if successful returns RESULT_OK, error returns: RESULT_BAD_PARAMETER or RESULT_FAIL
 * @see CNC_EPG_Final
 */
Result_Type_e CNC_EPG_Init(void);
/**
 * Finalialize the enviroment for EPG
 * @note
 * @return if successful returns RESULT_OK, error returns: RESULT_BAD_PARAMETER or RESULT_FAIL
 * @see CNC_EPG_Init
 */
Result_Type_e CNC_EPG_Final(void);
/**
 * Temporary stop EPG feature
 * @note The EPG was paused by disabling the demux filter, including PF filter and schedule filter
 * @return if successful returns RESULT_OK, error returns: RESULT_BAD_PARAMETER or RESULT_FAIL
 * @see CNC_EPG_Resume
 */
Result_Type_e CNC_EPG_Pause(void);
/**
 * Resume EPG feature from pause state
 * @note The EPG would be brought out from pause state by enabling the PF filter and the schedule filter
 * @return if successful returns RESULT_OK, error returns: RESULT_BAD_PARAMETER or RESULT_FAIL
 * @see CNC_EPG_Pause
 */
Result_Type_e CNC_EPG_Resume(void);
/**
 * Destroy the current stored EPG data and rebuild it with given param
 * @note The EPG tree has 4 levels: Network-TS-Service-Event
 * @param [in] NetWorkId to rebuild a new EPG tree based on this NetWorkId
 * @return if successful returns RESULT_OK, error returns: RESULT_BAD_PARAMETER or RESULT_FAIL
 */
Result_Type_e CNC_EPG_ResetEPGTree(CNC_UINT16 NetWorkId);
/**
 * Get the init status of EPG data
 * @note The status would be changed in API CNC_EPG_ResetEPGTree
 * @return The status of EPG data
 * @see CNC_EPG_ResetEPGTree
 */
CNC_BOOL CNC_EPG_GetResetEpgflag(void);
/**
 * Give up the usage of EPG filters
 * @note This would cause the EPG tree not updated any more because data sources are cut
 * @return if successful returns RESULT_OK, error returns: RESULT_BAD_PARAMETER or RESULT_FAIL
 * @see CNC_EPG_SetupFilter
 */
Result_Type_e CNC_EPG_ReleaseFilter(void);
/**
 * Set up Demux filters for EPG usage
 * @note It would setup filters for both P/F and schedule events. If the filters were already set up, then release it before go ahead.
 * @return if successful returns RESULT_OK, error returns: RESULT_BAD_PARAMETER or RESULT_FAIL
 * @see CNC_EPG_ReleaseFilter
 */
Result_Type_e CNC_EPG_SetupFilter(void);
/**
 * Fill the current time into a given DATETIME structure
 * @note The API would get the current system time and convert it to local time with the given timezone
 * @param [in] OffSet timezone
 * @param [out] DateTime pointer to a DATETIME structure, to store the output date time
 * @return if successful returns RESULT_OK, error returns: RESULT_BAD_PARAMETER or RESULT_FAIL
 */
Result_Type_e CNC_EPG_GetCurLocalDataTime(CNC_SINT16 OffSet, cncDateTime_t *DateTime);

/**
 * Fill the current time into a given string
 * @note The API would convert a UTC time into DATETIME, then print the date time into a given string
 * @param [out] Prompt a string to store the time information in text approach
 * @param [in] utc a given time in UTC format
 * @return void
 */
void CNC_EPG_DateTimePrint(char *Prompt, cncUTC_t *utc);

/**
 * An API to get how many P/F (Present/Following) events in given service
 * @param [in] TsId Index of the given transport stream
 * @param [in] ServiceId Index of the given service in the transport stream
 * @return number of PF event
 * @see CNC_EPG_GetPFEventList, CNC_EPG_GetSchEventCount
 */
CNC_UINT16 CNC_EPG_GetPFEventCount(CNC_UINT16 TsId, CNC_UINT16 ServiceId);
/**
 * Get request number of P/F (Present/Following) events in given service
 * @note The API would output an P/F event list
 * @param [in] TsId Index of the given transport stream
 * @param [in] ServiceId Index of the given service in the transport stream
 * @param [out] pf_info Pointer to the output event list
 * @param [in] Num Max number of events requested,
 * @return The actual event number
 * @see CNC_EPG_GetPFEventCount, CNC_EPG_GetDaySchEventList
 */
CNC_UINT16 CNC_EPG_GetPFEventList(CNC_UINT16 TsId, CNC_UINT16 ServiceId, cncBTVEPGEventInfo_t *pf_info, CNC_UINT16 Num);
/**
 * An API to get how many schedule events in given service, given date
 * @note The API would go over all events in the given service & TS, and only count the events in the given date
 * @param [in] TsId Index of the given transport stream
 * @param [in] ServiceId Index of the given service in the transport stream
 * @param [in] Day Specified date to count the schedule events
 * @return number of schedule event
 * @see CNC_EPG_GetDaySchEventList, CNC_EPG_GetSchEventCount
 */
CNC_UINT16 CNC_EPG_GetDaySchEventCount(CNC_UINT16 TsId, CNC_UINT16 ServiceId, CNC_UINT8 Day);
/**
 * Get request number of schedule events in given service
 * @note The API would go over all events in the given service & TS then output an schedule event list
 * @param [in] TsId Index of the given transport stream
 * @param [in] ServiceId Index of the given service in the transport stream
 * @param [in] Day Specified date to count the schedule events
 * @param [out] epg_info Pointer to the output event list
 * @param [in] Num Max number of events requested,
 * @return The actual event number
 * @see CNC_EPG_GetSchEventCount
 */
CNC_UINT16 CNC_EPG_GetDaySchEventList(CNC_UINT16 TsId, CNC_UINT16 ServiceId, CNC_UINT8 Day, cncBTVEPGEventInfo_t *epg_info, CNC_UINT16 Num);
/**
 * An API to get how many schedule events in given service
 * @note The API would go over all events in the given service & TS
 * @param [in] TsId Index of the given transport stream
 * @param [in] ServiceId Index of the given service in the transport stream
 * @return number of schedule event
 * @see CNC_EPG_GetSchEventList
 */
CNC_UINT16 CNC_EPG_GetSchEventCount(CNC_UINT16 TsId, CNC_UINT16 ServiceId);
/**
 * Get request number of schedule events in given service
 * @note The API would go over all events in the given service & TS then output an schedule event list
 * @param [in] TsId Index of the given transport stream
 * @param [in] ServiceId Index of the given service in the transport stream
 * @param [out] epg_info Pointer to the output event list
 * @param [in] Num Max number of events requested,
 * @return The actual event number
 * @see CNC_EPG_GetSchEventCount
 */
CNC_UINT16 CNC_EPG_GetSchEventList(CNC_UINT16 TsId, CNC_UINT16 ServiceId, cncBTVEPGEventInfo_t *epg_info, CNC_UINT16 Num);
/**
 * Get the event description of the specified P/F(Present/Following) event.
 * @param [in] TsId Index of the given transport stream
 * @param [in] ServiceId Index of the given service in the transport stream
 * @param [in] EventId Index of the specified P/F event
 * @param [out] pBuf a string buffer to store the output event description
 * @param [in] BufLen Max size of the output
 * @return if successful returns RESULT_OK, error returns: RESULT_BAD_PARAMETER or RESULT_FAIL
 * @see CNC_EPG_GetSchEventSummary
 */
Result_Type_e CNC_EPG_GetPFEventSummary(CNC_UINT16 TsId, CNC_UINT16 ServiceId, CNC_UINT16 EventId, char *pBuf, CNC_UINT16 BufLen);
/**
 * Get the event description of the specified schedule event.
 * @param [in] TsId Index of the given transport stream
 * @param [in] ServiceId Index of the given service in the transport stream
 * @param [in] EventId Index of the specified schedule event
 * @param [out] pBuf a string buffer to store the output event description
 * @param [in] BufLen Max size of the output
 * @return if successful returns RESULT_OK, error returns: RESULT_BAD_PARAMETER or RESULT_FAIL
 * @see CNC_EPG_GetPFEventSummary
 */
Result_Type_e CNC_EPG_GetSchEventSummary(CNC_UINT16 TsId, CNC_UINT16 ServiceId, CNC_UINT16 EventId, char *pBuf, CNC_UINT16 BufLen);
/**
 * Get how much of the Present event had been broadcasted from the event's beginning, in percentage
 * @param [in] TsId Index of the given transport stream
 * @param [in] ServiceId Index of the given service in the transport stream
 * @param [out] Progress The percentage of the broadcasting stage of the present event
 * @return if successful returns RESULT_OK, error returns: RESULT_BAD_PARAMETER or RESULT_FAIL
 */
Result_Type_e CNC_EPG_GetCurEventProgress(CNC_UINT16 TsId, CNC_UINT16 ServiceId, CNC_UINT8 *Progress);

/****************** Reserved Interface ******************/
Result_Type_e CNC_Reserved_Init(void);
Result_Type_e CNC_Reserved_Reset(void);
CNC_UINT16 CNC_Reserved_GetEventCount(void);
CNC_UINT16 CNC_Reserved_GetEventList(cncBTVReservedEventInfo_t *ResevedEventList, CNC_UINT16 Num);
CNC_BOOL CNC_Reserved_GetConflict(cncBTVReservedEventInfo_t *ReservedEvent, cncBTVReservedEventInfo_t *ConflictEvent);
Result_Type_e CNC_Reserved_AddEvent(cncBTVReservedEventInfo_t *ResevedEvent);
Result_Type_e CNC_Reserved_DeleteEvent(cncBTVReservedEventInfo_t *ResevedEvent);
Result_Type_e CNC_Reserved_SaveEvent(void);
Result_Type_e CNC_Reserved_InstallCallback(cncBTVReservedCallback_t pfCallback);
CNC_BOOL CNC_EPG_GetResetEpgflag(void);

/*
 * Subtitle Module
 */
typedef enum
{
    CNC_SUB_SCREEN_576 = 0,
    CNC_SUB_SCREEN_480,
    CNC_SUB_SCREEN_720,
    CNC_SUB_SCREEN_1080,
} cncBTVSubtitleScreenType_t;



typedef struct
{
    cncBTVSubtitleScreenType_t 	Type;
    CSOSD_COLOR_DEPTH color_depth;
    CSOSD_COLOR_FORMAT      Osdmode;
    CSOSD_HANDLE         handle;
} cncBTVSubtitleConfig_t;



/**
 * Initialize the subtitle feature
 * @param [in] value The default subtitle settings
 * @return return false if the input default settings is NULL
 *
 */
CNC_BOOL CNC_Subtitle_Init(cncBTVSubtitleConfig_t *value);

/**
 * Finalize the subtitle feature
 * @return always return true
 * @note It would reset the global flag cncSubtitleInitialized
 */
CNC_BOOL CNC_Subtitle_Final(void);

/**
 * Start the subtitle feature
 * @param [in] SubPid PID of subtitle
 * @return CNC_BOOL
 * - FALSE when SubPid invalid or subtitle decoder init failed
 * - other case means success, return TRUE
 */
CNC_BOOL CNC_Subtitle_Open(CNC_UINT16 SubPid);

/**
 * Stop subtitle and free the resources
 * @return false if subtitle is not running, else return true
 * @note This API would free all resources needed for decoding and displaying subtitle.
 */
CNC_BOOL CNC_Subtitle_Close(void);


/**
 * Enable subtitle display on screen
 * @return always return true
 * @note If the cncSubtitleDisableFlag is reset, the subtitle framebuffer would be updated
 */
CNC_BOOL CNC_Subtitle_Enable(void);

/**
 * Disable subtitle display on screen
 * @return Return TRUE always
 * @note Setting cncSubtitleDisableFlag would stop the subtitle framebuffer updating, and the subtitle framebuffer should be cleared after that
 */
CNC_BOOL CNC_Subtitle_Disable(void);


/*
 * Teletext Module
 */
typedef enum
{
    CS_TTX_KEY_MSG_0 = 0x10,
    CS_TTX_KEY_MSG_1 = 0x11,
    CS_TTX_KEY_MSG_2 = 0x12,
    CS_TTX_KEY_MSG_3 = 0x13,
    CS_TTX_KEY_MSG_4 = 0x14,
    CS_TTX_KEY_MSG_5 = 0x15,
    CS_TTX_KEY_MSG_6 = 0x16,
    CS_TTX_KEY_MSG_7 = 0x17,
    CS_TTX_KEY_MSG_8 = 0x18,
    CS_TTX_KEY_MSG_9 = 0x19,

    CS_TTX_KEY_MSG_OK = 0x20,
    CS_TTX_KEY_MSG_PAGE_UP = 0x21,
    CS_TTX_KEY_MSG_PAGE_DOWN = 0x22,
    CS_TTX_KEY_MSG_SUBPAGE_UP = 0x23,
    CS_TTX_KEY_MSG_SUBPAGE_DOWN = 0x24,
    CS_TTX_KEY_MSG_PAGE10_UP = 0x25,
    CS_TTX_KEY_MSG_PAGE10_DOWN = 0x26,

    CS_TTX_KEY_MSG_RED = 0x51,
    CS_TTX_KEY_MSG_BLUE = 0x52,
    CS_TTX_KEY_MSG_GREEN = 0x53,
    CS_TTX_KEY_MSG_YELLOW = 0x54,
} cncBTVTeltextKey_t;


typedef enum
{
    TTX_SCREEN_576 = 0,
    TTX_SCREEN_480,
    TTX_SCREEN_720,
    TTX_SCREEN_1080,
} cncBTVTeltextScreenType_t;



typedef struct
{
    cncBTVTeltextScreenType_t 	Type;
    CSOSD_COLOR_DEPTH color_depth;
    CSOSD_COLOR_FORMAT      Osdmode;
    CSOSD_HANDLE         handle;
} cncTeltextConfig_t;

/**
 * Initialize Teletext Module
 * @param value Teletext Configuration
 * @return TRUE if succcess, FALSE if error
 */
CNC_BOOL CNC_Teletext_Init(cncTeltextConfig_t *value);

/**
 * Open the teletext display
 * @param TtxPid PID of the telext data
 * @return TRUE if succcess, FALSE if error
 */
CNC_BOOL CNC_Teletext_Open(CNC_UINT16 TtxPid);

/**
 * Close the teletext display
 * @return TRUE if succcess, FALSE if error
 */
CNC_BOOL CNC_Teletext_Close(void);

/**
 * Send teletext key
 * @param wParam key value
 * @return TRUE if succcess, FALSE if error
 */
CNC_BOOL CNC_Teletext_SendKey(cncBTVTeltextKey_t wParam);

/*
 * CC Module
 */
typedef struct
{
    CNC_UINT16                           PmtPid;
    cncBTVClosedCaptionStream_t                      stream;
    cncBTVCloseCaptionDataContent_t                 content;
} cncBTVClosedCaptionOpenParams_t;

/**
 * Initialize the enviroment of Closed Caption service
 * @param [in] config Current graphic layer configuration
 * @return CNC_BOOL type
 * - TRUE on sucess
 * - FALSE on fail
 * @note This API would try to get OS resources and allocate memory for Closed Caption
 * @todo To optmize the error handling
 * @see CNC_CC_Fina
 */
CNC_BOOL CNC_CC_Init(CSOSD_Config *config);
/**
 * Destroy the enviroment of Closed Caption service
 * @return CNC_BOOL type
 * - TRUE on sucess
 * - FALSE on fail
 * @note The API would free the OS resources and de-alloc the memory for Closed Caption
 * @see CNC_CC_Init
 */
CNC_BOOL CNC_CC_Final(void);

/**
 * Start Closed Caption service
 * @param [in] pOpenParams
 * @return CNC_BOOL type
 * - TRUE on sucess
 * - FALSE on fail
 * @note It would enable and start Closed Caption service through DEBUX module
 * @remarks If Closed Caption service is already opened, then it would be closed and then reopened.
 * @see CNC_CC_Close
 */
CNC_BOOL CNC_CC_Open(cncBTVClosedCaptionOpenParams_t *pOpenParams);
/**
 * Stop Closed Caption service and clear the screen display
 * @return CNC_BOOL type
 * - TRUE on sucess
 * - FALSE on fail
 * @note It would stop Closed Caption service through DEMUX module
 * @see CNC_CC_Open
 */
CNC_BOOL CNC_CC_Close(void);

/**
 * Enable Closed Caption service
 * @return CNC_BOOL type
 * - TRUE on sucess
 * - FALSE on fail
 * @note A flag in CC manager structure will be set
 * @see CNC_CC_Disable
 */
CNC_BOOL CNC_CC_Enable(void);
/**
 * Disable Closed Caption service and clear the screen display
 * @return CNC_BOOL type
 * - TRUE on sucess
 * - FALSE on fail
 * @note A flag in CC manager structure will be reset
 * @see CNC_CC_Enable
 */
CNC_BOOL CNC_CC_Disable(void);

#if 0
/* NVOD Module
 *
 */
#define NVOD_MAX_TIMESHIFT_COUNT   10

#define NVOD_MAX_TIMESHIFT_EVENT    15
#define NVOD_MAX_SERVICE_NAME         40
#define NVOD_Evnlistnum                          15

typedef struct
{
    CNC_UINT16 EventId;
    CNC_UINT16 TsId;
    CNC_UINT16 ServiceId;
    CNC_UINT16 ServiceType;
    cncUTC_t StartTime;
    struct
    {
        CNC_UINT8 hh;
        CNC_UINT8 mm;
        CNC_UINT8 ss;
    } Duration;
    CNC_UINT8 Start_h;
    CNC_UINT8 Start_m;
    CNC_UINT8 End_h;
    CNC_UINT8 End_m;
    CNC_UINT8 reserveflag;
    char EventName[NVOD_MAX_SERVICE_NAME];
} cncBTVNVODEventList_t;

typedef struct
{
    cncBTVNVODEventList_t Event[NVOD_Evnlistnum];
} cncBTVNVODTimeshiftList_t;

typedef struct _cncBTVNVODEvent_t
{
    CNC_UINT16 EventId;
    cncUTC_t StartTime;
    struct
    {
        CNC_UINT8 hh;
        CNC_UINT8 mm;
        CNC_UINT8 ss;
    } Duration;
    CNC_UINT8 fFreeMode;
} cncBTVNVODEvent_t;

typedef struct _cncBTVNVODTimehshift_t
{
    CNC_UINT8 TimeShiftName[NVOD_MAX_SERVICE_NAME];
    CNC_UINT16 ProgramId;
    CNC_UINT16 RefServiceId;
    CNC_UINT16 RefEventId;
    CNC_UINT16 tsEventCount;
    cncBTVNVODEvent_t tsEvent[NVOD_MAX_TIMESHIFT_EVENT];
} cncBTVNVODTimeShift_t;

typedef struct _cncBTVNVODReferenceService_t
{
    CNC_UINT8 RefServiceName[NVOD_MAX_SERVICE_NAME];
    CNC_UINT8 RefName[NVOD_MAX_SERVICE_NAME];  // short event name
    CNC_UINT8 ShortDesc[NVOD_MAX_SERVICE_NAME];
    CNC_UINT8 TimeShiftCount;
    CNC_UINT16 TsId;
    CNC_UINT16 OriNetworkId;
    CNC_UINT16 ServiceId;
    cncBTVNVODTimeShift_t TimeShift[NVOD_MAX_TIMESHIFT_COUNT];
    //U16 ExtendLen;
    //U8 *ExtendText;
} cncBTVNVODReferenceService_t;

#include "cnc_av.h"

CNC_SINT32 CNC_Nvod_Initial(void);
CNC_BOOL CNC_Nvod_serv_SortRefServiceCallback(void *pgm1, void *pgm2);
void CNC_Nvod_AddOneTimeShiftEvnt(CNC_UINT16 Tsid, CNC_UINT16 Serviceid, CNC_UINT16 ReferServid, cncBTVNVODEvent_t *NvoeEvent);
CNC_BOOL CNC_Nvod_AddRefService(void *ts_info, void *ts_list);
void CNC_Nvod_ParseEitPF(CNC_UINT8 *buf);
CNC_BOOL CNC_Nvod_AddReserved(cncBTVNVODEventList_t *ResevedEvent);
CNC_BOOL CNC_Nvod_DeleteReserved(cncBTVNVODEventList_t *ResevedEvent);
CNC_BOOL CNC_Nvod_GetReservedConflict(cncBTVNVODEventList_t *ReservedEvent, cncBTVNVODEventList_t *ConflictEvent);
CNC_SINT32 CNC_Nvod_GetCurEventProgress(CNC_UINT16 select, CNC_UINT8 *Progress, CNC_UINT8 *LastTime);
CNC_UINT8 CNC_Nvod_GetLastService(void);
void CNC_Nvod_SetLastTimeShService(CNC_UINT8 ServiceIndex);
CNC_UINT8 CNC_Nvod_GetFullScreenViewFlag(void);
void CNC_Nvod_SetFullScreenViewFlag(CNC_UINT8 fullscreenflag);
CNC_SINT32 CNC_Nvod_GetRefServiceNum(void);
cncBTVNVODReferenceService_t *CNC_Nvod_GetRefServiceList(void);
CNC_BOOL CNC_Nvod_Find_Timshf_By_Sid(void *x, void *context);
void CNC_Nvod_Create(void);
CNC_UINT16 CNC_Nvod_Program_GetServiceByType(cncBTVServiceType_t Type, cncBTVNVODReferenceService_t *ServiceInfo, CNC_UINT16 Num);
#endif

#define DEBUG_CA
#ifdef DEBUG_CA
#define CA_DPRINTF(a) printf a
#else
#define CA_DPRINTF(a)
#endif

#define CNC_MAX_DEPOSIT_COUNT                   50          /* Number of records * E-Wallet Recharge*/
#define CNC_MAX_COST_COUNT                        50        /*Maximum number of consumer records*/
#define CNC_MAX_PROVIDER_NAME_LENGTH      22          /* Maximum length of vendor name */
#define CNC_MAX_SENDER_NAME_LENGTH          22          /* maximum length of the sender name*/
#define CNC_MAX_EMAIL_TITLE                         40          /*Maximum length of the message header */
#define CNC_MAX_EMAIL_CONTENT                   255         /* Maximum length of the message content */
#define CNC_MAX_MAIL_COUNT                        50          /* Maximum number of messages stored*/
#define CNC_MAX_AUTHINFO_ITEM                   2           /* Maximum number of authorization records*/
#define CNC_MAX_AUTHINFO_TIMES                  100         /* Maximum number of authorization*/
#define CNC_MAX_CA_LANGTH                           4           /* 4: marked 128, 8 marked 256*/

typedef enum
{
      CNC_CA_MONITOR_TYPE_NULL,
      CNC_CA_MONITOR_TYPE_CAT,
      CNC_CA_MONITOR_TYPE_PMT,
      CNC_CA_MONITOR_TYPE_EIT,
      CNC_CA_MONITOR_TYPE_SDT,
      CNC_CA_MONITOR_TYPE_NIT,
      CNC_CA_MONITOR_TYPE_UNKNOWN
}cncBTVCAMonitorType_t;


/*  Authorization information*/
typedef struct _cncBTVCAAuthorizationInfo_t
{
    CNC_UINT32    uReference;                                   /* where each is a set of four 128*/
    CNC_UINT16   uStartTime;                                 /* start time */
    CNC_UINT16   uEndTime;                                   /* end time */
    char   ProviderName[CNC_MAX_PROVIDER_NAME_LENGTH];   /*Vendor*/
    CNC_UINT8    display_name_flag;                  /* 1 shows ProviderName, 0 does not display ProviderName */
}cncBTVCAAuthorizationInfo_t;

typedef struct _cncBTVCAAuthorizationListInfo_t
{
    CNC_SINT32 AuthInfo_count;                                  /*  record number of authorized number */
    CNC_UINT8  uPacketCount[CNC_MAX_AUTHINFO_ITEM + 1];                              /* the number of packets */
      cncBTVCAAuthorizationInfo_t AuthInfoList[CNC_MAX_AUTHINFO_ITEM * CNC_MAX_AUTHINFO_TIMES * CNC_MAX_CA_LANGTH]; /* authorization list */
}cncBTVCAAuthorizationListInfo_t;


/*  Electronic wallet information */
typedef struct _cncBTVCADepositInfo_t
{
    CNC_UINT16 deposit_time;                        /* Recharge recording time */
    CNC_UINT16 deposit_amount;                      /* Record the amount of recharge*/
    CNC_UINT16 balance;                             /* Balance */
}cncBTVCADepositInfo_t;

typedef struct _cncBTVCADepositInfoList_t
{
    CNC_SINT32 Deposit_count;                                 /* number of deposit */
    cncBTVCADepositInfo_t DepositList[CNC_MAX_DEPOSIT_COUNT];    /* deposit list */
}cncBTVCADepositInfoList_t;

/* Consumer records */
typedef struct _cncBTVCACostInfo_t
{
    CNC_UINT16 time;                                          /*  spending time  */
    CNC_UINT8  cost;                                          /* cost */
      char ProviderName[CNC_MAX_PROVIDER_NAME_LENGTH];   /* Vendor */
}cncBTVCACostInfo_t;

typedef struct _cncBTVCACostInfoList_t
{
    CNC_SINT32 CostInfo_count;
    cncBTVCACostInfo_t  CostList[CNC_MAX_COST_COUNT];
}cncBTVCACostInfoList_t;


/* Email information*/
typedef struct _cncBTVCAEmailInfo_t
{
    CNC_SINT32  readtag;                               /* Message status 0 - unread,1 - read */
    CNC_UINT16  uMsgID;                    /* Message ID */
    CNC_UINT16  uTime;                     /* Send date */
    CNC_UINT16  uProvID;                         /* Vendor ID */
    char  SenderName[CNC_MAX_SENDER_NAME_LENGTH];/* Sender name */
    char    title_char[CNC_MAX_EMAIL_TITLE+1];     /* Message header */
    char    contents[CNC_MAX_EMAIL_CONTENT+1];     /*Message content*/
    CNC_UINT8   priority;                              /* Priority: 0 is normal, 1 is an emergency, and 3 for LSI user information */
    CNC_UINT8    delect_flag;                           /* Delete flag: 0 by default, a deletion (not shown) */
}cncBTVCAEmailInfo_t;

typedef struct _cncBTVCAEmailInfoList_t
{
    CNC_SINT32         EmailCount;                      /* Number of messages */
      cncBTVCAEmailInfo_t   EmailList[CNC_MAX_MAIL_COUNT];   /* email list*/

}cncBTVCAEmailInfoList_t;

/* Effective working time */
typedef struct _cncBTVCAWorkTime_t
{
    CNC_BOOL bEnableWorkHourLimit;
    CNC_UINT16      sWorktime;
    CNC_UINT16      eWorktime;
}cncBTVCAWorkTime_t;

/* User information */
typedef struct _cncBTVCAUserInfo_t
{
    CNC_UINT8  uAgegrade;      /* Age Level */
    CNC_UINT16 uPreResetPsw;    /* Last reset password */
    CNC_UINT16 uPassword;      /* User password */
    CNC_UINT16 uRegionCode;    /* User area code*/
    CNC_BOOL uLimTag;        /*Limit broadcast flag*/
    CNC_BOOL rLimTag;        /* Regional restrictions broadcast flag */
}cncBTVCAUserInfo_t;        /* Uer characteristic information structure */

//DepositInfo DepositInfoList_t

#define CAM_MAX_CAT_EMM 8
typedef struct
{
      CNC_UINT16 SystemId[CAM_MAX_CAT_EMM];
      CNC_UINT16 EmmPid[CAM_MAX_CAT_EMM];
}cncBTVCATEMM_t;

#define CAM_MAX_PMT_ECM 8
typedef struct
{
      CNC_UINT16 SystemId[CAM_MAX_PMT_ECM];
      CNC_UINT16 EcmPid[CAM_MAX_PMT_ECM];
      CNC_UINT16 AudioSystemId[CAM_MAX_PMT_ECM];
      CNC_UINT16 AudioEcmPid[CAM_MAX_PMT_ECM];
      CNC_UINT16 VideoSystemId[CAM_MAX_PMT_ECM];
      CNC_UINT16 VideoEcmPid[CAM_MAX_PMT_ECM];
}cncBTVPMTECM_t;

/**
 * Initialize the conditional access module; CA functions are determined by linked CA library
 * @return
 */
int CNC_CA_Init(void);

/**
 * Start the conditional access service
 * @param srv BTV Service Information
 * @return
 */
int CNC_CA_StartService(cncBTVServiceInfo_t *srv);

/**
 * Stop the conditional access service
 * @return
 */
int CNC_CA_StopService(void);

/*(10 QAM16, 20 QAM32, 40 QAM64, 80 QAM128, 00 QAM256) or (0 DVBT,1 DMBT) or (S Polar_H)*/
typedef enum _cncBTVOTADemodulation_t {
    OTA_DEMOD_QAM16 = 10, OTA_DEMOD_QAM32 = 20, OTA_DEMOD_QAM64 = 40, OTA_DEMOD_QAM128 = 80, OTA_DEMOD_QAM256 = 00
} cncBTVOTADemodulation_t;

typedef struct OTA_PARA_s {
    CNC_UINT16 pid;
    CNC_UINT16 tableid;
    CNC_UINT16 frequence;
    CNC_UINT16 updata_file_type;/* 1=KERNEL    4=ROOTFS*/
    CNC_UINT16 demod;
    CNC_UINT16 symble;
    CNC_UINT16 new_version;
    CNC_UINT16 clear_usrdb;
} cncBTVOTA_t;

#define OTA_TUNER_TYPE 0
#define OTA_RETRY_DELAY 60000000     // us,   1m

typedef struct _cncBTVOTAInformation_t
{
    CNC_UINT8 maker_id;
    CNC_UINT8 model_id;
    CNC_UINT8 sdtt_version;
    CNC_UINT8 reserve1;
    CNC_UINT16 sw_version;
    CNC_UINT16 reserve2;
}cncBTVOTAInformation_t;

typedef struct _cncBTVOTAParam_t
{
    CNC_UINT8 flag;
    CNC_UINT8 reboot;
    CNC_UINT8 add_on;
    CNC_UINT8 sdtt_version_num;
    CNC_UINT32 fre; // kHz
    CNC_UINT16 pid;
    CNC_UINT16 version_num;

    CNC_UINT16 reserve1;
    CNC_UINT16 reserve2;
}cncBTVOTAParam_t;

typedef CNC_BOOL (*cncBTVOTACallBack_t)(CNC_UINT32 flag, CNC_UINT32 new_ver, CNC_UINT32 old_ver);

/**
 * Set the Current OTA (over-the-air update) Information from EEPROM
 * @param ota_info OTA Information
 * @param offset EEPROM offset
 * @return
 */
CNC_SINT32 CNC_OTA_SetCurInfo(const cncBTVOTAInformation_t * ota_info , CNC_SINT32 offset);

/**
 * Get Current OTA Info from EEPROM
 * @param offset  EEPROM offset
 * @return
 */
cncBTVOTAInformation_t * CNC_OTA_GetCurInfo(CNC_SINT32 offset);

/**
 * Set OTA Updata Param into EEPROM
 * @param para
 * @param offset
 * @return
 */
CNC_SINT32 CNC_OTA_SetParam(cncBTVOTAParam_t *para, CNC_SINT32 offset);

/*OTA Init*/
/**
 * Initialize over-the-air update module
 * @param callback Callback for OTA
 * @return
 */
int CNC_OTA_Init(cncBTVOTACallBack_t callback);

/**
 * Pause the OTA monitor (should be used before a channel search)
 */
void CNC_OTA_Pause(void);

/**
 * Resume OTA monitoring
 */
void CNC_OTA_Resume(void);

/**
 * Get the OTA update flag
 * @return
 */
CNC_BOOL CNC_OTA_GetUpdateFlag(void);

Result_Type_e CNC_Diseqc_StartContinuous(int fd);
Result_Type_e CNC_Diseqc_StopContinuous(int fd);
Result_Type_e CNC_Diseqc_SendModulationData(int fd,unsigned char *ucData,unsigned char numb);
Result_Type_e CNC_Diseqc_ReadModulationData(int fd,unsigned char *ucData, unsigned char *numb);
Result_Type_e CNC_Diseqc_SelectModulationPort(int fd,unsigned char port );
//direction 0 east ; 1 west
Result_Type_e CNC_Diseqc_MotorRotationcontrol(int fd,unsigned char direction);

#include "cnc_btv/cnc_pvr.h"

#ifdef __cplusplus
}
#endif

#endif /* CNC_BTV_H_ */
