/*
# COPYRIGHT NOTICE
#
#***************************************************************************

The following software code is the property of Cavium, Inc.
Cavium, Inc. reserves all rights in this software, including but
not limited to copyright and shall be vested with all rights incidental
thereto.

Rights granted to you by or under any license shall be limited to those
explicitly granted. You may not record, store, reproduce, reverse engineer,
decompile, inter-operate and/or transmit this software in any medium,
electronic or otherwise, without the express written consent of Cavium, Inc.

ALL RIGHTS RESERVED.

Copyright 2012, Cavium, Inc.
#***************************************************************************
#
# END OF COPYRIGHT NOTICE
#
*/

/**
 * Provides an API to common crypto functions.
 * @file cnc_crypto.h
 */

#ifndef _CNC_CRYPTO_H_
#define _CNC_CRYPTO_H_

#ifdef __cplusplus
extern "C" {
#endif

/* ============================ System Includes =========================== */
// NONE

/* ============================= User Includes ============================ */
#include "cnc_common_typedef.h"
#include "cnc_common_error.h"

/* =================================== Macros ============================= */
#define CNC_CRYPTO_AES_IV_SIZE    16
#define CNC_CRYPTO_AES_KEY_SIZE   16 // 128 

/*!
 *  \brief
 */
typedef enum _cncCryptoAesMode_t
{
    CNC_CRYPTO_AES_MODE_NONE         = 0,
    CNC_CRYPTO_AES_MODE_128_CBC      = 1,
    CNC_CRYPTO_AES_MODE_192_CBC,
    CNC_CRYPTO_AES_MODE_256_CBC,
    CNC_CRYPTO_AES_MODE_128_OFB,
    CNC_CRYPTO_AES_MODE_128_CTR,

    CNC_CRYPTO_AES_MODE_ECB,  //TODO bit size
    CNC_CRYPTO_AES_MODE_CTR,
    CNC_CRYPTO_AES_MODE_OCB,
    CNC_CRYPTO_AES_MODE_CFB
} cncCryptoAesMode_t;

/*!
 *  \brief
 */
typedef enum _cncCryptoHashAlgo_t
{
    CNC_CRYPTO_HASH_ALGO_MD2,
    CNC_CRYPTO_HASH_ALGO_MD4,
    CNC_CRYPTO_HASH_ALGO_MD5,
    CNC_CRYPTO_HASH_ALGO_SHA1,
    CNC_CRYPTO_HASH_ALGO_SHA224,
    CNC_CRYPTO_HASH_ALGO_SHA256,
} cncCryptoHashAlgo_t;

/*!
 *  \brief
 */
typedef enum _cncCryptoTextType_t
{
   CNC_CRYPTO_PLAIN_TEXT_TYPE		= 0,
   CNC_CRYPTO_CIPHER_TEXT_TYPE
} cncCryptoTextType_t;

/*!
 *  \brief
 */
typedef struct _cncCryptoText_t
{
    cncCryptoTextType_t	 textType;
    CNC_UINT8           *p_text;
    CNC_SINT32	         len;
} cncCryptoText_t;

/*!
 *  \brief
 */
typedef enum _cncCryptoSigningAlgo_t
{
    //TODO   Add Standard Curve
    CNC_CRYPTO_SIGNING_ALGO_DSA		= 0,
    CNC_CRYPTO_SIGNING_ALGO_ECDSA
} cncCryptoSigningAlgo_t;


/*!
 *  \brief
 */
typedef struct _cncCryptoAESInfo_t
{
    CNC_UINT8           a_iv[CNC_CRYPTO_AES_IV_SIZE];
    CNC_UINT8           a_key[CNC_CRYPTO_AES_KEY_SIZE];
    CNC_BOOL            reset;                         //!< TRUE: AES Counter will be reset on every Encryption/Decryption 
    cncCryptoAesMode_t  mode;
} cncCryptoAESInfo_t;


/*!
 *  \func         CNC_Crypto_Initialize 
 *  \brief        Initializes Crypto      
 *  \param [in]    
 *  \param [out]  
 *  \return
 */
E_cncErrCode_t  CNC_Crypto_Initialize(CNC_VOID);

/*!
 *  \func         CNC_Crypto_DeInitialize 
 *  \brief        
 *  \param [in]    
 *  \param [out]  
 *  \return
 */
E_cncErrCode_t  CNC_Crypto_DeInitialize(CNC_VOID);

/*!
 *  \func         CNC_Crypto_Create 
 *  \brief        
 *  \param [in]    
 *  \param [out]  
 *  \return
 */
// TODO: Need to pass the Encrption/Decryption Type
E_cncErrCode_t  CNC_Crypto_Create(CNC_VOID **cryptoHandle, ...);

/*!
 *  \func         CNC_Crypto_Destroy 
 *  \brief        
 *  \param [in]    
 *  \param [out]  
 *  \return
 */
E_cncErrCode_t  CNC_Crypto_Destroy(CNC_VOID *cryptoHandle);


/*!
 *  \func         CNC_Crypto_CreateHash 
 *  \brief        
 *  \param [in]    
 *  \param [out]  
 *  \return
 */
E_cncErrCode_t  CNC_Crypto_CreateHash(CNC_VOID *cryptoHandle, cncCryptoHashAlgo_t hashAlgorithm, cncCryptoText_t *plainText, cncCryptoText_t *hashText);

/*!
 *  \func        CNC_Crypto_AesGeneratekeyIV 
 *  \brief         
 *  \param [in]    
 *  \param [out]  
 *  \return
 */
// TODO: Need to pass Cipher mode
E_cncErrCode_t  CNC_Crypto_AesGeneratekeyIV(CNC_UINT8 *keydata, CNC_SINT32 keydatalen, CNC_UINT8 key[CNC_CRYPTO_AES_KEY_SIZE], CNC_UINT8 iv[CNC_CRYPTO_AES_IV_SIZE]);

/*!
 *  \func        CNC_Crypto_AesEncrypt 
 *  \brief        
 *  \param [in]    
 *  \param [out]  
 *  \return
 */
E_cncErrCode_t  CNC_Crypto_AesEncrypt(CNC_VOID *cryptoHandle, cncCryptoAESInfo_t  *aesInfo, cncCryptoText_t *plainText, cncCryptoText_t *cipherText);

/*!
 *  \func       CNC_Crypto_AesDecrypt 
 *  \brief        
 *  \param [in]    
 *  \param [out]  
 *  \return
 */
E_cncErrCode_t  CNC_Crypto_AesDecrypt(CNC_VOID *cryptoHandle, cncCryptoAESInfo_t  *aesInfo, cncCryptoText_t	*cipherText, cncCryptoText_t *plainText);

/*!
 *  \func         CNC_Crypto_RandInit 
 *  \brief        
 *  \param [in]    
 *  \param [out]  
 *  \return
 */
E_cncErrCode_t  CNC_Crypto_RandInit();    //Move this into crypto Init ?

/*!
 *  \func         CNC_Crypto_GetRandomNumber 
 *  \brief        
 *  \param [in]    
 *  \param [out]  
 *  \return
 */
E_cncErrCode_t  CNC_Crypto_GetRandomNumber(CNC_VOID *cryptoHandle, CNC_SINT32 len, CNC_UINT8 *randomNum);


/*!
 *  \func         CNC_Crypto_GetRandomNumberEx 
 *  \brief        
 *  \param [in]    
 *  \param [out]  
 *  \return
 */
E_cncErrCode_t  CNC_Crypto_GetRandomNumberEx(CNC_VOID *cryptoHandle, CNC_UINT8 *maxBoundary, CNC_SINT32 len, CNC_UINT8 *randomNum);


/*!
 *  \func         CNC_Crypto_CreateSignature 
 *  \brief        Do Signing
 *  \param [in]    
 *  \param [out]  
 *  \return
 */
E_cncErrCode_t  CNC_Crypto_CreateSignature(CNC_VOID *cryptoHandle, cncCryptoSigningAlgo_t signingAlgo, cncCryptoText_t *plainText, CNC_UINT8 *key, CNC_UINT8 *signature);


/*!
 *  \func        CNC_Crypto_VerifySignature  
 *  \brief       Verify Signing 
 *  \param [in]    
 *  \param [out]  
 *  \return
 */
E_cncErrCode_t  CNC_Crypto_VerifySignature(CNC_VOID *cryptoHandle, cncCryptoSigningAlgo_t signingAlgo, cncCryptoText_t *plainText, cncCryptoText_t *signature, CNC_UINT8 *key);

/*!
 *  \func         CNC_Crypto_ECDH_phaseValue 
 *  \brief        Key Exchange 
 *  \param [in]    
 *  \param [out]  
 *  \return
 */
E_cncErrCode_t  CNC_Crypto_ECDH_phaseValue(CNC_VOID *cryptoHandle, CNC_UINT8  *privKey, CNC_UINT8 *phase1Value);


/*!
 *  \func         CNC_Crypto_ECDH_SharedKey 
 *  \brief         
 *  \param [in]    
 *  \param [out]  
 *  \return
 */
E_cncErrCode_t  CNC_Crypto_ECDH_SharedKey(CNC_VOID *cryptoHandle, CNC_UINT8  *phase1Value, CNC_UINT8 *privKey, CNC_UINT8 *sharedKey);

#ifdef __cplusplus
}
#endif

#endif /* END: _CNC_CRYPTO_H_*/
