/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

The following software code is the property of Cavium Networks.
Cavium Networks reserves all rights in this software, including but
not limited to copyright and shall be vested with all rights incidental
thereto.

Rights granted to you by or under any license shall be limited to those
explicitly granted. You may not record, store, reproduce, reverse engineer,
decompile, inter-operate and/or transmit this software in any medium,
electronic or otherwise, without the express written consent of Cavium Networks.

ALL RIGHTS RESERVED.

Copyright 2011, Cavium Network.
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */

/**
 * CRISP DLNA common types and headers
 * @file cnc_dlna.h
 * @ingroup dlna
 */

#ifndef _CNC_DLNA_COMMON_H_
#define _CNC_DLNA_COMMON_H_

#ifdef __cplusplus
extern "C" {
#endif

/* ============================ System Includes =========================== */
// NONE

/* ============================= User Includes ============================ */
#include "cnc_common_typedef.h"

/* =========================== Macro definitions ========================== */
#define CNC_DNSR_MAX_TITLE_LENTH                       (256)
#define CNC_DNSR_MAX_IP_LENGTH                         (24)
#define CNC_DNSR_MAX_UUID_LENTH                        (48)

/* =========================== Type definitions =========================== */
typedef enum _cncDlnaErrCode_t
{
    E_CNC_DLNA_ERR_CODE_NONE           = 0,         /**< 0 */
    E_CNC_DLNA_ERR_CODE_INVALID_PARAM  = -500,      /**< -500 */
    E_CNC_DLNA_ERR_CODE_NETWORK,                    /**< -499 */
    E_CNC_DLNA_ERR_CODE_INVALID_REQUEST,            /**< -498 */
    E_CNC_DLNA_ERR_CODE_NO_MEMORY,                  /**< -497*/
    E_CNC_DLNA_ERR_CODE_FILE_IO,                    /**< -496*/
    E_CNC_DLNA_ERR_CODE_PTHREAD,                    /**< -495*/
    E_CNC_DLNA_ERR_CODE_LOWER_LAYER,                /**< -494*/
    E_CNC_DLNA_ERR_CODE_FORMAT_INVALID,             /**< -493*/
    E_CNC_DLNA_ERR_CODE_CONNECTION_FAILED,          /**< -492*/
    E_CNC_DLNA_ERR_CODE_SERVER_NOT_RESPONDING,      /**< -491*/
    E_CNC_DLNA_ERR_CODE_SERVER_NOT_FOUND,           /**< -490 */
    E_CNC_DLNA_ERR_CODE_CONTENT_NOT_FOUND,          /**< -489 */
    E_CNC_DLNA_ERR_CODE_NW_TIMEOUT,                 /**< -488*/
    E_CNC_DLNA_ERR_CODE_NO_DATA,                    /**< -487 */
    E_CNC_DLNA_ERR_CODE_AUTHORIZATION_FAILED,       /**< -486 */
    E_CNC_DLNA_ERR_CODE_PROTECTED_CONTENT,          /**< -485 */
    E_CNC_DLNA_ERR_CODE_BUSY,                       /**< -484 */
    E_CNC_DLNA_ERR_CODE_API_SEQUENCE,               /**< -483 */
    E_CNC_DLNA_ERR_CODE_INVALID_RESOURCE_PATH,      /**< -482 */
    E_CNC_DLNA_ERR_CODE_INVALID_SHARE_CONFIG,       /**< -481 */
    E_CNC_DLNA_ERR_CODE_KEY_NOT_FOUND,              /**< -480 */
    E_CNC_DLNA_ERR_CODE_PTHREAD_MUTEX_INIT,         /**< -479 */
    E_CNC_DLNA_ERR_CODE_UPNP_LIB_ERROR,             /**< -478 */
    E_CNC_DLNA_ERR_CODE_UNSUPPORTED_STAT_VAR,       /**< -477 */
    E_CNC_DLNA_ERR_CODE_UNSUPPORTED_SERVICE,        /**< -476 */
    E_CNC_DLNA_ERR_CODE_UNSUPPORTED_ACTION,         /**< -475 */
    E_CNC_DLNA_ERR_CODE_UNSUPPORTED_FEATURE,        /**< -474 */
    E_CNC_DLNA_ERR_CODE_INVALID_OBJECT_ID,          /**< -473 */
    E_CNC_DLNA_ERR_CODE_INVALID_STATE,              /**< -472 */
    E_CNC_DLNA_ERR_CODE_PROTOCOL_INFO_NOT_MATCHED,  /**< -471 */

    /* Note: Add new errors above this comment */
    E_CNC_DLNA_ERR_CODE_UNKNOWN_ERROR
} cncDlnaErrCode_t;


/**
 * \enum : _cncDlnaConfigMode_t
 * \brief: DLNA Device Instance Specific Mode.
 */
typedef enum _cncDlnaConfigModeTag
{
    E_DLNA_INSTANCE_MODE_NONE       = 0,    /**< DLNA Instance Mode is not set.*/
    E_DLNA_INSTANCE_MODE_SINGLE     = 1,    /**< Single DLNA Instance Mode is set.*/
    E_DLNA_INSTANCE_MODE_MULTIPLE   = 2,    /**< Multiple DLNA Instance Mode is set.*/
    E_DLNA_INSTANCE_MODE_PLAYTO_EXT = 3     /**< Single DLNA Instance with PlayTo supported Mode is set.*/

    //TODO E_DLNA_INSTANCE_MODE_MULTI_PLAYTO_EXT

}cncDlnaConfigMode_t;


/**
 * \struct : cncDlnaDeviceConfig_t
 * \brief  : Device Configuration
 */
typedef struct _cncDlnaDeviceConfig_t
{
    CNC_CHAR   a_ch_deviceManufacturer[CNC_DNSR_MAX_TITLE_LENTH];       /**< Device Manufacturer Name */
    CNC_CHAR   a_ch_deviceManufacturerUrl[CNC_DNSR_MAX_TITLE_LENTH];    /**< Device Manufacturer URL */
    CNC_CHAR   a_ch_deviceModelName[CNC_DNSR_MAX_TITLE_LENTH];          /**< Device Model Name*/
    CNC_CHAR   a_ch_deviceModelDescription[CNC_DNSR_MAX_TITLE_LENTH];   /**< Device Model Description*/
    CNC_CHAR   a_ch_deviceModelNumber[CNC_DNSR_MAX_TITLE_LENTH];        /**< Device Model Number*/
    CNC_CHAR   a_ch_deviceModelUrl[CNC_DNSR_MAX_TITLE_LENTH];           /**< Device Model URL*/
    CNC_CHAR   a_ch_deviceSerialNumber[CNC_DNSR_MAX_TITLE_LENTH];       /**< Device Serial Number*/
    CNC_CHAR   a_u8_UUID[CNC_DNSR_MAX_UUID_LENTH];                      /**< Device UUID */

    //MicroSoft Specific PnP_X.
    CNC_CHAR   a_ch_X_HadwareID[CNC_DNSR_MAX_TITLE_LENTH];              /**< Device X_HardwareID with format VEN_xxxx&DEV_xxxx&REV_xx */
    CNC_CHAR   a_ch_X_ModeID[CNC_DNSR_MAX_UUID_LENTH];                  /**< Device X_ModelID with format should be similar to UUID  [xxxxxxxx-xxxx-xxxx-xxxxxxxxxxxx]*/
    CNC_CHAR   a_ch_X_CompatibleID[CNC_DNSR_MAX_TITLE_LENTH];           /**< X_CompatibleID like MS_DigitalMediaDeviceClass_DMR_V001*/

}cncDlnaDeviceConfig_t;

 /**
 * \struct : _cncDlnaConfig_t
 * \brief  : DNSR Configuration
 * */
typedef struct _cncDlnaConfig_t
{
    cncDlnaConfigMode_t     e_confMode;                                     /**< DLNA Configuration Mode Flag */
    CNC_CHAR                a_ch_friendlyName[CNC_DNSR_MAX_TITLE_LENTH];    /**< Device Friendly Name */
    CNC_CHAR                a_ch_networkIfName[CNC_DNSR_MAX_TITLE_LENTH];
    CNC_CHAR                a_ch_networkIP[CNC_DNSR_MAX_IP_LENGTH];
    CNC_SINT32              s32_upnpPort;                                   /**< UPnP Port*/
    CNC_SINT32              s32_numOfInstances;                             /**< Multiple Instances if confMode set to MULTI_INSTACE.*/
    CNC_SINT32              s32_subScribeTimeOut;                           /**< Subscribe in Sec[900, 1800]*/
    CNC_CHAR                a_resourcePath[CNC_DNSR_MAX_TITLE_LENTH];       /**< Path for Configuration XML files, Device Icons. */    

} cncDlnaConfig_t;


/* DNSR Current Status */
typedef enum _cncDlnaCurrentStatus_t
{
    E_CNC_DLNA_STATUS_INITIALIZED  = 0,    /**< 0 */
    E_CNC_DLNA_STATUS_STARTED,             /**< 1*/
    E_CNC_DLNA_STATUS_STOPPED,             /**< 2*/

}cncDlnaCurStatus_t;

/* DNSR Current Configured and running information */
typedef struct _cncDlnaCurrentInfo_t
{
    CNC_CHAR               a_ch_uuid[CNC_DNSR_MAX_UUID_LENTH];
    CNC_CHAR               a_ch_friendlyName[CNC_DNSR_MAX_TITLE_LENTH];
    cncDlnaCurStatus_t     e_curStatus;
}cncDlnaCurInfo_t;


/* UUID Information */
typedef struct _cncDlnaUuid_t
{
    cncDlnaErrCode_t    e_errCode;
    CNC_CHAR            a_u8_UUID[CNC_DNSR_MAX_UUID_LENTH];
}cncDlnaUuid_t;

/* ======================== Function declarations ========================= */
// NONE

#ifdef __cplusplus
}
#endif

#endif /* END: _CNC_DLNA_COMMON_H_ */

