/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Monta-Vista.
 Monta-Vista reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Monta-Vista

 ALL RIGHTS RESERVED.

 Copyright 2011-2012, Monta-Vista
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */
/**
 * @file cnc_dmr.h
 * @brief CRISP DLNA Digital Media Receiver Module API
 * @ingroup dlna
 */

#ifndef _CNC_DMR_API_H_
#define _CNC_DMR_API_H_

#ifdef __cplusplus
extern "C" {
#endif

/* ============================ System Includes =========================== */
// NONE

/* ============================= User Includes ============================ */
#include "cnc_common_typedef.h"
#include "cnc_mediainfo.h"
#include "cnc_dlna.h"

/* =========================== Macro definitions ========================== */
/* Device Description Params */
#define DMR_DEFAULT_FRIENDLY_NAME                           "CNC_DLNA_DMR"
#define DMR_DEFAULT_X_HARDWARE_ID                           "VEN_0B21&amp;DEV_0011&amp;REV_01"
#define DMR_DEFAULT_X_MODEL_ID                              "0F2a8B63-C4d7-Ac43-Eb8347501af9"
#define DMR_DEFAULT_X_COMPATIBLE_ID                         "MS_DigitalMediaDeviceClass_DMR_V001"
#define DMR_DEFAULT_DEV_MANUFACTURER                        "Cavium Inc"
#define DMR_DEFAULT_DEV_MANUFACTURER_URL                    "www.cavium.com"
#define DMR_DEFAULT_DEV_MODEL_NAME                          "TEST"
#define DMR_DEFAULT_DEV_MODEL_DESC                          "DMR"
#define DMR_DEFAULT_DEV_MODEL_URL                           "www.cavium.com"
#define DMR_DEFAULT_DEV_MODEL_NUM                           "v1.0"
#define DMR_DEFAULT_DEV_SERIAL_NUM                          "123"

/* =========================== Type definitions =========================== */
/* DMR event types */
typedef enum _cncDmrEventType_t
{
    E_CNC_DMR_EVENT_TYPE_NONE                 = 0,
    E_CNC_DMR_EVENT_TYPE_REQUEST,
    E_CNC_DMR_EVENT_TYPE_ERROR_NOTIFICATION,
} cncDmrEventType_t;

typedef enum _cncDmrRequestType_t
{
    E_CNC_DMR_REQUEST_NONE                    = 0,
    E_CNC_DMR_REQUEST_PLAYBACK,
    E_CNC_DMR_REQUEST_STOP,
    E_CNC_DMR_REQUEST_RENDERER_CONFIG,
    E_CNC_DMR_REQUEST_SET_AV_URI,
    E_CNC_DMR_REQUEST_PREPARE_NEXT_AVI_URI,
    E_CNC_DMR_REQUEST_PLAY_NEXT

} cncDmrRequestType_t;

/* DMR request info */
typedef struct _cncDmrRequestInfo_t
{
    cncDmrRequestType_t             e_type;
    cncMediaCtlInfo_t               s_mediaCtrlInfo;
} cncDmrRequestInfo_t;

/* DMR Notified event's Information */
typedef struct _cncDmrEventInfo_t
{
    cncDmrEventType_t               eventType;
    CNC_UINT32                      u32_instanceId;
    union _cncDmrEventData_t
    {
        cncDmrRequestInfo_t         s_requestInfo;
        cncDlnaErrCode_t            e_errCode;
    } eventData;
} cncDmrEventInfo_t;


/* DMR Media Info Query Structure */
typedef struct _cncDmrMediaInfoReq_t
{
    CNC_UINT32                      u32_instanceId;
    cncMediaInfoFlag_t              e_reqInfo;

}cncDmrMediaInfoReq_t;

typedef struct _cncDmrMediaInfo_t
{
    CNC_UINT32                      u32_instanceId;
    cncMediaCtlInfo_t               mediaCtlInfo;

}cncDmrMediaInfo_t;

/* DMR Instance Handle */
typedef void* cncDmrHandle_t;

/* Callback pointer for Event Notification From DMR */
typedef void (*cncDmrEventCbf_t)(cncDmrEventInfo_t  s_eventInfo);

/* Callback pointer for Get Playback Information request from MediaPlayback Module */
typedef cncMediaCtlInfo_t (*cncDmrGetMediaCtlInfoCbf_t)(cncDmrMediaInfoReq_t   s_medaiInfoReq);

/* ======================== Function declarations ========================= */
/* DMR Initialization
 * maxDmrInstances:: Maximum number of Instances can be created for DMR Given by Application.
 *                   MinValue :: 1
 *                   MaxValue :: Dependent on platform used
 *                   Returns error if  maxDmrInstances < 1
 * */
cncDlnaErrCode_t   CNC_DMR_Init(CNC_SINT32 maxDmrInstances);

/* Create a DMR instance
 *  F_cncDmrEventCbf_t           => Callback registered by application to get Event from DMR
 *  F_cncDmrGetMediaCtlInfoCbf_t => Callback registered by Application to get Playback status from Playback control Module.
 */
cncDlnaErrCode_t   CNC_DMR_Create(cncDmrHandle_t                **pp_s_dmrHandle,
                                  cncDlnaConfig_t               s_serviceConfig,
                                  cncDlnaDeviceConfig_t         s_deviceConfig,
                                  cncDmrEventCbf_t              p_f_dmrEventNotify,
                                  cncDmrGetMediaCtlInfoCbf_t    p_f_dmrGetMediaCtlInfo);

/* Run the created DMR instance */
cncDlnaErrCode_t   CNC_DMR_Start(cncDmrHandle_t *p_s_dmrHandle);

/* Stop the created DMR instance */
cncDlnaErrCode_t   CNC_DMR_Stop(cncDmrHandle_t *p_s_dmrHandle);


/* Set DMR playback status and renderer configuration information */
cncDlnaErrCode_t   CNC_DMR_SetMediaCtlInfo(cncDmrHandle_t *p_s_dmrHandle, cncDmrMediaInfo_t s_dmrMediaInfo);

/* Get DMR configured and playing information */
cncDlnaCurInfo_t   CNC_DMR_GetConfigInfo(cncDmrHandle_t  *p_s_dmrHandle);

/* Destroy the DMR instance */
cncDlnaErrCode_t   CNC_DMR_Destroy(cncDmrHandle_t *p_s_dmrHandle);

/* DMR De-Initialization */
cncDlnaErrCode_t   CNC_DMR_DeInit(void);

/* Create UUID */
cncDlnaUuid_t      CNC_DMR_CreateUuid(void);


#ifdef __cplusplus
}
#endif

#endif /* END: _CNC_DMR_API_H_ */

