/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Cavium, Inc.
 Cavium, Inc. reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium, Inc.

 ALL RIGHTS RESERVED.

 Copyright 2011-2012, Cavium, Inc.
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */
/*
 * @file cnc_icomm.h
 * @ingroup icomm
 * The CNCICOMM library is used to implement IP communications including video and voice conferencing, MMS
 */

#ifndef CNC_ICOMM_H_
#define CNC_ICOMM_H_

#include "cnc_common_typedef.h"

#ifdef __cplusplus
extern "C" {
#endif

#define   CNC_ICOMM_MESSAGE_QUEUE_ID              57005
#define   CNC_ICOMM_LN_FRIEND_LIST_MSG_QUEUE_ID   57125

/* For Friend List Info */
typedef struct _cncICommFriendInfo_t
{
    char    displayName[256];  /* Display name */
#if 0
    char    scheme[10];        /* Specified Scheme - Eg: sip */
    char    userName[256];     /* User Name of the Friend  */
    char    password[256];     /* Password for this Friend */
    char    host[256];         /* Specified Host name or IP Address */
    char    port[10];          /* Port */
    char    userstring[256];       /* TODO: Need to find out , what it gives */
#endif
} cncICommFriendInfo_t;

typedef struct _cncIComm_FriendList_t
{
    int                       numberOfFriends;    /* Specified No Of Friends information available */
    cncICommFriendInfo_t      friendInfo[8];    /* Currently Limiting to 1 members */
} cncICommFriendList_t;

typedef enum _cncICommEventType_t
{
    CNC_ICOMM_EVENT_NONE,
    CNC_ICOMM_EVENT_CALL_RECEIVED,
    CNC_ICOMM_EVENT_CALL_BYE,
    CNC_ICOMM_EVENT_PRESENCE_RECEIVED,
    CNC_ICOMM_EVENT_UNKNOWN_SUBSCRIBER,
    CNC_ICOMM_EVENT_DISPLAY_STATUS,
    CNC_ICOMM_EVENT_TEXT_RECEIVED,
    CNC_ICOMM_EVENT_GET_FRIEND_LIST,
    CNC_ICOMM_EVENT_CALLEE_BUSY,
    CNC_ICOMM_EVENT_CALLEE_OFFLINE,
    CNC_ICOMM_EVENT_END
} cncICommEventType_t;

typedef union _cncICommMsgData_t
{
    char data[256];
    cncICommFriendList_t friendList;
} cncICommMsgData_t;

typedef struct _cncICommMessageData_t
{
    cncICommEventType_t  eventType;
    cncICommMsgData_t    messageData;
} cncICommMessageData_t;

/* Common Message Data */
typedef struct _cncICommMessage_t
{
    signed int                messageType;
    cncICommMessageData_t     messageData;
} cncICommMessage_t;

void CNC_IComm_Init(void);
void CNC_IComm_ConfigService(char *user, char *serverIP);
void CNC_IComm_StartService(void);
void CNC_IComm_StopService(void);
void CNC_IComm_Call(char *name);
void CNC_IComm_Answer(void);
void CNC_IComm_Terminate(void);
void CNC_IComm_SetupDisplay(void);
void CNC_IComm_CloseDisplay(void);
CNC_UINT8 CNC_IComm_CheckEvents(void);
int CNC_IComm_SaveSetting(const char *User_Name,const char *Ip_Add);
char * CNC_IComm_GetUserName(void);
char * CNC_IComm_GetIPAddr(void);
CNC_UINT8 CNC_IComm_IsActive(void);

#ifdef __cplusplus
}
#endif


#endif /* CNC_ICOMM_H_ */
