/*
# COPYRIGHT NOTICE
#
#***************************************************************************

The following software code is the property of Cavium, Inc.
Cavium, Inc. reserves all rights in this software, including but
not limited to copyright and shall be vested with all rights incidental
thereto.

Rights granted to you by or under any license shall be limited to those
explicitly granted. You may not record, store, reproduce, reverse engineer,
decompile, inter-operate and/or transmit this software in any medium,
electronic or otherwise, without the express written consent of Cavium, Inc.

ALL RIGHTS RESERVED.

Copyright 2014, Cavium, Inc.
#***************************************************************************
#
# END OF COPYRIGHT NOTICE
#
*/

/**
 * Provides an API to common IR functions.
 * @file cnc_ir.h
 */

#ifndef _CNC_IR_H_
#define _CNC_IR_H_

typedef enum
{
    /* must start with zero and be sequential */
    CNCIR_EVENT_RAW = 0,   /**< Get every code with toggle and biased by 0x30 */
    CNCIR_EVENT_COOKED,    /**< Get only new codes (at least toggle changed) bias and toggle removed */
    CNCIR_EVENT_PEER_RAW,     /**< Get every code with toggle and biased by 0x30 */
    CNCIR_EVENT_PEER_COOKED,  /**< Get only new codes (at least toggle changed) bias and toggle removed */

    CNCIR_NUM_EVENTS /* Must be last */
} CNCIR_EVENT;

typedef enum
{
	CNCIR_SUCCEED = 0,   /**< Successful result */
	CNCIR_FAILED = -1    /**< Unuccessful result */
} CNCIR_RESULT;

typedef int (*pf_CNCIREventCallBack)(CNCIR_EVENT event, unsigned int code);   /**< Callback prototype */

/**
 * Register an IR callback
 * @param event             [in]  event requiring a callback
 * @param callback          [in]  callback function to be registered
 * @return CNCIR_RESULT     [out] returns error code
 */
CNCIR_RESULT CNCIR_registerCallback(CNCIR_EVENT event, pf_CNCIREventCallBack callback);

/**
 * Send an IR code to the peer
 * @param code             [in]  biased (by 0x30 with toggle in bit 7) RC-5 IR code to send to peer
 * @param callback         [in]  callback function to be registered
 * @return CNCIR_RESULT    [out] returns error code
 */
CNCIR_RESULT CNCIR_ir_write_peer(CNC_UINT8 code);

#endif //_CNC_IR_H_
