/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Cavium, Inc.
 Cavium, Inc. reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium, Inc.

 ALL RIGHTS RESERVED.

 Copyright 2011-2012, Cavium, Inc.
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */

/**
 * @file cnc_metadata.h
 * Find and download metadata from the internet
 * @ingroup media
 */

#include "cnc_http.h"
#include "libxml/parser.h"
#include "libxml/tree.h"

typedef enum _cncMetadataErrCode_t
{
    E_METADATA_ERROR_NONE            = 0,
    E_METADATA_ERROR_NO_RESPONSE=1,
    E_METADATA_ERROR_META_DATA_NULL,
    E_METADATA_ERROR_NO_CONDITION,
    E_METADATA_ERROR_DOWNLOAD_FAIL,
    E_METADATA_ERROR_URL_NULL,
    E_METADATA_ERROR_MALLOC_FAIL,
    E_METADATA_ERROR_OPEN_XML_FAIL,
    E_METADATA_ERROR_NO_MOVIE_FOUND,
    E_METADATA_ERROR_DB_INIT_FAIL,
    E_METADATA_ERROR_DB_CLEAR_FAIL,


} cncMetadataErrCode_t;

typedef struct _cncMetadata_t
{
    char *title;
    char *type;
    char *imdbID;
    char *imdbVotes;
    char *imdbRating;
    char *poster;
    char *plot;
    char *actors;
    char *writer;
    char *director;
    char *genre;
    char *runtime;
    char *released;
    char *rated;
    char *year;
    char *Website;
    char *Production;
    char *BoxOffice;
} cncMetadata_t;

typedef enum _cncMetadataCondition_t
{
    TITLE=0,
    IMDB_ID=1,
} cncMetadataCondition_t;
typedef void (*CNC_Metadata_DownLoad_cb)(void * pram);

cncMetadataErrCode_t CNC_Metadata_Download(cncMetadata_t **MetaData,char *condition,cncMetadataCondition_t type,CNC_Metadata_DownLoad_cb function);
cncMetadataErrCode_t CNC_MetaData_DB_Clear(void);
cncMetadataErrCode_t CNC_MetaData_DB_Init(void);
cncMetadataErrCode_t CNC_MetaData_Get_from_DB(cncMetadata_t **MetaData,char *title);
cncMetadataErrCode_t CNC_MetaData_Save_to_DB(cncMetadata_t **MetaData);
cncMetadataErrCode_t CNC_MetaData_Remove_from_DB(char *title);
cncMetadataErrCode_t CNC_Metadata_Free(cncMetadata_t **MetaData);
void CNC_Metadata_Download_stop(void);




