/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Cavium, Inc.
 Cavium, Inc. reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium, Inc.

 ALL RIGHTS RESERVED.

 Copyright 2011-2012, Cavium, Inc.
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */

/**
 * @file cnc_network.h
 * @ingroup network
 * The CNCNETWORK library is used to initialize, manage and control LAN and WIFI connections
 */
#ifndef  __CNC_NETWORK_H__
#define  __CNC_NETWORK_H__

#include "cnc_common_typedef.h"

#ifdef __cplusplus
extern "C" {
#endif

#define CHECK_NET_PARAMS_VALID(p) do { \
	if ((NULL == p) { \
		return RESULT_BAD_PARAMETER; \
	} \
} while(0)

#define NETWORK_DEBUG printf


/*family*/
#define CNC_AF_INET            (4)
#define CNC_AF_INET6           (6)

/*type*/
#define CNC_SOCK_STREAM        (6)
#define CNC_SOCK_DGRAM         (17)

/*protocol*/
#define CNC_IPPROTO_IP         (0)

/*connect timeout*/
#define CNC_CONNECT_TIMEOUT         (8)

typedef CNC_UINT32    cncNETIPv4Address_t;

typedef enum
{
    CNC_IPV4 = 4,
    CNC_IPV6 = 6
} cncNETIPVersion_t;

typedef struct
{
	CNC_UINT16 ip[8];
} cncNETIPv6Address_t;

typedef struct
{
	cncNETIPVersion_t  version;
	union {
		cncNETIPv4Address_t ipv4;
		cncNETIPv6Address_t ipv6;
	} addr;
} cncNETIPAddress_t;

typedef enum _cncNETConnect_t {
    CNC_NET_CONNECT_PREINIT = -1,
    CNC_NET_CONNECT_NONE = 0,
    CNC_NET_CONNECT_LAN = 1,    // eth0
    CNC_NET_CONNECT_WLAN = 2,   // wifi

}cncNETConnect_t;

typedef enum _cncNETInterfaceTypeEnable_t {
    CNC_NET_LAN_ENABLE      = 1,
    CNC_NET_WLAN_ENABLE   = 2,
    CNC_NET_ALL_ENABLE      = 3,        //  CNC_NET_LAN_ENABLE | CNC_NET_WLAN_ENABLE
}cncNETInterfaceEnable_t;

typedef enum {
    CNC_NET_NONE,
    CNC_NET_LAN,
    CNC_NET_WLAN
} cncNETLANSelect_t;

typedef void (*cncNetConnectCallback_t)(cncNETConnect_t status);

typedef struct _cncNETConfigParams_t {
    cncNETInterfaceEnable_t  connectMode;
    char               configPath[128];
    char               loginRecordsPath[128];
    char               logPath[128];
    char               lanPortName[8];
    char               wlanPortName[8];
} cncNETConfigParams_t;

Result_Type_e CNC_NET_Init(cncNETConfigParams_t *config);
Result_Type_e CNC_NET_SwitchLan(cncNETLANSelect_t type);
Result_Type_e CNC_NET_SetDNS(char *dns);
Result_Type_e CNC_NET_GetDNS(char *dns);
Result_Type_e CNC_NET_SetIPAddr(char *interface_name, char *ipaddr);
Result_Type_e CNC_NET_GetIPAddr (char *interface_name, unsigned long *ipaddr);
Result_Type_e CNC_NET_SetNetMask(char *interface_name, char *netmask);
Result_Type_e CNC_NET_GetNetMask(char *interface_name, unsigned long *netmask);
Result_Type_e CNC_NET_SetGateWay(char *interface_name, char *GateWay);
Result_Type_e CNC_NET_GetGateWay(char *interface_name, unsigned long *GateWay);
Result_Type_e CNC_NET_GetMAC(char *interface_name, char *mac, int len);
Result_Type_e CNC_NET_InterfaceUp(char *interface_name);
Result_Type_e CNC_NET_InterfaceDown(char *interface_name);
Result_Type_e CNC_NET_SetIfconfig(char *interface_name,char *ipaddr);
Result_Type_e CNC_NET_SetDHCP(char *interface_name, CNC_BOOL isblock);
cncNETConnect_t CNC_NET_ConnectionStatus(void);
CNC_SINT32 CNC_NET_GetStatus(char *InterfaceName);
void CNC_NET_RegisterCallback(cncNetConnectCallback_t callback);
CNC_CHAR *CNC_NET_GetPortName(void);
CNC_CHAR *CNC_NET_GetLANName(void);
CNC_CHAR *CNC_NET_GetWLANName(void);

/********************************************   WIFI   ************************************************/

#define WIFI_CONFIG_MAX 16

typedef enum _cncWIFIAuthenticationMode_t {
    WIFI_UNKNOWN = 0,
    WEPOPEN,        //For open system
    WEPSHARED,      //For shared key system
    WEPAUTO,     //Auto switch between OPEN and SHARED
    WPAPSK,     //For WPA pre-shared key  (Infra)
    WPA2PSK,     //For WPA2 pre-shared key (Infra)
    WPANONE,        //For WPA pre-shared key  (Adhoc)
    WPA,         //Use WPA-Supplicant
    WPA2,        //Use WPA-Supplicant
}cncWIFIAuthenticationMode_t;

typedef enum _cncWIFIEncryption_t {
    WIFI_NONE = 0,  //For AuthMode=OPEN
    WEP,                    //For AuthMode=OPEN or AuthMode=SHARED
    TKIP,                   //For AuthMode=WPAPSK or WPA2PSK
    AES,                    //For AuthMode=WPAPSK or WPA2PSK
}cncWIFIEncryption_t;

/**
Cell 01 - Address: F4:EC:38:67:F7:68
          Protocol:802.11b/g/n
          ESSID:"wireless.cs2"
          Mode:Managed
          Frequency:2.412 GHz (Channel 1)
          Quality=52/100  Signal level=-69 dBm  Noise level=-92 dBm
          Encryption key:on
          Bit Rates:54 Mb/s
          IE: WPA Version 1
**/
typedef struct _cncWIFIConnectionInfo_t
{
    int index;

    /* From wifi scan */
    CNC_UINT8 id;
    //char Address[32];
    //char Protocol[32];
    char ESSID[128];
    //char Mode[32];
    //U32 Frequency;      // KHz
    CNC_UINT16 Quality;     // 0-100
    //U16 SignalLevel;// dBm
    //U16 NoiseLevel;// dBm
    CNC_BOOL Encryptionkey; // on / off
    //U16 BitRates;   //Mb/s
    char mode_str[32];    // WPA / WEP

    /* For wifi connect */
    CNC_BOOL Linkflag; // on / off
    cncWIFIAuthenticationMode_t AutoMode;
    cncWIFIEncryption_t EncrypType;
    char Password[128];
    struct _cncWIFIConnectionInfo_t * next;
    struct _cncWIFIConnectionInfo_t * prev;
} cncWIFIConnectionInfo_t;

void CNC_WIFI_Init(void);
cncWIFIConnectionInfo_t * CNC_WIFI_LoadConfig(int * sum);
void CNC_WIFI_Up(void);
void CNC_WIFI_Down(void);
int CNC_WIFI_Detect(void);
void CNC_WIFI_Scan(void);
cncWIFIConnectionInfo_t * CNC_WIFI_GetScanList(int * sum);
int CNC_WIFI_SetKey(cncWIFIConnectionInfo_t * wifi , char* pin);
int CNC_WIFI_Udhcp(void);
int CNC_WIFI_SaveConfig(cncWIFIConnectionInfo_t * wifi_head);
void CNC_WIFI_FreeList(cncWIFIConnectionInfo_t * wifi_head);
void CNC_WIFI_SetSSID(char * essid);
char * CNC_WIFI_GetSSID(cncNETConnect_t * status);
CNC_SINT32 CNC_WIFI_EnableSoftAP ();
CNC_SINT32 CNC_WIFI_EnableSTA ();
CNC_CHAR* CNC_WIFI_GetSSIDFromSoftAPFile();
/********************************************   PPPOE   ************************************************/
Result_Type_e CNC_PPPOE_SetPPPOEAccount(char *usr,char *passwd);
Result_Type_e CNC_PPPOE_Start(void);
Result_Type_e CNC_PPPOE_Monitor(void);


#ifdef __cplusplus
}
#endif

#endif

