/**
 *# COPYRIGHT NOTICE
 *#
 *#***************************************************************************
 *
 *The following software code is the property of Cavium, Inc.
 *Cavium, Inc. reserves all rights in this software, including but not limited
 *to copyright and shall be vested with all rights incidental thereto.
 
 *Rights granted to you by or under any license shall be limited to those
 *explicitly granted. You may not record, store, reproduce, reverse engineer,
 *decompile, inter-operate and/or transmit this software in any medium,
 *electronic or otherwise, without the express written consent of Cavium, Inc.
 
 *ALL RIGHTS RESERVED.
 
 *Copyright 2013, Cavium, Inc.
 *#***************************************************************************
 *#
 *# END OF COPYRIGHT NOTICE
 *#
 */
 
#ifndef _CNC_NET_CMDS_H_
#define _CNC_NET_CMDS_H_
 
#ifdef __cplusplus
extern "C" {
#endif

/**
 * \brief Commands in eth mode
 *
 */
typedef enum _cncNetCmdEth_t
{
    CNC_NET_CMD_ETH_NONE,
    CNC_NET_CMD_ETH_GET_IP_PARAMS,              /**< get ip configuration */
    CNC_NET_CMD_ETH_SET_IP_CONFIG               /**< set ip configuration */
}cncNetCmdEth_t;

/**
 * \brief Commands in wifi mode ( common for all wifi sub types )
 *
 */
typedef enum _cncNetCmdWiFi_t
{
    CNC_NET_CMD_WIFI_NONE,
}cncNetCmdWiFi_t;

/**
 * \brief Commands in wifi station mode
 *
 */
typedef enum _cncNetCmdSta_t
{
    CNC_NET_CMD_STA_NONE,
    CNC_NET_CMD_STA_SCAN,                   /**< trigger a scan and get scan list if callback is provide */
    CNC_NET_CMD_STA_GET_SCAN_LIST,          /**< iterate scan list through callback function provided */
    CNC_NET_CMD_STA_CONNECT,                /**< connect to bss provided */
    CNC_NET_CMD_STA_RECONNECT,              /**< Trigger reconnection to known ap's */
    CNC_NET_CMD_STA_DISCONNECT,             /**< disconnect from connected AP */
    CNC_NET_CMD_STA_GET_IP_PARAMS,          /**< get ip configuration */
    CNC_NET_CMD_STA_SET_IP_CONFIG,          /**< set ip configuration */
    CNC_NET_CMD_STA_VIDEO_STREAM_STATE,     /**< video streaming is active/inactive */
    CNC_NET_CMD_STA_GET_SSID,               /**< get station SSID*/
    CNC_NET_CMD_STA_GET_LINK_STATS,			/**< get station link statistics*/
    CNC_NET_CMD_STA_LIST_NETWORKS,			/**< list networks */
    CNC_NET_CMD_STA_ADD_NETWORK,			/**< add a network */
    CNC_NET_CMD_STA_REMOVE_NETWORK,			/**< remove a network */
}cncNetCmdSta_t;

/**
 * \brief Commands in wifi AP mode
 *
 */
typedef enum _cncNetCmdAp_t
{
    CNC_NET_CMD_AP_NONE,
    CNC_NET_CMD_AP_SET_SSID,                /**< Update ssid of AP */
    CNC_NET_CMD_AP_GET_SSID,                /**< Fetch ssid of AP */
    CNC_NET_CMD_AP_SET_IEEE_CHANNEL,        /**< Update operating channel */
    CNC_NET_CMD_AP_CONNECT,                 /**< connect to sta provided */
    CNC_NET_CMD_AP_GET_IP_PARAMS,           /**< get ip configuration */
    CNC_NET_CMD_AP_SET_IP_CONFIG,           /**< set ip configuration */
    CNC_NET_CMD_AP_VIDEO_STREAM_STATE,      /**< video streaming is active/inactive */
    CNC_NET_CMD_AP_GET_LINK_STATS,			/**< get link statistics */
    CNC_NET_CMD_AP_SCAN,                   	/**< trigger a scan and get scan list if callback is provide */
    CNC_NET_CMD_AP_GET_SCAN_LIST,          	/**< iterate scan list through callback function provided */
    CNC_NET_CMD_AP_LIST_NETWORKS,			/**< list networks */
    CNC_NET_CMD_AP_ADD_NETWORK,				/**< add a network */
    CNC_NET_CMD_AP_REMOVE_NETWORK,			/**< remove a network */
}cncNetCmdAp_t;

/**
 * \brief Commands in wifi P2P mode
 *
 */
typedef enum _cncNetCmdP2p_t
{
    CNC_NET_CMD_P2P_NONE,
    CNC_NET_CMD_P2P_SET_IEEE_LISTEN_CHANNEL,        /**< Update p2p listen channel, will take effect when discovery mode starts */
    CNC_NET_CMD_P2P_SET_IEEE_OPERATE_CHANNEL,       /**< Update p2p operating channel, will take effect on next connection */
    CNC_NET_CMD_P2P_START_STOP_DISCOVERY,           /**< start/stop p2p discovery */
    CNC_NET_CMD_P2P_GET_PEER_LIST,                  /**< iterate p2p peer list through callback function provided */
    CNC_NET_CMD_P2P_CONNECT,                        /**< connect to p2p peer device */
    CNC_NET_CMD_P2P_DISCONNECT,                     /**< disconnect from connected p2p peer device */
    CNC_NET_CMD_P2P_GET_IP_PARAMS,                  /**< get ip configuration */
    CNC_NET_CMD_P2P_SET_IP_CONFIG,                  /**< set ip configuration */
    CNC_NET_CMD_P2P_VIDEO_STREAM_STATE,             /**< video streaming is active/inactive */
    CNC_NET_CMD_P2P_GET_DEVICE_NAME,                /**< get p2p device nanme */
    CNC_NET_CMD_P2P_SET_DEVICE_NAME,                /**< set p2p device nanme */
    CNC_NET_CMD_P2P_GET_LINK_STATS,                 /**< get link statistics */
}cncNetCmdP2p_t;

/**
 * \brief Commands in wifi P2P AutoGO mode
 *
 */
typedef enum _cncNetCmdP2pAutoGO_t
{
    CNC_NET_CMD_P2P_AUTOGO_NONE,
    CNC_NET_CMD_P2P_AUTOGO_SET_IEEE_OPERATE_CHANNEL,    /**< Update p2p operating channel, will take effect on next connection */
    CNC_NET_CMD_P2P_AUTOGO_GET_PEER_LIST,               /**< iterate p2p peer list through callback function provided */
    CNC_NET_CMD_P2P_AUTOGO_CONNECT,                     /**< connect to p2p peer device */
    CNC_NET_CMD_P2P_AUTOGO_DISCONNECT,                  /**< disconnect connected p2p peer device */
    CNC_NET_CMD_P2P_AUTOGO_GET_IP_PARAMS,               /**< get ip configuration */
    CNC_NET_CMD_P2P_AUTOGO_SET_IP_CONFIG,               /**< set ip configuration */
    CNC_NET_CMD_P2P_AUTOGO_VIDEO_STREAM_STATE,          /**< video streaming is active/inactive */
}cncNetCmdP2pAutoGO_t;

/**
 * \brief Commands in wifi widi station mode ( l2sd )
 *
 */
typedef enum _cncNetCmdWiDiSta_t
{
    CNC_NET_CMD_WIDI_STA_NONE,
    CNC_NET_CMD_WIDI_STA_IDLE,                  /**< disconnect and go to idle state */
    CNC_NET_CMD_WIDI_STA_CONNECT,       	    /**< intiate connection */
    CNC_NET_CMD_WIDI_STA_WPS,       			/**< intiate wps session */
}cncNetCmdWiDiSta_t;

/**
 * \brief Commands in WiDi mode
 *
 */
typedef enum _cncNetCmdWiDi_t
{
    CNC_NET_CMD_WIDI_NONE,
    CNC_NET_CMD_WIDI_SET_CONFIG,			/**< Update widi capabilities, these are common between sta and wfd modes */
}cncNetCmdWiDi_t;

/**
 * \brief Callback function definition for scan list iterator.
 *
 */
typedef cncNetError_t ( *pfn_cncNetWiFi_processBss )( CNC_VOID* p_context, cncNetWiFiBssInfo_t* p_bssInfo );

/**
 * \brief Callback function definition for network list iterator.
 *
 */
typedef cncNetError_t ( *pfn_cncNetWiFi_processNetwork )( CNC_VOID* p_context, cncNetWiFiNetworkInfo_t* p_networkInfo );


/**
 * \brief Command type
 *
 */
typedef union _cncNetCmdType_t
{
    cncNetCmdEth_t eth;

    cncNetCmdSta_t sta;

    cncNetCmdAp_t ap;

    cncNetCmdP2p_t p2p;

    cncNetCmdP2pAutoGO_t p2pAutoGO;

    cncNetCmdWiDiSta_t widiSta;

    cncNetCmdWiDi_t widi;

    cncNetCmdWiFi_t wifi;

}cncNetCmdType_t;

/**
 * \brief Infor for commands CNC_NET_CMD_WIFI_P2P_CONNECT
 *
 */
typedef struct _cncNetCmdP2pConnect_t
{
    CNC_CHAR a_peerAddr[CNC_NET_MAC_STR_LEN];       /**< Mac address of peer device */

    cncNetAuthWps_t wpsAuth;                        /**< wps information */

}cncNetCmdP2pConnect_t;

/**
 * \brief Infor for commands CNC_NET_CMD_WIFI_STA_CONNECT
 *
 */
typedef struct _cncNetCmdStaConnect_t
{
    CNC_CHAR a_ssid[CNC_NET_WIFI_SSID_LEN];  /**< SSID of network */
    CNC_CHAR a_bssid[CNC_NET_MAC_STR_LEN];   /**< Macaddress of AP */

    cncNetAuth_t auth;                       /**< Security credentials to be applied for this AP */

    CNC_SINT32 retryCount;							/**< -1 : indefinite retries, 0 : do not retry, >=1 : retry n times
    													connection will be retried according to autoConnectInterval in staConfig */

}cncNetCmdStaConnect_t;

/**
 * \brief Infor for commands CNC_NET_CMD_WIFI_AP_CONNECT
 *
 */
typedef struct _cncNetCmdApConnect_t
{
    CNC_CHAR a_ssid[CNC_NET_WIFI_SSID_LEN];  /**< SSID of network */

    cncNetAuth_t auth;                       /**< Security credentials to be applied for this sta */

}cncNetCmdApConnect_t;


/**
 * \brief Info for commands
 * 	CNC_NET_CMD_STA_GET_SCAN_LIST, CNC_NET_CMD_STA_SCAN,
 * 	CNC_NET_CMD_AP_GET_SCAN_LIST, CNC_NET_CMD_AP_SCAN
 *
 */
typedef struct _cncNetCmdGetScanList_t
{
    pfn_cncNetWiFi_processBss pfn_callback;             /**< callback function for receiving scan list, one bss entry per call */

    CNC_VOID* p_callbackContext;                        /**< context for callback function */

}cncNetCmdGetScanList_t;

/**
 * \brief Info for commands
 * 	CNC_NET_CMD_STA_LIST_NETWORKS,
 * 	CNC_NET_CMD_AP_LIST_NETWORKS
 *
 */
typedef struct _cncNetCmdGetNetworkList_t
{
    pfn_cncNetWiFi_processNetwork pfn_callback;             /**< callback function for receiving network list, one network entry per call */

    CNC_VOID* p_callbackContext;                        /**< context for callback function */

}cncNetCmdGetNetworkList_t;

/**
 * \brief Info for commands
 * 	CNC_NET_CMD_STA_REMOVE_NETWORK,
 * 	CNC_NET_CMD_AP_REMOVE_NETWORK
 *
 */
typedef struct _cncNetCmdRemoveNetwork_t
{
	cncNetWiFiNetworkInfo_t networkInfo;
	
}cncNetCmdRemoveNetwork_t;

/**
 * \brief Infor for commands CNC_NET_CMD_WIDI_STA_WPS, CNC_NET_CMD_WIDI_STA_CONNECT
 *
 */
typedef struct _cncNetCmdWiDiStaConnect_t
{
    CNC_CHAR a_ssid[CNC_NET_WIFI_SSID_LEN];  	/**< SSID of network */

    CNC_CHAR a_pin[16];							/**< pin to be displayed */

}cncNetCmdWiDiStaConnect_t;

/**
 * \brief Command information provided in event callback
 *
 */
typedef struct _cncNetCmd_t
{
    cncNetType_t type;                                  /**< Type of network defined in @cncNetType_t */

    cncNetSubType_t subType;                            /**< SubType of network defined in @cncNetSubType_t */

    cncNetCmdType_t cmd;                                /**< only 1 command will be set according to type/subtype */

    union
    {
        cncNetIpParams_t ipParams;                      /**< CNC_NET_CMD_*_GET_IP_PARAMS */

        cncNetIpConfig_t ipConfig;                      /**< CNC_NET_CMD_*_SET_IP_PARAMS */

        CNC_BOOL videoStreamState;                      /**< video stream state active=TRUE, inactive=FALSE */

        cncNetCmdP2pConnect_t p2pConnect;               /**< information for p2p connect */

        CNC_CHAR deviceName[CNC_NET_WIFI_SSID_LEN];     /**< Device name or SSID*/

        cncNetCmdStaConnect_t staConnect;               /**< CNC_NET_CMD_STA_CONNECT */
        
        cncNetCmdApConnect_t apConnect;                 /**< CNC_NET_CMD_AP_CONNECT */
        
        cncNetCmdGetScanList_t scanList;             	/**< information for CNC_NET_CMD_STA_GET_SCAN_LIST, CNC_NET_CMD_AP_GET_SCAN_LIST */
		
		cncNetCmdGetNetworkList_t networkList;			/**< information for CNC_NET_CMD_STA_LIST_NETWORKS, CNC_NET_CMD_AP_LIST_NETWORKS */
		
		cncNetCmdRemoveNetwork_t removeNetwork;		/**< information for CNC_NET_CMD_STA_REMOVE_NETWORK, CNC_NET_CMD_AP_REMOVE_NETWORK */
		
        cncNetWiFiLinkStat_t linkStats;                 /**< information for CNC_NET_CMD_P2P_GET_LINK_STATS */

        cncNetWiDiConfig_t widiConfig;					/**< information for CNC_NET_CMD_WIDI_SET_CONFIG */

        cncNetCmdWiDiStaConnect_t widiStaConnect;		/**< information for CNC_NET_CMD_WIDI_STA_WPS and CNC_NET_CMD_WIDI_STA_CONNECT */

        cncNetCmdP2pConnect_t p2pAutoGoConnect;     	/**< information for p2p autogo connect */

    }info;

}cncNetCmd_t;

#ifdef __cplusplus
}
#endif
 
#endif /* END:  _CNC_NET_CMDS_H_ */

