/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************
 
 The following software code is the property of Cavium, Inc.
 Cavium, Inc. reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.
 
 Rights granted to you by or under any license shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium, Inc.
 
 ALL RIGHTS RESERVED.
 
 Copyright 2012-2013, Cavium, Inc.
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */
 
 
 
#ifndef _CNC_NET_COMMON_H_
#define _CNC_NET_COMMON_H_
 
#ifdef __cplusplus
extern "C" {
#endif

#include "cnc_common_typedef.h"
 

#define CNC_NET_IF_NAME_LEN          16          /**< network interface name length in bytes */
#define CNC_NET_WIFI_SSID_LEN        32          /**< wifi ssid length in bytes */
#define CNC_NET_WPA_PSK_LEN          64          /**< wpa/wpa2 psk length in bytes */
#define CNC_NET_WEP_KEY_LEN          27          /**< WEP key length in bytes(WEP104 Hex key max length is 26) */
#define CNC_NET_MIN_WPA_PSK_LEN      8           /**< minimum wpa/wpa2 psk length in bytes */
#define CNC_NET_MAX_WPA_PSK_LEN      63          /**< maximum wpa/wpa2 psk length in bytes */
#define CNC_NET_IP_STR_LEN           16          /**< ip address string length in bytes xxx.xxx.xxx.xxx */
#define CNC_NET_MAC_STR_LEN          18          /**< mac address string length in bytes xx:xx:xx:xx:xx:xx */
#define CNC_NET_FLAG_STR_LEN         40          /**< Flag string length in bytes  */
#define CNC_NET_SSID_POSTFIX_LEN	 24			 /**< wifi direct SSID postfix length */

/**
 * \brief Errors values.
 *
 */
typedef enum _cncNetError_t
{
    CNC_NET_ERROR_NONE = 0,
    CNC_NET_ERROR_ABORT,
    CNC_NET_ERROR_FEATURE_NOT_SUPPORTED,
    CNC_NET_ERROR_FEATURE_UNAVAILABLE,
    CNC_NET_ERROR_INVALID_HANDLE,
    CNC_NET_ERROR_INVALID_ARG,
    CNC_NET_ERROR_TIMEOUT,
    CNC_NET_ERROR_NO_MEMORY,
    CNC_NET_ERROR_PLATFORM_API_FAIL,
    CNC_NET_ERROR_NOT_FOUND,

    CNC_NET_ERROR_PLUGIN_LOAD_FAIL = 75,
    CNC_NET_ERROR_PLUGIN_CONFIG_FAIL,
    CNC_NET_ERROR_PLUGIN_START_FAIL,
    CNC_NET_ERROR_PLUGIN_API_FAIL,

    CNC_NET_ERROR_INVALID_IFNAME = 100,
    CNC_NET_ERROR_INVALID_IPADDR,
    CNC_NET_ERROR_INVALID_NETMASK,
    CNC_NET_ERROR_INVALID_GW,
    CNC_NET_ERROR_INVALID_DNS,
    CNC_NET_ERROR_DIFFRNT_SUBNETS_FOR_IPADDR_AND_GW,
    CNC_NET_ERROR_CANNOT_SET_IPADDR,
    CNC_NET_ERROR_CANNOT_SET_NETMASK,
    CNC_NET_ERROR_CANNOT_SET_GW,

    CNC_NET_ERROR_WIFI_STA_ALREADY_CONNECTED = 150,
    CNC_NET_ERROR_WIFI_STA_BUSY,
    CNC_NET_ERROR_WIFI_STA_NETWORK_NOT_FOUND,
    CNC_NET_ERROR_WIFI_STA_NETWORK_LIST_EMPTY,
    CNC_NET_ERROR_WIFI_STA_INVALID_CREDENTIALS,

    CNC_NET_ERROR_WIFI_AP_BUSY = 160,

    CNC_NET_ERROR_WIFI_P2P_BUSY = 175,
    CNC_NET_ERROR_WIFI_P2P_NOT_CONNECTED,

    CNC_NET_ERROR_WIFI_POLICY_ERROR = 200,
    CNC_NET_ERROR_WIFI_POLICY_ABORT,

} cncNetError_t;

/**
 * \brief Supported network types
 *
 */
typedef enum _cncNetType_t
{
    CNC_NET_TYPE_WIFI = 0x1,
    CNC_NET_TYPE_ETH = ( 0x1 << 1 )
}cncNetType_t;

/**
 * \brief Supported Wifi network types
 *
 */
typedef enum _cncNetWifiSubType_t
{
    CNC_NET_WIFI_SUB_TYPE_STA = 0x1,
    CNC_NET_WIFI_SUB_TYPE_AP = ( 0x1 << 1 ),
    CNC_NET_WIFI_SUB_TYPE_P2P = ( 0x1 << 2 ),
    CNC_NET_WIFI_SUB_TYPE_P2P_AUTOGO = ( 0x1 << 3 ),
    CNC_NET_WIFI_SUB_TYPE_WIDI_STA = ( 0x1 << 4 ),
    CNC_NET_WIFI_SUB_TYPE_WIDI = ( 0x1 << 5 ),
}cncNetWifiSubType_t;

/**
 * \brief Supported ethernet network types
 *
 */
typedef enum _cncNetEthSubType_t
{
    CNC_NET_ETH_SUB_TYPE_802_3 = 0x1
}cncNetEthSubType_t;

/**
 * \brief Supported L2 protocols
 *
 */
typedef enum _cncNetL2Proto_t
{
    CNC_NET_L2_PROTO_WFD = 0x1,                  /**< Miracast */
    CNC_NET_L2_PROTO_WIDI_2 = ( 0x1 << 1 ),      /**< WIDI gen2 (L2SD) */
    CNC_NET_L2_PROTO_WIDI_3 = ( 0x1 << 2 ),      /**< WIDI gen3 (Miracast) */
    CNC_NET_L2_PROTO_WIDI_4 = ( 0x1 << 3 ),      /**< WIDI gen4 */
}cncNetL2Proto_t;

/**
 * \brief Supported network authentication types
 *
 */
typedef enum _cncNetAuthType_t
{
    CNC_NET_AUTH_TYPE_NONE,                  	/**< Open network */
    CNC_NET_AUTH_TYPE_WPA_PSK,  				/**< WPA PSK with TKIP */
    CNC_NET_AUTH_TYPE_WPA2_PSK,                 /**< WPA_PSK with TKIP/CCMP */
    CNC_NET_AUTH_TYPE_WPA_WPA2_PSK,
    CNC_NET_AUTH_TYPE_WPS,                      /**< WPS with pin/keypad/pbc */
    CNC_NET_AUTH_TYPE_WEP,						/**< WEP */
} cncNetAuthType_t;

/**
 * \brief Supported WEP authentication algorithm types
 *
 */
typedef enum _cncNetWepAuthAlg_t
{
    CNC_NET_WEP_AUTH_ALG_OPEN,                  	/**< Open System */
    CNC_NET_WEP_AUTH_ALG_SHAREDKEY					/**< Shared key */
} cncNetWepAuthAlg_t;

/**
 * \brief Supported WEP key types
 *
 */
typedef enum _cncNetWepKeyType_t
{
    CNC_NET_WEP_KEY_TYPE_HEX,                  	/**< Hex */
    CNC_NET_WEP_KEY_TYPE_ASCII					/**< ASCII */
} cncNetWepKeyType_t;

/**
 * \brief Supported WEP key index
 *
 */
typedef enum _cncNetWepKeyIdx_t
{
    CNC_NET_WEP_KEY_IDX_DEFAULT,                /**< Default key index 0 */
    CNC_NET_WEP_KEY_IDX1,						/**< Key index 1 */
    CNC_NET_WEP_KEY_IDX2,						/**< Key index 2 */
    CNC_NET_WEP_KEY_IDX3						/**< Key index 3 */
} cncNetWepKeyIdx_t;

/**
 * \brief Supported wps methods
 *          Values of these methods match with config methods exposed by wpa_supplicant.
 *
 */
typedef enum _cncNetWpsMethod_t
{
    CNC_NET_WPS_METHOD_NONE,
    CNC_NET_WPS_METHOD_DISPLAY = 0x0008,
    CNC_NET_WPS_METHOD_PBC = 0x0080,
    CNC_NET_WPS_METHOD_KEYPAD = 0x0100
} cncNetWpsMethod_t;

/**
 * \brief Supported wps 4 digits or 8 digits PIN
 *
 */
typedef enum _cncNetWpsPinType_t
{
    CNC_NET_WPS_PIN_LONG,				// 8 digits PIN
    CNC_NET_WPS_PIN_SHORT				// 4 digits PIN
} cncNetWpsPinType_t;

/**
 *  \brief wifi plugin P2P Find's scan type
 *
 */
typedef enum _cncNetP2pFindType_t
{
	CNC_NET_P2P_FIND_ONLY_SOCIAL,
	CNC_NET_P2P_FIND_START_WITH_FULL
} cncNetP2pFindType_t;

/**
 * \brief Supported ip configuration methods
 *
 */
typedef enum _cncNetIpMethod_t
{
    CNC_NET_IP_METHOD_NONE,
    CNC_NET_IP_METHOD_STATIC,
    CNC_NET_IP_METHOD_DHCP_CLIENT,
    CNC_NET_IP_METHOD_DHCP_SERVER
}cncNetIpMethod_t;

/**
 * \brief Supported p2p device capabilities
 *          Values of these capabilities match with those exposed by wpa_supplicant.
 *
 */
typedef enum _cncNetP2pDeviceCaps_t
{
    CNC_NET_P2P_DEVICE_CAPS_NONE,
    CNC_NET_P2P_DEVICE_CAPS_SERVICE_DISCOVREY = 0x0001,
    CNC_NET_P2P_DEVICE_CAPS_CLIENT_DISCOVERABILITY = 0x0002,
    CNC_NET_P2P_DEVICE_CAPS_CONCURRENT_OPERATION = 0x0004,
    CNC_NET_P2P_DEVICE_CAPS_INFRA_MANAGED = 0x0008,
    CNC_NET_P2P_DEVICE_CAPS_DEVICE_LIMIT = 0x0010,
    CNC_NET_P2P_DEVICE_CAPS_INVITATION = 0x0020
} cncNetP2pDeviceCaps_t;

/**
 * \brief Supported p2p group capabilities
 *          Values of these capabilities match with those exposed by wpa_supplicant.
 *
 */
typedef enum _cncNetP2pGroupCaps_t
{
    CNC_NET_P2P_GROUP_CAPS_NONE,
    CNC_NET_P2P_GROUP_CAPS_GROUP_OWNER = 0x0001,
    CNC_NET_P2P_GROUP_CAPS_PERSISTENT_GROUP = 0x0002,
    CNC_NET_P2P_GROUP_CAPS_GROUP_LIMIT = 0x0004,
    CNC_NET_P2P_GROUP_CAPS_INTRA_BSS_DISTRIBUTION = 0x0008,
    CNC_NET_P2P_GROUP_CAPS_CROSS_CONNECTION = 0x0010,
    CNC_NET_P2P_GROUP_CAPS_PERSISTENT_RECONNECT = 0x0020,
    CNC_NET_P2P_GROUP_CAPS_GROUP_FORMATION = 0x0040
} cncNetP2pGroupCaps_t;

/**
 * \brief enum for link status
 *
 */
typedef enum _cncNetLinkStatus_t {
    CNC_NET_LINK_STATUS_UNKNOWN = -1,
    CNC_NET_LINK_STATUS_DOWN    =  0,
    CNC_NET_LINK_STATUS_UP      =  1
} cncNetLinkStatus_t;

/**
 * \brief Flags for bss entries
 *
 * Each network can have different authentication and other functionalities
 * These flags are enabled based on information elements from beacons
 */
typedef enum _cncNetWiFiBssFlags_t
{
    CNC_NET_WIFI_BSS_FLAG_ESS = 1,                    /**< This is a regular AP */
    CNC_NET_WIFI_BSS_FLAG_WPS = ( 0x1 << 1 ),         /**< WPS authentication is supported */
    CNC_NET_WIFI_BSS_FLAG_WPA_PSK = ( 0x1 << 2 ),     /**< WPA_PSK (TKIP) based authentication */
    CNC_NET_WIFI_BSS_FLAG_WPA_EAP = ( 0x1 << 3 ),     /**< 802.1x supported */
    CNC_NET_WIFI_BSS_FLAG_WPA2_PSK = ( 0x1 << 4 ),    /**< 80211i WPA2_PSK (CCMP/TKIP) based authentication */
    CNC_NET_WIFI_BSS_FLAG_WPA2_EAP = ( 0x1 << 5 ),    /**< 802.1x supported based on 80211i */
    CNC_NET_WIFI_BSS_FLAG_P2P = ( 0x1 << 6 ),         /**< WiFi Direct supported */
    CNC_NET_WIFI_BSS_FLAG_INTEL_WIDI = ( 0x1 << 7 ),  /**< Intel WIDI gen2 (L2SD) supported */
    CNC_NET_WIFI_BSS_FLAG_WEP = ( 0x1 << 8 ),		  /**< WEP based authentication */
}cncNetWiFiBssFlags_t;

/**
 * \brief Supported ethernet network sub types
 *
 */
typedef struct _cncNetSubType_t
{
    cncNetWifiSubType_t wifi;
    cncNetEthSubType_t eth;
}cncNetSubType_t;


/**
 * \brief Structure to hold data for WPA-PSK/WPA2-PSK
 *
 */
typedef struct _cncNetAuthWpa2Psk_t
{
    CNC_CHAR a_psk[CNC_NET_WPA_PSK_LEN];
} cncNetAuthWpa2Psk_t;

/**
 * \brief Structure to hold data for WEP
 *
 */
typedef struct _cncNetAuthWep_t
{
	cncNetWepAuthAlg_t auth_alg;
	cncNetWepKeyType_t key_type;
	CNC_UINT8 idx;
	CNC_CHAR key[CNC_NET_WEP_KEY_LEN];
} cncNetAuthWep_t;

/**
 * \brief Structure to hold data for WPS
 *
 */
typedef struct _cncNetAuthWps_t
{
    cncNetWpsMethod_t method;
    CNC_CHAR a_pin[16];
    cncNetWpsPinType_t pin_type;
} cncNetAuthWps_t;

/**
 * \brief Structure to hold data for authentication
 *
 * This can be used to create a new network
 * or
 * Get security information of existing or connected network
 */
typedef struct _cncNetAuth_t
{
    cncNetAuthType_t type;                           /**< Type of security required/implemented */
    union                                           /**< one of the following will be valid based on type */
    {
    	cncNetAuthWep_t wep;
        cncNetAuthWpa2Psk_t wpa2Psk;
        cncNetAuthWps_t wps;
    }info;
} cncNetAuth_t;


/**
 * \brief Structure to hold IP parameters
 *
 */
typedef struct _cncNetIpParams_t
{
    CNC_CHAR a_ipaddr[CNC_NET_IP_STR_LEN];

    CNC_CHAR a_netmask[CNC_NET_IP_STR_LEN];

    CNC_CHAR a_gateway[CNC_NET_IP_STR_LEN];

    CNC_CHAR a_dns[CNC_NET_IP_STR_LEN];

}cncNetIpParams_t;

/**
 * \brief Structure to hold static ip configuration
 *
 */
typedef struct _cncNetIpStatic_t
{
    CNC_CHAR a_ip[CNC_NET_IP_STR_LEN];                  /**< static ip address */

    CNC_CHAR a_netmask[CNC_NET_IP_STR_LEN];             /**< netmask */

    CNC_CHAR a_gateway[CNC_NET_IP_STR_LEN];             /**< gateway */

    CNC_CHAR a_dns[128];                                /**< dns entry */

} cncNetIpStatic_t;

/**
 * \brief Structure to hold dhcp client configuration
 *
 */
typedef struct _cncNetIpDhcpClient_t
{
    CNC_UINT8 i_maxDhcpRetry;                       /**< maximum dhcp discovery request retries */

}cncNetIpDhcpClient_t;

/**
 * \brief Structure to hold dhcp server configuration
 *
 */
typedef struct _cncNetIpDhcpServer_t
{
    CNC_CHAR a_ip[CNC_NET_IP_STR_LEN];              /**< server ip address */

    CNC_CHAR a_netmask[CNC_NET_IP_STR_LEN];         /**< netmask */

    CNC_CHAR a_broadcast[CNC_NET_IP_STR_LEN];       /**< broadcast */

    CNC_SINT32 leaseTime;                           /**< client ip lease time in seconds */

    CNC_CHAR a_clientIpStart[CNC_NET_IP_STR_LEN];   /**< start address of client ip */

    CNC_CHAR a_clientIpEnd[CNC_NET_IP_STR_LEN];    	/**< end address of client ip */

    CNC_UINT8 maxClients;                           /**< max dhcp clients */

    CNC_CHAR a_dns[128];                            /**< dns entry */

}cncNetIpDhcpServer_t;

/**
 * \brief Structure to hold ip configuration
 *
 */
typedef struct _cncNetIpConfig_t
{
    cncNetIpMethod_t method;                            /**< ip configuration method */

    CNC_BOOL localNetwork;                              /**< set to CNC_BOL_TRUE if this is ip only local network
                                                            name resolution, dns, routing is not required */

    union                                               /**< one of the following will be valid based on method */
    {
        cncNetIpStatic_t staticIp;

        cncNetIpDhcpClient_t dhcpClient;

        cncNetIpDhcpServer_t dhcpServer;

    }info;

} cncNetIpConfig_t;

/**
 * \brief Structure to hold scanned AP information
 *
 */
typedef struct _cncNetWiFiBssInfo_t
{
    CNC_CHAR a_bssid[CNC_NET_MAC_STR_LEN];               /**< bssid */

    CNC_UINT32 freq;                                    /**< frequency */

    CNC_SINT32 signal;                                  /**< signal in rssi eg. ( -30(high) to -100(low) ) */

    CNC_CHAR a_ssid[CNC_NET_WIFI_SSID_LEN];             /**< ssid */

    cncNetWiFiBssFlags_t flags;                         /**< bss flags */

    CNC_CHAR a_infoElements[1024];                      /**< information elements if any */

} cncNetWiFiBssInfo_t;

/**
 * \brief Structure to hold list network information
 *
 */
typedef struct _cncNetWiFiNetworkInfo_t
{
    CNC_SINT32 index;                       /**< Index of network entry */

    CNC_CHAR a_ssid[CNC_NET_WIFI_SSID_LEN];  /**< SSID of network */

    CNC_CHAR a_bssid[CNC_NET_MAC_STR_LEN];   /**< Macaddress of AP */

} cncNetWiFiNetworkInfo_t;

/**
 * \brief Structure to hold wifi link statistics
 *
 */
typedef struct cncNetWiFiLinkStat_t
{
    CNC_CHAR a_mac[CNC_NET_MAC_STR_LEN];                /**< mac address */
    
    CNC_CHAR a_ssid[CNC_NET_WIFI_SSID_LEN];             /**< SSID */

    CNC_UINT32 freq;                                    /**< frequency of bss */

    CNC_UINT32 rxBytes;                                 /**< bytes received */

    CNC_UINT32 txBytes;                                 /**< bytes transmitted */

    CNC_UINT32 rxPackets;                               /**< packets received, wifi packet size is 2048 bytes */

    CNC_UINT32 txPackets;                               /**< bytes transmitted, wifi packet size is 2048 bytes */

    CNC_SINT32 signal;                                  /**< signal from AP in dBm*/

    CNC_SINT32 txBitrate;                               /**< in mbps */

    CNC_SINT32 txMcsIndex;                              /**< MCS index in case of 80211n connection */

    CNC_SINT32 channelBandwidth;                        /**< 20 or 40 MHz */

} cncNetWiFiLinkStat_t;

/**
 * \brief Configuration to setup WIDI in STA and/or P2P mode
 *
 */
typedef struct _cncNetWiDiConfig_t
{
	CNC_CHAR a_devName[CNC_NET_WIFI_SSID_LEN];		/**< device name subelement in wps and p2p ie */

    CNC_UINT8 a_wpsVendorExtension[64];				/**< vendor extension subelement in wps info element */

    CNC_UINT8 a_wpsPrimaryDev[32];					/**< primary device type attribute in wps/p2p info element */

    CNC_UINT8 a_wpsSecondaryDevList[32];			/**< secondary device list attribute in wps info element */

    CNC_UINT8 a_p2pSecondaryDevList[32];			/**< secondary device list attribute in p2p info element */

} cncNetWiDiConfig_t;

#ifdef __cplusplus
}
#endif
 
#endif /* END:  _CNC_NET_COMMON_H_ */

