/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************
 
 The following software code is the property of Cavium, Inc.
 Cavium, Inc. reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.
 
 Rights granted to you by or under any license shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium, Inc.
 
 ALL RIGHTS RESERVED.
 
 Copyright 2012-2013, Cavium, Inc.
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */
 
 
 
#ifndef _CNC_NET_CONFIG_H_
#define _CNC_NET_CONFIG_H_
 
#ifdef __cplusplus
extern "C" {
#endif
 
/**
 * \brief Configuration to setup ethernet mode
 *
 */
typedef struct _cncNetEthConfig_t
{
    cncNetIpConfig_t ip;                                /**< ip address config */

} cncNetEthConfig_t;

/**
 * \brief Configuration to setup STA mode
 *
 */
typedef struct _cncNetStaConfig_t
{
    CNC_BOOL persistence;                           /**< Persistence support */

    CNC_SINT32 maxPersistenceNwCount;               /**< max persistent nw count if persistence is enabled */

    CNC_BOOL autoConnect;                           /**< autoconnect to known ap on startup */

    CNC_UINT8 maxConnectAttempts;                 	/**< number of connect attempts to saved networks */

    CNC_UINT32 autoConnectInterval;               	/**< time between successive sta connect attempts to saved networks in seconds */

    CNC_UINT32 connectTimeout;                      /**< sta connection timeout in seconds */

    cncNetIpConfig_t ip;                            /**< ip address config */

    CNC_BOOL widiGen2;                              /**< WIDI gen2 (L2SD) support. Requires station mode support. */

} cncNetStaConfig_t;

/**
 * \brief Configuration to setup AP mode
 *
 */
typedef struct _cncNetApConfig_t
{
    CNC_CHAR a_ssid[CNC_NET_WIFI_SSID_LEN];         /**< SSID to be used */

    CNC_UINT32 ieeeChannel;                         /**< Operating Channel, set to 0 for auto channel selection */

    CNC_UINT32 inactiveTimeout;                     /**< inactivity timeout if no stations connected in seconds 0 = no timeout */

    CNC_UINT32 conctInactvTimeout;                  /**< inactivity timeout for connected station */

    cncNetAuth_t auth;                              /**< Security to be applied for this AP */

    cncNetIpConfig_t ip;                            /**< ip configuration */

} cncNetApConfig_t;

/**
 * \brief Configuration to setup P2P
 *
 */
typedef struct _cncNetP2PConfig_t
{
    CNC_SINT32 ieeeListenChannel;                   /**< ieee listen channel ( 1,6 or 11, 0 == auto ) */

    CNC_SINT32 ieeeOperatingChannel;                /**< ieee operating channel */

    CNC_CHAR a_ssidPostfix[CNC_NET_SSID_POSTFIX_LEN];	/**< postfix after DIRECT-xx, ssid is 32 bytes so postfix cannot be longer than 23 bytes */

    CNC_BOOL persistence;                           /**< Persistence support, invitation will not be supported if persistence is off */

    CNC_SINT32 maxPersistenceNwCount;               /**< max persistent nw count if persistence is enabled */

    cncNetIpConfig_t ipGc;                          /**< ip configuration in GC mode */

    cncNetIpConfig_t ipGo;                          /**< ip configuration in GO mode */

    CNC_SINT32 groupIdleTimeout;                    /**< wait number of seconds for peer device to connect */

    CNC_BOOL wfd;                                   /**< Miracast support */

    CNC_SINT32 wfdPort;                             /**< user overridden wfd port, port from wfd ie will not be used if this is provided */

    CNC_BOOL widiGen3;                              /**< WIDI gen3 (Miracast) support. Will implicitly enable wfd mode. */

    CNC_CHAR a_wpsPin[16];							/**< user specified pin to be used in case of pin based connection */

	cncNetWpsPinType_t wps_pin_type;				/**< WPS PIN length 4 digits or 8 digits. */

	cncNetP2pFindType_t p2p_find_type;				/**< P2P FIND scan type, full scan or social scan. */

	CNC_CHAR a_devName[CNC_NET_WIFI_SSID_LEN];      /**< p2p device name */

} cncNetP2PConfig_t;

/**
 * \brief Configuration to setup wifi mode
 *
 */
typedef struct _cncNetWifiConfig_t
{
    cncNetStaConfig_t sta;              /**< Configuration for station mode */

    cncNetApConfig_t ap;                /**< Configuration for ap mode */

    cncNetP2PConfig_t p2p;              /**< Configuration for p2p mode */

    cncNetP2PConfig_t p2pAutoGO;        /**< Configuration for p2p auto GO mode */

    cncNetWiDiConfig_t widi;			/**< Configuration for WiDi mode */

} cncNetWifiConfig_t;

/**
 *  \brief configuration of of various modes
 *
 */
typedef struct _cncNetConfig_t
{
    cncNetType_t type;              /**< Types of networks defined in @cncNetType_t */

    cncNetSubType_t subTypes;       /**< SubType of networks defined in @cncNetSubType_t */

    union
    {
        cncNetEthConfig_t eth;      /**< Configuration for ethernet interface */

        cncNetWifiConfig_t wifi;    /**< Configuration for wifi interface */
    }info;

} cncNetConfig_t;


#ifdef __cplusplus
}
#endif
 
#endif /* END:  _CNC_NET_CONFIG_H_ */

