/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************
 
 The following software code is the property of Cavium, Inc.
 Cavium, Inc. reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.
 
 Rights granted to you by or under any license shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium, Inc.
 
 ALL RIGHTS RESERVED.
 
 Copyright 2012-2013, Cavium, Inc.
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */
 
 
 
#ifndef _CNC_NET_DEVICE_H_
#define _CNC_NET_DEVICE_H_
 
#ifdef __cplusplus
extern "C" {
#endif

/**
 * \brief Supported devices
 *
 */
typedef enum _cncNetDevice_t
{
	CNC_NET_DEVICE_UNKNOWN,
    CNC_NET_DEVICE_AR6003_HW_1_0 = 0x1,
    CNC_NET_DEVICE_AR6203_HW_1_0 = ( 0x1 << 1 ),
    CNC_NET_DEVICE_AR9374_HW_1_1 = ( 0x1 << 2 ),
    CNC_NET_DEVICE_AR9374_HW_1_3 = ( 0x1 << 3 ),
    CNC_NET_DEVICE_AR9374_HW_3_0 = ( 0x1 << 4 ),
    CNC_NET_DEVICE_RT5572_HW_1_0 = ( 0x1 << 5 ),
    CNC_NET_DEVICE_RTL8188CU	 = ( 0x1 << 6 ),
    CNC_NET_DEVICE_AR7420_HW_1_0 = ( 0x1 << 7 ),
    CNC_NET_DEVICE_RTL8811AU	 = ( 0x1 << 8 )
} cncNetDevice_t;

/**
 * \brief Information about network interface
 *        Interface info will be populate when instance of cnc_network is created.
 *
 */
typedef struct _cncNetInterfaceInfo_t
{
    CNC_CHAR a_ifName[CNC_NET_IF_NAME_LEN];          /**< Name of interface, plugin will autoselect the interface name */

    CNC_CHAR a_hwAddr[CNC_NET_MAC_STR_LEN];          /**< Mac address of interface, plugin will fill this while on creation */

}cncNetInterfaceInfo_t;


/**
 * \brief Information about network device
 *
 */
typedef struct _cncNetDeviceInfo_t
{
    cncNetDevice_t device;                  /**< Supported devices */

    cncNetType_t type;                      /**< Supported network type */

    cncNetSubType_t subTypes[16];           /**< Combinations of supported network subtypes eg. STA, AP, P2P, STA|AP, STA|P2P, STA|AP|P2P */

    CNC_CHAR a_driverVersion[32];           /**< Driver version string eg. a.b.c.d */

    cncNetL2Proto_t l2Proto;                /**< Supported L2 protocols eg. WFD|WIDI_2|WIDI_3 */

    union
    {
        cncNetInterfaceInfo_t eth;          /**< information about ethernet interface */

        struct
        {
            cncNetInterfaceInfo_t sta;      	/**< information about station interface */

            cncNetInterfaceInfo_t ap;       	/**< information about ap interface */

            cncNetInterfaceInfo_t p2p;      	/**< information about p2p interface */

            cncNetInterfaceInfo_t p2pAutoGo;	/**< information about p2p autoGO interface */
        }wifi;

    }ifInfo;

}cncNetDeviceInfo_t;

#ifdef __cplusplus
}
#endif
 
#endif /* END:  _CNC_NET_DEVICE_H_ */

