/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************
 
 The following software code is the property of Cavium, Inc.
 Cavium, Inc. reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.
 
 Rights granted to you by or under any license shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium, Inc.
 
 ALL RIGHTS RESERVED.
 
 Copyright 2012-2013, Cavium, Inc.
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */
 
 
 
#ifndef _CNC_NET_POLICY_H_
#define _CNC_NET_POLICY_H_
 
#ifdef __cplusplus
extern "C" {
#endif

/**
 * \brief Preset policies
 *
 */
typedef enum _cncNetPolicyType_t
{
	CNC_NET_POLICY_TYPE_NONE,
    CNC_NET_POLICY_TYPE_AUTO_STA_AP,
    CNC_NET_POLICY_TYPE_AUTO_STA_P2P,
    CNC_NET_POLICY_TYPE_AUTO_STA_P2P_AP
}cncNetPolicyType_t;

/**
 * \brief PBC policy will activate when
 *      push button is not pressed and a new p2p device sends a GON req or INVITE
 *
 * PBC policy types will act as follows.
 *
 * ASK : Display a screen asking the user to push button within 2 minutes to allow connection
 * ALLOW : Allow the connection without user invervention.
 * DENY : Deny the connection request, do nothing.
 */
typedef enum _cncNetPolicyP2pPbc_t
{
    CNC_NET_POLICY_P2P_PBC_ASK,
    CNC_NET_POLICY_P2P_PBC_ALLOW,
    CNC_NET_POLICY_P2P_PBC_DENY
}cncNetPolicyP2pPbc_t;

/**
 * \brief During concurrent sta/p2p operation to set p2p listen on or off
 *
 * DENY: p2p listen mode off when mirroring is active over sta
 * ALLOW: p2p listen mode on when mirroring is inactive
 *
 */
typedef enum _cncNetPolicyConcurrentListen_t
{
    CNC_NET_POLICY_CONCURRENT_LISTEN_DENY,
    CNC_NET_POLICY_CONCURRENT_LISTEN_ALLOW
}cncNetPolicyConcurrentListen_t;

/**
 * \brief During concurrent sta/p2p operation to allow incoming connnection request or not
 *
 * DENY: Deny the connection request.
 * ALLOW: Allow the connection without user invervention.
 */
typedef enum _cncNetPolicyConcurrentConnect_t
{
    CNC_NET_POLICY_CONCURRENT_CONNECT_DENY,
    CNC_NET_POLICY_CONCURRENT_CONNECT_ALLOW
}cncNetPolicyConcurrentConnect_t;

/**
 * \brief To trigger p2p listen/find phase on recurrently at configured periodic intervals
 *
 * DENY:  Recurrent p2p listen/find mode off 
 * ALLOW: Recurrent p2p listen/find mode on at configured periodic intervals
 *
 */
typedef enum _cncNetPolicyRecurrentListen_t
{
    CNC_NET_POLICY_RECURRENT_LISTEN_DENY,
    CNC_NET_POLICY_RECURRENT_LISTEN_ALLOW
}cncNetPolicyRecurrentListen_t;

/**
 * \brief Information for tuning switching sta/ap policy
 *
 */
typedef struct _cncNetPolicyAutoStaApInfo_t
{
	CNC_SINT32 staConnectTimeout;	                    /**< timeout in seconds for station connection, switch to ap on timeout */

	CNC_SINT32 apIdleTimeout;		                    /**< timeout in seconds for ap, try station connection if no stations are connected */

	CNC_SINT32 staMaxConnectRetryCount;                 /**< retry count for station connection before switching to ap
                                                             <0 : unlimited retries, never switch to ap
                                                             0 : no retry, switch to ap mode right away
                                                             >0 : retry for n times and switch to ap if cannot get a connection */

}cncNetPolicyAutoStaApInfo_t;

/**
 *  \brief Policies for handling various modes
 *
 */
typedef struct _cncNetPolicy_t
{
	cncNetPolicyType_t type;								/**< type of policy */

    union
    {
    	cncNetPolicyAutoStaApInfo_t autoStaAp;				/**< information for policy type CNC_NET_POLICY_TYPE_SWITCHING_STA_AP */
    }info;

    cncNetPolicyP2pPbc_t p2pPbc;                        	/**< policy for handling pbc notifications in p2p mode */

    cncNetPolicyConcurrentListen_t concurrentListen;    	/**< policy for concurrent listen capability */

    cncNetPolicyConcurrentConnect_t concurrentConnect;  	/**< policy for concurrent connect capability */
    
    cncNetPolicyRecurrentListen_t recurrentListen;    	    /**< policy for Recurrent listen capability */

    CNC_SINT32 idleCheckTimeout;                        	/**< time out in seconds to trigger idle mode check */

} cncNetPolicy_t;


#ifdef __cplusplus
}
#endif
 
#endif /* END:  _CNC_NET_CONFIG_H_ */

