/*
# COPYRIGHT NOTICE
#
#***************************************************************************
The following software code is the property of Cavium, Inc.
Cavium, Inc. reserves all rights in this software, including but not limited
to copyright and shall be vested with all rights incidental thereto.
Rights granted to you by or under any license shall be limited to those
explicitly granted. You may not record, store, reproduce, reverse engineer,
decompile, inter-operate and/or transmit this software in any medium,
electronic or otherwise, without the express written consent of Cavium, Inc.
ALL RIGHTS RESERVED.
Copyright 2012-2013, Cavium, Inc.
#***************************************************************************
#
# END OF COPYRIGHT NOTICE
#
*/

#ifndef __CNC_NET_ETH_PLUGIN_H__
#define __CNC_NET_ETH_PLUGIN_H__

#include "cnc_nw_exports.h"

#ifdef __cplusplus
extern "C"
{
#endif

/**
 *  \brief Events emitted by plugin
 *
 */
typedef enum _cncNetEthPluginEvent_t
{
    CNC_NET_ETH_PLUGIN_EVENT_MIN,
    CNC_NET_ETH_PLUGIN_EVENT_MAX
} cncNetEthPluginEvent_t;

/**
 * \brief Configuration to setup eth
 *
 */
typedef struct _cncNetEthPluginConfig_t
{
    CNC_CHAR a_ifName[CNC_NET_IF_NAME_LEN];          /**< Name of interface. If empty, plugin will autoselect the interface name */

    CNC_CHAR a_hwAddr[CNC_NET_MAC_STR_LEN];          /**< Mac address of interface, plugin will fill this while configuring */

} cncNetEthPluginConfig_t;

/**
 * \brief Plugin event information
 *
 * Information available with each event from plugin
 */
typedef struct _cncNetEthPluginEventInfo_t
{
    CNC_CHAR a_ifName[CNC_NET_IF_NAME_LEN];              /**< Interface name */

    cncNetEthPluginEvent_t event;                       /**< Event type */

    CNC_CHAR a_mac[CNC_NET_MAC_STR_LEN];                 /**< Device mac address */

} cncNetEthPluginEventInfo_t;

/**
 * \brief Callback function definition for event processor.
 *
 */
typedef cncNetPluginError_t ( *pfn_cncNetEthPlugin_processEvent )( CNC_VOID* p_context, cncNetEthPluginEventInfo_t* p_eventInfo );

/**
 * \brief Function definitions for plugin operations.
 *
 * PluginInstance will expose all relevant functions required to
 * drive the eth interface
 */
typedef struct _cncNetEthPluginOps
{
    /**
     * \brief Plugin will initialize the provided structure with default configuration
     *
     * @param p_instance : [IN] plugin instance.
     * @param p_config : [IN/OUT] configuration will be filled in this structure
     *
     * @return : CNC_NET_PLUGIN_ERROR_NONE on success.
     */
    cncNetPluginError_t ( *getDefaultConfig )( cncNetPluginInstance_t* p_instance, cncNetEthPluginConfig_t* p_config );

    /**
     * \brief Plugin will configure itself with the provided configuration
     *
     * @param p_instance : [IN] plugin instance.
     * @param p_config : [IN] pointer to configuration
     *
     * @return : CNC_NET_PLUGIN_ERROR_NONE on success.
     */
    cncNetPluginError_t ( *configure )( cncNetPluginInstance_t* p_instance, cncNetEthPluginConfig_t* p_config );

    /**
     * \brief Start plugin
     *
     * @param p_instance : [IN] plugin instance.
     * @param pfn_cncNetWiFiPlugin_processEvent : [IN] pointer to event handler function
     *
     * @return : CNC_NET_PLUGIN_ERROR_NONE on success.
     */
    cncNetPluginError_t ( *start )( cncNetPluginInstance_t* p_instance, pfn_cncNetEthPlugin_processEvent pfn_eventHandler );

    /**
     * \brief Stop plugin
     *
     * @param p_instance : [IN] plugin instance.
     *
     * @return : CNC_NET_PLUGIN_ERROR_NONE on success.
     */
    cncNetPluginError_t ( *stop )( cncNetPluginInstance_t* p_instance );

    /**
     * \brief Get Link Status of eth interface
     *
     * @param p_instance : [IN] plugin instance.
     * @param cncNetEthPluginLinkStats_t : [IN] cncNetEthPluginLinkStats_t pointer
     *
     * @return : CNC_NET_PLUGIN_ERROR_NONE on success.
     */
    cncNetPluginError_t ( *getLinkStats )( cncNetPluginInstance_t* p_instance, cncNetEthLinkStat_t* linkStat );

} cncNetEthPluginOps_t;

#ifdef __cplusplus
}
#endif


#endif //__CNC_NET_WIFI_PLUGIN_H__
