/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************
 The following software code is the property of Cavium, Inc.
 Cavium, Inc. reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.
 Rights granted to you by or under any license shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium, Inc.
 ALL RIGHTS RESERVED.
 Copyright 2012-2013, Cavium, Inc.
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */

#ifndef _CNC_NET_UTILS_H
#define _CNC_NET_UTILS_H

#ifdef CNC_TRACE_LEVEL_NETWORK
#if (CNC_TRACE_LEVEL_NETWORK >= 0)
#define TRACE_MODULE_LEVEL   CNC_TRACE_LEVEL_NETWORK
#endif
#endif

#include <stdio.h>
#include <errno.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/stat.h>

#include "cnc_util.h"
#include "cnc_nw_exports.h"
#include "cnc_trace.h"

#ifndef TRACE_MODULE_ID
#define TRACE_MODULE_ID          CNC_TRACE_MODULE_NETWORK
#endif

#ifdef __cplusplus
extern "C"
{
#endif

static inline int _CNC_Net_ExecCmd( char *format, ... )
{
    char    command[2048];
    va_list args;
    int     len;

    va_start( args, format );
    len = vsnprintf( command, sizeof( command ), format, args );
    va_end( args );

    if( len > ( ( int )sizeof( command ) ) - 1 )
    {
        CNC_TRACE_ERROR( 0, "output truncated!!!\n" );
    }

    CNC_TRACE_DETAIL( 0, "[%s]\n", command );

    return CNC_UTIL_system( command );
}

static inline int _CNC_Net_ExecCmdEx( const char* cmd, char* out_buf, int out_buf_len )
{
    size_t bytesRead, bytesLeftInBuf, totalBytesRead;
    FILE* fp;

    CNC_TRACE_DETAIL( 0, "[%s]\n", cmd );

    fp = CNC_UTIL_popen(cmd, "r");
    if( fp == 0 )
    {
        CNC_TRACE_DETAIL( 0, "Failed to popen '%s' (error %d)\n", cmd, errno );
        return -1;
    }

    if( out_buf != CNC_NULL )
    {
        bytesLeftInBuf = out_buf_len - 1;
        totalBytesRead = 0;
        while( !feof( fp ) && bytesLeftInBuf )
        {
            bytesRead = fread( out_buf+totalBytesRead, 1, bytesLeftInBuf, fp );
            if( bytesRead <= 0 )
            {
                break;
            }
            totalBytesRead += bytesRead;
            bytesLeftInBuf -= bytesRead;
        }
        out_buf[totalBytesRead] = 0;
    }

    CNC_UTIL_pclose(fp);

    CNC_TRACE_DETAIL( 0, "[%s]\n", ( out_buf ) ? out_buf : CNC_NULL );
    return 0;
}

static inline CNC_SINT32 CNC_Net_IeeeChannelToFreq( CNC_SINT32 ieeeChannel )
{
    if( ieeeChannel <= 0 )
    {
        return 0;
    }
    if( ieeeChannel <= 14 )
    {
        return 2412 + ( ieeeChannel - 1 ) * 5;
    }
    if( ieeeChannel >=34 && ieeeChannel <= 165 )
    {
        return 5170 + ( ieeeChannel - 34) * 5;
    }
    return 0;
}

static inline CNC_CHAR* __skipSpaces( CNC_CHAR* buf )
{
    CNC_CHAR* p_buf = buf;
    while( ( p_buf != NULL ) && ( *p_buf == ' ' || *p_buf == '\t' ) )
    {
        p_buf++;
    }

    return p_buf;
}

#ifdef __cplusplus
}
#endif

#endif //_CNC_NET_UTILS_H
