/*
# COPYRIGHT NOTICE
#
#***************************************************************************
The following software code is the property of Cavium, Inc.
Cavium, Inc. reserves all rights in this software, including but not limited
to copyright and shall be vested with all rights incidental thereto.
Rights granted to you by or under any license shall be limited to those
explicitly granted. You may not record, store, reproduce, reverse engineer,
decompile, inter-operate and/or transmit this software in any medium,
electronic or otherwise, without the express written consent of Cavium, Inc.
ALL RIGHTS RESERVED.
Copyright 2012-2013, Cavium, Inc.
#***************************************************************************
#
# END OF COPYRIGHT NOTICE
#
*/

#ifndef __CNC_NET_WIFI_PLUGIN_H__
#define __CNC_NET_WIFI_PLUGIN_H__

#include "cnc_nw_exports.h"

#ifdef __cplusplus
extern "C"
{
#endif

/**
 * \brief Flags for p2p entries
 *
 */
typedef enum _cncNetWiFiPluginEventP2PFlags_t
{
    CNC_NET_WIFI_PLUGIN_EVENT_P2P_FLAG_KNOWN_NW = 1,          /**< We have credentials for this p2p peer device */
    CNC_NET_WIFI_PLUGIN_EVENT_P2P_FLAG_PERSISTENT_GROUP = 1<<1	/**< It is P2P persistent group */
}cncNetWiFiPluginEventP2PFlags_t;

/**
 *  \brief Events emitted by wifi plugin
 *
 */
typedef enum _cncNetWiFiPluginEvent_t
{
    CNC_NET_WIFI_PLUGIN_EVENT_MIN,
    CNC_NET_WIFI_PLUGIN_EVENT_P2P_GROUP_STARTED,
    CNC_NET_WIFI_PLUGIN_EVENT_P2P_GROUP_REMOVED,
    CNC_NET_WIFI_PLUGIN_EVENT_P2P_GROUP_FORMATION_SUCCCESS,
    CNC_NET_WIFI_PLUGIN_EVENT_P2P_GROUP_FORMATION_FAILURE,
    CNC_NET_WIFI_PLUGIN_EVENT_P2P_GO_NEG_REQUEST,
    CNC_NET_WIFI_PLUGIN_EVENT_P2P_GO_NEG_SUCCESS,
    CNC_NET_WIFI_PLUGIN_EVENT_P2P_GO_NEG_FAILURE,
    CNC_NET_WIFI_PLUGIN_EVENT_P2P_PROV_DISC_PBC_REQ,
    CNC_NET_WIFI_PLUGIN_EVENT_P2P_PROV_DISC_SHOW_PIN,
    CNC_NET_WIFI_PLUGIN_EVENT_P2P_PROV_DISC_ENTER_PIN,
    CNC_NET_WIFI_PLUGIN_EVENT_P2P_PROV_DISC_FAILURE,
    CNC_NET_WIFI_PLUGIN_EVENT_P2P_INVITATION_RECEIVED,
    CNC_NET_WIFI_PLUGIN_EVENT_WPA_CONNECTING,
    CNC_NET_WIFI_PLUGIN_EVENT_WPA_CONNECTED,
    CNC_NET_WIFI_PLUGIN_EVENT_WPA_DISCONNECTED,
    CNC_NET_WIFI_PLUGIN_EVENT_WPA_EAPOL_FAILED,
    CNC_NET_WIFI_PLUGIN_EVENT_AP_STA_CONNECTED,
    CNC_NET_WIFI_PLUGIN_EVENT_AP_STA_DISCONNECTED,
    CNC_NET_WIFI_PLUGIN_EVENT_WPS_OVERLAP,
    CNC_NET_WIFI_PLUGIN_EVENT_WPS_SUCCESS,
    CNC_NET_WIFI_PLUGIN_EVENT_WPS_FAIL,
    CNC_NET_WIFI_PLUGIN_EVENT_WPS_TIMEOUT,
    CNC_NET_WIFI_PLUGIN_EVENT_WPA_SCAN_RESULTS,
    CNC_NET_WIFI_PLUGIN_EVENT_MAX
} cncNetWiFiPluginEvent_t;

/**
 *  \brief P2P modes supported by wifi plugin
 *
 */
typedef enum _cncNetWiFiPluginP2PMode_t
{
    CNC_NET_WIFI_PLUGIN_MODE_P2P_NONE,
    CNC_NET_WIFI_PLUGIN_MODE_P2P_GC,
    CNC_NET_WIFI_PLUGIN_MODE_P2P_GO
} cncNetWiFiPluginP2PMode_t;

/**
 *  \brief Network types supported by wifi plugin
 *
 */
typedef enum _cncNetWiFiPluginNwInfoType_t
{
    CNC_NET_WIFI_PLUGIN_NW_INFO_TYPE_MIN,
    CNC_NET_WIFI_PLUGIN_NW_INFO_TYPE_WIFI = 1,
    CNC_NET_WIFI_PLUGIN_NW_INFO_TYPE_P2P,
    CNC_NET_WIFI_PLUGIN_NW_INFO_TYPE_P2P_PERSISTENT,
    CNC_NET_WIFI_PLUGIN_NW_INFO_TYPE_MAX
} cncNetWiFiPluginNwInfoType_t;

/**
 *  \brief wifi plugin AP states
 *
 */
typedef enum _cncNetWiFiPluginApState_t
{
    CNC_NET_WIFI_PLUGIN_AP_STATE_STOPPED,
    CNC_NET_WIFI_PLUGIN_AP_STATE_STARTING,
    CNC_NET_WIFI_PLUGIN_AP_STATE_STARTED,
    CNC_NET_WIFI_PLUGIN_AP_STATE_CONNECTED
} cncNetWiFiPluginApState_t;

/**
 *  \brief wifi plugin station states
 *
 */
typedef enum _cncNetWiFiPluginStaState_t
{
    CNC_NET_WIFI_PLUGIN_STA_STATE_DISCONNECTED,
    CNC_NET_WIFI_PLUGIN_STA_STATE_SCANNING,
    CNC_NET_WIFI_PLUGIN_STA_STATE_CONNECTING,
    CNC_NET_WIFI_PLUGIN_STA_STATE_CONNECTED
} cncNetWiFiPluginStaState_t;

/**
 *  \brief wifi plugin P2P states
 *
 */
typedef enum _cncNetWiFiPluginP2PState_t
{
    CNC_NET_WIFI_PLUGIN_P2P_STATE_INACTIVE,
    CNC_NET_WIFI_PLUGIN_P2P_STATE_LISTEN,
    CNC_NET_WIFI_PLUGIN_P2P_STATE_CONNECTING,
    CNC_NET_WIFI_PLUGIN_STA_STATE_GC,
    CNC_NET_WIFI_PLUGIN_STA_STATE_GO
} cncNetWiFiPluginP2PState_t;

/**
 *  \brief Flags for config methods support
 *
 */
typedef enum _cncnetworkConfigMethod_t
{
    CNCNETWORK_CONFIG_METOD_PBC = 1<<0,
    CNCNETWORK_CONFIG_METOD_PIN_DISPLAY = 1<<1,
    CNCNETWORK_CONFIG_METOD_PIN_KEYPAD = 1<<2
} cncnetworkConfigMethod_t;

/**
 *  \brief Flags set by caller in p2pConnect api
 *
 */
typedef enum _cncNetWiFiPluginP2PConnectType_t
{
    CNC_NET_WIFI_PLUGIN_P2P_CONNECT_TYPE_NEGOTIATE,      /**< respond to GO negotiation */
    CNC_NET_WIFI_PLUGIN_P2P_CONNECT_TYPE_JOIN,           /**< respond to invitation to join a group */
    CNC_NET_WIFI_PLUGIN_P2P_CONNECT_TYPE_START,          /**< respond to invitation to start a group */
} cncNetWiFiPluginP2PConnectType_t;

/**
 *  \brief Commands available under p2pGet
 *
 */
typedef enum _cncNetWiFiPluginP2PGetCmd_t
{
    CNC_NET_WIFI_PLUGIN_P2P_GET_CMD_STATE,               /**< get current state */
    CNC_NET_WIFI_PLUGIN_P2P_GET_DEVICE_NAME,             /**< get device name */
} cncNetWiFiPluginP2PGetCmd_t;


/**
 *  \brief Commands available under p2pSet
 *
 */
typedef enum _cncNetWiFiPluginP2PSetCmd_t
{
    CNC_NET_WIFI_PLUGIN_P2P_SET_CMD_LISTEN_CHANNEL,          /**< set listen channel */
    CNC_NET_WIFI_PLUGIN_P2P_SET_CMD_OPERATING_CHANNEL,       /**< set operating channel */
    CNC_NET_WIFI_PLUGIN_P2P_SET_CMD_DEVICE_NAME,             /**< set device name */
} cncNetWiFiPluginP2PSetCmd_t;

/**
 *  \brief Commands available under staGet
 *
 */
typedef enum _cncNetWiFiPluginStaGetCmd_t
{
    CNC_NET_WIFI_PLUGIN_STA_GET_CMD_STATE,                  /**< get current state */
    CNC_NET_WIFI_PLUGIN_STA_GET_CMD_SSID,                   /**< get current ssid */
    CNC_NET_WIFI_PLUGIN_STA_GET_CMD_IP,                     /**< get current ip */
    CNC_NET_WIFI_PLUGIN_STA_GET_CMD_BSSID,					/**< get current bssid */
} cncNetWiFiPluginStaGetCmd_t;

typedef enum _cncNetWiFiPluginStaSetCmd_t
{
    CNC_NET_WIFI_PLUGIN_STA_SET_CMD_SSID,                /**< Set current ssid */
} cncNetWiFiPluginStaSetCmd_t;

typedef enum _cncNetWiFiPluginAPGetCmd_t
{
    CNC_NET_WIFI_PLUGIN_AP_GET_CMD_SSID,                     /**< Get SSID of AP */ 
} cncNetWiFiPluginAPGetCmd_t;

typedef enum _cncNetWiFiPluginAPSetCmd_t
{
    CNC_NET_WIFI_PLUGIN_AP_SET_CMD_SSID,                     /**< set SSID of AP */ 
} cncNetWiFiPluginAPSetCmd_t;

/**
 *  \brief Commands available under p2pAutoGoGet
 *
 */
typedef enum _cncNetWiFiPluginP2PAutoGoGetCmd_t
{
    CNC_NET_WIFI_PLUGIN_P2P_AUTOGO_GET_CMD_STATE,               /**< get current state */
    CNC_NET_WIFI_PLUGIN_P2P_AUTOGO_GET_DEVICE_NAME,             /**< get device name */
} cncNetWiFiPluginP2PAutoGoGetCmd_t;

/**
 *  \brief Commands available under p2pAutoGoSet
 *
 */
typedef enum _cncNetWiFiPluginP2PAutoGoSetCmd_t
{
    CNC_NET_WIFI_PLUGIN_P2P_AUTOGO_SET_CMD_OPERATING_CHANNEL,       /**< set operating channel */
    CNC_NET_WIFI_PLUGIN_P2P_AUTOGO_SET_CMD_DEVICE_NAME,             /**< set device name */
} cncNetWiFiPluginP2PAutoGoSetCmd_t;

/**
 *
 * @param device : [IN] @cncNetDevice_t
 * @return : Index of this device in enumeration
 */
static inline CNC_SINT32 CNC_NET_INDEXOF_WIFI_DEVICE( cncNetDevice_t device )
{
    CNC_SINT32 index = 0;

    while( ( device >>= 1 ) )
    {
        index++;
    }
    return index;
}

/**
 * \brief Configuration to setup station mode
 *
 */
typedef struct _cncNetWiFiPluginStaConfig_t
{
    CNC_CHAR a_ifName[CNC_NET_IF_NAME_LEN];          /**< Name of sta interface If empty, plugin will autoselect the interface name */

    CNC_CHAR a_hwAddr[CNC_NET_MAC_STR_LEN];          /**< Mac address of interface, plugin will fill this while configuring */

    CNC_BOOL persistence;                           /**< Persistence support */

    CNC_BOOL autoConnect;                           /**< autoconnect to known ap on startup */

    CNC_BOOL widiGen2;                              /**< WIDI gen2 (L2SD) support. Requires station mode support. */
} cncNetWiFiPluginStaConfig_t;

/**
 * \brief Configuration to setup AP mode
 *
 */
typedef struct _cncNetWiFiPluginApConfig_t
{
    CNC_CHAR a_ifName[CNC_NET_IF_NAME_LEN];          /**< Name of ap interface If empty, plugin will autoselect the interface name */

    CNC_CHAR a_hwAddr[CNC_NET_MAC_STR_LEN];          /**< Mac address of interface, plugin will fill this while configuring */

    CNC_CHAR a_ssid[CNC_NET_WIFI_SSID_LEN];          /**< SSID to be used */

    CNC_UINT32 ieeeChannel;                         /**< Operating Channel, set to 0 for auto channel selection */

    cncNetAuth_t auth;                               /**< Security to be applied for this AP */
    
    CNC_UINT32 conctInactvTimeout;                  /**< inactivity timeout for connected station */
} cncNetWiFiPluginApConfig_t;

/**
 * \brief Configuration to setup P2P mode
 *
 */
typedef struct _cncNetWiFiPluginP2PConfig_t
{
    CNC_CHAR a_ifName[CNC_NET_IF_NAME_LEN];          /**< Name of p2p interface If empty, plugin will autoselect the interface name */

    CNC_CHAR a_devName[CNC_NET_WIFI_SSID_LEN];       /**< p2p device name */

    CNC_CHAR a_hwAddr[CNC_NET_MAC_STR_LEN];          /**< Mac address of interface, plugin will fill this while configuring */

    CNC_SINT32 ieeeListenChannel;                   /**< ieee listen channel ( 1,6 or 11, 0 == auto ) */

    CNC_SINT32 ieeeOperatingChannel;                /**< ieee operating channel */

    CNC_CHAR a_ssidPostfix[24];                     /**< postfix after DIRECT-xx, ssid is 32 bytes so postfix cannot be longer than 23 bytes */

    CNC_BOOL persistence;                           /**< Persistence support, invitation will not be supported if persistence is off */

    CNC_SINT32 groupIdleTimeout;                    /**< wait number of seconds for peer device to connect */

    CNC_BOOL wfd;                                   /**< Miracast support */

    CNC_BOOL widiGen3;                              /**< WIDI gen3 (Miracast) support. Will implicitly enable wfd mode. */

	cncNetWpsPinType_t wps_pin_type;				/**< WPS PIN length 4 digits or 8 digits. */
	
	cncNetP2pFindType_t p2p_find_type;				/**< P2P FIND scan type, full scan or social scan. */

	cncnetworkConfigMethod_t configMethod;			/**< Config methods */
} cncNetWiFiPluginP2PConfig_t;

/**
 *  \brief configuration of of various modes
 *
 */
typedef struct _cncNetWiFiPluginConfig_t
{
    cncNetType_t types;                      	/**< Types of networks defined in @cncNetType_t */

    cncNetSubType_t subTypes;                	/**< SubType of networks defined in @cncNetSubType_t */

    cncNetWiFiPluginStaConfig_t sta;         	/**< Configuration for station mode */

    cncNetWiFiPluginApConfig_t ap;           	/**< Configuration for ap mode */

    cncNetWiFiPluginP2PConfig_t p2p;         	/**< Configuration for p2p mode */

    cncNetWiFiPluginP2PConfig_t p2pAutoGO;   	/**< Configuration for p2p auto GO mode */

    cncNetWiDiConfig_t widi;					/**< configuration for widi sta/p2p mode */

    CNC_CHAR a_tmpPath[256];                	/**< Temporary filesystem path plugin will use this path to create any temporary
                                                 	 persistent files like supplicant config files. */
} cncNetWiFiPluginConfig_t;

/**
 * \brief Network configuration
 *
 * This can be used to connect to a network
 */
typedef struct _cncNetWiFiPluginNwConfig_t
{
    CNC_SINT32 index;                       /**< Index of network entry */

    CNC_CHAR a_ssid[CNC_NET_WIFI_SSID_LEN];  /**< SSID of network */

    CNC_CHAR a_bssid[CNC_NET_MAC_STR_LEN];   /**< Macaddress of AP */

    cncNetAuth_t auth;                       /**< Security credentials to be applied for this AP */
} cncNetWiFiPluginNwConfig_t;

/**
 * \brief Plugin event information
 *
 * Information available with every wifi event from plugin
 */
typedef struct _cncNetWiFiPluginEventInfo_t
{
    CNC_CHAR a_ifName[CNC_NET_IF_NAME_LEN];              /**< Interface name */

    cncNetWiFiPluginEvent_t event;                       /**< Event type */

    CNC_CHAR a_mac[CNC_NET_MAC_STR_LEN];                 /**< Device mac address */

    union
    {
        struct                                          /**< valid for p2p events */
        {
            cncNetWiFiPluginP2PMode_t mode;              /**< P2P mode GC/GO */

            CNC_CHAR a_goDevAddr[CNC_NET_MAC_STR_LEN];   /**< GO address */

            cncNetWpsMethod_t wpsMethod;                 /**< Supported wps methods */

            cncNetWiFiPluginEventP2PFlags_t flags;       /**< flags related to p2p mode */

            CNC_CHAR a_p2pDevAddr[CNC_NET_MAC_STR_LEN];  /**< P2P device address, can be different from a_mac */

            CNC_CHAR a_pin[12];                         /**< wps pin if provided by underlying layer or peer device */

            CNC_CHAR a_ssid[CNC_NET_WIFI_SSID_LEN];      /**< ssid of peer device */
            
            CNC_UINT32 op_freq;							/**< peer device's prefered operating or autoGO operating channel */

            CNC_CHAR a_passphrase[CNC_NET_WPA_PSK_LEN]; /**< passphrase in case of p2pGO or p2pAutoGO */

        } p2p;
    };
} cncNetWiFiPluginEventInfo_t;

/**
 * \brief Network information
 *
 * This can be used to create a new network
 * or
 * Get information of existing or connected network
 */
typedef struct _cncNetWiFiPluginNwInfo_t
{
    cncNetWiFiPluginNwInfoType_t type;                   /**< Wifi network type */

    CNC_CHAR a_ssid[CNC_NET_WIFI_SSID_LEN];              /**< ssid */

    cncNetAuth_t auth;                                   /**< Authentication information */

    CNC_CHAR a_bssid[CNC_NET_MAC_STR_LEN];               /**< bssid of peer device */

    CNC_CHAR a_flags[CNC_NET_FLAG_STR_LEN];

    CNC_UINT32 index;                                   /**< index of network configuration valid if >= 0 */

    CNC_UINT32 freq;                                    /**< frequency */
    
    cncNetWiFiPluginP2PMode_t p2pMode;					/**< P2P mode GO/GC */
} cncNetWiFiPluginNwInfo_t;

/**
 * \brief P2P information required to start a new connection
 *
 */
typedef struct _cncNetWiFiPluginP2PConnectInfo_t
{
    CNC_CHAR a_peerDevAddr[CNC_NET_MAC_STR_LEN];         /**< P2P peer device address */

    cncNetWiFiPluginP2PConnectType_t type;               /**< p2p connection type */

    cncNetP2pFindType_t findType;						/**< p2p find scan type */

    CNC_BOOL passive;                                   /**< do/donot initiate reverse connection */

    cncNetAuthWps_t wpsInfo;                             /**< wps methods and related information */
    
    CNC_UINT32 op_freq;									/**< force operating frequence */
    } cncNetWiFiPluginP2PConnectInfo_t;

/**
 * \brief Information about p2p peer device
 *
 */
typedef struct _cncNetWiFiPluginP2PPeerInfo_t
{
    CNC_CHAR a_peerDevAddr[CNC_NET_MAC_STR_LEN];        /**< P2P peer device address */

    cncNetWpsMethod_t wpsMethod;                        /**< supported wps methods */
    
    CNC_CHAR a_deviceName[CNC_NET_WIFI_SSID_LEN];       /**< Device Name */

    cncNetP2pDeviceCaps_t deviceCaps;                   /**< device capabilities */

    cncNetP2pGroupCaps_t groupCaps;                     /**< group capcbilities */
   
} cncNetWiFiPluginP2PPeerInfo_t;

/**
 * \brief P2P information related to p2pGet/p2pSet commands
 *
 */
typedef union _cncNetWiFiPluginP2PCmdInfo_t
{
    cncNetWiFiPluginP2PState_t state;                   /**< P2P state */

    CNC_SINT32 ieeeChannel;                             /**< listen/operating channel */
    
    CNC_CHAR a_deviceName[CNC_NET_WIFI_SSID_LEN];       /**< Device Name */

} cncNetWiFiPluginP2PCmdInfo_t;

/**
 * \brief STA information related to p2pGet/p2pSet commands
 *
 */
typedef union _cncNetWiFiPluginStaCmdInfo_t
{
    cncNetWiFiPluginStaState_t state;               /**< STA state */

    CNC_CHAR a_ssid[CNC_NET_WIFI_SSID_LEN];         /**< ssid */
    
    CNC_CHAR a_bssid[CNC_NET_MAC_STR_LEN];			/**< bssid */

    CNC_CHAR a_ip[CNC_NET_IP_STR_LEN];              /**< acquired ip address */

} cncNetWiFiPluginStaCmdInfo_t;

/**
 * \brief AP information related to p2pGet/p2pSet commands
 *
 */
typedef union _cncNetWiFiPluginApCmdInfo_t
{
    cncNetWiFiPluginApState_t state;                    /**< AP state */

    CNC_CHAR a_ssid[CNC_NET_WIFI_SSID_LEN];              /**< ssid */

} cncNetWiFiPluginApCmdInfo_t;

/**
 * \brief Callback function definition for network iterator.
 *
 */
typedef cncNetPluginError_t ( *pfn_cncNetWiFiPlugin_processNetwork )( CNC_VOID* p_context, cncNetWiFiPluginNwInfo_t* p_nwInfo );

/**
 * \brief Callback function definition for event processor.
 *
 */
typedef cncNetPluginError_t ( *pfn_cncNetWiFiPlugin_processEvent )( CNC_VOID* p_context, cncNetWiFiPluginEventInfo_t* p_eventInfo );

/**
 * \brief Callback function definition for scan list iterator.
 *
 */
typedef cncNetPluginError_t ( *pfn_cncNetWiFiPlugin_processBss )( CNC_VOID* p_context, cncNetWiFiBssInfo_t* p_bssInfo );

/**
 * \brief Function definitions for plugin operations.
 *
 * PluginInstance will expose all relevant functions required to
 * drive the wifi interface
 */
typedef struct _cncNetWiFiPluginOps
{
    /**
     * \brief Plugin will initialize the provided structure with default configuration
     *
     * @param p_instance : [IN] plugin instance.
     * @param p_config : [IN/OUT] configuration will be filled in this structure
     *
     * @return : CNC_NET_PLUGIN_ERROR_NONE on success.
     */
    cncNetPluginError_t ( *getDefaultConfig )( cncNetPluginInstance_t* p_instance, cncNetWiFiPluginConfig_t* p_config );

    /**
     * \brief Plugin will configure itself with the provided configuration
     *
     * @param p_instance : [IN] plugin instance.
     * @param p_config : [IN] pointer to configuration
     *
     * @return : CNC_NET_PLUGIN_ERROR_NONE on success.
     */
    cncNetPluginError_t ( *configure )( cncNetPluginInstance_t* p_instance, cncNetWiFiPluginConfig_t* p_config );

    /**
     * \brief Start plugin
     *
     * @param p_instance : [IN] plugin instance.
     * @param pfn_cncNetWiFiPlugin_processEvent : [IN] pointer to event handler function
     *
     * @return : CNC_NET_PLUGIN_ERROR_NONE on success.
     */
    cncNetPluginError_t ( *start )( cncNetPluginInstance_t* p_instance, pfn_cncNetWiFiPlugin_processEvent pfn_eventHandler );

    /**
     * \brief Stop plugin
     *
     * @param p_instance : [IN] plugin instance.
     *
     * @return : CNC_NET_PLUGIN_ERROR_NONE on success.
     */
    cncNetPluginError_t ( *stop )( cncNetPluginInstance_t* p_instance );

    /**
     * \brief Get currently set regulatory domain
     *
     * @param p_instance : [IN] plugin instance.
     * @param p_outBuf : [OUT] pointer to buffer
     * @param outBufLen : [IN] size of buffer
     *
     * @return : CNC_NET_PLUGIN_ERROR_NONE on success.
     */
    cncNetPluginError_t ( *getRegDomain )( cncNetPluginInstance_t* p_instance, CNC_CHAR* p_outBuf, CNC_UINT32 outBufLen );

    /**
     * \brief Set regulatory domain
     *
     * @param p_instance : [IN] plugin instance.
     * @param p_inBuf : [IN] pointer to buffer
     * @param inBufLen : [IN] size of buffer
     *
     * @return : CNC_NET_PLUGIN_ERROR_NONE on success.
     */
    cncNetPluginError_t ( *setRegDomain )( cncNetPluginInstance_t* p_instance, CNC_CHAR* p_inBuf, CNC_UINT32 inBufLen );

    /**
     * \brief Get STA information
     *
     * @param p_instance : [IN] plugin instance.
     * @param cmd : [IN] command defined in cncNetWiFiPluginStaGetCmd_t*
     * @param p_Info : [OUT] pointer to cncNetWiFiPluginStaCmdInfo_t
     *
     * @return : CNC_NET_PLUGIN_ERROR_NONE on success.
     */
    cncNetPluginError_t ( *staGet )( cncNetPluginInstance_t* p_instance, cncNetWiFiPluginStaGetCmd_t cmd, cncNetWiFiPluginStaCmdInfo_t* p_Info );
   
     /**
     * \brief Set STA information
     *
     * @param p_instance : [IN] plugin instance.
     * @param cmd : [IN] command defined in cncNetWiFiPluginStaSetCmd_t*
     * @param p_Info : [OUT] pointer to cncNetWiFiPluginStaCmdInfo_t
     *
     * @return : CNC_NET_PLUGIN_ERROR_NONE on success.
     */
    cncNetPluginError_t ( *staSet )( cncNetPluginInstance_t* p_instance, cncNetWiFiPluginStaSetCmd_t cmd, cncNetWiFiPluginStaCmdInfo_t* p_Info );

    /**
     * \brief Iterate saved networks
     *
     * @param p_instance : [IN] plugin instance.
     * @param pfn_processNetwork : [IN] pointer to pfn_cncNetWiFiPlugin_processNetwork
     *
     * @return : CNC_NET_PLUGIN_ERROR_NONE on success.
     */
    cncNetPluginError_t ( *staIterateNetworks )( cncNetPluginInstance_t* p_instance, pfn_cncNetWiFiPlugin_processNetwork pfn_processNetwork );

    /**
     * \brief Initiate connection to AP
     *
     * @param p_instance : [IN] plugin instance.
     * @param p_nwConfig : [IN] configuration to be used for connection
     * @param newNetwork : [IN} 1 if connecting to new network
     *
     * @return : CNC_NET_PLUGIN_ERROR_NONE on success.
     */
    cncNetPluginError_t ( *staConnectNetwork )( cncNetPluginInstance_t* p_instance, cncNetWiFiPluginNwConfig_t* p_nwConfig, CNC_BOOL newNetwork );

    /**
     * \brief Remove saved network credentials
     *
     * @param p_instance : [IN] plugin instance.
     * @param p_nwConfig : [IN] information about saved network
     *
     * @return : CNC_NET_PLUGIN_ERROR_NONE on success.
     */
    cncNetPluginError_t ( *staRemoveNetwork )( cncNetPluginInstance_t* p_instance, cncNetWiFiPluginNwConfig_t* p_nwConfig );

    /**
     * \brief Disconnect from AP
     *
     * @param p_instance : [IN] plugin instance.
     *
     * @return : CNC_NET_PLUGIN_ERROR_NONE on success.
     */
    cncNetPluginError_t ( *staDisconnect )( cncNetPluginInstance_t* p_instance );

    /**
     * \brief Start a full scan
     *
     * @param p_instance : [IN] plugin instance.
     *
     * @return : CNC_NET_PLUGIN_ERROR_NONE on success.
     */
    cncNetPluginError_t ( *staScan )( cncNetPluginInstance_t* p_instance );

    /**
     * \brief Iterate scan results
     *
     * @param p_instance : [IN] plugin instance.
     * @param pfn_processBss : [IN] pointer to pfn_cncNetWiFiPlugin_processBss
     *
     * @return : CNC_NET_PLUGIN_ERROR_NONE on success.
     */
    cncNetPluginError_t ( *staIterateScanResults )( cncNetPluginInstance_t* p_instance, pfn_cncNetWiFiPlugin_processBss pfn_processBss );

    /**
     * \brief Get Link Status of sta interface
     *
     * @param p_instance : [IN] plugin instance.
     * @param cncNetWifiPluginLinkStats_t : [IN] cncNetWifiPluginLinkStats_t pointer
     *
     * @return : CNC_NET_PLUGIN_ERROR_NONE on success.
     */
    cncNetPluginError_t ( *staGetLinkStats )( cncNetPluginInstance_t* p_instance, cncNetWiFiLinkStat_t* linkStat );


    /**
     * \brief Get P2P information
     *
     * @param p_instance : [IN] plugin instance.
     * @param cmd : [IN] command defined in cncNetWiFiPluginP2PGetCmd_t*
     * @param p_Info : [OUT] pointer to cncNetWiFiPluginP2PCmdInfo_t
     *
     * @return : CNC_NET_PLUGIN_ERROR_NONE on success.
     */
    cncNetPluginError_t ( *p2pGet )( cncNetPluginInstance_t* p_instance, cncNetWiFiPluginP2PGetCmd_t cmd, cncNetWiFiPluginP2PCmdInfo_t* p_Info );

    /**
     * \brief Get P2P information
     *
     * @param p_instance : [IN] plugin instance.
     * @param cmd : [IN] command defined in cncNetWiFiPluginP2PSetCmd_t
     * @param p_Info : [IN] pointer to cncNetWiFiPluginP2PCmdInfo_t
     *
     * @return : CNC_NET_PLUGIN_ERROR_NONE on success.
     */
    cncNetPluginError_t ( *p2pSet )( cncNetPluginInstance_t* p_instance, cncNetWiFiPluginP2PSetCmd_t cmd, cncNetWiFiPluginP2PCmdInfo_t* p_Info );

    /**
     * \brief Initiate P2P connection
     *
     * @param p_instance : [IN] plugin instance.
     * @param p_connectInfo : [IN] configuration to be used for connection
     *
     * @return : CNC_NET_PLUGIN_ERROR_NONE on success.
     */
    cncNetPluginError_t ( *p2pConnect )( cncNetPluginInstance_t* p_instance, cncNetWiFiPluginP2PConnectInfo_t* p_connectInfo );

    /**
     * \brief Disconnect from peer p2p device
     *
     * @param p_instance : [IN] plugin instance.
     *
     * @return : CNC_NET_PLUGIN_ERROR_NONE on success.
     */
    cncNetPluginError_t ( *p2pDisconnect )( cncNetPluginInstance_t* p_instance );

    /**
     * \brief Enable/disable discovery
     *
     * @param p_instance : [IN] plugin instance.
     * @param enable : [IN] 1=enable 0=disable
     * @param passive : [IN] 1=passive, 0=active
     *
     * @return : CNC_NET_PLUGIN_ERROR_NONE on success.
     */
    cncNetPluginError_t ( *p2pDiscover )( cncNetPluginInstance_t* p_instance, CNC_BOOL enable, CNC_BOOL  passive );

    /**
     * \brief Is ieee channel supported in p2p mode
     *
     * @param p_instance : [IN] plugin instance.
     * @param ieeeChannel : [IN] ieee channel
     *
     * @return : CNC_NET_PLUGIN_ERROR_NONE on success.
     */
    cncNetPluginError_t ( *p2pIsChannelSupported )( cncNetPluginInstance_t* p_instance, CNC_UINT32 ieeeChannel );

    /**
     * \brief Get P2P information of peer device
     *
     * @param p_instance : [IN] plugin instance.
     * @param p_peerInfo : [IN,OUT] pointer to cncNetWiFiPluginP2PPeerInfo_t, information will be filled in this structure
     *
     * @return : CNC_NET_PLUGIN_ERROR_NONE on success.
     */
    cncNetPluginError_t ( *p2pGetPeerInfo )( cncNetPluginInstance_t* p_instance, cncNetWiFiPluginP2PPeerInfo_t* p_peerInfo );

    /**
     * \brief Iterate saved p2p networks
     *
     * @param p_instance : [IN] plugin instance.
     * @param pfn_processNetwork : [IN] pointer to pfn_cncNetWiFiPlugin_processNetwork
     *
     * @return : CNC_NET_PLUGIN_ERROR_NONE on success.
     */
    cncNetPluginError_t ( *p2pIterateNetworks )(  cncNetPluginInstance_t* p_instance, pfn_cncNetWiFiPlugin_processNetwork pfn_processNetwork  );

    /**
     * \brief Remove saved p2p network credentials
     *
     * @param p_instance : [IN] plugin instance.
     * @param p_nwConfig : [IN] information about saved network
     *
     * @return : CNC_NET_PLUGIN_ERROR_NONE on success.
     */
    cncNetPluginError_t ( *p2pRemoveNetwork )( cncNetPluginInstance_t* p_instance, cncNetWiFiPluginNwConfig_t* p_nwConfig );

    /**
     * \brief Start pbc based connection with any p2p device
     *
     * @param p_instance : [IN] plugin instance.
     *
     * @return : CNC_NET_PLUGIN_ERROR_NONE on success.
     */
    cncNetPluginError_t ( *p2pAutoGoWpsPbcStart )( cncNetPluginInstance_t* p_instance );

    /**
     * \brief Get P2P AutoGO information
     *
     * @param p_instance : [IN] plugin instance.
     * @param cmd : [IN] command defined in cncNetWiFiPluginP2PGetCmd_t*
     * @param p_Info : [OUT] pointer to cncNetWiFiPluginP2PCmdInfo_t
     *
     * @return : CNC_NET_PLUGIN_ERROR_NONE on success.
     */
    cncNetPluginError_t ( *p2pAutoGoGet )( cncNetPluginInstance_t* p_instance, cncNetWiFiPluginP2PAutoGoGetCmd_t cmd, cncNetWiFiPluginP2PCmdInfo_t* p_Info );

    /**
     * \brief Get P2P AutoGO information
     *
     * @param p_instance : [IN] plugin instance.
     * @param cmd : [IN] command defined in cncNetWiFiPluginP2PSetCmd_t
     * @param p_Info : [IN] pointer to cncNetWiFiPluginP2PCmdInfo_t
     *
     * @return : CNC_NET_PLUGIN_ERROR_NONE on success.
     */
    cncNetPluginError_t ( *p2pAutoGoSet )( cncNetPluginInstance_t* p_instance, cncNetWiFiPluginP2PAutoGoSetCmd_t cmd, cncNetWiFiPluginP2PCmdInfo_t* p_Info );

    /**
     * \brief Get Link Status of p2p interface
     *
     * @param p_instance : [IN] plugin instance.
     * @param cncNetWifiPluginLinkStats_t : [IN] cncNetWifiPluginLinkStats_t pointer
     *
     * @return : CNC_NET_PLUGIN_ERROR_NONE on success.
     */
    cncNetPluginError_t ( *p2pGetLinkStats )( cncNetPluginInstance_t* p_instance, cncNetWiFiLinkStat_t* linkStat );

    /**
     * \brief Start WPS connection with any station device
     *
     * @param p_instance : [IN] plugin instance.
     * @param p_nwConfig : [IN,OUT] configuration to be used for connection and PIN returned in Display method
     *
     * @return : CNC_NET_PLUGIN_ERROR_NONE on success.
     */
    cncNetPluginError_t ( *apWpsStart )( cncNetPluginInstance_t* p_instance, cncNetWiFiPluginNwConfig_t* p_nwConfig );

     /**
     * \brief Get Link Status of ap interface
     *
     * @param p_instance : [IN] plugin instance.
     * @param cncNetWifiPluginLinkStats_t : [IN] cncNetWifiPluginLinkStats_t pointer
     *
     * @return : CNC_NET_PLUGIN_ERROR_NONE on success.
     */
    cncNetPluginError_t ( *apGetLinkStats )( cncNetPluginInstance_t* p_instance, cncNetWiFiLinkStat_t* linkStat );

     /**
     * \brief Get AP information
     *
     * @param p_instance : [IN] plugin instance.
     * @param cmd : [IN] command defined in cncNetWiFiPluginStaGetCmd_t*
     * @param p_Info : [OUT] pointer to cncNetWiFiPluginStaCmdInfo_t
     *
     * @return : CNC_NET_PLUGIN_ERROR_NONE on success.
     */
    cncNetPluginError_t ( *apGet )( cncNetPluginInstance_t* p_instance, cncNetWiFiPluginAPGetCmd_t cmd, cncNetWiFiPluginApCmdInfo_t* p_Info );
   
     /**
     * \brief Set AP information
     *
     * @param p_instance : [IN] plugin instance.
     * @param cmd : [IN] command defined in cncNetWiFiPluginAPSetCmd_t*
     * @param p_Info : [OUT] pointer to cncNetWiFiPluginApCmdInfo_t
     *
     * @return : CNC_NET_PLUGIN_ERROR_NONE on success.
     */
    cncNetPluginError_t ( *apSet )( cncNetPluginInstance_t* p_instance, cncNetWiFiPluginAPSetCmd_t cmd, cncNetWiFiPluginApCmdInfo_t* p_Info );

    /**
     * \brief Start a full scan
     *
     * @param p_instance : [IN] plugin instance.
     *
     * @return : CNC_NET_PLUGIN_ERROR_NONE on success.
     */
    cncNetPluginError_t ( *apScan )( cncNetPluginInstance_t* p_instance );

    /**
     * \brief Iterate scan results
     *
     * @param p_instance : [IN] plugin instance.
     * @param pfn_processBss : [IN] pointer to pfn_cncNetWiFiPlugin_processBss
     *
     * @return : CNC_NET_PLUGIN_ERROR_NONE on success.
     */
    cncNetPluginError_t ( *apIterateScanResults )( cncNetPluginInstance_t* p_instance, pfn_cncNetWiFiPlugin_processBss pfn_processBss );

    /**
     * \brief Get peer WFD port
     *
     * @param p_instance : [IN] plugin instance.
     * @param p_peerP2PDevAddr : [IN] peer p2p device address
     * @param pi_port : [OUT] pointer to bufffer for receiving peer port.
     *
     * @return : CNC_NET_PLUGIN_ERROR_NONE on success.
     */
    cncNetPluginError_t ( *wfdGetPeerPort )( cncNetPluginInstance_t* p_instance, CNC_CHAR* p_peerP2PDevAddr, CNC_SINT32* pi_port );

    /**
     * \brief Set/reset wfd in session bit in wfd information element
     *
     * @param p_instance : [IN] plugin instance.
     * @param set : [IN] 1=set 0=reset
     *
     * @return : CNC_NET_PLUGIN_ERROR_NONE on success.
     */
    cncNetPluginError_t ( *wfdSetInSession )( cncNetPluginInstance_t* p_instance, CNC_BOOL set );

    /**
     * \brief Iterate WIDI device scan results
     *
     * @param p_instance : [IN] plugin instance.
     * @param pfn_processBss : [IN] pointer to pfn_cncNetWiFiPlugin_processBss
     *
     * @return : CNC_NET_PLUGIN_ERROR_NONE on success.
     */
    cncNetPluginError_t ( *widiIterateScanResults )( cncNetPluginInstance_t* p_instance, pfn_cncNetWiFiPlugin_processBss pfn_processBss );

    /**
     * \brief Update WIDI information
     *
     * @param p_instance : [IN] plugin instance.
     * @param p_vndrExtnIe : [IN] pointer to vendor extension ie
     * @param p_primDevIe : [IN] pointer to primary device ie
     * @param p_devName : [IN] pointer to device name
     *
     * @return : CNC_NET_PLUGIN_ERROR_NONE on success.
     */
    cncNetPluginError_t ( *widiSetInfo )( cncNetPluginInstance_t* p_instance, CNC_CHAR* p_vndrExtnIe, CNC_CHAR* p_primDevIe, CNC_CHAR* p_wpsSecDevList, CNC_CHAR* p_p2pSecDevList, CNC_CHAR* p_devName );

} cncNetWiFiPluginOps_t;

#ifdef __cplusplus
}
#endif


#endif //__CNC_NET_WIFI_PLUGIN_H__
