/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Cavium Inc.
 Cavium Inc., reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium Inc.,

 ALL RIGHTS RESERVED.

 Copyright 2013-2014, Cavium Inc.,
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */

/**
 * @file cnc_rui.h
 * @brief CRISP Remote User Interface Module header file.
 * \n Contains all required information about CNC_RUI Core APIs, Structure and Enum members to Develop RUI Application
 */

#ifndef __CNC_RUI_H__
#define __CNC_RUI_H__
/* ======================== System Includes =============================== */
/* NONE*/
/*========================= User includes ================================= */
#include "cnc_common_typedef.h"

/*========================= (Internal) Macro definitions=================== */
#define MAX_NAME_SIZE               256 /**< Maximum Name String Size */

#define MAX_FORMAT_SIZE             32  /**< Maximum Format String length */

#define MIME_TYPE_SIZE              32  /**< MimeType size */

#define ICON_URL_SIZE               128 /**< Icon URL size */

#define USER_AGENT_LENGTH           64 /**<Maximum XML Doc User Agent Length*/

/* ================= (Internal) Type definitions =========================== */

/**
 * \ struct  _cncDeviceIconList_t
 * \brief    This Structure contains source device's iconList information
 */
typedef struct _cncDevIconList_t
{
    CNC_SINT16                      height;                     /**<  Height of Icon */
    CNC_SINT16                      width;                      /**<  Width of Icon */
    CNC_SINT16                      depth;                      /**<  Depth of Icon*/
    CNC_CHAR                        a_mimeType[MIME_TYPE_SIZE]; /**<  MimeType of Icon */
    CNC_CHAR                        a_iconURL[ICON_URL_SIZE];   /**<  Icon URL */
    struct _cncDevIconList_t        *p_next;                    /**<  Points next icon with different resolution (or) mimeType of same RUI-S device*/

}cncDevIconList_t;


/**
 * \ struct  _cncRuiDeviceList_t
 * \brief    This Structure will Contains List of all RUI Server devices present in the Network.
 * */
typedef struct _cncRuiDeviceList_t
{
    CNC_CHAR                        a_name[MAX_NAME_SIZE];      /**<  RUI Server Device Friendly Name */
    CNC_CHAR                        a_udn[MAX_NAME_SIZE];       /**<  UUID of RUI Server device */
    CNC_CHAR                        a_baseUrl[MAX_NAME_SIZE];   /**<  Base URL of RUI Server device*/
    cncDevIconList_t               *p_iconList;                 /**<  Points Icon List of cncDevIconList_t type Which Contains all Icon Types of Same RUI Server Device. */
    struct _cncRuiDeviceList_t     *p_nextDevice;               /**<  Next RUI Server Device in the List */

} cncRuiDeviceList_t;

/**
 * \ enum  cncRUIProtocolType_t
 * \brief  cncRUIProtocolType_t enumeration will Contains Different RUI protocol Types.
 */
typedef enum  _cncRUIProtocolType_t
{
    CNC_RUI_PROTOCOL_NONE           = 0,    /**< 0 */
    CNC_RUI_PROTOCOL_HTML5          = 1,    /**< 1 */
    CNC_RUI_PROTOCOL_LRDP,                  /**< 2 */
    CNC_RUI_PROTOCOL_RDP,                   /**< 3 */
    CNC_RUI_RROTOCOL_RVU,                   /**< 4 */
    CNC_RUI_PROTOCOL_SGXML,                 /**< 5 */
    CNC_RUI_PROTOCOL_UIF,                   /**< 6 */
    CNC_RUI_PROTOCOL_VNC,                   /**< 7 */
    CNC_RUI_PROTOCOL_XHT,                   /**< 8 */
    CNC_RUI_PROTOCOL_XRT,                   /**< 9 */

}cncRUIProtocolType_t;


/**
 * \ enum  cncRuiErrCode_t
 * \brief  cncRuiErrCode_t enumeration will Contains All possible RUI error codes.
 */
typedef enum _cncRuiErrCode_t
{
    CNC_RUI_ERR_CODE_NONE           = 0,         /**< 0 */
    CNC_RUI_ERR_CODE_INVALID_PARAM  = -500,      /**< -500 */
    CNC_RUI_ERR_CODE_NETWORK,                    /**< -499 */
    CNC_RUI_ERR_CODE_INVALID_REQUEST,            /**< -498 */
    CNC_RUI_ERR_CODE_NO_MEMORY,                  /**< -497*/
    CNC_RUI_ERR_CODE_LOWER_LAYER,                /**< -496*/
    CNC_RUI_ERR_CODE_CONNECTION_FAILED,          /**< -495*/
    CNC_RUI_ERR_CODE_SERVER_NOT_RESPONDING,      /**< -494*/
    CNC_RUI_ERR_CODE_SERVER_NOT_FOUND,           /**< -493 */
    CNC_RUI_ERR_CODE_NW_TIMEOUT,                 /**< -492*/
    CNC_RUI_ERR_CODE_AUTHORIZATION_FAILED,       /**< -491 */
    CNC_RUI_ERR_CODE_BUSY,                       /**< -490 */
    CNC_RUI_ERR_CODE_API_SEQUENCE,               /**< -489 */
    CNC_RUI_ERR_CODE_SERV_UI_LIST_NOT_AVAILABLE, /**< -488 */
    CNC_RUI_ERR_CODE_UPNP_API_FAILURE,           /**< -487 */

    /* Note: Add new errors above this comment */
    CNC_RUI_ERR_CODE_UNKNOWN_ERROR               /**< -486 */

}cncRuiErrCode_t;


/**
 * \ struct  cncRUIProtocolURI_t
 * \brief   This Structure will Contains Details about Protocol URL Lists
 */
typedef struct _cncRUIProtocolURI_t
{
    CNC_CHAR                    a_protocolURI[MAX_NAME_SIZE];   /**< Contains Details about Protocol URL >**/
    struct _cncRUIProtocolURI_t *p_next;

}cncRUIProtocolURI_t;


/**
 * \ struct  cncRUIProtocolDetails_t
 * \brief    This Structure will Contains Protocol Details of RUI Server.
 */
typedef struct _cncRUIProtocolDetail_t
{
    cncRUIProtocolType_t        RUIProtocol;                      /**< Protocol Type supported by RUI Server >**/
    cncRUIProtocolURI_t         *p_protocolURI;                   /**< Protocol URL List.*/
    CNC_CHAR                    a_protocolInfo[MAX_NAME_SIZE];    /**< Protocol Info */

}cncRUIProtocolDetails_t;


/**
 * \ struct  cncRUIList_t
 * \brief    This structure will Contains All User Interface Lists supported by RUI Server.
 */
typedef struct _cncRUIList_t
{
    CNC_CHAR                    a_uiID[MAX_NAME_SIZE];          /**< user Interface ID*/
    CNC_CHAR                    a_name[MAX_NAME_SIZE];          /**< UI List Name */
    CNC_CHAR                    a_description[MAX_NAME_SIZE];   /**< Description of UI List*/
    CNC_UINT8                   fork;                           /**< fork */
    CNC_SINT32                  lifeTime;                       /**< Life time */
    cncDevIconList_t            *p_iconList;                    /**< Icon List of Particular UI List.*/
    cncRUIProtocolDetails_t     *p_protocolDetails;             /**< Protocol Details of particular UI List*/
    struct _cncRUIList_t        *p_next;

}cncRUIList_t;


/**
 * \ struct  _cncDeviceList_t
 * \brief    This Structure will Contains List of all source devices present in the Network.
 * */
typedef struct _cncDeviceList_t
{
    CNC_CHAR                        a_name[MAX_NAME_SIZE];      /**<  Device Friendly Name */
    CNC_CHAR                        a_udn[MAX_NAME_SIZE];       /**<  UUID of RUI Server device */
    CNC_CHAR                        a_baseUrl[MAX_NAME_SIZE];   /**<  Base URL of device*/
    cncDevIconList_t                *p_iconList;                /**<  Points Icon List of cncDevIconList_t type Which Contains all Icon Types of Same RUI Server Device. */
    cncRUIList_t                    *p_uiList;
    struct _cncDeviceList_t         *p_nextDevice;              /**<  Next Device in the List */

} cncDeviceList_t;



/**
  * \brief  The enumeration which Contains Basic Event details of RUI-S device.
  */
typedef enum _cncRuiEventType_t
{
    CNC_RUI_NEW_DEVICE_FOUND        = 101,             /**<101 : */
    CNC_RUI_DEVICE_SHUT_DOWN,                          /**<102 : */
    CNC_RUI_DEVICE_SEARCH_COMPLETED,                   /**<103 : */
    CNC_RUI_DEVICE_CONTENT_MODIFIED,                   /**<104 : */
    CNC_RUI_SUBSCRIPTION_EVENT_RECEIVED,               /**<105 : */

} cncRuiEventType_t;


/**
 * \ struct cncRuiConfigParam_t
 * \brief   This Structure Contains All configurable parameters of RUI Control Point
 */
typedef struct _cncRuiConfigParam_t
{
    CNC_SINT32      networkTimeOut;                             /**< Connect timeout for getting Description XML of RUI-S.*/
    CNC_SINT32      searchTimeOut;                              /**< Max Time out value for SEARCH action.*/
    CNC_SINT32      maxActionRespSize;                          /**< Maximum size to store UPnP Browse Action response.*/
    CNC_SINT32      upnpPort;                                   /**< UPnP Port*/
    CNC_CHAR        a_xmlDocUsrAgent[USER_AGENT_LENGTH];        /**< User agent while sending GET request of RUI-S Description XML.*/
    CNC_CHAR        a_networkIfName[MAX_NAME_SIZE];             /**< Contains Network Interface Name */
}cncRuiConfigParam_t;

/**
 * \ struct cncRuiNotification_t
 * \brief   This Structure will Contains Information about RUI Event Notification.
 */
typedef struct _cncRuiNotification_t
{
    cncRuiEventType_t      eventType;
    CNC_CHAR               *p_deviceId;

}cncRuiNotification_t;


/**
 * \brief Event notification Callback Function which trigger backs to application if any event will occurs.
 */
typedef void (*F_RuiEventNotification_t)(cncRuiNotification_t s_event);


/**
 * \breif RUI Instance
 */
typedef void*  cncRuiCpHandle_t;


/* ======================== Function declarations ========================= */

/**
 * \ fn     CNC_RUI_Init
 * \brief   This Api can be Used for RUI Initialization.
 * \param   None
 * \return  CNC_RUI_ERR_CODE_NONE if success,
 *              On Error cncRuiErrCode_t
 */
cncRuiErrCode_t  CNC_RUI_Init(void);


/**
 * \ fn     CNC_RUI_Create
 * \brief   Creates RUI Control Point Instance (Client) And internally Starts the searching for RUI servers
 *            present in the same network.
 * \param   *pp_ruiHandle [OUT] : RUI Handle
 * \param   *p_ruiConfigParam [IN] : Passing structure which contains RUI configurable parameters.
 * \param   p_f_eventNotifier [IN] : RUI Event Notification Call back function.
 *
 * \return  CNC_RUI_ERR_CODE_NONE if success,
 *              On Error cncRuiErrCode_t
 */
cncRuiErrCode_t  CNC_RUI_Create(cncRuiCpHandle_t           *pp_ruiHandle,
                                cncRuiConfigParam_t        *p_ruiConfigParam,
                                F_RuiEventNotification_t   p_f_eventNotifier);


/**
 * \ fn     CNC_RUI_SearchServer
 * \brief   To search available RUI servers present in Network.
 * \param   p_ruiCpHandle [IN]: RUI Handle
 * \return  CNC_RUI_ERR_CODE_NONE if success,
 *              On Error cncRuiErrCode_t
 */
cncRuiErrCode_t CNC_RUI_SearchServer(cncRuiCpHandle_t  p_ruiCpHandle);


/**
 * \ fn     CNC_RUI_GetServerList
 * \brief   To get RUI Server List present in network
 * \param   p_ruiCpHandle [IN]    : RUI Handle
 * \param   **pp_deviceList [OUT] : RUI Server Devices List
 * \param   *p_numOfDevices [OUT] : Number of RUI Servers present in network.
 * \return  CNC_RUI_ERR_CODE_NONE if success,
 *              On Error cncRuiErrCode_t
 */
cncRuiErrCode_t CNC_RUI_GetServerList(cncRuiCpHandle_t p_ruiCpHandle,
                                      cncDeviceList_t  **pp_deviceList,
                                      CNC_SINT32       *p_numOfDevices);


/**
 * \ fn     CNC_RUI_GetUIList
 * \brief   To get UI List of RUI Server Device.
 * \param   p_ruiCpHandle [IN] : RUI Handle
 * \param   *p_deviceId [IN]   : RUI Server Device ID
 * \param   **pp_uiList [OUT]  : User Interface List of particular RUI Server
 * \return  CNC_RUI_ERR_CODE_NONE if success,
 *              On Error cncRuiErrCode_t
 */
cncRuiErrCode_t CNC_RUI_GetUIList(cncRuiCpHandle_t p_ruiCpHandle,
                                  CNC_CHAR         *p_deviceId,
                                  cncRUIList_t     **pp_uiList);

/**
 * \ fn     CNC_RUI_FreeUIList
 * \brief   To free UI List of RUI Server
 * \param   *p_UIList [IN] : UI List which need to be freed
 * \return  CNC_RUI_ERR_CODE_NONE if success,
 *              On Error cncRuiErrCode_t
 */
cncRuiErrCode_t CNC_RUI_FreeUIList(cncRUIList_t    *p_UIList);


/**
 * \ fn     CNC_RUI_FreeServerList
 * \brief   To free RUI Server List
 * \param   p_deviceList [IN] : RUI Server Devices List which need to be freed.
 * \return  CNC_RUI_ERR_CODE_NONE if success,
 *              On Error cncRuiErrCode_t
 */
cncRuiErrCode_t CNC_RUI_FreeServerList(cncDeviceList_t  *p_deviceList);


/**
 * \ fn     CNC_RUI_Destroy
 * \brief   To Destroy RUI instance or Handle.
 * \param   p_ruiCpHandle [IN] : RUI Handle
 * \return  CNC_RUI_ERR_CODE_NONE if success,
 *              On Error cncRuiErrCode_t
 */
cncRuiErrCode_t  CNC_RUI_Destroy(cncRuiCpHandle_t  p_ruiCpHandle);


/**
 * \fn      CNC_RUI_DeInit
 * \brief   This Api can be Used for RUI De initialization
 * \param   None
 * \return  CNC_RUI_ERR_CODE_NONE if success,
 *              On Error cncRuiErrCode_t
 */
cncRuiErrCode_t CNC_RUI_DeInit(void);


#endif /* __CNC_RUI_H_ */
