/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Cavium, Inc.
 Cavium, Inc. reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium, Inc.

 ALL RIGHTS RESERVED.

 Copyright 2011-2012, Cavium, Inc.
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */

/**
 * @file cnc_sqlite.h
 * The database library for specifically
 * handling different data to save
 *******************************************************************************/
#ifndef __CNC_SQLITE_H__
#define __CNC_SQLITE_H__

#ifdef __cplusplus
extern "C" {
#endif

#include "cnc_system.h"
#include "sqlite3.h"

typedef enum
{
    SQLITE_DATATYPE_NULL = 0,       //NULL
    SQLITE_DATATYPE_INTEGER,       //int
    SQLITE_DATATYPE_TEXT,            //text
    SQLITE_DATATYPE_REAL,            //float
    SQLITE_DATATYPE_BLOB,           //blob
    SQLITE_DATATYPE_UNKNOWN,
}SQLITE_DATATYPE_e;

/*****
*sqlite3_exec callback
*@first para: sqlite3_exec's fourth input para
*@second para :column size
*@third para:  column value
*@fourth para: column name
*@return vavlue is not 0, sqlite3_exec will over
*****/
typedef CNC_SINT32 (*CNC_SQLDB_Fun)(void*, CNC_SINT32, CNC_CHAR**, CNC_CHAR**);

typedef void * SQLDB_HANDLE;

/*****
*init data base
*@param:dbpath may set database path
*@return if successful returns db handle, error returns: null
*****/
SQLDB_HANDLE  CNC_SQLDB_Init(CNC_CHAR * dbpath);

/*****
*destroy sql db;
*@return if successful returns RESULT_OK, error returns: RESULT_FAIL
*****/
Result_Type_e CNC_SQLDB_Destroy(SQLDB_HANDLE handle);

/*****
*create table, name table
*@param: key is INTEGER PRIMARY KEY
*@when add row data the key will automatically increase
*@when key is null ,the key will use id replace
*@return if successful returns RESULT_OK, error returns: RESULT_FAIL
*****/
Result_Type_e CNC_SQLDB_CreateTable(SQLDB_HANDLE handle, CNC_CHAR * table, CNC_CHAR * key);

/*****
*delete table ,delete all datas
*@ table: will be deleted table's name
*@return if successful returns RESULT_OK, error returns: RESULT_FAIL
*****/
Result_Type_e CNC_SQLDB_DeleteTable(SQLDB_HANDLE handle, CNC_CHAR * table);

/*****
*clear table ,delete all datas
*@ table :name
*@return if successful returns RESULT_OK, error returns: RESULT_FAIL
*****/
Result_Type_e CNC_SQLDB_ClearTable(SQLDB_HANDLE handle, CNC_CHAR * table);

/*****
*execute sql output data by callback
*@param: content is callback first para
*@return if successful returns RESULT_OK, error returns: RESULT_FAIL
*****/
Result_Type_e CNC_SQLDB_Execute_TheSQL(SQLDB_HANDLE handle, CNC_CHAR * sql, CNC_SQLDB_Fun callback, void * content);

/*****
*insert column to table
*@insert column column_name to table ,data type is :
*@SQLITE_DATATYPE_e
*@param exg: ("channels", "ch_name", 2)
*@return if successful returns RESULT_OK, error returns: RESULT_FAIL
*****/
Result_Type_e CNC_SQLDB_InsertColumnToTable(SQLDB_HANDLE handle, CNC_CHAR * table, CNC_CHAR * column_name, SQLITE_DATATYPE_e datatype);

/*****
*insert row to table
*@insert row name column_names to table ,data type is :
*@cnc_char
*@param exg: ("channels", "ch_lcn, ch_name", "10, 'cctc5'")
*@return if successful return row keyid
*****/
CNC_SINT32 CNC_SQLDB_InsertNodeToTable(SQLDB_HANDLE handle, CNC_CHAR * table, CNC_CHAR * column_names, CNC_CHAR * values);

/*****
*delete one row data , the row
*where meet filter_conditions
*@para filter_conditions exg:"ch_index='10' and ch_type='1'"
*@return if successful returns RESULT_OK, error returns: RESULT_FAIL
*****/
Result_Type_e CNC_SQLDB_DeleteNodeData(SQLDB_HANDLE handle, CNC_CHAR * table, CNC_CHAR * filter_conditions);

/*****
*check indexname if exists, need para filter conditions
*@param filter_conditions exg:
*@ch_lcn = '10' and name = 'cctv5'
*@return : 0, not find ;1, exists
*****/
CNC_SINT32 CNC_SQLDB_Check_NodeExists(SQLDB_HANDLE handle, CNC_CHAR * table, CNC_CHAR * filter_conditions);

/*****
*set column' value is para value,
*@param exg:
*@("channels", "ch_type", 10, NULL);
*@return if successful returns RESULT_OK, error returns: RESULT_FAIL

*****/
Result_Type_e CNC_SQLDB_SetColumnIntegerValue(SQLDB_HANDLE handle, CNC_CHAR * table, CNC_CHAR *column, CNC_SINT32 value, CNC_CHAR * filter_conditions);

/*****
*set column' value is para value,
*@para exg:
*@("channels", "ch_name", &("CCTV2"), "ch_lcn='10'");
*@return if successful returns RESULT_OK, error returns: RESULT_FAIL
*****/
Result_Type_e CNC_SQLDB_SetColumnStringValue(SQLDB_HANDLE handle, CNC_CHAR * table, CNC_CHAR *column, CNC_CHAR *value, CNC_CHAR * filter_conditions);

/*****
*update table datas
*@para exg:
*@("channels", "ch_lcn='11', ch_name='cctv5'", "ch_index='10' and ch_name='cctv5'");
*@return if successful returns RESULT_OK, error returns: RESULT_FAIL

*****/
Result_Type_e CNC_SQLDB_UpdateTableValue(SQLDB_HANDLE handle, CNC_CHAR * table, CNC_CHAR *setting, CNC_CHAR * filter_conditions);

/*****
*get integer value where meet filter_conditions,
*@para exg:
*@("channels", "ch_lcn", "ch_index='10' and ch_name='cctv5'");
*@result fail return -1;
*****/
CNC_SINT32 CNC_SQLDB_Get_IntegerValue(SQLDB_HANDLE handle, CNC_CHAR * table, CNC_CHAR * column, CNC_CHAR * filter_conditions);

/*****
*get integer value where meet filter_conditions,
*@para exg:
*@(str, "channels", "ch_name", "ch_index='10' and ch_type='1'");
*****/
CNC_SINT32 CNC_SQLDB_Get_StringValue(SQLDB_HANDLE handle, CNC_CHAR * strreturn, CNC_CHAR * table, CNC_CHAR * column, CNC_CHAR * filter_conditions);
/*****
*get one row data , the row
*where meet filter_conditions
*output by callback
*content is callback first para
*@para filter_conditions exg:"ch_index='10' and ch_type='1'"
*@return if successful returns RESULT_OK, error returns: RESULT_FAIL
*****/
Result_Type_e CNC_SQLDB_GetColumnsValue(SQLDB_HANDLE handle, CNC_CHAR * table, CNC_SQLDB_Fun callback, CNC_CHAR * column_name , void * content, CNC_CHAR * filter_conditions);

/**
*@brief        : Function to returns range of row.
*@param[in]    : p_handle:
*                  Pointer to SqlDb Handler.
*@param[in]    : p_table:
*                  Table name
*@param[in/out]: callback:
*                  On success it will give rows in callback.
*@param[in]    : p_content:
*                  User Data.
*@param[in]    : p_filter_conditions:
*                  Search condition
*@param[in]    : start:
*                  Start row.(Row will starts with 1)
*@param[in]    : end:
*                  End row.
*@return       : if successful returns RESULT_OK, \n
*                if error returns: RESULT_FAIL
*/
Result_Type_e CNC_SQLDB_GetRowsValue(SQLDB_HANDLE p_handle, CNC_CHAR *p_table, CNC_SQLDB_Fun callback, void *p_content, CNC_CHAR *p_filter_conditions, CNC_SINT32 start, CNC_SINT32 end);

/*****
*batch insert data ,when Numerous datas need insert db ,will Consume a lot of time
*Below the method can save a lot of time
*use:
*first start ,insert data,  end
*exg:
*@para data :(1,13153,'one hundred');
*@return if successful returns RESULT_OK, error returns: RESULT_FAIL
*****/
Result_Type_e CNC_SQLDB_AddBatchData(SQLDB_HANDLE handle, CNC_CHAR * table, CNC_CHAR * data);

/*****
*batch insert data ,when Numerous datas need insert db ,will Consume a lot of time
*Below the method can save a lot of time
*use:
*first start ,insert data,  end
*exg:
*begin
*INSERT INTO table1 VALUES(1,13153,'one hundred');
*INSERT INTO table1 VALUES(1,13153,'one hundred');
*.....
*commit
*@return if successful returns RESULT_OK, error returns: RESULT_FAIL
*****/
Result_Type_e CNC_SQLDB_BatchInsertData_Start(SQLDB_HANDLE handle);

/*****
batch insert data end
*@return if successful returns RESULT_OK, error returns: RESULT_FAIL
*****/
Result_Type_e CNC_SQLDB_BatchInsertData_End(SQLDB_HANDLE handle);

/*****
*get data order by column sequence,
*output by callback
*@content is callback first para
*@sequence : 0-no order, 1-ASC, 2-DESC
*@return if successful returns RESULT_OK, error returns: RESULT_FAIL
*****/
Result_Type_e CNC_SQLDB_GetAllNodeDatas(SQLDB_HANDLE handle, CNC_CHAR * table, CNC_SQLDB_Fun callback, void * content, CNC_CHAR * column, CNC_SINT32 sequence, CNC_CHAR * filter_conditions);

/*****
*get one row data , the row
*where meet filter_conditions
*output by callback
*content is callback first para
*@para filter_conditions exg:"ch_index='10' and ch_type='1'"
*@return if successful returns RESULT_OK, error returns: RESULT_FAIL
*****/
Result_Type_e CNC_SQLDB_GetNodeData(SQLDB_HANDLE handle, CNC_CHAR * table, CNC_SQLDB_Fun callback, void * content, CNC_CHAR * filter_conditions);

/*****
*get selects data order by column sequence,
*@selects exg: "ch_lcn, ch_name"
*@output by callback
*@content is callback first para
*@sequence : 0-no order, 1-ASC, 2-DESC
*****/
Result_Type_e CNC_SQLDB_GetNodeColumnsDatas(SQLDB_HANDLE handle, CNC_CHAR * table, CNC_CHAR * selects, CNC_SQLDB_Fun callback, void * content, CNC_CHAR * column, CNC_SINT32 sequence, CNC_CHAR * filter_conditions);

/*****
*get table node num and return num
*@param: where meet filter_conditions
*****/
CNC_UINT32 CNC_SQLDB_GetNodeSum(SQLDB_HANDLE handle, CNC_CHAR * table, CNC_CHAR * filter_conditions);

/**
*@brief       : Get the current schema version.
*@param       : handle
*@param[out]  : p_version:
*                On success return the data base schema version.
*@return
*/
Result_Type_e CNC_SQLDB_GetSchemaVer(SQLDB_HANDLE handle, CNC_SINT32 *p_version);

/**
*brief        : Set the current schema version.
*@param       : handle
*@param[in]   : version:
*                On success Set the data base schema version specified.
*@return
*/
Result_Type_e CNC_SQLDB_SetSchemaVer(SQLDB_HANDLE handle, CNC_SINT32 version);


//sqlite3 * CNC_SQLDB_Getsqlitedb(void);

//test
void CNC_SQLDB_Test(void);

#ifdef __cplusplus
}
#endif

#endif /* __CNC_SQLITE_H__ */
