/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Cavium, Inc.
 Cavium, Inc. reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium, Inc.

 ALL RIGHTS RESERVED.

 Copyright 2012, Cavium, Inc.
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */

/**
 * @file cnc_update.h
 * @ingroup update
 * The CNCUPDATE library enables local, IP and HTTP upgrade of system images
 */


typedef enum
{
      CNC_CK_UP_DOWNLOADING=0,
      CNC_CK_UP_STOP_WITH_ERROR,
      CNC_CK_UP_VALIDATING_DATA,
      CNC_CK_UP_STOP_WITH_SUCCESS
}CNC_CK_UP_Status;

/**
 * Provide the download URL and local path to save on
 * @param dl_url    Download URL
 * @param save_path Local Path for saving download
 * @param eeprom_flag 0: With eeprom   1:Without eeprom
 * @return RESULT_OK on success, RESULT_FAIL on error
 */
Result_Type_e CNC_IP_Update_DownloadPackage(const char *dl_url, const char *save_path,int eeprom_flag);

/**
 * Check the downloaded image against the public key
 * @param signed_file Signed Update File
 * @param pub_key     Public RSA Key
 * @return RESULT_OK on success, RESULT_FAIL on error
 */
int CNC_Update_Check_Package(const char *signed_file, const char *pub_key);

/**
 * Check the downloaded image against the public key
 * @param signed_file Signed Update File
 * @param pub_key     Public RSA Key
 * @return RESULT_OK on success, RESULT_FAIL on error
 */
int CNC_Update_Check_Package_Cmd(const char *signed_file, const char *pub_key);

/**
 * Get the current status of the update
 * @return downloading, complete with error, complete with success, validating
 */
CNC_CK_UP_Status CNC_IP_Update_GetStatus(void);

/**
 * Stop the update process
 */
void CNC_IP_Update_StopDL(void);

/**
 * Get the update progress (in percentage complete)
 * @return progess percentage
 */
int CNC_IP_Update_Progress(void);

/**
 * Read the software version on the update
 * @param sw_ver Software Version
 * @param eeprom_flag 0: With eeprom   1:Without eeprom
 */
void CNC_IP_Update_Get_Sw_Ver(char *sw_ver);


/**
 * Set new software version in the system
 * @param sw_ver Software Version
 * @param eeprom_flag 0: With eeprom   1:Without eeprom
 */
void CNC_IP_Update_Set_Sw_Ver(char *sw_ver);



