/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Cavium, Inc.
 Cavium, Inc. reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium, Inc.

 ALL RIGHTS RESERVED.

 Copyright 2011-2012, Cavium, Inc.
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */


/**
 * @file cnc_usbbc.h
 * @ingroup wd
 * The CNC USB Backchannel library provides mechanism to send/receive USB backchannel data from the
 * USB devices like Keyboard, Mouse, Joystick etc.
 * */

#ifndef __CNC_USBBC_INTERFACE_H__
#define __CNC_USBBC_INTERFACE_H__

#ifdef __cplusplus
extern "C" {
#endif

/* ============================ System Includes =========================== */
// NONE

/* ============================= User Includes ============================ */
#include "cnc_common_typedef.h"
#include "cnc_system.h"


/* ======================== Enum declarations ========================= */
typedef enum _CNC_USBBCEvent_t {
    CNC_USBBC_EVENT_START,
    CNC_USBBC_EVENT_STOP,
} CNC_USBBCEvent_t;

typedef enum _CNC_USBBCBoolean_t{
    CNC_USBBC_TRUE,
    CNC_USBBC_FALSE
} CNC_USBBCBoolean_t;

typedef enum _CNC_USBBC_ReturnValue_t{
    CNC_USBBC_FAILURE = -1,
    CNC_USBBC_SUCCESS = 0
} CNC_USBBC_ReturnValue_t;

typedef enum _CNC_USBBC_MsgType_t {
    CNC_USBBC_MSG_TYPE_CONTROL = 0,
    CNC_USBBC_MSG_TYPE_DATA = 1
} CNC_USBBC_MsgType;

typedef enum _CNC_USBBC_CtrlCmd_t {
    CNC_USBBC_CMD_DISCONNECT = 0,
    CNC_USBBC_CMD_CONNECT = 1
} CNC_USBBC_CtrlCmd;

typedef enum _CNC_USBBC_DevType_t {
    CNC_USBBC_DEV_KEYBOARD = 0,
    CNC_USBBC_DEV_MOUSE = 1,
    CNC_USBBC_DEV_JOYSTICK = 2
} CNC_USBBC_DevType;

#define INTERFACE_PROTOCOL_KEYBOARD         (0x1)
#define INTERFACE_PROTOCOL_MOUSE            (0x2)

#define USAGE_TYPE_MOUSE                    (0x2)
#define USAGE_TYPE_JOYSTICK                 (0x4)
#define USAGE_TYPE_KEYBOARD                 (0x6)

#define MAX_HIDG_DEVICES                    (3)
#define CNC_USBBC_HIDG_MAX_REPORT_DESC_SIZE         (1024)      /* TODO: Verify this size would be sufficeint for all type of HID devices */

typedef struct _CNC_USBBC_HidgFuncDescriptor_t {
    unsigned char       subclass;
    unsigned char       protocol;
    unsigned char       feature_report[10][257];
    unsigned short      feature_report_length[10];
    unsigned short      report_length;
    unsigned short      report_desc_length;
    unsigned char       report_desc[CNC_USBBC_HIDG_MAX_REPORT_DESC_SIZE];
} CNC_USBBC_HidgFuncDescriptor;

typedef struct _CNC_USBBC_HidgDevInfo_t {
    int devType;
    CNC_USBBC_HidgFuncDescriptor hidgFuncDescriptor;
} CNC_USBBC_HidgDevInfo;

typedef struct _CNC_USBBC_ControlMsgType1_t {
    CNC_USBBC_MsgType       msgType;
    CNC_USBBC_CtrlCmd       ctrlCmd;      
    unsigned char           hidgDevInfoCount;
    CNC_USBBC_HidgDevInfo   hidgDevInfo;
} CNC_USBBC_ControlMsgType1;

/* ======================== Structs declarations ========================= */


typedef CNC_VOID* CNC_USBBCSK_Handle_t;
typedef CNC_VOID* CNC_USBBCSR_Handle_t;

typedef CNC_VOID (*cncUSBBCCallback_t) (CNC_USBBCSK_Handle_t, CNC_USBBCEvent_t);

/*
 * This has all the configuration parameters, that is used for initialization.
 */ 

/* ======================== Function declarations ========================= */
CNC_USBBCSK_Handle_t CNC_USBBCSink_Create(CNC_VOID);
CNC_USBBCSR_Handle_t CNC_USBBCSource_Create(CNC_VOID);
CNC_USBBC_ReturnValue_t CNC_USBBCSink_Destroy(CNC_USBBCSK_Handle_t handle);
CNC_USBBC_ReturnValue_t CNC_USBBCSource_Destroy(CNC_USBBCSR_Handle_t handle);
CNC_USBBC_ReturnValue_t CNC_USBBCSink_Start(CNC_USBBCSK_Handle_t handle, char* ip_address, int port);
CNC_USBBC_ReturnValue_t CNC_USBBCSource_Start(CNC_USBBCSR_Handle_t handle, int port);
CNC_USBBC_ReturnValue_t CNC_USBBCSink_Stop(CNC_USBBCSK_Handle_t handle);
CNC_USBBC_ReturnValue_t CNC_USBBCSource_Stop(CNC_USBBCSR_Handle_t handle);

#ifdef __cplusplus
}

#endif

#endif // __CNC_USBBC_INTERFACE_H__
