/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Cavium, Inc.
 Cavium, Inc. reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium, Inc.

 ALL RIGHTS RESERVED.

 Copyright 2011-2012, Cavium, Inc.
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */

/**
 * @file cnc_vod.h
 * The CNCVOD library is used to develop interfaces to external VOD servers
 */
#ifndef __CNC_VOD_CONTROLER_H__
#define __CNC_VOD_CONTROLER_H__

#ifdef __cplusplus
extern "C" {
#endif

#include "cnc_app/cnc_osd.h"
//#include "cnc_xml.h"
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>
#include <libxml/tree.h>


typedef enum
{
    CNC_VOD_UNKNOWN,
    CNC_VOD_START_DL_PIC,
    CNC_VOD_STOP_DL_PIC,
    CNC_VOD_PLAY_RTSP,
    CNC_VOD_STOP_RTSP,
    CNC_VOD_PLAY_HTTP,
    CNC_VOD_STOP_HTTP,
}CNC_VOD_CMD;


typedef struct CNC_VOD_Msg_
{
    CNC_VOD_CMD cmd;
    char *url;
    char *dir;
}CNC_VOD_Msg_t;

typedef enum
{
    DL_STATE_DOWNLOADING,
    DL_STATE_STOP,
}DL_State_e;

int CNC_VOD_Init(void);
int CNC_VOD_Close(void);
int CNC_VOD_SentMsg(CNC_VOD_Msg_t *pMsg);
cncOSDBitmap_t * CNC_VOD_Get_PicHandle(void);

typedef enum
{
    VOD_DIR = 0,
    VOD_FILE,
}CNC_VOD_NodeType;
/*
    <name>patagonia</name>
    <type>0</type>
    <description/>
    <director/>
    <taskid>1001</taskid>
    <domain>192.168.18.217</domain>
    <format>ts</format>
    <urllink>rtsp://192.168.18.217/patagonia.mp4</urllink>
    <img>http://221.194.44.188:8080/WCM_AUTO/DOC_IMG/www1/20100509/1273371052612.jpg</img>
*/

typedef struct CNC_VOD_PlayLink_t_{
    char *url;
    char *channel;
    char *mode;
    char *ratio;
    char *limit;
    struct CNC_VOD_PlayLink_t_ *next;
}CNC_VOD_PlayLink_t;


typedef struct CNC_Vod_Node_t_
{
    struct CNC_Vod_Node_t_    *next;
    struct CNC_Vod_Node_t_    *prev;
    struct CNC_Vod_Node_t_    *parents;
    struct CNC_Vod_Node_t_    *children;

    int index;
    char *name;
    void *bmp;  // picture handle
    char imgstatus; // 1 - completed; 0 - uncompleted; -1 - error
    char *image_url;

    CNC_VOD_NodeType state; //Judge whether is file
    //file attribute start
    int type;
    int stars;
    char *description;
    char *director;
    char *style;
    int   taskid;
    char *domain;
    char *format;
    char *casting;
    char *year;
    char *duration;
    char *rate;
    char *url;
    char *available;
    CNC_VOD_PlayLink_t *play_link;
    //file attribute end

}CNC_Vod_Node_t;


CNC_Vod_Node_t* CNC_VOD_GetInfo_FromXML(char *from_xml);
int CNC_VOD_FreeInfo(CNC_Vod_Node_t* pVodNode);
void CNC_VOD_SetVodServer(CNC_CHAR * server);

typedef void (*CNC_VOD_DownLoad_cb)(void * pram);

/*************************************************************************
 * Block file download function
 * Input :
 *      url: download url
 *      filename: save file name with directory
 *      timeout_ms: download time out (ms)
 * Output: Null
 * Return: RESULT_OK; RESULT_FAIL
 * Note: Block function, can not been multi-call
*************************************************************************/
int CNC_VOD_DownLoad(const char *url, const char * filename, const int timeout_ms);

/*************************************************************************
 * Unblock file download function
 * Input :
 *      url: download url
 *      filename: save file name with directory
 *      function: download callback
 * Output: Null
 * Return: Null
 * Note: unblock function, can not been multi-call
*************************************************************************/
void CNC_VOD_DownLoadEx_Start(const char *url, const char * filename, CNC_VOD_DownLoad_cb function);
void CNC_VOD_DownLoadEx_Stop(void);
int CNC_VOD_DownLoadEx_Get_Progress(void);


/*
 * Moving from cnc_xml.h; specific to vod exchange data
 */

typedef enum _cncXMLSettingType_t
{
   E_SETTING_USER_INFORMATION,
   E_SETTING_AUTHENTICATE,
   E_SETTING_NETWORK_ETHERNET,          //struct has to pass along with this
   E_SETTING_NETWORK_WIRELESS,          //struct has to pass along with this
   E_SETTING_SERIAL_NUMBER,
   E_SETTING_FIRMWARE_VERSION,
   E_SETTING_STATE,
   E_SETTING_AUDIO_VIDEO_CONFIGURATION, //struct has to pass along with this
   E_SETTING_AUDIO_IMAGE_CONFIGURATION, //struct has to pass along with this
   E_SETTING_MORALITYLEVEL,
   E_SETTING_PARENTALGUIDANCE,
   E_SETTING_PARENTALCONTROL,
   // TODO for other this is only initial version

} cncXMLSettingType_t;

typedef enum xmlBoolTag
{
    XML_BOOL_FALSE   = 0,
    XML_BOOL_TRUE

}xmlBool_t;


typedef enum E_UpdateSettingErrCodeTag
{
    E_UPDATE_SETTING_ERROR_NONE            = 0,
    E_UPDATE_SETTING_INIT_ERROR            = -100,
    E_UPDATE_SETTING_RE_INIT_ERROR,
    E_UPDATE_SETTING_EMPTY_DOC,
    E_UPDATE_SETTING_INVALID_PARME,
    E_UPDATE_SETTING_EMPTY_SETTING,
    E_UPDATE_SETTING_UNKNOWN_DOC,
    E_UPDATE_SETTING_BUSY,
    E_UPDATE_SETTING_INVALID_ACTION,

    //TODO
    // error code

} E_UpdateSettingErrCode_t;

typedef struct S_userDataTag
{
    cncXMLSettingType_t   e_settingName;
    char             *p_ch_val;
    void             *p_v_settingCore;  // this will typecast based on Enum type provided
}S_userData_t;

typedef struct S_networkDataTag
{
    char    *p_ch_address;
    char    *p_ch_val_macAddress;
    char    *p_ch_DNS;
    char    *p_ch_interface;

}S_networkData_t;

typedef struct S_supportedFormatTag
{
    // TODO Define All the format type DTD

}S_supporedFormat_t;

typedef struct S_audioVideoConfTag
{
    char                 *p_ch_resolution;
    char                 *p_ch_aspectRatio;
    xmlBool_t            b_hdmi;
    xmlBool_t            b_scart;
    S_supporedFormat_t   *p_s_supporedFormat;

    // TODO Define the DTD for all object

}S_audioVideoConf_t;

typedef struct S_userInformationTag
{
    char                *p_ch_accountId;
    char                *p_ch_deviceId;
}S_userInformation_t;

typedef struct S_authenticateTag
{
    char                *p_ch_securityToken;

}S_authenticate_t;

typedef struct S_imageConfTag
{
    //TODo Define the DTD for all object

}S_imageConf_t;

//function prototype
/**
 * Function : setting_write
 * Detail   : write the setting provided by user  into file xml/json
 */

E_UpdateSettingErrCode_t update_coreSetting(cncXMLSettingType_t   e_setting_name, void *p_v_settingCore, xmlDocPtr  p_s_doc);

#ifdef __cplusplus
}
#endif

#endif

