/*
 #	\$(TEST_FRAME_FOLDER)
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Cavium, Inc.
 Cavium, Inc. reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium, Inc.

 ALL RIGHTS RESERVED.

 Copyright 2011-2012, Cavium, Inc.
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */

/**
 * \file cnc_wfd.h
 * \ingroup wd
 * The CNCWFD library enables the development of applications using the WFD RTSP protocol
 */

#ifndef __CNC_WFD_RTSP_INTERFACE_H__
#define __CNC_WFD_RTSP_INTERFACE_H__

#ifdef __cplusplus
extern "C" {
#endif

#include "cnc_common_typedef.h"

    typedef enum _cncWFD_control_playback_t {
        CP_FLUSH = 1,
        CP_PLAY,
        CP_PAUSE,
        CP_INDICATOR,
        CP_FLUSH_PLAY,
        CP_FLUSH_PAUSE,
        CP_INCREASE_VOLUME,
        CP_DECREASE_VOLUME,
        CP_SET_VOLUME
    }
    cncWFD_control_playback_t;

    typedef enum _cncWFD_videoType_t {
        CNC_WFD_CEA_VIDEO,
        CNC_WFD_VESA_VIDEO,
        CNC_WFD_HH_VIDEO
    } cncWFD_videoType_t;

    typedef enum _cncWFD_Enabler_t {
        CNC_WFD_RTSP_DISABLE,
        CNC_WFD_RTSP_ENABLE
    } cncWFD_Enabler_t;

    typedef enum _cncWFD_ReturnValue_t {
        CNC_WFD_RTSP_FAILURE = -1,
        CNC_WFD_RTSP_SUCCESS = 0
    } cncWFD_ReturnValue_t;

    typedef CNC_VOID* cncWFD_RtspHnd_t;

    typedef enum _cncWFD_DecoderReady_t {
        CNC_WFD_RTSP_MEDIA_PAUSED,
        CNC_WFD_RTSP_MEDIA_ERROR
    } cncWFD_DecoderReady_t;
//------------------------------------------------------------------------------

typedef struct _cncWFD_event_string_t{  ///< struct to notify internal errors to client application
    CNC_CHAR name[20];                      ///< name given to the event that is triggered
    CNC_CHAR error[60];                     ///< error string describing the error that triggered the event
} cncWFD_event_string_t;

    typedef enum _cncWFD_Event_t {
        CNC_WFD_RTSP_EVENT_CONNECTED,                               ///< This event notifies when the TCP connection has been established
        CNC_WFD_RTSP_EVENT_PLAY,                                    ///< This event notifies when a PLAY RTSP command has been sent to peer
        CNC_WFD_RTSP_EVENT_RESUME,                                  ///< This event notifies when a PLAY RTSP command has been sent after being on PAUSE or STANDBY
        CNC_WFD_RTSP_EVENT_STOP,                                    ///< This event notifies when the decoder can be stopped
        CNC_WFD_RTSP_EVENT_PAUSE,                                   ///< This event notifies when a PAUSE command has been sent to peer
        CNC_WFD_RTSP_EVENT_SETUP,
        CNC_WFD_RTSP_EVENT_CAPABILITIES_NEGOTIATION,                ///< This event notifies when a capabilities renegotiation is in progress
        CNC_WFD_RTSP_EVENT_STANDBY,                                 ///< This event notifies when a STANDBY command has been sent to peer
        CNC_WFD_RTSP_EVENT_TEARDOWN,                                ///< This event notifies when a TEARDOWN RTSP command has been requested
        CNC_WFD_RTSP_EVENT_APPLY_CHANGES,                           ///< This event notifies when a change in AV parameters 
        CNC_WFD_RTSP_EVENT_CONNECTOR_TYPE_SUPPORTED_PLUGGED,        ///< This event notifies when a connector has been succesfully plugged
        CNC_WFD_RTSP_EVENT_CONNECTOR_TYPE_SUPPORTED_UNPLUGGED,      ///< This event notifies when a connector has been succesfully unplugged
        CNC_WFD_RTSP_EVENT_DECODER_LATENCY,                         ///< This event sets the decoder latency value
        CNC_WFD_RTSP_EVENT_ERROR_SKIP_MODE,                         ///< This event sets the error conceiling method
        CNC_WFD_RTSP_EVENT_REORDERING_BUFFER_SIZE,                  ///< This event sets the reordering buffer size
        CNC_WFD_RTSP_EVENT_RETRANSMISSION_BUFFER_SIZE,              ///< This event sets the UDP retransmission buffer size
        CNC_WFD_RTSP_EVENT_SET_INITIAL_BUFFER,                      ///< This event notifies the initial jitter buffer size
        CNC_WFD_RTSP_EVENT_ZOOM_X,                                  ///< This event sends the received value for zoom_x
        CNC_WFD_RTSP_EVENT_ZOOM_Y,                                  ///< This event sends the received value for zoom_y
        CNC_WFD_RTSP_EVENT_ROTATE,                                  ///< This event sends the received value for rotation angle
        CNC_WFD_RTSP_EVENT_CURRENT_AUDIO_BUFFER,                    ///< This event informs about the current audio buffer size
        CNC_WFD_RTSP_EVENT_LAST_AUDIO_PTS,                          ///< This event informs about the last PTS received from encoder
        CNC_WFD_RTSP_EVENT_CHANGE_FRAMERATE,
        CNC_WFD_RTSP_EVENT_CONTROLPLAYBACK,                         ///< SAMSUNG specific. Notifies when a video playback is being configured and initialized
        CNC_WFD_RTSP_EVENT_PAUSE_REQ,
        CNC_WFD_RTSP_EVENT_PAUSE_REQ_SUCCESS,
        CNC_WFD_RTSP_EVENT_PAUSE_REQ_FAILED,
        CNC_WFD_RTSP_EVENT_UIBC_START,                              ///< This event notifies when a UIBC session is being initialized
        CNC_WFD_RTSP_EVENT_UIBC_STOP,                               ///< This event notifies when a UIBC session is being stopped
        CNC_WFD_RTSP_EVENT_SEND_TIMEOUT,
        CNC_WFD_RTSP_EVENT_CLOSED_SOCKET,                           ///< This event notifies that the TCP socket is closed, so no messages can be sent
        CNC_WFD_RTSP_EVENT_CHANGE_TRANSPORT,                        ///< SAMSUNG specific. Notifies when transport is switching from TCP to UDP or viceversa
        CNC_WFD_RTSP_EVENT_CHANGE_JITTER_BUFFER,                    ///< SAMSUNG specific. Notifies that the jitter buffer size has been modified
        CNC_WFD_RTSP_EVENT_TERMINATED,                              ///< This event notifies that Miracast RTSP client has been terminated is not running anymore
        CNC_WFD_RTSP_EVENT_ERROR,
        CNC_WFD_RTSP_EVENT_INTEL_LOW_DELAY_MODE,                    ///< INTEL specific. Notifies when low delay mode must be switched
        CNC_WFD_RTSP_EVENT_INTEL_OVERSCAN,                          ///< INTEL specific. Notifies when a new video overscan value has been received and its new value
        CNC_WFD_RTSP_EVENT_RENAME_DONGLE,                           ///< INTEL And Samsung specific. Notifies when an INTEL friendly name has been received, so receiver must change its identifier
        CNC_WFD_RTSP_EVENT_INTEL_USBOIP,                            ///< INTEL specific. Not supported ywt
        CNC_WFD_RTSP_EVENT_INTEL_PIPELINE,                          ///< INTEL specific. Notifies when a pipeline INTEL message has been received
        CNC_WFD_RTSP_EVENT_SAMSUNG_DETECTED,                        ///< This event notifies when a SAMSUNG device has been detected as a source
		CNC_WFD_RTSP_EVENT_SET_HDCP_MODE,                           ///< This event notifies when the HDCP mode has been switched
        CNC_WFD_RTSP_EVENT_SOURCE_EVENT,                            ///< This event implies that an action (teardown, play, pause, etc) notification must be sent to the source
        CNC_WFD_RTSP_EVENT_ALL_EVENTS,                              ///< This event lets register all available events with the same function
        CNC_WFD_RTSP_ERROR_BADCOMMAND,
        CNC_WFD_RTSP_ERROR_ERRORCODE,                               ///< This event is triggered when an RTSP response code is different than 200 OK
        CNC_WFD_RTSP_ERROR_CONNECTION,                              ///< This event triggers when there's been a connection (TCP level) error. 
        CNC_WFD_RTSP_ERROR_PARSEREQUEST,                            ///< This event triggers when a server request message is not correctly constructed
        CNC_WFD_RTSP_ERROR_PARSEREQOPTIONS,                         ///< This event is triggered when an error happens parsing an OPTIONS message
        CNC_WFD_RTSP_ERROR_PARSEREQSETUP,                           ///< This event is triggered when an error happens parsing a SETUP message
        CNC_WFD_RTSP_ERROR_PARSEREQPLAY,                            ///< This event is triggered when an error happens parsing a PLAY message
        CNC_WFD_RTSP_ERROR_PARSEREQPAUSE,                           ///< This event is triggered when an error happens parsing a PAUSE message
        CNC_WFD_RTSP_ERROR_PARSEREQTEARDOWN,                        ///< This event is triggered when an error happens parsing a TEARDOWN message
        CNC_WFD_RTSP_ERROR_PARSEREQGETPARAM,                        ///< This event is triggered when an error happens parsing a GET_PARAM message
        CNC_WFD_RTSP_ERROR_PARSEREQSETPARAM,                        ///< This event is triggered when an error happens parsing a SET_PARAM message
        CNC_WFD_RTSP_ERROR_PARSERESPONSE,                           ///< This event triggers when a server response message is not correctly constructed
        CNC_WFD_RTSP_ERROR_PARSERESPOPTIONS,                        ///< This event is triggered when an error happens parsing an OPTIONS response message
        CNC_WFD_RTSP_ERROR_PARSERESPSETUP,                          ///< This event is triggered when an error happens parsing a SETUP response message
        CNC_WFD_RTSP_ERROR_PARSERESPPLAY,                           ///< This event is triggered when an error happens parsing a OPTIONS response message
        CNC_WFD_RTSP_ERROR_PARSERESPPAUSE,                          ///< This event is triggered when an error happens parsing a PAUSE response message
        CNC_WFD_RTSP_ERROR_PARSERESPTEARDOWN,                       ///< This event is triggered when an error happens parsing a TEARDOWN response message
        CNC_WFD_RTSP_ERROR_PARSERESPKEEPALIVE,                      ///< This event is triggered when an error happens parsing a KEEPALIVE response message
        CNC_WFD_RTSP_ERROR_PARSERESPGETPARAM,                       ///< This event is triggered when an error happens parsing a GET_PARAM response message
        CNC_WFD_RTSP_ERROR_PARSERESPSETPARAM,                       ///< This event is triggered when an error happens parsing a SET_PARAM response message
        CNC_WFD_RTSP_EVENT_CURSOR_UPDATE,                           ///< This event is triggered when there is a change in the cursor position or shape
        CNC_WFD_RTSP_ERROR_UNKNOWN
    } cncWFD_Event_t;

    typedef enum _cncWFD_Device_t
    {
        CNC_WFD_RTSP_SOURCE,                                        ///< The device is an AV source
        CNC_WFD_RTSP_PRIMARY_SINK                                   ///< The device is an AV sink
    } cncWFD_Device_t;

    typedef enum _cncWFD_AudioFormat_t
    {
        CNC_WFD_RTSP_AUDIO_FORMAT_AAC,                              ///< Audio format AAC
        CNC_WFD_RTSP_AUDIO_FORMAT_AC3,                              ///< Audio format AC3
        CNC_WFD_RTSP_AUDIO_FORMAT_EAC,                              ///< Audio format EAC
        CNC_WFD_RTSP_AUDIO_FORMAT_LPCM,                             ///< Audio format LPCM
    } cncWFD_AudioFormat_t;

    typedef enum _cncWFD_LpcmModes_t
    {
        CNC_WFD_RTSP_LPCM_MODES_44_1KHZ_16BITS_2CH        =   (1 << 0   )                 ,  ///< LPCM mode code 
        CNC_WFD_RTSP_LPCM_MODES_48KHZ_16BITS_2CH          =   (1 << 1   )                 ,  ///< LPCM mode code 
        CNC_WFD_RTSP_LPCM_MODES_48KHZ_16BITS_4CH          =   (1 << 2   )                 ,  ///< LPCM mode code 
        CNC_WFD_RTSP_LPCM_MODES_48KHZ_16BITS_6CH          =   (1 << 3   )                 ,  ///< LPCM mode code 
        CNC_WFD_RTSP_LPCM_MODES_48KHZ_16BITS_8CH          =   (1 << 4   )                 ,  ///< LPCM mode code 
        CNC_WFD_RTSP_LPCM_MODES_48KHZ_20BITS_2CH          =   (1 << 5   )                 ,  ///< LPCM mode code 
        CNC_WFD_RTSP_LPCM_MODES_48KHZ_24BITS_2CH          =   (1 << 5   ) | ( 1<< 31)     ,  ///< LPCM mode code 
        CNC_WFD_RTSP_LPCM_MODES_48KHZ_20BITS_4CH          =   (1 << 6   )                 ,  ///< LPCM mode code 
        CNC_WFD_RTSP_LPCM_MODES_48KHZ_24BITS_4CH          =   (1 << 6   ) | ( 1<< 31)     ,  ///< LPCM mode code 
        CNC_WFD_RTSP_LPCM_MODES_48KHZ_20BITS_6CH          =   (1 << 7   )                 ,  ///< LPCM mode code 
        CNC_WFD_RTSP_LPCM_MODES_48KHZ_24BITS_6CH          =   (1 << 7   ) | ( 1<< 31)     ,  ///< LPCM mode code 
        CNC_WFD_RTSP_LPCM_MODES_48KHZ_20BITS_8CH          =   (1 << 8   )                 ,  ///< LPCM mode code 
        CNC_WFD_RTSP_LPCM_MODES_48KHZ_24BITS_8CH          =   (1 << 8   ) | ( 1<< 31)     ,  ///< LPCM mode code 
        CNC_WFD_RTSP_LPCM_MODES_96KHZ_16BITS_2CH          =   (1 << 9   )                 ,  ///< LPCM mode code 
        CNC_WFD_RTSP_LPCM_MODES_96KHZ_16BITS_4CH          =   (1 << 10  )                 ,  ///< LPCM mode code 
        CNC_WFD_RTSP_LPCM_MODES_96KHZ_16BITS_6CH          =   (1 << 11  )                 ,  ///< LPCM mode code 
        CNC_WFD_RTSP_LPCM_MODES_96KHZ_16BITS_8CH          =   (1 << 12  )                 ,  ///< LPCM mode code 
        CNC_WFD_RTSP_LPCM_MODES_96KHZ_20BITS_2CH          =   (1 << 13  )                 ,  ///< LPCM mode code 
        CNC_WFD_RTSP_LPCM_MODES_96KHZ_24BITS_2CH          =   (1 << 13  ) | ( 1<< 31)     ,  ///< LPCM mode code 
        CNC_WFD_RTSP_LPCM_MODES_96KHZ_20BITS_4CH          =   (1 << 14  )                 ,  ///< LPCM mode code 
        CNC_WFD_RTSP_LPCM_MODES_96KHZ_24BITS_4CH          =   (1 << 14  ) | ( 1<< 31)     ,  ///< LPCM mode code 
        CNC_WFD_RTSP_LPCM_MODES_96KHZ_20BITS_6CH          =   (1 << 15  )                 ,  ///< LPCM mode code 
        CNC_WFD_RTSP_LPCM_MODES_96KHZ_24BITS_6CH          =   (1 << 15  ) | ( 1<< 31)     ,  ///< LPCM mode code 
        CNC_WFD_RTSP_LPCM_MODES_96KHZ_20BITS_8CH          =   (1 << 16  )                 ,  ///< LPCM mode code 
        CNC_WFD_RTSP_LPCM_MODES_96KHZ_24BITS_8CH          =   (1 << 16  ) | ( 1<< 31)     ,  ///< LPCM mode code 
        CNC_WFD_RTSP_LPCM_MODES_192KHZ_16BITS_2CH         =   (1 << 17  )                 ,  ///< LPCM mode code 
        CNC_WFD_RTSP_LPCM_MODES_192KHZ_16BITS_4CH         =   (1 << 18  )                 ,  ///< LPCM mode code 
        CNC_WFD_RTSP_LPCM_MODES_192KHZ_16BITS_6CH         =   (1 << 19  )                 ,  ///< LPCM mode code 
        CNC_WFD_RTSP_LPCM_MODES_192KHZ_20BITS_2CH         =   (1 << 20  )                 ,  ///< LPCM mode code 
        CNC_WFD_RTSP_LPCM_MODES_192KHZ_24BITS_2CH         =   (1 << 20  ) | ( 1<< 31)     ,  ///< LPCM mode code 
        CNC_WFD_RTSP_LPCM_MODES_192KHZ_20BITS_4CH         =   (1 << 21  )                 ,  ///< LPCM mode code 
        CNC_WFD_RTSP_LPCM_MODES_192KHZ_24BITS_4CH         =   (1 << 21  ) | ( 1<< 31)     ,  ///< LPCM mode code 
        CNC_WFD_RTSP_LPCM_MODES_192KHZ_20BITS_6CH         =   (1 << 22  )                 ,  ///< LPCM mode code 
        CNC_WFD_RTSP_LPCM_MODES_192KHZ_24BITS_6CH         =   (1 << 22  ) | ( 1<< 31)     ,  ///< LPCM mode code 
    } cncWFD_LpcmModes_t;

    typedef enum _cncWFD_AAC_t
    {
        CNC_WFD_RTSP_AAC_48KHZ_16BIT_2CH                 =   (1 << 0)    ,  ///< AAC mode code 
        CNC_WFD_RTSP_AAC_48KHZ_16BIT_4CH                 =   (1 << 1)    ,  ///< AAC mode code
        CNC_WFD_RTSP_AAC_48KHZ_16BIT_6CH                 =   (1 << 2)    ,  ///< AAC mode code
        CNC_WFD_RTSP_AAC_48KHZ_16BIT_8CH                 =   (1 << 3)    ,  ///< AAC mode code
    } cncWFD_AAC_t;

    typedef enum _cncWFD_Ac3_t
    {
        CNC_WFD_RTSP_EAC3_48KHZ_16BITS_2CH               =   (1 << 0)    ,  ///< EAC3 mode code
        CNC_WFD_RTSP_EAC3_48KHZ_16BITS_4CH               =   (1 << 1)    ,  ///< EAC3 mode code
        CNC_WFD_RTSP_EAC3_48KHZ_16BITS_6CH               =   (1 << 2)    ,  ///< EAC3 mode code
        CNC_WFD_RTSP_EAC3_PLUS_48KHZ_16BITS_2CH          =   (1 << 3)    ,  ///< EAC3 mode code
        CNC_WFD_RTSP_EAC3_PLUS_48KHZ_16BITS_4CH          =   (1 << 4)    ,  ///< EAC3 mode code
        CNC_WFD_RTSP_EAC3_PLUS_48KHZ_16BITS_6CH          =   (1 << 5)    ,  ///< EAC3 mode code
        CNC_WFD_RTSP_EAC3_PLUS_48KHZ_16BITS_8CH          =   (1 << 6)    ,  ///< EAC3 mode code
    } cncWFD_Ac3_t;
//------------------------------------------------------------------------------

    typedef enum _cncWFD_VideoFormatNative_t {
        CNC_WFD_RTSP_VIDEO_FORMAT_CEA = 0,          ///< Native video format
        CNC_WFD_RTSP_VIDEO_FORMAT_VESA,             ///< Native video format
        CNC_WFD_RTSP_VIDEO_FORMAT_HH,               ///< Native video format
    } cncWFD_VideoFormatNative_t;
//------------------------------------------------------------------------------

    typedef enum _cncCNC_WFD_RTSP_H264Profile_t
    {
        CNC_WFD_RTSP_CONSTRAINED_BASELINE_PROFILE = (1 << 0) ,  ///< CBP (H264 profile) code number
        CNC_WFD_RTSP_CONSTRAINED_HIGH_PROFILE     = (1 << 1),   ///< CHP (H264 profile) code number 
    } cncCNC_WFD_RTSP_H264Profile_t;

    typedef enum _cncCNC_WFD_RTSP_H264Level_t
    {
        CNC_WFD_RTSP_H264LEVEL_3_1          = (1 << 0) ,     ///< H264 Level code
        CNC_WFD_RTSP_H264LEVEL_3_2          = (1 << 1) ,     ///< H264 Level code
        CNC_WFD_RTSP_H264LEVEL_4            = (1 << 2) ,     ///< H264 Level code
        CNC_WFD_RTSP_H264LEVEL_4_1          = (1 << 3) ,     ///< H264 Level code
        CNC_WFD_RTSP_H264LEVEL_4_2          = (1 << 4) ,     ///< H264 Level code
    } cncCNC_WFD_RTSP_H264Level_t;

    typedef enum _cncWFD_CeaResolutionRefreshRateIndex_t
    {
        CNC_WFD_RTSP_CEA_RESOLUTION_REFRESH_RATE_640x480p60_INDEX   =  0    ,   ///<  
        CNC_WFD_RTSP_CEA_RESOLUTION_REFRESH_RATE_720x480p60_INDEX   =  1    ,   /* */
        CNC_WFD_RTSP_CEA_RESOLUTION_REFRESH_RATE_720x480i60_INDEX   =  2    ,   /* */
        CNC_WFD_RTSP_CEA_RESOLUTION_REFRESH_RATE_720x576p50_INDEX   =  3    ,   /* */
        CNC_WFD_RTSP_CEA_RESOLUTION_REFRESH_RATE_720x576i50_INDEX   =  4    ,   /* */
        CNC_WFD_RTSP_CEA_RESOLUTION_REFRESH_RATE_1280x720p30_INDEX  =  5    ,   /* */
        CNC_WFD_RTSP_CEA_RESOLUTION_REFRESH_RATE_1280x720p60_INDEX  =  6    ,   /* */
        CNC_WFD_RTSP_CEA_RESOLUTION_REFRESH_RATE_1920x1080p30_INDEX =  7    ,   /* */
        CNC_WFD_RTSP_CEA_RESOLUTION_REFRESH_RATE_1920x1080p60_INDEX =  8    ,   /* */
        CNC_WFD_RTSP_CEA_RESOLUTION_REFRESH_RATE_1920x1080i60_INDEX =  9    ,   /* */
        CNC_WFD_RTSP_CEA_RESOLUTION_REFRESH_RATE_1280x720p25_INDEX  =  10   ,   /* */
        CNC_WFD_RTSP_CEA_RESOLUTION_REFRESH_RATE_1280x720p50_INDEX  =  11   ,   /* */
        CNC_WFD_RTSP_CEA_RESOLUTION_REFRESH_RATE_1920x1080p25_INDEX =  12   ,   /* */
        CNC_WFD_RTSP_CEA_RESOLUTION_REFRESH_RATE_1920x1080p50_INDEX =  13   ,   /* */
        CNC_WFD_RTSP_CEA_RESOLUTION_REFRESH_RATE_1920x1080i50_INDEX =  14   ,   /* */
        CNC_WFD_RTSP_CEA_RESOLUTION_REFRESH_RATE_1280x720p24_INDEX  =  15   ,   /* */
        CNC_WFD_RTSP_CEA_RESOLUTION_REFRESH_RATE_1920x1080p24_INDEX =  16   ,   /* */
    } cncWFD_CeaResolutionRefreshRateIndex_t;

    typedef enum _cncWFD_VesaResolutionRefreshRateIndex_t
    {
        CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_800x600p30_INDEX   =  0      ,   /* */
        CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_800x600p60_INDEX   =  1      ,   /* */
        CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1024x768p30_INDEX  =  2      ,   /* */
        CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1024x768p60_INDEX  =  3      ,   /* */
        CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1152x864p30_INDEX  =  4      ,   /* */
        CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1152x864p60_INDEX  =  5      ,   /* */
        CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1280x768p30_INDEX  =  6      ,   /* */
        CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1280x768p60_INDEX  =  7      ,   /* */
        CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1280x800p30_INDEX  =  8      ,   /* */
        CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1280x800p60_INDEX  =  9      ,   /* */
        CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1360x768p30_INDEX  =  10     ,   /* */
        CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1360x768p60_INDEX  =  11     ,   /* */
        CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1366x768p30_INDEX  =  12     ,   /* */
        CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1366x768p60_INDEX  =  13     ,   /* */
        CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1280x1024p30_INDEX =  14     ,   /* */
        CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1280x1024p60_INDEX =  15     ,   /* */
        CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1400x1050p30_INDEX =  16     ,   /* */
        CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1400x1050p60_INDEX =  17     ,   /* */
        CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1440x900p30_INDEX  =  18     ,   /* */
        CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1440x900p60_INDEX  =  19     ,   /* */
        CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1600x900p30_INDEX  =  20     ,   /* */
        CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1600x900p60_INDEX  =  21     ,   /* */
        CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1600x1200p30_INDEX =  22     ,   /* */
        CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1600x1200p60_INDEX =  23     ,   /* */
        CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1680x1024p30_INDEX =  24     ,   /* */
        CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1680x1024p60_INDEX =  25     ,   /* */
        CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1680x1050p30_INDEX =  26     ,   /* */
        CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1680x1050p60_INDEX =  27     ,   /* */
        CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1920x1200p30_INDEX =  28     ,   /* */
        CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1920x1200p60_INDEX =  29     ,   /* */
    } cncWFD_VesaResolutionRefreshRateIndex_t;

    typedef enum _cncWFD_HhResolutionRefreshRateIndex_t
    {
        CNC_WFD_RTSP_HH_RESOLUTION_REFRESH_RATE_800x480p30_INDEX   =   0     ,   /* */
        CNC_WFD_RTSP_HH_RESOLUTION_REFRESH_RATE_800x480p60_INDEX   =   1     ,   /* */
        CNC_WFD_RTSP_HH_RESOLUTION_REFRESH_RATE_854x480p30_INDEX   =   2     ,   /* */
        CNC_WFD_RTSP_HH_RESOLUTION_REFRESH_RATE_854x480p60_INDEX   =   3     ,   /* */
        CNC_WFD_RTSP_HH_RESOLUTION_REFRESH_RATE_864x480p30_INDEX   =   4     ,   /* */
        CNC_WFD_RTSP_HH_RESOLUTION_REFRESH_RATE_864x480p60_INDEX   =   5     ,   /* */
        CNC_WFD_RTSP_HH_RESOLUTION_REFRESH_RATE_640x360p30_INDEX   =   6     ,   /* */
        CNC_WFD_RTSP_HH_RESOLUTION_REFRESH_RATE_640x360p60_INDEX   =   7     ,   /* */
        CNC_WFD_RTSP_HH_RESOLUTION_REFRESH_RATE_960x540p30_INDEX   =   8     ,   /* */
        CNC_WFD_RTSP_HH_RESOLUTION_REFRESH_RATE_960x540p60_INDEX   =   9     ,   /* */
        CNC_WFD_RTSP_HH_RESOLUTION_REFRESH_RATE_848x480p30_INDEX   =   10    ,   /* */
        CNC_WFD_RTSP_HH_RESOLUTION_REFRESH_RATE_848x480p60_INDEX   =   11    ,   /* */
    } cncWFD_HhResolutionRefreshRateIndex_t;

    typedef enum _cncWFD_CeaResolutionRefreshRate_t
    {
        CNC_WFD_RTSP_CEA_RESOLUTION_REFRESH_RATE_640x480p60   = (1 << CNC_WFD_RTSP_CEA_RESOLUTION_REFRESH_RATE_640x480p60_INDEX    )  ,   ///< CEA resolution mode code
        CNC_WFD_RTSP_CEA_RESOLUTION_REFRESH_RATE_720x480p60   = (1 << CNC_WFD_RTSP_CEA_RESOLUTION_REFRESH_RATE_720x480p60_INDEX    )  ,   ///< CEA resolution mode code
        CNC_WFD_RTSP_CEA_RESOLUTION_REFRESH_RATE_720x480i60   = (1 << CNC_WFD_RTSP_CEA_RESOLUTION_REFRESH_RATE_720x480i60_INDEX    )  ,   ///< CEA resolution mode code
        CNC_WFD_RTSP_CEA_RESOLUTION_REFRESH_RATE_720x576p50   = (1 << CNC_WFD_RTSP_CEA_RESOLUTION_REFRESH_RATE_720x576p50_INDEX    )  ,   ///< CEA resolution mode code
        CNC_WFD_RTSP_CEA_RESOLUTION_REFRESH_RATE_720x576i50   = (1 << CNC_WFD_RTSP_CEA_RESOLUTION_REFRESH_RATE_720x576i50_INDEX    )  ,   ///< CEA resolution mode code
        CNC_WFD_RTSP_CEA_RESOLUTION_REFRESH_RATE_1280x720p30  = (1 << CNC_WFD_RTSP_CEA_RESOLUTION_REFRESH_RATE_1280x720p30_INDEX   )  ,   ///< CEA resolution mode code
        CNC_WFD_RTSP_CEA_RESOLUTION_REFRESH_RATE_1280x720p60  = (1 << CNC_WFD_RTSP_CEA_RESOLUTION_REFRESH_RATE_1280x720p60_INDEX   )  ,   ///< CEA resolution mode code
        CNC_WFD_RTSP_CEA_RESOLUTION_REFRESH_RATE_1920x1080p30 = (1 << CNC_WFD_RTSP_CEA_RESOLUTION_REFRESH_RATE_1920x1080p30_INDEX  )  ,   ///< CEA resolution mode code
        CNC_WFD_RTSP_CEA_RESOLUTION_REFRESH_RATE_1920x1080p60 = (1 << CNC_WFD_RTSP_CEA_RESOLUTION_REFRESH_RATE_1920x1080p60_INDEX  )  ,   ///< CEA resolution mode code
        CNC_WFD_RTSP_CEA_RESOLUTION_REFRESH_RATE_1920x1080i60 = (1 << CNC_WFD_RTSP_CEA_RESOLUTION_REFRESH_RATE_1920x1080i60_INDEX  )  ,   ///< CEA resolution mode code
        CNC_WFD_RTSP_CEA_RESOLUTION_REFRESH_RATE_1280x720p25  = (1 << CNC_WFD_RTSP_CEA_RESOLUTION_REFRESH_RATE_1280x720p25_INDEX   )  ,   ///< CEA resolution mode code
        CNC_WFD_RTSP_CEA_RESOLUTION_REFRESH_RATE_1280x720p50  = (1 << CNC_WFD_RTSP_CEA_RESOLUTION_REFRESH_RATE_1280x720p50_INDEX   )  ,   ///< CEA resolution mode code
        CNC_WFD_RTSP_CEA_RESOLUTION_REFRESH_RATE_1920x1080p25 = (1 << CNC_WFD_RTSP_CEA_RESOLUTION_REFRESH_RATE_1920x1080p25_INDEX  )  ,   ///< CEA resolution mode code
        CNC_WFD_RTSP_CEA_RESOLUTION_REFRESH_RATE_1920x1080p50 = (1 << CNC_WFD_RTSP_CEA_RESOLUTION_REFRESH_RATE_1920x1080p50_INDEX  )  ,   ///< CEA resolution mode code
        CNC_WFD_RTSP_CEA_RESOLUTION_REFRESH_RATE_1920x1080i50 = (1 << CNC_WFD_RTSP_CEA_RESOLUTION_REFRESH_RATE_1920x1080i50_INDEX  )  ,   ///< CEA resolution mode code
        CNC_WFD_RTSP_CEA_RESOLUTION_REFRESH_RATE_1280x720p24  = (1 << CNC_WFD_RTSP_CEA_RESOLUTION_REFRESH_RATE_1280x720p24_INDEX   )  ,   ///< CEA resolution mode code
        CNC_WFD_RTSP_CEA_RESOLUTION_REFRESH_RATE_1920x1080p24 = (1 << CNC_WFD_RTSP_CEA_RESOLUTION_REFRESH_RATE_1920x1080p24_INDEX  )  ,   ///< CEA resolution mode code
    } cncWFD_CeaResolutionRefreshRate_t;

    typedef enum _cncWFD_VesaResolutionRefreshRate_t
    {
        CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_800x600p30   =  (1 << CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_800x600p30_INDEX      )  ,   ///< VESA resolution mode code
        CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_800x600p60   =  (1 << CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_800x600p60_INDEX      )  ,   ///< VESA resolution mode code
        CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1024x768p30  =  (1 << CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1024x768p30_INDEX     )  ,   ///< VESA resolution mode code
        CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1024x768p60  =  (1 << CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1024x768p60_INDEX     )  ,   ///< VESA resolution mode code
        CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1152x864p30  =  (1 << CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1152x864p30_INDEX     )  ,   ///< VESA resolution mode code
        CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1152x864p60  =  (1 << CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1152x864p60_INDEX     )  ,   ///< VESA resolution mode code
        CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1280x768p30  =  (1 << CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1280x768p30_INDEX     )  ,   ///< VESA resolution mode code
        CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1280x768p60  =  (1 << CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1280x768p60_INDEX     )  ,   ///< VESA resolution mode code
        CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1280x800p30  =  (1 << CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1280x800p30_INDEX     )  ,   ///< VESA resolution mode code
        CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1280x800p60  =  (1 << CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1280x800p60_INDEX     )  ,   ///< VESA resolution mode code
        CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1360x768p30  =  (1 << CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1360x768p30_INDEX     )  ,   ///< VESA resolution mode code
        CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1360x768p60  =  (1 << CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1360x768p60_INDEX     )  ,   ///< VESA resolution mode code
        CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1366x768p30  =  (1 << CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1366x768p30_INDEX     )  ,   ///< VESA resolution mode code
        CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1366x768p60  =  (1 << CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1366x768p60_INDEX     )  ,   ///< VESA resolution mode code
        CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1280x1024p30 =  (1 << CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1280x1024p30_INDEX    )  ,   ///< VESA resolution mode code
        CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1280x1024p60 =  (1 << CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1280x1024p60_INDEX    )  ,   ///< VESA resolution mode code
        CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1400x1050p30 =  (1 << CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1400x1050p30_INDEX    )  ,   ///< VESA resolution mode code
        CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1400x1050p60 =  (1 << CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1400x1050p60_INDEX    )  ,   ///< VESA resolution mode code
        CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1440x900p30  =  (1 << CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1440x900p30_INDEX     )  ,   ///< VESA resolution mode code
        CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1440x900p60  =  (1 << CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1440x900p60_INDEX     )  ,   ///< VESA resolution mode code
        CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1600x900p30  =  (1 << CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1600x900p30_INDEX     )  ,   ///< VESA resolution mode code
        CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1600x900p60  =  (1 << CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1600x900p60_INDEX     )  ,   ///< VESA resolution mode code
        CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1600x1200p30 =  (1 << CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1600x1200p30_INDEX    )  ,   ///< VESA resolution mode code
        CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1600x1200p60 =  (1 << CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1600x1200p60_INDEX    )  ,   ///< VESA resolution mode code
        CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1680x1024p30 =  (1 << CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1680x1024p30_INDEX    )  ,   ///< VESA resolution mode code
        CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1680x1024p60 =  (1 << CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1680x1024p60_INDEX    )  ,   ///< VESA resolution mode code
        CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1680x1050p30 =  (1 << CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1680x1050p30_INDEX    )  ,   ///< VESA resolution mode code
        CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1680x1050p60 =  (1 << CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1680x1050p60_INDEX    )  ,   ///< VESA resolution mode code
        CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1920x1200p30 =  (1 << CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1920x1200p30_INDEX    )  ,   ///< VESA resolution mode code
        CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1920x1200p60 =  (1 << CNC_WFD_RTSP_VESA_RESOLUTION_REFRESH_RATE_1920x1200p60_INDEX    )  ,   ///< VESA resolution mode code
    } cncWFD_VesaResolutionRefreshRate_t;

    typedef enum _cncWFD_HhResolutionRefreshRate_t
    {
        CNC_WFD_RTSP_HH_RESOLUTION_REFRESH_RATE_800x480p30   =   (1 <<  CNC_WFD_RTSP_HH_RESOLUTION_REFRESH_RATE_800x480p30_INDEX  )  ,   ///< HH resolution mode code
        CNC_WFD_RTSP_HH_RESOLUTION_REFRESH_RATE_800x480p60   =   (1 <<  CNC_WFD_RTSP_HH_RESOLUTION_REFRESH_RATE_800x480p60_INDEX  )  ,   ///< HH resolution mode code
        CNC_WFD_RTSP_HH_RESOLUTION_REFRESH_RATE_854x480p30   =   (1 <<  CNC_WFD_RTSP_HH_RESOLUTION_REFRESH_RATE_854x480p30_INDEX  )  ,   ///< HH resolution mode code
        CNC_WFD_RTSP_HH_RESOLUTION_REFRESH_RATE_854x480p60   =   (1 <<  CNC_WFD_RTSP_HH_RESOLUTION_REFRESH_RATE_854x480p60_INDEX  )  ,   ///< HH resolution mode code
        CNC_WFD_RTSP_HH_RESOLUTION_REFRESH_RATE_864x480p30   =   (1 <<  CNC_WFD_RTSP_HH_RESOLUTION_REFRESH_RATE_864x480p30_INDEX  )  ,   ///< HH resolution mode code
        CNC_WFD_RTSP_HH_RESOLUTION_REFRESH_RATE_864x480p60   =   (1 <<  CNC_WFD_RTSP_HH_RESOLUTION_REFRESH_RATE_864x480p60_INDEX  )  ,   ///< HH resolution mode code
        CNC_WFD_RTSP_HH_RESOLUTION_REFRESH_RATE_640x360p30   =   (1 <<  CNC_WFD_RTSP_HH_RESOLUTION_REFRESH_RATE_640x360p30_INDEX  )  ,   ///< HH resolution mode code
        CNC_WFD_RTSP_HH_RESOLUTION_REFRESH_RATE_640x360p60   =   (1 <<  CNC_WFD_RTSP_HH_RESOLUTION_REFRESH_RATE_640x360p60_INDEX  )  ,   ///< HH resolution mode code
        CNC_WFD_RTSP_HH_RESOLUTION_REFRESH_RATE_960x540p30   =   (1 <<  CNC_WFD_RTSP_HH_RESOLUTION_REFRESH_RATE_960x540p30_INDEX  )  ,   ///< HH resolution mode code
        CNC_WFD_RTSP_HH_RESOLUTION_REFRESH_RATE_960x540p60   =   (1 <<  CNC_WFD_RTSP_HH_RESOLUTION_REFRESH_RATE_960x540p60_INDEX  )  ,   ///< HH resolution mode code
        CNC_WFD_RTSP_HH_RESOLUTION_REFRESH_RATE_848x480p30   =   (1 <<  CNC_WFD_RTSP_HH_RESOLUTION_REFRESH_RATE_848x480p30_INDEX  )  ,   ///< HH resolution mode code
        CNC_WFD_RTSP_HH_RESOLUTION_REFRESH_RATE_848x480p60   =   (1 <<  CNC_WFD_RTSP_HH_RESOLUTION_REFRESH_RATE_848x480p60_INDEX  )  ,   ///< HH resolution mode code
    } cncWFD_HhResolutionRefreshRate_t;

    typedef enum _cncWFD_DisplayConnectorType_t
    {
        CNC_WFD_RTSP_DISPLAY_CONNECTOR_TYPE_VGA                  =   0      ,  ///< Video Graphics Array (VGA) Connector                                                 
        CNC_WFD_RTSP_DISPLAY_CONNECTOR_TYPE_SVIDEO               =   1      ,  ///< S-Video connector                                                                    
        CNC_WFD_RTSP_DISPLAY_CONNECTOR_TYPE_CVBS1                =   2      ,  ///< Composite video connector                                                            
        CNC_WFD_RTSP_DISPLAY_CONNECTOR_TYPE_CVBS2                =   3      ,  ///< Component video connector                                                            
        CNC_WFD_RTSP_DISPLAY_CONNECTOR_TYPE_DVI                  =   4      ,  ///< Digital video interface (DVI) connector                                              
        CNC_WFD_RTSP_DISPLAY_CONNECTOR_TYPE_HDMI                 =   5      ,  ///< High-definition multimedia interface (HDMI) connector                                
        CNC_WFD_RTSP_DISPLAY_CONNECTOR_TYPE_LVDS                 =   6      ,  ///< Low voltage differential signaling (LVDS) connector                                  
        CNC_WFD_RTSP_DISPLAY_CONNECTOR_TYPE_EIAJ_RC_5237_D       =   8      ,  ///< Japanese D connector. (A connector conforming to the EIAJ RC-5237 standard)          
        CNC_WFD_RTSP_DISPLAY_CONNECTOR_TYPE_SDI                  =   9      ,  ///< Serial Digital Image (SDI) connector                                                 
        CNC_WFD_RTSP_DISPLAY_CONNECTOR_TYPE_DP                   =   10     ,  ///< A display port that connects externally to a display device. (DP)                    
        CNC_WFD_RTSP_DISPLAY_CONNECTOR_TYPE_UDI                  =   12     ,  ///< A Unified Display Interface (UDI) that connects externally to a display device.      
        CNC_WFD_RTSP_DISPLAY_CONNECTOR_TYPE_NO_ACTIVE_CONNECTOR  =   254    ,  ///< No active connector                                                                  
        CNC_WFD_RTSP_DISPLAY_CONNECTOR_TYPE_UNDEFINED            =   255    ,  ///< Indicates a type of physical connector that is not on this list                      
    } cncWFD_DisplayConnectorType_t;

    typedef enum cncWFDHDCPContentProtectionType_t
    {
        CNC_WFD_RTSP_HDCP_CONTENT_PROTECTION_NOT_SUPPORTED=0,   ///< HDCP protection support
        CNC_WFD_RTSP_HDCP_CONTENT_PROTECTION_SUPPORTED=1        ///< HDCP protection support
    } cncWFD_HdcpContentProtection_t;

    typedef enum _cncWFD_NetworkProtocol_t
    {
        CNC_WFD_RTSP_TRANSPORT_UDP=0,                           ///< Transport is over UDP
        CNC_WFD_RTSP_TRANSPORT_TCP,                             ///< Transport is over TCP
    } cncWFD_NetworkProtocol_t;

    typedef struct _cncWFD_cursorCoord_t
    {
        CNC_SINT32 x;               ///< X cursor coordinate UIBC
        CNC_SINT32 y;               ///< Y cursor coordinate UIBC
    } cncWFD_cursorCoord_t;

    typedef struct _cncWFD_ControlPlayback_t {
        CNC_SINT32 flushTiming;     ///< flushtiming defined for SAMSUNG's handhelds
        CNC_SINT32 volume;          ///< Volume control for SAMSUNG's handhelds
        CNC_SINT32 controlPlayback; ///<
    } cncWFD_ControlPlayback_t;

    typedef struct _cncWFD_profile
    {
        CNC_SINT8 numDifferentProfilesSupported;///< number of different profiles supported by sink
        CNC_SINT32 profile;                     ///< profile bitmap
    } cncWFD_profile_t;

    typedef struct _cncWFD_VideoInfo_t {

        CNC_UINT32 bitmapCEA;                           ///< CEA Resolutions/Refresh rates bitmap
        CNC_UINT32 bitmapVESA;                          ///< VESA Resolutions/Refresh rates bitmap
        CNC_UINT32 bitmapHH;                            ///< HH Resolutions/Refresh rates bitmap
        CNC_SINT8  resolutionRefreshRateTableSelection; ///< Table selection
        CNC_SINT8  indexInToResolutionRefreshRateTable; ///< Index of the chosen resolution
        CNC_SINT8  profiles;                            ///< Bitmap detailing H264 profiles
        CNC_SINT8  levels;                              ///< Bitmap detailing H264 levels
        CNC_UINT8  latencyMultipleOf5mSec;              ///< Video decoder latency measured in integer multiple of 5ms
        CNC_UINT8  minimumSliceSize;                    ///< Minimum slice size in number of Macroblocks
        CNC_SINT16 maxNoOfSlicesPerPictureMinus1;       ///< Maximum number of slices per picture minus 1
        CNC_SINT8  ratioMaxSliceSizeAndMiniSliceSize;   ///< Ratio between maximum minimum slice size
        CNC_SINT8  frameSkippingSupport;                ///< Frame skip is supported by the decoder
        CNC_SINT16 frameRateControlSupported;           ///< Bitmap detailing video frame skipping support with its parameter and frame rate change
        CNC_SINT8  frameRateIntervalMultipleOf500mSEC;  ///< Interval measured in units of 500msec
        CNC_SINT8  videoFrameRateChangeSupported;
    } cncWFD_VideoInfo_t;

    typedef struct _cncWFD_AudioInfo_t {
        CNC_UINT32 codec;                   ///< Selected codec
        CNC_UINT32 bitmapAAC;               ///< AAC codec bitmap
        CNC_UINT32 bitmapLPCM;              ///< LPCM codec bitmap
        CNC_UINT32 bitmapAC3;               ///< AC3 codec bitmap
        CNC_UINT32 LpcmDecoderLatency;      ///< LPCM decoder latency in units of 5ms
        CNC_UINT32 AacDecoderLatency;       ///< AAC decoder latency in units of 5ms
        CNC_UINT32 AC3DecoderLatency;       ///< AC3 decoder latency in units of 5ms
    } cncWFD_AudioInfo_t;

    typedef struct _cncWFD_sinkAVConfig_t
    {
        cncWFD_videoType_t videoType;       ///< CEA/VESA/HH
        CNC_UINT32 videoMode;               ///< TVOut mode
        CNC_UINT32 frameRate;               ///< specified bitrate
        CNC_UINT32 frameType;               ///< Interlaced(2)/Progressive(1)
        cncWFD_AudioFormat_t codec;         ///< Audio codec
        CNC_UINT32 audioMode;               ///< Audio modes bitmap, (frequency, bitlength)
    } cncWFD_sinkAVConfig_t;
    
    typedef struct _cncWFD_AVLatSyncConfig_t
    {
        CNC_SINT32 avSync;                    ///< avSync type: 0- No avSync (should not be used), 1- avSync, 2- fast mode. Set -1 to use default.
        CNC_SINT32 avSync_error;              ///< avSync_error type: AV miss sync accepted whild avSync = 1, for video latency be improved.
        CNC_SINT32 decoderLatency;            ///< Specify the AV decoder latency. Set -1 to use default.
        CNC_SINT32 udpAudLatencyMarginHighEn; ///< Enable/disables mechanism to relax latency restrictions for avSync = 2. Set -1 to use default.
        CNC_SINT32 udpAudLatencyMarginHigh;   ///< When udpAudLatencyMarginHighEn = 1, High margin for latency value. Set -1 to use default.
        CNC_SINT32 udpAudLatencyMargin;       ///< When udpAudLatencyMarginHighEn = 1, final latency value after udpAudLatencyMarginHigh threshold is achieved. Set -1 to use default.
        CNC_SINT32 sqosMode;                  ///< SQOS Mode. SQOS is a mechanism to automatically adapt decoder latency. 0 = disabled; 1 = Fixed Step; 2 = Smart Step. Set -1 to use default.
        CNC_SINT32 videoOffset;               ///< Offset added to video PTS before sending it to the AV decoding firmware. Set 0 to use default.
        CNC_SINT32 audioOffset;               ///< Offset added to audio PTS before sending it to the AV decoding firmware. Set 0 to use default.
        CNC_SINT32 tcpVideoPtsOffset;         ///< Same thing as videoOffset but only applies for the TCP mode. Set to 0 to use default.
        CNC_SINT32 jitterBuffer;              ///< this sets the jitterBuffer of the decoder. This is mainly used in TCP mode.
    } cncWFD_AVLatSyncConfig_t;

    typedef struct _cncWFD_PlaybackData_t
    {
        CNC_SINT32 hdcpPort;            ///< HDCP port
        CNC_SINT32 audioPortSrc;        ///< source audio port
        CNC_SINT32 audioPortDst;        ///< destination audio port
        CNC_SINT32 videoPortSrc;        ///< source video port
        CNC_SINT32 videoPortDst;        ///< destination video port

        CNC_CHAR audioPortSrcIP[15+1];  ///< audio source IP
        CNC_CHAR audioPortDstIP[15+1];  ///< audio destination IP
        CNC_CHAR videoPortSrcIP[15+1];  ///< video source IP
        CNC_CHAR videoPortDstIP[15+1];  ///< video destination IP

        cncWFD_AudioFormat_t audioCodec;    ///< used audio codec
        cncWFD_NetworkProtocol_t rtpTransport;  ///< transport TCP or UDP
        cncWFD_AVLatSyncConfig_t AVLatSync; ///< AVlatSyncConfig struct
    } cncWFD_PlaybackData_t;

    typedef struct _cncWFD_CursorData_t
    {
        CNC_SINT32 xPos;                    ///< Horizontal position of the cursor
        CNC_SINT32 yPos;                    ///< Vertical position of the cursor
        CNC_SINT32 cursorId;                ///< Cursor ID. Cursor images can be found in /tmp/id.png. -1 means default cursor
        CNC_SINT32 imageType;               ///< Currently not used
        CNC_SINT32 hotSpotXPos;             ///< The X-axis offset of the hot spot offset from the top left hand corner of the cursor image.
        CNC_SINT32 hotSpotYPos;             ///< The Y-axis offset of the hot spot offset from the top left hand corner of the cursor image.
    } cncWFD_CursorData_t;
//-------------------------------------------------------------------------------------------------------------------------------------------

    typedef CNC_VOID (*cncWFD_Callback_t)(CNC_VOID* data, cncWFD_Event_t wfd_event);

    typedef struct _cncWFD_intel_version_t
    {
        CNC_SINT32 version_major;
        CNC_SINT32 version_minor;
        CNC_SINT32 version_sku;
        CNC_SINT32 version_build;
    } cncWFD_intel_version_t;

    typedef struct _cncWFD_intel_overscan_t
    {
        CNC_UINT16 x;                           ///< Horizontal overscan from -8 to 7
        CNC_UINT16 y;                           ///< Vertical overscan from -8 to 7
    } cncWFD_intel_overscan_t;

    typedef struct _cncWFD_intel_pipeline_t
    {
        CNC_SINT32 playbackdelay;
        CNC_SINT32 positivemaxstcpcr;
        CNC_SINT32 negativemaxstcpcr;
        CNC_SINT32 low_delay_mode_changed;
    } cncWFD_intel_pipeline_t;

    typedef struct _cncWFD_Widi_t
    {
        cncWFD_intel_version_t hw;              ///< HW version following Intel's definitions
        cncWFD_intel_version_t sw;              ///< SW version following Intel's definitions
        CNC_SINT32 rtcp_port;                   ///< RTCP port value
        cncWFD_intel_overscan_t overscan;       ///< Overscan value from -8 to 7
        cncWFD_intel_pipeline_t pipeline;       ///< 
        CNC_SINT32 lower_bandwidth;             ///<
        CNC_CHAR product_id[17];                ///< Product identification number
        CNC_CHAR friendly_name[19];             ///< Device Name with compatible symbols
        CNC_CHAR manufacturer_name[33];         ///< Manufacturer name
        CNC_CHAR model_name[33];                ///< Model Name
        CNC_CHAR url[256];                      ///<
        CNC_CHAR manufacturer_logo_filename[128]; ///< Manufacturer logo given its filename
    } cncWFD_Widi_t;

    typedef struct _cncWFD_Config_t
    {
        cncWFD_Device_t devType;        ///< Device type, sink or source.
        CNC_SINT32 supported_profiles;  ///< Combination of supported profiles supported by sink
                                        ///< for example
                                        ///<(CNC_WFD_RTSP_CONSTRAINED_BASELINE_PROFILE | CNC_WFD_RTSP_CONSTRAINED_HIGH_PROFILE)
        cncWFD_AudioInfo_t audioConfig; ///< Structure that holds the bitmaps of the different supported
                                        ///< supported audio formats conforming WFD spec.
        cncWFD_VideoInfo_t videoFormats[2];     ///< One for each profile. If only one profile is supported
        // use videoFormats[0] to configure RTSP state machine.
        cncWFD_VideoInfo_t video3DFormats[2];   ///< One for each profile. If only one profile is supported
        // use videoFormats[0] to configure RTSP state machine.
        cncWFD_HdcpContentProtection_t hdcpContentProtection;   ///< This variable sets if the sink supports HDCP or not (CNC_WFD_RTSP_HDCP_CONTENT_PROTECTION_NOT_SUPPORTED/
                                        ///< CNC_WFD_RTSP_HDCP_CONTENT_PROTECTION_SUPPORTED)
        CNC_SINT8 hdcpEnable;           ///< Even if the sink supports HDCP, it can be disabled for the current
                                        ///< session setting this varaible to 1.
        CNC_SINT32 hdcpPort;            ///< Sets HDCP port
        CNC_SINT32 edidLen;             ///< EDID Length
        CNC_CHAR edidPayload[65536];     ///< EDID data
        CNC_CHAR ipAddress[16];         ///< Sink IP address
        CNC_SINT32 rtpPort0;            ///< Sink incoming video source port
        CNC_SINT32 rtpPort1;            ///< Sink incoming audio source port
        CNC_SINT32 uibcPort;            ///< uibc port
        CNC_SINT8 uibcSetting;          ///< uibc setting (enable UIBC)
        CNC_SINT8 standbySupported;     ///< Standby resume capability
        cncWFD_DisplayConnectorType_t connectorType; ///< connector type used
        CNC_SINT32 tcpPort;             ///< RTSP TCP port
        CNC_CHAR pUserAgent[260];       ///< User Agent string
        CNC_SINT8 frameskipEnable;      ///< Sets/Unsets frame skip decoder capability as can be seen in WFD_RTSP spec
        CNC_SINT32 i2cPort;             ///< I2C port number 
        CNC_SINT8 videoRecovery;        ///< Sets if IDR is supported or not
        CNC_SINT32 fwUpgradePort;       ///< establishes the port number for firmware upgrade 
        CNC_SINT32 jitterBuffer;        ///< establishes the size of the jitter buffer 
        CNC_SINT8 reportJitterBufferEnable; ///< sets if jitter buffer size is sent in an special RTSP message or not
        cncWFD_profile_t profile;       ///< This value establishes the H264 accepted profile
        CNC_SINT32 level;               ///< This values establishes the H264 accepted level
        CNC_SINT32 tcpWindow;           ///< This value establishes the size of the tcp Window
        cncWFD_control_playback_t controlPlayback;  ///< sets the control playback value indicating the function remotely sent to source
        CNC_SINT32 audioBufferSize;     ///< sets the audio buffer size
        CNC_SINT32 numberConnectionAttempts;    ///< Number of RTSP TCP connection tries with source before closing 
                                        ///< closing TCP socket
        CNC_SINT32 decoderLatency;      ///< decoder latency in units of 5ms
        CNC_SINT8 errorSkipMode;        ///< 
        CNC_SINT32 reorderingBufferSize;///< establishes the size of the reordering buffer
        cncWFD_Widi_t widi;             ///< configuration when working on WIDI mode
        CNC_SINT32 internalUIBC;        ///< Set this value to 0 if you are using your own UIBC implementation. If you prefer to use our implementation set to 1.
        CNC_SINT32 IDRRequestCapability;///< Set to one if sink is able to send 'IDR Request' messages, 0 if don't
                                        ///< If using your own implementation you wil get two events CNC_WFD_RTSP_EVENT_UIBC_START and CNC_WFD_RTSP_EVENT_UIBC_STOP
                                        ///< that tells you when to start stop your UIBC process/thread
        CNC_SINT32 IntelFastCursorPort; ///< This value must be one between 49152 and 65535
        CNC_CHAR MiracastVersion[17];
        CNC_CHAR Gen4IntelWidiSinkVersion[17];
        CNC_CHAR IntelUoIPVersion[17];
        CNC_SINT32 IntelWiDiSupport;    ///< Set to 0 to disable widi specific parameters
    } cncWFD_Config_t;

/** Create handle to RTSP object
 *
 * @param handle: handle where the RTSP object will be saved
 * @param device: device indicating if it is a source or a sink
 * @return returns CNC_WFD_RTSP_FAILURE or CNC_WFD_RTSP_SUCCESS
 */
    CNC_SINT32 CNC_WFD_Create(cncWFD_RtspHnd_t* handle, cncWFD_Device_t device);// rtsp.c

/** Destroys handle to RTSP object and deallocate memory
 *
 * @param handle: handle to RTSP object
 * @return returns CNC_WFD_RTSP_FAILURE or CNC_WFD_RTSP_SUCCESS
 */
    CNC_SINT32 CNC_WFD_Destroy(cncWFD_RtspHnd_t handle);// rtsp.c

/** Sets configuration of the RTSP object
 *
 * @param handle: handle to RTSP object
 * @param config: configuration structure where the applying config is saved
 * @return returns CNC_WFD_RTSP_FAILURE or CNC_WFD_RTSP_SUCCESS
 */
    CNC_SINT32 CNC_WFD_SetConfigure(cncWFD_RtspHnd_t handle, cncWFD_Config_t* config); // dont forget configfilename in cncWFD_Config_t

/** Gets current configuration from RTSP object
 *
 * @param handle: handle to RTSP object
 * @param config: configuration structure where the current configuration will be stored
 * @return returns CNC_WFD_RTSP_FAILURE or CNC_WFD_RTSP_SUCCESS
 */
    CNC_SINT32 CNC_WFD_GetConfigure(cncWFD_RtspHnd_t handle, cncWFD_Config_t* config); // dont forget configfilename in cncWFD_Config_t

/** Start running RTSP object
 *
 * @param handle: handle to RTSP object
 * @return returns CNC_WFD_RTSP_FAILURE or CNC_WFD_RTSP_SUCCESS
 */
    CNC_SINT32 CNC_WFD_Start(cncWFD_RtspHnd_t handle);

/** Stop running RTSP object
 *
 * @param handle: handle to RTSP object
 * @return returns CNC_WFD_RTSP_FAILURE or CNC_WFD_RTSP_SUCCESS
 */
    CNC_SINT32 CNC_WFD_Stop(cncWFD_RtspHnd_t handle);

/** Stop running RTSP object due to disconnection, which does not try to send TEARDOWN
 *
 * @param handle: handle to RTSP object
 * @return returns CNC_WFD_RTSP_FAILURE or CNC_WFD_RTSP_SUCCESS
 */
    CNC_SINT32 CNC_WFD_P2PDisconnected(cncWFD_RtspHnd_t handle);
    
/** RTSP object sends a PLAY message to source
 *
 * @param handle: handle to RTSP object
 * @return returns CNC_WFD_RTSP_FAILURE or CNC_WFD_RTSP_SUCCESS
 */
    CNC_SINT32 CNC_WFD_Play(cncWFD_RtspHnd_t handle);// parser.c

/** RTSP object sends a PAUSE message to source
 *
 * @param handle: handle to RTSP object
 * @return returns CNC_WFD_RTSP_FAILURE or CNC_WFD_RTSP_SUCCESS
 */
    CNC_SINT32 CNC_WFD_Pause(cncWFD_RtspHnd_t handle);// parser.c

/** RTSP object sends a TEARDOWN message to source to stop trasnmission
 *
 * @param handle: handle to RTSP object
 * @return returns CNC_WFD_RTSP_FAILURE or CNC_WFD_RTSP_SUCCESS
 */
    CNC_SINT32 CNC_WFD_Teardown(cncWFD_RtspHnd_t handle);// parser.c

/** RTSP object sends a SET_PARAM IDR-REQUEST message to source, requesting an I-Frame to source
 *
 * @param handle: handle to RTSP object
 * @return returns CNC_WFD_RTSP_FAILURE or CNC_WFD_RTSP_SUCCESS
 */
    CNC_SINT32 CNC_WFD_IDRRequest(cncWFD_RtspHnd_t handle);// parser.c

/** RTSP object initiates a capabilities renegotiation message to the source
 *
 * @param handle: handle to RTSP object
 * @return returns CNC_WFD_RTSP_FAILURE or CNC_WFD_RTSP_SUCCESS
 */
    CNC_SINT32 CNC_WFD_Caprenego(cncWFD_RtspHnd_t handle);// parser.c

/** RTSP object adverts source that it has entered in 'standby' mode
 *
 * @param handle: handle to RTSP object
 * @return returns CNC_WFD_RTSP_FAILURE or CNC_WFD_RTSP_SUCCESS
 */
    CNC_SINT32 CNC_WFD_Standby(cncWFD_RtspHnd_t handle);// parser.c

/** Standby can be disabled/enabled on the sink
 *
 * @param handle: handle to RTSP object
 * @param flag: enable disable value
 * @return returns CNC_WFD_RTSP_FAILURE or CNC_WFD_RTSP_SUCCESS
 */
    CNC_SINT32 CNC_WFD_SetStandby(cncWFD_RtspHnd_t handle, cncWFD_Enabler_t flag);// parser.c

/** This function enables/disables UIBC
 *
 * @param handle: handle to RTSP object
 * @param flag: enable disable value
 * @return returns CNC_WFD_RTSP_FAILURE or CNC_WFD_RTSP_SUCCESS
 */
    CNC_SINT32 CNC_WFD_UIBCSetting(cncWFD_RtspHnd_t handle, cncWFD_Enabler_t flag);

/** This function makes appear the UIBC capability in the SET_PARAM message to ask for UIBC support
 *
 * @param handle: handle to RTSP object
 * @return returns CNC_WFD_RTSP_FAILURE or CNC_WFD_RTSP_SUCCESS
 */
    CNC_SINT32 CNC_WFD_UIBCCapability(cncWFD_RtspHnd_t handle);

/** This function activate/deactivate the use of the internal UIBC implementation
 *
 * @param handle: handle to RTSP object
 * @return returns CNC_WFD_RTSP_FAILURE or CNC_WFD_RTSP_SUCCESS
 */
    CNC_SINT32 CNC_WFD_InternalUIBC(cncWFD_RtspHnd_t handle, cncWFD_Enabler_t set);
/** Getter function to get the hpd status
 *
 * @param handle: handle to RTSP object
 * @return returns if hpd is on (1) or off(0)
 */
    CNC_SINT32 CNC_WFD_GetHPDConnection(cncWFD_RtspHnd_t handle);

/** If this function is called with an enable flag, it sets connector to HDMI, if not, HDMI is disconnected
 * and the session will shutdown
 *
 * @param handle: handle to RTSP object
 * @param flag: 
 * @return returns CNC_WFD_RTSP_FAILURE or CNC_WFD_RTSP_SUCCESS
 */
    CNC_SINT32 CNC_WFD_HDMIState(cncWFD_RtspHnd_t handle, cncWFD_Enabler_t flag);

/** With this function HDCP2x authentication can be retrieved from RTSP object
 *
 * @param handle: handle to RTSP object
 * @return returns CNC_WFD_RTSP_FAILURE or CNC_WFD_RTSP_SUCCESS
 */
    CNC_SINT32 CNC_WFD_GetHDCP2xAuth(cncWFD_RtspHnd_t handle);

/** Enables disables HDCP
 *
 * @param handle: handle to RTSP object
 * @return returns CNC_WFD_RTSP_FAILURE or CNC_WFD_RTSP_SUCCESS
 */
    CNC_SINT32 CNC_WFD_HDCPState(cncWFD_RtspHnd_t handle, cncWFD_Enabler_t flag);

/** Switch display video resolution
 * @param handle RTSP Object
 * @param videoRes tv video resolution index
 * @return CNC_WFD_RTSP_SUCCESS, CNC_WFD_RTSP_FAILURE
*/
    CNC_SINT32 CNC_WFD_SetTvResolution(cncWFD_RtspHnd_t handle, CNC_SINT32 videoRes);

/** Set EDID array to send it to the source via RTSP
 * @param handle RTSP object
 * @param edid buffer containing the raw edid
 * @param len edid buffer length
 * @return CNC_WFD_RTSP_SUCCESS, CNC_WFD_RTSP_FAILURE
*/
    CNC_SINT32 CNC_WFD_SetEDID(cncWFD_RtspHnd_t handle, const CNC_CHAR* edid, CNC_SINT32 len);

/** Returns a pointer to an array containing the EDID in use
 * @param handle RTSP object
 * @param reserved memory that will contain the edid
 * @param len returned length of the EDID array
 * @return VOID
*/
    CNC_VOID CNC_WFD_GetEDID(cncWFD_RtspHnd_t handle, CNC_CHAR* edid, CNC_SINT32* len);

/** Set sink IP value to the string contained in ipAddress
 * @param handle RTSP object
 * @param string containing IP address
 * @return CNC_WFD_RTSP_SUCCESS, CNC_WFD_RTSP_FAILURE
*/
    CNC_SINT32 CNC_WFD_SetIP(cncWFD_RtspHnd_t handle, CNC_CHAR* ipAddress);

/** Get sink IP value and store it into ipAddress
 * @param handle RTSP object
 * @param string where the IP address will be stored.
 * @return CNC_WFD_RTSP_SUCCESS, CNC_WFD_RTSP_FAILURE
*/
    CNC_SINT32 CNC_WFD_GetIP(cncWFD_RtspHnd_t handle, CNC_CHAR* ipAddress);

/** Set UUID value to the string contained in UUID
 * @param handle RTSP object
 * @param UUID zero terminated string containing UUID
 * @return CNC_WFD_RTSP_SUCCESS, CNC_WFD_RTSP_FAILURE
*/
    CNC_SINT32 CNC_WFD_SetUUID(cncWFD_RtspHnd_t handle, CNC_CHAR* UUID);

/** Returns a pointer to a string containing the UUID string
 * @param handle RTSP object
 * @return pointer to zero terminated string containing UUID
*/
    CNC_SINT8* CNC_WFD_GetUUID(cncWFD_RtspHnd_t handle);

/** Set the number of available audio channels at the sink end in order to advertise it in M3 WFD_RTSP message
 * @param handle RTSP object
 * @param numChannels number of audio channels available at sink end
 * @return CNC_WFD_RTSP_SUCCESS, CNC_WFD_RTSP_FAILURE
*/
    CNC_SINT32 CNC_WFD_SetNumAudioChannels(cncWFD_RtspHnd_t handle, CNC_SINT32 numChannels);

/** Send an original Sigma command to the command processor
 * @param handle RTSP object
 * @param sigma_command zero terminated string containing command
 * @return CNC_WFD_RTSP_SUCCESS when succeeded
*/
    CNC_SINT32 CNC_WFD_SigmaCommand(cncWFD_RtspHnd_t handle, CNC_CHAR* sigma_command);

/** Force an HDCP authentication success (val = 1) or failure (val = 0)
 * @param handle RTSP object
 * @param val
 * \return
*/
    CNC_SINT32 CNC_WFD_SetHDCPAuth(cncWFD_RtspHnd_t handle, CNC_SINT32 val);

/** Use this function to Enable/disable HDCP
 * @param [in] handle RTSP object
 * @param [in] flag HDCP Enable Flag
 * @return
*/
    CNC_SINT32 CNC_WFD_SetHDCP(cncWFD_RtspHnd_t handle, cncWFD_Enabler_t flag);

/** Register the WFDEVENT related callback in WFDHandle with function WFDCallback
 * @param handle RTSP object
 * @param wfd_event Event that triggers the callback
 * @param func callback function
 * @return CNC_WFD_RTSP_SUCCESS, CNC_WFD_RTSP_FAILURE
*/
    CNC_SINT32 CNC_WFD_registerCallback(cncWFD_RtspHnd_t handle, cncWFD_Event_t wfd_event, cncWFD_Callback_t func);

/** If received a trigger_method:pause, maybe it is needed to check if we can pause the decoder at the request time
 * @param handle RTSP object
 * @param ready CNC_SINT32 variable ready, set CNC_WFD_RTSP_MEDIA_PAUSE if ready, and not CNC_WFD_MEDIA_ERROR if not.
 * @return CNC_WFD_RTSP_SUCCESS, CNC_WFD_RTSP_FAILURE
*/
    CNC_SINT32 CNC_WFD_triggeredPause(cncWFD_RtspHnd_t handle, CNC_SINT32 ready);

/** Set the resolutions that the current connected TV supports
 * @param handle RTSP object
 * @param val each bit in this integer represents one of the TVOUT modes. If set to 1 it means it is supported by the TV.
 * @return CNC_WFD_RTSP_SUCCESS, CNC_WFD_RTSP_FAILURE
*/
    CNC_VOID CNC_WFD_SetSupportedResolution(cncWFD_RtspHnd_t handle, CNC_SINT32 val);

/** Inform the RTSP about the last decoded resolution. If it is bigger than the current resolution and enough time has 
 * elapsed, a resolution change will be requested
 * @param handle RTSP object
 * @param resSrcW decoded resolution width
 * @param resSrcH decoded resolution height
 * @param resSrcFR decoded resolution framerate
 * @param resChangePTS last decoded PTS
 * @return CNC_WFD_RTSP_SUCCESS, CNC_WFD_RTSP_FAILURE
*/
    CNC_SINT32 CNC_WFD_SetDecodedResolution(cncWFD_RtspHnd_t handle, CNC_SINT32 resSrcW, CNC_SINT32 resSrcH, CNC_SINT32 resSrcFR, CNC_UINT64 resChangePTS);
    
    CNC_SINT32 CNC_WFD_enableDiagServer(cncWFD_RtspHnd_t handle);
    
#ifdef __cplusplus
}
#endif

#endif // __CNC_WFD_INTERFACE_H__
