/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Cavium, Inc.
 Cavium, Inc. reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium, Inc.

 ALL RIGHTS RESERVED.

 Copyright 2011-2012, Cavium, Inc.
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */


/**
 * @file cnc_wivu.h
 * @ingroup wd
 * The CNCWIDI library enables the development of applications using the WIDI protocol
*/

#ifndef __CNC_WIDI_INTERFACE_H__
#define __CNC_WIDI_INTERFACE_H__

#ifdef __cplusplus
extern "C" {
#endif


//Values extracted from "WIDI UI v3.5 adapter r 0.80" specs
#define MAX_FRIENDLY_NAME_BUF   (18+1)
#define MAX_MANUFACTURER_NAME   (32+1)
#define	MAX_MODEL_NAME_BUF      (13+1)
#define MAX_RDS_SINK_NAME_BUF   (32+1)    // As per RDS Spec 3.2.2.2
#define MAX_SSID_BUF            32
#define MAX_WPS_KEY_BUF         64
#define MAX_PRODUCT_ID_BUF_LEN  (16+1)
#define MAX_VERSION_NUMBER      (16+1)
#define MAX_URL                 (256+1)
#define MAX_YES_NO              3
#define MAX_FILENAME            128
#define MAX_IFACE_NAME          6
#define MAX_IP_STR              16

#define DEFAULT_WPS_VNDR_EXTN_ATTR_STR          "10490013000157020003000b0000000000000000a20000"
#define DEFAULT_WPS_PRIM_DEV_ATTR_STR           "105400080007001735200001"
#define DEFAULT_WPS_SEC_DEV_LIST_ATTR_STR       "0007001735200102"
#define DEFAULT_P2P_WPS_SEC_DEV_LIST_ATTR_STR   "0007001735200002"

/* ============================ System Includes =========================== */
// NONE

/* ============================= User Includes ============================ */
#include "cnc_common_typedef.h"


/* ======================== Enum declarations ========================= */

typedef enum _cncWIDIBoolean_t{
    CNC_WIDI_TRUE,
    CNC_WIDI_FALSE
} cncWIDIBoolean_t;

typedef enum _cncWIDIReturnValue_t{
    CNC_WIDI_FAILURE = -1,
    CNC_WIDI_SUCCESS = 0
} cncWIDIReturnValue_t;


typedef enum _cncWIDIEvent_t {
    CNC_WIDI_EVENT_ALL_EVENTS = -1,             ///< This can be used in the CNC_WIDI_registerCallback to assign all events to the same callback
    CNC_WIDI_EVENT_START = 0,                   ///< This callback will be called to start connection
    CNC_WIDI_EVENT_STOP,                        ///< This callback gest called to stop a connection
    CNC_WIDI_EVENT_CURSOR,                      ///< This callback is sent to draw cursor in screen.
    CNC_WIDI_EVENT_OVERSCAN,                    ///< This callback is sent when overscan setting change.
    CNC_WIDI_EVENT_CHANGE_NAME,
    
    CNC_WIDI_L2EVENT_DISCONNECT,           
    CNC_WIDI_L2EVENT_TRIGGERING,           
    CNC_WIDI_L2EVENT_SECURITY_CHECK,       
    CNC_WIDI_L2EVENT_SECURITY_SUCCESSFUL,  
    CNC_WIDI_L2EVENT_SMOOTH_HANDOFF,       
    CNC_WIDI_L2EVENT_CONNECTED,            
    CNC_WIDI_L2EVENT_DEAUTHENTICATED,      
    CNC_WIDI_L2EVENT_SCAN_START,           
    CNC_WIDI_EVENT_SEND_L2SD_MESSAGE,
    CNC_WIDI_EVENT_NUMBER
} cncWIDIEvent_t;

typedef struct _cncWIDIStatus_t {
    int sinkPort;
    int cursor_w;
    int cursor_h;
    int cursor_x;
    int cursor_y;
    int cursor_version;
    int overscan_x;
    int overscan_y;
    
} cncWIDIStatus_t;



typedef struct _cncWIDIAdapterConfig_t{
    CNC_CHAR    model_name[MAX_MODEL_NAME_BUF];
    CNC_CHAR    manufacturer_name[MAX_MANUFACTURER_NAME];
    CNC_CHAR    logo_file[MAX_FILENAME];
    CNC_CHAR    edid_file[MAX_FILENAME];
    CNC_CHAR    profiles_file[MAX_FILENAME];
    CNC_CHAR    product_id[MAX_PRODUCT_ID_BUF_LEN];
    CNC_CHAR    hw_version[MAX_VERSION_NUMBER];
    CNC_CHAR    sw_version[MAX_VERSION_NUMBER];
    CNC_CHAR    device_url[MAX_URL];
    CNC_CHAR    rds_interface_name[MAX_IFACE_NAME];    
    CNC_CHAR    p2p_interface_name[MAX_IFACE_NAME];
    CNC_SINT32  hdcp2_port;
    CNC_SINT32  rtp_port;
    CNC_SINT32  capabilities_flag;
    CNC_SINT32  wfd_capable;
    CNC_SINT32  wfd_auto_go;
    CNC_SINT32  enterprise_enabled;
    CNC_CHAR    friendly_name[MAX_FRIENDLY_NAME_BUF];
    CNC_SINT32  exit_on_teardown;
    CNC_CHAR    usb_over_ip_executable[MAX_FILENAME];
    CNC_SINT32  usb_over_ip_discovery_port;
    CNC_SINT32  usb_over_ip_first_data_port;
    CNC_CHAR    qrcode_file[MAX_FILENAME];
    CNC_CHAR    player_ip[MAX_IP_STR];
    CNC_CHAR    wpsVendorExtension[MAX_WPS_KEY_BUF];
    CNC_CHAR    wpsPrimaryDev[MAX_WPS_KEY_BUF];
    CNC_CHAR    wpsSecondaryDevList[MAX_WPS_KEY_BUF];
    CNC_CHAR    p2pSecondaryDevList[MAX_WPS_KEY_BUF];
    
} cncWIDIConfig_t;
    


/* ======================== Structs declarations ========================= */

typedef CNC_VOID* cncWIDIHandle_t;

// GG. DEPENDING ON THE EVENT, DIFFERENT TYPES OF INFORMATION IS SENT AS DATA
typedef CNC_VOID (*cncWIDICallback_t) (cncWIDIHandle_t, cncWIDIEvent_t, void* data);



/* ======================== Function declarations ========================= */
/**
 *  \brief          Initializes WIDI protocol
 *  \return         handler for cncWivu
 */
cncWIDIHandle_t CNC_WIDI_Create(void);

/**
 *  \brief          Deinitializes WIDI protocol
 *  \param  [in]    handle
 *                      handler for cncWivu
 *  \return
 */

cncWIDIReturnValue_t CNC_WIDI_Destroy(cncWIDIHandle_t handle);


cncWIDIReturnValue_t CNC_WIDI_Get_Config(cncWIDIHandle_t handle, cncWIDIConfig_t *p_config);
cncWIDIReturnValue_t CNC_WIDI_Set_Config(cncWIDIHandle_t handle, cncWIDIConfig_t p_config);

/**
 *  \brief          Run the main thread 
 *  \param  [in]    handle
 *                      handler for cncnWIDI
 *  \return
 */

cncWIDIReturnValue_t CNC_WIDI_Start(cncWIDIHandle_t handle );

/**
 *  \brief          Stops main thread
 *  \param  [in]    handle
 *                      handler for cncnWIDI
 *  @param          argc
 *                      number of parameters 
 *  @param          argv  
 *                      paremeters 
 *  \return
 */

cncWIDIReturnValue_t CNC_WIDI_Stop(cncWIDIHandle_t handle);

/**
 *  \brief          Sets language for UI
 *  \param  [in]    handle
 *                      handler for cncnWIDI
 *  \return
 */

cncWIDIReturnValue_t CNC_WIDI_Process_L2sd (cncWIDIHandle_t handle,unsigned long long arg, char * buf, int len);
/**
 *  \brief          Process l2SD commands
 *  \param  [in]    handle
 *                      handler for cncnWIDI
 *  \return
 */

cncWIDIReturnValue_t CNC_WIDI_setLanguage(cncWIDIHandle_t handle, unsigned char* language);



/**
 *  \brief          Sets EDID file from buffer
 *  \param  [in]    handle
 *                      handler for cncnWIDI
 *  \param  [in]    edid
 *                      buffer to store edid 
 *  \param  [in]    len
 *                      edid length   
  *  \return
 */
cncWIDIReturnValue_t CNC_WIDI_setEDID(cncWIDIHandle_t handle, unsigned char *edid, int len);


/**
 *  \brief          Sets EDID file
 *  \param  [in]    handle
 *                      handler for cncnWIDI
 *  \return
 */

cncWIDIReturnValue_t CNC_WIDI_setEDID_file(cncWIDIHandle_t handle, unsigned char* edidFile);

/**
 *  \brief          Sets HDMI plugged
 *  \param  [in]    handle
 *                      handler for cncnWIDI
 *  \return
 */

cncWIDIReturnValue_t CNC_WIDI_setPlugged(cncWIDIHandle_t handle);

/**
 *  \brief          Sets HDMI unplugged
 *  \param  [in]    handle
 *                      handler for cncnWIDI
 *  \return
 */

cncWIDIReturnValue_t CNC_WIDI_setUnplugged(cncWIDIHandle_t handle);

/**
 *  \brief          Redraws last image
 *  \param  [in]    handle
 *                      handler for cncnWIDI
 *  \return
 */

cncWIDIReturnValue_t CNC_WIDI_setRedrawLastImage(cncWIDIHandle_t handle);

/**
 *  \brief          Stops data flow 
 *  \param  [in]    handle
 *                      handler for cncnWIDI
 *  \return
 */

cncWIDIReturnValue_t CNC_WIDI_setDataFlowStop(cncWIDIHandle_t handle);

/**
 *  \brief          Starts data flow 
 *  \param  [in]    handle
 *                      handler for cncnWIDI
 *  \return
 */

cncWIDIReturnValue_t CNC_WIDI_setDataFlowStart(cncWIDIHandle_t handle);


/**
 *  \brief          Register a callback for a particular event 
 *  \param  [in]    handle
 *                      handler for cncWIDI
 *  \param  [in]    widi_event event for which to register the callback
 *  \param  [in]    func function to use for the callback
 *  \return         returns CNC_WIDI_FAILURE or CNC_WIDI_SUCCESS
 *   
 */

cncWIDIReturnValue_t CNC_WIDI_registerCallback(cncWIDIHandle_t handle, cncWIDIEvent_t widi_event, cncWIDICallback_t func);


// GG. USED FOR READING OVERSCAN AND CURSOR
cncWIDIReturnValue_t CNC_WIDI_getStatus(cncWIDIHandle_t handle, cncWIDIStatus_t* status);


int cnc_widi_l2_event_diconnected(char* name_id);

#ifdef __cplusplus
}

#endif

#endif // __CNC_WIDI_INTERFACE_H__
