/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Cavium, Inc.
 Cavium, Inc. reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium, Inc.

 ALL RIGHTS RESERVED.

 Copyright 2011-2012, Cavium, Inc.
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */


/**
 * @file cnc_wivu.h
 * @ingroup wd
 * The CNCWIVU library enables the development of applications using the WIVU protocol
*/

#ifndef __CNC_WIVU_INTERFACE_H__
#define __CNC_WIVU_INTERFACE_H__

#ifdef __cplusplus
extern "C" {
#endif

/* ============================ System Includes =========================== */
// NONE

/* ============================= User Includes ============================ */
#include "cnc_common_typedef.h"
#include "cnc_system.h"


#define CNC_WIVU_CEC_TYPE_NETHD 0
#define CNC_WIVU_IR_TYPE_NETHD 0
#define MAX_INFOFRAME_DATA 1024
#define MAX_SOFTWARE_VERSION_LEN 64
#define E_MAX_NICKNAME 64

#ifdef ARCH_CSM1900
 #define CSMARCH "1900"
#else
 #define CSMARCH "1800"
#endif

/* ======================== Enum declarations ========================= */
typedef enum _cncWIVUEvent_t {
    CNC_WIVU_EVENT_START,
    CNC_WIVU_EVENT_STOP,
    CNC_WIVU_EVENT_SEND_START,
    CNC_WIVU_EVENT_SEND_STOP,
    CNC_WIVU_EVENT_ENCODER_DISAPPEARED,
    CNC_WIVU_EVENT_HDMI_PLUGGED, // source HDMI plugged
    CNC_WIVU_EVENT_HDMI_UNPLUGGED, // source HDMI unplugged
    CNC_WIVU_EVENT_PLUG_CODE_OFFLINE_REQUEST,
    CNC_WIVU_EVENT_SOURCE_INPUT_CHANGED, // source HDMI input changed
    CNC_WIVU_EVENT_VIDEO_CAPTURE_CHANGED,
    CNC_WIVU_EVENT_INFOFRAME_CHANGED,
    CNC_WIVU_EVENT_SOURCE_HDMI_PLUG_EVENT,
    CNC_WIVU_EVENT_ALL,
    CNC_WIVU_ERROR_BADCOMMAND=-500,
} cncWIVUEvent_t;

typedef enum _cncWIVUBoolean_t{
    CNC_WIVU_TRUE,
    CNC_WIVU_FALSE
} cncWIVUBoolean_t;

typedef enum _cncWIVUReturnValue_t{
    CNC_WIVU_FAILURE = -1,
    CNC_WIVU_SUCCESS = 0
} cncWIVUReturnValue_t;

typedef enum _cncWIVUVideoType_t{
    CNC_WIVU_VID_TYPE_3984 = 0,
    CNC_WIVU_VID_TYPE_WFD = 2,
} cncWIVUVideoType_t;

typedef enum _cncWIVUAudioType_t{
    CNC_WIVU_AUD_TYPE_NETHD = 0,
    CNC_WIVU_AUD_TYPE_HDCP2_NETHD = 1,
    CNC_WIVU_AUD_TYPE_RTP_16BIT_LPCM = 2,
    CNC_WIVU_AUD_TYPE_TS_RFC2250_NETHD = 3,
    CNC_WIVU_AUD_TYPE_TS_RFC2250_16BIT_LPCM = 4,
    CNC_WIVU_AUD_TYPE_TS_RFC2250_16BIT_LPCM_LGE = 5,
    CNC_WIVU_AUD_TYPE_AAC = 6,
} cncWIVUAudioType_t;


/* ======================== Structs declarations ========================= */


typedef CNC_VOID* cncWIVUHandle_t;
/*
typedef struct _cncWIVUHandle_t {
    CNC_SINT32 wivu_instance_id;
    cncWIVUHnd_t pWivuHandle;
} cncWIVUHandle_t;
*/
typedef CNC_VOID (*cncWIVUCallback_t) (cncWIVUHandle_t, cncWIVUEvent_t);

/*
 * This has all the configuration parameters, that is used for initialization.
 */ 
typedef struct _cncWIVU_Config_t {
    cncWIVUVideoType_t  vidEncType;
    cncWIVUVideoType_t  vidDecType;
    cncWIVUAudioType_t  audEncType;
    cncWIVUAudioType_t  audDecType;
    CNC_SINT32  IR_BlastType;
    CNC_SINT32  IR_CaptureType;
    CNC_SINT32  CEC_BlastType;
    CNC_SINT32  CEC_CaptureType;
    CNC_UINT32  groupnumber;
    CNC_SINT32  sources;
    CNC_SINT32  sinks;
    CNC_SINT32  do_3D;
    CNC_CHAR	iFace[6];
    CNC_SINT32	discovery;
    CNC_SINT32  bCastEnable;
    CNC_UINT32  infoFrameLength;
    CNC_CHAR    infoFrameData[MAX_INFOFRAME_DATA];
    CNC_SINT32  inactivityTimeoutSecs;
    CNC_CHAR    nickname[E_MAX_NICKNAME];
} cncWIVUConfig_t;

typedef struct _cncWIVU_StartParams_t {
    cncWIVUVideoType_t  vidEncTypeStart;
    cncWIVUAudioType_t  audEncTypeStart;
    CNC_SINT32  UDPVidMSourcePort;
    CNC_SINT32  UDPVidMDestPort;
    CNC_SINT32  UDPAudMSourcePort;
    CNC_SINT32  UDPAudMDestPort;
    CNC_CHAR    ip[16];
   
} cncWIVUStartParams_t; 


/* ======================== Function declarations ========================= */
/**
 *  \brief          Initializes WIVU protocol
 *  \return         handler for cncWivu
 */
cncWIVUHandle_t CNC_WIVU_Create(CNC_VOID);

/**
 *  \brief          Deinitializes WIVU protocol
 *  \param  [in]    handle
 *                      handler for cncWivu
 *  \return
 */

cncWIVUReturnValue_t CNC_WIVU_Destroy(cncWIVUHandle_t handle);

/**
 *  \brief          Run the main thread 
 *  \param  [in]    handle
 *                      handler for cncWivu
 *  \return
 */

cncWIVUReturnValue_t CNC_WIVU_Start(cncWIVUHandle_t handle );

/**
 *  \brief          Stops main thread
 *  \param  [in]    handle
 *                      handler for cncWivu
 *  \return
 */

cncWIVUReturnValue_t CNC_WIVU_Stop(cncWIVUHandle_t handle);

/**
 *  \brief          Fill parameteres struct needed to start rtp 
 *  \param  [in]    handle
 *                      handler for cncWivu
 *  \param  [in]    p_params
 *                      start parameters struct 
 *  \return
 */

cncWIVUReturnValue_t CNC_WIVU_GetStartParams(cncWIVUHandle_t handle, cncWIVUStartParams_t *p_params);


/**
 *  \brief          Get the configuration for the WIVU connection
 *  \param  [in]    handle
 *                      handler for cncWivu
 *  \param  [out]   p_config
 *                      Configuration params.  
 *  \return
 */

cncWIVUReturnValue_t CNC_WIVU_GetConfig(cncWIVUHandle_t handle, cncWIVUConfig_t *p_config);

/**
 *  \brief          Get the configuration to the WIVU connection
 *  \param  [in]    handle
 *                      handler for cncWivu
 *  \param  [in]    p_config
 *                      Configuration params.  
 *  \return
 */

cncWIVUReturnValue_t CNC_WIVU_SetConfig(cncWIVUHandle_t handle, cncWIVUConfig_t *p_config);

/**
 *  \brief          Get the source input status
 *  \param  [in]    handle
 *                      handler for cncWivu
 *  \param  [out]   *pstate
 *                      Returns the state of the source input (0..N-1)
 *  \return
 */
cncWIVUReturnValue_t CNC_WIVU_GetSourceInput(cncWIVUHandle_t handle, CNC_SINT32 *pstate);


/**
 *  \brief          Get the source video capture resolution
 *  \param  [in]    handle
 *                      handler for cncWivu
 *  \param  [out]   *resolution
 *                      Returns the resolution used on the encoder side.
 *  \return
 */
cncWIVUReturnValue_t CNC_WIVU_GetSourceCaptureResolution(cncWIVUHandle_t handle, cncResolutionDetails_t *resolution);

/**
 *  \brief          Get the plug status
 *  \param  [in]    handle
 *                      handler for cncWivu
 *  \param  [out]   state
 *                      Returns the state of the WIVU connection (0 not plugged - 1 plugged)  
 *  \return
 */
cncWIVUReturnValue_t CNC_WIVU_GetPlugStatus(cncWIVUHandle_t handle, CNC_SINT32 *state);

/**
 *  \brief          Get the source software version
 *  \param  [in]    handle
 *                      handler for cncWivu
 *  \param  [out]   *sourceVersion
 *                      Returns the source's software version string
 *  \return
 */
cncWIVUReturnValue_t CNC_WIVU_GetSourceSoftwareVersion(cncWIVUHandle_t handle, CNC_CHAR sourceVersion[MAX_SOFTWARE_VERSION_LEN]);


/**
 *  \brief          Set the plug status
 *  \param  [in]    handle
 *                      handler for cncWivu
 *  \param  [out]   state
 *                      1 or 0 if plugged or not 
 *  \return
 */

cncWIVUReturnValue_t CNC_WIVU_SetPlugStatus(cncWIVUHandle_t handle, CNC_SINT32 state);

/**
 *  \brief          Set the HDMI as plugged
 *  \param  [in]    handle
 *                      handler for cncWivu
 *  \return
 */

cncWIVUReturnValue_t CNC_WIVU_HdmiSetPlugged(cncWIVUHandle_t handle);

/**
 *  \brief          Get the status of HDMI
 *  \param  [in]    handle
 *                      handler for cncWivu
 *  \return
 */

cncWIVUReturnValue_t CNC_WIVU_HdmiSetUnPlugged(cncWIVUHandle_t handle);

/**
 *  \brief          Set Plug Code to Offline Request
 *  \param  [in]    handle
 *                      handler for cncWivu
 *  \return
 */
cncWIVUReturnValue_t CNC_WIVU_SetPlugcodeOfflineRequest(cncWIVUHandle_t handle);

/**
 *  \brief          Set Plug Code to Normal
 *  \param  [in]    handle
 *                      handler for cncWivu
 *  \return
 */
cncWIVUReturnValue_t CNC_WIVU_SetPlugcodeNormal(cncWIVUHandle_t handle);

/**
 *  \brief          Set HDCP Supported indication on connected sink
 *  \param  [in]    handle
 *          [in]    0 = not supported, 1 = supported 
 *                      handler for cncWivu
 *  \return
 */
cncWIVUReturnValue_t CNC_WIVU_SetSinkHDCPSupported(cncWIVUHandle_t handle, CNC_SINT32 hdcpSupported);

/**
 *  \brief          Reconfigure NIC if any change has happened.
 *  \param  [in]    handle
                        handler for cncWivu
 *  \param  [in]    ifName
 *                      Interface to be configured  
 *  \return
 */

cncWIVUReturnValue_t CNC_WIVU_ReconfigureInterface(cncWIVUHandle_t handle, CNC_CHAR *ifName);

/**
 *  \brief          Checks for a new software version
 *  \param  [in]    handle
                        handler for cncWivu
 *  \param  [in]    version
 *                      software version 
 *  \return
 */


cncWIVUReturnValue_t CNC_WIVU_CheckSWUpgrade(cncWIVUHandle_t handle, CNC_CHAR *version);

/**
 *  \brief          Load EDID from file and set it
 *  \param  [in]     handle
 *                      handler for cncWivu
 *  \param  [in]    file
 *                   file with EDID  
 *  \return
 */

cncWIVUReturnValue_t CNC_WIVU_SetEDIDFile(cncWIVUHandle_t handle, CNC_CHAR *file);

/**
 *  \brief          Set the EDID
 *  \param  [in]     handle
 *                      handler for cncWivu
 *  \param  [in]    edid
 *                   array containing the EDID  
 *  \param  [in]    len 
 *                   EDID length
 *  \return
 */
cncWIVUReturnValue_t CNC_WIVU_SetEDID(cncWIVUHandle_t handle, CNC_CHAR *edid, CNC_SINT32 len);

/**
 *  \brief         Check what source is active
 *  \param [in]    handle
                     handler for cncWivu
 *  \param  [in]   currtime
 *                   Current time  
 *  \return
 */

cncWIVUReturnValue_t CNC_WIVU_CheckSourceActive(cncWIVUHandle_t handle, CNC_SINT32 currtime);


/**
 *  \brief          Get the aspect ratio for the video stream
 *  \param  [in]    handle
 *                      handler for cncWivu
 *  \param  [out]   *aspect_ratio
 *                      Returns the aspect ratio (see CSTVOUT_HDMI_ASPECT_RATIO)
 *  \return
 */
cncWIVUReturnValue_t CNC_WIVU_GetAspectRatio(cncWIVUHandle_t handle, CNC_BYTE* aspect_ratio);

/**
 *  \brief          Initializes WIVU protocol
 *  \param  [in]    handle
 *                      handler for cncWivu
 *  \param  [in]    wivu_event
 *                      Event which trigger the callback.
 *  \param  [in]    pfnCallback
 *                      Callback function 
 *  \return
 */

cncWIVUReturnValue_t CNC_WIVU_RegisterCallback(cncWIVUHandle_t handle, cncWIVUEvent_t wivu_event, cncWIVUCallback_t pfnCallback);


/**
 *  \brief         Get Source HDMI status
 *  \param [in]    handle
                     handler for cncWivu
 *  \param  [out]   plug
 *                   hdmi plug status: (0 not plugged - 1 plugged)  
 *  \return
 */

cncWIVUReturnValue_t CNC_WIVU_GetSourceHDMIStatus(cncWIVUHandle_t handle, CNC_SINT32* plug);



#ifdef __cplusplus
}

#endif

#endif // __CNC_WIVU_INTERFACE_H__
