#ifndef WIVUENC_API_H
#define WIVUENC_API_H

#ifdef __cplusplus
extern "C" {
#endif

#include "cnc_common_typedef.h"
#include "cnc_system.h"


#define MAX_NUM_PEERS 6
#define MAX_CHAR 64
#define MAX_INFOFRAME_DATA 1024
#define CNC_WIVUENC_DEFAULT_ENC_BITRATE 15000
#define E_DISPLAY_OPTIONS_SUPPORTED_FLAGS_MASK_SOURCE_REQUEST 0x0001    ///< flags for display_options_supported parameter
#define E_DISPLAY_OPTIONS_SUPPORTED_FLAGS_MASK_X_ROTATE       0x0002    ///< flags for display_options_supported parameter
#define E_DISPLAY_OPTIONS_SUPPORTED_FLAGS_MASK_X_SCALE        0x0004    ///< flags for display_options_supported parameter
#define E_DISPLAY_OPTIONS_SUPPORTED_FLAGS_MASK_Y_SCALE        0x0008    ///< flags for display_options_supported parameter
#define E_DISPLAY_OPTIONS_SUPPORTED_FLAGS_MASK_X_OFFSET       0x0010    ///< flags for display_options_supported parameter
#define E_DISPLAY_OPTIONS_SUPPORTED_FLAGS_MASK_Y_OFFSET       0x0020    ///< flags for display_options_supported parameter
#define E_MAX_NICKNAME 64 


#define CNC_WIVUENC_CEC_TYPE_NETHD 0
#define CNC_WIVUENC_IR_TYPE_NETHD 0

typedef enum _cncWIVUENC_ReturnValue_t
{
    CNC_WIVUENC_FAILURE = -1,
    CNC_WIVUENC_SUCCESS = 0
} cncWIVUENC_ReturnValue_t;

typedef enum 
{
    OFF = 0,
    ON = 1
} cncWIVUENC_Discovery_t;

typedef enum
{
    UNPLUGGED = 0,
    PLUGGED,
    HPD_UNPLUGGED = 10,
    HPD_PLUGGED
} cncWIVUENC_Plugged_t;

typedef enum _cncWIVUVideoType_t{           ///< Video type
    CNC_WIVUENC_VID_TYPE_3984 = 0,          ///< h264 over rtp
    CNC_WIVUENC_VID_TYPE_WFD = 2,           ///< h264 wfd
} cncWIVUENC_VideoType_t;

typedef enum _cncWIVUAudioType_t{           ///< Audio type
    CNC_WIVUENC_AUD_TYPE_NETHD = 0,
    CNC_WIVUENC_AUD_TYPE_HDCP2_NETHD = 1,
    CNC_WIVUENC_AUD_TYPE_RTP_16BIT_LPCM = 2,
    CNC_WIVUENC_AUD_TYPE_TS_RFC2250_NETHD = 3,
    CNC_WIVUENC_AUD_TYPE_TS_RFC2250_16BIT_LPCM = 4,
    CNC_WIVUENC_AUD_TYPE_TS_RFC2250_16BIT_LPCM_LGE = 5,
    CNC_WIVUENC_AUD_TYPE_AAC = 6,
} cncWIVUENC_AudioType_t;

typedef enum _cncWIVUEvent_t {
    CNC_WIVUENC_EVENT_SINK_CONNECTED,       ///< Connection requested from sink
    CNC_WIVUENC_EVENT_SINK_DISCONNECTED,    ///< sink disconnected
    CNC_WIVUENC_EVENT_SINK_PLUGGED,         ///< sink has been plugged
    CNC_WIVUENC_EVENT_SINK_UNPLUGGED,       ///< sink has been unplugged (remains connected)
    CNC_WIVUENC_EVENT_SINK_PLUG_CODE_NORMAL,
    CNC_WIVUENC_EVENT_SINK_PLUG_CODE_OFFLINE_REQUEST,
    CNC_WIVUENC_EVENT_SINK_PLUG_CODE_NO_HDCP_SUPPORT,
    CNC_WIVUENC_EVENT_SINK_EDID,            ///< sink EDID received
    CNC_WIVUENC_EVENT_SINK_IDR,             ///< received IDR request from sink
    CNC_WIVUENC_EVENT_SINK_PACKETS_LOSS,    ///< received Packets Loss from the sink
    CNC_WIVUENC_EVENT_SINK_DROP_FRAME_REQ,  ///< received a drop frame request
    CNC_WIVUENC_EVENT_ALL                   ///< register all events callback
} cncWIVUENC_Event_t;

typedef CNC_VOID* cncWIVUENC_Hnd_t;

typedef CNC_VOID (*cncWIVUENC_Callback_t) (cncWIVUENC_Hnd_t, cncWIVUENC_Event_t);

typedef struct _cncWIVUENC_DisplayOptions_t {
    CNC_UINT16  Supported_Flags;        ///< [15:6]: Reserved, set to 0; set iff supported: [5]: Y_Offset, [4]: X_Offset, [3]: Y_Scale, [2] X_Scale, [1]: Rotate; [0]: Set iff request from Source to Sink
    CNC_SINT16  Rotate;                 ///< 2's complement clockwise rotation amount (around the center) in degrees.
    CNC_UINT16  X_Scale;                ///< Horizontal (width) scaling factor times 32768, from (1/32768)x to 2.0x (if 0xffff).  For example for overscan prevention.
    CNC_UINT16  Y_Scale;                ///< Vertical (height) scaling factor times 32768, from (1/32768)x to 2.0x (if 0xffff).  For example for overscan prevention.
    CNC_SINT16  X_Offset;               ///< 2's complement pixel offset for horizontal shifting to the right (if positive) or left (if negative).
    CNC_SINT16  Y_Offset;               ///< 2's complement pixel offset for vertical shifting down (if positive) or up (if negative).
} cncWIVUENC_DisplayOptions_t;

typedef struct _cncWIVUENC_AVConfig_t   ///< This structure is returned when a CNC_WIVUENC_EVENT_START is received
{
    CNC_SINT32  sink;                    ///< ID for the sink
    CNC_SINT32  videoDecPort;            ///< Received Video decoder port
    CNC_SINT32  videoDecType;            ///< Received Video decoder type
    CNC_SINT32  audioDecPort;            ///< Received Audio decoder port
    CNC_SINT32  audioDecType;            ///< Received Audio decoder type
    CNC_CHAR    IP[16];                  ///< Decoder IP
    CNC_UINT8*  EDID;
    CNC_SINT32  EDID_len;
    CNC_SINT32  sinkPluggedStatus;
    CNC_SINT32  plugged;
    CNC_CHAR    nickname[E_MAX_NICKNAME];
} cncWIVUENC_AVConfig_t;

typedef struct _cncWIVUENC_Config_t
{
    CNC_SINT32 groupNumber;             ///< WIVU GROUPNUMBER constant
    CNC_SINT32 broadCast;               ///< 0 if multicast is used, 1 if broadcast
    CNC_CHAR interface[8];              ///< net interface
    CNC_CHAR nickName[MAX_CHAR];        ///< NickName
    CNC_CHAR hostName[MAX_CHAR];        ///< HostName
    CNC_SINT32 sinks;                   ///< Max number of simultaneous sinks
    CNC_SINT32 sources;                 ///< Max number of sources
    cncWIVUENC_VideoType_t videoType;   ///< Encoder video codec
    CNC_SINT32 videoPort;               ///< If discovery == 0 then use this video port.
    cncWIVUENC_DisplayOptions_t displayOptions; ///< Encoder display options
    CNC_SINT32 bitrate;                 ///< Encoding Bitrate
    cncWIVUENC_AudioType_t audioType;   ///< Encoder audio codec
    CNC_CHAR UUID[40];                  ///< UUID string
    CNC_SINT32 discovery;               ///< Enable discovery
    CNC_SINT32 discoveryMode;           ///<
    CNC_SINT32 heartbeatSeconds;        ///< Heartbeat interval in seconds.
    CNC_SINT32 timeoutSink;             ///< Timeout to consider that a sink without responding has been disconnected
    cncResolutionDetails_t resolution;  ///< Capture resolution
    CNC_UINT32 infoFrameLength;         ///< Infoframe length in BYTES
    CNC_CHAR infoFrameData[MAX_INFOFRAME_DATA]; ///< Infoframe data array    
} cncWIVUENC_Config_t;

typedef struct _cncWIVUENC_Peer_t
{
    CNC_UINT8 mac[6];
} cncWIVUENC_Peer_t;

typedef struct _cncWIVUENC_PeerList_t
{
    cncWIVUENC_Peer_t peerList[MAX_NUM_PEERS];
} cncWIVUENC_PeerList_t;

/** Create a wivutx object an initialize it
 *
 * @return returns handle to wivutx object
 */
cncWIVUENC_Hnd_t         CNC_WIVUENC_create(CNC_VOID); ///< Returns NULL if could not allocate
/** Destroy wivutx object, deallocating it
 *
 * @param handle: handle to wivutx object
 * @return returns CNC_WIVUENC_FAILURE or CNC_WIVUENC_SUCCESS
 */
cncWIVUENC_ReturnValue_t CNC_WIVUENC_destroy(cncWIVUENC_Hnd_t handle);
/** Starts Wivu
 *
 * @param handle: handle to wivutx object
 * @return returns CNC_WIVUENC_FAILURE or CNC_WIVUENC_SUCCESS
 */
cncWIVUENC_ReturnValue_t CNC_WIVUENC_start(cncWIVUENC_Hnd_t handle);

/** Plugs a particular sink. 
 *
 * @param handle: handle to wivutx object
 * @param sink: sink to plug
 * @return returns CNC_WIVUENC_FAILURE or CNC_WIVUENC_SUCCESS
 */
cncWIVUENC_ReturnValue_t CNC_WIVUENC_plugSink(cncWIVUENC_Hnd_t handle, CNC_SINT32 sink);

/** Unplugs a particular sink. 
 *
 * @param handle: handle to wivutx object
 * @param sink: sink to unplug
 * @return returns CNC_WIVUENC_FAILURE or CNC_WIVUENC_SUCCESS
 */
cncWIVUENC_ReturnValue_t CNC_WIVUENC_unplugSink(cncWIVUENC_Hnd_t handle, CNC_SINT32 sink);

/** Stops WIVUENC completely. Closes all connections and stops WIVUENC thread.
 *
 * @param handle: handle to wivutx object
 * @return returns CNC_WIVUENC_FAILURE or CNC_WIVUENC_SUCCESS
 */
cncWIVUENC_ReturnValue_t CNC_WIVUENC_stop(cncWIVUENC_Hnd_t handle);

/** Set wivutx object configuration
 *
 * @param handle: handle to wivutx object
 * @param config: configuration struct to be applied
 * @return returns CNC_WIVUENC_FAILURE or CNC_WIVUENC_SUCCESS
 */
cncWIVUENC_ReturnValue_t CNC_WIVUENC_setConfig(cncWIVUENC_Hnd_t handle, const cncWIVUENC_Config_t *config);
/** Get wivutx object configuration
 *
 * @param handle: handle to wivutx object
 * @param config: configuration struct where the current configuration is going to be saved
 * @return returns CNC_WIVUENC_FAILURE or CNC_WIVUENC_SUCCESS
 */
cncWIVUENC_ReturnValue_t CNC_WIVUENC_getConfig(cncWIVUENC_Hnd_t handle, cncWIVUENC_Config_t *config);
/** Callbacks registering
 *
 * @param handle: handle to wivutx object
 * @param event: event to be registered
 * @param pfnCallback: callback function to be registered to that event
 * @return returns CNC_WIVUENC_FAILURE or CNC_WIVUENC_SUCCESS
 */
cncWIVUENC_ReturnValue_t CNC_WIVUENC_registerCallback(cncWIVUENC_Hnd_t handle, cncWIVUENC_Event_t event, cncWIVUENC_Callback_t pfnCallback);
/** Sets the HDMI status that we will transmit to the sinks
 *
 * @param handle: handle to wivutx object
 * @param plugged: 0 if the HDMI is unplugged 1 if it is plugged
 * @return returns CNC_WIVUENC_FAILURE or CNC_WIVUENC_SUCCESS
 */
cncWIVUENC_ReturnValue_t CNC_WIVUENC_setSourceHDMIStatus(cncWIVUENC_Hnd_t handle, int plugged);
/** Gets the status of a particular sink
 *
 * @param handle: handle to wivutx object
 * @param plugged: 0 if the HDMI is unplugged 1 if it is plugged
 * @return returns CNC_WIVUENC_FAILURE or CNC_WIVUENC_SUCCESS
 */
cncWIVUENC_ReturnValue_t CNC_WIVUENC_getSinkStatus(cncWIVUENC_Hnd_t handle, int sink, cncWIVUENC_AVConfig_t* sinkStatus);

/** Function to reconfigure net interface, close old and open new tx/rx sockets
 *
 * @param handle: handle to wivutx object
 * @param interface: name of the interface to be used
 * @return returns CNC_WIVUENC_FAILURE or CNC_WIVUENC_SUCCESS
 */
cncWIVUENC_ReturnValue_t CNC_WIVUENC_reconfigureInterface(cncWIVUENC_Hnd_t handle, CNC_CHAR* interface);

/** Set Source InputSelect for HDMI input
 *
 * @param handle: handle to wivutx object
 * @param peerList: array of peers containing the MAC addresses of sinks connected to the same network
 * @return returns CNC_WIVUENC_FAILURE or CNC_WIVUENC_SUCCESS
 */
cncWIVUENC_ReturnValue_t CNC_WIVUENC_setSourceInputSelect(cncWIVUENC_Hnd_t handle, CNC_SINT32 sourceInputSelect);

cncWIVUENC_ReturnValue_t CNC_WIVUENC_setPlugcodeNormal(cncWIVUENC_Hnd_t handle);

cncWIVUENC_ReturnValue_t CNC_WIVUENC_setPlugcodeOfflineRequest(cncWIVUENC_Hnd_t handle);

cncWIVUENC_ReturnValue_t setNetHDParam(CNC_UINT32 stream, CNC_UINT32 index, CNC_CHAR *block, CNC_CHAR *type, CNC_CHAR *param, CNC_CHAR* value);

cncWIVUENC_ReturnValue_t setIrBlastParam(CNC_UINT32 stream, CNC_UINT32 index, CNC_CHAR *param, CNC_CHAR* value);

/*
        Initialization (typically in this order):
            CNC_WIVUENC_create
            CNC_WIVUENC_getConfig
            CNC_WIVUENC_setConfig
            CNC_WIVUENC_registerCallback
            CNC_WIVUENC_setSourceHDMIStatus (will be called also any time there is a HPD on the source)
            CNC_WIVUENC_start
         
        when a sink connects with us we receive the event:
           CNC_WIVUENC_EVENT_SINK_CONNECTED
           Here we will decide whether to plug or not by calling CNC_WIVUENC_plugSink
         
        If we decide to plug the sink on our end, whenever the sink is ready we will then receive:
            CNC_WIVUENC_EVENT_SINK_PLUGGED
            Here we will setup the encoder to start streaming.
         
        The decoder might unplug the connection at any time (maintaining connectivity) and then we will receive:
            CNC_WIVUENC_EVENT_SINK_UNPLUGGED
            We will get this also if the sink HDMI is disconnected. We can stop the encoding to that sink here. We will receive the Plugged event when the sink is ready to start streaming again.
         
        If the decoder completely disconnects (leaves the network, shuts down, stops sending everything packet messages…) we will get:
           CNC_WIVUENC_EVENT_SINK_DISCONNECTED
            We should disconnect the sink here also (assuming we did not do already when received the unplugged)
         
        On sink status change we will receive one of the following accordingly. The action expected is shown after the event:
            CNC_WIVUENC_EVENT_SINK_EDID: User will have to Update EDID on RX. For the HDMI unplug case, we should receive an EDID of 0’s here
            CNC_WIVUENC_EVENT_SINK_IDR: User will have to request an IDR to the encoder
         
        At any time we can request sink status with CNC_WIVUENC_getSinkStatus, this gives: HPD, EDID, IP, connected status of a sink.
         
        At any time you may call CNC_WIVUENC_unplugSink/CNC_WIVUENC_plugSink to disconnect/connect a particular sink.
*/


#ifdef __cplusplus
}
#endif

#endif
