/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Cavium Inc.
 Cavium Inc reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium Inc.

 ALL RIGHTS RESERVED.

 Copyright 2011-2013, Cavium Inc
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */

/**
 * Wrapper for browser bookmark operations
 * @file cnc_browserBookmarks.h
 * @ingroup web
 */

#ifndef __CNC_BROWSER_BOOKMARKS_H__
#define __CNC_BROWSER_BOOKMARKS_H__

/* ============================= User Includes ============================ */
#include "cnc_browserTypes.h"

#ifdef __cplusplus
extern "C"
{
#endif

struct CNC_BookmarkItem
{
    CNC_BrowserString_t title;          /**< The title of this website */
    char* url;                            /**< The URL of this website */
};

typedef struct CNC_BookmarksList
{
    int size;                              /**< The number of items */
    void (*release) (struct CNC_BookmarksList *list);  /**< The release method to be used by the application */
    struct CNC_BookmarkItem** items;     /**< The list of stored bookmarks */

} CNC_BookmarksList_t;


/**
 * @brief Add a new bookmark
 */
void CNC_Browser_AddBookmark(const char* url, CNC_BrowserString_t *title);

/**
 * @brief Edit a bookmark
 */
void CNC_Browser_EditBookmark(const char* newurl, CNC_BrowserString_t *title, const char *oldUrl);

/**
 * @brief Returns the list of bookmarks
 */
CNC_BookmarksList_t* CNC_Browser_GetBookmarkItems(void);


/**
 * @brief Returns a range of bookmarks
 */
CNC_BookmarksList_t* CNC_Browser_GetBookmarkItemsRange(int from, int to);


/**
 * @brief Delete a single bookmark
 */
void CNC_Browser_DeleteBookmark(const char *url);


/**
 * @brief Delete all bookmarks
 */
void CNC_Browser_DeleteAllBookmarks(void);


/**
 * @brief Save bookmarks to disk
 */
void CNC_Browser_SaveBookmarks(void);


#ifdef __cplusplus
}
#endif

#endif //__CNC_BROWSER_BOOKMARKS_H__
