/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Cavium Inc.
 Cavium Inc reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium Inc.

 ALL RIGHTS RESERVED.

 Copyright 2011-2013, Cavium Inc
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */


/**
 * Wrapper for browser common operations
 * @file cnc_browserCommon.h
 * @ingroup web
 */

#ifndef __CNC_BROWSER_COMMON_H__
#define __CNC_BROWSER_COMMON_H__

/* ============================= User Includes ============================ */
#include "cnc_browserTypes.h"
#include <stdio.h>

#ifdef __cplusplus
extern "C"
{
#endif


/**
 * @brief Allocates \a size bytes and returns a pointer to the allocated memory.
 * @param size Number of bytes to allocate
 * @return Pointer to allocated memory
 */
void* CNC_Browser_MemAlloc(size_t size);


/**
 * @brief Allocates \a size bytes and returns a pointer to the allocated memory.  The memory is set to zero.
 * @param size Number of bytes to allocate
 * @return Pointer to allocated memory
 */
void* CNC_Browser_MemZeroedAlloc (size_t size);


/**
 * @brief Frees the memory pointed to by \a ptr which must have been returned by a previous call to CNC_Browser_MemAlloc or CNC_Browser_MemZeroedAlloc.
 * @param ptr Pointer to allocated memory
 */
void CNC_Browser_MemFree(void *ptr);


/**
 * @brief clean up the thread cache in the memory allocator
 */
void CNC_Browser_MemThreadCacheCleanup(void);


#ifdef __cplusplus
}
#endif

#endif /* __CNC_BROWSER_COMMON_H__ */

