/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Cavium Inc.
 Cavium Inc reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium Inc.

 ALL RIGHTS RESERVED.

 Copyright 2011-2013, Cavium Inc
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */


/**
 * Wrapper for browser download operations
 * @file cnc_browserDownload.h
 * @ingroup web
 */

/* ============================ System Includes =========================== */

/* ============================= User Includes ============================ */

#ifndef __CNC_BROWSER_DOWNLOAD_H__
#define __CNC_BROWSER_DOWNLOAD_H__

#include "cnc_browserFeatures.h"

#ifdef __cplusplus
extern "C"
{
#endif

typedef enum
{
    CNC_BrowserDownloadError_None,
    CNC_BrowserDownloadError_NotFound,
    CNC_BrowserDownloadError_Cancelled,
    CNC_BrowserDownloadError_TimeOut,
    CNC_BrowserDownloadError_Unknown,
} CNC_BrowserDownloadError_t;


typedef struct
{
	/**
	 * @brief the browser opened a download
	 * @param handle integer, >0 success, 0 failure
	 * @param uri from CNC_Browser_OpenDownload
     * @param context context data
	 * @return 1 to continue, 0 to stop
	 */
#ifdef CNC_BROWSER_ENABLE_NEW_API
	int (*onOpen)(int handle, const char* uri, void* context);
#else
	int (*onOpen)(int handle, const char* uri);
#endif

	/**
	 * @brief the browser received download response
	 * @param handle the return value from CNC_Browser_OpenDownload
	 * @param resUri the download uri, maybe different
	 * @param mime
	 * @param encoding
	 * @param httpStatusCode if HTTP, for other protocols, 0
	 * @param expectedContentLength -1 if unknown
     * @param context context data
	 * @return 1 to continue, 0 to stop
	 */
#ifdef CNC_BROWSER_ENABLE_NEW_API
	int (*onReceiveResponse)(int handle, const char* resUri, const char* mime, const char* encoding, int httpStatusCode, long long expectedContentLength, void* context);
#else
	int (*onReceiveResponse)(int handle, const char* resUri, const char* mime, const char* encoding, int httpStatusCode);
#endif

	/**
	 * @brief the browser received download data
	 * @param handle the return value from CNC_Browser_OpenDownload
	 * @param data
	 * @param length
     * @param context context data
	 * @return the number of bytes actually written
	 * 			if the written bytes are less than the length, then the remaining bytes
	 * 			are internally buffered, periodically, the browser will try to send it again.
	 * 			-1, for error, the handle is closed.
	 */
#ifdef CNC_BROWSER_ENABLE_NEW_API
	int (*onReceiveData)(int handle, const char* data, int length, void* context);
#else
	int (*onReceiveData)(int handle, const char* data, int length);
#endif

	/**
	 * @brief the browser finished download data
	 * @param handle the return value from CNC_Browser_OpenDownload
	 * @param context context data
	 * @return 1 to continue, 0 to stop
	 */
#ifdef CNC_BROWSER_ENABLE_NEW_API
	void (*onFinish)(int handle, void* context);
#else
	void (*onFinish)(int handle);
#endif

	/**
	 * @brief the browser failed download data
	 * @param handle the return value from CNC_Browser_OpenDownload
	 * @param reason
     * @param context context data
	 * @return 1 to continue, 0 to stop
	 */
#ifdef CNC_BROWSER_ENABLE_NEW_API
	void (*onFail)(int handle, CNC_BrowserDownloadError_t error, void* context);
#else
	void (*onFail)(int handle, CNC_BrowserDownloadError_t error);
#endif
} CNC_BrowserDownloadClient_t;



/**
 * @brief ask the browser to open a resource and start downloading
 * @param uri
 * @param client
 * @param context context data
 */
#ifdef CNC_BROWSER_ENABLE_NEW_API
void CNC_Browser_OpenDownload(const char* uri, CNC_BrowserDownloadClient_t* client, void* context);
#else
void CNC_Browser_OpenDownload(const char* uri, CNC_BrowserDownloadClient_t* client);
#endif

/**
 * @brief ask the browser to stop a resource download
 * @param handle
 */
void CNC_Browser_StopDownload(int handle);


#ifdef __cplusplus
} //extern "C"
#endif

#endif /*__CNC_BROWSER_DOWNLOAD_H__*/
