/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Cavium Inc.
 Cavium Inc reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium Inc.

 ALL RIGHTS RESERVED.

 Copyright 2011-2013, Cavium Inc
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */


/**
 * Wrapper for browser graphics operations
 * @file cnc_browserGfx.h
 * @ingroup web
 */

#ifndef __CNC_BROWSER_GFX_H__
#define __CNC_BROWSER_GFX_H__

/* ============================= User Includes ============================ */
#include "cairo/cairo.h"
#include "cnc_browserFeatures.h"
#include "cnc_browserTypes.h"

#ifdef __cplusplus
extern "C"
{
#endif

/**
 * CNC Browser surface types.
 */
typedef enum
{
    CNC_BrowserSurfaceType_forUnknown,
    CNC_BrowserSurfaceType_forImage,
    CNC_BrowserSurfaceType_forView,
    CNC_BrowserSurfaceType_forPlugin,
    CNC_BrowserSurfaceType_forCanvas,
} CNC_BrowserSurfaceType_e;

/*
 * main browser GFX surface
 */
typedef struct
{
	/*create a Cairo surface with the specified width and height*/
	cairo_surface_t* (*create)(void* platform_data, int width, int height);

	/*painting begain, can be null*/
    void (*paint_begin)(void* platform_data);

    /*copy one area to another to support fast scroll*/
    void (*scroll_blit)(void* platform_data, CNC_BrowserRect_t *srcRect, CNC_BrowserRect_t *dstRect);

    /*the surface rectangle is updated*/
    void (*blit)(void* platform_data, CNC_BrowserRect_t rects[], int nRect);

	/*painting end, can be null*/
    void (*paint_end)(void* platform_data);

	/*destroy the cairo surface*/
	void (*destroy)(void* platform_data, cairo_surface_t* cairoSurface);

	/*draw a frame hightlight on a multi-frame content, can be null*/
	void (*drawFrameHightlight)(void* platform_data, CNC_BrowserRect_t* rect, cairo_t* cr);

#ifdef CNC_BROWSER_ENABLE_GFX_ACCELERATION
	unsigned int (*pitch_alignment)(void);

	/* create an accelerated Cairo surface with the specified width, height and data
	 can return NULL*/
	cairo_surface_t* (*create_for_data)(
			void* platform_data,
			unsigned char *data,
			cairo_format_t format,
		    int			   width,
		    int			   height,
		    int			   stride,
		    CNC_BrowserSurfaceType_e purpose);

	/* create an accelerated Cairo surface with the specified width and height
	 can return NULL*/
	cairo_surface_t* (*create_for_size)(
			void* platform_data,
			cairo_format_t format,
		    int			   width,
		    int			   height,
		    CNC_BrowserSurfaceType_e purpose);
#endif

	/**< Pointer to platform specific data */
	void* platform_data;

#ifdef CNC_BROWSER_ENABLE_GFX_ACCELERATION
	/* get the surface pixal format */
	cairo_format_t (*get_format)(void* platform_data, cairo_surface_t *surface);

	/*
	 * get the surface size
	 * @return 0 for failure, otherwise success
	 */
	int (*get_size)(void* platform_data, cairo_surface_t* surface, int* ret_width, int* ret_height);

	/*
	 * lock the surface for directly read/write
	 * @return 0 for failure, otherwise success
	 */
	int (*lock)(void* platform_data, cairo_surface_t* surface, void** ret_ptr, int* ret_pitch);

	/*
	 * unlock the surface after directly read/write
	 */
	void (*unlock)(void* platform_data, cairo_surface_t* surface);

	/*
	 *
	 */
	int (*unused_1)(void* platform_data, cairo_surface_t* surface);

	/*
	 *
	 */
	int (*unused_2)(void* platform_data, cairo_surface_t* surface);

	/*
	 *
	 */
	int (*unused_3)(void* platform_data, cairo_surface_t* surface);

	/*
	 *
	 */
	int (*unused_4)(void* platform_data, cairo_surface_t* surface);

#endif
} CNC_BrowserSurface_t;


/**
 * enable/disable the browser's painting
 *
 * @param enabled
 * 		0, disable the Gfx, after the call, the browser stopped further painting into the Gfx buffer
 * 		otherwise, enable the Gfx, the browser will shedule a repaint
 */
void CNC_Browser_SetGfxEnabled(int enabled);

#ifdef __cplusplus
}
#endif

#endif /*__CNC_BROWSER_GFX_H__*/
