/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Cavium Inc.
 Cavium Inc reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium Inc.

 ALL RIGHTS RESERVED.

 Copyright 2011-2013, Cavium Inc
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */

/**
 * Wrapper for Browser History.
 * @file cnc_browserHistory.h
 * @ingroup web
 */

#ifndef __CNC_BROWSER_HISTORY_H__
#define __CNC_BROWSER_HISTORY_H__

/* ============================= User Includes ============================ */
#include "cnc_browserTypes.h"

#ifdef __cplusplus
extern "C"
{
#endif

struct CNC_HistoryItem
{
    CNC_BrowserString_t title;             /**< The title of this website */
    char* url;                             /**< The URL of this website */
};

typedef struct CNC_HistoryList
{
    int size;                              /**< The number of items */
    int current;                           /**< The index of current loaded website */
    void (*release) (struct CNC_HistoryList *list);  /**< The release method to be used by the application */
    struct CNC_HistoryItem** items;      /**< The list of visited website  history */

} CNC_HistoryList_t;


/**
 * Enable or disable browsing history (session and global)
 * @param enabled
 */
void CNC_Browser_SetHistoryEnabled(int enabled);


/**
 * Set number of global history items
 * @param size Number of items to store
 */
void CNC_Browser_SetHistorySize(int size);


/**
 * Clear the browsr global history.
 */
void CNC_Browser_ClearHistory(void);


/**
 * Delete one item from the browser global history.
 * @param item 0 refers to the newest item
 */
void CNC_Browser_ClearHistoryItem(int item);


/**
 * Retrieve the list of websites that CNC_ browser has visited (global)
 */
CNC_HistoryList_t* CNC_Browser_GetHistoryItems(void);


/**
 * Save the global history to a file
 */
void CNC_Browser_SaveHistory(void);


/**
 * Clear the browsr current session history.
 */
void CNC_Browser_ClearSessionHistory(void);

/**
 * Set the browser session history capacity
 * @param size
 */
void CNC_Browser_SetSessionHistoryCapacity(int size);

/**
 * Retrieve the current browser session history
 */
CNC_HistoryList_t* CNC_Browser_GetSessionHistory(void);


#ifdef __cplusplus
}
#endif

#endif //__CNC_BROWSER_HISTORY_H__
