/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Cavium Inc.
 Cavium Inc reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium Inc.

 ALL RIGHTS RESERVED.

 Copyright 2011-2013, Cavium Inc
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */

/**
 * Wrapper for Browser Navigation operations
 * @file cnc_browserNavigate.h
 * @ingroup web
 */

#ifndef __CNC_BROWSER_NAVIGATE_H__
#define __CNC_BROWSER_NAVIGATE_H__

/* ============================= User Includes ============================ */
#include "cnc_browserTypes.h"

#ifdef __cplusplus
extern "C"
{
#endif

/**
 * Navigate to next sub frame.
 */
void CNC_Browser_NavigateFrame(void);

/**
 * Eanble/disable Frame Hightlight.
 * @param  enabled frame hightlight
 * @return
 */
void CNC_Browser_setFrameHighlightEnabled(int enabled);

/**
 * Enable/disable Frame navigation.
 * @param  enabled frame navigation
 * @return
 */
void CNC_Browser_setFrameNavEnabled(int enabled);

/**
 * Navigate to next sub frame.
 * @return 0 No subframe exist
 *         1 Has sub frame inside
 *         2 Has sub iframe inside
 *         3 Has sub frame and sub iframe inside
 */
int CNC_Browser_getSubFrameCount(void);

/**
 * set the color of the 2D navigation highlight color
 * @param color set high light color(R,G,B)
 * @return
 */
void CNC_Browser_setNavHighlightColor(CNC_BrowserARGB8888_t color);

/**
 * set the color of the 2D navigation highlight color
 * @param color set focus color(R,G,B)
 * @return
 */
void CNC_Browser_setNavFocusColor(CNC_BrowserARGB8888_t color);

/**
 * set the color of the 2D navigation highlight color
 * @param width set high light width
 * @return
 */
void CNC_Browser_setNavHighlightWidth(int width);

/**
 * remove the focus from the current 2D navigation node.
 * @return
 */
void CNC_Browser_focusedNavNodeBlur(void);

#ifdef __cplusplus
}
#endif

#endif /* __CNC_BROWSER_NAVIGATE_H__ */

