/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Cavium Inc.
 Cavium Inc reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium Inc.

 ALL RIGHTS RESERVED.

 Copyright 2011-2013, Cavium Inc
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */

/**
 * Wrapper for Browser plugin Events
 * @file cnc_browserPluginEvents.h
 * @ingroup web
 */

#ifndef __CNC_PLUGIN_EVENTS_H__
#define __CNC_PLUGIN_EVENTS_H__

#ifdef __cplusplus
extern "C"
{
#endif

typedef enum CNC_PluginErrorType_e
{
    CNC_PluginErrorType_None        = 0,
	CNC_PluginErrorType_Generic     = 1,
	CNC_PluginErrorType_OutOfMemory = 2,
} CNC_PluginErrorType;

typedef enum CNC_PluginWarningType_e
{
    CNC_PluginWarningType_None    = 0,
	CNC_PluginWarningType_Generic = 101,
} CNC_PluginWarningType;

typedef enum CNC_PluginEventType_e
{
    CNC_PluginMouseEventType      = 0,
	CNC_PluginKeyEventType        = 1,
	CNC_PluginPaintEventType      = 2,
	CNC_PluginFullscreenEventType = 3,
	CNC_PluginTextInputEventType  = 4,

	CNC_PluginBrowserSafeRectEventType     = 11, /* notify the browser safe rect, called after NPP_New */
	CNC_PluginBrowserUserLanguageEventType = 12, /* notify the browser user language, called after NPP_New */
	CNC_PluginBrowserUserCountryEventType  = 13, /* notify the browser user country, called after NPP_New */
	CNC_PluginBrowserUserTimezoneEventType = 14, /* notify the browser user timezone, called after NPP_New */

	CNC_PluginFocusEventType     = 21,
} CNC_PluginEventType;

typedef enum CNC_PluginMouseState_e
{
    CNC_PluginMouseButtonDown,
	CNC_PluginMouseButtonUp,
	CNC_PluginMouseButtonMove
}CNC_PluginMouseState;

typedef enum CNC_PluginMouseButton_e
{
    CNC_PluginMouseButtonLeft,
	CNC_PluginMouseButtonCenter,
	CNC_PluginMouseButtonRight,
	CNC_PluginMouseButtonWheelUp,
	CNC_PluginMouseButtonWheelDown
}CNC_PluginMouseButton;

typedef enum CNC_PluginKeyState_e
{
    CNC_PluginKeyDown,
	CNC_PluginKeyUp
}CNC_PluginKeyState;

typedef enum CNC_PluginModifierKey_e
{
    CNC_PluginAltKey 		= 1 << 0,
	CNC_PluginCtrlKey 		= 1 << 1,
	CNC_PluginMetaKey 		= 1 << 2,
	CNC_PluginShiftKey 		= 1 << 3,
}CNC_PluginModifierKey;


typedef struct CNC_PluginRect_s
{
    int16_t		top;
    int16_t		left;
    int16_t		bottom;
    int16_t		right;
} CNC_PluginRect;

typedef struct
{
	uint8_t 			type;
	uint8_t 			state;
	uint16_t 			x;
	uint16_t			y;
	int16_t 			xrel;
	int16_t				yrel;
	uint8_t 			which;
	uint8_t 			button;
} CNC_PluginMouseEvent;


/**
 * Keyboard event source, same as cnc_browserEvents.h
 */
enum CNC_PluginKeySource_e
{
    CNC_PluginKeySource_USB,
	CNC_PluginKeySource_IR,
};

typedef struct
{
	uint8_t 			type;
	uint8_t 			state;
	uint8_t 			scancode;
	uint16_t 			sym;
	uint16_t 			mod;
	uint16_t 			unicode;
	unsigned int        source;
} CNC_PluginKeyEvent;

/**
 * see Adobe wmode - Possible values: window, direct, opaque, transparent, gpu
 *
 * Before the paint event,
 *   1, if plug-in is opaque or full screen, or plug-in transparency is disabled,
 *      Then the surface (pixels) is cleared with color (0, 0, 0, 0);
 *   2, otherwise, the browser content under the plug-in is copied into pixels;
 *
 * After the paint event
 *      The plug-in surface (pixels) is copied on the browser surface
 */
typedef struct
{
	uint8_t 			type;
	uint8_t             bUseAccelBlit;

	/**
	 * if PaintWithClip is set to 0 (default) in browserconfig.prop, passing plug-in window bounds;
	 * otherwise, passing screen dirty rectangle relative to plug-in window;
	 */
	CNC_PluginRect 	    inval;

	uint16_t 			w;
	uint16_t			h;
	uint16_t 			xoffset;
	uint16_t			yoffset;
	uint16_t 			pitch;
	uint8_t 			depth;
	uint8_t* 			pixels;
} CNC_PluginPaintEvent;

typedef struct
{
	uint8_t 			type; //PluginFocusEventType
	uint8_t 			state; // 1, focus gained; 0, focus lost
} CNC_PluginFocusEvent;

typedef struct
{
	uint8_t type;    //PluginTextInputEventType
	char*	utf8Str; // null terminated utf8 string, could be null
} CNC_PluginTextInputEvent;

typedef struct
{
	uint8_t type; //PluginBrowserSafeRectEventType
	int     x;
	int     y;
	int     w;
	int     h;
} CNC_PluginBrowserSafeRectEvent;

/**
 * Language Codes (2 Letters), http://www.abbreviations.com/acronyms/LANGUAGES2L
 * for example, 'EN' english, 'FR' french, 'ES' Spanish
 */
typedef struct
{
	uint8_t type;    //PluginBrowserUserLanguageEventType
	const char* str; // null terminated string, could be null
} CNC_PluginBrowserUserLanguageEvent;

/**
 *
 */
typedef struct
{
	uint8_t type;    //PluginBrowserUserCountryEventType
	const char*	str; // null terminated string, could be null
} CNC_PluginBrowserUserCountryEvent;

/**
 *
 */
typedef struct
{
	uint8_t type;    //PluginBrowserUserTimezoneEventType
	const char* str; // null terminated string, could be null
} CNC_PluginBrowserUserTimezoneEvent;

typedef union
{
	uint8_t				               type;
	CNC_PluginMouseEvent	           mouse;
	CNC_PluginKeyEvent		           key;
	CNC_PluginPaintEvent	           paint;
	CNC_PluginFocusEvent               focus;
	CNC_PluginTextInputEvent           textInput;
	CNC_PluginBrowserSafeRectEvent     safeRect;
	CNC_PluginBrowserUserLanguageEvent language;
	CNC_PluginBrowserUserCountryEvent  country;
	CNC_PluginBrowserUserTimezoneEvent timezone;
} CNC_PluginEvent;

typedef enum CNC_SigmaVideoCodecType_e
{
    CNC_SigmaVideoCodecTypeNone,
	CNC_SigmaVideoCodecTypeH264,
	CNC_SigmaVideoCodecTypeSorenson,
	CNC_SigmaVideoCodecTypeOn2
} CNC_SigmaVideoCodecType;

#ifdef __cplusplus
}
#endif

#endif // __CNC_PLUGIN_EVENTS_H__
