/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Cavium Inc.
 Cavium Inc reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium Inc.

 ALL RIGHTS RESERVED.

 Copyright 2011-2013, Cavium Inc
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */

/**
 * Wrapper for Browser Plugins.
 * @file cnc_browserPlugins.h
 * @ingroup web
 */

#ifndef __CNC_BROWSER_PLUGINS_H__
#define __CNC_BROWSER_PLUGINS_H__

/* ============================= User Includes ============================ */
#include "npapi/npfunctions.h"
#include "cnc_browserPluginEvents.h"

#ifdef __cplusplus
extern "C"
{
#endif


typedef struct
{
	/*
	 *  plugin gained focus
	 *
	 * @param instance the NPP instance
	 *
	 */
	void (*focusGained)(NPP instance);

	/*
	 *  plugin gained focus
	 *
	 * @param instance the NPP instance
	 *
	 */
	void (*focusLost)(NPP instance);

	/*
	 *  NPP_New called
	 *
	 * @param instance the NPP instance created
	 * @param np_error NPError returned by NPP_New
	 *
	 */
	void (*npp_new)(NPP instance, int np_error);


	/*
	 *  NPP_Destroy called
	 *
	 * @param instance the NPP instance created
	 * @param np_error NPError returned by NPP_Destroy
	 *
	 */
	void (*npp_destroy)(NPP instance, int np_error);

	/*
	 * @brief Indicated if the plugin is in fullscreen mode
	 * @param fullscreen 1=fullscreen, 0=not fullscreen
	 */
	void (*fullscreen)(NPP instance, int fullscreen);

	/**
	 * @brief if plug-in is disabled, do selective load based on the document URL
	 *
	 * @param docProtocol, the connection protocol, for example, "http" or "https"
	 * @param docHost,		the server, for example, "google.com"
	 * @param docFile,		the resource, for example, "/doc/libc/libc_600.html"
	 *
	 * @return 0, keep disabled; otherwise, do loading
	 */
	int (*canLoad)(const char* docProtocol, const char* docHost, const char* docFile);

	/*
	 * send paint event to plug-in instance, similar to pluginFuncs->event(instance, event);
	 *
	 * @return pluginFuncs->event()
	 * 		The plug-in must return true from NPP_HandleEvent if it has handled the event and false if it has not.
	 */
	int (*paint)(NPP instance, const NPPluginFuncs* pluginFuncs, CNC_PluginEvent* event);

	/*
	 * send key event to plug-in instance, similar to pluginFuncs->event(instance, event);
	 *
	 * @return pluginFuncs->event()
	 * 		The plug-in must return true from NPP_HandleEvent if it has handled the event and false if it has not.
	 */
	int (*keyEvent)(NPP instance, const NPPluginFuncs* pluginFuncs, CNC_PluginEvent* event);

	/*
	 * set window on plug-in instance, similar to pluginFuncs->setwindow(instance, window);
	 *
	 * @return pluginFuncs->setwindow()
	 */
	NPError (*setwindow)(NPP instance, const NPPluginFuncs* pluginFuncs, NPWindow* window);

	/*
	 * not used now
	 */
	int (*future_x)(NPP instance);
	int (*future_y)(NPP instance);
	int (*future_z)(NPP instance);

} CNC_BrowserPluginCallbacks_t;


/**
 * @brief Register the list of Plugin call back functions
 * @param cb Callback structure
 */
void CNC_Browser_RegisterPluginCallbacks(CNC_BrowserPluginCallbacks_t *cb);

/**
 * @brief Set the maximum number of plugins allowed per page
 * @param max
 */
void CNC_Browser_SetMaxPlugins(int max);

/**
 * @brief Set the image to be displayed by a plugin when a codec is not supported
 * @param filename
 */
void CNC_Browser_SetPluginCodecNotSupportedImage(const char *filename);

#ifdef __cplusplus
}
#endif

#endif //__CNC_BROWSER_PLUGINS_H__
