/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Cavium Inc.
 Cavium Inc reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium Inc.

 ALL RIGHTS RESERVED.

 Copyright 2011-2013, Cavium Inc
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
 */

/**
 * Wrapper for Browser Settings.
 * @file cnc_browserSettings.h
 * @ingroup web
 */

#ifndef __CNC__BROWSER_SETTINGS_H__
#define __CNC__BROWSER_SETTINGS_H__

/* ============================= User Includes ============================ */
#include "cnc_browserTypes.h"

#ifdef __cplusplus
extern "C"
{
#endif

/**
 * @brief CNC Browser Setting types
 */
typedef enum
{
    CNC_BrowserSettingType_CACHESIZE = 1,					/*in KB */
    CNC_BrowserSettingType_MODE,							/*CNC_BrowserMode_t*/
    CNC_BrowserSettingType_INCREMENTALDRAW,				    /*0 disabled, otherwise enabled*/
    CNC_BrowserSettingType_IMAGEENABLED,					/*0 disabled, otherwise enabled*/
    CNC_BrowserSettingType_CSSENABLED,					    /*0 disabled, otherwise enabled*/
    CNC_BrowserSettingType_SCRIPTENABLED,					/*0 disabled, otherwise enabled*/
    CNC_BrowserSettingType_REPAINTDELAY,					/*in millisecond*/
    CNC_BrowserSettingType_DefaultFontSize,				    /*integer*/
    CNC_BrowserSettingType_HOMEURL,						    /*a string*/
    CNC_BrowserSettingType_HISTORY_ENABLED,				    /*0 disabled, otherwise enabled*/
    CNC_BrowserSettingType_HISTORY_FILENAME,				/*a string*/
    CNC_BrowserSettingType_POPUPENABLED,					/*0 disabled, otherwise enabled*/
    CNC_BrowserSettingType_COOKIES_ENABLED,				    /*0 disabled, otherwise enabled*/
    CNC_BrowserSettingType_COOKIES_FILENAME,				/*a string*/
    CNC_BrowserSettingType_MAX_COOKIE_NUMBER,				/*integer*/
    CNC_BrowserSettingType_MAX_COOKIE_SIZE,				    /*integer*/
    CNC_BrowserSettingType_MAX_COOKIES_PERDOMAIN,     	    /*integer*/
    CNC_BrowserSettingType_SSLENABLED,					    /*0 disabled, otherwise enabled*/
    CNC_BrowserSettingType_SSLVERSION,					    /*integer*/
    CNC_BrowserSettingType_NAVIGATIONENABLED,				/*0 disabled, otherwise enabled*/
    CNC_BrowserSettingType_SCROLLBARSVISIBLE,				/*0 invisible, otherwise visible*/
    CNC_BrowserSettingType_USERAGENT,						/*a string*/
    CNC_BrowserSettingType_HTTPPROXY_ENABLED,				/*0 disabled, otherwise enabled*/
    CNC_BrowserSettingType_HTTPPROXY_HOST,				    /*a string*/
    CNC_BrowserSettingType_HTTPPROXY_PORT,				    /*integer*/
    CNC_BrowserSettingType_HTTPSPROXY_HOST,				    /*a string*/
    CNC_BrowserSettingType_HTTPSPROXY_PORT,				    /*integer*/
    CNC_BrowserSettingType_EXCLUDE_PROXY_LIST,	     	    /*a string*/
    CNC_BrowserSettingType_HTTPVERSION,					    /*integer*/
    CNC_BrowserSettingType_OOMHANDLER_THRESHOLD,			/*in KB*/
    CNC_BrowserSettingType_OOMHANDLER_MEMORYCAP,			/*in KB*/
    CNC_BrowserSettingType_OOMHANDLER_OOMTIMERPERIOD,		/*in millisecond*/
    CNC_BrowserSettingType_UNDERLINELINKS,				    /*0 no underline links, otherwise yes*/
    CNC_BrowserSettingType_JP_HYPHENATION,				    /*0 disabled, otherwise yes*/
    CNC_BrowserSettingType_WORDWRAP,						/*0 disabled, otherwise yes*/
    CNC_BrowserSettingType_ROOTCAFILE,					    /*a string*/
    CNC_BrowserSettingType_COOKIES_NETTV_FILENAME,    	    /*a string*/
    CNC_BrowserSettingType_SERVERCERTFILE,				    /*a string*/
    CNC_BrowserSettingType_SSLCiphers,					    /*a string*/
    CNC_BrowserSettingType_BOOKMARKS_FILENAME,			    /*a string*/
    CNC_BrowserSettingType_BOOKMARKS_MAXTITLESIZE,		    /*integer*/
    CNC_BrowserSettingType_BOOKMARKS_MAXURLSIZE,			/*integer*/
    CNC_BrowserSettingType_BOOKMARKS_MAXNUMBER,			    /*integer*/
    CNC_BrowserSettingType_LOADREPAINTDELAY,				/*in millisecond*/
    CNC_BrowserSettingType_TargetWindowWarning,             /*0 disabled, otherwise enabled*/
    CNC_BrowserSettingType_MaxServerCertNum,                /*integer*/
    CNC_BrowserSettingType_OOMHANDLER_JSMmapCap,			/*in KB*/
    CNC_BrowserSettingType_viewSecureDialogEnabled,         /*0 disabled, otherwise enabled*/
    CNC_BrowserSettingType_leaveSecureForInsecureDialogEnabled, /*0 disabled, otherwise enabled*/
    CNC_BrowserSettingType_submitInsecureDialogEnabled,         /*0 disabled, otherwise enabled*/
    CNC_BrowserSettingType_viewSecureWithInsecureInformationDialogEnabled, /*0 disabled, otherwise enabled*/
    CNC_BrowserSettingType_DefaultEncoding,    				/*a string*/
    CNC_BrowserSettingType_DisabledRootCAFile,    			/*a string*/
    CNC_BrowserSettingType_Scrollbar_Thickness,    			/*integer*/
    CNC_BrowserSettingType_Scrollbar_BorderWidth,    		/*integer*/
    CNC_BrowserSettingType_Scrollbar_BackgroundColor, 		/*CNC_BrowserARGB8888_t*/
    CNC_BrowserSettingType_Scrollbar_BorderColor,    		/*CNC_BrowserARGB8888_t*/
    CNC_BrowserSettingType_Scrollbar_ThumbColor,    		/*CNC_BrowserARGB8888_t*/
    CNC_BrowserSettingType_RenderTheme_DefaultBorderColor, 	/*CNC_BrowserARGB8888_t*/
    CNC_BrowserSettingType_RenderTheme_ButtonBorderColor, 	/*CNC_BrowserARGB8888_t*/
    CNC_BrowserSettingType_RenderTheme_ButtonBackgroundColor,/*CNC_BrowserARGB8888_t*/
    CNC_BrowserSettingType_RenderTheme_ButtonCornerRadius, 	/*integer*/
    CNC_BrowserSettingType_RenderTheme_ButtonBorderWidth, 	/*integer*/
    CNC_BrowserSettingType_PageTransition_BackgroundColor,	/*CNC_BrowserARGB8888_t*/
    CNC_BrowserSettingType_fullScreenModeEnabled, 	        /*integer*/
    CNC_BrowserSettingType_Last,
} CNC_BrowserSettingType_e;

/**
 * @brief A structure to represent a single setting
 */
typedef struct CNC_BrowserSetting_s
{
    CNC_ParameterType_e type; /**< Type of parameter */

	union {
	    int integer; /**< Value is an integer */
	    float floatv; /**< Value is a float */
	    char* str; /**<  Value is a string */
	    CNC_BrowserARGB8888_t color;
	} value;

	void (*release) (struct CNC_BrowserSetting_s *);
} CNC_BrowserSetting_t;


/**
 * @brief Save the settings to disk.
 */
void CNC_Browser_SaveSettings(void);

/**
 * @brief A request to get a setting
 * For settings that contain more than one value, they will be converted to a comma seperated list of values.  For example, SAFERECT will contain the x, y, width, and height in a string format, comman seperated eq. "10,10,1200,720".
 * @param type
 * @param setting to hold the returned setting
 *
 * @return
 * 		1, if setting is retrieved successfully, value is populated;
 * 		0 failure, value is not touched
 */
int CNC_Browser_GetSetting(CNC_BrowserSettingType_e type, CNC_BrowserSetting_t* setting);


/**
 * @brief A request to set a setting
 * @param type
 * @param setting release is not checked.
 *
 * @return
 * 		1, if setting is updated successfully
 * 		0 failure, value is not touched
 */
int CNC_Browser_SetSetting(CNC_BrowserSettingType_e type, const CNC_BrowserSetting_t* setting);


/**
 * @brief Replaces the font fallback list.
 *
 * @param fontNames A null-terminated list of font names.
 */
void CNC_Browser_SetFallbackFonts(const char* fontNames[]);


#ifdef __cplusplus
}//end extern "C"
#endif

#endif /*__CNC__BROWSER_SETTINGS_H__*/
