/*
# COPYRIGHT NOTICE
#
#***************************************************************************

The following software code is the property of Cavium, Inc.
Cavium, Inc. reserves all rights in this software, including but
not limited to copyright and shall be vested with all rights incidental
thereto.

Rights granted to you by or under any license shall be limited to those
explicitly granted. You may not record, store, reproduce, reverse engineer,
decompile, inter-operate and/or transmit this software in any medium,
electronic or otherwise, without the express written consent of Cavium, Inc.

ALL RIGHTS RESERVED.

Copyright 2013, Cavium, Inc.
#***************************************************************************
#
# END OF COPYRIGHT NOTICE
#
*/

/*! ============================= File Header ================================
 *\brief     : This file contain APIs for storing data using some Database(Currently using sqLite).
 *\file      : cnc_webAppDatabaseMgr.h
 *\author    : Abhinandan Choudhury
 *\date      : 31-July-2013
 *\version   : 1
 *\attention :
 *\see       :
 * ========================================================================
 */

#ifndef _CNC_WEB_APP_DATABASE_MGR_H_
#define _CNC_WEB_APP_DATABASE_MGR_H_

#ifdef __cplusplus
extern "C"
{
#endif

/*============================ System Includes ============================== */
#include "stdio.h"

/*============================= User Includes =============================== */
#include "cnc_webapptypes.h"
#include "cnc_common_typedef.h"

/* ========================== Macro definitions ============================= */
#define DATABASE_MGR_CURRENT_SCHEMA_VERSION                 (1)
#define DATABASE_MGR_DEFAULT_APP_LIST_FILE_NAME             "/home/system/html/defaultApps.txt"
#define DATABASE_MGR_DEFAULT_MEDIA_LIST_FILE_NAME           "/home/system/html/defaultMedia.txt"

#define WEBAPP_MGR_MAX_SIZE_AMONG_THE_ABOVE                 CNC_WEBAPPMGR_PATH_SIZE    //!<  Max size of one of the structure element, between specified in cnc_webapptypes.h
#define CNC_WEB_APP_MAX_DOWNLOAD_REQUEST                    (20)
#define CNC_WEB_APP_STORAGE_PATH                            "/home/usb0/"


/*========================= Typedef Declarations ============================ */
/**
 *\brief    : Error code.
 */
typedef enum _cncWebAppDBMgrErrCode_t
{
    CNC_WEBAPP_DB_MGR_ERROR_NONE = 0,
    CNC_WEBAPP_DB_MGR_ERROR_INVALID_PARAM,
    CNC_WEBAPP_DB_MGR_ERROR_SQLDB_INIT,
    CNC_WEBAPP_DB_MGR_ERROR_SQLDB_CREATE,
    CNC_WEBAPP_DB_MGR_ERROR_MEMORY_ALLOC,
    CNC_WEBAPP_DB_MGR_ERROR_SQLDB_ERROR,
    CNC_WEBAPP_DB_MGR_ERROR_NO_MEMORY,
    CNC_WEBAPP_DB_MGR_ERROR_RESOURCE_NOT_AVAILABLE,
    CNC_WEBAPP_DB_MGR_ERROR_DATA_NOT_FOUND,

} cncWebAppDBMgrErrCode_t;

/**
 *\brief    : Application Type.
 */
typedef enum _cncWebAppDBAppInstallationTypes_t
{
    CNC_WEBAPP_DB_APP_NONE            = 0x01,                                   /*! User can't remove this app */
    CNC_WEBAPP_DB_APP_UPGRADEABLE     = 0x02,                                   /*! User/System Can upgrade the app  */
    CNC_WEBAPP_DB_APP_UNINSTALLABLE   = 0x04,                                   /*! User can Remove the app */

} cncWebAppDBAppInstallationTypes_t;

/**
 * \brief   :   Specify run time environment type.
 */
typedef enum _cncWebAppDBRunTimeType_t
{
    CNC_WEBAPP_DB_RUN_TIME_TYPE_NONE = 0,                                       /*! None */
    CNC_WEBAPP_DB_RUN_TIME_TYPE_NATIVE,                                         /*! Application will run in Native UI */
    CNC_WEBAPP_DB_RUN_TIME_TYPE_WEB,                                            /*! Application will run in Web Space */
    CNC_WEBAPP_DB_RUN_TIME_TYPE_WIDGET,                                         /*! Application will run as Widget */
    CNC_WEBAPP_DB_RUN_TIME_TYPE_BACKGROUND,                                     /*! Application will run in Background */

} cncWebAppDBRunTimeType_t;

/*========================= Private Declarations ============================ */
/* None */

/*====================== Global Function declarations ======================= */

/**************************** Init / Deint ************************************/
/**
 * \brief       : Initialize the Web storage manager.
 *                This will create DB and all required tables.
 *\return       : CNC_WEBAPP_DB_MGR_ERROR_NONE : on Success. \n
 *                Error code : On Failure.
 */
cncWebAppDBMgrErrCode_t CNC_WebApp_DatabaseMgrInit(void);

/**
 * \brief       : De-Initialize the Web storage manager.
 *                This will remove DB and all Tables created in Init.
 *\return       : CNC_WEBAPP_DB_MGR_ERROR_NONE : on Success. \n
 *                Error code : On Failure.
 */
cncWebAppDBMgrErrCode_t CNC_WebApp_DatabaseMgrDeInit(void);

/***************************** Default App ************************************/
// Below 2 functions (app_webStorageMgrCreateDefaultAppList and app_webStorageMgrDestroyDefaultAppList) function will be used only in-case of factory reset.
/**
 *\brief        : This function is used to create the default APP database, for pre-installed Apps. \n
 *                This function should be called once to create the Default database before starting application.
 *\param[in]    : p_fileName:
 *                  Location of config file which contain pre-installed app list.
 *\return       : CNC_WEBAPP_DB_MGR_ERROR_NONE : on Success. \n
 *                Error code : On Failure.
 */
cncWebAppDBMgrErrCode_t CNC_WebApp_DatabaseMgrCreateDefaultAppList(CNC_CHAR *p_fileName);

/**
 *\brief        : This function is used to destroy the default APP database, for pre-installed Apps. \n
 *                This function will erase all the entry of default table.
 *\return       : CNC_WEBAPP_DB_MGR_ERROR_NONE : on Success. \n
 *                Error code : On Failure.
 */
cncWebAppDBMgrErrCode_t CNC_WebApp_DatabaseMgrDestroyDefaultAppList();

/**
 *\brief        : This function is used to create list for pre-installed downloaded Media. \n
 *                This function should be called once to create the downloaded media list.
 *\param[in]    : p_fileName:
 *                  Location of config file which contain pre-installed media list.
 *\return       : CNC_WEBAPP_DB_MGR_ERROR_NONE : on Success. \n
 *                Error code : On Failure.
 */
cncWebAppDBMgrErrCode_t CNC_WebApp_DatabaseMgrCreateDownloadedMediaList(CNC_CHAR *p_fileName);

/**
 *\brief        : This function is used to get the number of Default app(s) available in the platform.
 *\param[out]   : p_noOfList:
 *                  Returns number of Default app present.
 *\return       : CNC_WEBAPP_DB_MGR_ERROR_NONE : on Success. \n
 *                Error code : On Failure.
 */
cncWebAppDBMgrErrCode_t CNC_WebApp_DatabaseMgrGetNumberOfDefaultAppList(CNC_SINT32 *p_noOfList);
/**
 *\brief        : This function is used to get all the available default app(s) in the Target.
 *                This is a synchronous function and it can take more time for larger list.
 *\attention    : User of this function should free the memory allocated by this function. caller should be aware that it can allocate all the memory available in the system depending on the list size.
 *\param[out]   : p_size:
 *                  Size of the array.
 *\param[out]   : pp_appList:
 *                  Returns array of structure with populated value.
 *                  It will allocate memory for this pointer.
 *                  User of this function should free the memory.
 *\return       : CNC_WEBAPP_DB_MGR_ERROR_NONE : on Success. \n
 *                Error code : On Failure.
 */
cncWebAppDBMgrErrCode_t CNC_WebApp_DatabaseMgrGetDefaultAppList(CNC_SINT32 *p_size, cncWebAppMgrApplication_t **pp_appList[]);
/**
 *\brief        : This function is used to get all the available default app(s) in the Target with specified range.
 *\attention    : User of this function should free the memory allocated by this function.
 *\param[in]    : start:
 *                  Start of index.
 *\param[in,out]: p_end:
 *                  [in]End of index.
 *                  [out]If it does not contain element till "*p_end" index, then it will update the p_end value to appropriate value.
 *\param[out]   : pp_appList:
 *                  Returns array of structure with populated value.
 *                  It will allocate memory for this pointer.
 *                  User of this function should free the memory.
 *\return       : CNC_WEBAPP_DB_MGR_ERROR_NONE : on Success. \n
 *                Error code : On Failure.
 */
cncWebAppDBMgrErrCode_t CNC_WebApp_DatabaseMgrGetDefaultAppListSet(CNC_SINT32 start, CNC_SINT32 *p_end, cncWebAppMgrApplication_t **pp_appList[]);
/**
 *\brief        : This function is used to Add an entry to Default app list.
 *\param[in]    : p_appList:
 *                  Pointer to new app details structure.
 *\return       : CNC_WEBAPP_DB_MGR_ERROR_NONE : on Success. \n
 *                Error code : On Failure.
 */
cncWebAppDBMgrErrCode_t CNC_WebApp_DatabaseMgrAddAppToDefaultAppList(cncWebAppMgrApplication_t *p_appList);

/**************************** Installed App ***********************************/
/**
 *\brief        : This function is used to get the number of installed app(s) available in the platform.
 *\param[out]   : p_noOfList:
 *                  Returns number of Installed app present.
 *\return       : CNC_WEBAPP_DB_MGR_ERROR_NONE : on Success. \n
 *                Error code : On Failure.
 */
cncWebAppDBMgrErrCode_t CNC_WebApp_DatabaseMgrGetNumberOfInstalledAppList(CNC_SINT32 *p_noOfList);
/**
 *\brief        : This function is used to get all the available installed app(s) in the Target.
 *                This is a synchronous function and it can take more time for larger list.
 *\attention    : User of this function should free the memory allocated by this function. Caller should be aware that it can allocate all the memory available in the system depending on the list size.
 *\param[out]   : p_size:
 *                  Size of the array.
 *\param[out]   : pp_appList:
 *                  Returns array of structure with populated value.
 *                  It will allocate memory for this pointer.
 *                  User of this function should free the memory.
 *\return       : CNC_WEBAPP_DB_MGR_ERROR_NONE : on Success. \n
 *                Error code : On Failure.
 */
cncWebAppDBMgrErrCode_t CNC_WebApp_DatabaseMgrGetInstalledAppList(CNC_SINT32 *p_size, cncWebAppMgrApplication_t **pp_appList[]);
/**
 *\brief        : This function is used to get all the available installed app(s) in the Target with specified range.
 *\attention    : User of this function should free the memory allocated by this function.
 *\param[in]    : start:
 *                  Start of index.
 *\param[in,out]: p_end:
 *                  [in]End of index.
 *                  [out]If it does not contain element till "*p_end" index, then it will update the p_end value to appropriate value.
 *\param[out]   : pp_appList:
 *                  Returns array of structure with populated value.
 *                  It will allocate memory for this pointer.
 *                  User of this function should free the memory.
 *\return       : CNC_WEBAPP_DB_MGR_ERROR_NONE : on Success. \n
 *                Error code : On Failure.
 */
cncWebAppDBMgrErrCode_t CNC_WebApp_DatabaseMgrGetInstalledAppListSet(CNC_SINT32 start, CNC_SINT32 *p_end, cncWebAppMgrApplication_t **pp_appList[]);

/**
 *\brief        : This function is used to Add an entry to installed app list.
 *\param[in]    : p_appList:
 *                  Pointer to new app details structure.
 *\return       : CNC_WEBAPP_DB_MGR_ERROR_NONE : on Success. \n
 *                Error code : On Failure.
 */
cncWebAppDBMgrErrCode_t CNC_WebApp_DatabaseMgrAddAppToInstalledAppList(cncWebAppMgrApplication_t *p_appList);

/**
 *\brief        : This function is used to remove the entry from installed app list.
 *\param[in]    : p_appName:
 *                  Name of the App which we want to remove from the installed list.
 *\return       : CNC_WEBAPP_DB_MGR_ERROR_NONE : on Success. \n
 *                Error code : On Failure.
 */
cncWebAppDBMgrErrCode_t CNC_WebApp_DatabaseMgrRemoveInstalledApp(CNC_CHAR *p_appName);

/******************************* Downloaded Media *****************************/
/**
 *\brief        : This function is used to get the number of downloaded Media available in the platform.
 *\brief[in]    : mediaType:
 *                  Media Type (Audio/Video).
 *\param[out]   : p_noOfList:
 *                  Returns number of Downloaded Video/Audio present.
 *\return       : CNC_WEBAPP_DB_MGR_ERROR_NONE : on Success. \n
 *                Error code : On Failure.
 */
cncWebAppDBMgrErrCode_t CNC_WebApp_DatabaseMgrGetNumberOfDownloadedMediaList(cncWebAppMgrMediaType_t mediaType, CNC_SINT32 *p_noOfList);
/**
 *\brief        : This function is used to get all media(A/V) available in the Target.
 *                This is a synchronous function and it can take more time for larger list.
 *\attention    : User of this function should free the memory allocated by this function. Caller should be aware that it can allocate all the memory available in the system depending on the list size.
 *\param[in]    : mediaType:
 *                  Type of Media (Audio/Video).
 *\param[out]   : p_size:
 *                  On return it will update the size of the array.
 *                  If it will fails to allocate memory for all the elements then it will return partial array.
 *                  User of this function should free the memory.
 *\param[out]   : pp_mediaList:
 *                  Returns array of structure with populated value.
 *                  It will allocate memory for this pointer.
 *                  User of this function should free the memory.
 *\return       : CNC_WEBAPP_DB_MGR_ERROR_NONE : on Success. \n
 *                Error code : On Failure.
 */
cncWebAppDBMgrErrCode_t CNC_WebApp_DatabaseMgrGetDownloadedMediaList(cncWebAppMgrMediaType_t mediaType, CNC_SINT32 *p_size, cncWebAppMgrDownloadedMediaListEntry_t **pp_mediaList[]);
/**
 *\brief        : This function is used to get some media available in the Target with specified range.
 *\attention    : User of this function should free the memory allocated by this function.
 *\param[in]    : mediaType:
 *                  Type of Media (Audio/Video).
 *\param[in]    : start:
 *                  Start of index. Index starts from 1.
 *\param[in,out]: p_end:
 *                  [in]End of index.
 *                  [out]If it does not contain element till "*p_end" index, then it will update the p_end value to appropriate value.
 *\param[out]   : pp_mediaList:
 *                  Returns array of structure with populated value.
 *                  It will allocate memory for this pointer.
 *                  User of this function should free the memory.
 *\return       : CNC_WEBAPP_DB_MGR_ERROR_NONE : on Success. \n
 *                Error code : On Failure.
 */
cncWebAppDBMgrErrCode_t CNC_WebApp_DatabaseMgrGetDownloadedMediaListSet(cncWebAppMgrMediaType_t mediaType, CNC_SINT32 start, CNC_SINT32 *p_end, cncWebAppMgrDownloadedMediaListEntry_t **pp_mediaList[]);
/**
 *\brief        : This function is used to Add an entry to Downloaded media list.
 *\param[in]    : p_media:
 *                  Pointer to new Media details structure.
 *\return       : CNC_WEBAPP_DB_MGR_ERROR_NONE : on Success. \n
 *                Error code : On Failure.
 */
cncWebAppDBMgrErrCode_t CNC_WebApp_DatabaseMgrAddMediaToDownloadedMediaList(cncWebAppMgrDownloadedMediaListEntry_t *p_media);
/**
 *\brief        : This function is used to remove the entry from downloaded Media list.
 *\param[in]    : p_avName:
 *                  Name of the Media which we want to remove from the Downloaded list.
 *\return       : CNC_WEBAPP_DB_MGR_ERROR_NONE : on Success. \n
 *                Error code : On Failure.
 */
cncWebAppDBMgrErrCode_t CNC_WebApp_DatabaseMgrRemoveDownloadedMedia(CNC_CHAR *p_avName);

/*************************** Install List App *********************************/
/**
 *\brief        : This function is used to get the number of app(s) to download and install.
 *\param[out]   : p_noOfList:
 *                  Returns number of app list available to install.
 *\return       : CNC_WEBAPP_DB_MGR_ERROR_NONE : on Success. \n
 *                Error code : On Failure.
 */
cncWebAppDBMgrErrCode_t CNC_WebApp_DatabaseMgrGetNumberOfInstallListApp(CNC_SINT32 *p_noOfList);
/**
 *\brief        : This function is used to get all the app(s) to be install.
 *                This is a synchronous function and it can take more time for larger list.
 *\attention    : User of this function should free the memory allocated by this function. Caller should be aware that it can allocate all the memory available in the system depending on the list size.
 *\param[out]   : p_size:
 *                  Size of the array.
 *\param[out]   : pp_appList:
 *                  Returns array of structure with populated value.
 *                  It will allocate memory for this pointer.
 *                  User of this function should free the memory.
 *\return       : CNC_WEBAPP_DB_MGR_ERROR_NONE : on Success. \n
 *                Error code : On Failure.
 */
cncWebAppDBMgrErrCode_t CNC_WebApp_DatabaseMgrGetAllInstallListApp(CNC_SINT32 *p_size, cncWebAppMgrDownloadListEntry_t **pp_appList[]);
/**
 *\brief        : This function is used to get the details of App in InstallList DB.
 *                  Caller of this function should free the memory.
 *\param[in]    : p_name:
 *                  name of the App.
 *\param[out]   : p_app:
 *                  Returns the structure if p_name matches with the DB.
 *                  NULL in case of no app to install.
 *\return       : CNC_WEBAPP_DB_MGR_ERROR_NONE : on Success. \n
 *                Error code : On Failure.
 */
cncWebAppDBMgrErrCode_t CNC_WebApp_DatabaseMgrGetInstallListApp(CNC_CHAR *p_name, cncWebAppMgrDownloadListEntry_t **pp_app);
/**
 *\brief        : This function is used to Add an entry to Install app queue list.
 *\param[in]    : p_app:
 *                  Pointer to new App details structure.
 *\return       : CNC_WEBAPP_DB_MGR_ERROR_NONE : on Success. \n
 *                Error code : On Failure.
 */
cncWebAppDBMgrErrCode_t CNC_WebApp_DatabaseMgrAddAppToInstallList(cncWebAppMgrDownloadListEntry_t *p_app);
/**
 *\brief        : This function is used to remove the entry from database which is downloaded completely.
 *\param[in]    : p_appName:
 *                  Name of the App which we want to remove from the Queue list.
 *\return       : CNC_WEBAPP_DB_MGR_ERROR_NONE : on Success. \n
 *                Error code : On Failure.
 */
cncWebAppDBMgrErrCode_t CNC_WebApp_DatabaseMgrRemoveFromInstallListApp(CNC_CHAR *p_appName);

/************************* Download List Media and APP ************************/

/**
 *\brief        : This function is used to update Download status.
 *\param[in]    : p_download:
 *                  Pointer to download details structure.
 *\return       : CNC_WEBAPP_DB_MGR_ERROR_NONE : on Success. \n
 *                Error code : On Failure.
 */
cncWebAppDBMgrErrCode_t CNC_WebApp_DatabaseMgrSetDownloadStatus(cncWebAppMgrDownloadListEntry_t *p_download);
/**
 *\brief        : This function is used to Update download status.
 *\param[in]    : p_download:
 *                  Pointer to download details structure.
 *\return       : CNC_WEBAPP_DB_MGR_ERROR_NONE : on Success. \n
 *                (Error code) or (p_download->status = -1): On Failure.
 */
cncWebAppDBMgrErrCode_t CNC_WebApp_DatabaseMgrGetDownloadStatus(cncWebAppMgrDownloadListEntry_t *p_download);

/*************************** Download List Media ******************************/
/**
 *\brief        : This function is used to get the number of downloaded Media to download from Queue database.
 *\param[out]   : p_noOfList:
 *                  Returns number of queue media to download.
 *\return       : CNC_WEBAPP_DB_MGR_ERROR_NONE : on Success. \n
 *                Error code : On Failure.
 */
cncWebAppDBMgrErrCode_t CNC_WebApp_DatabaseMgrGetNumberOfDownloadListMedia(CNC_SINT32 *p_noOfList);
/**
 *\brief        : This function is used to get all media(A/V) to download.
 *                This is a synchronous function and it can take more time for larger list.
 *\attention    : User of this function should free the memory allocated by this function. Caller should be aware that it can allocate all the memory available in the system depending on the list size.
 *\param[out]   : p_size:
 *                  On return it will update the size of the array.
 *                  If it will fails to allocate memory for all the elements then it will return partial array.
 *                  User of this function should free the memory.
 *\param[out]   : pp_mediaList:
 *                  Returns array of structure with populated value.
 *                  It will allocate memory for this pointer.
 *                  User of this function should free the memory.
 *\return       : CNC_WEBAPP_DB_MGR_ERROR_NONE : on Success. \n
 *                Error code : On Failure.
 */
cncWebAppDBMgrErrCode_t CNC_WebApp_DatabaseMgrGetAllDownloadListMedia(CNC_SINT32 *p_size, cncWebAppMgrDownloadListEntry_t **pp_mediaList[]);
/**
 *\brief        : This function is used to get the list of media in DownloadList queue. User should free the memory.
 *\param[in]    : p_name:
 *                  name of the Media.
 *\param[out]   : pp_media:
 *                  Returns the structure if p_name matches with the DB.
 *                  NULL in case of no media to download.
 *\return       : CNC_WEBAPP_DB_MGR_ERROR_NONE : on Success. \n
 *                Error code : On Failure.
 */
cncWebAppDBMgrErrCode_t CNC_WebApp_DatabaseMgrGetDownloadListMedia(CNC_CHAR *p_name, cncWebAppMgrDownloadListEntry_t **pp_media);
/**
 *\brief        : This function is used to Add an entry to Download media queue list.
 *\param[in]    : p_media:
 *                  Pointer to new Media details structure.
 *\return       : CNC_WEBAPP_DB_MGR_ERROR_NONE : on Success. \n
 *                Error code : On Failure.
 */
cncWebAppDBMgrErrCode_t CNC_WebApp_DatabaseMgrAddMediaToDownloadList(cncWebAppMgrDownloadListEntry_t *p_media);
/**
 *\brief        : This function is used to remove the entry from database which is downloaded completely.
 *\param[in]    : p_avName:
 *                  Name of the video which we want to remove from the Queue list.
 *\return       : CNC_WEBAPP_DB_MGR_ERROR_NONE : on Success. \n
 *                Error code : On Failure.
 */
cncWebAppDBMgrErrCode_t CNC_WebApp_DatabaseMgrRemoveFromDownloadListMedia(CNC_CHAR *p_avName);
/**
 *\brief        : This function is used to update downloaded size.
 *\param[in]    : p_download:
 *                  Pointer to Media / App details structure.
 *\return       : CNC_WEBAPP_DB_MGR_ERROR_NONE : on Success. \n
 *                Error code : On Failure.
 */
cncWebAppDBMgrErrCode_t CNC_WebApp_DatabaseMgrSetDownloadedSize(cncWebAppMgrDownloadListEntry_t *p_media);
/**
 *\brief        : This function is used to get downloaded size.
 *\param[in]    : p_download:
 *                  Pointer to Media / App details structure.
 *\return       : CNC_WEBAPP_DB_MGR_ERROR_NONE : on Success. \n
 *                Error code : On Failure.
 */
cncWebAppDBMgrErrCode_t CNC_WebApp_DatabaseMgrGetDownloadedSize(cncWebAppMgrDownloadListEntry_t *p_download);


#ifdef __cplusplus
}
#endif

#endif  // _CNC_WEB_APP_DATABASE_MGR_H_
