/*-----------------------------------------------------------------------
//
// Proprietary Information of Elliptic Technologies
// Copyright (C) 2001-2011, all rights reserved
// Elliptic Technologies, Inc.
//
// As part of our confidentiality  agreement, Elliptic Technologies and
// the Company, as  a  Receiving Party, of  this  information  agrees to
// keep strictly  confidential  all Proprietary Information  so received
// from Elliptic Technologies. Such Proprietary Information can be used
// solely for  the  purpose  of evaluating  and/or conducting a proposed
// business  relationship  or  transaction  between  the  parties.  Each
// Party  agrees  that  any  and  all  Proprietary  Information  is  and
// shall remain confidential and the property of Elliptic Technologies.
// The  Company  may  not  use  any of  the  Proprietary  Information of
// Elliptic Technologies for any purpose other  than  the  above-stated
// purpose  without the prior written consent of Elliptic Technologies.
//
// This information is further subject to the terms and conditions of
// the DTLA license agreement.
//-----------------------------------------------------------------------
//
// Project:
//
//  DTCP-IP
//
// Description:
//
//
// This file defined global interface DTCP structures  and functions.
//
//
//-----------------------------------------------------------------------
//
// Language:         C
//
//
// Filename:         $Source: /home/repository/cvsrep/dtcp_sdk/dtcp/include/dtcp.h,v $
// Current Revision: $Revision: 1.148 $
// Last Updated:     $Date: 2012/01/19 21:22:47 $
// Current Tag:      $Name: DTCP-IP-SDK-V53-OFF-THE-SHELF-07-FEB-2012 $
//
//
//-----------------------------------------------------------------------*/


#ifndef _DTCP_H_
#define _DTCP_H_
#include "elptypes.h"

#ifndef DTCP_DEVICES
#define DTCP_DEVICES 34
#endif


#define DTLA_CERTIFICATE_SIZE_MAX 100 // 40(EC-SIGNATURE) + 40(EC-PK) + 8 (ID) + 8(HEADER) + 4(EXTRA)

#define DTLA_CERTIFICATE_RESTR_SIZE (384/8)
#define DTLA_CERTIFICATE_FULL_SIZE  (704/8)


#define DTCP_OK                     (0)
#define DTCP_FAILED                 (-1)
#define DTCP_INVALID_ARGUMENT       (-2)
#define DTCP_INVALID_HANDLE         (-3)
#define DTCP_INVALID_MODE           (-4)
#define DTCP_INVALID_SIZE           (-5)
#define DTCP_NO_MEM                 (-6)
#define DTCP_NOT_INITIALIZED        (-7)
#define DTCP_ERROR_NOT_SUPPORTED    (-8)
#define DTCP_ERROR_NETWORK          (-9)
#define DTCP_WOULDBLOCK             (-10)
#define DTCP_DEVICE_LIMIT           (-11)
#define DTCP_ERROR_AKE              (-12)
#define DTCP_ERROR_EXCHANGE         (-13)
#define DTCP_DEVICE_REVOKED         (-14)
#define DTCP_NEED_UPDATE            (-15)
#define DTCP_NO_MEMORY              (-16)
#define DTCP_BUFFER_SMALL           (-17)
#define DTCP_BAD_AKE_PACKET         (-18)
#define DTCP_HTTP_CONTENT_LENGTH    (-19)
#define DTCP_HTTP_200_OK            (-20)
#define DTCP_HTTP_LENGTH_ERROR      (-21)
#define DTCP_HTTP_CONTENT_DONE      (-22)
#define DTCP_ERROR_BG_RTT           (-23)
#define DTCP_ERROR_MOVE             (-24)
#define DTCP_ERROR_UNAUTHENTICATED  (-25)
#define DTCP_ERROR_INVALID_SUBF     (-26)
#define DTCP_ERROR_NOT_IMPLEMENTED  (-27)
#define DTCP_BAD_PCP_PACKET         (-28)
#define DTCP_BAD_SRM_PACKET         (-29)
#define DTCP_INVALID_DATA           (-30)
#define DTCP_INVALID_VERSION        (-31)
#define DTCP_ERROR_DATA_BUFFER      (-32)
#define DTCP_PCP_INVALID_EMI        (-33)
#define DTCP_PCP_INVALID_LABEL      (-34)
#define DTCP_PCP_INVALID_CIPHER     (-35)
#define DTCP_MOVE_CAN_RESUME        (-36)
#define DTCP_DEVICE_EXPIRED         (-37)
#define DTCP_INVALID_CMD            (-38)
#define DTCP_ERROR_TBD              (-39)
#define DTCP_AKE_NO_SRM_LOADED      (-40)
#define DTCP_CP_KX_EXPIRED          (-41)
#define DTCP_ALREADY_INITIALIZED    (-42)
#define DTCP_INVALID_POINTER        (-43)
#define DTCP_INVALID_COOKIE         (-44)
#define DTCP_INVALID_ALIGNMENT      (-45)


// definition for random number generator
#define DTCP_RNG_F            0x0101
#define DTCP_RNG_R            0x0110
#define RANDOM_NUMBER_MAX_SIZE      20  //bytes, 160 bits
#define ESM_RNG_F_SEED_SIZE         20  //bytes, 160 bits
#define ESM_RNG_F_SEED_SIZE_BITS   160  //bits
#define ESM_RNG_F_IV_SIZE            1  // 64 bits
#define ESM_RNG_R_SEED_SIZE          8  //bytes,  64 bits
#define ESM_RNG_R_SEED_SIZE_BITS    64  //bits
#define ESM_RNG_R_IV_SIZE            8  // 64 bits



#define DTCP_STATE_FREE             0
#define DTCP_STATE_UNAUTHENTICATED  1
#define DTCP_STATE_AUTHENTICATED    2

#define DTCP_NC_UPDATE_TIME       1
#define DTCP_NC_UPDATE_DATA       2
#define DTCP_NC_UPDATE_128MB      (128*1024*1024)

#define DTCP_KX_EXPIRED_2_HOURS    (2 * 60 * 60) // seconds

#define AKE_MAX_PACKET 1500

// Device structure indicator
#define DEVICE_FREE       0   // unallocated / invalid structure
#define DEVICE_AKE_SINK   1
#define DEVICE_AKE_SOURCE 2
#define DEVICE_CP_RTP     3   // RTP Content engine
#define DEVICE_CP_HTTP    4   // HTTP Content engine

#define DEVICE_AKE_FULL       0x0010  // supports full authentication
#define DEVICE_AKE_RESTRICTED 0x0020  // supports restricted aythentication
#define DEVICE_AES128         0x0100  // supports AES 128

// Bit mas Capabilty
#define CAP_BRIDGE        0x0001
#define CAP_CIH           0x0002
#define CAP_NO_RESPONSE2  0x0004
#define CAP_PCPUR         0x0008
#define CAP_NCRTP         0x0010

                                                // PCPUR[0] PCPUR[1]
#define CAP_SOURCE_PCPUR_MODE_NO_INFO   htons(0x0000)  // 00000000 00000000
#define CAP_SOURCE_PCPUR_MODE_HAS_CCI   htons(0x4000)  // 01000000 00000000
#define CAP_SOURCE_PCPUR_MODE_NO_CCI    htons(0x8000)  // 10000000 00000000
#define CAP_SOURCE_PCPUR_MODE_RESERVED  htons(0xC000)  // 11000000 00000000

#define CAP_SOURCE_PCPUR_TYPE_AV        htons(0x0000)  // 00000000 00000000
#define CAP_SOURCE_PCPUR_TYPE_1_AUDIO   htons(0x1000)  // 00010000 00000000
#define CAP_SOURCE_PCPUR_TYPE_RESERVED1 htons(0x2000)  // 00100000 00000000
#define CAP_SOURCE_PCPUR_TYPE_RESERVED2 htons(0x3000)  // 00110000 00000000

#define CAP_SOURCE_PCPUR_APS_COPY_FREE  htons(0x0000)  // 00000000 00000000
#define CAP_SOURCE_PCPUR_APS_TYPE_1     htons(0x0400)  // 00000100 00000000
#define CAP_SOURCE_PCPUR_APS_TYPE_2     htons(0x0800)  // 00001000 00000000
#define CAP_SOURCE_PCPUR_APS_TYPE_3     htons(0x0C00)  // 00001100 00000000

#define CAP_SOURCE_PCPUR_ICT            htons(0x0200)  // 00000010 00000000

#define CAP_SOURCE_PCPUR_AST            htons(0x0180)  // 00000001 10000000  - DTLA NA 2010-06
#define CAP_SOURCE_PCPUR_DOT            htons(0x0140)  // 00000001 01000000  - DTLA NA 2010-06

#define DTCP_EXCHANGE_LABEL_M6 0
#define DTCP_EXCHANGE_LABEL_AES128 1

#define DEVICE_ID_SIZE_BITS 40
#define DEVICE_ID_SIZE  (DEVICE_ID_SIZE_BITS/8)

#define AES_KEY_SIZE_BITS 128
#define AES_KEY_SIZE      (AES_KEY_SIZE_BITS/8)
#define AES_IV_SIZE       16

#define CONF_KEY_R_SIZE   (64/8)

#define IV_KEY_SIZE_BITS  64
#define IV_SIZE       (IV_KEY_SIZE_BITS/8)

#define EXCH_KEY_CTRL       2
#define EXCH_KEY_SIZE_BITS  96
#define EXCH_KEY_SIZE       (EXCH_KEY_SIZE_BITS/8)


#define DTLA_AKE_R_KEYS   12 // DTLA assigns 12 keys for the restricted AKE
#define DTLA_AKE_F_KEY_SIZE_BITS  128
#define DTLA_AKE_F_KEY_SIZE       (DTLA_AKE_F_KEY_SIZE_BITS/8)
#define DTLA_AKE_R_KEY_SIZE_BITS  64
#define DTLA_AKE_R_KEY_SIZE       (DTLA_AKE_R_KEY_SIZE_BITS/8)
#define DTCP_KXM_LABEL_SIZE       1

#define DTCP_AKE_MV_FINALIZE_P_SIZE     8
#define DTCP_AKE_MV_FINALIZE_P_SIZE_BITS     64


#define DTLA_AKE_KEY_SIZE  MAX(DTLA_AKE_F_KEY_SIZE,DTLA_AKE_R_KEY_SIZE)

#define DTCP_RTT_MAX    7
#define DTCP_RTT_N_SIZE 2
#define DTCP_RTT_MAC_SIZE (80/8)
#define DTCP_KEYCONF_MAC_SIZE (80/8)

#define DTCP_NC_NONCE_SIZE          8
#define DTCP_UR_NONCE_SIZE          6


#define AKE_NO_CMD             0
#define AKE_AKE_CMD            1
#define AKE_BGRTT_CMD          2
#define AKE_MOVE_CMD           3
#define AKE_MOVE_CAP_EXCH_CMD  31
#define AKE_CONT_KEY_CONF_CMD  4
#define AKE_CAP_REQ_CMD        5
#define AKE_CAP_EXCH_CMD       6
#define AKE_CONT_KEY_REQ_CMD   7
#define AKE_BGRTT_RSP_CMD      8
#define AKE_CANCEL_MOVE_CMD    9
#define AKE_COMMIT_MOVE_CMD    10
#define AKE_NO_CHLG_AKE_CMD    11
#define AKE_NO_INIT_MOVE_CMD   12
#define AKE_STATUS_CMD         13
#define AKE_STATUS_IND         14
#define AKE_CAP_REQ_IND        15
#define AKE_CAP_EXCH_IND       16
#define AKE_CLOSED_IND         17
#define AKE_MV_CONT_KEY_CONF_CMD  18

#define AKE_READ_WAIT     0xFACE  // use default timeout
#define AKE_READ_POLL     0       // just check if socket has any pending packets

#define DTCP_AKE_RESPONSE_TIMOUT  5 //2 // Default timeout

#define DTCP_AKE_MOVE_TIMOUT    10 // for the move protocol take longer

//AKE Status CType values
#define DTCP_AKE_CTYPE_CONTROL          (unsigned char)0   // Control command
#define DTCP_AKE_CTYPE_STATUS           (unsigned char)1   // Status command
#define DTCP_AKE_CTYPE_SPECIFICINQUIRY  (unsigned char)2   // Specific Inquiry command
#define DTCP_AKE_CTYPE_NOTIFY           (unsigned char)3   // Notify command
#define DTCP_AKE_CTYPE_GENERALINQUIRY   (unsigned char)4   // General Inquiry command
#define DTCP_AKE_CTYPE_NOT_IMPLEMENTED  (unsigned char)8   // Not Implemented
#define DTCP_AKE_CTYPE_ACCEPTED         (unsigned char)9   // Accepted
#define DTCP_AKE_CTYPE_REJECTED         (unsigned char)10  // Rejected
#define DTCP_AKE_CTYPE_STABLE           (unsigned char)12  // Implemented/Stable


// Response values
#define DTCP_AKE_RESPONSE_NOTIMPLEMENTED    (unsigned char)0x8    // Not Implemented response
#define DTCP_AKE_RESPONSE_ACCEPTED          (unsigned char)0x9    // Accepted response
#define DTCP_AKE_RESPONSE_REJECTED          (unsigned char)0xA    // Rejected response
#define DTCP_AKE_RESPONSE_INTRANSITION      (unsigned char)0xB    // In Transition response
#define DTCP_AKE_RESPONSE_IMPLEMENTEDSTABLE (unsigned char)0xC    // Implemented/Stable response
#define DTCP_AKE_RESPONSE_CHANGED           (unsigned char)0xD    // Changed response
#define DTCP_AKE_RESPONSE_INTERIM           (unsigned char)0xF    // Interim response

#define DTCP_AKE_STATUS_CHECK         (unsigned char)0x0F
#define DTCP_AKE_STATUS_OK            (unsigned char)0x00
#define DTCP_AKE_NO_AUTHENTICATION    (unsigned char)0x01
#define DTCP_AKE_NO_ISOCHRO_OUTPUT    (unsigned char)0x02
#define DTCP_AKE_NO_POINT_TO_POINT    (unsigned char)0x03
#define DTCP_AKE_UNAVAILABLE          (unsigned char)0x04
#define DTCP_AKE_NO_AC                (unsigned char)0x05
#define DTCP_AKE_OTHER_ERROR          (unsigned char)0x07 // ANY OTHER ERROR
#define DTCP_AKE_NO_INFO              (unsigned char)0x0F

#define DTCP_AKE_NOT_IMPLEMENTED      (unsigned char)0xF0 // INDICATE NOT_IMPLEMENTED Response frame

#define DTCP_AKE_SUBFUN_DEPEND_SET        (unsigned char)0x01
#define DTCP_AKE_SUBFUN_DEPEND_RESERVED   (unsigned char)0x00
#define DTCP_AKE_SUBFUN_DEPEND_STATUS     (unsigned char)0xFF
#define DTCP_AKE_SUBFUN_DEPEND_SINK       (unsigned char)0x01
#define DTCP_AKE_SUBFUN_DEPEND_SOURCE     (unsigned char)0x00
#define DTCP_AKE_SUBFUN_DEPEND_NONE       (unsigned char)0x00

#define DTCP_AKE_SUBFUN_CHALLENGE       (unsigned char)0x01
#define DTCP_AKE_SUBFUN_RESPONSE        (unsigned char)0x02
#define DTCP_AKE_SUBFUN_EXCHANGE_KEY    (unsigned char)0x03
#define DTCP_AKE_SUBFUN_SRM             (unsigned char)0x04
#define DTCP_AKE_SUBFUN_RESPONSE2       (unsigned char)0x05
#define DTCP_AKE_SUBFUN_AKE_CANCEL      (unsigned char)0xC0
#define DTCP_AKE_SUBFUN_CONTENT_KEY_REQ (unsigned char)0x80
#define DTCP_AKE_SUBFUN_SET_DTCP_MODE   (unsigned char)0x81
#define DTCP_AKE_SUBFUN_CAPABILITY_REQ  (unsigned char)0x82
#define DTCP_AKE_SUBFUN_RTT_READY       (unsigned char)0x91
#define DTCP_AKE_SUBFUN_RTT_SETUP       (unsigned char)0x11
#define DTCP_AKE_SUBFUN_RTT_TEST        (unsigned char)0x12
#define DTCP_AKE_SUBFUN_CONT_KEY_CONF   (unsigned char)0x13
#define DTCP_AKE_SUBFUN_RTT_VERIFY      (unsigned char)0x92
#define DTCP_AKE_SUBFUN_RTT_BG_INITIATE (unsigned char)0x90

#define DTCP_AKE_SUBFUN_MV_INITIATE     (unsigned char)0xA0

#define DTCP_AKE_SUBFUN_CAPABILITY_EXCHANGE (unsigned char)0x20
#define DTCP_AKE_SUBFUN_MV_EXCHANGE_KEY   (unsigned char)0x21
#define DTCP_AKE_SUBFUN_MV_CANCEL       (unsigned char)0x28
#define DTCP_AKE_SUBFUN_MV_FINALIZE       (unsigned char)0x22
#define DTCP_AKE_SUBFUN_MV_COMPLETE       (unsigned char)0x23
#define DTCP_AKE_SUBFUN_MV_CONT_KEY_CONF    (unsigned char)0x24

#define DTCP_AKE_PROCEDURE_REST_AUTH      (unsigned char)(1)
#define DTCP_AKE_PROCEDURE_EN_REST_AUTH   (unsigned char)(1<<1)
#define DTCP_AKE_PROCEDURE_FULL_AUTH      (unsigned char)(1<<2)
#define DTCP_AKE_PROCEDURE_EX_FULL_AUTH   (unsigned char)(1<<3)

#define DTCP_AKE_EXCH_KEY_M6_COPY_NEVER   1     //Full or ext Full
#define DTCP_AKE_EXCH_KEY_M6_COPY_ONE     (1<<1)// Any auth
#define DTCP_AKE_EXCH_KEY_M6_COPY_NO_MORE (1<<2)// any auth
#define DTCP_AKE_EXCH_KEY_AES128          (1<<3)// ext full
#define DTCP_AKE_CIPHER_ALGORITHM_AES128  (1<<4)// 0001->(unsigned char)0x10;

#define DTCP_RTT_OKMSG 10
#define DTCP_RTT_MAC1 1
#define DTCP_RTT_MAC2 2
#define DTCP_RTT_MAC3 4
#define DTCP_RTT_MAC4 4
#define DTCP_RTT_MAC5 5
#define DTCP_RTT_MAC6 6
#define DTCP_RTT_MAC7 7

#define DTCP_AKE_CONTROL_COMMAND  (unsigned char)1
#define DTCP_AKE_CONTROL_RESPONSE (unsigned char)2
#define DTCP_AKE_STATUS_COMMAND   (unsigned char)3
#define DTCP_AKE_STATUS_RESPONSE  (unsigned char)4

#define DTCP_AKE_PROC_FULL_EX 0x08
#define DTCP_AKE_PROC_FULL    0x04
#define DTCP_AKE_PROC_RESTR   0x01
#define DTCP_AKE_PROC_REST_EN 0x02

#define DTCP_EMI_MODE_MAX   (unsigned char)0x0F
#define DTCP_EMI_MODE_A0    (unsigned char)0x0C
#define DTCP_EMI_MODE_B1    (unsigned char)0x0A
#define DTCP_EMI_MODE_B0    (unsigned char)0x08
#define DTCP_EMI_MODE_C1    (unsigned char)0x06
#define DTCP_EMI_MODE_C0    (unsigned char)0x04
#define DTCP_EMI_MODE_D0    (unsigned char)0x02
#define DTCP_EMI_MODE_NA    (unsigned char)0x00


#define DTCP_EC_KEY_SIZE                 20
#define DTCP_EC_SIGNATURE_DATA_SIZE      40
#define DTCP_EC_SIGNATURE_VERIFY_VALID    1
#define DTCP_EC_SIGNATURE_VERIFY_INVALID  0

#define DTCP_SRM_UPDATE_REMOTE 1
#define DTCP_SRM_UPDATE_LOCAL 2
#define DTCP_SRM_MATCH 3

#define DTCP_SRM_HEADER_SIZE                4

#define DTCP_SRM_FIRST_GEN                  0
#define DTCP_SRM_SECOND_GEN                 1

#define DTCP_SRM_SUPPORTED_GEN        DTCP_SRM_SECOND_GEN

#define DTCP_SRM_FIRST_GEN_MAX_SIZE         128
#define DTCP_SRM_SECOND_GEN_MAX_SIZE        1024

#define DTCP_SRM_CRL_MIN_SIZE               (DTCP_SRM_CRL_LENGTH_SIZE +DTCP_EC_SIGNATURE_DATA_SIZE +DTCP_SRM_CRL_ENTRYTYPE_SIZE + 1)  // 1(Aligned on 4)

#define DTCP_SRM_MIN_SIZE                   (DTCP_SRM_HEADER_SIZE + DTCP_SRM_CRL_MIN_SIZE )
#define DTCP_SRM_MAX_SIZE                   (DTCP_SRM_SECOND_GEN_MAX_SIZE )

#define DTCP_SRM_CRL_LENGTH_SIZE            2
#define DTCP_SRM_CRL_ENTRYTYPE_SIZE     1


#define DTCP_NONCE_RANGE  5

/*
 * DTCP State machine
*/
enum ake_ip_state_
{
  AKE_STATE_UNKNOWN=0,
  AKE_STATE_CHALLENGE,
  AKE_STATE_RESPONSE,
  AKE_STATE_RTT_READY,
  AKE_STATE_RTT_READY_CMD,
  AKE_STATE_RTT_SETUP,
  AKE_STATE_RTT_TEST,
  AKE_STATE_RTT_TEST_ACCEPTED,
  AKE_STATE_RTT_VERIFY,
  AKE_STATE_RTT_VERIFY_ACCEPTED,
  AKE_STATE_EXCHANGE_KEY,
  AKE_STATE_EC_DH_EXCHANGE,
  AKE_STATE_MV_EXCHANGE_KEY,
  AKE_STATE_SRM_UPDATE_LOCAL,
  AKE_STATE_SRM_UPDATE_REMOTE,
  AKE_STATE_DONE_NO_WAIT_STATUS,  // Wait for a status command from a peer, Timeout in
                          // this case NOT ERROR!!
  AKE_STATE_DONE
};


// V1SE.4.22
#define DTCP_CP_PCP_N_SIZE(cl) (int)((((int)(cl-1)/AES_IV_SIZE)+1)*AES_IV_SIZE)


// OBFISCATION ENGINE
#define OENG_KEY_SIZE       DTCP_EC_KEY_SIZE
#define OENG_DTLA_KYES      3 // DTLA Keys in the dtla structure
#define OEMG_AKE_KEYS       5 // Confidential algorithms intermediate keys in the device structure
#define OEMG_CP_KEYS        3 // Content engine KEYS
#define OEMG_DINFO_KEYS     1 // Device infor exchange key

  //- Exchange Key (Kx)
  //- Move Exchange Key (Kxm)
  //- Authentication Key (Kauth)
  //- Content key (Kc)

#define OENG_DTLA_MAX_IDX   (OENG_KEY_SIZE*OENG_DTLA_KYES)
#define OENG_AKE_MAX_IDX    (OENG_KEY_SIZE*OEMG_AKE_KEYS)
#define OENG_CP_MAX_IDX     (OENG_KEY_SIZE*OEMG_CP_KEYS)
#define OENG_DINFO_MAX_IDX  (OENG_KEY_SIZE*OEMG_DINFO_KEYS)

#define OENG_MAX_KEYS 5

typedef unsigned int uint32_t;
typedef unsigned char uint8_t;
typedef unsigned short int uint16_t;
typedef signed   short int int16_t;

typedef struct dtcp_dtla_constants_
{
  uint32_t cookie;   // magic cookie
  uint16_t version;  // STRUCTURE VERSION
  uint16_t type;     // TYPE (FULL, RESTRICTED)
  int16_t size;     // structure size

  // IV
  uint8_t iv_c[IV_SIZE];

  // EMI Values
  uint8_t c_a0[EXCH_KEY_SIZE];
  uint8_t c_b1[EXCH_KEY_SIZE];
  uint8_t c_b0[EXCH_KEY_SIZE];
  uint8_t c_c1[EXCH_KEY_SIZE];
  uint8_t c_c0[EXCH_KEY_SIZE];
  uint8_t c_d0[EXCH_KEY_SIZE];

  // DTLA device certificate
  uint8_t certificate[DTLA_CERTIFICATE_SIZE_MAX];
  int16_t certificate_size;

  // DTLA EC160 Parameters
  uint8_t ec_p[DTCP_EC_KEY_SIZE];
  uint8_t ec_a[DTCP_EC_KEY_SIZE];
  uint8_t ec_b[DTCP_EC_KEY_SIZE];
  uint8_t ec_r[DTCP_EC_KEY_SIZE];
  uint8_t ec_Gx[DTCP_EC_KEY_SIZE];
  uint8_t ec_Gy[DTCP_EC_KEY_SIZE];

  // RNGf Seed
  uint8_t rngf_seed[ESM_RNG_F_SEED_SIZE];

  // OBFUSCATED DTLA PUBLIC AND PRIVATE KEYS
  uint8_t oeng_idx[OENG_DTLA_MAX_IDX];
  uint8_t oeng_kyes[OENG_DTLA_MAX_IDX];  // DTLA Keys
  uint8_t oeng_mask[OENG_DTLA_MAX_IDX];  // Mask

#ifdef OENG_DEBUG
  // DTLA KEYS
  uint8_t ec_Lx[DTCP_EC_KEY_SIZE]; // DTLA public key
  uint8_t ec_Ly[DTCP_EC_KEY_SIZE];
  uint8_t ec_X[DTCP_EC_KEY_SIZE];  // DTLA private key
#endif

} dtcp_dtla_constants;


// This structure maps DTLA SRM header only
// the crl is a variable data buffer which contains CRL
typedef struct dtla_device_srm_
{
  uint8_t  type_generation;
  uint8_t  reserverd;
  uint16_t version;
  uint16_t crl_length;
  uint8_t  crl[0];
  //NOTE the the next CRL block will be
  //uint16_t crl_length;
  //uint8_t  crl[crl_length-2];
} dtla_device_srm;

#define DTCP_PEER_RMP_AKE_INFO 128
// Peer descriptor
// this is in fact iaccepted sink devices for a source device
typedef struct dtcp_net_
{
  int ake_socket; // control socket number of an accepted connection
  unsigned short port;       // AKE port number of an accepted connection
  unsigned long  address;    // IP address number of an accepted connection
  int timeout;    // Read Timeout from this peer; depends on a current state
} dtcp_net;

typedef struct dtcp_device_peer_
{
  int type;     // TYPE (SINK, SOURCE)
  // See 8.3.4.7 and 8.3.4.8
  uint32_t capability;
  uint32_t capability_exchange;

  int state;    // Device STATE

  // Set when need to use Move exchnage key for contant transfer
  int move;

  // Nonce Master Source Device
  // Also temporary value for Sink  Device CONT_KEY_CONF confirmation
  unsigned char N_c[DTCP_NC_NONCE_SIZE];


  // Exchange key label value
  unsigned char Kx_label;
  // AKE label value
  unsigned char my_ake_label;
  unsigned char peer_ake_label;

  // SINK Device ID 40bit for Source Devices that implement Sink Limitation Procedure
  unsigned char ID_u[5];
  int response2;


  // Peer information

  // DTLA sertificate ; this is peer one
  unsigned char  AB_cert[DTLA_CERTIFICATE_SIZE_MAX];

  // Full authentication
  unsigned char  X_srmc;      // SRM Generation from Peer DTLA SRM
  uint16_t X_srmv;      // SRM Version
  unsigned char  B_k[DTCP_EC_KEY_SIZE+2];      // random for DH exchange
  unsigned char  A_k[DTCP_EC_KEY_SIZE+2];      // random for DH exchange
  unsigned char  B_v[2*DTCP_EC_KEY_SIZE+2];    // First phase value (AB_k*G)for DH exchange
  unsigned char  A_v[2*DTCP_EC_KEY_SIZE+2];    // First phase value (AB_k*G)for DH exchange

  unsigned char  A_n[DTLA_AKE_KEY_SIZE];     // random nonce
  unsigned char  B_n[DTLA_AKE_KEY_SIZE];


  // RTT Information
  uint16_t N_rtt;   // N is 16 bit number incremented from 0 to 1023
  int rtt;  // Result of Protected RTT check, only valid for Source

  int content_tx_time; //  40 hours elapse time

    // Content Key Confirmation R value
  unsigned char  R_n[CONF_KEY_R_SIZE]; // 64 bit value however stick with byte array

  unsigned char  ake_info[DTCP_PEER_RMP_AKE_INFO]; // temporary ake_info
  int p_info_size;

  //  INTERMEDIATE KEYS
#ifdef OENG_DEBUG
  //MOVE Exchange KEY
  unsigned char  K_xm[EXCH_KEY_SIZE];

  //Main Exchange KEY
  unsigned char  K_x[EXCH_KEY_SIZE];

  // Authentication Key
  unsigned char  K_auth[EXCH_KEY_SIZE];

  //Content Channel KEY & IV
  unsigned char K_c[AES_KEY_SIZE];
  unsigned char IV[AES_IV_SIZE];
#endif

  // OBFUSCATED INTERMEDIATE KEYS
  unsigned char oeng_idx[OENG_AKE_MAX_IDX];
  unsigned char oeng_kyes[OENG_AKE_MAX_IDX];
  unsigned char oeng_mask[OENG_AKE_MAX_IDX];



  dtcp_net net;

} dtcp_device_peer;


typedef struct dtcp_device_info_
{
  int status;       // Status returned by some commands
  int state;        // Device State [FRRE, UNUTHENTICATED, AUTHENTICATED]
  int move;         // Main AKE or MOVE
  int type;         // Device structure type [SINK, SOURCE, CONTENT_ENGINE]
  int srm_ver;      // Current SRM version
  uint32_t capability;   // Device capability (See 8.3.4.7 and 8.3.4.8 )
  uint32_t capability_exchange;  // Source Device capability returned by CAPABILITY_EXCHANGE (See V1SE.5.5.6)
  unsigned char  ID[5];  // SINK Device ID 40bit
  unsigned char  ID_u[5];// SINK Device IDu 40bit
  unsigned char  N_c[DTCP_NC_NONCE_SIZE];   // Current Nonce Value
  unsigned char  Kx_label;
  int socket;         // Device AKE socket number
  int port;           // Peer AKE Port
  int address;        // Peer IP address
  int rtt;                // Result of Protected RTT check, only valid for Source, Sink is just an information flag
  unsigned char  dev_AL;  // Device AL flag set or not
  unsigned char  dev_AP;  // Device AP flag set or not
  unsigned char  peer_AL;// Peer AL flag set or not
  unsigned char  peer_AP; // Peer AP flag set or not

#ifdef OENG_DEBUG
  //Main Exchange KEY
  unsigned char  K_x[EXCH_KEY_SIZE];
#endif


  // OBFUSCATED INTERMEDIATE KEYS
  unsigned char oeng_idx[OENG_DINFO_MAX_IDX];
  unsigned char oeng_kyes[OENG_DINFO_MAX_IDX];
  unsigned char oeng_mask[OENG_DINFO_MAX_IDX];


}dtcp_device_info;


#define DTCP_FULL_CERT_AL_FLAG  1
#define DTCP_FULL_CERT_AL_IS_SET(f) ((f) & (1<<DTCP_FULL_CERT_AL_FLAG))
#define DTCP_FULL_CERT_AL_SET(f) (f)=((f) | (1<<DTCP_FULL_CERT_AL_FLAG))
#define DTCP_FULL_CERT_AL_CLEAR(f) ((f) & (~(1<<DTCP_FULL_CERT_AL_FLAG)))

#define DTCP_FULL_CERT_AP_FLAG  0
#define DTCP_FULL_CERT_AP_IS_SET(f) ((f) & (1<<DTCP_FULL_CERT_AP_FLAG))
#define DTCP_FULL_CERT_AP_SET(f) (f)=((f) | (1<<DTCP_FULL_CERT_AP_FLAG))
#define DTCP_FULL_CERT_AP_CLEAR(f) ((f) & (~(1<<DTCP_FULL_CERT_AP_FLAG)))


typedef struct dtla_baseline_certificate
{
  unsigned char type_format; // 4 bits type, 4 bits format
  unsigned char generation;  // 4 bit Dev Gen, 4 bit Reserved
  unsigned char flags;       // 6 bits Reserved (Cont), 1-bit AL, 1-bit AP
  unsigned char device_id[5];
  unsigned char ec_public_key[2*DTCP_EC_KEY_SIZE];
  unsigned char ec_dsa[DTCP_EC_SIGNATURE_DATA_SIZE];
}dtla_full_cert;


typedef struct dtla_restricted_certificate
{
  unsigned char type_format;
  unsigned char ksv[2];  //ksv[1]- xxxAvvvv
  unsigned char device_id[5];
  unsigned char ec_dsa[DTCP_EC_SIGNATURE_DATA_SIZE];
}dtla_restricted_cert;

#define DTCP_CP_TYPE_PCP        0
#define DTCP_CP_TYPE_HTTP_HEADER    1
#define DTCP_CP_TYPE_PCP_HEADER     2
#define DTCP_CP_TYPE_HTTP_START     3
#define DTCP_CP_TYPE_HTTP_CHUNK     4
#define DTCP_CP_TYPE_HTTP_PCP_HEADER  5
#define DTCP_CP_TYPE_PCP_DATA       6

#define DTCP_AKE_MOVE_SAVE_SIZE 19 // FINALAZE AKE_info size

typedef int ( *dtcp_move_data_callback )( void * data,int *size,unsigned char label );
typedef int ( *dtcp_move_clear_callback )(unsigned char label );




int dtcp_init(unsigned char *pimg, int size, unsigned char (*get_cycle_time)(void), int flags, unsigned char *seed);
int dtcp_cleanup(void);

int dtcp_srm_get (unsigned char *buf, int *size);
int dtcp_srm_check (unsigned char *srm, int size, int *ver);
int dtcp_srm_parse(unsigned char *srm, int size, unsigned char *dId, int *revoked);
int dtcp_srm_load(unsigned char *srm, int size);

int dtcp_timer_init(int (*get_time_milliseconds)(void));
int dtcp_get_device_info(int handle, dtcp_device_info *info);
int dtcp_get_dtla_image(unsigned char *buf, int *size, unsigned char *seed);


int dtcp_source_init(unsigned long capability, unsigned short port);
int dtcp_source_cleanup(int options);
int dtcp_source_Kx_expire(void);
int dtcp_source_get_command(int handle, int *cmd, int tmout);
int dtcp_source_ake(int handle,int cmd, dtcp_device_info *info);
int dtcp_source_accept(int flags, int *handle);

int dtcp_source_move_commit(int handle,
    int ( *dtcp_move_data_callback )( void * data,int *size,unsigned char label ),
    int ( *dtcp_move_clear_callback )(unsigned char label ));
int dtcp_source_move_commit_resume_1(int handle, void * data,int size,unsigned char label,
  int ( *dtcp_move_unusable_callback )( void * data,int *size,unsigned char label ),
  int ( *dtcp_move_clear_callback)(unsigned char label ));
int dtcp_source_move_commit_resume_2(int handle, unsigned char label,
  int ( *dtcp_move_clear_callback )(unsigned char label ));

#define dtcp_source_close dtcp_sink_close

void dtcp_memory_log(void);

int dtcp_sink_start(unsigned long capability, unsigned long address, unsigned short akeport, int *handle);
int dtcp_sink_ake(int handle, int extcmd, dtcp_device_info *info);
int dtcp_sink_close(int handle);
int dtcp_sink_get_command(int handle, int *cmd, int tmout);

int dtcp_sink_move_commit(int handle, int ( *dtcp_move_data_callback )( void * data,int *size,unsigned char label ),
              int ( *dtcp_move_usable_callback )(unsigned char label ));
int dtcp_sink_move_commit_resume_1(int handle,void * data,int size,unsigned char label,
  int ( *dtcp_move_usable_callback)(unsigned char label ));
int dtcp_sink_move_commit_resume_2(int handle,unsigned char label);

int dtcp_source_move_finalize(int handle,void * data,int size,unsigned char label,
    int ( *dtcp_move_clear_callback )(unsigned char label ));


void dtcp_set_debug (uint16_t dflag);
int dtcp_is_debug (uint16_t dflag);
char *dtcp_get_version_ex(void);
void dtcp_get_version(unsigned char *ver);

void dtcp_source_rtt_test_timeout_ignore(void);
int dtcp_ec_dsa_signature (unsigned char *c_p, unsigned char *c_r, unsigned char *c_Gx, unsigned char *c_Gy, unsigned char *c_u, unsigned char *c_prikey, unsigned char *c_msg, int msgsize, unsigned char *sm);
int dtcp_ec_dsa_verify(unsigned char *c_p, unsigned char *c_r, unsigned char *c_Gx, unsigned char *c_Gy, unsigned char *c_pubkey_x, unsigned char *c_pubkey_y, unsigned char *c_msg, int msgsize, unsigned char *sm, int *valid);
int dtcp_ec_dh(unsigned char *c_p, unsigned char *c_Gx, unsigned char *c_Gy, unsigned char *c_xk, unsigned char *c_Yvx, unsigned char *c_Yvy, unsigned char *c_Xv);
int dtcp_ec_make_key(unsigned char *c_p, unsigned char *c_Gx, unsigned char *c_Gy, unsigned char *c_prikey, unsigned char *c_pubkey_x, unsigned char *c_pubkey_y);

extern dtcp_dtla_constants dtla;

#ifndef  BINARY_PACKAGE
#include <elpcrypto.h>
#include "dtcp_ex.h"
#include "dtcp_oeng.h"
#endif

#include "dtcp_ip.h"
#include "dtcp_cp.h"
#include "dtcp_oeng.h"

#endif
