/*-----------------------------------------------------------------------
//
// Proprietary Information of Elliptic Technologies
// Copyright (C) 2001-2012, all rights reserved
// Elliptic Technologies, Inc.
//
// As part of our confidentiality  agreement, Elliptic Technologies and
// the Company, as  a  Receiving Party, of  this  information  agrees to
// keep strictly  confidential  all Proprietary Information  so received
// from Elliptic Technologies. Such Proprietary Information can be used
// solely for  the  purpose  of evaluating  and/or conducting a proposed
// business  relationship  or  transaction  between  the  parties.  Each
// Party  agrees  that  any  and  all  Proprietary  Information  is  and
// shall remain confidential and the property of Elliptic Technologies.
// The  Company  may  not  use  any of  the  Proprietary  Information of
// Elliptic Technologies for any purpose other  than  the  above-stated
// purpose  without the prior written consent of Elliptic Technologies.
//
// This information is further subject to the terms and conditions of
// the DTLA license agreement.
//-----------------------------------------------------------------------
//
// Project:
//
//   DTCP-IP
//
// Description:
//
//   Content Protection Module
//
//
//-----------------------------------------------------------------------
//
// Language:         C
//
//
// Filename:         $Source: /home/repository/cvsrep/dtcp_sdk/dtcp/include/dtcp_cp.h,v $
// Current Revision: $Revision: 1.14 $
// Last Updated:     $Date: 2012/01/24 22:00:35 $
// Current Tag:      $Name: DTCP-IP-SDK-V53-OFF-THE-SHELF-07-FEB-2012 $
//
//
//-----------------------------------------------------------------------*/


#ifndef _DTCP_CP_H_
#define _DTCP_CP_H_


#pragma pack(1)
// Protected Content Packet (PCP) Header
typedef struct dtcp_ip_pcp_header_
{
  unsigned char emi_ex;   // 3 bits reserved ; 1 bit - 0-AES,1-Optional; 4 bits emi
  unsigned char label;    // exchange key label
  unsigned char n_c[8];   // Nonce - instead of odd/even bit
  uint32_t length;        // Network Order!!!
}dtcp_ip_pcp_header;
#pragma pack()


#define DTCP_CP_PCP_HEADER_SIZE 14

#define DTCP_CP_PCP_START  1
#define DTCP_CP_PCP_CONT   2
#define DTCP_CP_PCP_END    3


//typedef int elpsymmetric_cbc ;



typedef struct dtcp_device_ce_
{

  int device_set;
  int move;
  int type;

  // See 8.3.4.7 and 8.3.4.8
  uint32_t capability;

  uint32_t capability_exchange;

  unsigned char  emi;    // EMI mapping

  // V1SE.4.23.2 PCP-UR field
  uint16_t pcp_ur;

  // The NV field specifies whether the value of NC_for_RTP field is valid (12) or invalid (02).
  unsigned char  nv;
  int nc_update_period;
  int nc_update_128MB;
  // HTTP Nonce
  unsigned char N_c[DTCP_NC_NONCE_SIZE];

  int nc_updated;
  int set_iv;

  // Exchange key label value
  unsigned char Kx_label;


  // SINK Device ID 40bit for Source Devices that implement Sink Limitation Procedure
  unsigned char ID_u[5];

    // Content Key Confirmation R value
  unsigned char  R_n[DTLA_AKE_R_KEY_SIZE]; // 64 bit value however stick with byte array


  //  INTERMEDIATE KEYS
#ifdef OENG_DEBUG
  //Exchange KEY
  unsigned char  K_x[EXCH_KEY_SIZE];

  //Content Channel KEY & IV
  unsigned char K_c[AES_KEY_SIZE];
  unsigned char IV[AES_IV_SIZE];
#endif


  // OBFUSCATED INTERMEDIATE KEYS
  unsigned char oeng_idx[OENG_CP_MAX_IDX];
  unsigned char oeng_kyes[OENG_CP_MAX_IDX];
  unsigned char oeng_mask[OENG_CP_MAX_IDX];

  // Nees for partial PCP encode/decode
  elpsymmetric_cbc cbc;


} dtcp_device_ce;


int dtcp_cp_allocate(int emi, int pcpur, int flags, dtcp_device_info *info, int *handle);
int dtcp_cp_close(int handle);
int dtcp_cp_init(int ncupd, int (*get_time_seconds)(void));
int dtcp_cp_cleanup(void);

int dtcp_cp_encode_pcp(int handle, unsigned char *data,unsigned char *pcp,int *size);
int dtcp_cp_decode_pcp(int handle, unsigned char *pcp, unsigned char *data, int *size, int *ur);

int dtcp_cp_encode_pcp_ex(int handle, unsigned char *data,unsigned char *pcp,int *size, int *totalsize, int pcpctrl);
int dtcp_cp_decode_pcp_ex(int handle, unsigned char *pcp, unsigned char *data, int *size, int *ur, int *totalsize, int pcpctrl);


int dtcp_cp_get_info(int handle, dtcp_device_info *info);
int dtcp_cp_verify_nc(dtcp_device_info *info);

struct elpcipher_plugin;
void dtcp_cp_set_content_cipher(const struct elpcipher_plugin *plugin);


#endif
