/*-----------------------------------------------------------------------
//
// Proprietary Information of Elliptic Technologies
// Copyright (C) 2001-2011, all rights reserved
// Elliptic Technologies, Inc.
//
// As part of our confidentiality  agreement, Elliptic Technologies and
// the Company, as  a  Receiving Party, of  this  information  agrees to
// keep strictly  confidential  all Proprietary Information  so received
// from Elliptic Technologies. Such Proprietary Information can be used
// solely for  the  purpose  of evaluating  and/or conducting a proposed
// business  relationship  or  transaction  between  the  parties.  Each
// Party  agrees  that  any  and  all  Proprietary  Information  is  and
// shall remain confidential and the property of Elliptic Technologies.
// The  Company  may  not  use  any of  the  Proprietary  Information of
// Elliptic Technologies for any purpose other  than  the  above-stated
// purpose  without the prior written consent of Elliptic Technologies.
//
// This information is further subject to the terms and conditions of
// the DTLA license agreement.
//-----------------------------------------------------------------------
//
// Project:
//
//   DTCP-IP
//
// Description:
//
//
// This file defined obfuscation structures  and functions.
//
//
//-----------------------------------------------------------------------
//
// Language:         C
//
//
// Filename:         $Source: /home/repository/cvsrep/dtcp_sdk/dtcp/include/dtcp_oeng.h,v $
// Current Revision: $Revision: 1.2 $
// Last Updated:     $Date: 2011/09/15 16:41:29 $
// Current Tag:      $Name: DTCP-IP-SDK-V53-OFF-THE-SHELF-07-FEB-2012 $
//
//
//-----------------------------------------------------------------------*/


#ifndef _DTCP_OENG_H_
#define _DTCP_OENG_H_


//Using MACRO instead of the inline functions which is not quite portable
//static inline void memcpy_obfuscated_1(char *dst, char *src, int n)  __attribute__((always_inline));
//static inline void memcpy_obfuscated_2(char *dst, char *src, int n)  __attribute__((always_inline));

// assume x2(x+1)2 mod 4 = 0
#define memcpy_obfuscated_1(d, s, n) {\
  int i=1,j=3;\
  char *dst=(char *)(d), *src=(char *)(s);\
  while ((i<= n) && ((j*j*(j+1)*(j+1)%4)==0))\
  {\
    dst[i-1] = src[i-1];\
    i++;\
    j = j * i + 3;\
  }\
}

// and faster
#define memcpy_obfuscated_2(d, s, n) {\
  char *dst=(char *)(d), *src=(char *)(s);\
  int i=n%3;\
  switch(i)\
  {\
    case 1:\
      dst[0]   =src[0];\
      break;\
    case 2:\
      dst[0]   =src[0];\
      dst[1]   =src[1];\
      break;\
    default:\
      i=n%3;\
      break;\
  }\
  for (i=n%3;i<n;i+=3)\
  {\
    dst[i]   =src[i];\
    dst[i+1] =src[i+1];\
    dst[i+2] =src[i+2];\
  }\
}


#define MEMCPY_OBF1(d,s,z)      memcpy_obfuscated_1((char*)(d),(char*)(s),(int)(z))
#define MEMCPY_OBF2(d,s,z)      memcpy_obfuscated_2((char*)(d),(char*)(s),(int)(z))
#define MEMCPY_JAES   MEMCPY_OBF2
#define MEMCPY_AESIV  MEMCPY_OBF1
#define MEMCPY_FEMI   MEMCPY_OBF2
#define MEMCPY_KAUTH  MEMCPY_OBF1
#define MEMCPY_MAC12  MEMCPY_OBF2
#define MEMCPY_MAC56  MEMCPY_OBF1
#define MEMCPY_MAC34  MEMCPY_OBF2
#define MEMCPY_RNG    MEMCPY_OBF1


void dtcp_oeng_start(void);
void dtcp_oeng_stop(void);
void dtcp_oeng_init(unsigned char *pidx, unsigned char *pkeys,unsigned char *pmasks, int keys, int key_size);
int dtcp_oeng_putkey(unsigned char *key, int key_size, unsigned char *key_dst, unsigned char *key_mask,unsigned char *keyindex, int kidx);
unsigned char *dtcp_oeng_getkey(int key_size, unsigned char *key_dst, unsigned char *key_mask,unsigned char *keyindex, int kidx, unsigned char *out);
int dtcp_oeng_test_key_idx(unsigned char *key_idx, int max_idx);


#define OENG_DTLA_TEST_KEY_IDX(dtla) dtcp_oeng_test_key_idx((unsigned char *)&((dtcp_dtla_constants *)&dtla)->oeng_idx[0],OENG_KEY_SIZE)

#define OENG_DTLA_INIT(dtla)   dtcp_oeng_init(&((dtcp_dtla_constants *)&dtla)->oeng_idx[0], &((dtcp_dtla_constants *)&dtla)->oeng_kyes[0],&((dtcp_dtla_constants *)&dtla)->oeng_mask[0], OENG_DTLA_KYES, OENG_KEY_SIZE)


#define OENG_PUT_DTLA_KEY_EX(dtla,key,size,index)  dtcp_oeng_putkey(key,size,&((dtcp_dtla_constants *)&dtla)->oeng_kyes[0],&((dtcp_dtla_constants *)&dtla)->oeng_mask[0],&((dtcp_dtla_constants *)&dtla)->oeng_idx[0],index)
#define OENG_GET_DTLA_KEY_EX(dtla,size,out,index)      dtcp_oeng_getkey(size,&((dtcp_dtla_constants *)&dtla)->oeng_kyes[0],&((dtcp_dtla_constants *)&dtla)->oeng_mask[0],&((dtcp_dtla_constants *)&dtla)->oeng_idx[0],index,out)

#define OENG_PUT_DTLA_PUBLIC_X_EX(dtla,key) OENG_PUT_DTLA_KEY_EX(dtla,(key),DTCP_EC_KEY_SIZE,0)
#define OENG_PUT_DTLA_PUBLIC_Y_EX(dtla,key) OENG_PUT_DTLA_KEY_EX(dtla,(key),DTCP_EC_KEY_SIZE,1)
#define OENG_PUT_DTLA_PRIVATE_EX(dtla,key)  OENG_PUT_DTLA_KEY_EX(dtla,(key),DTCP_EC_KEY_SIZE,2)

#define OENG_GET_DTLA_PUBLIC_X_EX(dtla,out)    OENG_GET_DTLA_KEY_EX(dtla,DTCP_EC_KEY_SIZE,out,0)
#define OENG_GET_DTLA_PUBLIC_Y_EX(dtla,out)    OENG_GET_DTLA_KEY_EX(dtla,DTCP_EC_KEY_SIZE,out,1)
#define OENG_GET_DTLA_PRIVATE_EX(dtla,out)     OENG_GET_DTLA_KEY_EX(dtla,DTCP_EC_KEY_SIZE,out,2)



#define OENG_PUT_DTLA_KEY(key,size,index)  dtcp_oeng_putkey(key,size,(unsigned char *)&dtla.oeng_kyes[0],(unsigned char *)&dtla.oeng_mask[0],(unsigned char *)&dtla.oeng_idx[0],index )
#define OENG_GET_DTLA_KEY(size,out,index)      dtcp_oeng_getkey(size,(unsigned char *)&dtla.oeng_kyes[0],(unsigned char *)&dtla.oeng_mask[0],(unsigned char *)&dtla.oeng_idx[0],index, out)

#define OENG_PUT_DTLA_PUBLIC_X(key) OENG_PUT_DTLA_KEY(key,DTCP_EC_KEY_SIZE,0)
#define OENG_PUT_DTLA_PUBLIC_Y(key) OENG_PUT_DTLA_KEY(key,DTCP_EC_KEY_SIZE,1)
#define OENG_PUT_DTLA_PRIVATE(key)  OENG_PUT_DTLA_KEY(key,DTCP_EC_KEY_SIZE,2)


#define OENG_GET_DTLA_PUBLIC_X(out)    OENG_GET_DTLA_KEY(DTCP_EC_KEY_SIZE,out,0)
#define OENG_GET_DTLA_PUBLIC_Y(out)    OENG_GET_DTLA_KEY(DTCP_EC_KEY_SIZE,out,1)
#define OENG_GET_DTLA_PRIVATE(out)     OENG_GET_DTLA_KEY(DTCP_EC_KEY_SIZE,out,2)



#define OENG_PUT_CALG_KEY_EX(dev,key,size,index)  dtcp_oeng_putkey(key,size,&dev->oeng_kyes[0],&dev->oeng_mask[0],&dev->oeng_idx[0],index)
#define OENG_GET_CALG_KEY_EX(dev,size,out,index)  dtcp_oeng_getkey(size,&dev->oeng_kyes[0],&dev->oeng_mask[0],&dev->oeng_idx[0],index,out)


#define OENG_AKE_INIT(dev)   dtcp_oeng_init(&dev->oeng_idx[0], &dev->oeng_kyes[0],&dev->oeng_mask[0], OEMG_AKE_KEYS, OENG_KEY_SIZE)
#define OENG_PUT_AKE_KC(dev,key)     OENG_PUT_CALG_KEY_EX(dev,key,AES_KEY_SIZE, 0)
#define OENG_PUT_AKE_IV(dev,key)     OENG_PUT_CALG_KEY_EX(dev,key,AES_IV_SIZE,  1)
#define OENG_PUT_AKE_KAUTH(dev,key)  OENG_PUT_CALG_KEY_EX(dev,key,EXCH_KEY_SIZE,2)
#define OENG_PUT_AKE_KX(dev,key)     OENG_PUT_CALG_KEY_EX(dev,key,EXCH_KEY_SIZE,3)
#define OENG_PUT_AKE_KXM(dev,key)    OENG_PUT_CALG_KEY_EX(dev,key,EXCH_KEY_SIZE,4)


#define OENG_GET_AKE_KC(dev,out)     OENG_GET_CALG_KEY_EX(dev,AES_KEY_SIZE,out, 0)
#define OENG_GET_AKE_IV(dev,out)     OENG_GET_CALG_KEY_EX(dev,AES_IV_SIZE,out,  1)
#define OENG_GET_AKE_KAUTH(dev,out)  OENG_GET_CALG_KEY_EX(dev,EXCH_KEY_SIZE,out,2)
#define OENG_GET_AKE_KX(dev,out)     OENG_GET_CALG_KEY_EX(dev,EXCH_KEY_SIZE,out,3)
#define OENG_GET_AKE_KXM(dev,out)    OENG_GET_CALG_KEY_EX(dev,EXCH_KEY_SIZE,out,4)



#define OENG_CP_INIT(dev)   dtcp_oeng_init(&dev->oeng_idx[0], &dev->oeng_kyes[0],&dev->oeng_mask[0], OEMG_CP_KEYS, OENG_KEY_SIZE)
#define OENG_PUT_CP_KC(dev,key)     OENG_PUT_CALG_KEY_EX(dev,key,AES_KEY_SIZE, 0)
#define OENG_PUT_CP_IV(dev,key)     OENG_PUT_CALG_KEY_EX(dev,key,AES_IV_SIZE,  1)
#define OENG_PUT_CP_KX(dev,key)     OENG_PUT_CALG_KEY_EX(dev,key,EXCH_KEY_SIZE,2)


#define OENG_GET_CP_KC(dev,out)     OENG_GET_CALG_KEY_EX(dev,AES_KEY_SIZE,out, 0)
#define OENG_GET_CP_IV(dev,out)     OENG_GET_CALG_KEY_EX(dev,AES_IV_SIZE,out,  1)
#define OENG_GET_CP_KX(dev,out)     OENG_GET_CALG_KEY_EX(dev,EXCH_KEY_SIZE,out,2)


#define OENG_DINFO_INIT(dev)   dtcp_oeng_init(&dev->oeng_idx[0], &dev->oeng_kyes[0],&dev->oeng_mask[0], OEMG_DINFO_KEYS, OENG_KEY_SIZE)
#define OENG_PUT_DINFO_KX(dev,key)     OENG_PUT_CALG_KEY_EX(dev,key,EXCH_KEY_SIZE,0)


#define OENG_GET_DINFO_KX(dev,out)     OENG_GET_CALG_KEY_EX(dev,EXCH_KEY_SIZE,out,0)



#endif


