/*-----------------------------------------------------------------------
//
// Proprietary Information of Elliptic Technologies
// Copyright (C) 2001-2011, all rights reserved
// Elliptic Technologies, Inc.
//
// As part of our confidentiality  agreement, Elliptic Technologies and
// the Company, as  a  Receiving Party, of  this  information  agrees to
// keep strictly  confidential  all Proprietary Information  so received
// from Elliptic Technologies. Such Proprietary Information can be used
// solely for  the  purpose  of evaluating  and/or conducting a proposed
// business  relationship  or  transaction  between  the  parties.  Each
// Party  agrees  that  any  and  all  Proprietary  Information  is  and
// shall remain confidential and the property of Elliptic Technologies.
// The  Company  may  not  use  any of  the  Proprietary  Information of
// Elliptic Technologies for any purpose other  than  the  above-stated
// purpose  without the prior written consent of Elliptic Technologies.
//
//-----------------------------------------------------------------------
//
// Project:
//
//   DTCP-IP
//
// Description:
//
//
// This file maps defines a system abstraction layer for some generic
// functions and data structures
//
//
//-----------------------------------------------------------------------
//
// Language:         C
//
//
// Filename:         $Source: /home/repository/cvsrep/dtcp_sdk/include/elptypes.h,v $
// Current Revision: $Revision: 1.57 $
// Last Updated:     $Date: 2012/01/12 17:46:33 $
// Current Tag:      $Name: DTCP-IP-SDK-V53-OFF-THE-SHELF-07-FEB-2012 $
//
//
//-----------------------------------------------------------------------*/


#ifndef _ELPTYPES_H_
#define _ELPTYPES_H_

#define USE_IT(x) ((x)=(x))


#ifndef _U8_
#define _U8_
typedef unsigned char U8;
#endif

#ifdef WIN32
typedef unsigned __int64 U64; /* unsigned 64-bit integer */
#else
#ifndef _U64_
#define _U64_
typedef unsigned long long U64; /* unsigned 64-bit integer */
#endif
#endif

#ifdef WIN32
typedef signed __int64 S64;     /* 64 bit signed integer */
#else
typedef signed long long S64;     /* 64 bit signed integer */
#endif

#define _SPRINTF  sprintf
#if (defined (DO_LINUX))

#ifdef _ANDROID_
#define _PRINTF   __android_log_print
#else
#define _PRINTF   printf
#endif
#else
#define _PRINTF   printf
#endif

#ifndef ntoh64
#ifndef  CROSS_PPC

#define hton64(x) \
    ((((x) & 0xff00000000000000LL) >> 56) | \
    (((x) & 0x00ff000000000000LL) >> 40) | \
    (((x) & 0x0000ff0000000000LL) >> 24) | \
    (((x) & 0x000000ff00000000LL) >> 8) | \
    (((x) & 0x00000000ff000000LL) << 8) | \
    (((x) & 0x0000000000ff0000LL) << 24) | \
    (((x) & 0x000000000000ff00LL) << 40) | \
    (((x) & 0x00000000000000ffLL) << 56))
#define ntoh64 hton64
#else
#define ntoh64(x)     (x)
#define hton64(x)     (x)
#endif
#endif

#ifndef in_port_t
#define in_port_t uint16_t
#endif

#define ELP_COOKIE    (uint32_t)0xE117D7C4
#define ELP_DTLA_CONSTANT_STRUCTURE_VERSION_1 1
#define ELP_DTLA_CONSTANT_STRUCTURE_VERSION_2 2
#define ELP_DTLA_CONSTANT_STRUCTURE_VERSION ELP_DTLA_CONSTANT_STRUCTURE_VERSION_2
// This is USER SPACE ONLY implementattion

#ifdef _ANDROID_
#include <jni.h>
#include <android/log.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <elpsoft.h>

#if (defined (DO_LINUX)) || (defined (WIN32))
char * get_line (FILE * in);
void read_words (FILE * in, unsigned char *dst, int nwords);
int read_4_bytes (FILE * in, unsigned char *buf, int size);
void outdiff (char *s, unsigned char *buf1, unsigned char *buf2, int len);
int read_int (FILE * in);
#endif

#if defined (DO_LINUX)

#include <getopt.h>
#include <unistd.h>
#include <sys/types.h>

#ifndef  NO_BSD_SOCKETS

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#else
// Experimental only to compile
typedef short int16_t;
typedef unsigned short uint16_t;
typedef unsigned long uint32_t;
typedef long int32_t;
typedef unsigned char uint8_t;
#define AF_INET 1
#define SOCK_STREAM 1
#define IPPROTO_TCP 1
#define INADDR_ANY 1

#define AF_INET 1
#define IPPROTO_IP 1
#define IP_TTL 1
#define SHUT_RDWR 1
#define MSG_NOSIGNAL 1

#define  in_addr_t int
#define socklen_t int
struct timeval
{
	int tv_sec;
	int tv_usec;
};

struct in_addr
{
	in_addr_t s_addr;
};


struct sockaddr_in{
	short sin_family;
	unsigned short sin_port;
	struct in_addr sin_addr;
	char sin_zero[8];
};

#endif


#define RI        read_int(in)
#define RW(b, l)  read_words(in, b, l)

#if defined (ELPSEC_PCI)
#include <sys/ioctl.h>
#include <sys/types.h>
#include <fcntl.h>
#include <sys/io.h>
#include <sys/mman.h>
#include <pci/pci.h>
#endif

#elif WIN32
#include "elpgetopt.h"

//socket
#ifndef WIN32_LEAN_AND_MEAN
#define WIN32_LEAN_AND_MEAN
#endif

#include <sys/types.h>
#include <windows.h>
#include <winsock2.h>
#include <ws2tcpip.h>

#ifndef SHUT_RD
#   define SHUT_RD SD_RECEIVE
#endif

#ifndef SHUT_WR
#   define SHUT_WR SD_SEND
#endif

#ifndef SHUT_RDWR
#   define SHUT_RDWR SD_BOTH
#endif

#define close closesocket

#ifndef INET_ADDRSTRLEN
typedef unsigned int socklen_t;
#endif
typedef unsigned int in_addr_t;

#else

#endif

#ifndef ELP_THREADS

   typedef int elp_mutex;
   #define elp_mutex_init(m) (m =0)
   #define elp_mutex_delete(m)
   #define elp_mutex_lock(m)
   #define elp_mutex_unlock(m)

#else

#if (defined (DO_LINUX))
   // fetch LINUX user space mutex definitions
   //#ifndef S_SPLINT_S
   //#include <pthread.h>
   #include <bits/pthreadtypes.h>

   //extern int pthread_mutex_init (pthread_mutex_t *__mutex, __const pthread_mutexattr_t *__mutexattr);
   extern int pthread_mutex_init (pthread_mutex_t *__mutex, void *attr);
   extern int pthread_mutex_destroy (pthread_mutex_t *__mutex);
   extern int pthread_mutex_trylock (pthread_mutex_t *_mutex);
   extern int pthread_mutex_lock (pthread_mutex_t *__mutex);
   extern int pthread_mutex_unlock (pthread_mutex_t *__mutex);

   typedef pthread_mutex_t elp_mutex;

   //#define elp_mutex_init(m)   pthread_mutex_init(&m,(__const pthread_mutexattr_t *__mutexattr)0)
   #define elp_mutex_init(m)   (void)(pthread_mutex_init(&m,(void *)0))
   #define elp_mutex_delete(m) (void)(pthread_mutex_destroy(&m))
   #define elp_mutex_lock(m) (void)(pthread_mutex_lock(&m))
   #define elp_mutex_unlock(m) (void)(pthread_mutex_unlock(&m))

#elif (defined (WIN32))

   typedef HANDLE elp_mutex;
   #define elp_mutex_init(m) ((m = CreateMutex(0, FALSE, 0)) == 0 ? -1 : 0)
   #define elp_mutex_delete(m) CloseHandle(m)
   #define elp_mutex_lock(m) WaitForSingleObject(m, INFINITE)
   #define elp_mutex_unlock(m) ReleaseMutex(m)

#else

   typedef int elp_mutex;
   #define elp_mutex_init(m) (m =0)
   #define elp_mutex_delete(m)
   #define elp_mutex_lock(m)
   #define elp_mutex_unlock(m)

#endif

#endif


#ifndef NO_PRINTOUT

#define ESPRINTF1(b,s,a1)           _SPRINTF((char*)b,s,a1)
#define ESPRINTF2(b,s,a1,a2)        _SPRINTF((char*)b,s,a1,a2)
#define ESPRINTF3(b,s,a1,a2,a3)       _SPRINTF((char*)b,s,a1,a2,a3)
#define ESPRINTF4(b,s,a1,a2,a3,a4)      _SPRINTF((char*)b,s,a1,a2,a3,a4)

#ifdef _ANDROID_
#define DTCPIP_TAG "ELPDTCP"
#define EPRINTF0(s)            _PRINTF(ANDROID_LOG_DEBUG, DTCPIP_TAG, s)
#define EPRINTF1(s,a1)            _PRINTF(ANDROID_LOG_DEBUG, DTCPIP_TAG,s,a1)
#define EPRINTF2(s,a1,a2)           _PRINTF(ANDROID_LOG_DEBUG, DTCPIP_TAG,s,a1,a2)
#define EPRINTF3(s,a1,a2,a3)        _PRINTF(ANDROID_LOG_DEBUG, DTCPIP_TAG,s,a1,a2,a3)
#define EPRINTF4(s,a1,a2,a3,a4)       _PRINTF(ANDROID_LOG_DEBUG, DTCPIP_TAG,s,a1,a2,a3,a4)
#define EPRINTF5(s,a1,a2,a3,a4,a5)      _PRINTF(ANDROID_LOG_DEBUG, DTCPIP_TAG,s,a1,a2,a3,a4,a5)
#define EPRINTF6(s,a1,a2,a3,a4,a5,a6)   _PRINTF(ANDROID_LOG_DEBUG, DTCPIP_TAG,s,a1,a2,a3,a4,a5,a6)
#define EPRINTF7(s,a1,a2,a3,a4,a5,a6,a7)  _PRINTF(ANDROID_LOG_DEBUG, DTCPIP_TAG,s,a1,a2,a3,a4,a5,a6,a7)
#define EPRINTF8(s,a1,a2,a3,a4,a5,a6,a7,a8) _PRINTF(ANDROID_LOG_DEBUG, DTCPIP_TAG,s,a1,a2,a3,a4,a5,a6,a7,a8)
#define EPRINTF9(s,a1,a2,a3,a4,a5,a6,a7,a8,a9) _PRINTF(ANDROID_LOG_DEBUG, DTCPIP_TAG,s,a1,a2,a3,a4,a5,a6,a7,a8,a9)
#else
#define EPRINTF1(s,a1)            _PRINTF(s,a1)
#define EPRINTF2(s,a1,a2)           _PRINTF(s,a1,a2)
#define EPRINTF3(s,a1,a2,a3)        _PRINTF(s,a1,a2,a3)
#define EPRINTF4(s,a1,a2,a3,a4)       _PRINTF(s,a1,a2,a3,a4)
#define EPRINTF5(s,a1,a2,a3,a4,a5)      _PRINTF(s,a1,a2,a3,a4,a5)
#define EPRINTF6(s,a1,a2,a3,a4,a5,a6)   _PRINTF(s,a1,a2,a3,a4,a5,a6)
#define EPRINTF7(s,a1,a2,a3,a4,a5,a6,a7)  _PRINTF(s,a1,a2,a3,a4,a5,a6,a7)
#define EPRINTF8(s,a1,a2,a3,a4,a5,a6,a7,a8) _PRINTF(s,a1,a2,a3,a4,a5,a6,a7,a8)
#define EPRINTF9(s,a1,a2,a3,a4,a5,a6,a7,a8,a9) _PRINTF(s,a1,a2,a3,a4,a5,a6,a7,a8,a9)
#endif

#define DUMPHEX(o,b,s,m)    dumphex((U8 *)o,(U8 *)b, (int)s,(U8 *)m)
#define DUMPMEM(b,s,m)      dumpmem((U8*)b,(int)s,(U8*)m,0)

#else

#define ESPRINTF1(b,s,a1)              0
#define ESPRINTF2(b,s,a1,a2)           0
#define ESPRINTF3(b,s,a1,a2,a3)
#define ESPRINTF4(b,s,a1,a2,a3,a4)
#define EPRINTF1(s,a1)
#define EPRINTF2(s,a1,a2)
#define EPRINTF3(s,a1,a2,a3)
#define EPRINTF4(s,a1,a2,a3,a4)
#define EPRINTF5(s,a1,a2,a3,a4,a5)
#define EPRINTF6(s,a1,a2,a3,a4,a5,a6)
#define EPRINTF7(s,a1,a2,a3,a4,a5,a6,a7)
#define EPRINTF8(s,a1,a2,a3,a4,a5,a6,a7,a8)
#define EPRINTF9(s,a1,a2,a3,a4,a5,a6,a7,a8,a9)

#define DUMPHEX(o,b,s,m)    0
#define DUMPMEM(b,s,m)
#endif


#undef MAX
#undef MIN
#define MAX(x, y) ( ((x)>(y))?(x):(y) )
#define MIN(x, y) ( ((x)<(y))?(x):(y) )

#define crypto_is_debug dtcp_is_debug

#define EDINIT    0x0001 // Initialization
#define EDM6      0x0002
#define EDAES     0x0004
#define EDHASH    0x0010
#define EDPRNG    0x0020
#define EDCTX     0x0040
#define EDUTIL    0x0080
#define EDAKE     0x0100
#define EDCE      0x0200
#define EDSRM     0x0400
#define EDIP      0x0800
#define EDDUMP    0x1000
#define EDKEYS    0x2000
#define EDALLM    0xFFFF


#define PDEBUG_ID   "<0: ecrypto> "
#define PINFO_ID    "<1: ecrypto> "
#define PERROR_ID   "<2: ecrypto> "

#define PERROR(s,a1)        EPRINTF1(s,a1)
#define PERROR1(s,a1)       EPRINTF1(s,a1)
#define PERROR2(s,a1,a2)    EPRINTF2(s,a1,a2)
#define PERROR3(s,a1,a2,a3) EPRINTF3(s,a1,a2,a3)
#define PERROR4(s,a1,a2,a3,a4) EPRINTF4(s,a1,a2,a3,a4)
#define PERROR5(s,a1,a2,a3,a4,a5) EPRINTF5(s,a1,a2,a3,a4,a5)
#define PERROR6(s,a1,a2,a3,a4,a5,a6) EPRINTF6(s,a1,a2,a3,a4,a5,a6)
#define PERROR7(s,a1,a2,a3,a4,a5,a6,a7) EPRINTF7(s,a1,a2,a3,a4,a5,a6,a7)
#define PERROR8(s,a1,a2,a3,a4,a5,a6,a7,a8) EPRINTF8(s,a1,a2,a3,a4,a5,a6,a7,a8)
#define PERROR9(s,a1,a2,a3,a4,a5,a6,a7,a8,a9) EPRINTF9(s,a1,a2,a3,a4,a5,a6,a7,a8,a9)
#define PINFO1(s,a1)        EPRINTF1(s,a1)
#define PINFO2(s,a1,a2)     EPRINTF2(s,a1,a2)
#define PINFO3(s,a1,a2,a3)  EPRINTF3(s,a1,a2,a3)
#define PINFO4(s,a1,a2,a3,a4)   EPRINTF4(s,a1,a2,a3,a4)
#define PINFO(s,a1,a2,a3)   EPRINTF3(s,a1,a2,a3)

#define INETADDR(sa)        (char *)inetaddr(sa )

#ifndef NO_DEBUG_LOG

#define PDEBUG(t,s,a1,a2,a3)   do {if(crypto_is_debug(t)==1){ EPRINTF3(s,a1,a2,a3);}}  while(0)
#define PDEBUG1(t,s,a1)       do {if(crypto_is_debug(t)==1){ EPRINTF1(s,a1);}}  while(0)
#define PDEBUG2(t,s,a1,a2)    do {if(crypto_is_debug(t)==1){ EPRINTF2(s,a1,a2);}}  while(0)
#define PDEBUG3(t,s,a1,a2,a3)   do {if(crypto_is_debug(t)==1){ EPRINTF3(s,a1,a2,a3);}}  while(0)
#define PDEBUG4(t,s,a1,a2,a3,a4)   do {if(crypto_is_debug(t)==1){ EPRINTF4(s,a1,a2,a3,a4);}}  while(0)
#define PDEBUG5(t,s,a1,a2,a3,a4,a5)   do {if(crypto_is_debug(t)==1){ EPRINTF5(s,a1,a2,a3,a4,a5);}}  while(0)
#define PDEBUG6(t,s,a1,a2,a3,a4,a5,a6)   do {if(crypto_is_debug(t)==1){ EPRINTF6(s,a1,a2,a3,a4,a5,a6);}}  while(0)
#define PDEBUG7(t,s,a1,a2,a3,a4,a5,a6,a7)   do {if(crypto_is_debug(t)==1){ EPRINTF7(s,a1,a2,a3,a4,a5,a6,a7);}}  while(0)
#define PDEBUG8(t,s,a1,a2,a3,a4,a5,a6,a7,a8)   do {if(crypto_is_debug(t)==1){ EPRINTF8(s,a1,a2,a3,a4,a5,a6,a7,a8);}}  while(0)
#define PDEBUG9(t,s,a1,a2,a3,a4,a5,a6,a7,a8,a9)   do {if(crypto_is_debug(t)==1){ EPRINTF9(s,a1,a2,a3,a4,a5,a6,a7,a8,a9);}}  while(0)
#define PDUMPMEM(t,b,s,m)      do {if(crypto_is_debug(t)==1){ dumpmem((U8*)b,(int)s,(U8*)m,0); }}  while(0)
#define PDUMPWORD(t,b,s,m,e)  do {if(crypto_is_debug(t)==1){ DUMPWORD(b,s,m,e);}}  while(0)
#else
#define PDEBUG(t,s,a1,a2,a3)
#define PDEBUG1(t,s,a1)
#define PDEBUG2(t,s,a1,a2)
#define PDEBUG3(t,s,a1,a2,a3)
#define PDEBUG4(t,s,a1,a2,a3,a4)
#define PDEBUG5(t,s,a1,a2,a3,a4,a5)
#define PDEBUG6(t,s,a1,a2,a3,a4,a5,a6)
#define PDEBUG7(t,s,a1,a2,a3,a4,a5,a6,a7)
#define PDEBUG8(t,s,a1,a2,a3,a4,a5,a6,a7,a8)
#define PDEBUG9(t,s,a1,a2,a3,a4,a5,a6,a7,a8,a9)
#define PDUMPMEM(t,b,s,m)
#define PDUMPWORD(t,b,s,m,e)
#endif

#define PDPROC(t,f)           {if (crypto_is_debug(t)==1 ) (f);}
#define EASSERT(a)            if(!(a)) {EPRINTF2("Assertion: line %d in %s\n", __LINE__, __FILE__); }

void  dumpmem(U8 *mem, int size, U8 *msg,int bige);
void  memxor8 (U8 * dst, U8 * src, int len);
void  memsum8 (U8 * dst, U8 * src, int len);
int bytecmp (unsigned char * src, unsigned char * dst, int size);
unsigned char *inetaddr( int sa );


#define MEMCPY(d,s,z)           memcpy((char*)(d),(char*)(s),(size_t)(z))
#define MEMSET(d,v,z)           memset((char *)(d),(int)(v),(size_t)(z))
#define MALLOC(s)               XMALLOC(s)
#define FREE(m)                 XFREE(m)
#define MEMSUM8(d,s,z)          memsum8((U8*)(d),(U8*)(s),(int)(z))
#define MEMXOR8(d,s,z)          memxor8((U8*)(d),(U8*)(s),(int)(z))
#define MEMCMP8(s1,s2,z)        bytecmp((U8*)(s1),(U8*)(s2),(int)(z))

#ifdef WIN32

typedef HANDLE pthread_t;
typedef unsigned int pthread_attr_t;
int pthread_create(pthread_t *thread, const pthread_attr_t *attr, void *func(void *), void *arg);
int pthread_join(pthread_t thread, void **value_ptr);

#pragma message ( "USLEEP translated to SLEEP (in milliseconds)" )

typedef unsigned int useconds_t;
int usleep(useconds_t useconds);

#define sleep Sleep

#if defined(_MSC_VER) || defined(_MSC_EXTENSIONS)
  #define DELTA_EPOCH_USEC  11644473600000000Ui64
#else
  #define DELTA_EPOCH_USEC  11644473600000000ULL
#endif


int gettimeofday(struct timeval *tv, struct timezone *tz);

#define ctime_r(x,y) strcpy(y,ctime(x))

#endif

#endif
