#!/bin/bash
# Author: xiaodong.fan@celestialsemi.com
# build for automatic generating of rootfs of different platform

ROOTFS_TARGET=../../rootfs
FIRMWARE_SRC=../firmware
FIRMWARE_DES=$ROOTFS_TARGET/lib/firmware
JFFS2_BIN=${CRISP_RELEASE}/tools/bin/mkfs.jffs2
SUMTOOL=${CRISP_RELEASE}/tools/bin/sumtool
#JFFS2_BIN=mkfs.jffs2
ERASEBLOCK_NOR=""
ERASEBLOCK_NAND=""
PAGESIZE_NAND=""

FILENAME_ORG1=rootfs.nor.jffs2
FILENAME_ORG2=rootfs.nor_128ksector.jffs2
FILENAME_ORG3=rootfs.nand.jffs2

FW_DIR=`ls ${FIRMWARE_SRC} ` 
generate_rootfs()
{
    echo 'generating jffs2 filesystem for NOR Flash: erase block size:64KB'
    $JFFS2_BIN  -p  -d $ROOTFS_TARGET  -o ${FILENAME1}_temp
	$SUMTOOL -i ${FILENAME1}_temp -o $FILENAME1 
	rm -f ${FILENAME1}_temp

    if [ "$ERASEBLOCK_NOR" != "" ]; then
        echo 'generating jffs2 filesystem for NOR Flash: erase block size:128KB'
        $JFFS2_BIN   -p -n --eraseblock=$ERASEBLOCK_NOR -d $ROOTFS_TARGET -n -o ${FILENAME2}_temp
		$SUMTOOL --eraseblock=$ERASEBLOCK_NOR -i ${FILENAME2}_temp -o $FILENAME2 
		rm -f ${FILENAME2}_temp

    fi    

    echo 'generating jffs2 filesystem for NAND Flash'
    echo 'JFFS2 for NAND is depend on the type of NAND, the command is special for HYNIX HYUF081G2A (erase block size: 128KB; page size:2KB)'
    $JFFS2_BIN -p -d $ROOTFS_TARGET -n --eraseblock=$ERASEBLOCK_NAND_LARGEPAGE --pagesize=$PAGESIZE_NAND_LARGEPAGE  -o ${FILENAME3}_temp
    $SUMTOOL --eraseblock=$ERASEBLOCK_NAND_LARGEPAGE  -i ${FILENAME3}_temp -o ${FILENAME3}_largepage
	rm -f ${FILENAME3}_temp

	echo 'generating jffs2 filesystem for NAND Flash'
    echo 'JFFS2 for NAND is depend on the type of NAND (erase block size: 16KB; page size:512B)'
    $JFFS2_BIN -p -d $ROOTFS_TARGET -n --eraseblock=$ERASEBLOCK_NAND_SMALLPAGE --pagesize=$PAGESIZE_NAND_SMALLPAGE  -o ${FILENAME3}_temp
    $SUMTOOL --eraseblock=$ERASEBLOCK_NAND_SMALLPAGE  -i ${FILENAME3}_temp -o ${FILENAME3}_smallpage
	rm -f ${FILENAME3}_temp

}

if test -d $ROOTFS_TARGET; then
	echo "found rootfs"
	if test ! -d $FIRMWARE_DES; then
		mkdir $FIRMWARE_DES
	fi

    if test -d $FIRMWARE_SRC && test -d $FIRMWARE_DES; then
        for arg in $FW_DIR 
          do
          echo "Generating for $arg"
          rm -f $FIRMWARE_DES/*
          cp -f $FIRMWARE_SRC/$arg/* $FIRMWARE_DES
          FILENAME1=${FILENAME_ORG1}_for_$arg
          FILENAME2=${FILENAME_ORG2}_for_$arg
          FILENAME3=${FILENAME_ORG3}_for_$arg
          ERASEBLOCK_NOR=""
          ERASEBLOCK_NAND_LARGEPAGE="0x20000"
          PAGESIZE_NAND_LARGEPAGE="0x800"
          ERASEBLOCK_NAND_SMALLPAGE="0x4000"
          PAGESIZE_NAND_SMALLPAGE="0x200"

		  generate_rootfs
        done
    else
          FILENAME1=${FILENAME_ORG1}
          FILENAME2=${FILENAME_ORG2}
          FILENAME3=${FILENAME_ORG3}
          ERASEBLOCK_NOR=""
          ERASEBLOCK_NAND_LARGEPAGE="0x20000"
          PAGESIZE_NAND_LARGEPAGE="0x800"
          ERASEBLOCK_NAND_SMALLPAGE="0x4000"
          PAGESIZE_NAND_SMALLPAGE="0x200"

		  generate_rootfs
    fi

 
fi

