#!/usr/bin/perl

# Does something similar to the cp command when the last argument (the 
# destination) happens to be a directory.  This script actually requires that 
# the last argument be a directory.  It will not copy any file or directory 
# with the name ".svn" or "CVS".

if ($#ARGV < 1) {
    print "Wrong number of arguments\n";
    exit 1;
}

my $limit = $#ARGV;
my $i;
my @is_directory = ();

for ($i = 0; $i < $limit; $i++) {

    if (-d $ARGV[$i]) {

        push(@is_directory, 1);

    } elsif (-e $ARGV[$i]) {

        push(@is_directory, 0);

    } else {

        print "$ARGV[$i] does not exist\n";
        exit 2;

    }
}

my $dest = $ARGV[$limit];

if (-d $dest) {
} else {
    print "$dest is not a directory\n";
    exit 3;
}

my $exit_value;
my $value;
for ($i = 0; $i < $limit; $i++) {

    $value = 0;

    if ($ARGV[$i] eq ".svn") {

        # do nothing

    } elsif ($ARGV[$i] eq "CVS") {

        # do nothing

    } elsif ($is_directory[$i]) {

        $value = taruntar($ARGV[$i], $dest);

    } else {

        print "cp -v $ARGV[$i] $dest\n";
        $value = system("cp", "-v", $ARGV[$i], $dest);
    }

    $exit_value = $value >> 8;
    if ($exit_value) {
        exit $exit_value;
    }
}

exit 0;

############################################

sub taruntar {

    my ($basedir, $leaf, $destdir, $retval, $c1pid, $c2pid);

    if ($_[0] =~ /\/([^\/]+$)/) {
        $basedir = $`;
        $leaf = $1;

        if ($basedir eq '') {
            $basedir = '/';
        }
    } else {
        $basedir = '.';
        $leaf = $_[0];
    }

    if ($leaf eq ".svn") {
        return 0;
    }

    if ($leaf eq "CVS") {
        return 0;
    }

    $destdir = $_[1];

    pipe READH, WRITEH;

    if ($c1pid = fork) {

        if ($c2pid = fork) {

            # this is the parent
            close READH;
            close WRITEH;

            waitpid($c2pid, 0);
            $retval = $?;

            waitpid($c1pid, 0);
            $retval = $exit_value | $?;

            return $retval;

        } else { # this is the second child which writes to the pipe

            die "cannot fork second child: $!" unless defined $c2pid;

            print "cd $basedir\n";
            print "tar cp --exclude .svn --exclude CVS $leaf\n";

            close READH;

            # redirect STDOUT to the write side of the pipe
            open(STDOUT, ">&WRITEH") || die "Cannot dup STDOUT : $!";
            close WRITEH;

            chdir $basedir;
            exec("tar cp --exclude .svn --exclude CVS $leaf");
        }

    } else { # this is the first child which reads from the pipe

        die "cannot fork first child: $!" unless defined $c1pid;

        close WRITEH;

        # redirect STDIN to the read side of the pipe
        open(STDIN, "<&READH") || die "Cannot dup STDIN : $!";
        close READH;

        print "cd $destdir\n";
        chdir $destdir;

        print "tar xpv\n";
        exec("tar xpv");
    }
}
