#!/bin/sh

##########################################################
# wpa_supplicant 
# p2p
# widi
##########################################################

if [ "$CLEAR_CRUMBS" = "1" ] ; then
$LOGINFO "Clearing crumbs folder"
rm -rf /mnt/jffs2/cavium/crumbs
mkdir -p /mnt/jffs2/cavium/crumbs
fi


# Configure these variables to customize your device names
echo "ADAPTER_NAME=$ADAPTER_NAME"
echo "MODEL_NAME=$MODEL_NAME"
echo "MAC_SKIP_DIGITS=$MAC_SKIP_DIGITS"

# STA mode
if [ ! -e /mnt/jffs2/cavium/crumbs/wpa_supplicant.conf ]
then
    cp -r /etc/conf/wifi/wpa_supplicant.conf /mnt/jffs2/cavium/crumbs/

    # The following steps are done to populatate the softAp config during the boot process ...
    # ...so that Proper Adapter Name is shown in stdGui during Firmware Upgrade
    local IF_NAME=wlan0
    local MAC=`ifconfig ${IF_NAME} | grep "HWaddr " | awk '{print $5}' | sed -e s/://g`
    local SHORTMAC=${MAC:${MAC_SKIP_DIGITS}}
    local WPA_CONF=/mnt/jffs2/cavium/crumbs/wpa_supplicant.conf
    sed "s/device_name=.*/device_name=${ADAPTER_NAME}${SHORTMAC}/g" -i ${WPA_CONF}

    sync
fi

# P2P mode
if [ ! -e /mnt/jffs2/cavium/crumbs/p2p.conf ]
then
    cp -r /etc/conf/wifi/p2p.conf /mnt/jffs2/cavium/crumbs/

    local P2P_CONF=/mnt/jffs2/cavium/crumbs/p2p.conf
    sed "s/device_name=.*/device_name=${ADAPTER_NAME}/g" -i ${P2P_CONF}
    change_adapter_id.sh p2p /mnt/jffs2/cavium/crumbs/p2p.conf
    if [ "$MODEL_NAME" != "" ];then
        sed "s/device_name=.*/&-${MODEL_NAME}/g" -i ${P2P_CONF}
    fi
sync
fi

# Some dongles may have developed an extremely long list of persistent p2p con-
# nections (due to the fact that some handsets always create a persistent connection
# and then never use it again), which can cause grief for wpa_supplicant and friends.
# Pare down the file to a manageable size (maximum of 8 connections).
#$LOGINFO "-------- prepare p2p -------"
newconfname=`/cavium/parep2p -m 8 /mnt/jffs2/cavium/crumbs/p2p.conf`
if [ "$newconfname" != "/mnt/jffs2/cavium/crumbs/p2p.conf" ]; then
    $LOGINFO "Paring down p2p.conf to a manageable size."
    cp "$newconfname" "/mnt/jffs2/cavium/crumbs/p2p.conf"
    sync
    rm "$newconfname"
fi


# AP mode
local AP_IF_NAME=wlan0
local MAC=`ifconfig ${AP_IF_NAME} | grep "HWaddr " | awk '{print $5}' | sed -e s/://g`
local SHORTMAC=${MAC:${MAC_SKIP_DIGITS}}
FW_UPGRADE_AP_SSID=${ADAPTER_NAME}${SHORTMAC}
if [ ! -e /mnt/jffs2/cavium/crumbs/softap.conf ]; then
    cp -r /etc/conf/wifi/softap.conf /mnt/jffs2/cavium/crumbs/

#The following steps are done to populatate the softAp config during the boot process
#so that Proper Adapter Name is shown in stdGui during Firmware Upgrade
    local SOFTAP_CONF=/mnt/jffs2/cavium/crumbs/softap.conf
    FW_UPGRADE_AP_SSID=`grep "ssid=" ${SOFTAP_CONF} | cut -d "=" -f 2 | sed s/\"//g`
                                                                        
    if [ "$FW_UPGRADE_AP_SSID" = "SSID" ] || [ "$FW_UPGRADE_AP_SSID" = "" ] || [ "$FW_UPGRADE_AP_SSID" = " " ]  
    then                                       
        FW_UPGRADE_AP_SSID=${ADAPTER_NAME}${SHORTMAC}
        sed "s/ssid=.*/ssid=\"${FW_UPGRADE_AP_SSID}\"/g" -i ${SOFTAP_CONF}
        sync
    fi     
    sync
fi
export AP_SSID=$FW_UPGRADE_AP_SSID
    
######################################################

######################################################
# widi
######################################################

if [ ! -e /mnt/jffs2/cavium/crumbs/adapter.config ]
then
    
        cp -r /etc/widi/adapter.config /mnt/jffs2/cavium/crumbs/adapter.config
        change_adapter_id.sh adapter /mnt/jffs2/cavium/crumbs/adapter.config
        sync
        
        sed -i "/rds_interface_name=/ c rds_interface_name=wlan0" /mnt/jffs2/cavium/crumbs/adapter.config
        sed -i "/p2p_interface_name=/ c p2p_interface_name=p2p0" /mnt/jffs2/cavium/crumbs/adapter.config
fi

######################################################

if [ "$USE_UNFILTERED_EDID" = "1" ] ; then
	$LOGINFO "Use unfiltered EDID"
	ln -s /cavium/netHD_tx_edid_unfiltered /cavium/netHD_tx_edid
else
	$LOGINFO "Use filtered EDID"
	ln -s /cavium/netHD_tx_edid_filtered /cavium/netHD_tx_edid
fi


######################################################
