/*
 # COPYRIGHT NOTICE
 #
 #***************************************************************************

 The following software code is the property of Cavium, Inc.
 Cavium, Inc. reserves all rights in this software, including but not limited
 to copyright and shall be vested with all rights incidental thereto.

 Rights granted to you by or under any licence shall be limited to those
 explicitly granted. You may not record, store, reproduce, reverse engineer,
 decompile, inter-operate and/or transmit this software in any medium,
 electronic or otherwise, without the express written consent of Cavium, Inc.

 ALL RIGHTS RESERVED.

 Copyright 2013, Cavium, Inc.
 #***************************************************************************
 #
 # END OF COPYRIGHT NOTICE
 #
*/

/**
 * @file cnc_input_modules.h
 * @ingroup core
 * Functions and definitions to implement input modules
 */
#ifndef __CNC_INPUT_MODULES_H__
#define __CNC_INPUT_MODULES_H__

#include <sys/select.h>
#include <sys/time.h>

#include "cnc_input.h"

typedef struct {
    void (*init)();

    // Notify device select has ended with this set to read
    void (*selectComplete)(fd_set *set);
    
    // Notify the input module 
    void (*notifyToInput)(cncInputEventToInput_t event, const char *eventText);

    // Timer expired
    void (*timerExpired)();

    // Set int param
    void (*paramInt)(cncInputParam_t param, int value);

    // Set string param
    void (*paramStr)(cncInputParam_t param, const char *value);

    cncInputDeviceType_t type;
} cncInputDevice_t;


/** Generate a new input event (key, mouse, ...)
 * A input module (or any other module) could insert data in the input
 * module with this function. 
 * 
 * @param data Data to be notify. It is only garanteed to be valid in
 * the call to the function.
 * @return The logical or of the return value of all callbacks
 * subscribed to the notified event. If there is no callback
 * it will return CNC_FALSE.
 */
CNC_BOOL CNC_InputInternal_Notify(cncInputData_t *data);

/** Register a new input module 
 * @param device the device to add, cncInput makes a copy of the device struct
 */
void CNC_InputInternal_AddDevice(cncInputDevice_t *device);

/** Add a file descriptor to monitor 
 * @param fd The file descriptor to monitor when is data available
 */
void CNC_InputInternal_AddFd(int fd);

/** Remove a file descriptor from monitoring
 * @param fd The file descriptor to stop monitoring
 */
void CNC_InputInternal_RemoveFd(int fd);

/** Add a timer in the input module
 * Only will work if called from the input thread. Will call the
 * timerExpired of every input module.
 * The input module only manages one timer, and always is the one with 
 * earlier timeout.
 * @param timeout The relative time of the timeout
 */
void CNC_InputInternal_AddTimer(const struct timeval *timeout);

#endif // __CNC_INPUT_MODULES_H__
